/* Mayara Felix
	UPDATED: 4/20/2015
	- Edited to import updated charter list in line 347

	Goal: create a wide dataset on student with enrollment and outcome information
	Used code from "Y:\DPS\Peter\DPSVA\Programs\qual_2012_2013.do"
	
	Spyear: spring year of academic year, i.e. 2012 for 2011-2012
	
	*Updated enrollment files HAVE grade information
	
*/	

clear all
cap log close
set more off
set trace off
set maxvar 32767
set seed 42
adopath + "Z:\ado\plus\"

local rawdata   "Y:/DPS/Raw Data/"
local asgndata  "Y:/DPS/Raw Data/2013_03_21_enrollment-match/"
local newasgndata	 "Y:/DPS/Raw Data/2014_12_19_2014_updated_oct_counts/"
local newdata2 "Y:/DPS/Raw Data/2014_11_12_match_enrollment_tests/data_2014_10/"
local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local yusuke	 "Y:/DPS/yusuke/"
local graph "Y:/DPS/MDmeetsRD Denver/Graphs/"

scalar sims_csap_wide    = 1
local graph=0
local date 20150603	/*20150428 20150420 20150107*/
local outlierdate 20150602

if sims_csap_wide == 1 {

	************************************
	******** Enrollment files **********
	************************************
	
	*2009-2010
	
		insheet using "`newasgndata'03182013_OctCoutDataV2_2009tab.csv", names clear
		keep if internaluse==1
		gen spyear = 2010
		drop resschnum resschool_name
		ren curschool_name schoolname

		tempfile bl2010
		save `bl2010', replace
	
	*2010-2011
	
		insheet using "`newasgndata'03182013_OctCoutDataV2_2010tab.csv", names clear
		keep if internaluse==1
		gen spyear = 2011
		drop resschnum resschool_name
		ren curschool_name schoolname

		tempfile bl2011
		save `bl2011', replace
	
	*2011-2012
		insheet using "`newasgndata'03182013_OCTCountStudentDataV2_2011tab.csv", names clear
		keep if internaluse==1
		gen spyear = 2012
		drop resschoolnum v5	
		ren school_name schoolname

		ren billingual bilingual
		
		tempfile bl2012
		save `bl2012', replace
	
	*2012-2013
		insheet using "`newasgndata'03182013_OCTCountStudentDataV2_2012tab.csv", names clear
		keep if internaluse==1
		gen spyear = 2013
		drop resschnum school_name
		ren curschool_name schoolname
		
		tempfile bl2013
		save `bl2013', replace
		
	*2013-2014 - This dataset is in very different format than all others
	
		*NEED CODEBOOK TO CLARIFY DESCRIPTIVE CHARACTERISTICS FOR ALL VARIABLES IN THIS YEAR
		*Okay to proceed since 2013-2014 is only outcome, not baseline
		
		insheet using "`newdata2'OctCnt13.csv", names clear
		keep if internaluse==1
		
		keep masked lastname firstnam dpsgender dpsgrade frl schoolnum bilingualsvc eslsvc sped gt 
		gen spyear = 2014
		gen schoolname=""
		
		ren masked_id maskedid
		ren dpsgender gender
		ren dpsgrade grade
		
		gen bilingual = ""
		replace bilingual = "Yes - Bilingual Program" if bilingualsvc!=0
		drop bilingualsvc
		
		
		/*	ESL variable values for other years:
		
			Not Applicable
			Yes - ESL Program
			Exited ESL Program (Year 3+)
			Parent Refusal of Formal ESL Program
			Redesignated ESL Program (Monitored Year 1)
			Redesignated ESL Program (Monitored Year 2)
		
		*/
		
		gen esl =""
		replace esl = "Yes - ESL Program" if eslsvc!=0
		drop eslsvc
	
		gen nsped = "N"
		replace nsped="Y" if sped==1
		drop sped
		ren nsped sped
		
		gen nfrl =""
		replace nfrl="Free Lunch Eligible" if frl==1
		replace nfrl="Reduced Lunch Eligible" if frl==2
		drop frl
		ren nfrl frl
		
		ren gt gt_any
		
		*** Add school names to 2014 data using last name used based on previous years****
		preserve
			u `bl2013', clear
			append using `bl2012'
			append using `bl2011'
			append using `bl2010'
			keep schoolnum schoolname spyear
			keep if schoolname!=""
			bys schoolnum: egen max_year = max(spyear)
			keep if spyear==max_year
			drop spyear max_year
			
			duplicates drop schoolnum, force
			tempfile schools
			sa `schools'
		restore
		
		merge m:1 schoolnum using `schools', update replace
		keep if _merge!=2
		drop _merge
		
		*Need codebook for this new data. Replace all with missing.
		gen raceethnicity=""
		
	*Append
		append using `bl2013'
		append using `bl2012'
		append using `bl2011'
		append using `bl2010'
		
		drop internaluse
		tab raceethnicity spyear, m 
		tab sped spyear, m
		
		gen black     = (raceethnicity=="Black or African American" | raceethnicity=="Black (Not Hispanic)") 	if raceethnicity!=""
		gen hisp      = (raceethnicity=="Hispanic or Latino" | raceethnicity=="Hispanic")						if raceethnicity!=""
		gen asian     = (raceethnicity=="Asian" | raceethnicity=="Asian OR Pacific Islander")					if raceethnicity!=""
		gen white     = (raceethnicity=="White" | raceethnicity=="White(Not Hispanic)") 						if raceethnicity!=""
		gen otherrace = (1-black-hisp-asian-white) 																if raceethnicity!=""
		drop raceethnicity
		
		gen female = (gender=="F")
		drop gender
		
		/*Standardize demos that should not change over time
			foreach var of varlist female black hisp asian white otherrace{
				bys maskedid: egen max_`var' = max(`var')
				replace `var' = max_`var' if `var'!= max_`var'
			}
			drop max_*
		*/
		
		*Choose a single name
		preserve
			keep maskedid lastname firstname
			sample 1, count by(maskedid)
			tempfile names
			sa `names'
		restore
		
		merge m:1 maskedid using `names', update replace
		drop _merge
		
		gen issped = (sped=="Y")
		drop sped
		rename issped sped
		
		gen foodfred = frl=="Free Lunch Eligible" | frl=="Reduced Lunch Eligible"
		drop frl

		gen lep = (esl=="Yes - ESL Program" | esl=="Redesignated ESL Program (Monitored Year 1)" | esl=="Redesignated ESL Program (Monitored Year 2)")
		drop esl
		
		gen gt_ela = (gt=="Language Arts Gifted") if gt!=""
		gen gt_math = (gt=="Mathematics Gifted") if gt!=""
		gen gt_math_ela = (gt=="Both Language Arts & Mathematics Gifted") if gt!=""
		gen gt_other = (gt=="Other Gifted (i.e. leadership, creativity, spatial, visual, performing or musical arts)") if gt!=""
		
		gen gifted = (gt_ela==1 | gt_math==1 | gt_math_ela==1 | gt_other==1 | gt_any==1)
		
		drop gt gt_any
		
		gen bil = (bilingual=="Yes - Bilingual Program")
		drop bilingual
		ren bil bilingual
		
		tab grade spyear, m
		
		order maskedid spyear schoolnum grade schoolname black-lep bilingual gt_*
		sort spyear
			
		tempfile enrollment
		sa `enrollment'
		
	*****************************************************************************************
	************* Clean Outcome files (Use scores for SPY 2010 through SPY 2013) ************
	*****************************************************************************************
	
	set seed 12345678
	
	insheet using "Y:\DPS\Raw Data\2014_11_14_tests20082012\ARE_CSAP_ACT_AP.txt", names clear
	rename maskedstudentnumber maskedid
	tempfile temp1
	sa `temp1'
	
	insheet using "`newdata2'ARE_CSAP_ACT_AP(2013).txt", names clear
	rename maskedstudentnumber maskedid
	append using `temp1'
	
	tempfile temp2
	sa `temp2'
	
	insheet using "`newdata2'ARE_CSAP_ACT_AP(2014).txt", names clear
	rename maskedstudentnumber maskedid
	append using `temp2'
	
	ren year spyear
	keep if spyear>=2010
	
	keep if testcategory=="CSAP" & (testname=="CSAP Math" | testname=="CSAP Reading" | testname=="CSAP Science" | testname=="CSAP Writing")
	destring ss, replace
	rename ss ss_	/*score*/
	
	replace measurecode = lower(measurecode)


	
	
	/*
		CSAPA (altermatove CSAP) proficiency levels: [Currently not using any alternative testing data]
		DE - Developing
		EM - Emerging
		EX - Exploring
		IN - Inconclusive
		NO - Novice
		NS - No score

	*/
	
	drop if ss_==.

	gen pp_up_    = (proflvl=="PP" | proflvl=="P" | proflvl=="A") if ss!=.	/*part proficient, proficient, advanced*/
	gen prof_up_  = (proflvl=="P" | proflvl=="A") if ss!=.
	gen adv_      = (proflvl=="A") if ss!=.
	drop proflvl rs testcategory testname name
	
	
	*Keep earliest attempt at each grade
	bys maskedid grade measurecode: egen min_year= min(spyear)
	*br  if spyear!=min_year
	
	drop if spyear!=min_year | grade==.
	drop min_year
	
	*Keep 1 score per subject and grade
	
	sample 1, count by(maskedid grade measurecode)
	
	reshape wide ss_ pp_up_ prof_up_ adv_, i(maskedid grade spyear schoolnum) j(measurecode) str
	
	drop if grade==.
	unique maskedid spyear grade
	duplicates tag maskedid spyear grade, gen(dup)
	drop dup
	*br if dup
	
	*Few kids with different scores showing up in different obs
	foreach var of varlist ss_* pp_* prof_* adv_*{
		bys maskedid spyear grade: egen max_`var' = max(`var')
		replace `var'=max_`var' if `var'==. 
	}
	drop max_*
	
	duplicates drop maskedid grade, force

	if `graph'==1{
		foreach t in math reading writing science{
			hist ss_`t', by(spyear grade)
			graph export "`graph'`t'.pdf", replace
		}
	}
	
	*string grades
	tostring grade, replace
	
	tempfile outcomes
	sa `outcomes'
	
	******************************************************
	******* Merge enrollment and outcomes. Go wide *******
	******************************************************
	
	*Keep if in enrollment
		u `enrollment', clear

		tab grade spyear
		merge m:1 maskedid grade using `outcomes'
		keep if _merge!=2
		drop _merge
		
		*Repeaters - take the first attempt
		
			unique maskedid grade
			duplicates tag maskedid grade, gen(repeat)
			*br if dup
			bys maskedid grade: egen min_year = min(spyear)
			drop if min_year!=spyear
			drop min_year
			
			order grade, after(spyear)
			tab spyear
		
		tab grade spyear, m

		*Identify charters and alternatives
		preserve
			insheet using "`rawdata'dps_charters_20112012_20150420 update.csv", double clear
			keep schoolname schoolnum charter alternative
			tempfile charter
			sa `charter'
		restore
		
		merge m:1 schoolnum using `charter'
		keep if _merge!=2
		drop _merge
		replace charter=0 if charter==.
		order charter alternative, after(schoolname)
		
		
		*Flag more alternative schools - should not contribute to scores
		preserve
			insheet using "`rawdata'dps_alternatives_2013.csv", double clear
			keep if alternative==1
			ren alternative alt
			tempfile temp
			sa `temp'
		restore
		
		merge m:1 schoolnum using `temp', keepusing(alt)
		keep if _merge!=2
		replace alternative=alt if  alt==1
		replace alternative=0 if alternative!=1
		drop alt _merge
		
		tab charter alternative
	
			
		** ADDED June 3rd, 2015. To exclude the lowest scores.
		
		local lowest_3_math = 150
		local lowest_4_math = 180
		local lowest_5_math = 220
		local lowest_6_math = 240
		local lowest_7_math = 280
		local lowest_8_math = 310
		local lowest_9_math = 340
		local lowest_10_math = 370


		local lowest_3_reading = 150
		local lowest_4_reading = 180
		local lowest_5_reading = 220
		local lowest_6_reading = 260
		local lowest_7_reading = 300
		local lowest_8_reading = 330
		local lowest_9_reading = 350
		local lowest_10_reading = 370


		local lowest_3_writing = 150
		local lowest_4_writing = 190
		local lowest_5_writing = 220
		local lowest_6_writing = 230
		local lowest_7_writing = 240
		local lowest_8_writing = 250
		local lowest_9_writing = 260
		local lowest_10_writing = 270
		
		foreach subject in math reading writing {
			forvalues g = 3(1)10 {
				replace ss_`subject' = . if ss_`subject' == `lowest_`g'_`subject'' & grade == "`g'"			
				}
			}
			
		
		*Standardize scores at this point - using all kids in the enrollent files
		
		* Standardize tests at grade-year level
		levelsof spyear, local(years)
		foreach t in math writing reading science {
			gen raw_ss_`t' = ss_`t'
			forval g = 3/10 {
				foreach y in `years' {
					*qui summ ss_`t' if grade=="`g'" & spyear==`y' & alternative==0
					qui summ ss_`t' if grade=="`g'" & spyear==`y'
					replace ss_`t' = (ss_`t'-r(mean))/r(sd) if grade=="`g'" & spyear==`y' & ss_`t' != .
				}
			}
		}
	
	*************** Go wide *******************

	tostring grade, replace
	tab grade
	
	order charter, after(schoolname)
	order repeat, after(spyear)
	
	saveold "`cleandata'sims_csap_long_20102014 `date'.dta", replace
	
	reshape wide 	spyear schoolnum schoolname charter alternative black hisp asian white otherrace female sped foodfred lep bilingual repeat ///
					gifted gt_* raw_* ss_* pp_* prof_* adv_*, i(maskedid lastname firstname) j(grade) string
	
	unique maskedid
	order *EC *K, before(spyear1)
	order *10 *11 *12, after(adv_writing9)
	saveold "`cleandata'sims_csap_wide_20102014 `date'.dta", replace	
}

