/* Mayara Felix

	Goal: 	Read in simulated pscore file and match to AT RUN to get maskedids
	
	Ignacio Rodriguez
	04/25/2016
	We use the application file generated by Yusuke's subroutine code found at 
	Y:\DPS\Yusuke\create_pscore_charter_subroutine_20160208
	the file is 
	"Y:\DPS\yusuke\data\application_data_year_one_20160208.dta"
	We use this file because it drops applications where the student is ineligible (apppriority == 0)
	or the applied school has 0 capacity. These students should have a pscore of 0 anyway, so the distinction 
	only really matters for the fulltype column in the tables. 
	We used to run the raw at run files but these differences cause for differt types to be created. 
	
	
*/	

clear all
cap log close
set more off
set trace off
set maxvar 32767
adopath + "Z:\ado\plus\"

local rawdata   "Y:/DPS/Raw Data/"
local asgndata  "Y:/DPS/Raw Data/2013_03_21_enrollment-match/"
local testdata  "Y:/DPS/Raw Data/2013_10_03_tests2013/"
local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local yusuke	 "Y:/DPS/yusuke/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/"

local date 					20160815 /*20160425  20160208*/
local outputdate 			20170310 /*20160816 20160425 20160421	20160208 20150528 20150525 20150519*/
local appdata 				20160815
local find_types						= 0	/*This take a long time to run. Only re-run if redefining what type is*/
local find_first_choice_types 			= 0
local find_first_second_choice_types 	= 0
local find_upto_third_choice_types 		= 0

*Simulations
	insheet using "Y:/DPS/Raw Data/2014_12_02_DA_probabilities_and_results/probabilities/fixed/1000000_Probabilities.csv", double clear
	
	ren assignment_probability pscore_charter_simulation
	
	tempfile sim
	sa `sim'

*Retrieve application, offer, and grade information from yusuke file. this used to be the at run file
	*insheet using "`asgndata'1_AT_RUN_022262012.csv", double clear
	use "`cleandata'application_data_year_one_`appdata'.dta", clear
	 
	ren maskedstudentids maskedid
	ren requestedschool appschoolnum
	ren grade appgrade
	
	drop if maskedid=="#VALUE!"

	
	*Find combinations of cells (that is, define types)
	if `find_types'==1{
	
	foreach grade in "EC" "00" "01" "02" "03" "04" "05" "06" "07" "08" "09" "10" "11" "12" {
	
		preserve
		keep if appgrade == "`grade'"
		/*IR dont need to do this anymore since we are using yusukes file
			*Edit choiceselection for those that do not have buckets
			forval i = 1/5 {
				qui replace choiceselection = `i'*10 if choiceselection==`i'
			}
			*/
			
			*Grade, school, capacity program, choiceselection, priority code,
			egen scell = group(appschoolnum capacityprogram choiceselection apppriorityseq), missing
	
			keep maskedid scell
			duplicates drop
			
			gen in_scell = 1
			qui levelsof scell, local(cells)
			
			*Create variable for each cell
			qui tab scell, gen(in_scell_)
			
			*Make sure flag for each cell shows up for all obs for the same maskedid
			foreach var of varlist in_scell_*{
				bys maskedid: egen max_`var' = max(`var')
			}
			keep maskedid max_*
			renvars max_*, predrop(4)
			duplicates drop maskedid, force
			
			*Check that each kid is in some cell
			egen check = rowmax(in_*)

			*Generate type - unique set of schools-capacity-priorities, regardless of ranking
			egen type = group(in_*), missing
			
			keep maskedid type
			
			save "`cleandata'DA_year1_types `date' grade`grade'.dta", replace
	
		restore			
	}
}


	
	*Find combinations of cells (that is, define types)
	if `find_first_choice_types'==1{
	
	foreach grade in "EC" "00" "01" "02" "03" "04" "05" "06" "07" "08" "09" "10" "11" "12" {
	
		preserve
		
		keep if appgrade == "`grade'"
		
		keep if inlist(choiceselection, 10, 11, 12, 13, 14)
		
			*Grade, school, capacity program, choiceselection, priority code,
			egen scell = group(appschoolnum capacityprogram choiceselection apppriorityseq), missing
	
			keep maskedid scell
			duplicates drop
			
			gen in_scell = 1
			qui levelsof scell, local(cells)
			
			*Create variable for each cell
			qui tab scell, gen(in_scell_)
			
			*Make sure flag for each cell shows up for all obs for the same maskedid
			foreach var of varlist in_scell_*{
				bys maskedid: egen max_`var' = max(`var')
			}
			keep maskedid max_*
			renvars max_*, predrop(4)
			duplicates drop maskedid, force
			
			*Check that each kid is in some cell
			egen check = rowmax(in_*)

			*Generate type - unique set of schools-capacity-priorities, regardless of ranking
			egen type_fc = group(in_*), missing
			
			keep maskedid type_fc
			
			save "`cleandata'DA_year1_fc_types `date' grade`grade'.dta", replace
	
		restore			
	}
}


	*Find combinations of cells (that is, define types)
	if `find_first_second_choice_types'==1{
	
	foreach grade in "EC" "00" "01" "02" "03" "04" "05" "06" "07" "08" "09" "10" "11" "12" {
	
		preserve
		
		keep if appgrade == "`grade'"
		
		keep if inlist(choiceselection, 10, 11, 12, 13, 14, 20, 21, 22, 23, 24)
		
			*Grade, school, capacity program, choiceselection, priority code,
			egen scell = group(appschoolnum capacityprogram choiceselection apppriorityseq), missing
	
			keep maskedid scell
			duplicates drop
			
			gen in_scell = 1
			qui levelsof scell, local(cells)
			
			*Create variable for each cell
			qui tab scell, gen(in_scell_)
			
			*Make sure flag for each cell shows up for all obs for the same maskedid
			foreach var of varlist in_scell_*{
				bys maskedid: egen max_`var' = max(`var')
			}
			keep maskedid max_*
			renvars max_*, predrop(4)
			duplicates drop maskedid, force
			
			*Check that each kid is in some cell
			egen check = rowmax(in_*)

			*Generate type - unique set of schools-capacity-priorities, regardless of ranking
			egen type_sc = group(in_*), missing
			
			keep maskedid type_sc
			
			save "`cleandata'DA_year1_sc_types `date' grade`grade'.dta", replace
	
		restore			
	}
}

	
	*Find combinations of cells (that is, define types)
	if `find_upto_third_choice_types'==1{
	
	foreach grade in "EC" "00" "01" "02" "03" "04" "05" "06" "07" "08" "09" "10" "11" "12" {
	
		preserve
		
		keep if appgrade == "`grade'"
		
		keep if inlist(choiceselection, 10, 11, 12, 13, 14, 20, 21, 22, 23, 24, 30, 31, 32, 33, 34)
		
			*Grade, school, capacity program, choiceselection, priority code,
			egen scell = group(appschoolnum capacityprogram choiceselection apppriorityseq), missing
	
			keep maskedid scell
			duplicates drop
			
			gen in_scell = 1
			qui levelsof scell, local(cells)
			
			*Create variable for each cell
			qui tab scell, gen(in_scell_)
			
			*Make sure flag for each cell shows up for all obs for the same maskedid
			foreach var of varlist in_scell_*{
				bys maskedid: egen max_`var' = max(`var')
			}
			keep maskedid max_*
			renvars max_*, predrop(4)
			duplicates drop maskedid, force
			
			*Check that each kid is in some cell
			egen check = rowmax(in_*)

			*Generate type - unique set of schools-capacity-priorities, regardless of ranking
			egen type_tc = group(in_*), missing
			
			keep maskedid type_tc
			
			save "`cleandata'DA_year1_tc_types `date' grade`grade'.dta", replace
	
		restore			
	}
}
	
	
	forvalues i = 0(1)9 {
	merge m:1 maskedid using "`cleandata'DA_year1_types `date' grade0`i'.dta"
	rename type type_gr0`i'
	keep if _merge != 2
	drop _merge
	}

	
	foreach choice in fc sc tc {
		forvalues i = 0(1)9 	{
			merge m:1 maskedid using "`cleandata'DA_year1_`choice'_types `date' grade0`i'.dta"
			rename type_`choice' type_`choice'_gr0`i'
			keep if _merge != 2
			drop _merge
		}
	}
		
	merge m:1 maskedid using "`cleandata'DA_year1_types `date' gradeEC.dta"
	rename type type_grEC
	keep if _merge != 2
	drop _merge
	
	merge m:1 maskedid using "`cleandata'DA_year1_types `date' grade10.dta"
	rename type type_gr10
	keep if _merge != 2
	drop _merge
	
	merge m:1 maskedid using "`cleandata'DA_year1_types `date' grade11.dta"
	rename type type_gr11
	keep if _merge != 2
	drop _merge
	
	
	merge m:1 maskedid using "`cleandata'DA_year1_types `date' grade12.dta"
	rename type type_gr12
	keep if _merge != 2
	drop _merge
	
	foreach choice in fc sc tc{
		merge m:1 maskedid using "`cleandata'DA_year1_`choice'_types `date' gradeEC.dta"
		rename type_`choice' type_`choice'_grEC
		keep if _merge != 2
		drop _merge
		
		merge m:1 maskedid using "`cleandata'DA_year1_`choice'_types `date' grade10.dta"
		rename type_`choice' type_`choice'_gr10
		keep if _merge != 2
		drop _merge
		
		merge m:1 maskedid using "`cleandata'DA_year1_`choice'_types `date' grade11.dta"
		rename type_`choice' type_`choice'_gr11
		keep if _merge != 2
		drop _merge
		
		
		merge m:1 maskedid using "`cleandata'DA_year1_`choice'_types `date' grade12.dta"
		rename type_`choice' type_`choice'_gr12
		keep if _merge != 2
		drop _merge	
	}
	
	keep dstlink maskedid appschoolnum appgrade type_* familylinkmasked

	*Merge type back
	merge 1:1 dstlink using `sim'
	keep if _merge==3
	drop _merge
	
	
	gen type = .
	replace type = type_grEC if appgrade == "EC"
	egen max_type_EC = max(type)
	
	replace type = type_gr00 + max_type_EC if appgrade == "00"
	
	forvalues i = 0(1)8 {
	local j = `i' + 1
		egen max_type_`i' = max(type)
		replace type = type_gr0`j' + max_type_`i' if appgrade == "0`j'"
		}
	
	forvalues i = 9(1)11 {
	local j = `i' + 1
		egen max_type_`i' = max(type)
		replace type = type_gr`j' + max_type_`i' if appgrade == "`j'"
		}
		
	drop max_type_* type_gr*
		
	foreach choice in fc sc tc {
		gen type_`choice' = .
		replace type_`choice' = type_`choice'_grEC if appgrade == "EC"
		egen max_type_`choice'_EC = max(type_`choice')
		
		replace type_`choice' = type_`choice'_gr00 + max_type_`choice'_EC if appgrade == "00"
		
		forvalues i = 0(1)8 {
		local j = `i' + 1
			egen max_type_`choice'_`i' = max(type_`choice')
			replace type_`choice' = type_`choice'_gr0`j' + max_type_`choice'_`i' if appgrade == "0`j'"
			}
		
		forvalues i = 9(1)11 {
		local j = `i' + 1
			egen max_type_`choice'_`i' = max(type_`choice')
			replace type_`choice' = type_`choice'_gr`j' + max_type_`choice'_`i' if appgrade == "`j'"
			}
			
		drop max_type_`choice'* type_`choice'_gr*	
	}
		
	rename type type_all
	
	rename type_fc type_fc_all
	rename type_sc type_sc_all
	rename type_tc type_tc_all
	
	
	save "`cleandata'type_for_all_`date'", replace
	

	*this student is in two types because he applies to two grades but he is only eligible for EC (ineligible for 00)
	drop if type_all ==  5085 
	
		* This is for the full applicant type control column (do file 4)
	preserve

		keep maskedid type_all type_fc_all type_sc_all type_tc_all
		duplicates drop
		save "`cleandata'fulltypes`outputdate'", replace		
	restore
	
	rename type_all type
	drop type_fc_all type_sc_all type_tc_all

	
	*School and school-grade level pscore
	preserve
		bys maskedid appschoolnum appgrade: egen pscore_mask_grade = sum(pscore_charter)
		bys maskedid appschoolnum: egen pscore_mask = sum(pscore_charter)
	
		bys type appschoolnum appgrade: egen pscore_school_grade= mean(pscore_mask_grade)
		bys type appschoolnum: egen pscore_school= mean(pscore_mask)
		
		*If kid has family link, define pscore at maskedid level (i.e. just sum), otherwise
		*define it as average across kids within type
		
		gen family = (familylinkmasked!="")
		bys maskedid: egen max_fam = max(family)
		
		replace pscore_school_grade = pscore_mask_grade if max_fam==1
		replace pscore_school = pscore_mask if max_fam==1
		
		keep maskedid appschoolnum appgrade pscore_school pscore_school_grade type pscore_mask* max_fam
		duplicates drop
		order maskedid type, first
		save "`cleandata'pscore_simulation_school_grade", replace
	restore
	
	
	*Study charters
	gen study= (	inlist(appschoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
					inlist(appschoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
					inlist(appschoolnum,462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
				)
							
	*No-excuses charters
	gen noex =  (    inlist(appschoolnum, 392, 393, 394, 426, 428, 429, 435, 436) | ///
					 inlist(appschoolnum, 438, 443, 444, 445, 462, 481, 498, 513) ///
				 )
	
	**Non no-excuses charters
	gen nonnoex = (  inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
					 inlist(appschoolnum, 330, 391, 441, 446, 485, 496, 512) ///
				  )			
				  
	 *Calculate average score within student type level
	 
	foreach sch_type in study noex nonnoex 	{
		
		preserve
		
		*1) Aggregate at study, appgrade, kid level
		ren pscore_charter_simulation pscore
		bys maskedid `sch_type': egen pscore_maskedid = sum(pscore)
		
		*If kid has family link, define pscore at maskedid level (i.e. just sum), otherwise
		*define it as average across kids within type
		gen family = (familylink!="")
		bys maskedid: egen max_fam = max(family)
		
		keep maskedid `sch_type' pscore_maskedid type max_fam
		duplicates drop
		
		
		*2) Take average across types
		bys type `sch_type': egen pscore_`sch_type'_simulation = mean(pscore_maskedid)

		replace pscore_`sch_type'_simulation = pscore_maskedid if max_fam==1
		
		*Probability of assignment at any charter
		keep if `sch_type'
		
		keep maskedid type pscore_`sch_type'_simulation max_fam
		ren max_fam has_familylink
		duplicates drop
		
		sum pscore_`sch_type'
		
		save "`cleandata'pscore_`sch_type'_simulation_`outputdate'", replace
		
		restore
	
	}
	
	
