/* 	Ye Ji Kee (March 11, 2016)
Last Edited: 04/26/2016

	Goal: 	Create analysis files (instruments + treatment + baseline + outcome) for MDmRD 
			One file is created for each instrument
			
			This file is for
			Z_f = first-choice charter offer
			D_f = first-choice charter enrollment
			pscore = score at first-choice charter
			
			Z_o = other-choice charter offer
			D_o = other-choice charter enrollment
			pscore = score at other-choice charters
			
	IMPORTANT: 	Files pscore_charter_formula_`date' and pscore_charter_frequency_`date' must be
				placed into cleandata folder. These files are prepared using Yusuke's pscore
				routines, and can be found here: Y:\DPS\Yusuke\data.
				
Ignacio Rodriguez 05/16/2016
Adding in non charter first choice. will be appstudy_noncharter_fc (non charter because its not charter
fc = first choice)

	
*/	

clear all
cap log close
set more off
set trace off
set tracedepth 1 
set maxvar 32767
adopath + "Z:\ado\plus\"

pause on 

local rawdata   "Y:/DPS/Raw Data/"
local asgndata  "Y:/DPS/Raw Data/2013_03_21_enrollment-match/"
local testdata  "Y:/DPS/Raw Data/2013_10_03_tests2013/"
local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local yusuke	 "Y:/DPS/yusuke/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/"

local pscore_setup =1		/*Propensity score matching setup*/

local simsdate 		20150603	/* 20150428 */
local outputdate 	20170310	/*20160516  20160513 20160426		 20160317 20160311 */
local appdatadate 	year_one_20170310 /*year_one_20160208  year_one_20150708 */

matrix obs = J(10,6,.)


*Retrieve application, offer, and grade information from AT RUN file
insheet using "`asgndata'1_AT_RUN_022262012.csv", double clear
	ren maskedstudentids maskedid
	ren requestedschool appschoolnum
	
	*Not sure what rank=99 means, ask Atila
	gen apprank = choiceselection
	replace apprank = apprank*10 if apprank<10
	replace apprank = floor(apprank/10)
	
	drop if maskedid=="#VALUE!"
	destring maskedid, ignore("#VALUE!") replace

	gen offer = (status=="A")
	bys maskedid appschoolnum: egen max_offer = max(offer)
	drop offer
	ren max_offer offer
	
	gen family = (family!="")
	
	keep grade maskedid offer appschoolnum apprank family

	ren grade appgrade
	duplicates drop appgrade maskedid offer appschoolnum apprank, force
	
	duplicates tag maskedid appgrade appschoolnum, gen(dup)
	tab dup
	
	*Cases of multiple rankings for the same school. Choose the smallest rank
	bys maskedid appgrade appschoolnum: egen min_rank = min(apprank)
	replace apprank = min_rank if dup
	
	*Keep if has nonmissing family link within school
	bys maskedid appgrade appschoolnum: egen max_fam=max(family)
	drop family
	ren max_fam family
	
	duplicates drop maskedid appgrade appschoolnum, force

	*All applicantions (school level)
		unique maskedid 
		matrix obs[1,1]= r(sum)
		matrix obs[1,2]= r(N)
	
	*Identify charters in application
	preserve
		insheet using "`rawdata'dps_charters_20112012_20150420 update.csv", double clear
		keep schoolname schoolnum charter alternative
		ren schoolnum appschoolnum
		gen appcharter = (charter==1)
		keep appschoolnum appcharter
		tempfile charter
		sa `charter'
	restore
	
	merge m:1 appschoolnum using `charter'
	keep if _merge!=2
	drop _merge
	replace appcharter=0 if appcharter==.
	
	*First-choice non charters
	gen appstudy_nonc_fc = 	(apprank == 1 & !(inlist(appschoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(appschoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
						inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391)))	
	
	*First-choice Study charters
	gen appstudy = 	(inlist(appschoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(appschoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
						inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
					)
	
	*First-choice Study charters
	gen appstudy_fc = 	(apprank == 1 &	(inlist(appschoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(appschoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
						inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391)) ///
					)
					
	*other-choice study charters
	gen appstudy_oc = 	(apprank != 1 &	(inlist(appschoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(appschoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
						inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391)) ///
					)
					
	
	*First-choice, 3 CMO charters
	gen appstudy_fc_3cmo = (apprank == 1 & (inlist(appschoolnum, 392, 393, 394, 426, 428, 429, 435, 436) | ///
											inlist(appschoolnum, 438, 443, 444, 445, 462, 481, 498, 513)) ///
							)
	
	*First-choice, non 3-CMO charters
	gen appstudy_fc_n3cmo = (apprank == 1 & (inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
												inlist(appschoolnum, 330, 391, 441, 446, 485, 496, 512)) ///
							)
	*Other-choice, 3 CMO charters
	gen appstudy_oc_3cmo = (apprank != 1 & (inlist(appschoolnum, 392, 393, 394, 426, 428, 429, 435, 436) | ///
											inlist(appschoolnum, 438, 443, 444, 445, 462, 481, 498, 513)) ///
							)
	
	*OTher-choice, non 3-CMO charters
	gen appstudy_oc_n3cmo = (apprank != 1 & (inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
												inlist(appschoolnum, 330, 391, 441, 446, 485, 496, 512)) ///
							)
	
	*Get latest school names
	preserve
		u "`cleandata'sims_csap_long_20102014 `simsdate'", clear
		keep schoolnum schoolname spyear
		bys schoolnum: egen max_year = max(spyear)
		keep if spyear==max_year
		drop spyear max_year
		
		*schools with slighlty different names
		duplicates drop schoolnum, force
		ren schoolnum appschoolnum
		ren schoolname appschoolname
		tempfile schools
		sa `schools'
	restore	
	
	merge m:1 appschoolnum using `schools'
	keep if _merge!=2
	drop _merge
	order maskedid appschoolnum appschoolname
	label var appschoolname "Requested school name"
	
	tab appgrade

	save "`cleandata'AT_RUN_school_level_fc_nonfc.dta", replace
		
	
if `pscore_setup' == 1 {
	
	*One file for each instrument
		foreach file in frequency formula  simulation  {
			
			if "`file'"=="simulation"{
				local pscoredate 20160425 /*20160421 20160208*/
			}
			else{
				local pscoredate year_one_20160512 /*year_one_20160208*/
			}
			
			*Get charter propensity scores
			preserve
				u "`cleandata'pscore_study_`file'_`pscoredate'.dta", clear
				
				if "`file'"=="simulation" {
					rename pscore_study_simulation pscore_charter_simulation
				}
				
				gen spyear =2013 
				cap ren maskedstu maskedid
				cap destring maskedid, replace
						
				*Grades that have scores
				merge 1:m maskedid using "`cleandata'AT_RUN_school_level_fc_nonfc.dta", keepusing(offer appgrade appcharter ///
				appstudy appstudy_nonc_fc appstudy_fc appstudy_oc appstudy_fc_3cmo appstudy_fc_n3cmo appstudy_oc_3cmo appstudy_oc_n3cmo appschoolnum appschoolname apprank family)

				
				*Applicants to any charters
				bys maskedid: egen any_study = max(appstudy) 
				bys maskedid: egen any_study_nonc_fc = max(appstudy_nonc_fc)				
				bys maskedid: egen any_study_fc = max(appstudy_fc)
				bys maskedid: egen any_study_oc = max(appstudy_oc)
				bys maskedid: egen any_study_fc_3cmo = max(appstudy_fc_3cmo)
				bys maskedid: egen any_study_fc_n3cmo = max(appstudy_fc_n3cmo)
				bys maskedid: egen any_study_oc_3cmo = max(appstudy_oc_3cmo)
				bys maskedid: egen any_study_oc_n3cmo = max(appstudy_oc_n3cmo)
			
				*All kids with missing pscores should be treated as pscore==0
				replace pscore_charter = 0 if pscore_charter==. & any_study==1
				
				gen has_study_pscore = (pscore_charter!=.)
				drop _merge
				
				*Grades 4-10 (baseline and outcome scores are available)
				tab appgrade
				keep if inlist(appgrade,"04","05","06","07","08","09","10")
				unique maskedid
				matrix obs[2,1]= r(sum)
				matrix obs[2,2]= r(N)
				
				destring appgrade, replace
				
				gen gradebase = 	appgrade-1
				gen pregradebase =	appgrade-2
				
				tostring appgrade, replace
				tostring gradebase, replace
				tostring pregradebase, replace
				
				tempfile temp1
				sa `temp1'
			restore
			
					
			*Get pscore with probability of assignment at each school, and generate
			*pscore at innov schools based on these
			
			if "`file'"!="simulation"{			
				preserve	
					u "`cleandata'application_data_`appdatadate'", clear
				
					ren masked maskedid
					ren requestedschool appschoolnum
					ren grade appgrade
					destring maskedid, replace
					
					keep if inlist(appgrade,"04","05","06","07","08","09","10")
					
					if "`file'"=="frequency"{
						drop pscore_formula
						ren frequency_offer pscore_frequency
					}
					
					*School-level pscore
					bys maskedid appschoolnum: egen pscore_school_`file' = sum(pscore_`file')
					
					*First-choice non charters
					gen appstudy_nonc_fc = 	(inlist(choiceselection, 10, 11, 12, 13, 14) & ///
											!(inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
											inlist(appschoolnum, 330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
											inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391)))

					*first-choice is charter pscore
					gen appstudy_fc = 	(inlist(choiceselection, 10, 11, 12, 13, 14) & ///
										(inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
										inlist(appschoolnum, 330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
										inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391)))
					
					*other-choice is charter pscore
					gen appstudy_oc =   (!inlist(choiceselection, 10, 11, 12, 13, 14) & ///
										(inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
										inlist(appschoolnum, 330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
										inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391)))

					*First-choice, 3 CMO charters
					gen appstudy_fc_3cmo = (inlist(choiceselection, 10, 11, 12, 13, 14) & ///
											(inlist(appschoolnum, 392, 393, 394, 426, 428, 429, 435, 436) | ///
											inlist(appschoolnum, 438, 443, 444, 445, 462, 481, 498, 513)) ///
										)
					
					*First-choice, non 3-CMO charters
					gen appstudy_fc_n3cmo = (inlist(choiceselection, 10, 11, 12, 13, 14) & ///
												(inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
												inlist(appschoolnum, 330, 391, 441, 446, 485, 496, 512)) ///
											)
					*Other-choice, 3 CMO charters
					gen appstudy_oc_3cmo = (!inlist(choiceselection, 10, 11, 12, 13, 14) & ///
												(inlist(appschoolnum, 392, 393, 394, 426, 428, 429, 435, 436) | ///
												inlist(appschoolnum, 438, 443, 444, 445, 462, 481, 498, 513)) ///
											)
					
					*OTher-choice, non 3-CMO charters
					gen appstudy_oc_n3cmo = (!inlist(choiceselection, 10, 11, 12, 13, 14) & ///
													(inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
													inlist(appschoolnum, 330, 391, 441, 446, 485, 496, 512)) ///
											)	
											
					foreach type in fc oc fc_3cmo fc_n3cmo oc_3cmo oc_n3cmo nonc_fc{
					
						bys maskedid: egen any_study_`type' = max(appstudy_`type')
						
						*firstchoice-level (pscore for appinnov==0 means nothing, it is replaced below by missing)
						bys maskedid appstudy_`type': egen pscore_study`type'_`file' = sum(pscore_`file')
						
						
						*If not first-choice/other-choice, there should not exist an first-choice/other-choice pscore
						replace pscore_study`type'_`file'=. if appstudy_`type'==0				
						
						*No kid should have more than one pscore
						bys maskedid: egen temp_`type' = max(pscore_study`type'_`file')
						replace pscore_study`type'_`file' = temp_`type'
						
						*Scores should be missing if did not apply to any first-choice/other-choice charters
						replace pscore_study`type'_`file' = . if any_study_`type'==0
						
						*Scores should be zero if applied to some first-choice/other choice charters but is coded missing
						replace pscore_study`type'_`file' = 0 if pscore_study`type'_`file'==. & any_study_`type'==1

						replace pscore_study`type'_`file' = 1 if pscore_study`type'_`file' > 1 & pscore_study`type'_`file' != .
					
						gen has_study`type'_pscore = (pscore_study`type'_`file'!=.)
					}			
					duplicates drop maskedid appschoolnum, force

					keep maskedid appschoolnum pscore_school pscore_studyfc_`file' pscore_studyoc_`file' pscore_studyfc_3cmo  pscore_studyfc_n3cmo ///
						pscore_studyoc_3cmo pscore_studyoc_n3cmo pscore_studynonc_fc has_studyfc_pscore has_studyoc_pscore has_studyfc_3cmo_pscore has_studyfc_n3cmo_pscore ///
						has_studyoc_3cmo_pscore has_studyoc_n3cmo_pscore  has_studynonc_fc
						
					tempfile temp2
					sa `temp2'

				restore
			}
			
			else{
				*Calculate first-choice charter level / other-charters level
				preserve
					u "`cleandata'pscore_simulation_school_grade", clear
					
					keep maskedid pscore_mask_grade pscore_school type appschoolnum appgrade max_fam
					
					destring maskedid, replace
					
					merge 1:1 maskedid appschoolnum using "`cleandata'AT_RUN_school_level_fc_nonfc.dta", keepusing(apprank)
					keep if _merge != 2
					drop _merge
					
					keep if inlist(appgrade,"04","05","06","07","08","09","10")
					
					*First-choice non charters
					gen appstudy_nonc_fc = 	(apprank == 1 & ///
											!(inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
											inlist(appschoolnum, 330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
											inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391)))
						
					*first-choice is charter pscore
					gen appstudy_fc = 	(apprank == 1 & ///
										(inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
										inlist(appschoolnum, 330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
										inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391)))
					
					*other-choice is charter pscore
					gen appstudy_oc =   (apprank != 1 & ///
										(inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
										inlist(appschoolnum, 330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
										inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391)))

					*First-choice, 3 CMO charters
					gen appstudy_fc_3cmo = (apprank == 1 & (inlist(appschoolnum, 392, 393, 394, 426, 428, 429, 435, 436) | ///
															inlist(appschoolnum, 438, 443, 444, 445, 462, 481, 498, 513)) ///
											)
					
					*First-choice, non 3-CMO charters
					gen appstudy_fc_n3cmo = (apprank == 1 & (inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
																inlist(appschoolnum, 330, 391, 441, 446, 485, 496, 512)) ///
											)
					*Other-choice, 3 CMO charters
					gen appstudy_oc_3cmo = (apprank != 1 & (inlist(appschoolnum, 392, 393, 394, 426, 428, 429, 435, 436) | ///
															inlist(appschoolnum, 438, 443, 444, 445, 462, 481, 498, 513)) ///
											)
					
					*OTher-choice, non 3-CMO charters
					gen appstudy_oc_n3cmo = (apprank != 1 & (inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
																inlist(appschoolnum, 330, 391, 441, 446, 485, 496, 512)) ///
											)
					
					foreach type in fc oc fc_3cmo fc_n3cmo oc_3cmo oc_n3cmo nonc_fc {

						bys maskedid: egen any_study_`type' = max(appstudy_`type')
					
						*Simulated score - defined at type level and appstudy_fc / appstudy_oc level (pscore for appstudy_fc==0 means nothing, it is replaced
						*below by missing)
						
						bys maskedid appstudy_`type' : egen pscore_temp_`type' = sum(pscore_mask_grade)
						bys type appstudy_`type': egen pscore_study`type'_`file' = mean(pscore_temp_`type')
						
						*If family link, still use score at kid level
						replace pscore_study`type'_`file' = pscore_temp_`type' if max_fam==1
					}
					
					drop pscore_mask* pscore_temp* type
					
					
					foreach type in fc oc fc_3cmo fc_n3cmo oc_3cmo oc_n3cmo nonc_fc {

						*If not first-choice charter / other-choice charters, there should not exist as pscores
						replace pscore_study`type'_`file'=. if appstudy_`type'==0
					
						*No kid should have more than one pscore
						bys maskedid: egen temp_`type' = max(pscore_study`type'_`file')
						replace pscore_study`type'_`file' = temp_`type'
						drop temp_`type'
							
						
						*Scores should be missing if did not apply to any first-choice charter / other charters
						replace pscore_study`type'_`file' = . if any_study_`type'==0
					
						*Scores should be zero if applied to some first-choice charter / other charters but is coded missing
						replace pscore_study`type'_`file' = 0 if pscore_study`type'_`file' ==. & any_study_`type' ==1

						replace pscore_study`type'_`file' = 1 if pscore_study`type'_`file' > 1 & pscore_study`type'_`file' != .
						
						gen has_study`type'_pscore = (pscore_study`type'_`file'!=.)
					}
					
					
					duplicates drop maskedid appschoolnum, force
					
					keep maskedid appschoolnum pscore_school max_fam pscore_studyfc_`file' pscore_studyoc_`file' pscore_studyfc_3cmo pscore_studyfc_n3cmo ///
					pscore_studyoc_3cmo pscore_studyoc_n3cmo pscore_studynonc_fc has_studyfc_pscore has_studyoc_pscore has_studyfc_3cmo_pscore has_studyfc_n3cmo_pscore ///
					has_studyoc_3cmo_pscore has_studyoc_n3cmo_pscore has_studynonc_fc
					
					destring maskedid, replace
					tempfile temp2
					sa `temp2'
				restore	
			}
			

				************ Combine pscore files ************
				
				/*Note: application_data file excludes all student-applicant combinations where apppriorityseq==0
						Because of that, there are some student-school combinations present in temp1 below but
						not in temp2. The correct probabilities of assignment to an innovation schools are, however,
						fixed at this point to be unique at the kid level (i.e. fixed in temp2).
						
						Therefore, when merging temp1 and temp2 at maskedid-appschoolnum level, redefine innovation
						score to be unique at kid level
				*/
				
				
				u `temp1', clear
				merge 1:1 maskedid appschoolnum using `temp2'
				keep if _merge!=2
				drop _merge
				
				foreach type in fc oc fc_3cmo fc_n3cmo oc_3cmo oc_n3cmo nonc_fc{

					*Redefine pscore at kid level

					bys maskedid: egen temp_`type' = max(pscore_study`type'_`file')
					replace pscore_study`type'_`file' = temp_`type'
					
					drop temp_`type'
				
					** If applied to any first-choice charter or other-choice charter, change pscore to 0 instead of missing
					replace pscore_study`type'_`file' = 0 if pscore_study`type'_`file' == . & any_study_`type' == 1
					
					replace has_study`type'_pscore = 1 if pscore_study`type'_`file' != .
					
					replace has_study`type'_pscore =0 if has_study`type'_pscore==.
				}
				
				*find school code of the first-choice charter
				gen appschoolnum_fc_tmp = appschoolnum if apprank == 1 & appstudy == 1
				bys maskedid: egen appschoolnum_fc = max(appschoolnum_fc_tmp)				
				drop appschoolnum_fc_tmp
				
				*find school code of the first-choice non-charter
				gen appschoolnum_nonc_fc_tmp = appschoolnum if appstudy_nonc_fc == 1
				bys maskedid: egen appschoolnum_nonc_fc = max(appschoolnum_nonc_fc_tmp)				
				drop appschoolnum_nonc_fc_tmp
	
				replace has_study_pscore = 0 if has_study_pscore == .	
				
				*Countinue count of applicants and applications
				
					*All applicants who rank charter first
						unique maskedid if any_study_fc
						matrix obs[3,1]= r(sum)
						
						count if any_study_fc
						matrix obs[3,2]= r(N)
						
					*All applicants who rank charters, but not first
						unique maskedid if any_study_oc
						matrix obs[3,3]= r(sum)
						
						count if any_study_oc
						matrix obs[3,4]= r(N)
					
					*All applicants to first-choice & other-choice charters (all charters)
						unique maskedid if (any_study_fc | any_study_oc)
						matrix obs[3,5]= r(sum)
						
						count if (any_study_fc | any_study_oc)
						matrix obs[3,6]= r(N)
						
					drop any_study any_study_fc any_study_oc
						
					*Applicants with nonmissing score (first-choice charter)
					*keep if _merge==3
					
					unique maskedid if has_studyfc_pscore
					matrix obs[4,1]= r(sum)
					
					count if has_studyfc_pscore
					matrix obs[4,2]= r(N)
					
					*All applicants to other charters
						unique maskedid if has_studyoc_pscore
						matrix obs[4,3]= r(sum)
						
						count if has_studyoc_pscore
						matrix obs[4,4]= r(N)
					
					*All applicants to first choice charters and other charters with non missing pscore (for either)
						unique maskedid if (has_studyfc_pscore | has_studyoc_pscore)
						matrix obs[4,5]= r(sum)
						
						count if (has_studyfc_pscore | has_studyoc_pscore)
						matrix obs[4,6]= r(N)	
				
			tempfile temp
			sa `temp'
			
			u "`cleandata'sims_csap_wide_20102014 `simsdate'", clear
			unique maskedid
			
			merge 1:m maskedid using `temp'
			*keep if applicants only
			keep if _merge!=1
			drop _merge
			
			*Construct baseline and outcome variables
			
				foreach x in 	spyear	schoolnum charter alternative female black white hisp asian otherrace ///
								repeat	sped	foodfred	lep	bilingual ///
								gifted gt_ela gt_math	gt_math_ela	gt_other	///
								raw_ss_math	 ss_math		pp_up_math		prof_up_math	adv_math ///
								raw_ss_reading	ss_reading	pp_up_reading 	prof_up_reading	adv_reading	///
								raw_ss_writing ss_writing	pp_up_writing	prof_up_writing adv_writing{
						
						gen prebaseline`x'=.
						gen baseline`x'=.	
						gen outcome`x'=.
				
					foreach grade in 2 3 4 5 6 7 8 9 10{
					
						replace prebaseline`x' = `x'`grade' if pregradebase=="`grade'"
						replace baseline`x' = `x'`grade' if gradebase=="`grade'"
						replace outcome`x' = `x'`grade' if appgrade=="`grade'"
						
					}
					
				}
				
				foreach x in prebaseline baseline outcome{
					*Study charter
					gen `x'study= (inlist(`x'schoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(`x'schoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
						inlist(`x'schoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
						) if `x'spyear!=.
						
					gen `x'study_nonc_fc = (`x'schoolnum == appschoolnum_nonc_fc) if `x'spyear != .
					
					gen `x'study_fc = (`x'schoolnum == appschoolnum_fc) if `x'spyear != .
					gen `x'study_oc = ((`x'schoolnum != appschoolnum_fc) & ///
										(inlist(`x'schoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
										inlist(`x'schoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
										inlist(`x'schoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
										)) if `x'spyear!=.

					*First-choice, 3 CMO charters
					gen `x'study_fc_3cmo = ((`x'schoolnum == appschoolnum_fc) & ///
											(inlist(`x'schoolnum, 392, 393, 394, 426, 428, 429, 435, 436) | ///
											inlist(`x'schoolnum, 438, 443, 444, 445, 462, 481, 498, 513) ///
											)) if `x'spyear!=.
					
					*First-choice, non 3-CMO charters
					gen `x'study_fc_n3cmo = ((`x'schoolnum == appschoolnum_fc) & ///
												(inlist(`x'schoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
												inlist(`x'schoolnum, 330, 391, 441, 446, 485, 496, 512) ///
											)) if `x'spyear!=.
											
					*Other-choice, 3 CMO charters
					gen `x'study_oc_3cmo = ((`x'schoolnum != appschoolnum_fc) & ///
											(inlist(`x'schoolnum, 392, 393, 394, 426, 428, 429, 435, 436) | ///
											inlist(`x'schoolnum, 438, 443, 444, 445, 462, 481, 498, 513) ///
											)) if `x'spyear!=.
					
					*OTher-choice, non 3-CMO charters
					gen `x'study_oc_n3cmo = ((`x'schoolnum != appschoolnum_fc) & ///
												(inlist(`x'schoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
												inlist(`x'schoolnum, 330, 391, 441, 446, 485, 496, 512) ///
											)) if `x'spyear!=.
																
											
				}
					
					
				
				keep  maskedid lastname firstname appschoolnum appschoolname appcharter appstudy appstudy_fc appstudy_oc appstudy_fc_3cmo appstudy_fc_n3cmo appstudy_oc_3cmo appstudy_oc_n3cmo appstudy_nonc_fc ///
						appgrade apprank family offer pregradebase gradebase prebaseline* baseline* outcome* ///
						pscore_charter_`file' pscore_studyfc_`file' pscore_studyoc_`file' pscore_studyfc_3cmo_`file' pscore_studyfc_n3cmo_`file' ///
						pscore_studyoc_3cmo_`file' pscore_studyoc_n3cmo_`file' pscore_studyoc_n3cmo_ pscore_studynonc_fc_`file' has_* pscore_school

				order  outcome*, after(gradebase)
				renvars outcome*, predrop(7)
				order appgrade gradebase pscore_charter_`file' pscore_studyfc_`file' pscore_studyoc_`file' pscore_studyfc_3cmo_`file' pscore_studyfc_n3cmo_`file' pscore_studyoc_3cmo_`file' pscore_studyoc_n3cmo_`file', after(maskedid)
				
				*Study Charter offer
					gen offer_study = (offer==1) if appstudy == 1
					
				*Non charter offer from first choice
					gen offer_study_nonc_fc = (offer==1) if appstudy_nonc_fc == 1
					
				*Study Charter offer from first-choice
					gen offer_study_fc = (offer==1) if appstudy_fc == 1
					gen offer_study_fc_3cmo = (offer==1) if appstudy_fc_3cmo == 1
					gen offer_study_fc_n3cmo = (offer==1) if appstudy_fc_n3cmo == 1
				
				*Study Charter offer from non first-choice
					gen offer_study_oc = (offer==1) if appstudy_oc == 1
					gen offer_study_oc_3cmo = (offer==1) if appstudy_oc_3cmo == 1
					gen offer_study_oc_n3cmo = (offer==1) if appstudy_oc_n3cmo == 1
				
					
					order offer_study offer_study_fc offer_study_oc offer_study_fc_3cmo offer_study_fc_n3cmo offer_study_oc_3cmo offer_study_oc_n3cmo, after(offer)
					
				*Count obs with baseline
					
					*All applicants to first-choice charter 
					unique maskedid if has_studyfc_pscore & baselinespyear==2012
					matrix obs[5,1]= r(sum)
					
					count if has_studyfc_pscore & baselinespyear==2012
					matrix obs[5,2]= r(N)
					
					*All applicants to other charters
					unique maskedid if has_studyoc_pscore & baselinespyear==2012
					matrix obs[5,3]= r(sum)
					
					count if has_studyoc_pscore & baselinespyear==2012
					matrix obs[5,4]= r(N)
				
					*All applicants to both first-choice and other charters
					unique maskedid if (has_studyfc_pscore | has_studyoc_pscore) & baselinespyear==2012
					matrix obs[5,5]= r(sum)
					
					count if (has_studyfc_pscore | has_studyoc_pscore) & baselinespyear==2012
					matrix obs[5,6]= r(N)
			
				** grade dummies
					tab grade, gen(g_)	
				
				*In sample means we have well-defined instrument
					gen in_studynonc_fc_sample	=	(has_studynonc_fc_pscore==1 & offer_study_nonc_fc!=.)
					gen in_studyfc_sample	=	(has_studyfc_pscore==1 & offer_study_fc!=.)
					gen in_studyoc_sample	=	(has_studyoc_pscore==1 & offer_study_oc!=.)
					gen in_studyfc_3cmo_sample = (has_studyfc_3cmo_pscore == 1 & offer_study_fc_3cmo != . )
					gen in_studyfc_n3cmo_sample = (has_studyfc_n3cmo_pscore == 1 & offer_study_fc_n3cmo != .)
					gen in_studyoc_3cmo_sample = (has_studyoc_3cmo_pscore == 1 & offer_study_oc_3cmo != . )
					gen in_studyoc_n3cmo_sample = (has_studyoc_n3cmo_pscore == 1 & offer_study_oc_n3cmo != .)
					
					gen has_sc 	=	ss_math!=. | ss_writing!=. | ss_reading!=.
					
					label var in_studyfc_sample "In first-choice charters analysis sample (Z and Pscores nonmissing)"
					
					label var in_studyoc_sample "In lower-choice charteres analysis sample (Z and Pscore nonmissing)"
								
					foreach x in math reading writing{
						gen has_sc_`x' = (ss_`x'!=.)
					}
					
					
					order has_*, after(pscore_studyoc_n3cmo_`file')
					order maskedid in_*_sample appschoolnum appschoolname appcharter appstudy appstudy_fc appstudy_oc appstudy_fc_3cmo appstudy_fc_n3cmo appstudy_oc_3cmo appstudy_oc_n3cmo ///
						apprank appgrade family pscore_charter_`file' pscore_studyfc_`file' pscore_studyoc_`file' ///
						pscore_studyfc_3cmo_`file' pscore_studyfc_n3cmo_`file' pscore_studyoc_3cmo_`file' pscore_studyoc_n3cmo_`file' pscore_studynonc_fc_`file' pscore_school offer 
					ren gradebase baselinegrade
					order study study_fc study_oc study_fc_3cmo study_fc_n3cmo study_oc_3cmo study_oc_n3cmo study_nonc_fc, after(charter)

				save "`cleandata'mdrd_denver_fc_nonfc_`file' `outputdate'.dta", replace
				
				svmat obs
				keep obs*
				keep if obs2!=.
				gen reason=""
				ren obs1 applicants_charter_fc
				ren obs2 applications_charter_fc
				ren obs3 applicants_charters_nonfc
				ren obs4 applications_charters_nonfc
				ren obs5 applicants_either
				ren obs6 applications_either
				replace reason ="All applicants" if _n==1
				replace reason ="Applicants to grades 4 through 10" if _n==2
				replace reason ="Applicants to (charters, innovation, either)" if _n==3
				replace reason ="Applicants with nonmissing pscore" if _n==4
				replace reason ="Denver at baseline (count, not dropped)" if _n==5
				order reason, first
				outsheet using "`tables'`file'_fc_nonfc_obscount.csv", comma replace
	}
}
