/* 	Mayara Felix

	Goal: 	Create analysis files (instruments + treatment + baseline + outcome) for MDmRD
			One file is created for each instrument
			
	IMPORTANT: 	Files pscore_charter_formula_`date' and pscore_charter_frequency_`date' must be
				placed into cleandata folder. These files are prepared using Yusuke's pscore
				routines, and can be found here: Y:\DPS\Yusuke\data.
				
				The code that generates propensity scores are:
					"Y:\DPS\Yusuke\create_pscore_charter_year_two_20150420.do", which runs:
					"Y:\DPS\Yusuke\create_pscore_charter_subroutine_20150420.do"
	
	SIMS NOTE:	1) 	Starting in 4/28/2015, the SIMS includes alternative schools in the score
					centering
*/	

clear all
cap log close
set more off
set trace off
set maxvar 32767
adopath + "Z:\ado\plus\"

pause on

local rawdata   "Y:/DPS/Raw Data/"
local asgndata  "Y:/DPS/Raw Data/2013_03_21_enrollment-match/"
local testdata  "Y:/DPS/Raw Data/2013_10_03_tests2013/"
local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local yusuke	 "Y:/DPS/yusuke/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/"

local pscore_setup = 1		/*Propensity score matching setup*/

local simsdate 		20150603	/*20150428 20150420 20150107*/
local outputdate 	20170310    /*20160815 20160715 20160516 	20160510 20160428    20160421 20160208 */
local appdatadate 	year_one_20170310	/*year_one_20160815	year_one_20160512 year_one_20160509 year_one_20160208	 year_one_20150708 */
local typedate 20160815

matrix obs = J(50,50,.)
		
*Retrieve application, offer, and grade information from AT RUN file
insheet using "`asgndata'1_AT_RUN_022262012.csv", double clear
	ren maskedstudentids maskedid
	ren requestedschool appschoolnum
	
	*Not sure what rank=99 means, ask Atila
	gen apprank = choiceselection
	replace apprank = apprank*10 if apprank<10
	replace apprank = floor(apprank/10)
	
	drop if maskedid=="#VALUE!"
	destring maskedid, ignore("#VALUE!") replace

	gen offer = (status=="A")
	bys maskedid appschoolnum: egen max_offer = max(offer)
	drop offer
	ren max_offer offer
	
	gen family = (family!="")
	
	keep grade maskedid offer appschoolnum apprank family

	ren grade appgrade
	duplicates drop appgrade maskedid offer appschoolnum apprank, force
	
	duplicates tag maskedid appgrade appschoolnum, gen(dup)
	tab dup
	
	*Cases of multiple rankings for the same school. Choose the smallest rank
	bys maskedid appgrade appschoolnum: egen min_rank = min(apprank)
	replace apprank = min_rank if dup
	
	*Keep if has nonmissing family link within school
	bys maskedid appgrade appschoolnum: egen max_fam=max(family)
	drop family
	ren max_fam family
	
	duplicates drop maskedid appgrade appschoolnum, force

	*All applicantions (school level)
		unique maskedid 
		matrix obs[1,1]= r(sum)
		matrix obs[1,2]= r(N)
	
	** All types
		preserve
		
		use "`cleandata'type_for_all_`typedate'.dta", clear

		duplicates drop maskedid appgrade appschoolnum, force

		unique type_all
		matrix obs[1,5] = r(sum)
		
		unique type_fc_all
		matrix obs[1,10] = r(sum)
		
		unique type_sc_all
		matrix obs[1,11] = r(sum)
		
		unique type_tc_all
		matrix obs[1,12] = r(sum)
		
		
		** No baseline for "EC"
				drop if appgrade == "EC"
				destring appgrade, replace
				
				gen gradebase = 	appgrade-1
				gen pregradebase =	appgrade-2
				
				tostring appgrade, replace
				tostring gradebase, replace
				
				replace gradebase = "EC" if gradebase == "-1"
				replace gradebase = "K" if gradebase == "0"
				
				destring maskedid, replace 
				merge m:1 maskedid using "`cleandata'sims_csap_wide_20102014 `simsdate'"
				drop if _merge == 2
				drop _merge
				
				gen baselinespyear = .
				
				foreach grade in EC K 1 2 3 4 5 6 7 8 9 10 11 {
				replace baselinespyear = spyear`grade' if gradebase == "`grade'"
				}
								
				unique type_all if baselinespyear == 2012
				matrix obs[1,6] = r(sum)	
				
				unique type_fc_all if baselinespyear == 2012
				matrix obs[1,13] = r(sum)	
				
				unique type_sc_all if baselinespyear == 2012
				matrix obs[1,14] = r(sum)	

				unique type_tc_all if baselinespyear == 2012
				matrix obs[1,15] = r(sum)	
				
			restore
		
	*Identify charters in application
	preserve
		insheet using "`rawdata'dps_charters_20112012_20150420 update.csv", double clear
		keep schoolname schoolnum charter alternative
		ren schoolnum appschoolnum
		gen appcharter = (charter==1)
		keep appschoolnum appcharter
		tempfile charter
		sa `charter'
	restore
	
	merge m:1 appschoolnum using `charter'
	keep if _merge!=2
	drop _merge
	replace appcharter=0 if appcharter==.
	
	*Study charters
	gen appstudy = (	inlist(appschoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(appschoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
						inlist(appschoolnum,462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
					)
					
	**Study no-excuses charters
	gen appnoex =  (    inlist(appschoolnum, 392, 393, 394, 426, 428, 429, 435, 436) | ///
						inlist(appschoolnum, 438, 443, 444, 445, 462, 481, 498, 513) ///
					)
	
	**Study non no-excuses charters
	gen appnonnoex = (  inlist(appschoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(appschoolnum, 330, 391, 441, 446, 485, 496, 512) ///
					)


	gen appstudy_fc = ( apprank == 1 & appstudy == 1)
	gen appstudy_sc = ( (apprank == 1  | apprank == 2) & appstudy == 1)
	gen appstudy_tc = ( (apprank == 1 | apprank == 2 | apprank == 3) & appstudy == 1)

	
	*Get latest school names
	preserve
		u "`cleandata'sims_csap_long_20102014 `simsdate'", clear
		keep schoolnum schoolname spyear
		bys schoolnum: egen max_year = max(spyear)
		keep if spyear==max_year
		drop spyear max_year
		
		*schools with slighlty different names
		duplicates drop schoolnum, force
		ren schoolnum appschoolnum
		ren schoolname appschoolname
		tempfile schools
		sa `schools'
	restore	
	
	merge m:1 appschoolnum using `schools'
	keep if _merge!=2
	drop _merge
	order maskedid appschoolnum appschoolname
	label var appschoolname "Requested school name"
	
	tab appgrade
	
	*Merge with type data so that we can count how many types there are
	tostring maskedid, replace
	
	preserve
	use "`cleandata'fulltypes`typedate'.dta", clear

	duplicates drop maskedid, force
	keep maskedid type_all type_fc_all type_sc_all type_tc_all
	
	tempfile type_all
	sa `type_all'
	
	restore
	
	merge m:1 maskedid using `type_all'
	drop _merge
	
	destring maskedid, gen(maskedid_tmp)
	drop maskedid
	rename maskedid_tmp maskedid
	ren type_all applicant_type
	ren type_fc_all applicant_type_fc
	ren type_sc_all applicant_type_sc
	ren type_tc_all applicant_type_tc
	label var applicant_type "Unique combination of all bucket preferences and priorities"
	
	save "`cleandata'AT_RUN_school_level.dta", replace

if `pscore_setup' == 1 {
	
	*One file for each instrument
		foreach file in simulation frequency formula {
			foreach sch_type in study /*noex nonnoex */	{
			
			if "`file'"=="simulation"{
				*20150420 is for old definition of simulated pscore, aggregated at student level
				local pscoredate  20160816 /* 20160425 20160421 */
			}
			else{
				local pscoredate year_one_20160512
			}
					
			preserve
				u "`cleandata'pscore_`sch_type'_`file'_`pscoredate'.dta", clear
				gen spyear =2013 
				cap ren maskedstu maskedid
				cap destring maskedid, replace
						
				*Grades that have scores
				merge 1:m maskedid using "`cleandata'AT_RUN_school_level.dta", keepusing(offer appgrade appcharter appstudy appstudy_fc appstudy_sc appstudy_tc appnoex appnonnoex appschoolnum appschoolname apprank family applicant_type applicant_type_fc applicant_type_sc applicant_type_tc)
				*Applicants to any charters
				bys maskedid: egen any_`sch_type' = max(app`sch_type')
				bys maskedid: egen any_study_fc = max(appstudy_fc)
				bys maskedid: egen any_study_sc = max(appstudy_sc)
				bys maskedid: egen any_study_tc = max(appstudy_tc)
				
				*All kids with missing pscores should be treated as pscore==0
				replace pscore = 0 if pscore==. & any_`sch_type'==1
				
				gen has_pscore = (pscore!=.)
				drop _merge
				
				*Grades 4-10 (baseline and outcome scores are available)
				tab appgrade
				keep if inlist(appgrade,"04","05","06","07","08","09","10")
				unique maskedid
				matrix obs[2,1]= r(sum)
				matrix obs[2,2]= r(N)
				
				unique applicant_type
				matrix obs[2,5]= r(sum)
				
				unique applicant_type_fc
				matrix obs[2,10] = r(sum)
				
				unique applicant_type_sc
				matrix obs[2,11] = r(sum)				
				
				unique applicant_type_tc
				matrix obs[2,12] = r(sum)
								

				*All applicants to charters
					unique maskedid if any_`sch_type'
					matrix obs[3,1]= r(sum)
					
					count if any_`sch_type'
					matrix obs[3,2]= r(N)
					
					unique applicant_type if any_`sch_type'
					matrix obs[3,5]= r(sum)

					unique applicant_type_fc if any_`sch_type'
					matrix obs[3,10]= r(sum)
					
					unique applicant_type_sc if any_`sch_type'
					matrix obs[3,11]= r(sum)

					unique applicant_type_tc if any_`sch_type'
					matrix obs[3,12]= r(sum)
					
					drop any_`sch_type'
					
				*Applicants with risksets and offers
				*keep if _merge==3
				
				unique maskedid if has_pscore
				matrix obs[4,1]= r(sum)
				
				count if has_pscore
				matrix obs[4,2]= r(N)

				unique applicant_type if has_pscore
				matrix obs[4,5]= r(sum)
				
				unique applicant_type_fc if has_pscore
				matrix obs[4,10]= r(sum)
				
				unique applicant_type_sc if has_pscore
				matrix obs[4,11]= r(sum)
				
				unique applicant_type_tc if has_pscore
				matrix obs[4,12]= r(sum)

				
				local row = 5
				
				foreach choice in fc sc tc {
				
				
				*All applicants to first/upto second/upto third-choice charters
					unique maskedid if any_study_`choice' == 1
					matrix obs[`row',1]= r(sum)
					
					count if any_study_`choice' == 1
					matrix obs[`row',2]= r(N)
					
					unique applicant_type if any_study_`choice' == 1
					matrix obs[`row',5]= r(sum)

					unique applicant_type_fc if any_study_`choice' == 1
					matrix obs[`row',10]= r(sum)
					
					unique applicant_type_sc if any_study_`choice' == 1
					matrix obs[`row',11]= r(sum)

					unique applicant_type_tc if any_study_`choice' == 1
					matrix obs[`row',12]= r(sum)
					
					drop  any_study_`choice'
					
					local ++row
				}
				
				destring appgrade, replace
				
				gen gradebase = 	appgrade-1
				gen pregradebase =	appgrade-2
				
				tostring appgrade, replace
				tostring gradebase, replace
				tostring pregradebase, replace
				
				tempfile temp1
				sa `temp1'
			restore
			
			
			*Get pscore with probability of assignment at each school
			if "`file'"!="simulation"{			
				preserve	
					u "`cleandata'application_data_`appdatadate'", clear
					
					ren masked maskedid
					ren requestedschool appschoolnum
					destring maskedid, replace
				
					if "`file'"=="frequency"{
						ren frequency_offer pscore_frequency
						drop pscore_formula
					}
						
					drop pscore_charter_formula
					bys maskedid appschoolnum: egen pscore_school_`file' = sum(pscore)
					
					duplicates drop maskedid appschoolnum, force
					keep maskedid appschoolnum pscore_school
					tempfile temp2
					sa `temp2'
				restore
			}
			
			else{
				preserve
					u "`cleandata'pscore_simulation_school_grade", clear
					keep maskedid pscore_school appschoolnum
					destring maskedid, replace
					tempfile temp2
					sa `temp2'
				restore	
			}
			
				*Combine pscore files
				u `temp1', clear
				merge 1:1 maskedid appschoolnum using `temp2'
				keep if _merge!=2
				drop _merge
				
				tempfile temp
				sa `temp'
			
			u "`cleandata'sims_csap_wide_20102014 `simsdate'", clear
			unique maskedid
			
			merge 1:m maskedid using `temp'
			*keep if applicants only
			keep if _merge!=1
			drop _merge
			
			*Construct baseline and outcome variables
			
				foreach x in 	spyear	schoolnum charter alternative female black white hisp asian otherrace ///
								repeat	sped	foodfred	lep	bilingual ///
								gifted gt_ela gt_math	gt_math_ela	gt_other	///
								raw_ss_math	 ss_math		pp_up_math		prof_up_math	adv_math ///
								raw_ss_reading	ss_reading	pp_up_reading 	prof_up_reading	adv_reading	///
								raw_ss_writing ss_writing	pp_up_writing	prof_up_writing adv_writing{
						
						gen prebaseline`x'=.
						gen baseline`x'=.	
						gen outcome`x'=.
				
					foreach grade in 2 3 4 5 6 7 8 9 10{
					
						replace prebaseline`x' = `x'`grade' if pregradebase=="`grade'"
						replace baseline`x' = `x'`grade' if gradebase=="`grade'"
						replace outcome`x' = `x'`grade' if appgrade=="`grade'"
						
					}
					
				}
				
				foreach x in prebaseline baseline outcome{
					gen `x'study= (inlist(`x'schoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(`x'schoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
						inlist(`x'schoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
					) if `x'spyear!=.
					
					gen `x'noex= (inlist(`x'schoolnum, 392, 393, 394, 426, 428, 429, 435, 436) | ///
						inlist(`x'schoolnum, 438, 443, 444, 445, 462, 481, 498, 513) ///
					) if `x'spyear!=.
			
								
					gen `x'nonnoex= (inlist(`x'schoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(`x'schoolnum, 330, 391, 441, 446, 485, 496, 512) ///
					) if `x'spyear!=.

				}
				
				
				if "`file'"!="simulation"{			
					if "`sch_type'"=="study"{
					rename pscore_charter_`file' pscore_study_`file'
					}
				}
				
				
				keep  maskedid lastname firstname appschoolnum appschoolname appcharter appstudy appnoex appnonnoex appgrade apprank family offer pregradebase gradebase prebaseline* baseline* outcome* ///
					 pscore_`sch_type'_`file' pscore_school has_pscore applicant_type

				order  outcome*, after(gradebase)
				renvars outcome*, predrop(7)
				order appgrade gradebase pscore_`sch_type'_`file', after(maskedid)
				
				*Study Charter offer to kids who applied to charters
					gen offer_`sch_type' = (offer==1) if app`sch_type' ==1
					order offer_`sch_type', after(offer)
				
				** grade dummies
					tab grade, gen(g_)	
				
				gen app_right_grade = (spyear==2013)
					
					
				*In sample means we have well-defined instrument (you applied to a charter)
					gen in_sample	=	(has_pscore==1 & offer_`sch_type'!=.)
					gen has_sc 	=	ss_math!=. | ss_writing!=. | ss_reading!=.
					
					label var in_sample "In analysis sample (Z and pscores nonmissing)"
					
					foreach x in math reading writing{
						gen has_sc_`x' = (ss_`x'!=.)
					}
					
					order has_*, after(pscore_`sch_type'_`file')
					order maskedid in_sample app_right_grade appschoolnum appschoolname appcharter appstudy appnoex appnonnoex apprank appgrade family pscore_`sch_type'_`file' pscore_school offer 
					ren gradebase baselinegrade
					order `sch_type', after(charter)
				save "`cleandata'mdrd_denver_`sch_type'_`file' `outputdate'.dta", replace
			
		}
	}
}
