/*	Mayara Felix
	3/2/2015
	
	Goal: 	To create qualification instruments for DPS student assignment mechanism
			STEPS:
			
				1) Create risksets. A riskset is the combination of schools, capacity programs, and priorities
				in each of these capacity programs. Any two students are in the same riskset if and only if
				they rank the same schools, at the same capacity programs, with the same priority levels.
				
				2) 	Find each riskset's cuttof: the lowest "priority" (lottery number + apppriorityseq) that received 
												an offer in that riskset
				
				3) 	A student qualifies for an offer in that riskset if their priority is smaller than
					or equal to the cuttoff

*/

clear all
cap log close
set more off
set trace off
set maxvar 32767
set seed 42
adopath + "Z:\ado\plus\"

pause on 

local rawdata   "Y:/DPS/Raw Data/"
local asgndata  "Y:/DPS/Raw Data/2013_03_21_enrollment-match/"
local testdata  "Y:/DPS/Raw Data/2013_10_03_tests2013/"
local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local  tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/"

matrix oversub = J(30,5,.)

local simsdate 		20150603		/* 20150428 20150420 20150107*/
local outputdate 	20170310 		/*20150708	 20150608 20150604 20150528 20150420 20150224 20150211 20141201 20141124 20141118 20141110 20141106*/0
local qual_setup    	= 1
local merge 			= 0
local sch_demo			= 0

if `qual_setup' == 1 {

	**********************************************
	* STEP 1 clean assignment data
	**********************************************
	
	insheet using "`asgndata'1_AT_RUN_022262012.csv", names clear
	drop v15-v36
	
	ren maskedstudentids maskedid
	ren requestedschool appschoolnum
		
	drop if maskedid=="#VALUE!"
	destring maskedid, ignore("#VALUE!") replace
	
	replace waitinglistrank = "" if waitinglistrank == "NULL"
	destring waitinglistrank, replace
	gen yearapp = 2012
	
	*Identify charters in application
		preserve
			insheet using "`rawdata'dps_charters_20112012_20150420 update.csv", double clear
			keep schoolname schoolnum charter alternative
			ren schoolnum appschoolnum
			gen appcharter = (charter==1)
			keep appschoolnum appcharter
			tempfile charter
			sa `charter'
		restore
		
		merge m:1 appschoolnum using `charter'
		keep if _merge!=2
		drop _merge
	
	*Study charters
	gen appstudy= (inlist(appschoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
					inlist(appschoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
					inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
					)

	
	*APP covariates
		gen apprank = choiceselection
		replace apprank = apprank*10 if apprank<10
		replace apprank = floor(apprank/10)

		*Charter app covs
		bys maskedid: egen num_schools = count(appschoolnum)
		bys maskedid: egen num_study = count(cond(appstudy==1,appschoolnum,.))
		gen pct_study = num_study/num_schools
		
		gen temp = (appstudy==1 & apprank==1)
		bys maskedid: egen rank1_study = max(temp)

		drop appcharter apprank temp
	
	forval i = 1/5 {
		qui replace choiceselection = `i'*10 if choiceselection==`i'
	}
	
	encode capacityprogram, gen(cap_code)

	
	sum choiceselection
	local max_choice = r(max) + 1
	
	* Added on march 10, 2016: Got a study charter seat
	gen gotch = 1 if status=="A" & appstudy == 1
	replace gotch = 0 if gotch == .
	
	*Flag for ever guaranteed at a any charter
	gen guarantee = (apppriorityseq==1)
	bys maskedid appstudy: egen ever_guaranteed = max(guarantee)

	*Flag for always ineligible
	gen ineligible = (apppriorityseq==0)
	bys maskedid appstudy: egen max_ineligible = max(ineligible)
	bys maskedid appstudy: egen min_ineligible = min(ineligible)
	
	gen always_ineligible = (max_ineligible==1 & min_ineligible==1)
	drop guarantee ineligible max_inel min_inel
	
	tempfile y2012
	save `y2012', replace
	
	************************************************************
	**************** Qualification instruments *****************
	************************************************************
	
	foreach qualtype in standard naive{
			u `y2012', clear
			
			if "`qualtype'"=="standard"{
				di in red "Keep guaranteed and ineligible kids"
			}
			else if "`qualtype'"=="naive"{
				drop if ever_guaranteed==1 | always_ineligible==1
			}
			
			*******************************************************
			********** Define bucket-level qualification **********
			*******************************************************

			*Qualification will be based on "lotto priority" = lotterynumber + application priority number
			gen double lottopri = lotterynumber + apppriorityseq
			
			*Restrict sample to year and grades of interest, charter applications
			
				keep if inlist(grade,"04","05","06","07","08","09","10") & yearapp == 2012 & appstudy==1
			
			*Keep variables of interest
				keep maskedid yearapp grade choiceselection appschoolnum apppriorityseq cap_code lottopri gotch family ///
					 num_schools num_study pct_study rank1_study appstudy dstlink

			*School-capacity-program-prioritycode status (i.e. ignore school ranking)
			egen bucket = group(yearapp grade appschoolnum apppriorityseq cap_code), missing
			
			 
			*Find combinations of buckets
			
				*Because original data was long on choices, for each cell we now edit the dummy variable for whether
				*the kid was in that cell, regardless of their choices. So we define this by maskedid. Similarly,
				*we keep track of the max lottopri that got an offer in that cell.
				
				*Generate riskset separately for levels of qualification - unique set of schools-capacity-priorities, regardless of ranking					
				foreach choice in 20 30 `max_choice'{
					
					preserve
						*keep maskedid bucket choiceselection
						*duplicates drop
						* this step actually does not drop any duplicates. 
						duplicates drop maskedid bucket choiceselection, force
						
						gen in_bucket_lt`choice' = 1 if choiceselection < `choice'
			
						qui tab bucket if choiceselection < `choice', gen(in_bucket_lt`choice') 

						foreach var of varlist in_bucket_lt`choice'*{
							bys maskedid: egen max_`var' = max(`var')
						}
						
						keep maskedid max_*
						
						renvars max_*, predrop(4)
						duplicates drop maskedid, force
						
						*Check that each kid is in some cell
						egen check = rowmax(in_*)
						tab check
												
						*Generate riskset - unique set of schools-capacity-priorities, regardless of ranking
						egen riskset_lt`choice' = group(in_*), missing

						keep maskedid riskset_lt`choice'
						order maskedid riskset_lt`choice'
						
						duplicates drop
						unique maskedid
						
						save "`cleandata'da_qual`qualtype'_buckets_lt`choice'", replace
						
					restore		
					
					*Merge back using bucket
					
					merge m:1 maskedid using "`cleandata'da_qual`qualtype'_buckets_lt`choice'"
					tab _merge
					assert _merge == 3
					drop _merge
				}
				order maskedid riskset*
				
				
				*Define qualification at bucket level
				*for a certain appschool, priority, program code, find the largest lottopri there. 
					bys bucket: egen bucket_maxlottopri_find = max(cond(gotch==1,lottopri,.))
					replace bucket_maxlottopri_find =0 if bucket_maxlottopri_find==.
					
					bys bucket: egen bucket_maxlottopri = max(bucket_maxlottopri_find)
					gen bucket_qual = (lottopri <= bucket_maxlottopri)
				
					order maskedid choiceselection bucket riskset* bucket_qual
				
				*******************************************************
				********** Define school-level qualification **********
				*******************************************************
				
				foreach choice in 20 30 `max_choice'{
					
					*Ones if set of rankings, missing otherwise
					gen bucket_lt`choice' = 1 if choiceselection <`choice'
					
					*Qualify at set of rankings we are interested in?
					bys maskedid bucket_lt`choice': egen qual_lt`choice' = max(bucket_qual)
					
					*If not set of rankings we are interested in, code as missing
					replace qual_lt`choice'  =. if bucket_lt`choice'==.
					label var qual_lt`choice' "Qualifies in any buckets ranked below `choice'"
						
					
				}
				
				*Charter offer at the school-level (instead of at the program-level)
				bys maskedid grade yearapp appschoolnum: egen gotch_sch = max(gotch)
				drop gotch
				
				*Note - need to keep appschoolnum because treatment is defined as enrolling in any listed charter
					keep maskedid grade yearapp appschoolnum riskset* qual_lt* num_schools num_study pct_study rank1_study gotch_sch appstudy
					duplicates drop 
					order maskedid yearapp grade riskset*
					
					sort yearapp grade riskset*
					ren grade appgrade
					
				*Save separate datasets
				foreach choice in 20 30 `max_choice'{	
					
						preserve
						
					*Keep only schools below the choices of interest	
						keep if qual_lt`choice'!=.
						
						if "`choice'" == "20" {
							** We want first-choice offer, not first-choice qualification.
							replace qual_lt20 = 0 if gotch_sch == 0 & qual_lt20 == 1
						}
						drop gotch_sch
						
					save "`cleandata'da_qual`qualtype'_lt`choice' `outputdate'", replace
						
						restore
				}
		}	/*Close choice loop*/
	}	/*Close standard & naive loop*/

	**********************************************
	***** Merge enrollment and outcome data ******
	**********************************************
if `merge'==1{
	
	foreach qualtype in standard naive{
		foreach choice in 20 30 100{
		
			*merge m:1 with enrollment and outcome data	
		
				u "`cleandata'sims_csap_wide_20102014 `simsdate'", clear
				merge 1:m maskedid using "`cleandata'da_qual`qualtype'_lt`choice' `outputdate'"
				
				*Keep if in qual sample only
				keep if _merge!=1
				drop _merge
			
			destring appgrade, replace
			gen double gradebase = appgrade -1
			gen double pregradebase = appgrade - 2
			
			tostring appgrade, replace
			tostring gradebase, replace
			tostring pregradebase, replace
				
		*Construct baseline and outcome variables
			gen baselineschoolname=""
			foreach x in 	spyear	schoolnum charter	alternative sped repeat ///
							female black white hisp asian otherrace ///
							foodfred	lep	bilingual ///
							gifted gt_ela gt_math	gt_math_ela	gt_other	///
							raw_ss_math	 ss_math		pp_up_math		prof_up_math	adv_math ///
							raw_ss_reading	ss_reading	pp_up_reading 	prof_up_reading	adv_reading	///
							raw_ss_writing ss_writing	pp_up_writing	prof_up_writing adv_writing ///
							raw_ss_science	ss_science	pp_up_science	prof_up_science	adv_science{
					
					gen prebaseline`x'=.
					gen baseline`x'=.	
					gen outcome`x'=.
			
				foreach grade in 2 3 4 5 6 7 8 9 10{
					replace prebaseline`x' = `x'`grade' if pregradebase=="`grade'"
					replace baseline`x' = `x'`grade' if gradebase=="`grade'"
					replace outcome`x' = `x'`grade' if appgrade=="`grade'"
				}
				
			}
			
			foreach x in prebaseline baseline outcome{
				gen `x'study= (inlist(`x'schoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
					inlist(`x'schoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444,445, 446) | ///
					inlist(`x'schoolnum,462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
				) if `x'spyear!=.
			
			}
					
			keep  maskedid riskset* qual_lt* appschoolnum appstudy appgrade gradebase num_schools num_study pct_study rank1_study prebaseline* baseline* outcome* ///
				 
			order  outcome*, after(gradebase)
			renvars outcome*, predrop(7)
			
			** grade dummies
				tab grade, gen(g_)
			
			gen app_right_grade = (spyear==2013)
			
					
			*In sample means we have well-defined instrument
			gen in_sample = (qual_lt`choice'!=.)
			gen has_sc 	=	ss_math!=. | ss_writing!=. | ss_reading!=.
			
			label var in_sample "Applicants to buckets in choices less than `choice'"

			foreach x in math reading writing science{
				gen has_sc_`x' = (ss_`x'!=.)
			}
			
			order has_*, last
			order maskedid appgrade riskset* appschoolnum
			order study, after(charter)
			ren gradebase baselinegrade
			order baselinegrade, after(baselineschoolname)
			
			save "`cleandata'mdrd_denver_qual`qualtype'_lt`choice' `outputdate'.dta", replace
		}
	}
}
	**********************************************
	****** School oversubscription patterns ******
	**********************************************
if `sch_demo'==1{

	foreach qualtype in standard naive{

		*Count # of kids ranking school first, and how that compares to school capacity
		insheet using "`asgndata'1_AT_RUN_022262012.csv", names clear
		
		ren maskeds maskedid
		ren requestedschool appschoolnum
		
		*Study charters
		gen appstudy= (	inlist(appschoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
							inlist(appschoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444,445, 446) | ///
							inlist(appschoolnum,462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
						)
			
		keep if inlist(grade,"04","05","06","07","08","09","10") & appstudy==1
		
		*Flag for ever guaranteed at a any charter
			gen guarantee = (apppriorityseq==1)
			bys maskedid appstudy: egen ever_guaranteed = max(guarantee)
		
		*Flag for always ineligible
			gen ineligible = (apppriorityseq==0)
			bys maskedid appstudy: egen max_ineligible = max(ineligible)
			bys maskedid appstudy: egen min_ineligible = min(ineligible)
			
			gen always_ineligible = (max_ineligible==1 & min_ineligible==1)
			drop guarantee ineligible max_inel min_inel
			
		if "`qualtype'"=="standard"{
			di in red "Keep guaranteed and ineligible kids"
		}
		else if "`qualtype'"=="naive"{
			drop if ever_guaranteed==1 | always_ineligible==1
		}
		
		gen apprank = choiceselection
		replace apprank = apprank*10 if apprank<10
		replace apprank = floor(apprank/10)
	
		keep maskedid appschoolnum apprank
		keep if apprank==1
		drop apprank
		
		drop if maskedid=="#VALUE!"
		destring maskedid, ignore("#VALUE!") replace
		
		bys appschoolnum: egen rank1_apps = count(maskedid)
		drop maskedid
		
		duplicates drop appschoolnum, force
		
		tempfile rank1
		sa `rank1'
	
	*Total school capacity
		insheet using "`asgndata'AssignmentTool/SchoolFile.csv", clear
		ren schoolcode appschoolnum
		
		gen appstudy= (	inlist(appschoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
					inlist(appschoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444,445, 446) | ///
					inlist(appschoolnum,462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
				)
		
		keep if inlist(grade,"04","05","06","07","08","09","10") & appstudy==1
		
		keep appschoolnum capacity
		collapse (sum) capacity, by (appschoolnum)
		
		merge 1:1 appschoolnum using `rank1'
		drop _merge
		
		gen rank1_oversubscribed = (capacity < rank1_apps)
		
		tempfile oversub
		sa `oversub'
		
		foreach choice in 20 30 100{
			
			preserve
				u "`cleandata'mdrd_denver_qual`qualtype'_lt`choice' `outputdate'.dta", clear
				
				*In Denver at Baseline
				keep if baselinespyear==2012
				
				*Risksets with var in offer
				bys riskset_lt`choice': egen sd_z_lt`choice' = sd(qual_lt`choice')
				label var sd_z_lt`choice' "Riskset with variation in instrument for choices below `choice'"	
				keep if sd_z_lt`choice'>0 & sd_z_lt`choice'!=.
			
				bys appschoolnum: egen rand_kids = count(maskedid)
				
				keep appschoolnum rand_kids
				duplicates drop
				drop if appschoolnum==.
				
				merge 1:1 appschoolnum using `oversub', nogen

				outsheet using "`tables'inDPSbase_qual`qualtype'_schools_lt`choice'.csv", comma replace

			restore
		}
	}
}
