/* Mayara Felix

	Goal: 	Create analysis files (instruments + treatment + baseline + outcome) for MDmRD
			One file is created for each instrument
			
Ignacio Rodriguez
03/31/2016

This file used to pull in types from a file called
"Y:/DPS/MDmeetsRD Denver/Data/full_student_type_20150420.dta"
but I wasn't able to locate how this file was generated or how the types were made
with that file you ended up with 61 points of support and 301 students in the sample
I updated this file so that it uses the 
"${cleandata}fulltypes20160208" file generated from do file #1 1_merge_simulated_pscore


*/	

clear all
cap log close
set more off
set trace off
set maxvar 32767
adopath + "Z:\ado\plus\"

global rawdata   "Y:/DPS/Raw Data/"
global asgndata  "Y:/DPS/Raw Data/2013_03_21_enrollment-match/"
global testdata  "Y:/DPS/Raw Data/2013_10_03_tests2013/"
global cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
global yusuke	 "Y:/DPS/yusuke/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/"

scalar setup =1	

local simsdate 		20150603	/*20150428 20150420 20150107*/
local typedate 		20170310  	/*20160815 20160425 20160208 */
local fulldate 		20150420	/*20150420 20141201 20141110 20141106 20141104*/
local outputdate	20170310   /*20160815 20160502 20160425 20160411 20160331*/

matrix obs = J(10,2,.)
		
*Retrieve application, offer, and grade information from AT RUN file
insheet using "${asgndata}1_AT_RUN_022262012.csv", double clear
	ren maskedstudentids maskedid
	ren requestedschool appschoolnum
	
	*Not sure what rank=99 means, ask Atila
	gen apprank = choiceselection
	replace apprank = apprank*10 if apprank<10
	replace apprank = floor(apprank/10)
	
	drop if maskedid=="#VALUE!"
	destring maskedid, ignore("#VALUE!") replace

	gen offer = (status=="A")
	bys maskedid appschoolnum: egen max_offer = max(offer)
	drop offer
	ren max_offer offer
	
	gen family = (family!="")
	
	keep grade maskedid offer appschoolnum apprank family

	ren grade appgrade
	duplicates drop appgrade maskedid offer appschoolnum apprank, force
	
	duplicates tag maskedid appgrade appschoolnum, gen(dup)
	tab dup
	
	*Cases of multiple rankings for the same school. Choose the smallest rank
	bys maskedid appgrade appschoolnum: egen min_rank = min(apprank)
	replace apprank = min_rank if dup
	
	*Keep if has nonmissing family link within school
	bys maskedid appgrade appschoolnum: egen max_fam=max(family)
	drop family
	ren max_fam family
	
	duplicates drop maskedid appgrade appschoolnum, force

	*All applicantions (school level)
		unique maskedid 
		matrix obs[1,1]= r(sum)
		matrix obs[1,2]= r(N)
	
	*Identify charters in application
	preserve
		insheet using "${rawdata}dps_charters_20112012_20150420 update.csv", double clear
		keep schoolname schoolnum charter alternative
		ren schoolnum appschoolnum
		gen appcharter = (charter==1)
		keep appschoolnum appcharter
		tempfile charter
		sa `charter'
	restore
	
	merge m:1 appschoolnum using `charter'
	keep if _merge!=2
	drop _merge
	replace appcharter=0 if appcharter==.
	
	*Study charters
	gen appstudy= (	inlist(appschoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(appschoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444,445, 446) | ///
						inlist(appschoolnum,462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
					)

	
	*Get latest school names
	preserve
		u "${cleandata}sims_csap_long_20102014 `simsdate'", clear
		keep schoolnum schoolname spyear
		bys schoolnum: egen max_year = max(spyear)
		keep if spyear==max_year
		drop spyear max_year
		
		*schools with slighlty different names
		duplicates drop schoolnum, force
		ren schoolnum appschoolnum
		ren schoolname appschoolname
		tempfile schools
		sa `schools'
	restore	
	
	merge m:1 appschoolnum using `schools'
	keep if _merge!=2
	drop _merge
	order maskedid appschoolnum appschoolname
	
	tab appgrade

	saveold "${cleandata}AT_RUN_school_level_full.dta", replace

if setup == 1 {
	
	preserve
	
		* new file made at 1 dofile (1_merge_simulated etc)
		u "${cleandata}fulltypes`typedate'.dta", clear

		
		* used to use this file
		*u "${cleandata}full_student_type_`fulldate'.dta", clear
		
		*IR: result when I try to replicate, (different results than the original) 
		* use "${cleandata}fulltypes`typedate'", clear

		
		gen spyear =2013 
		cap ren maskedstu maskedid
		cap destring maskedid, replace
		*ren type_all type
		
		*Grades that have stent typ
		merge 1:m maskedid using "${cleandata}AT_RUN_school_level_full.dta", keepusing(offer appgrade appcharter appstudy appschoolnum appschoolname apprank family)
		gen has_type 	= (type_all!=.)
		drop _merge
		
		*Grades 4-10 (baseline and outcome scores are available)
		tab appgrade
		keep if inlist(appgrade,"04","05","06","07","08","09","10")
		
		destring appgrade, replace
		gen gradebase = appgrade-1
		gen pregradebase = appgrade-2
		
		tostring appgrade, replace
		tostring gradebase, replace
		tostring pregradebase, replace
		tempfile temp1
		sa `temp1'
	restore
	

		u "${cleandata}sims_csap_wide_20102014 `simsdate'", clear
		merge 1:m maskedid using `temp1'
			
		*Keep only if kid is an applicant
		keep if _merge!=1
		drop _merge
		
		
		*Construct baseline and outcome variables
		gen baselineschoolname=""
		foreach x in 	spyear	schoolnum charter	alternative repeat ///
						female black white hisp asian otherrace ///
						sped	foodfred	lep	bilingual ///
						gifted gt_ela gt_math	gt_math_ela	gt_other	///
						raw_ss_math	 ss_math		pp_up_math		prof_up_math	adv_math ///
						raw_ss_reading	ss_reading	pp_up_reading 	prof_up_reading	adv_reading	///
						raw_ss_writing ss_writing	pp_up_writing	prof_up_writing adv_writing ///
						raw_ss_science	ss_science	pp_up_science	prof_up_science	adv_science {
					
		gen prebaseline`x'=.
		gen baseline`x'=.	
		gen outcome`x'=.
		
		foreach grade in 2 3 4 5 6 7 8 9 10{
		
			replace prebaseline`x' = `x'`grade' if pregradebase=="`grade'"
			replace baseline`x' = `x'`grade' if gradebase=="`grade'"
			replace outcome`x' = `x'`grade' if appgrade=="`grade'"
		}
			
		}
		
		foreach x in prebaseline baseline outcome{
					gen `x'study= (inlist(`x'schoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(`x'schoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444,445, 446) | ///
						inlist(`x'schoolnum,462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
					) if `x'spyear!=.
				
				}
				
		
		keep  maskedid lastname firstname appschoolnum appschoolname appcharter appstudy appgrade apprank family offer pregradebase gradebase prebaseline* baseline* outcome* ///
			 type* has_type*

		order  outcome*, after(gradebase)
		renvars outcome*, predrop(7)
		order appgrade gradebase type*, after(maskedid)
		
		*Study Charter offer
		gen offer_study = (offer==1) if appstudy==1
		order offer_study, after(offer)
		
		/*Exclude kids who applied to a grade they repeated
		keep if spyear==2013	/*that is, first outcome for 2013-2014 academic year is in 2013-2014 academic year*/
				
		unique maskedid if has_type
		matrix obs[5,1]= r(sum)
				
		count if has_type
		matrix obs[5,2]= r(N)
		*/
		tab grade, gen(g_)	

		gen app_right_grade = (spyear==2013)
				

		
		*In sample means we have well-defined instrument
		gen in_sample	=	(has_type & offer_study!=.)
		gen has_sc 	=	ss_math!=. | ss_writing!=. | ss_reading!=.
		
		label var in_sample "In analysis sample (Z and risksets nonmissing)"
		
		foreach x in math reading writing science{
			gen has_sc_`x' = (ss_`x'!=.)
		}
		
		order maskedid in_sample app_right_grade appschoolnum appschoolname appcharter appstudy apprank appgrade family type* offer 
		ren gradebase baselinegrade
		order baselinegrade, after(baselineschoolname)
		order study, after(charter)
		saveold "${cleandata}mdrd_denver_fulltype `outputdate'.dta", replace
		
}
clear
