/* 	Mayara Felix Silva
	Denver main deck tables. These comments last edited: 1/26/2015
		For grades 4-10
		Output: Balance, 2SLS, and subgroup for pscore risksets and qualification instruments
*/

clear all
cap log close
set more off
set trace off
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/"

matrix demo			   		   = J(100,40,.)
matrix balance         		   = J(500,40,.)
matrix twosls          		   = J(500,50,.)
matrix twosls_qualgroup		   = J(500,50,.)
matrix rf              		   = J(500,40,.)
matrix subtwosls	   		   = J(500,40,.)
matrix twosls_nocov    		   = J(500,40,.)
matrix twosls_scorecov 		   = J(500,40,.)
matrix stacked_twosls_nocov    = J(500,40,.)
matrix stacked_twosls_scorecov = J(500,40,.)
matrix twosls_by_pscore		   = J(500,40,.)

matrix qualbalance    = J(500,20,.)
matrix qualtwosls	  = J(500,20,.)
matrix qualsubtwosls  = J(500,20,.)

local pscore_setup = 1


local qual_balance= 0
local qual_twosls = 0
local qual_subtwosls = 0

*Dates of analysis files
local pscoredate		20160815 /*20160715 20160516 20160510 20160421  20160208 */
local simsdate 			20150603
local qualdate 			20150708
local applicationdate 	20160208
local qualpscoredate	20150708

local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_*"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1				/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
		
	************************************************
	**************** Pscore Setup ******************
	************************************************		
	
	if `pscore_setup'==1{
		
		foreach file in  simulation frequency formula   {
			
		foreach sch_type in /*noex*/ study /*nonnoex */{
			
			use "`cleandata'mdrd_denver_`sch_type'_`file' `pscoredate'", clear
			
			destring baselinegrade, replace
			destring appgrade, gen(appgrade_tmp)
			drop appgrade
			rename appgrade_tmp appgrade
			
			*High school flag			
				gen hsgrade = ( appgrade>=9)
				gen msgrade = ( appgrade<9)
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
				
			*Analysis at the student level. Generate max for charter offer
			bys maskedid: egen any_`sch_type'_offer 	= max(offer_`sch_type')
			bys maskedid: egen num_schools = count(appschoolnum)
			bys maskedid: egen num_study = count(cond(appstudy==1,appschoolnum,.))
			gen pct_study = num_study/num_schools
			
			*Enrolls at charter flag (student level)
			bys maskedid: egen enroll_any_`sch_type' = max(`sch_type')
			
			*Rank 1 at school level
			gen rank1_study = (appstudy==1 & apprank==1)
		
			************************** School oversubscription table **************************
			*Schools ranked, assigned, 0 < prob assignment < 1 
			preserve
				keep if app`sch_type'==1
				bys appschoolnum: egen tot_rank1 = sum(rank1_study)
				bys appschoolnum: egen tot_rank = count(maskedid)
				bys appschoolnum: egen tot_offer = sum(offer)
				bys appschoolnum: egen tot_`sch_type'_not01 = count(cond(pscore_`sch_type'_`file'>0 & pscore_`sch_type'_`file'<1,maskedid,.))
				bys appschoolnum: egen tot_school_not01 = count(cond(pscore_school>0 & pscore_school<1,maskedid,.))
				bys appschoolnum: egen tot_school_is1 = count(cond(pscore_school==1,maskedid,.))
				bys appschoolnum: egen tot_school_not01_firstchoice = count(cond(pscore_school > 0 & pscore_school < 1 & apprank == 1,maskedid,.))
				
				duplicates drop appschoolnum, force
				keep appschoolnum appschoolname tot_* MS HS
				
				tempfile temp
				sa `temp'
			restore
			
			*Subject to randomization at the FULLY SATURATED PSCORE level (variation in instrument for every value of PSCORE) preserve

			preserve
			
				*Keep if in Denver at baseline
				keep if baselinespyear==2012
				keep if app`sch_type'==1
				
				*Simulation has huge support. Use rounded values for full saturation
					if "`file'"=="simulation"{
						ren pscore_`sch_type'_simulation sim_pscore
						gen double pscore_`sch_type'_simulation = round(sim_pscore,0.0001)
					}
				
				*Any charter offer pscore - by school, the number of kids in pscore sets with variation in any offer
					bys pscore_`sch_type'_`file': egen min_offer = min(any_`sch_type'_offer)
					bys pscore_`sch_type'_`file': egen max_offer = max(any_`sch_type'_offer)
					
					gen sum = min_offer + max_offer
					bys appschoolnum: egen tot_`sch_type'_rand = count(cond(sum==1 ,maskedid,.))
					bys appschoolnum: egen tot_`sch_type'_rand_not01 = count(cond(sum==1 & pscore_`sch_type'_`file'>0 & pscore_`sch_type'_`file'<1,maskedid,.))
						
				*School offer - if within the same school, there are kids with the same pscore and var in offer
					drop min_offer max_offer sum
					bys appschoolnum pscore_school: egen min_offer = min(offer)
					bys appschoolnum pscore_school: egen max_offer = max(offer)
					
					gen sum = min_offer + max_offer
					bys appschoolnum: egen tot_school_rand = count(cond(sum==1,maskedid,.))
					bys appschoolnum: egen tot_school_rand_not01 = count(cond(sum==1 & pscore_school>0 & pscore_school<1,maskedid,.))
					
					drop min_offer max_offer
					
					*Number of kids with family link
					bys appschoolnum: egen tot_family = sum(family)
					
					duplicates drop appschoolnum, force
					keep appschoolnum  tot_*
					
					merge 1:1 appschoolnum using `temp'
					drop _merge
					
					order appschoolnum appschoolname MS HS ///
						tot_rank1 tot_rank	tot_offer ///
						tot_`sch_type'_not01	tot_school_not01 tot_school_is1 ///
						tot_school_not01_firstchoice ///
						tot_`sch_type'_rand	tot_`sch_type'_rand_not01	///
						tot_school_rand	tot_school_rand_not01
				
					outsheet using "`tables'inDPSbase_`file'_`sch_type'_schools.csv", comma replace
			
			restore
			******************************************************************************

			*First ranked school is charter (student level, for cov balance)
			bys maskedid: egen temp = max(rank1_study)
			drop rank1_study
			ren temp rank1_study
			
			
			*First ranked charter is oversubscribed (first-choice IV strategy sample)
			gen rank1_study_os_tmp = (rank1_study == 1 & ( inlist(appschoolnum, 175, 196, 211, 328, 426, 428, 429) | ///
									   inlist(appschoolnum, 438, 443, 462, 481, 498, 513, 167, 176) ))
									   
			bys maskedid: egen rank1_study_os = max(rank1_study_os_tmp)
			drop rank1_study_os_tmp
			
			
			*Flag - apply to any study
			bys maskedid: egen max = max(app`sch_type')
			drop app`sch_type'
			ren max app`sch_type'
			
			*Enrolls next year (i.e. has nonmissing schoolnum)
			gen next = (schoolnum!=.)
			bys maskedid: egen enrolls_next = max(next)
			
			*** Dataset at student level from here on ****
			
			*In_sample was previously defined at the student-school level. Redefine it so
			*that it is at the student level
			
			bys maskedid: egen in_sample_max = max(in_sample)
			tab in_sample_max in_sample
			
			drop in_sample
			renvars in_sample_max, postdrop(4)
			
			duplicates drop maskedid, force
			drop next
			
			*Redefine has scores so that it excludes sciences - very few kids
			replace has_sc = (has_sc_math | has_sc_reading | has_sc_writing)
			
			*School-level obs
			
			drop appschoolnum appschoolname offer offer_`sch_type' appcharter apprank `sch_type' charter
			
			*Keep only if in sample
			keep if in_sample==1
			
			****** Rounded scores for full saturation ******
				
				*Define r01 as meaning pscore rounded to 0.01
				gen r01 = round(pscore_`sch_type', 0.01)
				tab r01, gen(r01_)
			
				*Define r001 as meaning pscore rounded to 0.001
				gen r001 = round(pscore_`sch_type', 0.001)
				tab r001, gen(r001_)
				
			*Polynomials
			forvalues i=1/4{
				gen double pscore`i'=pscore_`sch_type'^(`i')
			}
			
			
			*Keep if in Denver at baseline before defining good cells
			keep if baselinespyear==2012
			
			*Treatment is any study, instrument is study offer, riskset is pscore
			ren any_`sch_type'_offer 	Z
			ren enroll_any_`sch_type'  	D
			
			**********************************************************************************
			******************** Flag kids who are ineligble and guaranteed ******************
			**********************************************************************************
			*Flag kids who are ineligble and guaranteed - do not drop them, we still want them 
			*for OLS regressions and naive 2SLS (i.e. not controlling for risksets)
			
				*If simulation, first define in_zero1_pscore based on true score, then
				*round for true saturation (to tens of thousands)				
				if "`file'"=="simulation"{
					
					ren pscore_`sch_type'_simulation sim_pscore
					gen in_zero1_pscore = (sim_pscore > 0 & sim_pscore < 1)
					gen double pscore_`sch_type'_simulation = round(sim_pscore,0.0001)
					
					tab pscore_`sch_type'_`file', gen(r0_)			
					drop pscore_school
					ren pscore_`sch_type'_`file' r0
				}
				*Else use actual score to both drop kids and create saturated risksets
				else{
									
					tab pscore_`sch_type'_`file', gen(r0_)				
					drop pscore_school
					ren pscore_`sch_type'_`file' r0
					
					gen in_zero1_pscore = (r0 > 0 & r0 < 1)
				
				}
				
				foreach var in r0 r001 r01 {
					
					egen ind_`var' = group(`var')
				
				}
				

			**********************************************************************************
			********** Flag cells with variation in instrument and characteristics************
			**********************************************************************************
					
			/* 	Flag observations that actually make it into balance (i.e. obs in cells with
				variation in instrument). These flags are used for all types 
				of F-tests, conducted below.
				
				We generate these flags here so that no re-sorting of the data (done in bys x egen:)
				is done between the balanceregs and F tests, which affects how the variable
				"var", which stores the names of the variables corresponding to each balance
				coefficient, appears in the final dataset once we svmat the balance output.
			*/
			
			foreach x in r01 r001 r0{
				
				*Variation in instruments for each cell
					bys `x': egen sd_z_`x'	= 	sd(Z)
				
				*Good cell
					gen in_good_cell_`x' = (sd_z_`x'>0 & !missing(sd_z_`x') & in_zero1_pscore==1)
				
				gen double tag_`x' = `x'
				replace tag_`x' = -9999999 if in_good_cell_`x'==0
				tab tag_`x', gen(tag_`x'_)
				
				*Drop var for things with no var (the first one). These observations will not be included
				*in bin-based regression and fully saturated regressios. So, do not include their corresponding
				*variables either
				
				drop tag_`x'_1
				
				*Obs per cell
				
				bys `x': egen obs_cell_`x' = count(maskedid)
			}
	
			drop sd_z_*
			
			save "`cleandata'analysis_`sch_type'_`file'.dta", replace	
			
			
			
		}
	}
}			
