/*	Created by: Yusuke Narita
	Original file: "Y:\DPS\yusuke\create_pscore_charter_subroutine.do"
	
	Edited by: Mayara Felix
	1/12/2015

	NOTE: 	1) 	This code applies the subroutine created to generate theoretical pscores (frequeny and formula methods)
				for each set of simulated offers and corresponding simulated lottery numbers.
			
			2)	The beginning of the original code has been edited to read in the master .dta AT_RUN file, containing
				simulated offers and simulated lottery numbers instead of the actual .csv AT_RUN file.
				
			3)	Globals have been set to locals so that other stata work is not disrupted
			
			4)  To save space, the code destroys the "application_data" files at the end, which would be needed only
				if estimating balance at the school level
				
	Edited by Ignacio Rodriguez
	17/05/2016
	FIxing floating point error in the yusuke (the one that constructs formula and frequency scores) part of the code. Look for comments IR05172016 for  specific fixes. 

*/
clear
set more off


************************************************************
************** Set simulation-invariant locals *************
************************************************************ 

	*Inputs
	local master_date 20160518 /*20150708*/
	
	local raw_at_run_data "Y:/DPS/Raw Data/2013_03_21_enrollment-match/1_AT_RUN_022262012.csv"	/*Only used to get reqschool, grade, capacityprogram*/
	local master_at_run_data "Y:/DPS/MDmeetsRD Denver/Data/simulated_offers/DA_offers_master_AT_RUN `master_date'.dta"
	local raw_capacity_data "Y:/DPS/Raw Data/2013_03_21_enrollment-match/AssignmentTool/SchoolFile.csv"

	*Parematers
	local max_pref_rank 31
	local max_lottery_number 1 /* the max is 1 because for simulated offers and lotteries, the lottery numbers are strictly between 0 and 1. */

forval sims=1/400{
		
		************************************************************
		************** Set simulation-variant locals ***************
		************************************************************
	
		*Output
		local application_data "Y:/DPS/MDmeetsRD Denver/Data/simulated_offers/theoretical_pscore_files/application_data_year_one_`sims'.dta"
		local pscore_formula_data "Y:/DPS/MDmeetsRD Denver/Data/simulated_offers/theoretical_pscore_files/pscore_charter_formula_year_one_`sims'.dta"
		local pscore_frequency_data "Y:/DPS/MDmeetsRD Denver/Data/simulated_offers/theoretical_pscore_files/pscore_charter_frequency_year_one_`sims'.dta"
		
		********************************************************************************
		*************************** Start Yusuke's subroutine **************************
		********************************************************************************
	
			*step 0: data cleaning
			
			******************************* EDITS MADE BY MS *******************************
			*insheet using "${raw_at_run_data}"
			u "`master_at_run_data'", clear

			*Rename simulation-specific offer/lotterynumber as the main offer/lotterynumber variable
			ren status`sims' _status
			ren lotterynumber`sims' _lotterynumber
			
			drop status* lotterynumber*
			
			*Drop underscore from these variables
			renvars _status _lotterynumber, predrop(1)
			
			********************************************************************************
			
			*drop ineligible choices
			drop if apppriorityseq==0
			*call "phantom" maskedstudentids=="#VALUE!"
			*create IDs for them
			gen phantom=0
			replace phantom=1 if maskedstudentids=="#VALUE!"
			sort maskedstudentids
			by maskedstudentids: gen id_intermediate=string(_n)
			replace maskedstudentids=id_intermediate if phantom==1
			drop id_intermediate
			save "`application_data'", replace
			*create bucket id
			clear
			insheet using "`raw_at_run_data'"
			collapse (sum) lotterynumber, by(requestedschool grade capacityprogram)
			drop lotterynumber
			gen bucket_id=_n
			merge 1:m requestedschool grade capacityprogram using "`application_data'"
			drop if _merge==1
			drop _merge
			save "`application_data'", replace
			*add capacity variable, which is max(capacity in capacity file, # of applicants assigned to each school)
			*start with capacity in capacity file
			clear
			insheet using "`raw_capacity_data'"
			keep schoolcode grade programcode capacity
			rename schoolcode requestedschool
			rename programcode capacityprogram
			sort requestedschool grade capacityprogram
			merge 1:m requestedschool grade capacityprogram using "`application_data'"
			drop if _merge==1
			drop _merge
			replace capacity=0 if capacity==.
			save "`application_data'", replace
			*next calculate # of assigned students
			keep if status=="A"
			gen realized_capacity=1
			collapse (sum) realized_capacity, by(bucket_id)
			merge 1:m bucket_id using "`application_data'"
			drop _merge
			replace realized_capacity=0 if capacity==.
			*finally define capacity 
			*this is an assumed definition that may be incorrect
			replace capacity=max(capacity, realized_capacity)
			*drop schools with capacity=0
			drop if capacity==0
			save "`application_data'", replace
			*step 1: find marginal priority group of each bucket
			keep if status=="A"
			collapse (max) apppriorityseq, by(bucket_id)
			rename apppriorityseq marginal_priority
			merge 1:m bucket_id using "`application_data'"
			drop if _merge==1
			drop _merge
			save "`application_data'", replace
			*change marginal_priority=*-*999 if a school is filled up by guaranteed students only
			replace marginal_priority=-999 if marginal_priority==1
			save "`application_data'", replace
			*change marginal_priority=999 if a school is undersubscribed (# of assigned students<capacity) 
			replace marginal_priority=999 if realized_capacity<capacity
			save "`application_data'", replace
			
			*step 2: find cutoff of each bucket
			******* We exclude family link kids because they have a problem
			******* Family linked kids get the best number out of the siblings
			******* But we never observe the shared number so it messes up the 
			******* cutoff calculation. It shouldn't affect marginal priority though
			keep if status=="A" & apppriorityseq==marginal_priority & familylinkmasked == "" 			
			
			collapse (max) lotterynumber, by(bucket_id)
			rename lotterynumber lottery_cutoff
			merge 1:m bucket_id using "`application_data'"
			drop _merge
			*change lottery_cutoff=max_lottery_number if a school is undersubscribed (# of assigned students<capacity)
			replace lottery_cutoff=`max_lottery_number' if marginal_priority==999
			*create "marginal"=1{choice is in the marginal priority group} and "effective_cutoff"=lottery_cutoff if marginal=1; 0 if worse than marginal; max_lottery_number if better than marginal
			gen marginal=0
			replace marginal=1 if marginal_priority==apppriorityseq & marginal_priority!=1
			gen effective_cutoff=lottery_cutoff*marginal
			replace effective_cutoff=`max_lottery_number' if marginal_priority>apppriorityseq | apppriorityseq==1
			** When a student is not guaranteed at a school filled up by guaranteed students, his effective_cutoff = 0
			replace effective_cutoff = 0 if marginal_priority == -999 & effective_cutoff == .
			save "`application_data'", replace
			*create pref_rank ranging from 1, 2, ..., N
			collapse (max) apppriorityseq, by(choiceselection)
			drop apppriorityseq
			sort choiceselection
			gen pref_rank=_n
			merge 1:m choiceselection using "`application_data'"
			drop _merge
			save "`application_data'", replace
			*in year 2, 8 students make 2 choices of the same pref rank
			sort maskedstudentids pref_rank
			by maskedstudentids pref_rank: gen size=_N
			by maskedstudentids pref_rank: gen order=_n
			drop if size>1 & order>1
			drop size order
			save "`application_data'", replace
			*step 3: create MID assiciated with each choice
			forvalues rank=2(1)`max_pref_rank'{
				use "`application_data'", clear
				keep if pref_rank<`rank'
				egen MID`rank'=max(effective_cutoff), by(maskedstudentids)
				replace pref_rank=`rank'
				collapse (mean) pref_rank MID`rank', by(maskedstudentids) 
				merge 1:1 maskedstudentids pref_rank using "`application_data'"
				drop if _merge==1
				*_merge==2 means the applicant does not make any choice with pref_rank<`rank'. so MID`rank' should be 0
				replace MID`rank'=0 if _merge==2
				drop _merge
				save "`application_data'", replace
			}
			gen MID=0 if pref_rank==1
			forvalues rank=2(1)`max_pref_rank'{
				replace MID=MID`rank' if pref_rank==`rank'
				drop MID`rank'
			}
			save "`application_data'", replace
			*step 4: create large \Theta012 in the paper (dummies for conditionally always assigned/sometimes assigned/never assigned)
			gen better_than_marginal=0
			replace better_than_marginal=1 if marginal_priority>apppriorityseq | apppriorityseq==1
			gen worse_than_marginal=0
			replace worse_than_marginal=1 if marginal_priority<apppriorityseq & apppriorityseq!=1
			save "`application_data'", replace
			forvalues rank=2(1)`max_pref_rank'{
				use "`application_data'", clear
				keep if pref_rank<`rank'
				egen always_assigned_more_preferred`rank'=max(better_than_marginal), by(maskedstudentids)
				replace always_assigned_more_preferred`rank'=0 if always_assigned_more_preferred`rank'==.
				replace pref_rank=`rank'
				collapse (mean) pref_rank always_assigned_more_preferred`rank', by(maskedstudentids) 
				merge 1:1 maskedstudentids pref_rank using "`application_data'"
				drop if _merge==1
				drop _merge
				replace always_assigned_more_preferred`rank'=0 if always_assigned_more_preferred`rank'==.
				save "`application_data'", replace
			}
			gen always_assigned_more_preferred=0
			forvalues rank=2(1)`max_pref_rank'{
				replace always_assigned_more_preferred=always_assigned_more_preferred`rank' if pref_rank==`rank'
				drop always_assigned_more_preferred`rank'
			}
			gen never_assigned=0
			replace never_assigned=1 if always_assigned_more_preferred==1 | worse_than_marginal==1
			gen sometimes_assigned=0
			replace sometimes_assigned=1 if always_assigned_more_preferred==0 & marginal==1
			gen conditionally_always_assigned=0
			replace conditionally_always_assigned=1 if always_assigned_more_preferred==0 & better_than_marginal==1
			save "`application_data'", replace

			
			*** IR05172016
			*** floating error precision is important here. 
			*** basically stata introduces a small error because these numbers are floats. 
			*** this is important because you end up with 0.123456 - 0 != 0.123456, 
			*** rather the end result is equal to 0.12345678. So I'm fixing this error 
			*** for mid, pscore and lottery cutoff. taken from stata blog (look for floating point precision)
			** I'm effectivle rounding to the millionth digit, just as in the normal balance table 

			*normalize lottery numbers and MIDs

			replace MID=MID/`max_lottery_number'
			gen double MIDtmp = round( MID * 1000000 ) / 1000000
			drop MID
			ren MIDtmp MID

			replace lottery_cutoff=lottery_cutoff/`max_lottery_number'
			gen double lottery_cutofftmp = round( lottery_cutoff * 1000000 ) / 1000000
			drop lottery_cutoff
			ren lottery_cutofftmp lottery_cutoff 			
			
			*step 5: code estimated propensity scores using the continuum market formula
			*Ignacio 05/12/2016 pscore NEEDS to be generated as a double, otherwise get floating error 
			** ^ notice the caps, it means its serious!
			gen double pscore_formula=0
			replace pscore_formula=1-MID if conditionally_always_assigned==1
			replace pscore_formula=lottery_cutoff-MID if sometimes_assigned==1
			replace pscore_formula=0 if pscore_formula<0
			*pscore should be 0 if ineligible
			replace pscore_formula=0 if apppriority==0
			save "`application_data'", replace
			*drop students with maskedstudentids=="#VALUE!"
			drop if phantom==1
			*step 6: aggregate them into propensity score of being assigned to any charter
			egen charter=anymatch(requestedschool), values(167 168 175 176 196 211 212 266 328 330 392 393 394 426 428 429 435 436 438 443 444 445 446 462 481 485 496 498 513 441 512 391)
			keep if charter
			rename pscore_formula pscore_charter_formula
			collapse (sum) pscore_charter_formula, by(maskedstudentids)
			save "`pscore_formula_data'", replace
			*step 7: create pscore_charter_frequency_year_two.dta 
			*for each school, calculate empirical frequency of getting an offer conditional on MID and \Theta012 (never_assigned sometimes_assigned conditionally_always_assigned)
			use "`application_data'", clear
			gen offer=0
			replace offer=1 if status=="A"
			sort bucket_id MID never_assigned sometimes_assigned conditionally_always_assigned
			by bucket_id MID never_assigned sometimes_assigned conditionally_always_assigned: egen frequency_offer=mean(offer)
			*for those never assigned, MID is irrelevant. need to take it into account
			sort bucket_id never_assigned
			by bucket_id never_assigned: egen frequency_offer_intermediate=mean(offer)
			replace frequency_offer=frequency_offer_intermediate if never_assigned==1
			drop frequency_offer_intermediate
			save "`application_data'", replace
			*drop phantom students with maskedstudentids=="#VALUE!" 
			drop if phantom==1
			*sum up them into pscore of getting any charter offer, which is pscore_charter_frequency 
			egen charter=anymatch(requestedschool), values(167 168 175 176 196 211 212 266 328 330 392 393 394 426 428 429 435 436 438 443 444 445 446 462 481 485 496 498 513 441 512 391)
			keep if charter
			rename frequency_offer pscore_charter_frequency
			collapse (sum) pscore_charter_frequency, by(maskedstudentids)
			replace pscore_charter_frequency=1 if pscore_charter_frequency>1
			save "`pscore_frequency_data'", replace
			
			************* Erase application dataset *************
			erase "`application_data'"
			
}	/*Close simulation loop*/

