/* Ignacio Rodriguez

	Goal: 	Read in simulated pscore file and match to AT RUN to get maskedids
	
	problem: dstlink isn't unique in at run data. 
	
		Ignacio Rodriguez
	04/25/2016
	We use the application file generated by Yusuke's subroutine code found at 
	Y:\DPS\Yusuke\create_pscore_charter_subroutine_20160411_year2
	the file is 
	"Y:\DPS\yusuke\data\application_data_year_two_20160411.dta"
	We use this file because it drops applications where the student is ineligible (apppriority == 0)
	or the school has 0 capacity. These students should have a pscore of 0 anyway, so the distinction 
	only really matters for the fulltype column in the tables. 
	We used to run the raw at run files but these differences cause for differt types to be created. 
	
*/	
pause on
clear all
cap log close
set more off
set trace off
set maxvar 32767
adopath + "Z:\ado\plus\"

local rawdata   "Y:/DPS/Raw Data/"
local asgndata  "Y:/DPS/Raw Data/2013_03_21_enrollment-match/"
local testdata  "Y:/DPS/Raw Data/2013_10_03_tests2013/"
local cleandata "Y:\DPS\MDmeetsRD Denver\Data\"
local yusuke	 "Y:/DPS/yusuke/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/"
local sim_data "Y:\DPS\MDmeetsRD Denver\Data\"

local appdatadate 	 20170310   /*20160815 20160512*/

local date 			20160816 	/* 20160815 20160524 20160425 20160324 20160208	20150528 20150525 20150519*/
local outputdate 	20170310

local find_types						= 0	/*This take a long time to run. Only re-run if redefining what type is*/
local find_first_choice_types 			= 0
local find_first_second_choice_types 	= 0
local find_upto_third_choice_types 		= 0
*Simulations
	use "`sim_data'simulated_pscore_year2.dta",  clear
	ren pscore_simulated pscore_charter_simulation
	tempfile sim
	sa `sim'
	
	
*Retrieve application, offer, and grade information from yusuke file. this used to be the at run file
	*insheet using "`asgndata'1_AT_RUN_02262013.csv", double clear
	use "`cleandata'application_data_year_two_`appdatadate'.dta", clear
	ren maskedstudentids maskedid
	ren requestedschool appschoolnum
	ren grade appgrade
	
	drop if maskedid=="#VALUE!"
	
	
	//Ignacio: This kid has information for application in two different grades. He makes it through
	// the entire cleaning process and causes masked id to not be unique 
	// do files further down the line expect this to be unique so I am dropping one of the grades. 
	// this kid isn't even in the sims csap data. dropping grade 3
	drop if masked == "786731" & appgrade == "03"	
	// Thiese kids don't have any sims data and cause same problem
	drop if masked == "765282" | masked ==  "786755"
	
	
		*Find combinations of cells (that is, define types)
	if `find_types'==1 {
	
		foreach grade in "EC" "00" "01" "02" "03" "04" "05" "06" "07" "08" "09" "10" "11" "12" {
	
		preserve
		keep if appgrade == "`grade'"
		di "`grade'"

			*Edit choiceselection for those that do not have buckets

			forval i = 1/5 {
				qui replace choiceselection = `i'*10 if choiceselection==`i'
			}
			
			
			*IR: these groups won't necessarily match the same groups done in year 1 
			*Grade, school, capacity program, choiceselection, priority code,
			egen scell = group(appschoolnum capacityprogram choiceselection apppriorityseq), missing
			
			keep maskedid scell			
			
			*Create dummy variable for each cell
			qui tab scell, gen(in_scell_)
			di "after scell"
			
			*Make sure flag for each cell shows up for all obs for the same maskedid
			
			sort maskedid
			foreach var of varlist in_scell_*{
				by maskedid: egen max_`var' = max(`var')
			}
			di "after for loop scell"
			keep maskedid max_*
			renvars max_*, predrop(4)
			
			duplicates drop maskedid, force

			
			*Check that each kid is in some cell
			egen check = rowmax(in_*)
			egen row_sum = rowtotal(in_*)

			*Generate type - unique set of schools-capacity-priorities
			egen type = group(in_*), missing
			
			keep maskedid type
			
			save "`cleandata'DA_year2_types `date' grade`grade'.dta", replace
	
		restore			
		}
	}

	
		*Find combinations of cells (that is, define types)
	if `find_first_choice_types'==1 {
	
		foreach grade in "EC" "00" "01" "02" "03" "04" "05" "06" "07" "08" "09" "10" "11" "12" {
	
		preserve
		keep if appgrade == "`grade'"
		di "`grade'"
		
		keep if inlist(choiceselection, 10, 11, 12, 13, 14)
	
			*IR: these groups won't necessarily match the same groups done in year 1 
			*Grade, school, capacity program, choiceselection, priority code,
			egen scell = group(appschoolnum capacityprogram choiceselection apppriorityseq), missing
			
			keep maskedid scell			
			
			*Create dummy variable for each cell
			qui tab scell, gen(in_scell_)
			di "after scell"
			
			*Make sure flag for each cell shows up for all obs for the same maskedid
			
			sort maskedid
			foreach var of varlist in_scell_*{
				by maskedid: egen max_`var' = max(`var')
			}
			di "after for loop scell"
			keep maskedid max_*
			renvars max_*, predrop(4)
			
			duplicates drop maskedid, force

			
			*Check that each kid is in some cell
			egen check = rowmax(in_*)
			egen row_sum = rowtotal(in_*)

			*Generate type - unique set of schools-capacity-priorities
			egen type_fc = group(in_*), missing
			
			keep maskedid type_fc
			
			save "`cleandata'DA_year2_fc_types `date' grade`grade'.dta", replace
	
		restore			
		}
	}	

	
	
		*Find combinations of cells (that is, define types)
	if `find_first_second_choice_types'==1 {
	
		foreach grade in "EC" "00" "01" "02" "03" "04" "05" "06" "07" "08" "09" "10" "11" "12" {
	
		preserve
		keep if appgrade == "`grade'"
		di "`grade'"
		
		keep if inlist(choiceselection, 10, 11, 12, 13, 14, 20, 21, 22, 23, 24)
	
			*IR: these groups won't necessarily match the same groups done in year 1 
			*Grade, school, capacity program, choiceselection, priority code,
			egen scell = group(appschoolnum capacityprogram choiceselection apppriorityseq), missing
			
			keep maskedid scell			
			
			*Create dummy variable for each cell
			qui tab scell, gen(in_scell_)
			di "after scell"
			
			*Make sure flag for each cell shows up for all obs for the same maskedid
			
			sort maskedid
			foreach var of varlist in_scell_*{
				by maskedid: egen max_`var' = max(`var')
			}
			di "after for loop scell"
			keep maskedid max_*
			renvars max_*, predrop(4)
			
			duplicates drop maskedid, force

			
			*Check that each kid is in some cell
			egen check = rowmax(in_*)
			egen row_sum = rowtotal(in_*)

			*Generate type - unique set of schools-capacity-priorities
			egen type_sc = group(in_*), missing
			
			keep maskedid type_sc
			
			save "`cleandata'DA_year2_sc_types `date' grade`grade'.dta", replace
	
		restore			
		}
	}		
	
	
		*Find combinations of cells (that is, define types)
	if `find_upto_third_choice_types'==1 {
	
		foreach grade in "EC" "00" "01" "02" "03" "04" "05" "06" "07" "08" "09" "10" "11" "12" {
	
		preserve
		keep if appgrade == "`grade'"
		di "`grade'"
		
		keep if inlist(choiceselection, 10, 11, 12, 13, 14, 20, 21, 22, 23, 24, 30, 31, 32, 33, 34)
	
			*IR: these groups won't necessarily match the same groups done in year 1 
			*Grade, school, capacity program, choiceselection, priority code,
			egen scell = group(appschoolnum capacityprogram choiceselection apppriorityseq), missing
			
			keep maskedid scell			
			
			*Create dummy variable for each cell
			qui tab scell, gen(in_scell_)
			di "after scell"
			
			*Make sure flag for each cell shows up for all obs for the same maskedid
			
			sort maskedid
			foreach var of varlist in_scell_*{
				by maskedid: egen max_`var' = max(`var')
			}
			di "after for loop scell"
			keep maskedid max_*
			renvars max_*, predrop(4)
			
			duplicates drop maskedid, force

			
			*Check that each kid is in some cell
			egen check = rowmax(in_*)
			egen row_sum = rowtotal(in_*)

			*Generate type - unique set of schools-capacity-priorities
			egen type_tc = group(in_*), missing
			
			keep maskedid type_tc
			
			save "`cleandata'DA_year2_tc_types `date' grade`grade'.dta", replace
	
		restore			
		}
	}
	
	//Adding type back in to app data, and we flag each type according to the application type
	
	forvalues i = 0(1)9 {
		merge m:1 maskedid using "`cleandata'DA_year2_types `date' grade0`i'.dta"
		rename type type_gr0`i'
		keep if _merge != 2
		drop _merge
	}
	
	foreach choice in fc sc tc {
		forvalues i = 0(1)9 	{
			merge m:1 maskedid using "`cleandata'DA_year2_`choice'_types `date' grade0`i'.dta"
			rename type_`choice' type_`choice'_gr0`i'
			keep if _merge != 2
			drop _merge
		}
	}
	
	merge m:1 maskedid using "`cleandata'DA_year2_types `date' gradeEC.dta"
	rename type type_grEC
	keep if _merge != 2
	drop _merge
	
	merge m:1 maskedid using "`cleandata'DA_year2_types `date' grade10.dta"
	rename type type_gr10
	keep if _merge != 2
	drop _merge
	
	merge m:1 maskedid using "`cleandata'DA_year2_types `date' grade11.dta"
	rename type type_gr11
	keep if _merge != 2
	drop _merge
	
	
	merge m:1 maskedid using "`cleandata'DA_year2_types `date' grade12.dta"
	rename type type_gr12
	keep if _merge != 2
	drop _merge

	foreach choice in fc sc tc{
		merge m:1 maskedid using "`cleandata'DA_year2_`choice'_types `date' gradeEC.dta"
		rename type_`choice' type_`choice'_grEC
		keep if _merge != 2
		drop _merge
		
		merge m:1 maskedid using "`cleandata'DA_year2_`choice'_types `date' grade10.dta"
		rename type_`choice' type_`choice'_gr10
		keep if _merge != 2
		drop _merge
		
		merge m:1 maskedid using "`cleandata'DA_year2_`choice'_types `date' grade11.dta"
		rename type_`choice' type_`choice'_gr11
		keep if _merge != 2
		drop _merge
		
		
		merge m:1 maskedid using "`cleandata'DA_year2_`choice'_types `date' grade12.dta"
		rename type_`choice' type_`choice'_gr12
		keep if _merge != 2
		drop _merge	
	}	
	
	keep dstlink maskedid appschoolnum appgrade type_* familylinkmasked
	
	*Merge simscore back
	* putting type back into one variable
	* you need to add the max to end up with unique values of type per grade. 
	* for example grade 1 type goes from 1-3000. Grade 2 type goes from 1-2034. Add the max
	* of grade 1 to all grade 2 and you end up with unique values overall
	merge m:1 dstlink using `sim'
	keep if _merge==3
	drop _merge
	
	gen type = .
	replace type = type_grEC if appgrade == "EC"
	egen max_type_EC = max(type)
	
	replace type = type_gr00 + max_type_EC if appgrade == "00"
	
	forvalues i = 0(1)8 {
		local j = `i' + 1
		egen max_type_`i' = max(type)
		replace type = type_gr0`j' + max_type_`i' if appgrade == "0`j'"
		}
	
	forvalues i = 9(1)11 {
	local j = `i' + 1
		egen max_type_`i' = max(type)
		replace type = type_gr`j' + max_type_`i' if appgrade == "`j'"
	} 
	
	drop max_type_* type_gr*

	foreach choice in fc sc tc {
		gen type_`choice' = .
		replace type_`choice' = type_`choice'_grEC if appgrade == "EC"
		egen max_type_`choice'_EC = max(type_`choice')
		
		replace type_`choice' = type_`choice'_gr00 + max_type_`choice'_EC if appgrade == "00"
		
		forvalues i = 0(1)8 {
		local j = `i' + 1
			egen max_type_`choice'_`i' = max(type_`choice')
			replace type_`choice' = type_`choice'_gr0`j' + max_type_`choice'_`i' if appgrade == "0`j'"
			}
		
		forvalues i = 9(1)11 {
		local j = `i' + 1
			egen max_type_`choice'_`i' = max(type_`choice')
			replace type_`choice' = type_`choice'_gr`j' + max_type_`choice'_`i' if appgrade == "`j'"
			}
			
		drop max_type_`choice'* type_`choice'_gr*	
	}	
		
	rename type type_all
	rename type_fc type_fc_all
	rename type_sc type_sc_all
	rename type_tc type_tc_all
	
	save "`cleandata'type_for_all_year_2_`outputdate'", replace

	
	
	* This is for the full applicant type control column (do file 4)
	preserve
		keep maskedid type_all type_fc_all type_sc_all type_tc_all
		duplicates drop
		save "`cleandata'fulltypes_year_2_`outputdate'", replace	
	restore
	
	rename type_all type
	drop type_fc_all type_sc_all type_tc_all

		*School and school-grade level pscore
	preserve
		bys maskedid appschoolnum appgrade: egen pscore_mask_grade = sum(pscore_charter)
		bys maskedid appschoolnum: egen pscore_mask = sum(pscore_charter)
	
		bys type appschoolnum appgrade: egen pscore_school_grade= mean(pscore_mask_grade)
		bys type appschoolnum: egen pscore_school= mean(pscore_mask)
		
		*If kid has family link, define pscore at maskedid level (i.e. just sum), otherwise
		*define it as average across kids within type
		
		gen family = (familylinkmasked!="")
		bys maskedid: egen max_fam = max(family)
		
		replace pscore_school_grade = pscore_mask_grade if max_fam==1
		replace pscore_school = pscore_mask if max_fam==1
		
		keep maskedid appschoolnum appgrade pscore_school pscore_school_grade type pscore_mask* max_fam
		duplicates drop
		order maskedid type, first

				
		save "`cleandata'pscore_simulation_school_grade_year2", replace
	restore
	
	//study school changed from 2012 (year1)
	
	*Study charters
	gen study = (	inlist(appschoolnum, 167, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(appschoolnum, 330, 391, 392, 393, 394, 426, 428, 429, 435, 436, 437, 438, 441, 443, 444, 445, 446) | ///
						inlist(appschoolnum, 462, 481, 485, 496, 498, 512, 513, 517) ///
					)
	
	**Study no-excuses charters
	gen noex =  (	inlist(appschoolnum, 392, 393, 394, 426, 428, 429, 435, 436, 437) | ///
						inlist(appschoolnum, 438, 443, 444, 445, 462, 481, 498, 513, 517) ///
					)
	
	
	**Study non no-excuses charters
	gen nonnoex = (	  inlist(appschoolnum, 167, 175, 176, 196, 211, 212) | ///
						  inlist(appschoolnum, 266, 328, 330, 391, 441, 446, 485, 496, 512) ///
					)
					
	
	** Traditional public schools
	gen trad = ( inlist(appschoolnum, 172, 178, 179, 190, 194, 195, 199, 203, 206, 207) | ///
						inlist(appschoolnum, 208, 209, 210, 213, 214, 215, 217, 218, 219, 220, 221) | ///
						inlist(appschoolnum, 222, 223, 224, 227, 228, 230, 231, 235, 236, 238, 240) | ///
						inlist(appschoolnum, 241, 242, 244, 246, 248, 249, 250, 252, 253, 254, 258) | ///
						inlist(appschoolnum, 259, 260, 261, 262, 263, 264, 269, 270, 271, 275, 276) | ///
						inlist(appschoolnum, 277, 278, 279, 280, 281, 283, 284, 285, 287, 290, 292) | ///
						inlist(appschoolnum, 294, 295, 296, 298, 301, 406, 407, 408, 412, 413, 416) | ///
						inlist(appschoolnum, 417, 418, 423, 448, 450, 451, 452, 453, 455, 456, 457, 458, 459, 463) ///
					)
		
	
		foreach sch_type in study /*noex nonnoex*/ trad	{
		
			preserve
				
				*1) Aggregate at study, appgrade, kid level
				ren pscore_charter_simulation pscore
				bys maskedid `sch_type': egen pscore_maskedid = sum(pscore)
				
				*If kid has family link, define pscore at maskedid level (i.e. just sum), otherwise
				*define it as average across kids within type
				gen family = (familylink!="")
				bys maskedid: egen max_fam = max(family)
				
				keep maskedid `sch_type' pscore_maskedid type max_fam
				duplicates drop
				
				
				*2) Take average across types
				bys type `sch_type': egen pscore_`sch_type'_simulation = mean(pscore_maskedid)

				replace pscore_`sch_type'_simulation = pscore_maskedid if max_fam==1
				
				*Probability of assignment at any charter
				keep if `sch_type'
				
				keep maskedid type pscore_`sch_type'_simulation max_fam
				ren max_fam has_familylink
				
				
				
				duplicates drop
				
				sum pscore_`sch_type'
				

				
				save "`cleandata'pscore_`sch_type'_simulation_year_two_`outputdate'", replace
			
			restore
		
		}
		
		
	
	
	
	

