/*  Ye Ji Kee

	Goal: 	Create analysis files (instruments + treatment + baseline + outcome) for MDmRD
			One file is created for each instrument
			
	IMPORTANT: 	Files pscore_charter_formula_`date' and pscore_charter_frequency_`date' must be
				placed into cleandata folder. These files are prepared using Yusuke's pscore
				routines, and can be found here: Y:\DPS\Yusuke\data.
	
*/	

clear all
cap log close
set more off
set trace off
set maxvar 32767
adopath + "Z:\ado\plus\"

local rawdata   "Y:/DPS/Raw Data/"
local asgndata  "Y:/DPS/Raw Data/2013_03_21_enrollment-match/"
local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local yusuke	 "Y:/DPS/yusuke/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/year2/"

local pscore_setup =1		/*Propensity score matching setup*/

local simsdate 		20150603	
local outputdate 	20170310 	/*20160715 20160425   20160411 20160222 */
local appdatadate 	year_two_20170310 /*year_two_20160411 year_two_20160208 */ 

matrix obs = J(10,6,.)
		
*Retrieve application, offer, and grade information from AT RUN file
insheet using "`asgndata'1_AT_RUN_02262013.csv", double clear
	ren maskedstudentids maskedid
	ren requestedschool appschoolnum
	
	*Not sure what rank=99 means, ask Atila
	gen apprank = choiceselection
	replace apprank = apprank*10 if apprank<10
	replace apprank = floor(apprank/10)
	
	drop if maskedid=="#VALUE!"
	destring maskedid, ignore("#VALUE!") replace

	gen offer = (status=="A" | status == "FLA")
	bys maskedid appschoolnum: egen max_offer = max(offer)
	drop offer
	ren max_offer offer
	
	gen family = (family!="")
	
	keep grade maskedid offer appschoolnum apprank family

	ren grade appgrade
	duplicates drop appgrade maskedid offer appschoolnum apprank, force
	
	duplicates tag maskedid appgrade appschoolnum, gen(dup)
	tab dup
	
	*Cases of multiple rankings for the same school. Choose the smallest rank
	bys maskedid appgrade appschoolnum: egen min_rank = min(apprank)
	replace apprank = min_rank if dup
	
	*Keep if has nonmissing family link within school
	bys maskedid appgrade appschoolnum: egen max_fam=max(family)
	drop family
	ren max_fam family
	
	duplicates drop maskedid appgrade appschoolnum, force

	*All applicantions (school level)
		unique maskedid 
		matrix obs[1,1]= r(sum)
		matrix obs[1,2]= r(N)
	
	*Identify charters in application
	preserve
		insheet using "`rawdata'dps_charters_20122013_20150428 update.csv", double clear
		keep schoolname schoolnum charter alternative
		ren schoolnum appschoolnum
		gen appcharter = (charter==1)
		keep appschoolnum appcharter
		tempfile charter
		sa `charter'
	restore
	
	merge m:1 appschoolnum using `charter'
	keep if _merge!=2
	drop _merge
	replace appcharter=0 if appcharter==.
	
	*Study charters
	gen appstudy= 	(	inlist(appschoolnum,167, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(appschoolnum,330, 426, 428, 429, 435, 436, 437, 438, 443, 444, 445, 446) | ///
						inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391, 517) ///
					)
	
	*innov
	gen appinnov =  (   inlist(appschoolnum, 159, 165, 166, 174, 177, 188, 189, 205, 243, 257, 282, 289, 297, 299) | ///
						inlist(appschoolnum, 390, 395, 396, 397, 405, 419, 434, 447, 464, 466, 467, 468, 469, 471, 510, 511, 515) ///
					)

	*In schools 297, grades 6-8 are not innovation
	replace appinnov = 0 if (   inlist(appschoolnum, 297) & inlist(appgrade, "6","7","8")	)
	
	*In school 468, grade 11 is not innovation
	replace appinnov = 0 if (	inlist(appschoolnum, 468) & inlist(appgrade, "11")	)
	
	*Get latest school names
	preserve
		u "`cleandata'sims_csap_long_20102014 `simsdate'", clear
		keep schoolnum schoolname spyear
		bys schoolnum: egen max_year = max(spyear)
		keep if spyear==max_year
		drop spyear max_year
		
		*schools with slighlty different names
		duplicates drop schoolnum, force
		ren schoolnum appschoolnum
		ren schoolname appschoolname
		tempfile schools
		sa `schools'
	restore	
	
	merge m:1 appschoolnum using `schools'
	keep if _merge!=2
	drop _merge
	order maskedid appschoolnum appschoolname
	label var appschoolname "Requested school name"
	
	tab appgrade

	save "`cleandata'AT_RUN_year_2_school_level_innov.dta", replace
	
if `pscore_setup' == 1 {
	
	*One file for each instrument
		foreach file in frequency formula simulation{
			
			if "`file'"=="simulation"{
				local pscoredate year_two_20160425 /* year_two_20160324 */
			}
			else{
				local pscoredate year_two_20160411
			}
			
			*Get charter propensity scores
			preserve
				u "`cleandata'pscore_study_`file'_`pscoredate'.dta", clear
				
				if "`file'"=="simulation" {
					rename pscore_study_simulation pscore_charter_simulation
				}
				
				gen spyear =2014 
				cap ren maskedstu maskedid
				cap destring maskedid, replace
						
				*Grades that have scores
				merge 1:m maskedid using "`cleandata'AT_RUN_year_2_school_level_innov.dta", keepusing(offer appgrade appcharter appstudy appinnov appschoolnum appschoolname apprank family)

				*Applicants to any charters
				bys maskedid: egen any_study = max(appstudy)
				
				*Applicants to any innov
				bys maskedid: egen any_innov = max(appinnov)
			
				*All kids with missing pscores should be treated as pscore==0
				replace pscore_charter = 0 if pscore_charter==. & any_study==1
				
				gen has_study_pscore = (pscore_charter!=.)
				drop _merge
				
				*Grades 4-10 (baseline and outcome scores are available)
				tab appgrade
				keep if inlist(appgrade,"04","05","06","07","08","09","10")
				unique maskedid
				matrix obs[2,1]= r(sum)
				matrix obs[2,2]= r(N)
				
				destring appgrade, replace
				
				gen gradebase = 	appgrade-1
				gen pregradebase =	appgrade-2
				
				tostring appgrade, replace
				tostring gradebase, replace
				tostring pregradebase, replace
				
				tempfile temp1
				sa `temp1'
			restore
			
			
			*Get pscore with probability of assignment at each school, and generate
			*pscore at innov schools based on these
			
			if "`file'"!="simulation"{			
				preserve	
					u "`cleandata'application_data_`appdatadate'", clear
				
					ren masked maskedid
					ren requestedschool appschoolnum
					ren grade appgrade
					destring maskedid, replace
					
					keep if inlist(appgrade,"04","05","06","07","08","09","10")
					
					if "`file'"=="frequency"{
						drop pscore_formula
						ren frequency_offer pscore_frequency
					}
					
					*School-level pscore
					bys maskedid appschoolnum: egen pscore_school_`file' = sum(pscore_`file')

					*innov-level pscore
					gen appinnov =  (   inlist(appschoolnum, 159, 165, 166, 174, 177, 188, 189, 205, 243, 257, 282, 289, 297, 299) | ///
										inlist(appschoolnum, 390, 395, 396, 397, 405, 419, 434, 447, 464, 466, 467, 468, 469, 471, 510, 511, 515) ///
									)

						*In schools 297, grades 6-8 are not innovation
						replace appinnov = 0 if (   inlist(appschoolnum, 297) & inlist(appgrade, "6","7","8")	)
	
						*In school 468, grade 11 is not innovation
						replace appinnov = 0 if (	inlist(appschoolnum, 468) & inlist(appgrade, "11")	)									
						
						bys maskedid: egen any_innov = max(appinnov)
						
						*Innovation-level (pscore for appinnov==0 means nothing, it is replaced below by missing)
						bys maskedid appinnov: egen pscore_innov_`file' = sum(pscore_`file')
						
						*If not innovation, there should not exist an innovation pscore
						replace pscore_innov_`file'=. if appinnov==0
						
						*No kid should have more than one pscore
						bys maskedid: egen temp = max(pscore_innov_`file')
						replace pscore_innov_`file' = temp
						
						*Scores should be missing if did not apply to any innovations
						replace pscore_innov_`file' = . if any_innov==0
						
						*Scores should be zero if applied to some innovation but is coded missing
						replace pscore_innov = 0 if pscore_innov==. & any_innov==1

						**Added by Ye Ji (June 1st) replace pscore_innov with 1 if larger than 1
						replace pscore_innov_`file' = 1 if pscore_innov_`file' > 1 & pscore_innov_`file' != .
						
						gen has_innov_pscore = (pscore_innov_`file'!=.)
								
					duplicates drop maskedid appschoolnum, force
					
					keep maskedid appschoolnum pscore_school pscore_innov has_innov_pscore any_innov
					tempfile temp2
					sa `temp2'
				restore
			}
			
			else{
				*Calculate innovation-level
				preserve
					u "`cleandata'pscore_simulation_school_grade_year2", clear
					keep maskedid pscore_mask_grade pscore_school type appschoolnum appgrade max_fam
					
					keep if inlist(appgrade,"04","05","06","07","08","09","10")

					*innov
					*innov-level pscore
					gen appinnov =  (   inlist(appschoolnum, 159, 165, 166, 174, 177, 188, 189, 205, 243, 257, 282, 289, 297, 299) | ///
										inlist(appschoolnum, 390, 395, 396, 397, 405, 419, 434, 447, 464, 466, 467, 468, 469, 471, 510, 511, 515) ///
									)

						*In schools 297, grades 6-8 are not innovation
						replace appinnov = 0 if (   inlist(appschoolnum, 297) & inlist(appgrade, "6","7","8")	)
	
						*In school 468, grade 11 is not innovation
						replace appinnov = 0 if (	inlist(appschoolnum, 468) & inlist(appgrade, "11")	)		
						
						
					bys maskedid: egen any_innov = max(appinnov)
					
					*Simulated score - defined at type level and appinnov level (pscore for appinnov==0 means nothing, it is replaced
					*below by missing)
					
					bys maskedid appinnov : egen pscore_temp = sum(pscore_mask_grade)
					bys type appinnov: egen pscore_innov_`file' = mean(pscore_temp)
					
					*If family link, still use score at kid level
					replace pscore_innov_`file' = pscore_temp if max_fam==1
					
					drop pscore_mask* pscore_temp type
					
					*If not innovation, there should not exist an innovation pscore
					replace pscore_innov_`file'=. if appinnov==0
					
					*No kid should have more than one pscore
					bys maskedid: egen temp = max(pscore_innov_`file')
					replace pscore_innov_`file' = temp
					drop temp
					
					*Scores should be missing if did not apply to any innovations
					replace pscore_innov_`file' = . if any_innov==0
					
					*Scores should be zero if applied to some innovation but is coded missing
					replace pscore_innov = 0 if pscore_innov==. & any_innov==1
					
					*** Added on June 1st, 2015 (Ye Ji). Replace innovation pscore with 1 if larger than 1
					replace pscore_innov_`file' = 1 if pscore_innov_`file' > 1 & pscore_innov_`file' != .
					
					gen has_innov_pscore = (pscore_innov!=.)
					
					duplicates drop maskedid appschoolnum, force
					destring maskedid, replace
					tempfile temp2
					sa `temp2'
				restore	
			}
			

				************ Combine pscore files ************
				
				/*Note: application_data file excludes all student-applicant combinations where apppriorityseq==0
						Because of that, there are some student-school combinations present in temp1 below but
						not in temp2. The correct probabilities of assignment to an innovation schools are, however,
						fixed at this point to be unique at the kid level (i.e. fixed in temp2).
						
						Therefore, when merging temp1 and temp2 at maskedid-appschoolnum level, redefine innovation
						score to be unique at kid level
				*/
				
				
				u `temp1', clear
				merge 1:1 maskedid appschoolnum using `temp2'
				keep if _merge!=2
				drop _merge
				
				*Redefine pscore at kid level

				bys maskedid: egen temp = max(pscore_innov_`file')
				replace pscore_innov_`file' = temp
				drop temp

				replace has_innov_pscore =0 if has_innov_pscore==.
				replace has_study_pscore =0 if has_study_pscore==.

				*Countinue count of applicants and applications
				
					*All applicants to charters
						unique maskedid if any_study
						matrix obs[3,1]= r(sum)
						
						count if any_study
						matrix obs[3,2]= r(N)
						
					*All applicants to innovation
						unique maskedid if any_innov
						matrix obs[3,3]= r(sum)
						
						count if any_innov
						matrix obs[3,4]= r(N)
					
					*All applicants to charters & innovation
						unique maskedid if (any_innov | any_study)
						matrix obs[3,5]= r(sum)
						
						count if (any_innov | any_study)
						matrix obs[3,6]= r(N)
						
					drop any_study	any_innov
						
					*Applicants with nonmissing score
					*keep if _merge==3
					
					unique maskedid if has_study_pscore
					matrix obs[4,1]= r(sum)
					
					count if has_study_pscore
					matrix obs[4,2]= r(N)
					
					*All applicants to innovation
						unique maskedid if has_innov_pscore
						matrix obs[4,3]= r(sum)
						
						count if has_innov_pscore
						matrix obs[4,4]= r(N)
					
					*All applicants to charters & innovation
						unique maskedid if (has_study_pscore | has_innov_pscore)
						matrix obs[4,5]= r(sum)
						
						count if (has_study_pscore | has_innov_pscore)
						matrix obs[4,6]= r(N)	
				
			tempfile temp
			sa `temp'
			
			u "`cleandata'sims_csap_wide_20102014 `simsdate'", clear
			unique maskedid
			
			merge 1:m maskedid using `temp'
			*keep if applicants only
			keep if _merge!=1
			drop _merge
			
			*Construct baseline and outcome variables
			
				foreach x in 	spyear	schoolnum charter alternative female black white hisp asian otherrace ///
								repeat	sped	foodfred	lep	bilingual ///
								gifted gt_ela gt_math	gt_math_ela	gt_other	///
								raw_ss_math	 ss_math		pp_up_math		prof_up_math	adv_math ///
								raw_ss_reading	ss_reading	pp_up_reading 	prof_up_reading	adv_reading	///
								raw_ss_writing ss_writing	pp_up_writing	prof_up_writing adv_writing{
						
						gen prebaseline`x'=.
						gen baseline`x'=.	
						gen outcome`x'=.
				
					foreach grade in 2 3 4 5 6 7 8 9 10{
					
						replace prebaseline`x' = `x'`grade' if pregradebase=="`grade'"
						replace baseline`x' = `x'`grade' if gradebase=="`grade'"
						replace outcome`x' = `x'`grade' if appgrade=="`grade'"
						
					}
					
				}

				foreach x in prebaseline baseline outcome{
					*Study charter
						gen `x'study= (inlist(`x'schoolnum,167, 175, 176, 196, 211, 212, 266, 328) | ///
							inlist(`x'schoolnum,330, 426, 428, 429, 435, 436, 437, 438, 443, 444, 445, 446) | ///
							inlist(`x'schoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391, 517) ///
							) if `x'spyear!=.
					
					*innov
						gen `x'innov = (inlist(`x'schoolnum, 159, 165, 166, 174, 177, 188, 189, 205, 243, 257, 282, 289, 297) | ///
							 inlist(`x'schoolnum, 299, 390, 395, 396, 397, 405, 419, 434, 447, 464, 466, 467, 468, 469, 471, 510, 511, 515) ///
							 ) if `x'spyear!=.
				}

				*In schools 419 469, grades 9 to 12 are magnet
				foreach grade in 6 7 8{
					replace prebaselineinnov = 0 	if pregradebase=="`grade'" & inlist(prebaselineschoolnum, 297) & prebaselinespyear!=.
					replace baselineinnov = 0 		if gradebase=="`grade'" & inlist(baselineschoolnum, 297) & baselinespyear!=.
					replace outcomeinnov = 0 		if appgrade=="`grade'" & inlist(outcomeschoolnum, 297) & outcomespyear!=.
				}
				
				foreach grade in 11{
					replace prebaselineinnov = 0 	if pregradebase=="`grade'" & inlist(prebaselineschoolnum, 468) & prebaselinespyear!=.
					replace baselineinnov = 0 		if gradebase=="`grade'" & inlist(baselineschoolnum, 468) & baselinespyear!=.
					replace outcomeinnov = 0 		if appgrade=="`grade'" & inlist(outcomeschoolnum, 468) & outcomespyear!=.				
				}
						
				
				keep  maskedid lastname firstname appschoolnum appschoolname appcharter appstudy appinnov appgrade apprank family offer pregradebase gradebase prebaseline* baseline* outcome* ///
					 pscore_charter_`file' pscore_innov_`file' pscore_school has_*

				order  outcome*, after(gradebase)
				renvars outcome*, predrop(7)
				order appgrade gradebase pscore_charter_`file' pscore_innov_`file', after(maskedid)
				
				*Study Charter offer
					gen offer_study = (offer==1) if appstudy==1
					
				*Innovation school offer	
					gen offer_innov = (offer==1) if appinnov==1	
					order offer_study offer_innov, after(offer)
					
				*Count obs with baseline
					
					*All applicants to charters 
					unique maskedid if has_study_pscore & baselinespyear==2013
					matrix obs[5,1]= r(sum)
					
					count if has_study_pscore & baselinespyear==2013
					matrix obs[5,2]= r(N)
					
					*All applicants to innovation
					unique maskedid if has_innov_pscore & baselinespyear==2013
					matrix obs[5,3]= r(sum)
					
					count if has_innov_pscore & baselinespyear==2013
					matrix obs[5,4]= r(N)
				
					*All applicants to charters & innovation
					unique maskedid if (has_study_pscore | has_innov_pscore) & baselinespyear==2013
					matrix obs[5,5]= r(sum)
					
					count if (has_study_pscore | has_innov_pscore) & baselinespyear==2013
					matrix obs[5,6]= r(N)
			
				** grade dummies
					tab grade, gen(g_)	
				
				*In sample means we have well-defined instrument
					gen in_study_sample	=	(has_study_pscore==1 & offer_study!=.)
					gen in_innov_sample	=	(has_innov_pscore==1 & offer_innov!=.)
					gen has_sc 	=	ss_math!=. | ss_writing!=. | ss_reading!=.
					
					label var in_study_sample "In study charters analysis sample (Z and Pscores nonmissing)"
					
					label var in_innov_sample "In innovation schools analysis sample (Z and Pscores nonmissing)"
								
					foreach x in math reading writing{
						gen has_sc_`x' = (ss_`x'!=.)
					}
					
					order has_*, after(pscore_innov_`file')
					order maskedid in_*_sample appschoolnum appschoolname appcharter appstudy appinnov apprank appgrade family pscore_charter_`file' pscore_innov_`file' pscore_school offer 
					ren gradebase baselinegrade
					order study innov, after(charter)

				save "`cleandata'mdrd_denver_innov_year_2_`file' `outputdate'.dta", replace

				svmat obs
				keep obs*
				keep if obs2!=.
				gen reason=""
				ren obs1 applicants_charter
				ren obs2 applications_charter
				ren obs3 applicants_innovation
				ren obs4 applications_innovation
				ren obs5 applicants_either
				ren obs6 applications_either
				replace reason ="All applicants" if _n==1
				replace reason ="Applicants to grades 4 through 10" if _n==2
				replace reason ="Applicants to (charters, innovation, either)" if _n==3
				replace reason ="Applicants with nonmissing pscore" if _n==4
				replace reason ="Denver at baseline (count, not dropped)" if _n==5
				order reason, first
				outsheet using "`tables'`file'_year_2_obscount.csv", comma replace
	}
}
