/* 	Ye Ji Kee

	Goal: 	Create analysis files (instruments + treatment + baseline + outcome) for MDmRD - for year 2 (School year 2013-2014)
			One file is created for each instrument
			
	IMPORTANT: 	Files pscore_charter_formula_`date' and pscore_charter_frequency_`date' must be
				placed into cleandata folder. These files are prepared using Yusuke's pscore
				routines, and can be found here: Y:\DPS\Yusuke\data.
				
				The code that generates propensity scores are:
					"Y:\DPS\Yusuke\create_pscore_charter_year_two_20160112.do", which runs:
					"Y:\DPS\Yusuke\create_pscore_charter_subroutine_20160112.do"
	
*/	

clear all
cap log close
set more off
set trace off
set maxvar 32767
adopath + "Z:\ado\plus\"

pause on

local rawdata   "Y:/DPS/Raw Data/"
local asgndata  "Y:/DPS/Raw Data/2013_03_21_enrollment-match/"
local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local yusuke	 "Y:/DPS/yusuke/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/year2/"


local pscore_setup =1		/*Propensity score matching setup*/

local simsdate 		20150603 	/*20150428 20150420 20150107*/
local date 			20170310 /*20160512  20160510 20160411 20160208 20160325 20160112 20151120 20150820 20150514 20150427 20150420 20150109 20141201 20141110 20141106 20141104*/
local outputdate	20170310 /*20160815 20160715 20160516	20160510 20160428  20160411 */
local typedate 		20170310

matrix obs = J(50,50,.)
		
*Retrieve application, offer, and grade information from AT RUN file
insheet using "`asgndata'1_AT_RUN_02262013.csv", double clear

	ren maskedstudentids maskedid
	ren requestedschool appschoolnum
	
	*Not sure what rank=99 means, ask Atila
	gen apprank = choiceselection
	replace apprank = apprank*10 if apprank<10
	replace apprank = floor(apprank/10)
	
	drop if maskedid=="#VALUE!"
	destring maskedid, ignore("#VALUE!") replace

	gen offer = (status=="A" | status == "FLA")
	/******************************************************************************************
	NOTE:
	In the year 2 data, there are 6 students (2 out of 6 students who are applying to grade 6)
	with 2 offers. The offers are given by the same school, but they are from different
	capacity programs. This problem also makes the "dstlink" variable non-unique. 
	
	bys maskedid: egen offer_count = sum(offer)
	tab offer_count
	browse if offer_count == 2
	*******************************************************************************************/
	
	bys maskedid appschoolnum: egen max_offer = max(offer)
	drop offer
	ren max_offer offer
	
	gen family = (familylinkmasked!="")
	
	keep grade maskedid offer appschoolnum apprank family

	ren grade appgrade
	duplicates drop maskedid appgrade appschoolnum offer apprank, force
	
	duplicates tag maskedid appgrade appschoolnum, gen(dup)
	tab dup
	
	*Cases of multiple rankings for the same school. Choose the smallest rank
	bys maskedid appgrade appschoolnum: egen min_rank = min(apprank)
	replace apprank = min_rank if dup
	
	
	*Keep if has nonmissing family link within school
	bys maskedid appgrade appschoolnum: egen max_fam=max(family)
	drop family
	ren max_fam family
	
	duplicates drop maskedid appgrade appschoolnum, force

	*All applicantions (school level)
		unique maskedid 
		matrix obs[1,1]= r(sum)
		matrix obs[1,2]= r(N)

		
	
	** All types
		preserve
		
		use "`cleandata'type_for_all_year_2_`typedate'.dta", clear
		
		duplicates drop maskedid appgrade appschoolnum, force
		
		unique type_all
		matrix obs[1,5] = r(sum)
		
		unique type_fc_all
		matrix obs[1,10] = r(sum)
		
		unique type_sc_all
		matrix obs[1,11] = r(sum)
	
		unique type_tc_all
		matrix obs[1,12] = r(sum)
		
		
		** No baseline for "EC"
				drop if appgrade == "EC"
				destring appgrade, replace
				
				gen gradebase = 	appgrade-1
				gen pregradebase =	appgrade-2
				
				tostring appgrade, replace
				tostring gradebase, replace
				
				replace gradebase = "EC" if gradebase == "-1"
				replace gradebase = "K" if gradebase == "0"
				
				destring maskedid, replace 
				merge m:1 maskedid using "`cleandata'sims_csap_wide_20102014 `simsdate'"
				drop if _merge == 2
				drop _merge
				
				gen baselinespyear = .
				
				foreach grade in EC K 1 2 3 4 5 6 7 8 9 10 11 {
				replace baselinespyear = spyear`grade' if gradebase == "`grade'"
				}
								
				unique type_all if baselinespyear == 2013
				matrix obs[1,6] = r(sum)	
				
				unique type_fc_all if baselinespyear == 2013
				matrix obs[1,13] = r(sum)
				
				unique type_sc_all if baselinespyear == 2013
				matrix obs[1,14] = r(sum)
				
				unique type_tc_all if baselinespyear == 2013
				matrix obs[1,15] = r(sum)
												
				
				
		restore
		
	*Identify charters in application
	preserve
		insheet using "`rawdata'dps_charters_20122013_20150428 update.csv", double clear
		keep schoolname schoolnum charter alternative
		ren schoolnum appschoolnum
		gen appcharter = (charter == 1 & alternative != 1)
		keep appschoolnum appcharter
		tempfile charter
		sa `charter'
	restore
	
	merge m:1 appschoolnum using `charter'
	keep if _merge!=2
	drop _merge
	replace appcharter=0 if appcharter==.
	
	*Study charters
	gen appstudy = (	inlist(appschoolnum, 167, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(appschoolnum, 330, 391, 392, 393, 394, 426, 428, 429, 435, 436, 437, 438, 441, 443, 444, 445, 446) | ///
						inlist(appschoolnum, 462, 481, 485, 496, 498, 512, 513, 517) ///
					)
	
	**Study no-excuses charters
	gen appnoex =  (	inlist(appschoolnum, 392, 393, 394, 426, 428, 429, 435, 436, 437) | ///
						inlist(appschoolnum, 438, 443, 444, 445, 462, 481, 498, 513, 517) ///
					)
	
	
	**Study non no-excuses charters
	gen appnonnoex = (	  inlist(appschoolnum, 167, 175, 176, 196, 211, 212) | ///
						  inlist(appschoolnum, 266, 328, 330, 391, 441, 446, 485, 496, 512) ///
					)
	
	** Traditional public schools
	gen apptrad = ( inlist(appschoolnum, 172, 178, 179, 190, 194, 195, 199, 203, 206, 207) | ///
						inlist(appschoolnum, 208, 209, 210, 213, 214, 215, 217, 218, 219, 220, 221) | ///
						inlist(appschoolnum, 222, 223, 224, 227, 228, 230, 231, 235, 236, 238, 240) | ///
						inlist(appschoolnum, 241, 242, 244, 246, 248, 249, 250, 252, 253, 254, 258) | ///
						inlist(appschoolnum, 259, 260, 261, 262, 263, 264, 269, 270, 271, 275, 276) | ///
						inlist(appschoolnum, 277, 278, 279, 280, 281, 283, 284, 285, 287, 290, 292) | ///
						inlist(appschoolnum, 294, 295, 296, 298, 301, 406, 407, 408, 412, 413, 416) | ///
						inlist(appschoolnum, 417, 418, 423, 448, 450, 451, 452, 453, 455, 456, 457, 458, 459, 463) ///
					)
	
	gen appstudy_fc = ( apprank == 1 & appstudy == 1)
	gen appstudy_sc = ( (apprank == 1  | apprank == 2) & appstudy == 1)
	gen appstudy_tc = ( (apprank == 1 | apprank == 2 | apprank == 3) & appstudy == 1)
	
	
	*Get latest school names
	preserve
		u "`cleandata'sims_csap_long_20102014 `simsdate'", clear
		
		keep schoolnum schoolname spyear
		bys schoolnum: egen max_year = max(spyear)
		keep if spyear==max_year
		drop spyear max_year
		
		*schools with slighlty different names
		duplicates drop schoolnum, force
		ren schoolnum appschoolnum
		ren schoolname appschoolname
		tempfile schools
		sa `schools'
	restore	
	
	merge m:1 appschoolnum using `schools'
	keep if _merge!=2
	drop _merge
	order maskedid appschoolnum appschoolname
	label var appschoolname "Requested school name"
	
	tab appgrade
	
	**Duplicates exist for maskedid = 786731 (Applied to the same set of schools, but different grade)
	drop if maskedid == 786731 & appgrade == "03"
	
	*Merge with type data so that we can count how many types there are
	tostring maskedid, replace
	
	preserve
	use "`cleandata'fulltypes_year_2_`typedate'.dta", clear

	duplicates drop maskedid, force
	keep maskedid type_all type_fc_all type_sc_all type_tc_all
	
	tempfile type_all
	sa `type_all'
	
	restore
	
	merge m:1 maskedid using `type_all'
	drop _merge
	
	destring maskedid, gen(maskedid_tmp)
	drop maskedid
	rename maskedid_tmp maskedid
	ren type_all applicant_type
	ren type_fc_all applicant_type_fc
	ren type_sc_all applicant_type_sc
	ren type_tc_all applicant_type_tc
	label var applicant_type "Unique combination of all bucket preferences and priorities"
	
	
	sa "`cleandata'AT_RUN_year_2_school_level.dta", replace
	
if `pscore_setup' == 1 {
	
	*One file for each instrument (no simulation file for year 2 at the moment)
		foreach file in simulation  formula frequency {
		foreach sch_type in study /*noex nonnoex trad*/ {
			
			if "`file'"=="simulation"{
				local pscoredate 20160815 /* 20160324 */
			}
			else{
				local pscoredate 20160411 /*20160512 20160510  20160208*/
			}

			** Counting applicants and applications that are in denver in baseline		
			
			preserve
				u "`cleandata'pscore_`sch_type'_`file'_year_two_`pscoredate'.dta", clear
				
				gen spyear =2014
				cap ren maskedstu maskedid
				cap destring maskedid, replace
						
				*Grades that have scores
				merge 1:m maskedid using "`cleandata'AT_RUN_year_2_school_level.dta", keepusing(offer appgrade appcharter appstudy appstudy_fc appstudy_sc appstudy_tc apptrad appnoex appnonnoex appschoolnum appschoolname apprank family applicant_type applicant_type_fc applicant_type_sc applicant_type_tc)
	
				*Applicants to any charters
				bys maskedid: egen any_`sch_type' = max(app`sch_type')
				bys maskedid: egen any_study_fc = max(appstudy_fc)
				bys maskedid: egen any_study_sc = max(appstudy_sc)
				bys maskedid: egen any_study_tc = max(appstudy_tc)
						
				*All kids with missing pscores should be treated as pscore==0
				replace pscore = 0 if pscore==. & any_`sch_type'==1
				
				gen has_pscore = (pscore!=.)
				drop _merge
				
				*Grades 4-10 (baseline and outcome scores are available)
				tab appgrade
				keep if inlist(appgrade,"04","05","06","07","08","09","10")
				unique maskedid
				matrix obs[2,1]= r(sum)
				matrix obs[2,2]= r(N)
	
				unique applicant_type
				matrix obs[2,5]= r(sum)
				
				unique applicant_type_fc
				matrix obs[2,10] = r(sum)
				
				unique applicant_type_sc
				matrix obs[2,11] = r(sum)				
				
				unique applicant_type_tc
				matrix obs[2,12] = r(sum)
				
				*All applicants to charters
					unique maskedid if any_`sch_type'
					matrix obs[3,1]= r(sum)
					
					count if any_`sch_type'
					matrix obs[3,2]= r(N)
				
					unique applicant_type if any_`sch_type'
					matrix obs[3,5]= r(sum)
					
					unique applicant_type_fc if any_`sch_type'
					matrix obs[3,10]= r(sum)
					
					unique applicant_type_sc if any_`sch_type'
					matrix obs[3,11]= r(sum)

					unique applicant_type_tc if any_`sch_type'
					matrix obs[3,12]= r(sum)
					
					
					
				drop any_`sch_type'	
					
				*Applicants with risksets and offers
				*keep if _merge==3
				
				unique maskedid if has_pscore
				matrix obs[4,1]= r(sum)
				
				count if has_pscore
				matrix obs[4,2]= r(N)

				unique applicant_type if has_pscore
				matrix obs[4,5]= r(sum)

				unique applicant_type_fc if has_pscore
				matrix obs[4,10]= r(sum)
				
				unique applicant_type_sc if has_pscore
				matrix obs[4,11]= r(sum)
				
				unique applicant_type_tc if has_pscore
				matrix obs[4,12]= r(sum)
	
				local row = 5
				
				foreach choice in fc sc tc {
				
				
				*All applicants to first/upto second/upto third-choice charters
					unique maskedid if any_study_`choice' == 1
					matrix obs[`row',1]= r(sum)
					
					count if any_study_`choice' == 1
					matrix obs[`row',2]= r(N)
					
					unique applicant_type if any_study_`choice' == 1
					matrix obs[`row',5]= r(sum)

					unique applicant_type_fc if any_study_`choice' == 1
					matrix obs[`row',10]= r(sum)
					
					unique applicant_type_sc if any_study_`choice' == 1
					matrix obs[`row',11]= r(sum)

					unique applicant_type_tc if any_study_`choice' == 1
					matrix obs[`row',12]= r(sum)
					
					drop  any_study_`choice'
					
					local ++row
				}
								
				destring appgrade, replace
				
				gen gradebase = 	appgrade-1
				gen pregradebase =	appgrade-2
				
				tostring appgrade, replace
				tostring gradebase, replace
				tostring pregradebase, replace
				
				tempfile temp1
				sa `temp1'
			restore

			
			*Get pscore with probability of assignment at each school
			if "`file'"!="simulation"{			
				preserve	
					u "`cleandata'application_data_year_two_`date'", clear
					
					ren maskedstudentids maskedid
					ren requestedschool appschoolnum
					destring maskedid, replace
				
				if "`file'"=="frequency"{
					drop pscore_formula
					ren frequency_offer pscore_frequency
				}


					bys maskedid appschoolnum: egen pscore_school_`file' = sum(pscore)
					
					duplicates drop maskedid appschoolnum, force
					keep maskedid appschoolnum pscore_school
					tempfile temp2
					sa `temp2'
				restore
			}
			
			else{
				preserve
					u "`cleandata'pscore_simulation_school_grade_year2", clear
					keep maskedid pscore_school appschoolnum
					destring maskedid, replace
					tempfile temp2
					sa `temp2'
				restore	
			}
				*Combine pscore files
				u `temp1', clear
				
				merge 1:1 maskedid appschoolnum using `temp2'
				keep if _merge!=2
				drop _merge		
				tempfile temp
				sa `temp'
		
			
			u "`cleandata'sims_csap_wide_20102014 `simsdate'", clear
			
			unique maskedid
			merge 1:m maskedid using `temp'
			*keep if applicants only
			keep if _merge!=1
			drop _merge
			
			*Construct baseline and outcome variables
			
				foreach x in 	spyear	schoolnum charter alternative female black white hisp asian otherrace ///
								repeat	sped	foodfred	lep	bilingual ///
								gifted gt_ela gt_math	gt_math_ela	gt_other	///
								raw_ss_math	 ss_math		pp_up_math		prof_up_math	adv_math ///
								raw_ss_reading	ss_reading	pp_up_reading 	prof_up_reading	adv_reading	///
								raw_ss_writing ss_writing	pp_up_writing	prof_up_writing adv_writing{
						
						gen prebaseline`x'=.
						gen baseline`x'=.	
						gen outcome`x'=.
				
					foreach grade in 2 3 4 5 6 7 8 9 10{
					
						replace prebaseline`x' = `x'`grade' if pregradebase=="`grade'"
						replace baseline`x' = `x'`grade' if gradebase=="`grade'"
						replace outcome`x' = `x'`grade' if appgrade=="`grade'"
						
					}
					
				}
				
				foreach x in prebaseline baseline outcome{
					gen `x'study= (inlist(`x'schoolnum, 167, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(`x'schoolnum, 330, 391, 392, 393, 394, 426, 428, 429, 435, 436, 437, 438, 441, 443, 444, 445, 446) | ///
						inlist(`x'schoolnum, 462, 481, 485, 496, 498, 512, 513, 517) ///
					) if `x'spyear!=.
					
					gen `x'noex = (inlist(`x'schoolnum, 392, 393, 394, 426, 428, 429, 435, 436, 437) | ///
								   inlist(`x'schoolnum, 438, 443, 444, 445, 462, 481, 498, 513, 517) ///
								   ) if `x'spyear != .
								   
					gen `x'nonnoex = (inlist(`x'schoolnum, 167, 175, 176, 196, 211, 212) | ///
									  inlist(`x'schoolnum, 266, 328, 330, 391, 441, 446, 485, 496, 512) /// 
									  ) if `x'spyear != .
				
					gen `x'trad =	( inlist(`x'schoolnum, 172, 178, 179, 190, 194, 195, 199, 203, 206, 207) | ///
									inlist(`x'schoolnum, 208, 209, 210, 213, 214, 215, 217, 218, 219, 220, 221) | ///
									inlist(`x'schoolnum, 222, 223, 224, 227, 228, 230, 231, 235, 236, 238, 240) | ///
									inlist(`x'schoolnum, 241, 242, 244, 246, 248, 249, 250, 252, 253, 254, 258) | ///
									inlist(`x'schoolnum, 259, 260, 261, 262, 263, 264, 269, 270, 271, 275, 276) | ///
									inlist(`x'schoolnum, 277, 278, 279, 280, 281, 283, 284, 285, 287, 290, 292) | ///
									inlist(`x'schoolnum, 294, 295, 296, 298, 301, 406, 407, 408, 412, 413, 416) | ///
									inlist(`x'schoolnum, 417, 418, 423, 448, 450, 451, 452, 453, 455, 456, 457, 458, 459, 463) ///
								)
				}
				
				
				if "`file'"!="simulation"{			
					if "`sch_type'"=="study"{
						rename pscore_charter_`file' pscore_study_`file'
					}
				}
				
				keep  maskedid lastname firstname appschoolnum appschoolname appcharter appstudy apptrad appnoex appnonnoex appgrade apprank family offer pregradebase gradebase prebaseline* baseline* outcome* ///
					 pscore_`sch_type'_`file' pscore_school has_pscore applicant_type

				order  outcome*, after(gradebase)
				renvars outcome*, predrop(7)
				order appgrade gradebase pscore_`sch_type'_`file', after(maskedid)
				
				*Study Charter offer
					gen offer_`sch_type' = (offer == 1) if app`sch_type' == 1
					order offer_`sch_type', after(offer)
		
				** grade dummies
				tab grade, gen(g_)	
				
				gen app_right_grade = (spyear==2014)
		
					
				*In sample means we have well-defined instrument
					gen in_sample	=	(has_pscore==1 & offer_`sch_type'!=.)
					gen has_sc 	=	ss_math!=. | ss_writing!=. | ss_reading!=.
					
					label var in_sample "In analysis sample (Z and pscores nonmissing)"
					
					foreach x in math reading writing{
						gen has_sc_`x' = (ss_`x'!=.)
					}
					
					order has_*, after(pscore_`sch_type'_`file')
					order maskedid in_sample app_right_grade appschoolnum appschoolname appcharter appstudy apptrad appnoex appnonnoex apprank appgrade family pscore_`sch_type'_`file' pscore_school offer 
					ren gradebase baselinegrade
					order `sch_type', after(charter)
					
				sa "`cleandata'mdrd_denver_`sch_type'_year_2_`file' `outputdate'.dta", replace

				svmat obs
				keep obs*
				gen reason=""
				ren obs1 applicants
				ren obs2 applications
				ren obs3 applicants_baseline
				ren obs4 applications_baseline
				ren obs5 applicant_type
				ren obs6 applicant_type_baseline
				ren obs7 applicants_baseline_in01
				ren obs8 applications_baseline_in01
				ren obs9 applicant_type_baseline_in01
				ren obs10 applicant_type_fc
				ren obs11 applicant_type_sc
				ren obs12 applicant_type_tc
				ren obs13 applicant_type_fc_baseline
				ren obs14 applicant_type_sc_baseline
				ren obs15 applicant_type_tc_baseline
				ren obs16 applicant_type_fc_baseline_in01
				ren obs17 applicant_type_sc_baseline_in01
				ren obs18 applicant_type_tc_baseline_in01
				ren obs19 offer_first_choice_baseline
				ren obs20 offer_sc_baseline
				ren obs21 offer_tc_baseline
				ren obs22 charter_applications_baseline
				ren obs23 charter_applicants_baseline
				ren obs24 offer_first_choice_baseline_in01
				ren obs25 offer_sc_baseline_in01
				ren obs26 offer_tc_baseline_in01
				ren obs27 charter_apps_baseline_in01
				ren obs28 charter_applicants_baseline_in01
				replace reason ="All applicants" if _n==1
				replace reason ="Applicants to grades 4 through 10" if _n==2
				replace reason ="Applicants to any `sch_type' charters" if _n==3
				replace reason ="Applicants with nonmissing pscore" if _n==4
				replace reason ="Applicants to anystudy charters (first choice)" if _n==5
				replace reason ="Applicants to anystudy charters (upto second choice)" if _n==6
				replace reason ="Applicants to anystudy charters (upto third choice)" if _n==7		
				order reason, first
				outsheet using "`tables'`file'_`sch_type'_obscount_year_2.csv", comma replace
		}
	}
}

