/* 	Ye Ji Kee
	Denver main deck tables. These comments last edited: 1/14/2016
		For grades 4-10
		Output: Balance, 2SLS, and subgroup for pscore risksets and qualification instruments
		
	Ignacio Rodriguez Hurtado
	04/05/2016
	
	There is a problem in the pscore_balance section. 
	* For one of the balance regessions, WHen you do it with the variable rank1_study ,you end up with only 1 as an outcome for all the outcomes in the sample
	* When you then use the Srank1_study post estimation commmands later on, stata throws an error because of missing values. 
	The reason why rank1_study == 1 for all people in the sample [if in_good_cell_r0==1 & obs_cell_r0>= `min_obs' & !missing(obs_cell_r0)]
	Is that the simulated score has a lot of support. As a result few cells have variation and are therefore not suitable. You end up with 
	around 190 kids in the sample, and all of them have rank1_study == 1. 
	Compared to year 1, the same statement results in 300 kids, but only 3 have rank1_study == 0. 
	This only happens when you drop the small cells  in the loop (drop_small_cells)
	I'm handling this by just excluding the Srank1_study post estimation for simulated score, 
	Search for IR 04/05/2016 for further comments
*/

clear all
cap log close
set more off
set trace off
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/year2/"

matrix demo			  = J(100,40,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,40,.)
matrix subtwosls	  = J(500,40,.)
matrix twosls_nocov    		   = J(500,40,.)
matrix twosls_scorecov 		   = J(500,40,.)

matrix qualbalance    = J(500,20,.)
matrix qualtwosls	  = J(500,20,.)
matrix qualsubtwosls  = J(500,20,.)

local pscore_setup = 1

*Dates of analysis files
local pscoredate	20170310 /*20160815 20160715 20160516 20160510 20160428 20160411 20160208  20160112 20151120 20150514 */
local simsdate 		20150603 /* 20150428 */
local qualdate 		20160303 /*20150420*/

local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinetrad baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_*"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1        		/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
local outsheet_rescorr=0	/* If want to outsheet matrix with residual correlations */


	************************************************
	**************** Pscore Setup ******************
	************************************************		
	
	if `pscore_setup'==1{
		
		foreach file in frequency simulation formula  {
		foreach sch_type in study /*noex nonnoex*/ {
	
			use "`cleandata'mdrd_denver_`sch_type'_year_2_`file' `pscoredate'", clear
			
			*Simulation has huge support. Use rounded values for full saturation
				if "`file'"=="simulation"{
					ren pscore_`sch_type'_simulation sim_pscore
					gen double pscore_`sch_type'_simulation = round(sim_pscore,0.0001)
				}
			
			destring baselinegrade, replace
			destring appgrade, gen(appgrade_tmp)
			drop appgrade
			gen appgrade = appgrade_tmp
			drop appgrade_tmp
			
			*High school flag			
				gen hsgrade = ( appgrade>=9)
				gen msgrade = ( appgrade<9)
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
				
			*Analysis at the student level. Generate max for charter offer
			bys maskedid: egen any_`sch_type'_offer 	= max(offer_`sch_type')
			bys maskedid: egen num_schools = count(appschoolnum)
			bys maskedid: egen num_study = count(cond(appstudy==1,appschoolnum,.))
			gen pct_study = num_study/num_schools
			
			*Enrolls at charter flag (student level)
			bys maskedid: egen enroll_any_`sch_type' 	= max(`sch_type')
			
			*Rank 1 at school level
			gen rank1_study= (appstudy==1 & apprank==1)
	
			*Offer from first choice study charter
			gen rank1_study_offer = (appstudy == 1 & apprank == 1 & offer == 1)
				
			************************** School oversubscription table **************************
			*Schools ranked, assigned, 0 < prob assignment < 1 
			preserve
				keep if app`sch_type'==1
				bys appschoolnum: egen tot_rank1 = sum(rank1_study)
				bys appschoolnum: egen tot_rank = count(maskedid)
				bys appschoolnum: egen tot_offer = sum(offer)
				bys appschoolnum: egen tot_`sch_type'_not01 = count(cond(pscore_`sch_type'_`file'>0 & pscore_`sch_type'_`file'<1,maskedid,.))
				bys appschoolnum: egen tot_school_not01 = count(cond(pscore_school>0 & pscore_school<1,maskedid,.))
				bys appschoolnum: egen tot_school_is1 = count(cond(pscore_school==1,maskedid,.))
				bys appschoolnum: egen tot_school_not01_firstchoice = count(cond(pscore_school > 0 & pscore_school < 1 & apprank == 1,maskedid,.))
		
				duplicates drop appschoolnum, force
				keep appschoolnum appschoolname tot_* MS HS
				
				tempfile temp
				sa `temp'
			restore
			
			*Subject to randomization at the FULLY SATURATED PSCORE level (variation in instrument for every value of PSCORE) preserve
			foreach cut in inDPSbase /*allapps*/{
				
				preserve

					if "`cut'"=="inDPSbase"{
						drop if baselinespyear!=2013
					}
					else{
						di in red "Keep all applicants"
					}
			
					keep if app`sch_type'==1
					
					*Any charter offer pscore - by school, the number of kids in pscore sets with variation in any offer
						bys pscore_`sch_type'_`file': egen min_offer = min(any_`sch_type'_offer)
						bys pscore_`sch_type'_`file': egen max_offer = max(any_`sch_type'_offer)
						
						gen sum = min_offer + max_offer
						bys appschoolnum: egen tot_`sch_type'_rand = count(cond(sum==1 ,maskedid,.))
						bys appschoolnum: egen tot_`sch_type'_rand_not01 = count(cond(sum==1 & pscore_`sch_type'_`file'>0 & pscore_`sch_type'_`file'<1,maskedid,.))
							
					*School offer - if within the same school, there are kids with the same pscore and var in offer
						drop min_offer max_offer sum
						bys appschoolnum pscore_school: egen min_offer = min(offer)
						bys appschoolnum pscore_school: egen max_offer = max(offer)
						
						gen sum = min_offer + max_offer
						bys appschoolnum: egen tot_school_rand = count(cond(sum==1,maskedid,.))
						bys appschoolnum: egen tot_school_rand_not01 = count(cond(sum==1 & pscore_school>0 & pscore_school<1,maskedid,.))
						drop min_offer max_offer
						
						*Number of kids with family link
						bys appschoolnum: egen tot_family = sum(family)
						
						duplicates drop appschoolnum, force
						keep appschoolnum  tot_*
						
						merge 1:1 appschoolnum using `temp'
						drop _merge
						
						order appschoolnum appschoolname MS HS ///
							tot_rank1 tot_rank	tot_offer ///
							tot_`sch_type'_not01	tot_school_not01 tot_school_is1 ///
							tot_`sch_type'_rand	tot_`sch_type'_rand_not01	///
							tot_school_rand	tot_school_rand_not01
					
						outsheet using "`tables'`cut'_`file'_`sch_type'_year_2_schools.csv", comma replace

				restore
			}

			******************************************************************************
			
			*First ranked school is charter (student level, for cov balance)
			bys maskedid: egen temp = max(rank1_study)
			drop rank1_study
			ren temp rank1_study
				
			bys maskedid: egen rank1_offer_temp = max(rank1_study_offer)
			drop rank1_study_offer
			ren rank1_offer_temp rank1_study_offer		
			
			*First ranked charter is oversubscribed (first-choice IV strategy sample)
			gen rank1_study_os_tmp = (rank1_study == 1 & ( inlist(appschoolnum, 175, 211, 328, 426, 428, 429) | ///
									   inlist(appschoolnum, 438, 443, 444, 462, 481, 513, 167, 176) ))
									   
			bys maskedid: egen rank1_study_os = max(rank1_study_os_tmp)
			drop rank1_study_os_tmp
			
			
			*Flag - apply to any study
			bys maskedid: egen max = max(app`sch_type')
			drop app`sch_type'
			ren max app`sch_type'
			
			*Enrolls next year (i.e. has nonmissing schoolnum)
			gen next = (schoolnum!=.)
			bys maskedid: egen enrolls_next = max(next)
			
			*** Dataset at student level from here on ****
			
			*In_sample was previously defined at the student-school level. Redefine it so
			*that it is at the student level
			
			bys maskedid: egen in_sample_max = max(in_sample)
			tab in_sample_max in_sample
			
			drop in_sample
			rename in_sample_max in_sample
			
			duplicates drop maskedid, force
			drop next
			
			
			*Redefine has scores so that it excludes sciences - very few kids
			replace has_sc = (has_sc_math | has_sc_reading | has_sc_writing)
			
			*Generate application school-grade dummies
			*egen sg = group(appschoolnum appgrade)
			*tab sg, gen(sg_)
			
			*School-level obs
			drop appschoolnum appschoolname offer offer_`sch_type' appcharter apprank `sch_type' charter
			
			*Keep only if in sample
			keep if in_sample==1
			
			*If simulation, replace back pscore with full support (just for bins)
			if "`file'"=="simulation"{
				replace pscore_`sch_type'_simulation = sim_pscore
			}
			
			
			****** Rounded scores for full saturation ******
				
				*Define r01 as meaning pscore rounded to 0.01
				gen r01 = round(pscore_`sch_type', 0.01)
				tab r01, gen(r01_)
			
				*Define r001 as meaning pscore rounded to 0.001
				gen r001 = round(pscore_`sch_type', 0.001)
				tab r001, gen(r001_)

			
			*Polynomials
			forvalues i=1/4{
				gen double pscore`i'=pscore_`sch_type'^(`i')
			}
			
			*If simulation, use rounded for full saturation
			if "`file'"=="simulation"{
				replace pscore_`sch_type'_simulation = round(sim_pscore, 0.0001)
			}
			else{
				di in red "Do nothing"
			}
			
			tab pscore_`sch_type'_`file', gen(r0_)
					
			drop pscore_school
			ren pscore_`sch_type'_`file' r0
			
			gen baselinemissing =(baselinespyear!=2013)
			keep if baselinespyear == 2013
			
			*Treatment is any study, instrument is study offer, riskset is pscore
			ren any_`sch_type'_offer 	Z
			ren enroll_any_`sch_type'  	D
			
			**********************************************************************************
			******************** Flag kids who are ineligble and guaranteed ******************
			**********************************************************************************
			*Flag kids who are ineligble and guaranteed - do not drop them, we still want them 
			*for OLS regressions and naive 2SLS (i.e. not controlling for risksets)
			
			gen in_zero1_pscore = (r0 > 0 & r0 < 1)
			
			**********************************************************************************
			********** Flag cells with variation in instrument and characteristics************
			**********************************************************************************
					
			/* 	Flag observations that actually make it into balance (i.e. obs in cells with
				variation in instrument). These flags are used for all types 
				of F-tests, conducted below.
				
				We generate these flags here so that no re-sorting of the data (done in bys x egen:)
				is done between the balanceregs and F tests, which affects how the variable
				"var", which stores the names of the variables corresponding to each balance
				coefficient, appears in the final dataset once we svmat the balance output.
			*/
			
			foreach x in r01 r001 r0{
				
				*Variation in instruments for each cell
					bys `x': egen sd_z_`x'	= 	sd(Z)
				
				*Good cell
					gen in_good_cell_`x' = (sd_z_`x'>0 & !missing(sd_z_`x') & in_zero1_pscore==1)
				
				gen double tag_`x' = `x'
				replace tag_`x' = -9999999 if in_good_cell_`x'==0
				tab tag_`x', gen(tag_`x'_)
				
				*Drop var for things with no var (the first one). These observations will not be included
				*in bin-based regression and fully saturated regressios. So, do not include their corresponding
				*variables either
				
				drop tag_`x'_1
				
				*Obs per cell
				
				bys `x': egen obs_cell_`x' = count(maskedid)
			}
	
			drop sd_z_*
			
			sa "`cleandata'analysis_`sch_type'_year_2_`file'.dta", replace
		}
		}
	}	

