/* 	Mayara Felix
	10/25/2014
	
	Ignacio Rodriguez
	03/31/2016
	
	This file now uses the updated analysis file produced from do file 4 (4_create_fulltype_analysis_file)
*/

clear all
cap log close
set more off
set trace off
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:\ado\plus\"

global cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/year2/"


matrix balance        = J(500,20,.)

local file_setup = 1
local balance = 1

local date = 20170310    /*20160815 20160502 20160425 20160331 20150708 20150603 20150428 20150107 20141201 20141124 20141118 20141110 20141106 20141104*/

local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

local cov_grade "g_*"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1

	************************************************
	**************** File Setup ********************
	************************************************		
	
	if `file_setup'==1{
		use "${cleandata}mdrd_denver_fulltype_year_2 `date'", clear
		destring baselinegrade, replace
		destring appgrade, gen(appgrade_temp)
		drop appgrade
		rename appgrade_temp appgrade
		
		*Analysis at the student level. Generate max for charter offer
		bys maskedid: egen any_study_offer 	= max(offer_study)
		bys maskedid: egen num_schools = count(appschoolnum)
		bys maskedid: egen num_study = count(cond(appstudy==1,appschoolnum,.))
		gen pct_study = num_study/num_schools
		
		*only keep if at baseline
		keep if baselinespyear==2013

		
		*Enrolls at charter flag (student level)
		bys maskedid: egen enroll_any_study 	= max(study)
		
		*Rank 1 at school level
		gen rank1_study= (appstudy==1 & apprank==1)
	
		*First ranked school is charter (student level, for cov balance)
		bys maskedid: egen temp = max(rank1_study)
		drop rank1_study
		ren temp rank1_study
		
		*Flag - apply to any study
		bys maskedid: egen max = max(appstudy)
		drop appstudy
		ren max appstudy
		
		gen next = (schoolnum!=.)
		
		*** NOTE: in_sample currently at the school level. Need to change it to be at the individual level.
		bys maskedid: egen any_in_sample = max(in_sample)
		
		bys maskedid: egen enrolls_next = max(next)
		duplicates drop maskedid, force
		drop next
		
		*Redefine has scores so that it excludes sciences - very few kids
		replace has_sc = (has_sc_math | has_sc_reading | has_sc_writing)
		
		*Generate application school-grade dummies
		*egen sg = group(appschoolnum appgrade)
		*tab sg, gen(sg_)
		
		*School-level obs
		drop appschoolnum appschoolname offer offer_study appcharter apprank study charter
		
		
		**Treatment is any study, instrument is study offer, riskset is pscore**
		ren any_study_offer 	Z
		ren enroll_any_study  	D
				
		bys type_all: egen sd_z = sd(Z)

		gen in_good_cell = (	sd_z>0 & !missing(sd_z) & type_all != .	)
		

		save "${cleandata}analysis_fulltype_year_2.dta", replace

	}
