/* 	Ye Ji Kee
	Denver main deck tables. These comments last edited: 4/25/2015
		For grades 4-10
		Output: Balance, 2SLS, and subgroup for pscore risksets and qualification instruments
*/

clear all
cap log close
set more off
set trace off
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"

matrix demo			  = J(30,30,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,100,.)
matrix jointtwosls	  = J(500,100,.)

local pscore_setup_yr1 = 1
local pscore_setup_yr2 = 1
local pscore_stacking = 0
local pscore_balance = 0


*Dates of analysis files
local pscoredate	20160715 /*20160425*/
local simsdate 		20150603	/*20150428*/

local appcov_c 	"num_schools num_study rank1_study"
local cov_c 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov_innov 	"baselineinnov baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_1 g_2 g_3 g_4 g_5 g_6 g_7"

local pscoredate_year2	20160425
local pscoredate_year1  20160425

local appcov_i  "num_schools num_innov rank1_innov"
local cov_i		"baselineinnov baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local appcov_either  "num_schools num_study rank1_study num_innov rank1_innov"
local cov_either 	"baselinestudy baselineinnov baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1				/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
		
	


	************************************************
	**************** Pscore Setup (for yr 1) ******************
	************************************************	
	
	if `pscore_setup_yr1'==1{
		
		foreach file in simulation frequency formula{
	
			use "`cleandata'mdrd_denver_innov_`file' `pscoredate'", clear
			
			destring baselinegrade, replace
			destring appgrade, gen(appgrade_tmp)
			drop appgrade
			rename appgrade_tmp appgrade
			
			*High school flag			
				gen hsgrade = ( appgrade>=9)
				gen msgrade = ( appgrade<9)
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
				
			*Analysis at the student level. Generate max for charter offer
			bys maskedid: egen any_study_offer 	= max(offer_study)
			bys maskedid: egen any_innov_offer 	= max(offer_innov)
			
			bys maskedid: egen num_study = count(cond(appstudy==1,appschoolnum,.))
			bys maskedid: egen num_innov = count(cond(appinnov==1,appschoolnum,.))
			
			bys maskedid: egen num_schools = count(appschoolnum)
			
			*Enrolls at charter flag (student level)
			bys maskedid: egen enroll_any_study 	= max(study)
			bys maskedid: egen enroll_any_innov		= max(innov)
			
			*Rank 1 at school level
			gen rank1_study= (appstudy==1 & apprank==1)
			gen rank1_innov= (appinnov==1 & apprank==1)
		
			************************** School oversubscription table **************************
			*Schools ranked, assigned, 0 < prob assignment < 1 
			preserve
				keep if appinnov==1
				bys appschoolnum: egen tot_rank1 = sum(rank1_innov)
				bys appschoolnum: egen tot_rank = count(maskedid)
				bys appschoolnum: egen tot_offer = sum(offer)
				bys appschoolnum: egen tot_innov_not01 = count(cond(pscore_innov_`file'>0 & pscore_innov_`file'<1,maskedid,.))
				bys appschoolnum: egen tot_school_not01 = count(cond(pscore_school>0 & pscore_school<1,maskedid,.))
				bys appschoolnum: egen tot_school_is1 = count(cond(pscore_school==1,maskedid,.))
				bys appschoolnum: egen tot_school_not01_firstchoice = count(cond(pscore_school > 0 & pscore_school < 1 & apprank == 1,maskedid,.))
						
				duplicates drop appschoolnum, force
				keep appschoolnum appschoolname tot_* MS HS
				
				tempfile temp
				sa `temp'
			restore
			
			
			*Subject to randomization at the FULLY SATURATED PSCORE level (variation in instrument for every value of PSCORE) preserve
				
			preserve
				keep if baselinespyear==2012
				keep if appinnov==1
				
				*Simulation has huge support. Use rounded values for full saturation
				if "`file'"=="simulation"{
					ren pscore_innov_simulation sim_pscore_i
					gen double pscore_innov_simulation = round(sim_pscore_i,0.001)
				}
				
				*Any charter offer pscore - by school, the number of kids in pscore sets with variation in any offer
					bys pscore_innov_`file': egen min_offer = min(any_innov_offer)
					bys pscore_innov_`file': egen max_offer = max(any_innov_offer)
					
					gen sum = min_offer + max_offer
					bys appschoolnum: egen tot_innov_rand = count(cond(sum==1 ,maskedid,.))
					bys appschoolnum: egen tot_innov_rand_not01 = count(cond(sum==1 & pscore_innov_`file'>0 & pscore_innov_`file'<1,maskedid,.))
						
				*School offer - if within the same school, there are kids with the same pscore and var in offer
					drop min_offer max_offer sum
					bys appschoolnum pscore_school: egen min_offer = min(offer)
					bys appschoolnum pscore_school: egen max_offer = max(offer)
					
					gen sum = min_offer + max_offer
					bys appschoolnum: egen tot_school_rand = count(cond(sum==1,maskedid,.))
					bys appschoolnum: egen tot_school_rand_not01 = count(cond(sum==1 & pscore_school>0 & pscore_school<1,maskedid,.))
					drop min_offer max_offer
					
					*Number of kids with family link
					bys appschoolnum: egen tot_family = sum(family)
					
					duplicates drop appschoolnum, force
					keep appschoolnum  tot_*
					
					merge 1:1 appschoolnum using `temp'
					drop _merge
					
					order appschoolnum appschoolname MS HS ///
						tot_rank1 tot_rank	tot_offer ///
						tot_innov_not01	tot_school_not01 ///
						tot_innov_rand	tot_innov_rand_not01	///
						tot_school_rand	tot_school_rand_not01
				
					outsheet using "`tables'inDPSbase_`file'_innov_schools.csv", comma replace

			restore

			******************************************************************************
			
			*First ranked school is charter (student level, for cov balance)
			foreach type in study innov{
				bys maskedid: egen temp`type' = max(rank1_`type')
				drop rank1_`type'
				ren temp`type' rank1_`type'
				
				*Flag - apply to any study
				bys maskedid: egen max`type' = max(app`type')
				drop app`type'
				ren max`type' app`type'
			}
			
			*Enrolls next year (i.e. has nonmissing schoolnum)
			gen next = (schoolnum!=.)
			bys maskedid: egen enrolls_next = max(next)
			
			*In_sample was previously defined at the student-school level. Redefine it so
			*that it is at the student level
			
			foreach type in study innov{
				bys maskedid: egen in_`type'_sample_max = max(in_`type'_sample)
				tab in_`type'_sample_max in_`type'_sample
				
				drop in_`type'_sample
				renvars in_`type'_sample_max, postdrop(4)
			}
			
			*** Dataset at student level from here on ****
			duplicates drop maskedid, force
			drop next
			
			*Redefine has scores so that it excludes sciences - very few kids
			replace has_sc = (has_sc_math | has_sc_reading | has_sc_writing)
			
			*School-level obs
			drop appschoolnum appschoolname offer offer_study offer_innov appcharter apprank study innov charter

			*Keep only if in sample
			keep if in_study_sample==1 | in_innov_sample==1
			
			*Define r001 as meaning pscore rounded to 0.001 and r01 as rounded to 0.01
			gen c001 = round(pscore_charter, 0.001)
			gen c01 = round(pscore_charter, 0.01)
			
			gen i001 = round(pscore_innov, 0.001)
			gen i01 = round(pscore_innov, 0.01)
			
			tab c001, gen(c001_)
			tab c01, gen(c01_)
			
			tab i01, gen(i01_)
			tab i001, gen(i001_)
		
			*Polynomials
			forvalues i=1/4{
				gen double pscore_c`i'=pscore_charter^(`i')
				gen double pscore_i`i'=pscore_innov^(`i')
			}
			
			keep if baselinespyear==2012
			
			*Treatment is any study, instrument is study offer, riskset is pscore
			ren any_study_offer 	Z_c
			ren any_innov_offer 	Z_i
			
			ren enroll_any_study  	D_c
			ren enroll_any_innov  	D_i
			
			**********************************************************************************
			******************** Flag kids who are ineligble and guaranteed ******************
			**********************************************************************************
			*Flag kids who are ineligble and guaranteed - do not drop them, we still want them 
			*for OLS regressions and naive 2SLS (i.e. not controlling for risksets)
			
				*If simulation, first define in_zero1_pscore based on true score, then
			
					if "`file'"=="simulation"{
					
						ren pscore_charter_simulation sim_pscore_c
						ren pscore_innov_simulation sim_pscore_i
						
						gen in_zero1_cpscore = (sim_pscore_c > 0 & sim_pscore_c < 1)
						gen in_zero1_ipscore = (sim_pscore_i > 0 & sim_pscore_i < 1)
						
						gen double pscore_charter_simulation = round(sim_pscore_c, 0.0001)
						gen double pscore_innov_simulation = round(sim_pscore_i, 0.0001)
						
						tab pscore_charter_`file', gen(c0_)
						tab pscore_innov_`file', gen(i0_)
						
						ren pscore_charter_`file' c0
						ren pscore_innov_`file' i0
					}
					else{
						
						tab pscore_charter_`file', gen(c0_)
						tab pscore_innov_`file', gen(i0_)
						
						ren pscore_charter_`file' c0
						ren pscore_innov_`file' i0
						
						gen in_zero1_cpscore = (c0 > 0 & c0 < 1)
						gen in_zero1_ipscore = (i0 > 0 & i0 < 1)
					}
					
			**********************************************************************************
			********** Flag cells with variation in instrument and characteristics************
			**********************************************************************************
					
			/* 	Flag observations that actually make it into balance (i.e. obs in cells with
				variation in instrument). These flags are used for all types 
				of F-tests, conducted below.
				
				We generate these flags here so that no re-sorting of the data (done in bys x egen:)
				is done between the balanceregs and F tests, which affects how the variable
				"var", which stores the names of the variables corresponding to each balance
				coefficient, appears in the final dataset once we svmat the balance output.
			*/
			
			ren in_study_sample in_c_sample
			ren in_innov_sample in_i_sample
			
			tab in_c_sample, m
			tab in_i_sample, m
			
			foreach type in c i{
				foreach x in 01 001 0{
					
					*Variation in instruments for each cell
						bys `type'`x': egen sd_z_`type'`x'	= 	sd(Z_`type')
					
					*Good cell
						gen in_good_cell_`type'`x' = (sd_z_`type'`x'>0 & !missing(sd_z_`type'`x') & in_zero1_`type'pscore==1) if in_`type'_sample==1
					
					gen double tag_`type'`x' = `type'`x' if in_`type'_sample==1
					replace tag_`type'`x' = -9999999 if in_good_cell_`type'`x'==0 & in_`type'_sample==1
					tab tag_`type'`x' if in_`type'_sample==1, gen(tag_`type'`x'_)
					
					*Drop var for things with no var (the first one). These observations will not be included
					*in bin-based regression and fully saturated regressios. So, do not include their corresponding
					*variables either
					
					drop tag_`type'`x'_1
					
					*Obs per cell
					
					bys `type'`x': egen obs_cell_`type'`x' = count(maskedid) if in_`type'_sample==1
				}
			}
			
			drop sd_z_*
			
			save "`cleandata'analysis_innov_`file'.dta", replace
		}	/*Close file loop*/	
	}	/*Close pscore condition*/

	
	

	************************************************
	**************** Pscore Setup (for yr 2) ******************
	************************************************		
	
	if `pscore_setup_yr2'==1{
		
		foreach file in simulation frequency formula{
	
			use "`cleandata'mdrd_denver_innov_year_2_`file' `pscoredate'", clear
			
			destring baselinegrade, replace
			destring appgrade, gen(appgrade_tmp)
			drop appgrade
			rename appgrade_tmp appgrade
			
			*High school flag			
				gen hsgrade = ( appgrade>=9)
				gen msgrade = ( appgrade<9)
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
				
			*Analysis at the student level. Generate max for charter offer
			bys maskedid: egen any_study_offer 	= max(offer_study)
			bys maskedid: egen any_innov_offer 	= max(offer_innov)
			
			bys maskedid: egen num_study = count(cond(appstudy==1,appschoolnum,.))
			bys maskedid: egen num_innov = count(cond(appinnov==1,appschoolnum,.))
			
			bys maskedid: egen num_schools = count(appschoolnum)
			
			*Enrolls at charter flag (student level)
			bys maskedid: egen enroll_any_study 	= max(study)
			bys maskedid: egen enroll_any_innov		= max(innov)
			
			*Rank 1 at school level
			gen rank1_study= (appstudy==1 & apprank==1)
			gen rank1_innov= (appinnov==1 & apprank==1)
		
			************************** School oversubscription table **************************
			*Schools ranked, assigned, 0 < prob assignment < 1 
			preserve
				keep if appinnov==1
				bys appschoolnum: egen tot_rank1 = sum(rank1_innov)
				bys appschoolnum: egen tot_rank = count(maskedid)
				bys appschoolnum: egen tot_offer = sum(offer)
				bys appschoolnum: egen tot_innov_not01 = count(cond(pscore_innov_`file'>0 & pscore_innov_`file'<1,maskedid,.))
				bys appschoolnum: egen tot_school_not01 = count(cond(pscore_school>0 & pscore_school<1,maskedid,.))
				bys appschoolnum: egen tot_school_is1 = count(cond(pscore_school==1,maskedid,.))
				bys appschoolnum: egen tot_school_not01_firstchoice = count(cond(pscore_school > 0 & pscore_school < 1 & apprank == 1,maskedid,.))
						
				duplicates drop appschoolnum, force
				keep appschoolnum appschoolname tot_* MS HS
				
				tempfile temp
				sa `temp'
			restore
			
			
			*Subject to randomization at the FULLY SATURATED PSCORE level (variation in instrument for every value of PSCORE) preserve
				
			preserve
				keep if baselinespyear==2013
				keep if appinnov==1
				
				*Simulation has huge support. Use rounded values to 0.001
				if "`file'"=="simulation"{
					ren pscore_innov_simulation sim_pscore_i
					gen double pscore_innov_simulation = round(sim_pscore_i,0.001)
				}
				
				*Any charter offer pscore - by school, the number of kids in pscore sets with variation in any offer
					bys pscore_innov_`file': egen min_offer = min(any_innov_offer)
					bys pscore_innov_`file': egen max_offer = max(any_innov_offer)
					
					gen sum = min_offer + max_offer
					bys appschoolnum: egen tot_innov_rand = count(cond(sum==1 ,maskedid,.))
					bys appschoolnum: egen tot_innov_rand_not01 = count(cond(sum==1 & pscore_innov_`file'>0 & pscore_innov_`file'<1,maskedid,.))
						
				*School offer - if within the same school, there are kids with the same pscore and var in offer
					drop min_offer max_offer sum
					bys appschoolnum pscore_school: egen min_offer = min(offer)
					bys appschoolnum pscore_school: egen max_offer = max(offer)
					
					gen sum = min_offer + max_offer
					bys appschoolnum: egen tot_school_rand = count(cond(sum==1,maskedid,.))
					bys appschoolnum: egen tot_school_rand_not01 = count(cond(sum==1 & pscore_school>0 & pscore_school<1,maskedid,.))
					drop min_offer max_offer
					
					*Number of kids with family link
					bys appschoolnum: egen tot_family = sum(family)
					
					duplicates drop appschoolnum, force
					keep appschoolnum  tot_*
					
					merge 1:1 appschoolnum using `temp'
					drop _merge
					
					order appschoolnum appschoolname MS HS ///
						tot_rank1 tot_rank	tot_offer ///
						tot_innov_not01	tot_school_not01 ///
						tot_innov_rand	tot_innov_rand_not01	///
						tot_school_rand	tot_school_rand_not01
				
					outsheet using "`tables'inDPSbase_`file'_innov_year_2_schools.csv", comma replace

			restore

			******************************************************************************
			
			*First ranked school is charter (student level, for cov balance)
			foreach type in study innov{
				bys maskedid: egen temp`type' = max(rank1_`type')
				drop rank1_`type'
				ren temp`type' rank1_`type'
				
				*Flag - apply to any study
				bys maskedid: egen max`type' = max(app`type')
				drop app`type'
				ren max`type' app`type'
			}
			
			*Enrolls next year (i.e. has nonmissing schoolnum)
			gen next = (schoolnum!=.)
			bys maskedid: egen enrolls_next = max(next)
			
			*In_sample was previously defined at the student-school level. Redefine it so
			*that it is at the student level
			
			foreach type in study innov{
				bys maskedid: egen in_`type'_sample_max = max(in_`type'_sample)
				tab in_`type'_sample_max in_`type'_sample
				
				drop in_`type'_sample
				renvars in_`type'_sample_max, postdrop(4)
			}
			
			*** Dataset at student level from here on ****
			duplicates drop maskedid, force
			drop next
			
			*Redefine has scores so that it excludes sciences - very few kids
			replace has_sc = (has_sc_math | has_sc_reading | has_sc_writing)
			
			*School-level obs
			drop appschoolnum appschoolname offer offer_study offer_innov appcharter apprank study innov charter

			*Keep only if in sample
			keep if in_study_sample==1 | in_innov_sample==1
			
			*Define r001 as meaning pscore rounded to 0.001 and r01 as rounded to 0.01
			gen c001 = round(pscore_charter, 0.001)
			gen c01 = round(pscore_charter, 0.01)
			
			gen i001 = round(pscore_innov, 0.001)
			gen i01 = round(pscore_innov, 0.01)
			
			tab c001, gen(c001_)
			tab c01, gen(c01_)
			
			tab i01, gen(i01_)
			tab i001, gen(i001_)
		
			*Polynomials
			forvalues i=1/4{
				gen double pscore_c`i'=pscore_charter^(`i')
				gen double pscore_i`i'=pscore_innov^(`i')
			}
			
			keep if baselinespyear==2013
			
			*Treatment is any study, instrument is study offer, riskset is pscore
			ren any_study_offer 	Z_c
			ren any_innov_offer 	Z_i
			
			ren enroll_any_study  	D_c
			ren enroll_any_innov  	D_i
			
			**********************************************************************************
			******************** Flag kids who are ineligble and guaranteed ******************
			**********************************************************************************
			*Flag kids who are ineligble and guaranteed - do not drop them, we still want them 
			*for OLS regressions and naive 2SLS (i.e. not controlling for risksets)
			
				*If simulation, first define in_zero1_pscore based on true score, then
			
					if "`file'"=="simulation"{
					
						ren pscore_charter_simulation sim_pscore_c
						ren pscore_innov_simulation sim_pscore_i
						
						gen in_zero1_cpscore = (sim_pscore_c > 0 & sim_pscore_c < 1)
						gen in_zero1_ipscore = (sim_pscore_i > 0 & sim_pscore_i < 1)
						
						gen double pscore_charter_simulation = round(sim_pscore_c, 0.0001)
						gen double pscore_innov_simulation = round(sim_pscore_i, 0.0001)
						
						tab pscore_charter_`file', gen(c0_)
						tab pscore_innov_`file', gen(i0_)
						
						ren pscore_charter_`file' c0
						ren pscore_innov_`file' i0
					}
					else{
						
						tab pscore_charter_`file', gen(c0_)
						tab pscore_innov_`file', gen(i0_)
						
						ren pscore_charter_`file' c0
						ren pscore_innov_`file' i0
						
						gen in_zero1_cpscore = (c0 > 0 & c0 < 1)
						gen in_zero1_ipscore = (i0 > 0 & i0 < 1)
					}
					
			**********************************************************************************
			********** Flag cells with variation in instrument and characteristics************
			**********************************************************************************
					
			/* 	Flag observations that actually make it into balance (i.e. obs in cells with
				variation in instrument). These flags are used for all types 
				of F-tests, conducted below.
				
				We generate these flags here so that no re-sorting of the data (done in bys x egen:)
				is done between the balanceregs and F tests, which affects how the variable
				"var", which stores the names of the variables corresponding to each balance
				coefficient, appears in the final dataset once we svmat the balance output.
			*/
			
			ren in_study_sample in_c_sample
			ren in_innov_sample in_i_sample
			
			tab in_c_sample, m
			tab in_i_sample, m
			
			foreach type in c i{
				foreach x in 01 001 0{
					
					*Variation in instruments for each cell
						bys `type'`x': egen sd_z_`type'`x'	= 	sd(Z_`type')
					
					*Good cell
						gen in_good_cell_`type'`x' = (sd_z_`type'`x'>0 & !missing(sd_z_`type'`x') & in_zero1_`type'pscore==1) if in_`type'_sample==1
					
					gen double tag_`type'`x' = `type'`x' if in_`type'_sample==1
					replace tag_`type'`x' = -9999999 if in_good_cell_`type'`x'==0 & in_`type'_sample==1
					tab tag_`type'`x' if in_`type'_sample==1, gen(tag_`type'`x'_)
					
					*Drop var for things with no var (the first one). These observations will not be included
					*in bin-based regression and fully saturated regressios. So, do not include their corresponding
					*variables either
					
					drop tag_`type'`x'_1
					
					*Obs per cell
					
					bys `type'`x': egen obs_cell_`type'`x' = count(maskedid) if in_`type'_sample==1
				}
			}
			
			drop sd_z_*
			
			save "`cleandata'analysis_innov_year_2_`file'.dta", replace
		}	/*Close file loop*/	
	}	/*Close pscore condition*/

	
	
	
						
	********************************************************************
	********** Revising risksets and in_good_cell variables ************
	********************************************************************
	if `pscore_stacking' == 1{
		
		foreach file in simulation frequency formula {
		
			u "`cleandata'analysis_innov_`file'.dta", clear
			
			append using "`cleandata'analysis_innov_year_2_`file'.dta"
			
			drop c001_* c01_* c0_* i001_* i01_* i0_* in_good_cell_* tag_* obs_cell_*

	
			tab c001, gen(c001_)
			tab c01, gen(c01_)
			
			tab i001, gen(i001_)
			tab i01, gen(i01_)
			
			tab c0, gen(c0_)
			tab i0, gen(i0_)
			
			
			*gen indicator for factor variable, much faster
			egen ind_i0 = group(i0)
			egen ind_i01 = group(i01)
			egen ind_i001 = group(i001)
			
				foreach type in c i{
				foreach x in 01 001 0{
					
					*Variation in instruments for each cell
						bys `type'`x': egen sd_z_`type'`x'	= 	sd(Z_`type')
					
					*Good cell
						gen in_good_cell_`type'`x' = (sd_z_`type'`x'>0 & !missing(sd_z_`type'`x') & in_zero1_`type'pscore==1) if in_`type'_sample==1
					
					gen double tag_`type'`x' = `type'`x' if in_`type'_sample==1
					replace tag_`type'`x' = -9999999 if in_good_cell_`type'`x'==0 & in_`type'_sample==1
					tab tag_`type'`x' if in_`type'_sample==1, gen(tag_`type'`x'_)
					
					*Drop var for things with no var (the first one). These observations will not be included
					*in bin-based regression and fully saturated regressios. So, do not include their corresponding
					*variables either
					
					drop tag_`type'`x'_1
					
					*Obs per cell
					
					bys `type'`x': egen obs_cell_`type'`x' = count(maskedid) if in_`type'_sample==1
					}
				}
			
			drop sd_z_*
			
			*dropping duplicates from year 2. 
			duplicates tag masked, gen(dup_masked)
			drop if dup_masked > 0 & baselinespyear == 2013			
		
			save "`cleandata'analysis_innov_stacked_`file'.dta", replace
		}
	}
	
	
	*****************************************
	******** Balance and attrition **********
	*****************************************
	
	if `pscore_balance'==1{
	
		foreach file in  frequency simulation formula{
			
				u "`cleandata'analysis_innov_stacked_`file'.dta", clear
				
				**********************************************************************************
				************************ Balance and attrition coefficients **********************
				**********************************************************************************
		
				gen var =""
				
				*Count points in the support
				replace var = "Riskset points of support" if _n ==1
			
				
				foreach type in c i{
						
						if "`type'"=="c"{
							local colbump = 0
						}
						else if "`type'"=="i"{
							local colbump = 7
						}
						
						local row = 2
						local col = 1 + `colbump'
						
						*Place it on top of balance table
						unique pscore_`type'1 		if in_zero1_`type'pscore==1
						matrix balance[1,3 + `colbump'] = r(sum)
						
						unique `type'01				if in_good_cell_`type'01==1
						matrix balance[1,5 + `colbump'] = r(sum)
						
						unique `type'001				if in_good_cell_`type'001==1
						matrix balance[1,6 + `colbump'] = r(sum)

						unique `type'0				if in_good_cell_`type'0==1
						matrix balance[1,7 + `colbump'] = r(sum)

					
					foreach var in `appcov_`type'' `cov_`type'' `cov2' `attrition_vars'{
						replace var="`var'" if _n==`row'
						replace var="`var'_se" if _n==`row'+1
						replace var="`var'_N" if _n==`row'+2
						replace var="`var'_N(var in Z)" if _n==`row'+3
							
						*All non-offered applicants
						qui summ `var' if Z_`type'==0 & in_`type'_sample==1
							matrix balance[`row',`col']   = r(mean)
							matrix balance[`row'+1,`col'] = r(sd)
							matrix balance[`row'+2,`col' ] = r(N)
					
						*Balance and attrition, no control(OLS)
						qui reg `var' Z_`type' if in_`type'_sample==1, vce(cluster maskedid)
							matrix balance[`row',`col' + 1]   = _b[Z]
							matrix balance[`row'+1,`col'  +1] = _se[Z]
							matrix balance[`row'+2,`col'  +1] = e(N)
						
						*Balance and attrition, pscore linear control(OLS)
						qui reg `var' Z_`type' pscore_`type'1 if in_zero1_`type'pscore==1, vce(cluster maskedid)
							matrix balance[`row',`col'  +2]   = _b[Z]
							matrix balance[`row'+1,`col'  +2] = _se[Z]
							matrix balance[`row'+2,`col'  +2] = e(N)
							
						*Balance and attrition, pscore polynomial control(OLS)
						qui reg `var' Z_`type' pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 if in_zero1_`type'pscore==1, vce(cluster maskedid)
							matrix balance[`row',`col'  +3]   = _b[Z]
							matrix balance[`row'+1,`col'  +3] = _se[Z]
							matrix balance[`row'+2,`col' + 3] = e(N)
						
						*Balance and attrition, risks
						local c = 0
						foreach i in 01 001 0{
							local c = `c' +1
							
							preserve
								qui areg `var' Z_`type' i.baselinespyear if in_zero1_`type'pscore==1, absorb(`type'`i') vce(cluster maskedid)
							
								matrix balance[`row',`col' + 3 + `c']   = _b[Z]
								matrix balance[`row'+1,`col' + 3 + `c'] = _se[Z]
								matrix balance[`row'+2,`col' + 3 + `c'] = e(N)
								
								bys `type'`i': egen sd = sd(`var')
								bys `type'`i': egen sd_z = sd(Z_`type')
								
								*Count kids with variation in instrument within cell (no ref to characteristic)
								count if sd_z > 0 & !missing(sd_z)  & e(sample)
								matrix balance[`row'+3,`col' + 3 + `c'] = r(N)
								
								drop sd sd_z		
							restore
						}
						
					local row = `row' + 6
					} /*Close covariates loop*/
				
			} /*Close c, i loop*/
			
			************************************* F tests **************************************	
				*MS 2/23/2015: 
				*Minimum number of observations neeed for the SUEST F test to run according to each pscore type
				*Arrived at these numbers by running F tests for several different minimum number of obs. These
				*are the minimum needed for the set of covariates being tested as of 2/23/2015.
				
				local min_obs_cell_simulation 	= 9			
				local min_obs_cell_formula 		= 9
				local min_obs_cell_frequency 	= 9
				
			
			if `ftest'==1{
				
				**********************************************************************************
				************************************* Logit Fs ***********************************
				**********************************************************************************
				
				foreach f in robust traditional{
					
						foreach covs in apponly balanceonly all{
						
							foreach type in c i{
						
								if "`type'"=="c"{
									local colbump = 0
								}
								else if "`type'"=="i"{
									local colbump = 7
								}
								
								local col = 1 + `colbump'
						
								if "`covs'"=="apponly"{
									local balF "`appcov_`type''"
									local Fname "Application Balance"
								}
								
								else if "`covs'"=="balanceonly"{
									local balF "`cov_`type'' `cov2'"
									local Fname "Baseline Balance"
								}
								else if "`covs'"=="all"{
									local balF "`appcov_`type'' `cov_`type'' `cov2'"
									local Fname "All Balance"
								}
								
								
								if "`f'"=="robust"{
									local option "nocnsreport vce(cluster maskedid)"
								}
								else{
									local option "nocnsreport"
								}
							
							
								* Balance logit
								replace var="Logit `f' `Fname' F" if _n==`row'
								replace var="Logit `f' `Fname' F (pval)" if _n==`row'+1
								replace var="Logit `f' `Fname' F (df)" if _n==`row'+2
								replace var="Logit `f' `Fname' F (N)" if _n==`row'+3
							
								*No riskset
								qui logit Z_`type' `balF', `option'
								test `balF'
								matrix balance[`row', `col' +1] = r(chi2)
								matrix balance[`row'+1, `col' +1] = r(p)
								matrix balance[`row'+2, `col' +1] = r(df)
								matrix balance[`row'+3, `col' +1] = e(N)	
			
								*Linear riskset
								qui logit Z_`type' pscore_`type'1 `balF' if in_zero1_`type'pscore==1, `option'
								test `balF'
								matrix balance[`row', `col' +2] = r(chi2)
								matrix balance[`row'+1, `col' +2] = r(p)
								matrix balance[`row'+2, `col' +2] = r(df)
								matrix balance[`row'+3, `col' +2] = e(N)
								
								*Poly riskset
								qui logit Z_`type' pscore_`type'* `balF' if in_zero1_`type'pscore==1, `option'
								test `balF'
								matrix balance[`row', `col' +3] = r(chi2)
								matrix balance[`row'+1, `col' +3] = r(p)
								matrix balance[`row'+2, `col' +3] = r(df)
								matrix balance[`row'+3, `col' +3] = e(N)
								
								*Bins
								local c=0
								foreach i in 01 001 0{
									local c = `c' + 1
									qui logit Z_`type' i.ind_i`i' `balF' if in_good_cell_`type'`i'==1, `option'
									test `balF'
									matrix balance[`row', `col'+3 + `c'] = r(chi2)
									matrix balance[`row' +1, `col'  +3 + `c'] = r(p)
									matrix balance[`row'+2, `col'  +3 + `c'] = r(df)
									matrix balance[`row'+3, `col'  +3 + `c'] = e(N)
								}

						} /*CLose c i loop*/			
					local row = `row' + 4
					}	/*Close balance covariates loop*/
			}	/*CLose robust/traditional F*/
				
				********************************************************
				*************** Balance Traditional F ******************
				********************************************************
						
				foreach covs in apponly balanceonly all{
				
					foreach type in c i{
						
						if "`type'"=="c"{
							local colbump = 0
						}
						else if "`type'"=="i"{
							local colbump = 7
						}
						
						local col = 1 + `colbump'
											
							if "`covs'"=="apponly"{
								local balF "`appcov_`type''"
								local Fname "Application Balance"
							}
							
							else if "`covs'"=="balanceonly"{
								local balF "`cov_`type'' `cov2'"
								local Fname "Baseline Balance"
							}
							else if "`covs'"=="all"{
								local balF "`appcov_`type'' `cov_`type'' `cov2'"
								local Fname "All Balance"
							}
									
							replace var="Traditional `Fname' F-test" if _n==`row'
							replace var="Traditional `Fname' F-test (pval)" if _n==`row' +1
							replace var="Traditional `Fname' F-test (residuals df)" if _n==`row' +2
							replace var="Traditional `Fname' F-test (model df)" if _n==`row' +3
							replace var="Traditional `Fname' F-test (N)" if _n==`row' +4
							replace var="Traditional `Fname' F-test (Breusch-Pagan chi2)" if _n==`row'+5
					
						
						*no propensity score
							mvreg `balF' = Z_`type', corr
							test Z_`type'
							matrix balance[`row',`col'  + 1] = r(F)
							matrix balance[`row'+1,`col' + 1]= r(p)
							matrix balance[`row'+2, `col' +1] = r(df_r)
							matrix balance[`row'+3, `col' +1] = r(df)
							matrix balance[`row'+4, `col' +1] = e(N)
							matrix balance[`row'+5, `col' +1] = 1- chi2(e(df_chi2),e(chi2))
						
						*linear pscore control
							mvreg `balF' = Z_`type' pscore_`type'1 if in_zero1_`type'pscore==1, corr
							test Z_`type'
							matrix balance[`row',`col'  +2]=r(F)
							matrix balance[`row'+1,`col'  +2]=r(p)
							matrix balance[`row'+2, `col' +2] = r(df_r)
							matrix balance[`row'+3, `col' +2] = r(df)
							matrix balance[`row'+4, `col' +2] = e(N)
							matrix balance[`row'+5, `col' +2] = 1- chi2(e(df_chi2),e(chi2))
							
						*polynomial pscore control
							mvreg `balF' = Z_`type' pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 if in_zero1_`type'pscore==1, corr
							test Z_`type'
							matrix balance[`row',`col'  + 3]=r(F)
							matrix balance[`row'+1,`col' + 3]=r(p)
							matrix balance[`row'+2, `col' +3] = r(df_r)
							matrix balance[`row'+3, `col' +3] = r(df)
							matrix balance[`row'+4, `col' +3] = e(N)
							matrix balance[`row'+5, `col' +3] = 1- chi2(e(df_chi2),e(chi2))
							
						*Bins
							local c=0
							foreach i in 01 001 0{
								local c = `c' + 1
								mvreg `balF' = Z_`type' i.ind_i`i' i.baselinespyear if in_good_cell_`type'`i'==1, corr
								test Z_`type'
								matrix balance[`row',`col'  + 3 + `c'] = r(F)
								matrix balance[`row'+1,`col'  + 3 + `c'] = r(p)
								matrix balance[`row'+2,`col'  + 3 + `c'] = r(df_r)
								matrix balance[`row'+3,`col'  + 3 + `c'] = r(df)
								matrix balance[`row'+4,`col'  + 3 + `c'] = e(N)
								matrix balance[`row'+5,`col'  + 3 + `c'] = 1- chi2(e(df_chi2),e(chi2))
								
							}

										
						}	/*Close c i loop*/
				local row = `row' + 6
				}	/*Close covariates loop*/
			
				
				**********************************************************************************
				*************************** Balance Stacked Robust F *****************************
				**********************************************************************************
				
				*Run a cut keeping cells with small size and one dropping cells with small size
				
				local col = 1	
					
				foreach f_cut in keep_small_cells drop_small_cells{
					
						foreach covs in apponly balanceonly all{
						
							foreach type in c i{
				
								if "`type'"=="c"{
									local colbump = 0
								}
								else if "`type'"=="i"{
									local colbump = 7
								}
								
								local col = 1 + `colbump'
							
								if "`covs'"=="apponly"{
									local balF "`appcov_`type''"
									local Fname "Application Balance"
									local expand = 3
									local test "Z_1 Z_2 Z_3"
								}
								
								else if "`covs'"=="balanceonly"{
									local balF "`cov_`type'' `cov2'"
									local Fname "Baseline Balance"
									local expand = 12
									local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12"
								}
								else if "`covs'"=="all"{
									local balF "`appcov_`type'' `cov_`type'' `cov2'"
									local Fname "All Balance"
									local expand = 15
									local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12 Z_13 Z_14 Z_15"
								}
								
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test" if _n==`row'
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (pval)" if _n==`row'+1
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (residuals df)" if _n==`row'+2
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (model df)" if _n==`row'+3
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (N)" if _n==`row'+4
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (cells)" if _n==`row'+5
								
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test" if _n==`row'+6
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (pval)" if _n==`row'+7
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (residuals df)" if _n==`row'+8
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (model df)" if _n==`row'+9
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (N)" if _n==`row'+10
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (cells)" if _n==`row'+11
								
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test" if _n==`row' +12
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (pval)" if _n==`row'+13
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (residuals df)" if _n==`row'+14
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (model df)" if _n==`row'+15
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (N)" if _n==`row'+16
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (cells)" if _n==`row'+17
						
						
								preserve
									
									if "`f_cut'" === "keep_small_cells"{
										local min_obs = 1
									}
									else if "`f_cut'"=="drop_small_cells"{
										local min_obs = `min_obs_cell_`file''
									}
									
									keep 	maskedid `balF' Z_`type' tag_`type'01 tag_`type'001 tag_`type'0 `type'01 `type'001 `type'0 ///
											pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 in_*_sample in_good_cell_* obs_cell_* in_zero1_`type'pscore baselinespyear
											
									expand `expand'
									bys maskedid baselinespyear: gen copy = _n
									sort copy
									
									* if kids repeat years you get a larger max value of copy, which will break 
									* the logic used for the tokenize chunk below. 
									su copy 
									if `r(max)' !=  `expand' {
										di as error " Somethign is off when you make a copy of the data. You need to end up with only `expand' copies per kid. Rn you have `r(max)'"
										exit  
										
									}
																			
									
									*Single vector of outcomes
									gen cov =.
									gen covname=""
									tokenize `balF'
									forval i=1/`expand'{
										replace cov = ``i'' if copy==`i' 
										replace covname = "``i''" if copy==`i' 
									}
									drop `balF'
									
									*Instruments, pscore variables, and dummies for each copy
									forval i=1/`expand'{
										gen Z_`i' = Z_`type'*(copy == `i')
										gen alpha_`i' = (copy == `i')
										
										*Pscore variables for each copy
										foreach x in `type'01 `type'001 `type'0 pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 baselinespyear{
											gen c_`x'_`i'= `x'*(copy == `i')
										}
									}
									
									*Absorb takes only one var. Create grouped vars
									foreach x in tag_`type'01 tag_`type'001 tag_`type'0 `type'01 `type'001 `type'0 {
											egen group_`x' = group(copy `x')
										
									}
									
									*Replace with missing if first category
									foreach x in tag_`type'01 tag_`type'001 tag_`type'0{
										replace group_`x' =. if `x'==-9999999
									
									}
															
									*** One tall regression for each specification ***
									
									*No controls				
										*Clustered
										qui reg cov alpha_* Z_* if in_`type'_sample==1, vce(cluster maskedid)
										test `test'
										
										matrix balance[`row',`col' + 1]=r(F)
										matrix balance[`row'+1,`col' +1]=r(p)
										matrix balance[`row'+2,`col' +1]=r(df_r)
										matrix balance[`row'+3,`col' +1]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+4,`col' + 1]=r(sum)
										
										*Traditional
										qui reg cov alpha_* Z_*  if in_`type'_sample==1
										test `test'
										
										matrix balance[`row'+6,`col' + 1]=r(F)
										matrix balance[`row'+7,`col' +1]=r(p)
										matrix balance[`row'+8,`col' +1]=r(df_r)
										matrix balance[`row'+9,`col' +1]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+10,`col' + 1]=r(sum)
										
										*Robust
										qui reg cov alpha_* Z_* if in_`type'_sample==1, r
										test `test'
										
										matrix balance[`row'+12,`col' + 1]=r(F)
										matrix balance[`row'+13,`col' +1]=r(p)
										matrix balance[`row'+14,`col' +1]=r(df_r)
										matrix balance[`row'+15,`col' +1]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+16,`col' + 1]=r(sum)
										

									*Linear
										*Cluster
										qui reg cov alpha_* Z_* c_pscore_`type'1_* if in_zero1_`type'pscore==1, vce(cluster maskedid)
										test `test'
										
										matrix balance[`row',`col' + 2]=r(F)
										matrix balance[`row'+1,`col' +2]=r(p)
										matrix balance[`row'+2,`col' +2]=r(df_r)
										matrix balance[`row'+3,`col' +2]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+4,`col' + 2]=r(sum)
										
										*Traditional
										qui reg cov alpha_* Z_* c_pscore_`type'1_* if in_zero1_`type'pscore==1
										test `test'
										
										matrix balance[`row'+6,`col' + 2]=r(F)
										matrix balance[`row'+7,`col' +2]=r(p)
										matrix balance[`row'+8,`col' +2]=r(df_r)
										matrix balance[`row'+9,`col' +2]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+10,`col' + 2]=r(sum)
										
										*Robust
										qui reg cov alpha_* Z_* c_pscore_`type'1_* if in_zero1_`type'pscore==1, r
										test `test'
										
										matrix balance[`row'+12,`col' + 2]=r(F)
										matrix balance[`row'+13,`col' +2]=r(p)
										matrix balance[`row'+14,`col' +2]=r(df_r)
										matrix balance[`row'+15,`col' +2]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+16,`col' + 2]=r(sum)
									
									*Polynomial
										*Cluster
										qui reg cov alpha_* Z_* c_pscore_`type'1_* c_pscore_`type'2_* c_pscore_`type'3_* c_pscore_`type'4_* if in_zero1_`type'pscore==1, vce(cluster maskedid)
										test `test'
										
										matrix balance[`row',`col' + 3]=r(F)
										matrix balance[`row'+1,`col' +3]=r(p)
										matrix balance[`row'+2,`col' +3]=r(df_r)
										matrix balance[`row'+3,`col' +3]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+4,`col' + 3]=r(sum)
										
										*Traditional
										qui reg cov alpha_* Z_* c_pscore_`type'1_* c_pscore_`type'2_* c_pscore_`type'3_* c_pscore_`type'4_* if in_zero1_`type'pscore==1
										test `test'
										
										matrix balance[`row'+6,`col' + 3]=r(F)
										matrix balance[`row'+7,`col' +3]=r(p)
										matrix balance[`row'+8,`col' +3]=r(df_r)
										matrix balance[`row'+9,`col' +3]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+10,`col' + 3]=r(sum)
										
										*Robust
										qui reg cov alpha_* Z_* c_pscore_`type'1_* c_pscore_`type'2_* c_pscore_`type'3_* c_pscore_`type'4_* if in_zero1_`type'pscore==1, r
										test `test'
										
										matrix balance[`row'+12,`col' + 3]=r(F)
										matrix balance[`row'+13,`col' +3]=r(p)
										matrix balance[`row'+14,`col' +3]=r(df_r)
										matrix balance[`row'+15,`col' +3]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+16,`col' + 3]=r(sum)
										
									*Bins
										local c = 0
										foreach i in 01 001 0{
											local c = `c' +1
											
											*Cluster
											qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_`type'`i'==1 & obs_cell_`type'`i' >= `min_obs' & !missing(obs_cell_`type'`i'), absorb(group_tag_`type'`i') vce(cluster maskedid)
											test `test'
											
											matrix balance[`row',`col' +3 + `c']   = r(F)
											matrix balance[`row'+1,`col' +3 + `c'] = r(p)
											matrix balance[`row'+2,`col'+ 3 + `c'] = r(df_r)
											matrix balance[`row'+3,`col'+3 + `c'] = r(df)
											
											unique maskedid if e(sample)
											matrix balance[`row'+4,`col'+3 + `c'] = r(sum)
											
											unique group_tag_`type'`i' if e(sample)
											matrix balance[`row'+5,`col'+3 + `c'] = r(sum)
											
											*Traditional
											qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_`type'`i'==1 & obs_cell_`type'`i' >= `min_obs' & !missing(obs_cell_`type'`i'), absorb(group_tag_`type'`i')
											test `test'
											
											matrix balance[`row'+6,`col'+3 + `c'] = r(F)
											matrix balance[`row'+7,`col'+3 + `c'] = r(p)
											matrix balance[`row'+8,`col'+3 + `c'] = r(df_r)
											matrix balance[`row'+9,`col'+3 + `c'] = r(df)
											
											unique maskedid if e(sample)
											matrix balance[`row'+10,`col'+3 + `c'] = r(sum)
											
											unique group_tag_`type'`i' if e(sample)
											matrix balance[`row'+11,`col'+3 + `c'] = r(sum)
											
											*Robust
											qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_`type'`i'==1 & obs_cell_`type'`i' >= `min_obs' & !missing(obs_cell_`type'`i'), absorb(group_tag_`type'`i') r
											test `test'
											
											matrix balance[`row'+12,`col'+3 + `c'] = r(F)
											matrix balance[`row'+13,`col'+3 + `c'] = r(p)
											matrix balance[`row'+14,`col'+3 + `c'] = r(df_r)
											matrix balance[`row'+15,`col'+3 + `c'] = r(df)
											
											unique maskedid if e(sample)
											matrix balance[`row'+16,`col'+3 + `c'] = r(sum)
											
											unique group_tag_`type'`i' if e(sample)
											matrix balance[`row'+17,`col'+3 + `c'] = r(sum)
										}
								
								restore
							} /*Close c, i loop*/
						
						local row = `row' + 18	
						} /*Close vocs loop*/		
				} /*Close f cut*/	
				
				
				**********************************************************************************
				*********************** Suest Robust standard errors *****************************
				**********************************************************************************
				
				** F-tests using suest, which allows for robus or clustered SEs **
				* 1) Run models without clustering option
				* 2) Store results
				* 3) suest [stored values], vce(cluster clustvar)
				* 4) test coefficient
				
				local col = 1
				
				foreach f_cut in keep_small_cells drop_small_cells{

						if "`f_cut'" === "keep_small_cells"{
							local min_obs = 1
						}
						else if "`f_cut'"=="drop_small_cells"{
							local min_obs = `min_obs_cell_`file''
						}
						
							replace var="[`f_cut'] Suest Application Balance F" if _n==`row'
							replace var="[`f_cut'] Suest Application Balance F (pval)" if _n==`row' +1
							replace var="[`f_cut'] Suest Application Balance F (df)" if _n==`row' +2
							replace var="[`f_cut'] Suest Application Balance F (N)" if _n==`row' +3
							replace var="[`f_cut'] Suest Application Balance F (cells)" if _n==`row' +4
							
							replace var="[`f_cut'] Suest Baseline Balance F" if _n==`row' + 5
							replace var="[`f_cut'] Suest Baseline Balance F (pval)" if _n==`row' +6
							replace var="[`f_cut'] Suest Baseline Balance F (df)" if _n==`row' +7
							replace var="[`f_cut'] Suest Baseline Balance F (N)" if _n==`row' +8
							replace var="[`f_cut'] Suest Baseline Balance F (cells)" if _n==`row' +9
							
							replace var="[`f_cut'] Suest All Balance F" if _n==`row' + 10
							replace var="[`f_cut'] Suest All Balance F (pval)" if _n==`row' +11
							replace var="[`f_cut'] Suest All Balance F (df)" if _n==`row' +12
							replace var="[`f_cut'] Suest All Balance F (N)" if _n==`row' +13
							replace var="[`f_cut'] Suest All Balance F (cells)" if _n==`row' +14
						
					foreach type in c i{
						
						if "`type'"=="c"{
							local colbump = 0
						}
						else if "`type'"=="i"{
							local colbump = 7
						}
						
						local col = 1 + `colbump'
								

						*Balance F
							foreach var in `appcov_`type'' `cov_`type'' `cov2'{
								
								* no pscores
								qui reg `var' Z_`type'
								est store `var'
								
								* linear
								qui reg `var' Z_`type' pscore_`type'1 if in_zero1_`type'pscore==1
								est store L`var'
								
								* poly
								qui reg `var' Z_`type' pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 if in_zero1_`type'pscore==1
								est store P`var'
								
								* Pscore bins
								foreach i in 01 001 0{
									qui reg `var' Z_`type' i.ind_i`i' 	if in_good_cell_`type'`i'==1 & obs_cell_`type'`i'>=`min_obs' & !missing(obs_cell_`type'`i')
									est store B`i'`var'
								}
	
							}
							
							*** Apcov suest ***
								cap suest 	num_schools num_`type' rank1_`type', vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row', `col' + 1] = r(chi2)
									matrix balance[`row'+1, `col' + 1] = r(p)
									matrix balance[`row'+2, `col' + 1] = r(df)
									matrix balance[`row'+3, `col' + 1] = e(N)
								}	
									
								cap suest 	Lnum_schools Lnum_`type'  Lrank1_`type', vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row', `col' +2] = r(chi2)
									matrix balance[`row'+1, `col' +2] = r(p)
									matrix balance[`row'+2, `col' +2] = r(df)
									matrix balance[`row'+3, `col' +2] = e(N)
								}
								
								cap suest 	Pnum_schools Pnum_`type' Prank1_`type', vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row', `col' +3] = r(chi2)
									matrix balance[`row'+1, `col' +3] = r(p)
									matrix balance[`row'+2, `col' +3] = r(df)
									matrix balance[`row'+3, `col' +3] = e(N)
								}
								
								local c=0
								foreach i in 01 001 0{
									local c = `c' + 1
									
									cap suest 	B`i'num_schools B`i'num_`type' B`i'rank1_`type', vce(cluster maskedid)
									if !_rc{
										test Z_`type'
										matrix balance[`row', `col' + 3 + `c'] = r(chi2)
										matrix balance[`row'+1, `col' + 3 + `c'] = r(p)
										matrix balance[`row'+2, `col' + 3 + `c'] = r(df)
										matrix balance[`row'+3, `col' + 3 + `c'] = e(N)
										
										unique tag_`type'`i' if e(sample)
										matrix balance[`row'+4, `col' + 3 + `c'] = r(sum)
									}
								}
								
						** Baseline covs suest ***
							
							cap suest	baseline`type' baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
										baseliness_math baseliness_reading baseliness_writing, vce(cluster maskedid)
							if !_rc{			
								test Z_`type'
								matrix balance[`row'+5, `col'+1] = r(chi2)
								matrix balance[`row'+6, `col'+1] = r(p)
								matrix balance[`row'+7, `col'+1] = r(df)
								matrix balance[`row'+8, `col'+1] = e(N)
							}
								
							cap suest	Lbaseline`type' Lbaselinefemale Lbaselinehisp Lbaselineblack Lbaselinefoodfred Lbaselinesped Lbaselinebilingual Lbaselinelep Lbaselinegifted ///
									Lbaseliness_math Lbaseliness_reading Lbaseliness_writing, vce(cluster maskedid)
							if !_rc{
								test Z_`type'
								matrix balance[`row'+5, `col'+2] = r(chi2)
								matrix balance[`row'+6, `col' +2] = r(p)
								matrix balance[`row'+7, `col'+2] = r(df)
								matrix balance[`row'+8, `col'+2] = e(N)
							}
							
							cap suest	Pbaseline`type' Pbaselinefemale Pbaselinehisp Pbaselineblack Pbaselinefoodfred Pbaselinesped Pbaselinelep Pbaselinebilingual Pbaselinegifted ///
									Pbaseliness_math Pbaseliness_reading Pbaseliness_writing, vce(cluster maskedid)
							if !_rc{
								test Z_`type'
								matrix balance[`row'+5, `col'+3] = r(chi2)
								matrix balance[`row'+6, `col' +3] = r(p)
								matrix balance[`row'+7, `col'+3] = r(df)
								matrix balance[`row'+8, `col'+3] = e(N)
							}
							
							local c=0
							foreach i in 01 001 0{
								local c = `c' + 1
								cap suest	B`i'baseline`type' B`i'baselinefemale B`i'baselinehisp B`i'baselineblack B`i'baselinefoodfred B`i'baselinesped B`i'baselinelep B`i'baselinebilingual B`i'baselinegifted ///
										B`i'baseliness_math B`i'baseliness_reading B`i'baseliness_writing, vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row'+5, `col' +3 + `c'] = r(chi2)
									matrix balance[`row'+6, `col' +3 + `c'] = r(p)
									matrix balance[`row'+7, `col' +3 + `c'] = r(df)
									matrix balance[`row'+8, `col' +3 + `c'] = e(N)
									
									unique tag_`type'`i' if e(sample)
									matrix balance[`row'+9, `col' +3 + `c'] = r(sum)
								}
							}
							
							
						*** All covariates suest ***

							cap suest	num_schools num_`type' rank1_`type' ///
									baseline`type' baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
									baseliness_math baseliness_reading baseliness_writing, vce(cluster maskedid)
							
							if !_rc{
								test Z_`type'
								matrix balance[`row'+10, `col'+1] = r(chi2)
								matrix balance[`row'+11, `col'+1] = r(p)
								matrix balance[`row'+12, `col'+1] = r(df)
								matrix balance[`row'+13, `col'+1] = e(N)
							}
							
								
							cap suest	Lnum_schools Lnum_`type' Lrank1_`type' ///
									Lbaseline`type' Lbaselinefemale Lbaselinehisp Lbaselineblack Lbaselinefoodfred Lbaselinesped Lbaselinebilingual Lbaselinelep Lbaselinegifted ///
									Lbaseliness_math Lbaseliness_reading Lbaseliness_writing, vce(cluster maskedid)
							
							if !_rc{
								test Z_`type'
								matrix balance[`row'+10, `col'+2] = r(chi2)
								matrix balance[`row'+11, `col' +2] = r(p)
								matrix balance[`row'+12, `col'+2] = r(df)
								matrix balance[`row'+13, `col'+2] = e(N)
							}
							
							cap suest	Pnum_schools Pnum_`type'  Prank1_`type' ///
									Pbaseline`type' Pbaselinefemale Pbaselinehisp Pbaselineblack Pbaselinefoodfred Pbaselinesped Pbaselinelep Pbaselinebilingual Pbaselinegifted ///
									Pbaseliness_math Pbaseliness_reading Pbaseliness_writing, vce(cluster maskedid)
							
							if !_rc{
								test Z_`type'
								matrix balance[`row'+10, `col'+3] = r(chi2)
								matrix balance[`row'+11, `col' +3] = r(p)
								matrix balance[`row'+12, `col'+3] = r(df)
								matrix balance[`row'+13, `col'+3] = e(N)
							}
							
							local c=0
							foreach i in 01 001 0{
								local c = `c' + 1
								cap suest	B`i'num_schools B`i'num_`type'  B`i'rank1_`type' ///
										B`i'baseline`type' B`i'baselinefemale B`i'baselinehisp B`i'baselineblack B`i'baselinefoodfred B`i'baselinesped B`i'baselinelep B`i'baselinebilingual B`i'baselinegifted ///
										B`i'baseliness_math B`i'baseliness_reading B`i'baseliness_writing, vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row'+10, `col' +3 + `c'] = r(chi2)
									matrix balance[`row'+11, `col' +3 + `c'] = r(p)
									matrix balance[`row'+12, `col' +3 + `c'] = r(df)
									matrix balance[`row'+13, `col' +3 + `c'] = e(N)
									
									unique tag_`type'`i' if e(sample)
									matrix balance[`row'+14, `col' +3 + `c'] = r(sum)
								}
							}
						
					} /*Close c i loop*/
					
				local row = `row'+ 15
				} /*Close f_cut loop*/
				
					**********************************************************************************
					
				*Outsheet
				preserve
					svmat balance
					keep var balance*
					keep if _n<=`row'+1
					ren balance1 z0_c
					ren balance2 ols_z1_c
					ren balance3 ols_z1_pscore_c
					ren balance4 ols_z1_poly_c
					ren balance5 ols_z1_r01_c
					ren balance6 ols_z1_r001_c
					ren balance7 ols_z1_psat_c
					
					ren balance8 	z0_i
					ren balance9 	ols_z1_i
					ren balance10 	ols_z1_pscore_i
					ren balance11 	ols_z1_poly_i
					ren balance12 	ols_z1_r01_i
					ren balance13 	ols_z1_r001_i
					ren balance14 	ols_z1_psat_i
					
					outsheet using "`tables'inDPSbase_`file'_innov_stacked_balance.csv", comma replace
				restore
				
				} /*Close F test condition*/
		}	/*Close file loop*/
	} /*Close balance condition*/
	
