/* 	Ye Ji Kee
	Denver tables (years 1 and 2 stacked). These comments last edited: 1/14/2016
		For grades 4-10
		Output: Balance, 2SLS, and subgroup for pscore risksets and qualification instruments
		
		NOTE: This do-file uses datasets created in 5_pscore_balance_2SLS_tables.do and 5_pscore_balance_2SLS_year_2_tables.do
		
	Ignacio. We are going to drop duplicates from year 2 (duplicates are kids that show up in both year 1 and 2). Only keeping first time application. 
	
	STEPS TO RUN
	You need to run the previous 2_create_pscore_analysis_files.do for year 1 and year 2. The year 2 version is in the year_2_only folder. 
	The file 2_ creates the pscore file, which then is further cleaned and dropped to the student level in files 5_pscore_balance_2SLS_tables. 
	Again you need to do that for year 1 and year 2 separately. Also, the part that actually sets up the files you need is the first part of that code. 
	The files have a switcher pscore_setup you can use. 
	with those two files in place, you can run this dofile. 
	
*/




clear all
cap log close
set more off
set trace off
set tracedepth 1 
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"
pause on

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"
local rawdata "Y:\DPS\Raw Data\"

matrix demo			  = J(30,30,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,40,.)
matrix subtwosls	  = J(500,40,.)
matrix twosls_nocov    		   = J(500,40,.)
matrix twosls_scorecov 		   = J(500,40,.)
matrix twosls_fcsample   		= J(50	0,40,.)

matrix qualbalance    = J(500,20,.)
matrix qualtwosls	  = J(500,20,.)
matrix qualsubtwosls  = J(500,20,.)

local pscore_stacking 	= 0
local schools 			= 1
local student_demo 		= 0
local pscore_balance 	= 0
local pscore_twosls 	= 0
local pscore_twosls_fcsample = 0
local nocov_twosls 		= 0
local pscore_subtwosls 	= 0

local qual_stacking = 0
local qual_balance=0
local qual_twosls = 0
local qual_subtwosls =0

*Dates of analysis files
local pscoredate_year2	20170310 	/*20160516 20160510 20160428 20160411 20160208 20160112 20151120 20150514 */
local pscoredate_year1  20170310 /*20160516    	20160516 20160510 20160421  20160208 */
local simsdate 			20150603 /* 20150428 */
local qualdate_year1 	20160715 /*20150708  20150420 */
local qualdate_year2	20160715 /*20160303*/
local qualdate			20160303
local qualpscoredate	20160208
local appdatayr1		20170310
local appdatayr2		20170310


local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_*"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1        		/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
local outsheet_rescorr=0	/* If want to outsheet matrix with residual correlations */


	if `pscore_stacking' == 1{
		
		foreach file in simulation frequency formula      {
		foreach sch_type in study /*noex nonnoex*/ {
		
			u "`cleandata'analysis_`sch_type'_year_2_`file'.dta", clear
					br if masked == 31170
							
			append using "`cleandata'analysis_`sch_type'_`file'.dta"
			
			*dropping duplicates from year 2. 
			duplicates tag masked, gen(dup_masked)
			drop if dup_masked > 0 & baselinespyear == 2013			
			
			drop in_good_cell_* tag_* obs_cell_* r01_* r001_* r0_*
			
			tab r01, gen(r01_)
			tab r001, gen(r001_)
			tab r0, gen(r0_)
			
			* creating indicator variable from continuous pscore variables. Need this to use factor variables
			drop ind_*
			egen ind_r0 = group(r0)
			egen ind_r01 = group(r01)
			egen ind_r001 = group(r001)
			
			foreach x in r01 r001 r0{
				
				*Variation in instruments for each cell
					bys `x': egen sd_z_`x'	= 	sd(Z)
				
				*Good cell you have rounded pscore not 0 1
					gen in_good_cell_`x' = (sd_z_`x'>0 & !missing(sd_z_`x') & `x' >0 & `x' < 1)
					gen in_good_cell_org_`x' = (sd_z_`x'>0 & !missing(sd_z_`x') & in_zero1_pscore == 1)
					gen double tag_`x' = `x'
					replace tag_`x' = -9999999 if in_good_cell_`x'==0
					tab tag_`x', gen(tag_`x'_)
				
				*Drop var for things with no var (the first one). These observations will not be included
				*in bin-based regression and fully saturated regressios. So, do not include their corresponding
				*variables either
				
				drop tag_`x'_1
				
				*Obs per cell
				
				bys `x': egen obs_cell_`x' = count(maskedid)
			}
			
			drop sd_z_*
			

			
			save "`cleandata'analysis_stacked_`sch_type'_`file'.dta", replace	
			
			}
		}
	}
	
	************************************************
	**************** Schools Table *****************
	************************************************
	
	if `schools' == 1 {
	
		* preparing school capacities 
		use "`cleandata'application_data_year_one_`appdatayr1'.dta", clear
		
		keep capacity requestedschool bucket_id 
		duplicates drop  
		collapse (sum) capacity, by(requested)
		ren requested appschoolnum		
		tempfile capacityyr1
		sa `capacityyr1'
		
		use "`cleandata'application_data_year_two_`appdatayr2'.dta", clear
		
		keep capacity requestedschool bucket_id 
		duplicates drop  
		collapse (sum) capacity, by(requested)
		ren requested appschoolnum
		tempfile capacityyr2
		sa `capacityyr2'

		foreach file in simulation frequency formula{
		
			foreach sch_type in study /*noex nonnoex */ {
		
				use "`cleandata'mdrd_denver_`sch_type'_`file' `pscoredate_year1'", clear

				merge m:1 appschoolnum using `capacityyr1'
				ren capacity capacity_year_1
				drop _merge
				
				append using  "`cleandata'mdrd_denver_`sch_type'_year_2_`file' `pscoredate_year2'", gen(appended)
				merge m:1 appschoolnum using `capacityyr2'
				ren capacity capacity_year_2	

				*Simulation has huge support. Use rounded values for full saturation
				if "`file'"=="simulation"{
					ren pscore_`sch_type'_simulation sim_pscore
					* USing rounded to 1000 
					gen double pscore_`sch_type'_simulation = round(sim_pscore,0.001)
				}
				

				destring baselinegrade, replace
				destring appgrade, gen(appgrade_tmp)
				drop appgrade
				gen appgrade = appgrade_tmp
				drop appgrade_tmp
				
				*High school flag			
				gen hsgrade = ( appgrade >= 9 )
				gen msgrade = ( appgrade < 9 )
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
					
				*Analysis at the student level. Generate max for charter offer
				bys maskedid appended: egen any_`sch_type'_offer 	= max(offer_`sch_type')
				bys maskedid appended: egen num_schools = count(appschoolnum)
				bys maskedid appended: egen num_study = count(cond(appstudy==1,appschoolnum,.))
				gen pct_study = num_study / num_schools
				
				*Enrolls at charter flag (student level)
				bys maskedid appended: egen enroll_any_`sch_type' 	= max(`sch_type')
				
				*Rank 1 at school level
				gen rank1_study = (appstudy==1 & apprank==1)
			
				************************** School oversubscription table **************************
				*Schools ranked, assigned, 0 < prob assignment < 1 
				preserve
					keep if app`sch_type'==1
					
					bys appschoolnum: egen tot_rank1 = sum(rank1_study)
					bys appschoolnum: egen tot_rank_year_1 = count(maskedid) if appended == 0
					bys appschoolnum: egen tot_rank_year_2 = count(maskedid) if appended == 1
					bys appschoolnum: egen tot_offer_year_1 = sum(offer) if appended == 0
					bys appschoolnum: egen tot_offer_year_2 = sum(offer) if appended == 1
					
					*IR: Breaking down by year 1 and year 2. 
					
					bys appschoolnum: egen tot_`sch_type'_not01_year_1 = count(cond(pscore_`sch_type'_`file' > 0 & pscore_`sch_type'_`file'<1,maskedid,.)) if appended == 0
					bys appschoolnum: egen tot_`sch_type'_not01_year_2 = count(cond(pscore_`sch_type'_`file' > 0 & pscore_`sch_type'_`file'<1,maskedid,.)) if appended == 1
					
					bys appschoolnum: egen tot_school_not01_year_1 = count(cond(pscore_school>0 & pscore_school<1,maskedid,.)) if appended == 0
					bys appschoolnum: egen tot_school_not01_year_2 = count(cond(pscore_school>0 & pscore_school<1,maskedid,.)) if appended == 1

					bys appschoolnum: egen tot_school_is1_year_1 = count(cond(pscore_school==1,maskedid,.)) if appended == 0
					bys appschoolnum: egen tot_school_is1_year_2 = count(cond(pscore_school==1,maskedid,.)) if appended == 1

					bys appschoolnum: egen tot_school_not01_1st_year_1 = count(cond(pscore_school > 0 & pscore_school < 1 & apprank == 1,maskedid,.)) if appended == 0 
					bys appschoolnum: egen tot_school_not01_1st_year_2 = count(cond(pscore_school > 0 & pscore_school < 1 & apprank == 1,maskedid,.)) if appended == 1

					*IR variables stacked. 
					bys appschoolnum: egen tot_`sch_type'_not01 = count(cond(pscore_`sch_type'_`file'>0 & pscore_`sch_type'_`file'<1,maskedid,.))
					bys appschoolnum: egen tot_school_not01_stacked = count(cond(pscore_school>0 & pscore_school<1,maskedid,.))
					bys appschoolnum: egen tot_school_is1_stacked = count(cond(pscore_school==1,maskedid,.))
					bys appschoolnum: egen tot_school_not01_1st_stacked = count(cond(pscore_school > 0 & pscore_school < 1 & apprank == 1,maskedid,.))
					la var tot_school_not01_1st_stacked "tot school pscore not 01 and first choice in study with years stacked"
					collapse (firstnm) appschoolname tot_* MS HS capacity*, by(appschoolnum)
										
					tempfile temp
					sa `temp'
				restore
				
				*Subject to randomization at the FULLY SATURATED PSCORE level (variation in instrument for every value of PSCORE) preserve
				
				preserve

					drop if ( baselinespyear != 2013 & appended == 1 ) | ( baselinespyear != 2012 & appended == 0)
			
					keep if app`sch_type'==1
					
					*Any charter offer pscore - by school, the number of kids in pscore sets with variation in any offer
						bys pscore_`sch_type'_`file': egen min_offer = min(any_`sch_type'_offer)
						bys pscore_`sch_type'_`file': egen max_offer = max(any_`sch_type'_offer)
						
						gen sum = min_offer + max_offer
						bys appschoolnum: egen tot_`sch_type'_rand = count(cond(sum==1 ,maskedid,.))
						bys appschoolnum: egen tot_`sch_type'_rand_not01 = count(cond(sum==1 & pscore_`sch_type'_`file'>0 & pscore_`sch_type'_`file'<1,maskedid,.))
						
					*School offer - if within the same school, there are kids with the same pscore and var in offer
						drop min_offer max_offer sum
						bys appschoolnum pscore_school: egen min_offer = min(offer)
						bys appschoolnum pscore_school: egen max_offer = max(offer)
						
						gen sum = min_offer + max_offer
						bys appschoolnum: egen tot_school_rand = count(cond(sum==1,maskedid,.))
						bys appschoolnum: egen tot_school_rand_not01 = count(cond(sum==1 & pscore_school>0 & pscore_school<1,maskedid,.))
						drop min_offer max_offer
						
						*Number of kids with family link
						bys appschoolnum: egen tot_family = sum(family)
						
						duplicates drop appschoolnum, force
						keep appschoolnum  tot_*
						
						merge 1:1 appschoolnum using `temp'
						drop _merge
						
						order appschoolnum appschoolname MS HS ///
							tot_rank1 tot_rank_year_?	tot_offer_year_? ///
							tot_`sch_type'_not01	tot_school_not01_stacked tot_school_is1_stacked ///
							tot_`sch_type'_rand	tot_`sch_type'_rand_not01	///
							tot_school_rand	tot_school_rand_not01
						gsort - MS + appschoolname
						
						outsheet using "`tables'inDPSbase_`file'_`sch_type'_years_stacked_schools.csv", comma replace

				restore
				
			}
		}
	}
		
	************************************************
	**************** Student Demo ******************
	************************************************
	if `student_demo'==1{
		

		local row = 1
		local col = 1
		
		*Denver mean : 3-9 graders in 2011-2012
			use "`cleandata'sims_csap_long_20102014 `simsdate'", clear
			
			unique maskedid
			
			keep if inlist(grade,"3","4","5","6","7","8","9") & (spyear==2012 | spyear == 2013)
			
			*Ignacio: these are the year 2 study schools, not year 1. 
			
			*Study charters for years 1 and 2
			gen study = (((inlist(schoolnum, 167, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(schoolnum, 330, 391, 392, 393, 394, 426, 428, 429, 435, 436, 437, 438, 441, 443, 444, 445, 446) | ///
						inlist(schoolnum, 462, 481, 485, 496, 498, 512, 513, 517)) & spyear == 2013) | ///
						((inlist(schoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(schoolnum, 330, 391, 392, 393, 394, 426, 428, 429, 435, 436, 438, 441, 443, 444, 445, 446) | ///
						inlist(schoolnum, 462, 481, 485, 496, 498, 512, 513)) & spyear == 2012) ///
					)

			**Study no-excuses charters
			gen noex =  (((inlist(schoolnum, 392, 393, 394, 426, 428, 429, 435, 436, 437) | ///
						inlist(schoolnum, 438, 443, 444, 445, 462, 481, 498, 513, 517)) & spyear == 2013) | ///
						((inlist(schoolnum, 392, 393, 394, 426, 428, 429, 435, 436) | ///
						inlist(schoolnum, 438, 443, 444, 445, 462, 481, 498, 513)) & spyear == 2012) ///
					)
	
			**Study non no-excuses charters
			gen nonnoex = (((inlist(schoolnum, 167, 175, 176, 196, 211, 212) | ///
						  inlist(schoolnum, 266, 328, 330, 391, 441, 446, 485, 496, 512)) & spyear == 2013) | ///
						  ((inlist(schoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
							 inlist(schoolnum, 330, 391, 441, 446, 485, 496, 512)) & spyear == 2012) ///
					)
					
			gen spyear2012_flag =  (spyear==2012)

			
			*Means (exclude alternatives for scores)
			foreach var of varlist study female hisp black foodfred sped lep bilingual gifted white asian ss_math ss_reading ss_writing spyear2012_flag noex nonnoex {
				
				qui summ `var'
				matrix demo[`row',`col']   = r(mean)
				
				qui summ `var' if study == 1
				matrix demo[`row', `col'+1] = r(mean)
				
				qui summ `var' if noex == 1
				matrix demo[`row', `col'+2] = r(mean)
				
				qui summ `var' if nonnoex == 1
				matrix demo[`row', `col'+3] = r(mean)
				local ++row
			}
			
			*Count
				unique maskedid
				local denver_N = r(sum)
				
				
				count if charter == 1
				local charter_N = r(N)
				
				count if noex == 1
				local noex_N = r(N)
				
				count if nonnoex == 1
				local nonnoex_N = r(N)
		
		*Charter applicants who were in Denver at baseline (simulated score not calculated by CMOs. Calculated just for whether charter app or not.)
			
			foreach sch_type in study /*noex nonnoex */	{
			
			
			*starting with year 1 pscore
			use "`cleandata'mdrd_denver_`sch_type'_simulation `pscoredate_year1'", clear
				
			*Go get other scores - assert that files have same obs
			merge 1:1 maskedid appschoolnum using "`cleandata'mdrd_denver_`sch_type'_formula `pscoredate_year1'", keepusing(pscore_`sch_type'_formula)
			assert _merge==3 
			drop _merge
			
			merge 1:1 maskedid appschoolnum using "`cleandata'mdrd_denver_`sch_type'_frequency `pscoredate_year1'", keepusing(pscore_`sch_type'_frequency)
			assert _merge==3 
			drop _merge
				
			*Drop if not in Denver at baseline or not an applicant to any schools in year 1 
			drop if baselinespyear!=2012 | appschoolnum==. | !inlist(appgrade,"4","5","6","7","8","9","10")
			tempfile year1_scores
			sa `year1_scores'
			
			
			
			*adding year 2 applications and pscore
			
			use "`cleandata'mdrd_denver_`sch_type'_year_2_simulation `pscoredate_year2'", clear
			
			merge 1:1 maskedid appschoolnum using "`cleandata'mdrd_denver_`sch_type'_year_2_formula `pscoredate_year2'", keepusing(pscore_`sch_type'_formula)
			assert _merge==3
			drop _merge
			merge 1:1 maskedid appschoolnum spyear using "`cleandata'mdrd_denver_`sch_type'_year_2_frequency `pscoredate_year2'", keepusing(pscore_`sch_type'_frequency)
			assert _merge==3
			drop _merge
			
			*Drop if not in Denver at baseline or not an applicant to any schools in year 2 	
			drop if ( baselinespyear!=2013  | appschoolnum==. | !inlist(appgrade,"4","5","6","7","8","9","10")) 	
						
			**********************************************
			*** We are rounding simulated score to 0.001 here. 
			*** Unrounded doesn't quite work - too much support. 
			*** Simply replacing simulated score with rounded equivalent because
			*** it keeps the matrix logic intact
			**********************************************
			
			replace pscore_study_simulation = round(pscore_study_simulation, 0.001)

			append using `year1_scores', gen(year1) 
			
			/*
			*dropping duplicates from year 2. 
			bys masked: egen min_year = min(baselinespyear)
			bys masked: egen max_year = max(baselinespyear)
			drop if min_year != max_year & baselinespyear == 2013			
			*/
			bys maskedid baselinespyear: egen apply_`sch_type' = max(app`sch_type')
			*Enrolls at charter flag
			bys maskedid baselinespyear: egen enroll_any_`sch_type' 	= max(`sch_type')	
			bys maskedid baselinespyear: egen offer_any_`sch_type'      = max(offer_`sch_type')
			*If study is missing, kid left DPS. Flag as enroll elsewhere
			tab enroll_any_`sch_type', m
			replace enroll_any_`sch_type'=0 if enroll_any_`sch_type'==.
			replace offer_any_`sch_type' =0 if offer_any_`sch_type' == .
			
			
			
						*Ignacio 
			* this tabs the number of charter offers according to the student's choice
			preserve
				tab apprank if offer ==1 &  appstudy == 1 , matcell(freq) matrow(choice)
			
				tab apprank if offer ==1 &  appstudy == 1 & baselinespyear == 2012, matcell(freqy1) 
				tab apprank if offer ==1 &  appstudy == 1 & baselinespyear == 2012 & pscore_study_simulation < 1 & pscore_study_simulation > 0, matcell(freq01y1)
				
				tab apprank if offer ==1 &  appstudy == 1 & baselinespyear == 2013, matcell(freqy2) 
				tab apprank if offer ==1 &  appstudy == 1 & baselinespyear == 2013 & pscore_study_simulation < 1 & pscore_study_simulation > 0, matcell(freq01y2) 
				
				tab apprank if   appstudy == 1 & pscore_study_simulation < 1 & pscore_study_simulation > 0, matcell(applicants01) 
				tab apprank if   appstudy == 1 & baselinespyear == 2012 & pscore_study_simulation < 1 & pscore_study_simulation > 0, matcell(applicants01y1) 
				tab apprank if   appstudy == 1 & baselinespyear == 2013 & pscore_study_simulation < 1 & pscore_study_simulation > 0, matcell(applicants01y2) 
				
				matrix applicants01 = J(5,1,.)
				matrix applicants01y1 = J(5,1,.)
				matrix applicants01y2 = J(5,1,.)
				
				foreach rank of numlist 1/5 {
					unique masked if appstudy == 1 & apprank == `rank' & pscore_study_simulation < 1 & pscore_study_simulation > 0 
					matrix applicants01[`rank',1] = `r(sum)'
					unique masked if appstudy == 1 & apprank == `rank' & pscore_study_simulation < 1 & pscore_study_simulation > 0 & baselinespyear == 2012 
					matrix applicants01y1[`rank',1] = `r(sum)'
					unique masked if appstudy == 1 & apprank == `rank' & pscore_study_simulation < 1 & pscore_study_simulation > 0 & baselinespyear == 2013
					matrix applicants01y2[`rank',1] = `r(sum)'
					
				}
				
				matrix result = choice,freq,freqy1,freq01y1,freqy2,freq01y2,applicants01,applicants01y1,applicants01y2
				svmat result
				keep result* 
				ren result1 choice
				ren result2 frequency
				ren result3 frequencyy1
				ren result4 frequency01y1
				ren result5 frequencyy2
				ren result6 frequency01y2
				ren result7 applicants01
				ren result8 applicants01y1
				ren result9 applicants01y2
				
				 
				
				outsheet using "`tables'denver_offers_and_choice_tab.csv", comma replace
			restore
			
			
			********************************************************************************
			*Analysis file is at application level until here. Drop to get at student-year level*
			********************************************************************************
			
			gen D = enroll_any_`sch_type'
			gen Z = offer_any_`sch_type'
			
			gen kappa = 1 - (((1-D)*Z) / pscore_`sch_type'_frequency) - ((D*(1-Z)) / (1-pscore_`sch_type'_frequency))
			gen gamma = ((D*(1-Z)) / (1-pscore_`sch_type'_frequency))
			
			duplicates drop maskedid baselinespyear, force
			
			*creating year 1 flag
			gen year1_applicant = (baselinespyear == 2012)			
			
			local row = 1
			local col = 4
			gen var=""
			
			*Means
				foreach var of varlist `cov' baselinewhite baselineasian `cov2' year1 /*year1_applicant*/ {

					replace var="`var'" if _n==`row'
					
					*All applicants (excluding applicants to alternative schools)
					qui summ `var'
					matrix demo[`row',`col'+1]   = r(mean)
					
					*Charter applicants
					qui summ `var' if apply_`sch_type'==1
					matrix demo[`row',`col'+2]   = r(mean)

					*Charter enrollees
					qui summ `var' if enroll_any_`sch_type' == 1
					matrix demo[`row',`col'+3]  = r(mean)

					local c = -2
					
					
					foreach file in frequency formula simulation{

						local c = `c' + 3
						
						*All
						qui summ `var' if apply_`sch_type'==1 & !inlist(pscore_`sch_type'_`file',1,0)
						matrix demo[`row',`col'+3 + `c']   = r(mean)
						*Erolls at charter
						qui summ `var' if apply_`sch_type'==1 & !inlist(pscore_`sch_type'_`file',1,0) & enroll_any_`sch_type'==1
						matrix demo[`row',`col'+4 + `c']   = r(mean)
						
						*Enrolls elsewhere
						qui summ `var' if apply_`sch_type'==1 & !inlist(pscore_`sch_type'_`file',1,0) & enroll_any_`sch_type'==0
						matrix demo[`row',`col'+5 + `c']   = r(mean)
						
					}
					
					
						
					gen kappa_`var' = kappa*`var'
					gen gamma_`var' = gamma*`var'
					
					qui summ kappa_`var' if apply_`sch_type' == 1 & !inlist(pscore_`sch_type'_frequency,1,0)
					matrix demo[`row',`col'+6 + `c']   = r(mean)
	
					qui summ kappa if apply_`sch_type' == 1 & !inlist(pscore_`sch_type'_frequency, 1, 0)
					matrix demo[`row', `col'+7 + `c'] = r(mean)
				
					qui summ gamma_`var' if apply_`sch_type' == 1 & !inlist(pscore_`sch_type'_frequency,1,0)
					matrix demo[`row',`col'+8 + `c']   = r(mean)
	
					qui summ gamma if apply_`sch_type' == 1 & !inlist(pscore_`sch_type'_frequency, 1, 0)
					matrix demo[`row', `col'+9 + `c'] = r(mean)
					
					qui summ `var' if enroll_any_`sch_type' == 1
					matrix demo[`row', `col'+10 + `c'] = r(mean)
					local ++row
				
				}
			
			*Count
				
				local col = 1
				
				*Denver 2011-2012
				matrix demo[`row'+3, `col'] = `denver_N'

				*Enrolled in charter
				matrix demo[`row'+3, `col'+1] = `charter_N'

				*Enrolled in noex
				matrix demo[`row'+3, `col'+2] = `noex_N'
				
				*Enrolled in non noex
				matrix demo[`row'+3, `col'+3] = `nonnoex_N'
				
				*All applicants
				count
				matrix demo[`row'+3,`col'+4]   = r(N)
				
				*Charter applicants
				count if apply_`sch_type'==1
				matrix demo[`row'+3,`col'+5]   = r(N)

				*Charter enrollees (among applicants)
				count if enroll_any_`sch_type' == 1
				matrix demo[`row'+3,`col'+6]   = r(N)
								
				
				local c = -2
				foreach file in frequency formula simulation{
					local c = `c' + 3

					*Pscore in (0,1) (all)
					count if apply_`sch_type' ==1 & !inlist(pscore_`sch_type'_`file',1,0)
					matrix demo[`row'+3,`col'+6 + `c']   = r(N)
					
					*Pscore in (0,1)  (enrolls at charter)
					count if apply_`sch_type' ==1 & !inlist(pscore_`sch_type'_`file',1,0) & enroll_any_`sch_type'==1
					matrix demo[`row'+3,`col'+7 + `c']   = r(N)
					
					*Pscore in (0,1)  (enrolls elsewhere)
					count if apply_`sch_type'==1 & !inlist(pscore_`sch_type'_`file',1,0) & enroll_any_`sch_type'==0
					matrix demo[`row'+3,`col'+8 + `c']   = r(N)
				}
					
				* Charter enrollees
				count if enroll_any_`sch_type' == 1
				matrix demo[`row'+3, `col'+13 + `c'] = r(N)
				
				
				*adding in rounded 0.001 for simualted 
				
				
		
		svmat demo
		keep var demo*
		keep if _n <= `row' +3
		
		di "row `row'"
		replace var="baselinenoex" if _n == `row'
		replace var="baselinenonnoex" if _n == `row' + 1
		replace var="N" if _n==`row' +3
		
		ren demo1 denver
		ren demo2 charter
		ren demo3 noex
		ren demo4 nonnoex
		ren demo5 applicants
		ren demo6 study_app
		ren demo7 study_enroll
      		
		ren demo8 frequency01
		ren demo9 frequency01_study
		ren demo10 frequency01_elsewhere
		
		ren demo11 formula01
		ren demo12 formula01_study
		ren demo13 formula01_elsewhere
		
		ren demo14 simulation01_rnd_001
		ren demo15 simulation01_rnd_001_study
		ren demo16 simulation01_rnd_001_elsewhere
		
		ren demo17 kappa_cov
		ren demo18 kappa
		ren demo19 gamma_cov
		ren demo20 gamma
		
		ren demo21 study_enrolled
		outsheet using "`tables'inDPSbase_`sch_type'_years_stacked_student_demo.csv", comma replace
	}
}


	if `pscore_balance'==1{
	
		foreach file in formula   simulation frequency     {
		foreach sch_type in study {

			foreach cut in inDPSbase {
			
				u "`cleandata'analysis_stacked_`sch_type'_`file'.dta", clear

				if "`cut'"=="inDPSbase"{
					keep if baselinespyear==2012 | baselinespyear == 2013
				}
				else{
					di in red "Keep all applicants"
				}
				
				**********************************************************************************
				************************ Balance and attrition coefficients **********************
				**********************************************************************************
				
				local row = 1
				local col = 1
				gen var =""
				
				*Count points in the support
				replace var = "Riskset points of support" if _n ==1
							
				*Place it on top of balance table
				
				unique pscore1 		if in_zero1_pscore==1
				matrix balance[1,3] = r(sum)
			
				unique r01			if in_good_cell_r01==1
				matrix balance[1,5] = r(sum)
				
				unique r001			if in_good_cell_r001==1
				matrix balance[1,6] = r(sum)
				
				unique r0			if in_good_cell_r0==1
				matrix balance[1,7] = r(sum)

				
				local row = `row' + 1
				
				foreach var in `attrition_vars' `appcov' `cov' `cov2' {
				
					replace var="`var'" if _n==`row'
					replace var="`var'_se" if _n==`row'+1
					replace var="`var'_N" if _n==`row'+2
					replace var="`var'_N(var in Z)" if _n==`row'+3
					replace var="`var'_N(var in Z & X)" if _n==`row'+4
						
					*All non-offered applicants
					qui summ `var' if Z==0
						matrix balance[`row',`col']   = r(mean)
						matrix balance[`row'+1,`col'] = r(sd)
						matrix balance[`row'+2,`col'] = r(N)

					*Balance and attrition, no control(OLS)
					reg `var' Z i.baselinespyear, vce(robust)
						matrix balance[`row',`col'+1]   = _b[Z]
						matrix balance[`row'+1,`col'+1] = _se[Z]
						matrix balance[`row'+2,`col'+1] = e(N)
					
					*Balance and attrition, pscore linear control(OLS)
					qui reg `var' Z pscore1 i.baselinespyear if !inlist(r0,1,0) , vce(robust)
						matrix balance[`row',`col'+2]   = _b[Z]
						matrix balance[`row'+1,`col'+2] = _se[Z]
						matrix balance[`row'+2,`col'+2] = e(N)
						
					*Balance and attrition, pscore polynomial control(OLS)
					qui reg `var' Z i.baselinespyear pscore1 pscore2 pscore3 pscore4 if !inlist(r0,1,0), vce(robust)
						matrix balance[`row',`col'+3]   = _b[Z]
						matrix balance[`row'+1,`col'+3] = _se[Z]
						matrix balance[`row'+2,`col'+3] = e(N)
							
					*Balance and attrition, risks
					* IR 04152016 now includes 1000's
							
							
					local c = 0
					foreach i in 01 001{
						local c = `c' +1
							
						preserve
							
							areg `var' Z i.baselinespyear  if !inlist(r`i',1,0), absorb(r`i') vce(robust)
							areg `var' Z i.baselinespyear  if in_zero1_pscore == 1, absorb(r`i') vce(robust)
							
							matrix balance[`row',`col'+3 + `c']   = _b[Z]
							matrix balance[`row'+1,`col'+3 + `c'] = _se[Z]
							matrix balance[`row'+2,`col'+3 + `c'] = e(N)
							
							bys r`i': egen sd = sd(`var')
							bys r`i': egen sd_z = sd(Z)
							
							*Count kids with variation in instrument within cell (no ref to characteristic)
							count if sd_z > 0 & !missing(sd_z)  & e(sample)
							matrix balance[`row'+3,`col'+3 + `c'] = r(N)
							
							*Count kids with variation in characteristic & instrument at cell level
							count if sd>0 & sd_z >0 & !missing(sd)  & !missing(sd_z)  & e(sample)
							matrix balance[`row'+4,`col'+3 + `c'] = r(N)
							
							drop sd sd_z		
							
						restore
					}
					
					
					*Fully saturated in pscore
						preserve
						
							qui areg `var' Z i.baselinespyear if !inlist(r0,1,0) , absorb(r0) vce(robust)
							matrix balance[`row',`col'+4 + `c']   = _b[Z]
							matrix balance[`row'+1,`col'+4 + `c'] = _se[Z]
							matrix balance[`row'+2,`col'+4 + `c'] = e(N)
							
							bys r0: egen sd = sd(`var')
							bys r0: egen sd_z = sd(Z)

							*Count kids with variation in instrument within cell (no ref to characteristic)
							count if sd_z > 0 & !missing(sd_z) & e(sample)
							matrix balance[`row'+3,`col'+4 + `c'] = r(N)
							
							*Count kids with variation in characteristic & instrument at cell level
							count if sd>0 & sd_z >0 & !missing(sd)  & !missing(sd_z)  & e(sample)
							matrix balance[`row'+4,`col'+4 + `c'] = r(N)
							
							drop sd sd_z
							
						restore
					
				local row = `row' + 6
				}
				
			************************************* F tests **************************************	
				*MS 2/23/2015: 
				*Minimum number of observations neeed for the SUEST F test to run according to each pscore type
				*Arrived at these numbers by running F tests for several different minimum number of obs. These
				*are the minimum needed for the set of covariates being tested as of 2/23/2015.
				
				local min_obs_cell_simulation 	= 9			
				local min_obs_cell_formula 		= 9
				local min_obs_cell_frequency 	= 9

				
			if `ftest'==1{
				
				**********************************************************************************
				************************************* Logit Fs ***********************************
				**********************************************************************************
				
				foreach type in robust traditional{
				
						foreach covs in apponly balanceonly all{
						
							if "`covs'"=="apponly"{
								local balF "`appcov'"
								local Fname "Application Balance"
							}
							
							else if "`covs'"=="balanceonly"{
								local balF "`cov' `cov2'"
								local Fname "Baseline Balance"
							}
							else if "`covs'"=="all"{
								local balF "`appcov' `cov' `cov2'"
								local Fname "All Balance"
							}
							
							local row = `row' + 3
							
							if "`type'"=="robust"{
								local option "nocnsreport vce(robust)"
							}
							else{
								local option "nocnsreport"
							}
							
							* Balance logit
							replace var="Logit `type' `Fname' F (chi2)" if _n==`row'
							replace var="Logit `type' `Fname' F (pval)" if _n==`row'+1
							replace var="Logit `type' `Fname' F (df)" if _n==`row'+2
							replace var="Logit `type' `Fname' F (N)" if _n==`row'+3
						
							*No riskset
							qui logit Z i.baselinespyear `balF', `option'
							test `balF'
							matrix balance[`row', `col'+1] = r(chi2)
							matrix balance[`row'+1, `col'+1] = r(p)
							matrix balance[`row'+2, `col'+1] = r(df)
							matrix balance[`row'+3, `col'+1] = e(N)	
		
							*Linear riskset
							qui logit Z i.baselinespyear pscore1 `balF' if in_zero1_pscore==1, `option'
							test `balF'
							matrix balance[`row', `col'+2] = r(chi2)
							matrix balance[`row'+1, `col'+2] = r(p)
							matrix balance[`row'+2, `col'+2] = r(df)
							matrix balance[`row'+3, `col'+2] = e(N)
							
							*Poly riskset
							qui logit Z i.baselinespyear pscore* `balF' if in_zero1_pscore==1, `option'
							test `balF'
							matrix balance[`row', `col'+3] = r(chi2)
							matrix balance[`row'+1, `col'+3] = r(p)
							matrix balance[`row'+2, `col'+3] = r(df)
							matrix balance[`row'+3, `col'+3] = e(N)
							
							*Bins
							local c=0
							foreach i in 01 001{
								local c = `c' + 1
								qui logit Z i.baselinespyear i.ind_r`i' `balF' if in_good_cell_r`i'==1, `option'
								test `balF'
								matrix balance[`row', `col' +3 + `c'] = r(chi2)
								matrix balance[`row' +1, `col' +3 + `c'] = r(p)
								matrix balance[`row'+2, `col' +3 + `c'] = r(df)
								matrix balance[`row'+3, `col' +3 + `c'] = e(N)
							}
							
							*Fully saturated
							qui logit Z i.baselinespyear i.ind_r0 `balF' if in_good_cell_r0==1, `option'
							test `balF'
							matrix balance[`row', `col' +4 + `c'] = r(chi2)
							matrix balance[`row' +1,`col' +4 + `c'] = r(p)
							matrix balance[`row'+2, `col' +4 + `c'] = r(df)
							matrix balance[`row'+3, `col' +4 + `c'] = e(N)
					
						local row = `row' + 4
					}	/*Close balance covariates loop*/

			} /*CLose robust/traditional F*/
				
				local row = `row' + 4
				********************************************************
				*************** Balance Traditional F ******************
				********************************************************
				
				foreach covs in apponly balanceonly all{
						
					if "`covs'"=="apponly"{
						local balF "`appcov'"
						local Fname "Application Balance"
					}
					
					else if "`covs'"=="balanceonly"{
						local balF "`cov' `cov2'"
						local Fname "Baseline Balance"
					}
					else if "`covs'"=="all"{
						local balF "`appcov' `cov' `cov2'"
						local Fname "All Balance"
					}
							
					replace var="Traditional `Fname' F-test" if _n==`row'
					replace var="Traditional `Fname' F-test (pval)" if _n==`row' +1
					replace var="Traditional `Fname' F-test (residuals df)" if _n==`row' +2
					replace var="Traditional `Fname' F-test (model df)" if _n==`row' +3
					replace var="Traditional `Fname' F-test (N)" if _n==`row' +4
					replace var="Traditional `Fname' F-test (Breusch-Pagan chi2)" if _n==`row'+5
					
					*no propensity score
						mvreg `balF' = Z i.baselinespyear , corr
						test Z
						matrix balance[`row',`col' + 1] = r(F)
						matrix balance[`row'+1,`col' + 1]= r(p)
						matrix balance[`row'+2, `col'+1] = r(df_r)
						matrix balance[`row'+3, `col'+1] = r(df)
						matrix balance[`row'+4, `col'+1] = e(N)
						matrix balance[`row'+5, `col'+1] = 1- chi2(e(df_chi2),e(chi2))
					
					*linear pscore control
						mvreg `balF' = Z i.baselinespyear pscore1 if !inlist(r0,1,0), corr
						test Z
						matrix balance[`row',`col' +2]=r(F)
						matrix balance[`row'+1,`col' +2]=r(p)
						matrix balance[`row'+2, `col'+2] = r(df_r)
						matrix balance[`row'+3, `col'+2] = r(df)
						matrix balance[`row'+4, `col'+2] = e(N)
						matrix balance[`row'+5, `col'+2] = 1- chi2(e(df_chi2),e(chi2))
						
					*polynomial pscore control
						mvreg `balF' = Z i.baselinespyear pscore1 pscore2 pscore3 pscore4 if !inlist(r0,1,0), corr
						test Z
						matrix balance[`row',`col' + 3]=r(F)
						matrix balance[`row'+1,`col'+ 3]=r(p)
						matrix balance[`row'+2, `col'+3] = r(df_r)
						matrix balance[`row'+3, `col'+3] = r(df)
						matrix balance[`row'+4, `col'+3] = e(N)
						matrix balance[`row'+5, `col'+3] = 1- chi2(e(df_chi2),e(chi2))
						
					*Bins
						local c=0
						foreach i in 01 001{
							local c = `c' + 1
							mvreg `balF' = Z i.baselinespyear i.ind_r`i' if  !inlist(r`i',1,0), corr
							test Z
							matrix balance[`row',`col' + 3 + `c'] = r(F)
							matrix balance[`row'+1,`col' + 3 + `c'] = r(p)
							matrix balance[`row'+2,`col' + 3 + `c'] = r(df_r)
							matrix balance[`row'+3,`col' + 3 + `c'] = r(df)
							matrix balance[`row'+4,`col' + 3 + `c'] = e(N)
							matrix balance[`row'+5,`col' + 3 + `c'] = 1- chi2(e(df_chi2),e(chi2))
							
							matrix B_ols_z1_bin`i'_rescorr = corr(e(Sigma))
						}
					
						*Pscore saturated
						mvreg `balF' = Z i.baselinespyear i.ind_r0  if in_good_cell_r0==1, corr
						test Z
						matrix balance[`row',`col' + 4 + `c'] = r(F)
						matrix balance[`row'+1,`col' + 4 + `c'] = r(p)
						matrix balance[`row'+2,`col' + 4 + `c'] = r(df_r)
						matrix balance[`row'+3,`col' + 4 + `c'] = r(df)
						matrix balance[`row'+4,`col' + 4 + `c'] = e(N)
						matrix balance[`row'+5,`col' + 4 + `c'] = 1- chi2(e(df_chi2),e(chi2))
						
					local row = `row' + 6
				}	/*Close covariates loop*/
				
			
				
				**********************************************************************************
				*************************** Balance Stacked Robust F *****************************
				**********************************************************************************
				
				*Run a cut keeping cells with small size and one dropping cells with small size
				foreach f_cut in keep_small_cells drop_small_cells{
					
					foreach covs in apponly balanceonly all{
						
						if "`covs'"=="apponly"{
							local balF "`appcov'"
							local Fname "Application Balance"
							local expand = 3
							local test "Z_1 Z_2 Z_3"
						}
						
						else if "`covs'"=="balanceonly"{
							local balF "`cov' `cov2'"
							local Fname "Baseline Balance"
							local expand = 12
							local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12"
						}
						else if "`covs'"=="all"{
							local balF "`appcov' `cov' `cov2'"
							local Fname "All Balance"
							local expand = 15
							local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12 Z_13 Z_14 Z_15"
						}
					
						preserve
							
							if "`f_cut'" == "keep_small_cells"{
								local min_obs = 1
							}
							else if "`f_cut'"=="drop_small_cells"{
								local min_obs = `min_obs_cell_`file''
							}
							
							keep maskedid `balF' Z D tag_r01 tag_r0 tag_r001 r01 r001 r0 pscore1 pscore2 pscore3 pscore4 family in_sample in_good_cell_* obs_cell_* in_zero1_pscore baselinespyear

							expand `expand'						

							bys maskedid baselinespyear: gen copy = _n
							sort copy
						
							* if kids repeat years you get a larger max value of copy, which will break 
							* the logic used for the tokenize chunk below. 
							su copy 
							if `r(max)' !=  `expand' {
								di as error " Somethign is off when you make a copy of the data. You need to end up with only `expand' copies per kid. Rn you have `r(max)'"
								exit  
								
							}
																		
							*Single vector of outcomes
							gen cov = .
							gen covname = "" 
	
							tokenize `balF'


							forval i=1/`expand'{

								replace cov = ``i'' if copy==`i' 
								replace covname = "``i''" if copy==`i' 
								
							}					
							
							drop `balF'
							
							

							
							*Instruments, pscore variables, and dummies for each copy
							forval i=1/`expand'{
								gen Z_`i' = Z*(copy == `i')
								gen alpha_`i' = (copy == `i')
								
								*Pscore variables for each copy
								foreach x in r01 r0 r001 pscore1 pscore2 pscore3 pscore4 baselinespyear{
									gen c_`x'_`i'= `x'*(copy == `i')
								}
							}
							

							
							*Absorb takes only one var. Create grouped vars
							foreach x in tag_r01 tag_r0 tag_r001 r001 r01 r0 {
									egen group_`x' = group(copy `x')
								
							}
							
							*Replace with missing if first categoryobs_
							foreach x in tag_r001 tag_r01 tag_r0{
								replace group_`x' =. if `x'==-9999999
							
							}
							*** One tall regression for each specification ***BREAK

							
							*No controls				
								*Clustered

								
								reg cov alpha_* Z_* i.c_baselinespyear*, vce(cluster maskedid)
								test `test'
								
								matrix balance[`row',`col' + 1]=r(F)
								matrix balance[`row'+1,`col' +1]=r(p)
								matrix balance[`row'+2,`col' +1]=r(df_r)
								matrix balance[`row'+3,`col' +1]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+4,`col' + 1]=r(sum)
							
								
								*Traditional
								qui reg cov alpha_* Z_*  i.c_baselinespyear*
								test `test'
								
								matrix balance[`row'+6,`col' + 1]=r(F)
								matrix balance[`row'+7,`col' +1]=r(p)
								matrix balance[`row'+8,`col' +1]=r(df_r)
								matrix balance[`row'+9,`col' +1]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+10,`col' + 1]=r(sum)
								
								
								*Robust
								qui reg cov alpha_* Z_* i.c_baselinespyear*, vce(cluster maskedid)
								test `test'
								
								matrix balance[`row'+12,`col' + 1]=r(F)
								matrix balance[`row'+13,`col' +1]=r(p)
								matrix balance[`row'+14,`col' +1]=r(df_r)
								matrix balance[`row'+15,`col' +1]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+16,`col' + 1]=r(sum)
								
								

							*Linear
								*Cluster
								qui reg cov alpha_* Z_* i.c_baselinespyear* c_pscore1_* if in_zero1_pscore==1, vce(cluster maskedid)
								test `test'
								
								matrix balance[`row',`col' + 2]=r(F)
								matrix balance[`row'+1,`col' +2]=r(p)
								matrix balance[`row'+2,`col' +2]=r(df_r)
								matrix balance[`row'+3,`col' +2]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+4,`col' + 2]=r(sum)
								
								*Traditional
								qui reg cov alpha_* Z_* i.c_baselinespyear* c_pscore1_* if in_zero1_pscore==1
								test `test'
								
								matrix balance[`row'+6,`col' + 2]=r(F)
								matrix balance[`row'+7,`col' +2]=r(p)
								matrix balance[`row'+8,`col' +2]=r(df_r)
								matrix balance[`row'+9,`col' +2]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+10,`col' + 2]=r(sum)
								
								*Robust
								qui reg cov alpha_* Z_* c_pscore1_* i.c_baselinespyear* if in_zero1_pscore==1, vce(cluster maskedid)
								test `test'
								
								matrix balance[`row'+12,`col' + 2]=r(F)
								matrix balance[`row'+13,`col' +2]=r(p)
								matrix balance[`row'+14,`col' +2]=r(df_r)
								matrix balance[`row'+15,`col' +2]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+16,`col' + 2]=r(sum)
							
							*Polynomial
								*Cluster
								qui reg cov alpha_* Z_* i.c_baselinespyear* c_pscore1_* c_pscore2_* c_pscore3_* c_pscore4_* if in_zero1_pscore==1, vce(cluster maskedid)
								test `test'
								
								matrix balance[`row',`col' + 3]=r(F)
								matrix balance[`row'+1,`col' +3]=r(p)
								matrix balance[`row'+2,`col' +3]=r(df_r)
								matrix balance[`row'+3,`col' +3]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+4,`col' + 3]=r(sum)
								
								*Traditional
								qui reg cov alpha_* Z_* i.c_baselinespyear* c_pscore1_* c_pscore2_* c_pscore3_* c_pscore4_* if in_zero1_pscore==1
								test `test'
								
								matrix balance[`row'+6,`col' + 3]=r(F)
								matrix balance[`row'+7,`col' +3]=r(p)
								matrix balance[`row'+8,`col' +3]=r(df_r)
								matrix balance[`row'+9,`col' +3]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+10,`col' + 3]=r(sum)
								
								*Robust
								qui reg cov alpha_* Z_* i.c_baselinespyear* c_r0_* c_pscore2_* c_pscore3_* c_pscore4_* if in_zero1_pscore==1, vce(cluster maskedid)
								test `test'
								
								matrix balance[`row'+12,`col' + 3]=r(F)
								matrix balance[`row'+13,`col' +3]=r(p)
								matrix balance[`row'+14,`col' +3]=r(df_r)
								matrix balance[`row'+15,`col' +3]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+16,`col' + 3]=r(sum)
								
							*Bins
								local c = 0
								foreach i in 01 001{
									local c = `c' +1
									
									*Cluster
									qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_r`i'==1 & obs_cell_r`i' >= `min_obs' & !missing(obs_cell_r`i'), absorb(group_tag_r`i') vce(cluster maskedid)
									test `test'
									
									matrix balance[`row',`col'+3 + `c']   = r(F)
									matrix balance[`row'+1,`col'+3 + `c'] = r(p)
									matrix balance[`row'+2,`col'+3 + `c'] = r(df_r)
									matrix balance[`row'+3,`col'+3 + `c'] = r(df)
									
									unique maskedid if e(sample)
									matrix balance[`row'+4,`col'+3 + `c'] = r(sum)
									
									unique group_tag_r`i' if e(sample)
									matrix balance[`row'+5,`col'+3 + `c'] = r(sum) / `expand'
									
									*Traditional
									qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_r`i'==1 & obs_cell_r`i' >= `min_obs' & !missing(obs_cell_r`i'), absorb(group_tag_r`i')
									test `test'
									
									matrix balance[`row'+6,`col'+3 + `c'] = r(F)
									matrix balance[`row'+7,`col'+3 + `c'] = r(p)
									matrix balance[`row'+8,`col'+3 + `c'] = r(df_r)
									matrix balance[`row'+9,`col'+3 + `c'] = r(df)
									
									unique maskedid if e(sample)
									matrix balance[`row'+10,`col'+3 + `c'] = r(sum)
									
									unique group_tag_r`i' if e(sample)
									matrix balance[`row'+11,`col'+3 + `c'] = r(sum) / `expand'
									
									*Robust
									qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_r`i'==1 & obs_cell_r`i' >= `min_obs' & !missing(obs_cell_r`i'), absorb(group_tag_r`i') vce(robust)
									test `test'
									
									matrix balance[`row'+12,`col'+3 + `c'] = r(F)
									matrix balance[`row'+13,`col'+3 + `c'] = r(p)
									matrix balance[`row'+14,`col'+3 + `c'] = r(df_r)
									matrix balance[`row'+15,`col'+3 + `c'] = r(df)
									
									unique maskedid if e(sample)
									matrix balance[`row'+16,`col'+3 + `c'] = r(sum)
									
									unique group_tag_r`i' if e(sample)
									matrix balance[`row'+17,`col'+3 + `c'] = r(sum) / `expand'
								}

							*Saturated
								*Cluster
								qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_r0==1 & obs_cell_r0 >= `min_obs' & !missing(obs_cell_r0), absorb(group_tag_r0) vce(cluster maskedid)
								test `test'
								
								matrix balance[`row',`col' + 4 + `c']	=	r(F)
								matrix balance[`row'+1,`col' + 4 + `c']	=	r(p)
								matrix balance[`row'+2,`col' + 4 + `c']	=	r(df_r)
								matrix balance[`row'+3,`col' + 4 + `c'] =	r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+4,`col' + 4 + `c'] = r(sum)
								
								unique group_tag_r0 if e(sample)
								matrix balance[`row'+5,`col' + 4 + `c'] = r(sum) / `expand'
								
								*Traditional
								qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_r0==1 & obs_cell_r0 >= `min_obs' & !missing(obs_cell_r0), absorb(group_tag_r0)
								test `test'
								
								matrix balance[`row'+6,`col' + 4 + `c']	=	r(F)
								matrix balance[`row'+7,`col' + 4 + `c']	=	r(p)
								matrix balance[`row'+8,`col' + 4 + `c']	=	r(df_r)
								matrix balance[`row'+9,`col' + 4 + `c'] =	r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+10,`col' + 4 + `c'] = r(sum)
								
								unique group_tag_r0 if e(sample)
								matrix balance[`row'+11,`col' + 4 + `c'] = r(sum) / `expand'
								
								*Robust
								qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_r0==1 & obs_cell_r0 >= `min_obs' & !missing(obs_cell_r0), absorb(group_tag_r0) vce(robust)
								test `test'
								
								matrix balance[`row'+12,`col' + 4 + `c'] =	r(F)
								matrix balance[`row'+13,`col' + 4 + `c'] =	r(p)
								matrix balance[`row'+14,`col' + 4 + `c'] =	r(df_r)
								matrix balance[`row'+15,`col' + 4 + `c'] =	r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+16,`col' + 4 + `c'] = r(sum)
								
								unique group_tag_r0 if e(sample)
								matrix balance[`row'+17,`col' + 4 + `c'] = r(sum) / `expand'
						restore
						
						replace var = "[`f_cut'] Stacked Clustered `Fname' F-test" if _n==`row'
						replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (pval)" if _n==`row'+1
						replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (residuals df)" if _n==`row'+2
						replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (model df)" if _n==`row'+3
						replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (N)" if _n==`row'+4
						replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (cells)" if _n==`row'+5
						
						replace var = "[`f_cut'] Stacked Traditional `Fname' F-test" if _n==`row'+6
						replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (pval)" if _n==`row'+7
						replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (residuals df)" if _n==`row'+8
						replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (model df)" if _n==`row'+9
						replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (N)" if _n==`row'+10
						replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (cells)" if _n==`row'+11
						
						replace var = "[`f_cut'] Stacked Robust `Fname' F-test" if _n==`row' +12
						replace var = "[`f_cut'] Stacked Robust `Fname' F-test (pval)" if _n==`row'+13
						replace var = "[`f_cut'] Stacked Robust `Fname' F-test (residuals df)" if _n==`row'+14
						replace var = "[`f_cut'] Stacked Robust `Fname' F-test (model df)" if _n==`row'+15
						replace var = "[`f_cut'] Stacked Robust `Fname' F-test (N)" if _n==`row'+16
						replace var = "[`f_cut'] Stacked Robust `Fname' F-test (cells)" if _n==`row'+17
						
						
						local row = `row' + 18
				}	
				
				**********************************************************************************
				*********************** Suest Robust standard errors *****************************
				**********************************************************************************
				
				** F-tests using suest, which allows for robust or clustered SEs **
				* 1) Run models without clustering option
				* 2) Store results
				* 3) suest [stored values], vce(cluster clustvar)
				* 4) test coefficient
				
				
				foreach f_cut in keep_small_cells drop_small_cells{
				
						if "`f_cut'" === "keep_small_cells"{
							local min_obs = 1
						}
						else if "`f_cut'"=="drop_small_cells"{
							local min_obs = `min_obs_cell_`file''
						}
						

					*Balance F
						replace var="[`f_cut'] Suest Application Balance F" if _n==`row'
						replace var="[`f_cut'] Suest Application Balance F (pval)" if _n==`row' +1
						replace var="[`f_cut'] Suest Application Balance F (df)" if _n==`row' +2
						replace var="[`f_cut'] Suest Application Balance F (N)" if _n==`row' +3
						replace var="[`f_cut'] Suest Application Balance F (cells)" if _n==`row' +4
						
						replace var="[`f_cut'] Suest Baseline Balance F" if _n==`row' + 5
						replace var="[`f_cut'] Suest Baseline Balance F (pval)" if _n==`row' +6
						replace var="[`f_cut'] Suest Baseline Balance F (df)" if _n==`row' +7
						replace var="[`f_cut'] Suest Baseline Balance F (N)" if _n==`row' +8
						replace var="[`f_cut'] Suest Baseline Balance F (cells)" if _n==`row' +9
						
						replace var="[`f_cut'] Suest All Balance F" if _n==`row' + 10
						replace var="[`f_cut'] Suest All Balance F (pval)" if _n==`row' +11
						replace var="[`f_cut'] Suest All Balance F (df)" if _n==`row' +12
						replace var="[`f_cut'] Suest All Balance F (N)" if _n==`row' +13
						replace var="[`f_cut'] Suest All Balance F (cells)" if _n==`row' +14
						
						foreach var in `appcov' `cov' `cov2'{
							
							* no pscores
							qui reg `var' Z  i.baselinespyear
							est store `var'
							
							* linear
							qui reg `var' Z i.baselinespyear pscore1 if in_zero1_pscore==1
							est store L`var'
							
							* poly
							qui reg `var' Z i.baselinespyear pscore1 pscore2 pscore3 pscore4 if in_zero1_pscore==1
							est store P`var'
							
							* Pscore bins
							foreach i in 01 001{
								qui reg `var' Z i.baselinespyear i.ind_r`i' 	if in_good_cell_r`i'==1 & obs_cell_r`i'>=`min_obs' & !missing(obs_cell_r`i')
								est store B`i'`var'
							}
							
							*Pscore saturated
							qui reg `var' Z i.baselinespyear i.ind_r0 if in_good_cell_r0==1 & obs_cell_r0>= `min_obs' & !missing(obs_cell_r0) 
							est store S`var'
						}
						
						*** Apcov suest ***
							suest 	num_schools num_study rank1_study, vce(robust)
							test Z
							matrix balance[`row', `col'+1] = r(chi2)
							matrix balance[`row'+1, `col'+1] = r(p)
							matrix balance[`row'+2, `col'+1] = r(df)
							matrix balance[`row'+3, `col'+1] = e(N)
							
								
							suest 	Lnum_schools Lnum_study  Lrank1_study, vce(robust)
							test Z
							matrix balance[`row', `col'+2] = r(chi2)
							matrix balance[`row'+1, `col' +2] = r(p)
							matrix balance[`row'+2, `col'+2] = r(df)
							matrix balance[`row'+3, `col'+2] = e(N)
							
							suest 	Pnum_schools Pnum_study Prank1_study, vce(robust)
							test Z
							matrix balance[`row', `col'+3] = r(chi2)
							matrix balance[`row'+1, `col' +3] = r(p)
							matrix balance[`row'+2, `col'+3] = r(df)
							matrix balance[`row'+3, `col'+3] = e(N)
							
							local c=0
							foreach i in 01 001{
								local c = `c' + 1
								suest 	B`i'num_schools B`i'num_study  B`i'rank1_study, vce(robust)
								test Z
								matrix balance[`row', `col' +3 + `c'] = r(chi2)
								matrix balance[`row'+1, `col' +3 + `c'] = r(p)
								matrix balance[`row'+2, `col' +3 + `c'] = r(df)
								matrix balance[`row'+3, `col' +3 + `c'] = e(N)
								
								unique tag_r`i' if e(sample)
								matrix balance[`row'+4, `col' +3 + `c'] = r(sum)
							}
								suest 	Snum_schools Snum_study Srank1_study, vce(robust)
								test Z
								matrix balance[`row',`col'+ 4 + `c'] = r(chi2)
								matrix balance[`row'+1, `col'+ 4 + `c'] = r(p)
								matrix balance[`row'+2, `col'+ 4 + `c'] = r(df)
								matrix balance[`row'+3, `col' +4 + `c'] = e(N)
								
								unique tag_r0 if e(sample)
								matrix balance[`row'+4, `col' +4 + `c'] = r(sum)

						** Baseline covs suest ***
						
						local row = `row' + 5
						suest	baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
								baseliness_math baseliness_reading baseliness_writing, vce(robust)
						test Z
						matrix balance[`row', `col'+1] = r(chi2)
						matrix balance[`row'+1, `col'+1] = r(p)
						matrix balance[`row'+2, `col'+1] = r(df)
						matrix balance[`row'+3, `col'+1] = e(N)
							
						suest	Lbaselinestudy Lbaselinefemale Lbaselinehisp Lbaselineblack Lbaselinefoodfred Lbaselinesped Lbaselinebilingual Lbaselinelep Lbaselinegifted ///
								Lbaseliness_math Lbaseliness_reading Lbaseliness_writing, vce(robust)
						test Z
						matrix balance[`row', `col'+2] = r(chi2)
						matrix balance[`row'+1, `col' +2] = r(p)
						matrix balance[`row'+2, `col'+2] = r(df)
						matrix balance[`row'+3, `col'+2] = e(N)
						
						suest	Pbaselinestudy Pbaselinefemale Pbaselinehisp Pbaselineblack Pbaselinefoodfred Pbaselinesped Pbaselinelep Pbaselinebilingual Pbaselinegifted ///
								Pbaseliness_math Pbaseliness_reading Pbaseliness_writing, vce(robust)
						test Z
						matrix balance[`row', `col'+3] = r(chi2)
						matrix balance[`row'+1, `col' +3] = r(p)
						matrix balance[`row'+2, `col'+3] = r(df)
						matrix balance[`row'+3, `col'+3] = e(N)
						
						local c=0
						foreach i in 01 001{
							local c = `c' + 1
							suest	B`i'baselinestudy B`i'baselinefemale B`i'baselinehisp B`i'baselineblack B`i'baselinefoodfred B`i'baselinesped B`i'baselinelep B`i'baselinebilingual B`i'baselinegifted ///
									B`i'baseliness_math B`i'baseliness_reading B`i'baseliness_writing, vce(robust)
							test Z
							matrix balance[`row', `col' +3 + `c'] = r(chi2)
							matrix balance[`row'+1, `col' +3 + `c'] = r(p)
							matrix balance[`row'+2, `col' +3 + `c'] = r(df)
							matrix balance[`row'+3, `col' +3 + `c'] = e(N)
							
							unique tag_r`i' if e(sample)
							matrix balance[`row'+4, `col' +3 + `c'] = r(sum)
						}
						
							suest	Sbaselinestudy Sbaselinefemale Sbaselinehisp Sbaselineblack Sbaselinefoodfred Sbaselinesped Sbaselinelep Sbaselinebilingual Sbaselinegifted ///
									Sbaseliness_math Sbaseliness_reading Sbaseliness_writing, vce(robust)
							test Z
							matrix balance[`row',`col'+ 4 + `c'] = r(chi2)
							matrix balance[`row'+1, `col'+ 4 + `c'] = r(p)
							matrix balance[`row'+2, `col'+ 4 + `c'] = r(df)
							matrix balance[`row'+3, `col' +4 + `c'] = e(N)
							
							unique tag_r0 if e(sample)
							matrix balance[`row'+4, `col' +4 + `c'] = r(sum)

					*** All covariates suest ***
					
						local row = `row' + 5
						suest	num_schools num_study rank1_study ///
								baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
								baseliness_math baseliness_reading baseliness_writing, vce(robust)
						test Z
						matrix balance[`row', `col'+1] = r(chi2)
						matrix balance[`row'+1, `col'+1] = r(p)
						matrix balance[`row'+2, `col'+1] = r(df)
						matrix balance[`row'+3, `col'+1] = e(N)
						
							
						suest	Lnum_schools Lnum_study Lrank1_study ///
								Lbaselinestudy Lbaselinefemale Lbaselinehisp Lbaselineblack Lbaselinefoodfred Lbaselinesped Lbaselinebilingual Lbaselinelep Lbaselinegifted ///
								Lbaseliness_math Lbaseliness_reading Lbaseliness_writing, vce(robust)
						test Z
						matrix balance[`row', `col'+2] = r(chi2)
						matrix balance[`row'+1, `col' +2] = r(p)
						matrix balance[`row'+2, `col'+2] = r(df)
						matrix balance[`row'+3, `col'+2] = e(N)
						
						suest	Pnum_schools Pnum_study  Prank1_study ///
								Pbaselinefemale Pbaselinehisp Pbaselineblack Pbaselinefoodfred Pbaselinesped Pbaselinelep Pbaselinebilingual Pbaselinegifted ///
								Pbaseliness_math Pbaseliness_reading Pbaseliness_writing, vce(robust)
						test Z
						matrix balance[`row', `col'+3] = r(chi2)
						matrix balance[`row'+1, `col' +3] = r(p)
						matrix balance[`row'+2, `col'+3] = r(df)
						matrix balance[`row'+3, `col'+3] = e(N)
						
						local c=0
						foreach i in 01 001{
							local c = `c' + 1
							suest	B`i'num_schools B`i'num_study  B`i'rank1_study ///
									B`i'baselinestudy B`i'baselinefemale B`i'baselinehisp B`i'baselineblack B`i'baselinefoodfred B`i'baselinesped B`i'baselinelep B`i'baselinebilingual B`i'baselinegifted ///
									B`i'baseliness_math B`i'baseliness_reading B`i'baseliness_writing, vce(robust)
							test Z
							matrix balance[`row', `col' +3 + `c'] = r(chi2)
							matrix balance[`row'+1, `col' +3 + `c'] = r(p)
							matrix balance[`row'+2, `col' +3 + `c'] = r(df)
							matrix balance[`row'+3, `col' +3 + `c'] = e(N)
							
							unique tag_r`i' if e(sample)
							matrix balance[`row'+4, `col' +3 + `c'] = r(sum)
						}
							suest	Snum_schools Snum_study Srank1_study ///
									Sbaselinestudy Sbaselinefemale Sbaselinehisp Sbaselineblack Sbaselinefoodfred Sbaselinesped Sbaselinelep Sbaselinebilingual Sbaselinegifted ///
									Sbaseliness_math Sbaseliness_reading Sbaseliness_writing, vce(robust)
							test Z
							matrix balance[`row',`col'+ 4 + `c'] = r(chi2)
							matrix balance[`row'+1, `col'+ 4 + `c'] = r(p)
							matrix balance[`row'+2, `col'+ 4 + `c'] = r(df)
							matrix balance[`row'+3, `col' +4 + `c'] = e(N)
							
							unique tag_r0 if e(sample)
							matrix balance[`row'+4, `col' +4 + `c'] = r(sum)

						local row = `row' + 5
					} /*Close cov group loop*/
					
				} /*Close f_cut loop*/
				
					**********************************************************************************
					
				*Outsheet
				preserve
					svmat balance
					keep var balance*
					keep if _n<=`row'+5
					ren balance1 z0
					ren balance2 ols_z1
					ren balance3 ols_z1_pscore
					ren balance4 ols_z1_poly
					ren balance5 ols_z1_bin01
					ren balance6 ols_z1_bin001
					ren balance7 ols_z1_psat
					
					outsheet using "`tables'`cut'_`sch_type'_`file'_years_stacked_balance.csv", comma replace
				restore
				
				} /*Close F test condition*/
			}	/*Close balance condition*/
		}	/*Close cut (in DPS at baseline or not) loop*/
		} /* Close sch_type loop */
	} /*Close file loop*/
	
	*****************************************
	***************** 2SLS ******************
	*****************************************
	
	if `pscore_twosls'==1{
		
		foreach file in simulation frequency formula   {
		foreach sch_type in study /*noex nonnoex*/ {
			foreach cut in inDPSbase /*allapps*/{
			
				u "`cleandata'analysis_stacked_`sch_type'_`file'.dta", clear
								
				if "`cut'"=="inDPSbase"{
					keep if baselinespyear==2012 | baselinespyear == 2013
				}
				else{
					di in red "Keep all obs"
				}
				
				keep if in_sample == 1
				
				gen var =""
				
				*By grade vars
				gen group = .
				
				local col = 1
				local row = 1
				
				
				
				foreach group in 410 /*45 68 48 910*/{
							if `group'==410{
								replace group=410
							}
							else if `group'==48{
								replace group=48 if inlist(appgrade,4,5,6,7,8)
							}
							else{
							replace group=45 if inlist(appgrade,4,5)
							replace group=68 if inlist(appgrade,6,7,8)
							replace group=910 if inlist(appgrade,9,10)
							}
	
							foreach t in math writing reading {
								foreach out in ss /*prof_up adv*/ {
						
								
								replace var="`out'_`t'_`group'" if _n==`row'
								replace var="`out'_`t'_`group'_se" if _n==`row'+1
								replace var="`out'_`t'_`group'_N" if _n==`row'+2
								
			
								qui summ `out'_`t' if group==`group' & in_zero1_pscore==1
								
								if r(max)==0 & r(min)==0{
									di in red "No observations for `group'-level 2SLS estimates"
								}
							
								else{
							
									qui summ `out'_`t' if Z == 0 & group==`group'
									matrix twosls[`row',`col'] = r(mean)
									matrix twosls[`row'+2,`col'] = r(N)
								
									/*OLS - no control for pscore*/
									qui reg `out'_`t' D `cov' `cov2' `cov_grade' i.baselinespyear if group==`group', vce(robust)
									matrix twosls[`row',`col'+1]   = _b[D]
									matrix twosls[`row'+1,`col'+1] = _se[D]
									matrix twosls[`row'+2,`col'+1] = e(N)
										

										/*OLS conditional on fully saturated pscore sample */
										qui reg `out'_`t' D `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & in_good_cell_r0==1, vce(robust)
										matrix twosls[`row',`col'+2]   = _b[D]
										matrix twosls[`row'+1,`col'+2] = _se[D]
										matrix twosls[`row'+2,`col'+2] = e(N)
									
										
										/*OLS conditional controlling for fully saturated pscores*/
										qui reg `out'_`t' D i.ind_r0 `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & in_good_cell_r0==1, vce(robust)
										matrix twosls[`row',`col'+3]   = _b[D]
										matrix twosls[`row'+1,`col'+3] = _se[D]
										matrix twosls[`row'+2,`col'+3] = e(N)
									
									
									/*OLS conditional controlling for pscores rounded to the hundredths*/
									qui reg `out'_`t' D i.ind_r01 `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & in_good_cell_r01==1, vce(robust)
									matrix twosls[`row',`col'+4]   = _b[D]
									matrix twosls[`row'+1,`col'+4] = _se[D]
									matrix twosls[`row'+2,`col'+4] = e(N)
									
									/*OLS conditional controlling for pscores rounded to the thousandths*/
									qui reg `out'_`t' D i.ind_r001 `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & in_good_cell_r001==1, vce(robust)
									matrix twosls[`row',`col'+5]   = _b[D]
									matrix twosls[`row'+1,`col'+5] = _se[D]
									matrix twosls[`row'+2,`col'+5] = e(N)
										
									foreach risk in  r01 r001 r0 {
						
										local col = `col' + 2
										
										local controls "i.ind_`risk' `cov' `cov2' `cov_grade'"		
											
										/*2SLS - outcome on treatment instrumented by instrument*/
										ivreg2 `out'_`t' (D = Z) `controls' i.baselinespyear if group==`group' & in_good_cell_`risk'==1 , savefirst savefprefix(first) partial(`controls') robust
										ivreg2 `out'_`t' (D = Z) `controls' i.baselinespyear if group==`group' & in_good_cell_org_`risk'==1 , savefirst savefprefix(first) partial(`controls') robust
										
										pause check reg 

										
										
										*2SLS
										matrix twosls[`row',`col'+5]   = _b[D]
										matrix twosls[`row'+1,`col'+5] = _se[D]
										matrix twosls[`row'+2,`col'+5] = e(N)
										
										estimates restore firstD
										
										*First stage
										matrix twosls[`row',`col'+4]   = _b[Z]
										matrix twosls[`row'+1,`col'+4] = _se[Z]
											
											
										*If last loop, do these
										if "`risk'"=="r0"{
												
												 
											*NAIVE 2SLS - no riskset controlsq
												
												*2SLS naive
												ivreg2 `out'_`t' (D = Z) `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & !inlist(`risk', 0, 1) , savefirst savefprefix(first) partial(`cov' `cov2') robust

												*2SLS
												matrix twosls[`row',`col'+7]   = _b[D]
												matrix twosls[`row'+1,`col'+7] = _se[D]
												matrix twosls[`row'+2,`col'+7] = e(N)
												
												estimates restore firstD
										
												*First stage
												matrix twosls[`row',`col'+6]   = _b[Z]
												matrix twosls[`row'+1,`col'+6] = _se[Z]
													
												*2SLS naive conditional on r0 sample
												ivreg2 `out'_`t' (D = Z) `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & in_good_cell_r0==1 & !inlist(`risk', 0, 1) , savefirst savefprefix(first) partial(`cov' `cov2') robust

												*2SLS
												matrix twosls[`row',`col'+9]   = _b[D]
												matrix twosls[`row'+1,`col'+9] = _se[D]
												matrix twosls[`row'+2,`col'+9] = e(N)
												
												estimates restore firstD
										
												*First stage
												matrix twosls[`row',`col'+8]   = _b[Z]
												matrix twosls[`row'+1,`col'+8] = _se[Z]													
											
											*2SLS with no cov controls								
												*2SLS
												ivreg2 `out'_`t' (D = Z) i.baselinespyear i.ind_`risk' if group==`group' & in_good_cell_r0==1 & !inlist(`risk', 0, 1) , savefirst savefprefix(first) partial(i.ind_`risk') robust
													
												*2SLS
												matrix twosls[`row',`col'+11]   = _b[D]
												matrix twosls[`row'+1,`col'+11] = _se[D]
												matrix twosls[`row'+2,`col'+11] = e(N)
												
												estimates restore firstD
										
												*First stage
												matrix twosls[`row',`col'+10]   = _b[Z]
												matrix twosls[`row'+1,`col'+10] = _se[Z]

										} /*close if condition*/
									
									
									} /*risk loop*/
							
							} /*close condition on number of observations*/
							
							local row = `row' + 3
							local col = 1
						} 	 /*out loop*/
						
						local ++row
					}	/* t loop*/
					
				}	/*group loop*/
				
				*ivreg2 ss_math (D = Z) baselinespyear i.ind_r0 if in_good_cell_r0==1, savefirst savefprefix(first) partial(i.ind_r0) cluster(maskedid) 				

				
				preserve
					svmat twosls
					keep var twosls*
					keep if _n<=`row'
					ren twosls1 z0
					ren twosls2 OLS
					ren twosls3 OLS_psample
					ren twosls4 OLS_psat
					ren twosls5 OLS_r01
					ren twosls6 OLS_r001
					
					ren twosls7  FS_r01
					ren twosls8  twoSLS_r01
					ren twosls9  FS_r001
					ren twosls10 twosls_r001
					
					ren twosls11 FS_psat
					ren twosls12 twoSLS_psat
					
					ren twosls13 FS_naive
					ren twosls14 twoSLS_naive
					
					ren twosls15 FS_naive_psample
					ren twosls16 twoSLS_naive_psample
					
					ren twosls17 FS_nocovs
					ren twosls18 twoSLS_nocovs

					outsheet using "`tables'`cut'_`sch_type'_`file'_years_stacked_twosls.csv", comma replace
					
				restore
			clear	

			} /*Close 2SLS condition*/
		}	/*Close cut (in DPS at baseline or not) loop*/
	} /* Close sch_type loop */
	} /*Close file loop*/

	
	
	*****************************************
	***************** 2SLS (sample = first-choice is charter) ******************
	*****************************************
	
	if `pscore_twosls_fcsample'==1{
		
		foreach file in simulation frequency formula{
		foreach sch_type in study /*noex nonnoex*/ {
			foreach cut in inDPSbase /*allapps*/{
			
				u "`cleandata'analysis_stacked_`sch_type'_`file'.dta", clear
								
				if "`cut'"=="inDPSbase"{
					keep if baselinespyear==2012 | baselinespyear == 2013
				}
				else{
					di in red "Keep all obs"
				}
				
				keep if in_sample == 1
				
				*Sample restricted to those who rank charter first
				keep if rank1_study_os == 1
				
				gen var =""
				
				*By grade vars
				gen group = .
				
				local col = 1
				local row = 1
			
				foreach group in 410 /*45 68 48 910*/{
							if `group'==410{
								replace group=410
							}
							else if `group'==48{
								replace group=48 if inlist(appgrade,4,5,6,7,8)
							}
							else{
							replace group=45 if inlist(appgrade,4,5)
							replace group=68 if inlist(appgrade,6,7,8)
							replace group=910 if inlist(appgrade,9,10)
							}
	
							foreach t in math writing reading {
								foreach out in ss /*prof_up adv*/ {
						
								
								replace var="`out'_`t'_`group'" if _n==`row'
								replace var="`out'_`t'_`group'_se" if _n==`row'+1
								replace var="`out'_`t'_`group'_N" if _n==`row'+2
								
			
								qui summ `out'_`t' if group==`group' & in_zero1_pscore==1
								
								if r(max)==0 & r(min)==0{
									di in red "No observations for `group'-level 2SLS estimates"
								}
							
								else{
							
									qui summ `out'_`t' if Z == 0 & group==`group'
									matrix twosls_fcsample[`row',`col'] = r(mean)
									matrix twosls_fcsample[`row'+2,`col'] = r(N)
									
									/*OLS - no control for pscore*/
									qui reg `out'_`t' D `cov' `cov2' `cov_grade' baselinespyear if group==`group', vce(cluster maskedid)
									matrix twosls_fcsample[`row',`col'+1]   = _b[D]
									matrix twosls_fcsample[`row'+1,`col'+1] = _se[D]
									matrix twosls_fcsample[`row'+2,`col'+1] = e(N)
									
									/*OLS conditional on fully saturated pscore sample*/
									qui reg `out'_`t' D `cov' `cov2' `cov_grade' baselinespyear if group==`group' & in_good_cell_r0==1, vce(cluster maskedid)
									matrix twosls_fcsample[`row',`col'+2]   = _b[D]
									matrix twosls_fcsample[`row'+1,`col'+2] = _se[D]
									matrix twosls_fcsample[`row'+2,`col'+2] = e(N)
									
								
									/*OLS conditional controlling for fully saturated pscores*/
									qui reg `out'_`t' D i.ind_r0 `cov' `cov2' `cov_grade' baselinespyear if group==`group' & in_good_cell_r0==1, vce(cluster maskedid)
									matrix twosls_fcsample[`row',`col'+3]   = _b[D]
									matrix twosls_fcsample[`row'+1,`col'+3] = _se[D]
									matrix twosls_fcsample[`row'+2,`col'+3] = e(N)
									
									
									foreach risk in r001 r01 r0 {
						
										local col = `col' + 2
										
										local controls "`risk'_* `cov' `cov2' `cov_grade'"		
										
										if ("`file'" != "simulation" | "`risk'" != "r0"  | ("`file'" == "simulation" & "`risk'" == "r0" & `sim_saturated' == 1)) {
											
											/*2SLS - outcome on treatment instrumented by instrument*/
											ivreg2 `out'_`t' (D = Z) `controls' baselinespyear if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls') cluster(maskedid) 
												
											*2SLS
											matrix twosls_fcsample[`row',`col'+4]   = _b[D]
											matrix twosls_fcsample[`row'+1,`col'+4] = _se[D]
											matrix twosls_fcsample[`row'+2,`col'+4] = e(N)
											
											estimates restore firstD
											
											*First stage
											matrix twosls_fcsample[`row',`col'+3]   = _b[Z]
											matrix twosls_fcsample[`row'+1,`col'+3] = _se[Z]
												
												
											*If last loop, do these
											if "`risk'"=="r0"{
												
												*NAIVE 2SLS - no riskset controls
													*2SLS naive
													ivreg2 `out'_`t' (D = Z) `cov' `cov2' `cov_grade' baselinespyear if group==`group', savefirst savefprefix(first) partial(`cov' `cov2') cluster(maskedid) 

														*2SLS
														matrix twosls_fcsample[`row',`col'+6]   = _b[D]
														matrix twosls_fcsample[`row'+1,`col'+6] = _se[D]
														matrix twosls_fcsample[`row'+2,`col'+6] = e(N)
														
														estimates restore firstD
												
														*First stage
														matrix twosls_fcsample[`row',`col'+5]   = _b[Z]
														matrix twosls_fcsample[`row'+1,`col'+5] = _se[Z]
														
													*2SLS naive conditional on r0 sample
													ivreg2 `out'_`t' (D = Z) `cov' `cov2' `cov_grade' baselinespyear if group==`group' & in_good_cell_r0==1, savefirst savefprefix(first) partial(`cov' `cov2') cluster(maskedid) 

														*2SLS
														matrix twosls_fcsample[`row',`col'+8]   = _b[D]
														matrix twosls_fcsample[`row'+1,`col'+8] = _se[D]
														matrix twosls_fcsample[`row'+2,`col'+8] = e(N)
														
														estimates restore firstD
												
														*First stage
														matrix twosls_fcsample[`row',`col'+7]   = _b[Z]
														matrix twosls_fcsample[`row'+1,`col'+7] = _se[Z]													
												
												*2SLS with no cov controls								
													*2SLS
													ivreg2 `out'_`t' (D = Z) baselinespyear `risk'_* if group==`group' & in_good_cell_r0==1, savefirst savefprefix(first) partial(`risk'_*) cluster(maskedid) 
														
														*2SLS
														matrix twosls_fcsample[`row',`col'+10]   = _b[D]
														matrix twosls_fcsample[`row'+1,`col'+10] = _se[D]
														matrix twosls_fcsample[`row'+2,`col'+10] = e(N)
														
														estimates restore firstD
												
														*First stage
														matrix twosls_fcsample[`row',`col'+9]   = _b[Z]
														matrix twosls_fcsample[`row'+1,`col'+9] = _se[Z]
											} /*close if condition*/
										} /* close if simsaturated not 1 exclusion */
									} /*risk loop*/
							
							} /*close condition on number of observations*/
							
							local row = `row' + 3
							local col = 1
						} 	 /*out loop*/
						
						local ++row
					}	/* t loop*/
					
				}	/*group loop*/
	
			
				preserve
					svmat twosls_fcsample
					keep var twosls_fcsample*
					keep if _n<=`row'
					ren twosls_fcsample1 z0
					ren twosls_fcsample2 OLS
					ren twosls_fcsample3 OLS_psample
					ren twosls_fcsample4 OLS_psat
					
					ren twosls_fcsample6 FS_r01
					ren twosls_fcsample7 twoSLS_r01
					
					ren twosls_fcsample8 FS_psat
					ren twosls_fcsample9 twoSLS_psat
					
					ren twosls_fcsample10 FS_naive
					ren twosls_fcsample11 twoSLS_naive
					
					ren twosls_fcsample12 FS_naive_psample
					ren twosls_fcsample13 twoSLS_naive_psample
					
					ren twosls_fcsample14 FS_nocovs
					ren twosls_fcsample15 twoSLS_nocovs

					outsheet using "`tables'`cut'_`sch_type'_`file'_years_stacked_twosls_fc_charter.csv", comma replace
				restore
			clear	

			} /*Close 2SLS condition*/
		}	/*Close cut (in DPS at baseline or not) loop*/
	} /* Close sch_type loop */
	} /*Close file loop*/	
	

	
	*****************************************
	***************** 2SLS (no covariates) ******************
	*****************************************
	
	if `nocov_twosls'==1{
		
		foreach file in  simulation /*frequency formula */ {

			u "`cleandata'analysis_stacked_study_`file'.dta", clear
							
			keep if in_sample == 1
			keep if baselinespyear == 2012 | baselinespyear == 2013
		
			gen var =""
			gen Z_HS = Z*HS
			
			*By grade vars
			gen group = .
			
			local col = 1
			local row = 1
					
		
			foreach group in 410 /*48 910*/{
						if `group'==410{
							replace group=410
						}
						else if `group'==48{
							replace group=48 if inlist(appgrade,4,5,6,7,8)
						}
						else{
							replace group=910 if inlist(appgrade,9,10)
						}

						foreach t in math writing reading {
							foreach out in ss /*prof_up adv*/ {
					
							
							replace var="`out'_`t'_`group'" if _n==`row'
							replace var="`out'_`t'_`group'_se" if _n==`row'+1
							replace var="`out'_`t'_`group'_N" if _n==`row'+2
							
		
							qui summ `out'_`t' if group==`group' & in_zero1_pscore==1
							
							if r(max)==0 & r(min)==0{
								di in red "No observations for `group'-level 2SLS estimates"
							}
						
							else{
								
								qui: summ `out'_`t' if Z == 0 & group==`group'
								matrix twosls_nocov[`row',`col'] = r(mean)
								matrix twosls_nocov[`row'+2,`col'] = r(N)
								
								qui summ `out'_`t' if Z == 0 & group==`group'
								matrix twosls_scorecov[`row',`col'] = r(mean)
								matrix twosls_scorecov[`row'+2,`col'] = r(N)
								
									/*OLS - no control for pscore nor covariates (but including baseline score covariates) */
									qui reg `out'_`t' D g_* i.baselinespyear `cov2' if group==`group', vce(robust)
									matrix twosls_scorecov[`row',`col'+1]   = _b[D]
									matrix twosls_scorecov[`row'+1,`col'+1] = _se[D]
									matrix twosls_scorecov[`row'+2,`col'+1] = e(N)
									
									/*OLS - no control for pscore nor covariates */
									qui reg `out'_`t' D g_* i.baselinespyear if group==`group' & baseliness_math != . & baseliness_reading != . & baseliness_writing != ., vce(robust)
									matrix twosls_nocov[`row',`col'+1]   = _b[D]
									matrix twosls_nocov[`row'+1,`col'+1] = _se[D]
									matrix twosls_nocov[`row'+2,`col'+1] = e(N)

									/*OLS conditional controlling for fully saturated pscores (only including baseline score covariates) */
									qui reg `out'_`t' D i.ind_r0 g_* i.baselinespyear `cov2' if group==`group' & in_good_cell_r0==1, vce(robust)
									matrix twosls_scorecov[`row',`col'+2]   = _b[D]
									matrix twosls_scorecov[`row'+1,`col'+2] = _se[D]
									matrix twosls_scorecov[`row'+2,`col'+2] = e(N)
									
									/*OLS conditional controlling for fully saturated pscores*/
									qui reg `out'_`t' D i.ind_r0 g_* i.baselinespyear if group==`group' & in_good_cell_r0==1 & baseliness_math != . & baseliness_reading != . & baseliness_writing != ., vce(robust)
									matrix twosls_nocov[`row',`col'+2]   = _b[D]
									matrix twosls_nocov[`row'+1,`col'+2] = _se[D]
									matrix twosls_nocov[`row'+2,`col'+2] = e(N)
								
							
								foreach instrument_type in ordinary /*over_id */ {
								
									if "`instrument_type'" == "ordinary" {
									local instrument  "Z"
									}
									else {
									local instrument "Z Z_HS"
									}
									
									
										*2SLS with neither score controls nor covariates (but including baseline score covs)							
											*2SLS
											ivreg2 `out'_`t' (D = `instrument') g_* i.baselinespyear `cov2' if group==`group', savefirst savefprefix(first) robust
												 
											*2SLS
											matrix twosls_scorecov[`row',`col'+4]   = _b[D]
											matrix twosls_scorecov[`row'+1,`col'+4] = _se[D]
											matrix twosls_scorecov[`row'+2,`col'+4] = e(N)
											
											estimates restore firstD
									
											*First stage
											matrix twosls_scorecov[`row',`col'+3]   = _b[Z]
											matrix twosls_scorecov[`row'+1,`col'+3] = _se[Z]
											
											*estimates restore reduced`out'_`t'
											
											*reduced form
											matrix twosls_scorecov[`row',`col'+5]   = 0
											matrix twosls_scorecov[`row'+1,`col'+5] = 0
										
										*2SLS with neither score controls nor covariates							
											*2SLS
											ivreg2 `out'_`t' (D = `instrument') g_* i.baselinespyear if group==`group' & baseliness_math != . & baseliness_reading != . & baseliness_writing != ., savefirst savefprefix(first) robust
												
											*2SLS
											matrix twosls_nocov[`row',`col'+4]   = _b[D]
											matrix twosls_nocov[`row'+ 1,`col'+4] = _se[D]
											matrix twosls_nocov[`row'+ 2, `col'+4] = e(N)
											
											estimates restore firstD
									
											*First stage
											matrix twosls_nocov[`row',`col'+3]   = _b[Z]
											matrix twosls_nocov[`row'+ 1,`col'+3] = _se[Z]
											
											*estimates restore reduced`out'_`t'
											
											*reduced form
											matrix twosls_nocov[`row' ,`col'+5]   = 0
											matrix twosls_nocov[`row'+ 1,`col'+5] = 0
											
											di "row `row'"
											
							
																					
										***Fully Saturated 2sls
											
										*2SLS with no cov controls (but with grade and baseline score covs) rounded to the hundredths
											*2SLS
											ivreg2 `out'_`t' (D = `instrument') i.ind_r0 g_* i.baselinespyear `cov2' if group==`group' & in_good_cell_r0==1, savefirst savefprefix(first) partial(i.ind_r0 g_* `cov2') robust 
												
											*2SLS
											matrix twosls_scorecov[`row',`col'+7]   = _b[D]
											matrix twosls_scorecov[`row'+1,`col'+7] = _se[D]
											matrix twosls_scorecov[`row'+2,`col'+7] = e(N)
											
											estimates restore firstD
									
											*First stage
											matrix twosls_scorecov[`row',`col'+6]   = _b[Z]
											matrix twosls_scorecov[`row'+1,`col'+6] = _se[Z]
											
											*estimates restore reduced`out'_`t'
											
											*reduced form
											matrix twosls_scorecov[`row',`col'+8]   = 0
											matrix twosls_scorecov[`row'+1,`col'+8] = 0
									
										
										
										*2SLS with no cov controls (but with grade) rounded to the hundredths
											*2SLS
											ivreg2 `out'_`t' (D = `instrument') i.ind_r0 g_* i.baselinespyear if group==`group' & in_good_cell_r0==1 & baseliness_math != . & baseliness_reading != . & baseliness_writing != ., savefirst savefprefix(first)  partial(i.ind_r0 g_*)  robust
											
											*2SLS
											matrix twosls_nocov[`row',`col'+7]   = _b[D]
											matrix twosls_nocov[`row'+1,`col'+7] = _se[D]
											matrix twosls_nocov[`row'+2,`col'+7] = e(N)
											
											estimates restore firstD
									
											*First stage
											matrix twosls_nocov[`row',`col'+6]   = _b[Z]
											matrix twosls_nocov[`row'+1,`col'+6] = _se[Z]
											
											*estimates restore reduced`out'_`t'
											
											*reduced form
											matrix twosls_nocov[`row',`col'+8]   = 0
											matrix twosls_nocov[`row'+1,`col'+8] = 0
											
									
									local col = 11
									
									foreach risk in r01 r001{
										*2SLS with no cov controls (but with grade and baseline score covs)
											*2SLS
											ivreg2 `out'_`t' (D = `instrument') i.ind_`risk' g_* i.baselinespyear `cov2' if group==`group' & in_good_cell_`risk'==1,  savefirst savefprefix(first) partial(i.ind_`risk' g_* `cov2') robust
												
												*2SLS
												matrix twosls_scorecov[`row',`col']   = _b[D]
												matrix twosls_scorecov[`row'+1,`col'] = _se[D]
												matrix twosls_scorecov[`row'+2,`col'] = e(N)
												
												estimates restore firstD
										
												*First stage
												matrix twosls_scorecov[`row',`col'-1]   = _b[Z]
												matrix twosls_scorecov[`row'+1,`col'-1] = _se[Z]
												
												*estimates restore reduced`out'_`t'
												
												*reduced form
												matrix twosls_scorecov[`row',`col'+1]   = 0
												matrix twosls_scorecov[`row'+1,`col'+1] = 0
											
										
											
										*2SLS with no cov controls (but with grade)
											*2SLS
											ivreg2 `out'_`t' (D = `instrument') i.ind_`risk' g_* i.baselinespyear if group==`group' & in_good_cell_`risk'==1 & baseliness_math != . & baseliness_reading != . & baseliness_writing != ., savefirst savefprefix(first) partial(i.ind_`risk' g_*) robust 
												
												*2SLS
												matrix twosls_nocov[`row',`col']   = _b[D]
												matrix twosls_nocov[`row'+1,`col'] = _se[D]
												matrix twosls_nocov[`row'+2,`col'] = e(N)
												
												estimates restore firstD
										
												*First stage
												matrix twosls_nocov[`row',`col'-1]   = _b[Z]
												matrix twosls_nocov[`row'+1,`col'-1] = _se[Z]
												
												*estimates restore reduced`out'_`t'
												
												*reduced form
												matrix twosls_nocov[`row',`col'+1]   = 0
												matrix twosls_nocov[`row'+1,`col'+1] = 0
											
											local col = `col' + 3
					
								} /*risk loop*/
								
							} /* instrument loop */
						} /*close condition on number of observations*/
						
						local row = `row' + 3
						local col = 1
					} 	 /*out loop*/
					
					local ++row
				}	/* t loop*/
				
			}	/*group loop*/

		
			preserve
				svmat twosls_nocov
				keep var twosls_nocov*
				keep if _n<=`row'
				ren twosls_nocov1 z0
				ren twosls_nocov2 OLS
				ren twosls_nocov3 OLS_psat
				
				ren twosls_nocov4 FS_nocov
				ren twosls_nocov5 twoSLS_nocov
				ren twosls_nocov6 RF_nocov
				
				ren twosls_nocov7 FS_psat_nocov
				ren twosls_nocov8 twoSLS_psat_nocov
				ren twosls_nocov9 RF_psat_nocov
				
				ren twosls_nocov10 FS_r01_nocov
				ren twosls_nocov11 twoSLS_r01_nocov
				ren twosls_nocov12 RF_r01_nocov
				
				ren twosls_nocov13 FS_r001_nocov
				ren twosls_nocov14 twoSLS_r001_nocov
				ren twosls_nocov15 RF_r001_nocov
				
				/*
				ren twosls_nocov16 FS_nocov_overid
				ren twosls_nocov17 twoSLS_nocov_overid
				ren twosls_nocov18 RF_nocov_overid
				
				ren twosls_nocov19 FS_psat_nocov_overid
				ren twosls_nocov20 twoSLS_psat_nocov_overid
				ren twosls_nocov21 RF_psat_nocov_overid
				
				ren twosls_nocov22 FS_r01_nocov_overid
				ren twosls_nocov23 twoSLS_r01_nocov_overid
				ren twosls_nocov24 RF_r01_nocov_overid
					*/			
				
				outsheet using "`tables'inDPSbase_`file'_twosls_years_stacked_nocovs.csv", comma replace
			restore
		
				
			preserve
				svmat twosls_scorecov
				keep var twosls_scorecov*
				keep if _n<=`row'
				
				ren twosls_scorecov1 z0
				ren twosls_scorecov2 OLS
				ren twosls_scorecov3 OLS_psat
				
				ren twosls_scorecov4 FS_nocov
				ren twosls_scorecov5 twosls_scorecov
				ren twosls_scorecov6 RF_nocov
				
				ren twosls_scorecov7 FS_psat_nocov
				ren twosls_scorecov8 twoSLS_psat_nocov
				ren twosls_scorecov9 RF_psat_nocov
				
				ren twosls_scorecov10 FS_r01_nocov
				ren twosls_scorecov11 twoSLS_r01_nocov
				ren twosls_scorecov12 RF_r01_nocov
				
				ren twosls_scorecov13 FS_r001_nocov
				ren twosls_scorecov14 twoSLS_r001_nocov
				ren twosls_scorecov15 RF_r001_nocov
				
				
				outsheet using "`tables'inDPSbase_`file'_twosls_years_stacked_scorecov.csv", comma replace
			restore
			
		
		clear	

			} /*Close 2SLS condition*/
	} /*Close file loop*/	
	
	
	
	
	***************************************************
	***************** Subgroups 2SLS ******************
	***************************************************
	
	if `pscore_subtwosls'==1{
	
		foreach file in /*simulation*/ frequency formula{		
			foreach cut in inDPSbase /*allapps*/{
			
				u "`cleandata'analysis_stacked_`file'.dta", clear
				
				if "`cut'"=="inDPSbase"{
					keep if baselinespyear==2012 | baselinespyear == 2013
				}
				else{
					di in red "Keep all obs"
				}
				
				local ability_math 		"baselinepp_up_math baselineprof_up_math baselineadv_math"
				local ability_reading 	"baselinepp_up_reading baselineprof_up_reading baselineadv_reading"
				local ability_writing 	"baselinepp_up_writing baselineprof_up_writing baselineadv_writing"
				
				keep if in_samp == 1

				gen var =""
				
				local col = 1
				local row = 1
					
					foreach group in `ability_math' `ability_reading' `ability_writing' `cov'{
						gen group = (`group' == 1)
						
							foreach t in math writing reading{
								foreach out in ss /*prof_up adv*/ {
						
								replace var="`out'_`t'_`group'" if _n==`row'
								replace var="`out'_`t'_`group'_se" if _n==`row'+1
								replace var="`out'_`t'_`group'_N" if _n==`row'+2
									
								qui summ `out'_`t' if group==1 & in_zero1_pscore==1
								
								di in red " `r(N)' obs in `group'"
								
								/*Regressions are not running with less than 3 obs*/
								if ( (r(max)==0 & r(min)==0)) | r(N) < 40{
									di in red "No observations for `group'-level 2SLS estimates"
								}
						
								else{
							
									qui summ `out'_`t' if Z == 0 & group==1
									matrix subtwosls[`row',`col'] = r(mean)
									matrix subtwosls[`row'+2,`col'] = r(N)
									
										/*OLS - no control for pscore*/
										qui reg `out'_`t' D `cov' `cov2' if group==1, r
										matrix subtwosls[`row',`col'+1]   = _b[D]
										matrix subtwosls[`row'+1,`col'+1] = _se[D]
										matrix subtwosls[`row'+2,`col'+1] = e(N)
										
										/*OLS conditional on fully saturated pscore*/
										qui reg `out'_`t' D r0_* `cov' `cov2' if group==1 & in_good_cell_r0==1, r
										matrix subtwosls[`row',`col'+2]   = _b[D]
										matrix subtwosls[`row'+1,`col'+2] = _se[D]
										matrix subtwosls[`row'+2,`col'+2] = e(N)
										
									foreach risk in r01 r0{
						
										local col = `col' + 2
										
										local controls "`risk'_* `cov' `cov2'"		
										
											
										/*2SLS - outcome on treatment instrumented by instrument*/
										ivreg2 `out'_`t' (D = Z) `controls' if group==1 & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls') r 
											
											*2SLS
											matrix subtwosls[`row',`col'+4]   = _b[D]
											matrix subtwosls[`row'+1,`col'+4] = _se[D]
											matrix subtwosls[`row'+2,`col'+4] = e(N)
											
											estimates restore firstD
											
											*First stage
											matrix subtwosls[`row',`col'+3]   = _b[Z]
											matrix subtwosls[`row'+1,`col'+3] = _se[Z]
											
										*If last loop, do these
										if "`risk'"=="r0"{
											
											*NAIVE 2SLS - no riskset controls
												*2SLS naive
												ivreg2 `out'_`t' (D = Z) `cov' `cov2' if group==1, savefirst savefprefix(first) partial(`cov' `cov2') r

													*2SLS
													matrix subtwosls[`row',`col'+6]   = _b[D]
													matrix subtwosls[`row'+1,`col'+6] = _se[D]
													matrix subtwosls[`row'+2,`col'+6] = e(N)
													
													estimates restore firstD
											
													*First stage
													matrix subtwosls[`row',`col'+5]   = _b[Z]
													matrix subtwosls[`row'+1,`col'+5] = _se[Z]
													
												*2SLS naive conditional on pscore sample
												ivreg2 `out'_`t' (D = Z) `cov' `cov2' if group==1 & in_good_cell_r0==1, savefirst savefprefix(first) partial(`cov' `cov2') r

													*2SLS
													matrix subtwosls[`row',`col'+8]   = _b[D]
													matrix subtwosls[`row'+1,`col'+8] = _se[D]
													matrix subtwosls[`row'+2,`col'+8] = e(N)
													
													estimates restore firstD
											
													*First stage
													matrix subtwosls[`row',`col'+7]   = _b[Z]
													matrix subtwosls[`row'+1,`col'+7] = _se[Z]	
											
											*2SLS with no cov controls								
												*2SLS
												ivreg2 `out'_`t' (D = Z) `risk'_* if group==1 & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`risk'_*) r 
													
													*2SLS
													matrix subtwosls[`row',`col'+10]   = _b[D]
													matrix subtwosls[`row'+1,`col'+10] = _se[D]
													matrix subtwosls[`row'+2,`col'+10] = e(N)
													
													estimates restore firstD
											
													*First stage
													matrix subtwosls[`row',`col'+9]   = _b[Z]
													matrix subtwosls[`row'+1,`col'+9] = _se[Z]
										} /*close if condition*/
									} /*risk loop*/
							} /*close condition on number of observations*/
							
							local row = `row' + 3
							local col = 1
						} 	 /*out loop*/
						
						local ++row
					}	/* t loop*/
				drop group
				}	/*group loop*/
				
				preserve
					svmat subtwosls
					keep var subtwosls*
					keep if _n<=`row'
					ren subtwosls1 z0
					ren subtwosls2 OLS
					ren subtwosls3 OLS_psample
					
					ren subtwosls6 FS_r01
					ren subtwosls7 twoSLS_r01
					
					ren subtwosls8 FS_psat
					ren subtwosls9 twoSLS_psat
					
					ren subtwosls10 FS_naive
					ren subtwosls11 twoSLS_naive
					
					ren subtwosls12 FS_naive_psample
					ren subtwosls13 twoSLS_naive_psample
					
					ren subtwosls14 FS_nocovs
					ren subtwosls15 twoSLS_nocovs

					outsheet using "`tables'`cut'_`file'_year_2_subtwosls.csv", comma replace
				restore
			clear	
			}	/*Close 2sls subgroup loop*/
		} /*Close cut (in DPS at baseline or not) loop*/
	} /*Close file loop*/

	

	

	***************************************************************************
	***************** Qual instruments stacking ******************
	***************************************************************************

	if `qual_stacking' == 1{
	
		foreach qnum in 20 100{
			
			foreach type in standard naive{
		
				u "`cleandata'mdrd_denver_qual`type'_lt`qnum' `qualdate_year1'.dta", clear
			
				rename riskset_lt`qnum' riskset_lt`qnum'_year1
				keep if baselinespyear == 2012
			
				append using "`cleandata'mdrd_denver_qual`type'_lt`qnum'_year_2 `qualdate_year2'.dta", gen(year2)
				drop if year2 == 1 & baselinespyear != 2013
			
				rename riskset_lt`qnum' riskset_lt`qnum'_year2
				
				egen riskset_lt`qnum'_year1_max = max(riskset_lt`qnum'_year1)
				
				gen riskset_lt`qnum' = riskset_lt`qnum'_year1 if riskset_lt`qnum'_year2 == .
				replace riskset_lt`qnum' = riskset_lt`qnum'_year2 + riskset_lt`qnum'_year1_max if riskset_lt`qnum'_year1 == .
				
				save "`cleandata'mdrd_denver_qual`type'_lt`qnum'_stacked `qualdate'.dta", replace		
			}
		}
	}	
	
	
	
	***************************************************************************
	***************** Qual instruments balance and attrition ******************
	***************************************************************************
	
	if `qual_balance'==1{

			foreach choice in 1 all{
					
				if "`choice'"=="1"{
					local qnum = 20
					local expand = 14
					local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12 Z_13 Z_14"
					local rank1 ""
					local Srank1 ""
					local Frank1 ""
				}
				else if "`choice'"=="all"{
					local qnum = 100
					local expand = 15
					local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12 Z_13 Z_14 Z_15"
					local rank1 "rank1_study"
					local Srank1 "Srank1_study"
					local Frank1 "Frank1_study"
				}
			
				foreach type in standard naive{
				
					local qualdata "`cleandata'mdrd_denver_qual`type'_lt`qnum'_stacked `qualdate'"
				
					u "`qualdata'", clear
					
						*Analysis is at student level
						drop appschoolnum
						
						duplicates drop maskedid spyear, force
						
						destring baselinegrade, replace
						gen appgrade_tmp = appgrade
						destring appgrade_tmp, replace
						drop appgrade
						rename appgrade_tmp appgrade
						
						gen next = (schoolnum!=.)
						bys maskedid spyear: egen enrolls_next = max(next)
						drop next
						
						bys maskedid spyear: egen enroll_any_study = max(study)
						
						** Keep if in Denver at baseline and in sample
						keep if in_samp == 1 & (baselinespyear == 2012 | baselinespyear == 2013)

						*Flag for whether you are subject to randomization in frequency01
						preserve
							u "`cleandata'analysis_stacked_study_frequency", clear
							keep if in_sample & pscore1!=0 & pscore1!=1

							bys r0: egen sd_z= sd(Z)
							keep if sd_z>0 & sd_z!=.
							
							count
							keep maskedid spyear
							tempfile rand
							sa `rand'
						restore
						
						merge 1:1 maskedid spyear using `rand'
						gen psample = (_merge==3)
						drop _merge
						
						ren qual_lt`qnum' Z
						ren enroll_any_study D
						
						tab riskset_lt`qnum', gen(r_)
						ren riskset_lt`qnum' r
						
						bys r: egen obs_cell_r = count(maskedid)
					
						**********************************************************************************
						******************** Find cells with variation in instrument**********************
						**********************************************************************************
						
						*NOTE that qualification instruments file was already constructed such that obs are
						*in cells with variation in instrument
						
						/* 	Flag observations that actually make it into balance (i.e. obs in cells with
							variation in instrument). These flags are used for all types 
							of F-tests, conducted below.
							
							We generate these flags here so that no re-sorting of the data (done in bys x egen:)
							is done between the balanceregs and F tests, which affects how the variable
							"var", which stores the names of the variables corresponding to each balance
							coefficient, appears in the final dataset once we svmat the balance output.
						*/
						
							foreach x in r{
									
								*Variation in instruments for each cell
									bys `x': egen sd_z_`x'	= 	sd(Z)
								
								*Good cell
									gen in_good_cell_`x' = (sd_z_`x'>0 & !missing(sd_z))
								
								drop sd_z_* 
							}
							

							

							local row = 1
							local col = 1
							gen var =""
							
							replace var = "Riskset points of support" if _n==1
							
							unique r if in_good_cell_r==1
							matrix qualbalance[1,3] = r(sum)
							
							unique r if in_good_cell_r==1 & psample==1
							matrix qualbalance[1,4] = r(sum)
							
							local row = `row' + 1
							
							foreach var in `appcov' `cov' `cov2' `attrition_vars'{
								replace var="`var'" if _n==`row'
								replace var="`var'_se" if _n==`row'+1
								replace var="`var'_N" if _n==`row'+2
							
									
								*All non-offered applicants
								qui summ `var' if  Z==0
									matrix qualbalance[`row',`col']   = r(mean)
									matrix qualbalance[`row'+1,`col'] = r(sd)
									matrix qualbalance[`row'+2,`col'] = r(N)
							
								*qualbalance and attrition, no control(OLS)
								qui reg `var' Z , vce(cluster maskedid)
									matrix qualbalance[`row',`col'+1]   = _b[Z]
									matrix qualbalance[`row'+1,`col'+1] = _se[Z]
									matrix qualbalance[`row'+2,`col'+1] = e(N)
									
								*qualbalance and attrition, qual inst control
								qui reg `var' Z r_* , vce(cluster maskedid)
									matrix qualbalance[`row',`col'+2]   = _b[Z]
									matrix qualbalance[`row'+1,`col'+2] = _se[Z]
									matrix qualbalance[`row'+2,`col'+2] = e(N)
									
								*qualbalance for kids in formula pscore sample
								qui reg `var' Z r_* if psample==1 , vce(cluster maskedid)
									matrix qualbalance[`row',`col'+3]   = _b[Z]
									matrix qualbalance[`row'+1,`col'+3] = _se[Z]
									matrix qualbalance[`row'+2,`col'+3] = e(N)	
									
								
							local row = `row' + 4
							}
						
							**********************************************************************************
							************************************* Logit Fs ***********************************
							**********************************************************************************	
							
							foreach type in robust traditional{
									
									local row = `row' + 3

									if "`type'"=="robust"{
										local option "nocnsreport vce(cluster maskedid)"
									}
									else{
										local option "nocnsreport"
									}
									
								* Balance logit
									replace var="Logit Balance F" if _n==`row'
									replace var="Logit Balance F (pval)" if _n==`row'+1
									replace var="Logit Balance F (df)" if _n==`row'+2
									replace var="Logit Balance F (N)" if _n==`row'+3
								
									*No riskset
									qui logit Z `appcov' `cov' `cov2', `option'
									test `appcov' `cov' `cov2'
									matrix qualbalance[`row', `col'+1] = r(chi2)
									matrix qualbalance[`row'+1, `col'+1] = r(p)
									matrix qualbalance[`row'+2, `col'+1] = r(df)
									matrix qualbalance[`row'+3, `col'+1] = e(N)
									
									*Risksets
									qui logit Z r_* `appcov' `cov' `cov2', `option'
									test `appcov' `cov' `cov2'
									matrix qualbalance[`row', `col'+2] = r(chi2)
									matrix qualbalance[`row'+1, `col'+2] = r(p)
									matrix qualbalance[`row'+2, `col'+2] = r(df)
									matrix qualbalance[`row'+3, `col'+2] = e(N)

									*Riskset restricted to psample
									qui logit Z r_* `appcov' `cov' `cov2' if psample==1, `option'
									test `appcov' `cov' `cov2'
									matrix qualbalance[`row', `col'+3] = r(chi2)
									matrix qualbalance[`row'+1, `col'+3] = r(p)
									matrix qualbalance[`row'+2, `col'+3] = r(df)
									matrix qualbalance[`row'+3, `col'+3] = e(N)
									
								local row = `row' + 4
								
							}
							
							********************* Traditional Fs **************************
								replace var="Traditional Balance F-test" if _n==`row'
								replace var="Traditional Balance F-test (pval)" if _n==`row'+1
								replace var="Traditional Balance F-test (residuals df)" if _n==`row'+2
								replace var="Traditional Balance F-test (model df)" if _n==`row'+3
								replace var="Traditional Balance F-test (N)" if _n==`row'+4
									
								*no risksets
								mvreg `appcov' `cov' `cov2' = Z
								test Z
								matrix qualbalance[`row',`col' + 1]=r(F)
								matrix qualbalance[`row'+1,`col' + 1]=r(p)
								matrix qualbalance[`row'+2, `col'+1] = r(df_r)
								matrix qualbalance[`row'+3, `col'+1] = r(df)
								matrix qualbalance[`row'+4, `col'+1] = e(N)
								
								
								*with risksets
								mvreg `appcov' `cov' `cov2' = Z r_*
								test Z
								matrix qualbalance[`row',`col' + 2]=r(F)
								matrix qualbalance[`row'+1,`col' + 2]=r(p)
								matrix qualbalance[`row'+2, `col'+2] = r(df_r)
								matrix qualbalance[`row'+3, `col'+2] = r(df)
								matrix qualbalance[`row'+4, `col'+2] = e(N)
										
								*with risksets & limit to pscore sample
								mvreg `appcov' `cov' `cov2' = Z r_* if psample==1
								test Z
								matrix qualbalance[`row',`col' + 3]=r(F)
								matrix qualbalance[`row'+1,`col' + 3]=r(p)
								matrix qualbalance[`row'+2, `col'+3] = r(df_r)
								matrix qualbalance[`row'+3, `col'+3] = r(df)
								matrix qualbalance[`row'+4, `col'+3] =  e(sample)
										
								local row = `row' +5
								
								**********************************************************************************
								*********************** Robust F Stack on outcomes (covariates) ******************
								**********************************************************************************
								
								*Run a cut keeping cells with small size and one dropping cells with small size
								foreach f_cut in keep_small_cells drop_small_cells{
										
									preserve
										
										if "`f_cut'" === "keep_small_cells"{
											local min_obs = 1
										}
										else if "`f_cut'"=="drop_small_cells"{
											local min_obs = 5
										}
										
										keep maskedid `appcov' `cov' `cov2' Z D r in_sample psample in_good_cell obs_cell_r
										expand `expand'
										bys maskedid: gen copy = _n
										sort copy
										
										*Single vector of outcomes
										gen cov =.
										gen covname=""
										
										tokenize `appcov' `cov' `cov2'

										forval i=1/`expand'{
											replace cov = ``i'' if copy==`i'
											replace covname = "``i''" if copy==`i'
										}
						
										drop `appcov' `cov' `cov2'
										
										*Instruments for each copy, risksets for each copy, dummies for each copy
										forval i=1/`expand'{
											gen Z_`i' = Z*(copy == `i')
											gen alpha_`i' = (copy == `i')
											
											gen c_r`i' = r*(copy == `i')
										}
										
										egen group_r = group(copy r)

										*** One tall regression for each specification ***
										
										*No controls
											*Clustered
											qui reg cov alpha_* Z_*, vce(cluster maskedid)
											test `test'
											
											matrix qualbalance[`row',`col' + 1]=r(F)
											matrix qualbalance[`row'+1,`col' + 1]=r(p)
											matrix qualbalance[`row'+2, `col'+1] = r(df_r)
											matrix qualbalance[`row'+3, `col'+1] = r(df)
											matrix qualbalance[`row'+4, `col'+1] = e(N)
												
											*Traditional
											qui reg cov alpha_* Z_*
											test `test'
											
											matrix qualbalance[`row'+5,`col' + 1]=r(F)
											matrix qualbalance[`row'+6,`col' + 1]=r(p)
											matrix qualbalance[`row'+7, `col'+1] = r(df_r)
											matrix qualbalance[`row'+8, `col'+1] = r(df)
											matrix qualbalance[`row'+9, `col'+1] = e(N)
											
											*Robust
											qui reg cov alpha_* Z_*, r
											test `test'
											
											matrix qualbalance[`row'+10,`col' + 1]=r(F)
											matrix qualbalance[`row'+11,`col' + 1]=r(p)
											matrix qualbalance[`row'+12, `col'+1] = r(df_r)
											matrix qualbalance[`row'+13, `col'+1] = r(df)
											matrix qualbalance[`row'+14, `col'+1] = e(N)

										*Saturated
											*Clustered
											qui areg cov alpha_* Z_* if in_good_cell==1 & obs_cell_r>= `min_obs', absorb(group_r) vce(cluster maskedid)
											test `test'
											
											matrix qualbalance[`row',`col' + 2]=r(F)
											matrix qualbalance[`row'+1,`col' + 2]=r(p)
											matrix qualbalance[`row'+2, `col'+2] = r(df_r)
											matrix qualbalance[`row'+3, `col'+2] = r(df)
											matrix qualbalance[`row'+4, `col'+2] = e(N)
									
											
											*Traditional
											qui areg cov alpha_* Z_* if in_good_cell==1 & obs_cell_r>= `min_obs', absorb(group_r)
											test `test'
											
											matrix qualbalance[`row'+5,`col' + 2]=r(F)
											matrix qualbalance[`row'+6,`col' + 2]=r(p)
											matrix qualbalance[`row'+7, `col'+ 2] = r(df_r)
											matrix qualbalance[`row'+8, `col'+ 2] = r(df)
											matrix qualbalance[`row'+9, `col'+ 2] = e(N)
											
											*Robust
											qui areg cov alpha_* Z_* if in_good_cell==1 & obs_cell_r>= `min_obs', absorb(group_r) r
											test `test'
											
											matrix qualbalance[`row'+10,`col' + 2]=r(F)
											matrix qualbalance[`row'+11,`col' + 2]=r(p)
											matrix qualbalance[`row'+12, `col'+ 2] = r(df_r)
											matrix qualbalance[`row'+13, `col'+ 2] = r(df)
											matrix qualbalance[`row'+14, `col'+ 2] = e(N)

										
										*Saturated and psample
											*Robust
											qui areg cov alpha_* Z_* if psample==1 & in_good_cell==1 & obs_cell_r>= `min_obs', absorb(group_r) vce(cluster maskedid)
											test `test'
											
											matrix qualbalance[`row',`col' + 3]=r(F)
											matrix qualbalance[`row'+1,`col'+ 3] = r(p)
											matrix qualbalance[`row'+2, `col'+3] = r(df_r)
											matrix qualbalance[`row'+3, `col'+3] = r(df)
											matrix qualbalance[`row'+4, `col'+3] = e(N)
											
											*Traditional
											qui areg cov alpha_* Z_* if psample==1 & in_good_cell==1 & obs_cell_r>= `min_obs', absorb(group_r)
											test `test'
											
											matrix qualbalance[`row'+5,`col' + 3] = r(F)
											matrix qualbalance[`row'+6,`col' + 3] = r(p)
											matrix qualbalance[`row'+7, `col'+ 3] = r(df_r)
											matrix qualbalance[`row'+8, `col'+ 3] = r(df)
											matrix qualbalance[`row'+9, `col'+ 3] = e(N)
											
											*Robust
											qui areg cov alpha_* Z_* if psample==1 & in_good_cell==1 & obs_cell_r>= `min_obs', absorb(group_r) r
											test `test'
											
											matrix qualbalance[`row'+10,`col' + 3] = r(F)
											matrix qualbalance[`row'+11,`col' + 3] = r(p)
											matrix qualbalance[`row'+12, `col'+ 3] = r(df_r)
											matrix qualbalance[`row'+13, `col'+ 3] = r(df)
											matrix qualbalance[`row'+14, `col'+ 3] = e(N)
									
									restore
								
								replace var = "[`f_cut'] Stacked Clustered Balance F-test" if _n==`row'
								replace var = "[`f_cut'] Stacked Clustered Balance F-test (pval)" if _n==`row'+1
								replace var = "[`f_cut'] Stacked Clustered Balance F-test (residuals df)" if _n==`row'+2
								replace var = "[`f_cut'] Stacked Clustered Balance F-test (model df)" if _n==`row'+3
								replace var = "[`f_cut'] Stacked Clustered Balance F-test (N)" if _n==`row'+4
								replace var = "[`f_cut'] Stacked Clustered Balance F-test (cells)" if _n==`row'+5
								
								replace var = "[`f_cut'] Stacked Traditional Balance F-test" if _n==`row'+6
								replace var = "[`f_cut'] Stacked Traditional Balance F-test (pval)" if _n==`row'+7
								replace var = "[`f_cut'] Stacked Traditional Balance F-test (residuals df)" if _n==`row'+8
								replace var = "[`f_cut'] Stacked Traditional Balance F-test (model df)" if _n==`row'+9
								replace var = "[`f_cut'] Stacked Traditional Balance F-test (N)" if _n==`row'+10
								replace var = "[`f_cut'] Stacked Traditional Balance F-test (cells)" if _n==`row'+11
								
								replace var = "[`f_cut'] Stacked Robust Balance F-test" if _n==`row' +12
								replace var = "[`f_cut'] Stacked Robust Balance F-test (pval)" if _n==`row'+13
								replace var = "[`f_cut'] Stacked Robust Balance F-test (residuals df)" if _n==`row'+14
								replace var = "[`f_cut'] Stacked Robust Balance F-test (model df)" if _n==`row'+15
								replace var = "[`f_cut'] Stacked Robust Balance F-test (N)" if _n==`row'+16
								replace var = "[`f_cut'] Stacked Robust Balance F-test (cells)" if _n==`row'+17
											
								local row = `row' + 15
							}	/*Close small cell loop*/
							
								**********************************************************************************
								*********************** Suest Robust standard errors *****************************
								**********************************************************************************
								
								** F-tests using suest, which allows for robus or clustered SEs **
								* 1) Run models without clustering option
								* 2) Store results
								* 3) suest [stored values], vce(cluster clustvar)
								* 4) test coefficient
								
							*Run a cut keeping cells with small size and one dropping cells with small size
								foreach f_cut in keep_small_cells drop_small_cells{
								
									if "`f_cut'" === "keep_small_cells"{
											local min_obs = 1
									}
									else if "`f_cut'"=="drop_small_cells"{
										local min_obs = 9
									}
								
									*Balance F
									replace var="[`f_cut'] Suest Balance F" if _n==`row'
									replace var="[`f_cut'] Suest Balance F (pval)" if _n==`row' +1
									replace var="[`f_cut'] Suest Balance F (df)" if _n==`row' +2
									replace var="[`f_cut'] Suest Balance F (N)" if _n==`row' +3
									
									foreach var in `appcov' `cov' `cov2'{
										
										* no riskset
										qui reg `var' Z
										est store `var'
										
										*Saturated
										qui reg `var' Z r_* if in_good_cell==1 & obs_cell_r >= `min_obs'
										est store S`var'
										
										*Saturated and in pscore sample
										qui reg `var' Z r_* if psample==1 & in_good_cell==1 & obs_cell_r >= `min_obs'
										est store F`var'
									}
									
									suest	num_schools num_study `rank1' ///
											baselinefemale baselinehisp baselineblack baselinestudy baselinefoodfred baselinesped baselinelep baselinegifted ///
											baseliness_math baseliness_reading baseliness_writing, vce(cluster maskedid)
									test Z
									matrix qualbalance[`row', `col'+1] = r(chi2)
									matrix qualbalance[`row'+1, `col'+1] = r(p)
									matrix qualbalance[`row'+2, `col'+1] = r(df)
									matrix qualbalance[`row'+3, `col'+1] = e(N)
											
									cap suest	Snum_schools Snum_study `Srank1' ///
											Sbaselinefemale Sbaselinehisp Sbaselineblack Sbaselinestudy Sbaselinefoodfred Sbaselinesped Sbaselinelep Sbaselinegifted ///
											Sbaseliness_math Sbaseliness_reading Sbaseliness_writing, vce(cluster maskedid)
									
									if !_rc{
										test Z
										matrix qualbalance[`row', `col'+2] = r(chi2)
										matrix qualbalance[`row'+1, `col'+2] = r(p)
										matrix qualbalance[`row'+2, `col'+2] = r(df)
										matrix qualbalance[`row'+3, `col'+2] = e(N)						
									}
									
									
									cap suest 	Fnum_schools Fnum_study `Frank1' ///
												Fbaselinefemale Fbaselinehisp Fbaselineblack Fbaselinestudy Fbaselinefoodfred Fbaselinesped Fbaselinelep Fbaselinegifted ///
												Fbaseliness_math Fbaseliness_reading Fbaseliness_writing, vce(cluster maskedid)
									if !_rc{
										test Z
										matrix qualbalance[`row', `col'+3] = r(chi2)
										matrix qualbalance[`row'+1, `col'+3] = r(p)
										matrix qualbalance[`row'+2, `col'+3] = r(df)
										matrix qualbalance[`row'+3, `col'+3] = e(N)
									}
							local row = `row' + 5
							
							} /*Close small cell loop*/
							
							*Outsheet
							preserve
								svmat qualbalance
								keep var qualbalance*
								keep if _n<=`row'+5
								ren qualbalance1 z0
								ren qualbalance2 ols_z1_OLS
								ren qualbalance3 ols_z1_qual
								ren qualbalance4 ols_z1_qual_psample
								outsheet using "`tables'`cut'_qual`type'_balance_choices_`choice'_stacked.csv", comma replace
							restore
					
					} /*Close cut loop*/
				}	/*Close balance condition*/
			}	/*Close cut (in DPS at baseline or not) loop*/
	
	**********************************************************
	***************** Qual instruments 2SLS ******************
	**********************************************************

	if `qual_twosls'==1{
	
	foreach choice in  1  all /*2*/ {
	
			if "`choice'"=="1"{
				local qnum = 20
			}
			else if "`choice'"=="all"{
				local qnum = 100
			}
		
		foreach type in standard /*naive*/{
		
			local qualdata "`cleandata'mdrd_denver_qual`type'_lt`qnum'_stacked `qualdate'"
			
			foreach cut in inDPSbase /*allapps*/{
				
				u "`qualdata'", clear
				
				*Analysis is at student level
				duplicates drop maskedid baselinespyear, force
				
				*dropping duplicates from year 2. 
				drop if baselinespyear == . 
				duplicates tag masked, gen(dup_masked)
				drop if dup_masked > 0 & baselinespyear == 2013
				
				
				destring baselinegrade, replace
				gen appgrade_tmp = appgrade
				destring appgrade_tmp, replace
				drop appgrade
				rename appgrade_tmp appgrade
				
				bys maskedid spyear: egen enroll_any_study = max(study)
				duplicates drop maskedid spyear, force
				
				
				*Bring in variable with any charter offer for "offer on qualification" first stage
				* (Doesn't matter which score we pull this from)
				preserve
					u "`cleandata'mdrd_denver_study_formula `qualpscoredate'", clear
					
					append using "`cleandata'mdrd_denver_study_year_2_formula `qualpscoredate'"
					
					cap drop offer_study
					gen offer_study = (offer==1 & appstudy ==1)
					bys maskedid spyear: egen any_study_offer = max(offer_study)
					
					keep maskedid spyear any_study_offer
					duplicates drop
					
					tempfile offers
					sa `offers'
				restore
				
				merge 1:1 maskedid spyear using `offers'
				keep if _merge!=2
				drop _merge
				
				*Keep if in Denver at baseline
				keep if (baselinespyear==2012 | baselinespyear == 2013) & in_sample==1
				
				
				
				*Flag for whether you are subject to randomization in frequency01
				preserve
					u "`cleandata'analysis_stacked_study_frequency", clear
					keep if in_sample & pscore1!=0 & pscore1!=1

					bys r0: egen sd_z= sd(Z)
					keep if sd_z>0 & sd_z!=.
					
					count
					keep maskedid spyear
					tempfile rand
					sa `rand'
				restore
				
				merge 1:1 maskedid spyear using `rand'
				gen psample = (_merge==3)
				drop _merge
				
				ren qual_lt`qnum' Z
				ren enroll_any_study D
				
				
				tab riskset_lt`qnum', gen(r_)
				ren riskset_lt`qnum' r
				
				**********************************************************************************
				******************** Find cells with variation in instrument**********************
				**********************************************************************************
				
				*NOTE that qualification instruments file was already constructed such that obs are
				*in cells with variation in instrument, but do this so that qual balance code can follow
				*the same structure as pscore code

					foreach x in r{
							
						*Variation in instruments for each cell
							bys `x': egen sd_z_`x'	= 	sd(Z)
						
						*Good cell
							gen in_good_cell_`x' = (sd_z_`x'>0 & !missing(sd_z))
						
						drop sd_z_* 
					}
				
	
				******* We want to know the schools applied to by the risksets
				***** with variation in offer. We then want to see if these schools 
				***** have variation in sample for the applicants in the sample. 
				***** Not pretty but it will make sense <3
				
				preserve
					keep if in_good_cell == 1 
					keep masked r in_good_cell
					tempfile good_cells
					sa 	`good_cells'
					unique r if in_good_cell == 1 
					
					u "`qualdata'", clear
					
					merge m:1 masked using `good_cells', keep(match)
					*Risksets with var in offer
					bys r: egen sd_z_lt`choice' = sd(qual_lt`qnum')
					 
					collapse (max) sd_z_lt`choice'  , by(appschoolnum)
					keep if sd_z_lt`choice'  >0 & sd_z_lt`choice' != .
					unique appschoolnum if sd >0 & sd != .
					* now have unique set of schools with var in offer and such. 
					*recording the number and adding it to the output of this section. 
					local unique_schools_var = `r(sum)'
				restore
				
							
				local col = -4
				gen var =""
				
				*By grade vars
				gen group = .
				

					local row =1
					local col = `col' + 5
					
					local risk "r_"
					local controls "i.r `cov' `cov2' `cov_grade'"
				
				foreach group in 410  /* 48 910 */{
							if `group'==410{
								replace group=410
							}
							else if `group'==48{
								replace group=48 if inlist(appgrade,4,5,6,7,8)
							}
							else{
							replace group=45 if inlist(appgrade,4,5)
							replace group=68 if inlist(appgrade,6,7,8)
							replace group=910 if inlist(appgrade,9,10)
							}
				
				
						foreach t in math writing reading {
							foreach out in ss prof_up adv {
							replace var="`out'_`t'_`group'" if _n==`row'
							replace var="`out'_`t'_`group'_se" if _n==`row'+1
							replace var="`out'_`t'_`group'_N" if _n==`row'+2
							
						
							qui summ `out'_`t' if Z == 0 & group==`group'
							matrix qualtwosls[`row',`col'] = r(mean)
							matrix qualtwosls[`row' +2,`col'] = r(N)

								/*OLS*/
								reg `out'_`t' D  `cov1' `cov2' `cov_grade' i.baselinespyear if group==`group', vce(robust) 
								matrix qualtwosls[`row',`col'+1]   = _b[D]
								matrix qualtwosls[`row'+1,`col'+1] = _se[D]
								matrix qualtwosls[`row'+2,`col'+1] = e(N)
								
								/*RF - outcome on instrument*/
								reg `out'_`t' Z `controls' i.baselinespyear if group==`group' & in_good_cell_r == 1, vce(robust)  
								matrix qualtwosls[`row',`col'+2]   = _b[Z]
								matrix qualtwosls[`row'+1,`col'+2] = _se[Z]
								matrix qualtwosls[`row'+2,`col'+2] = e(N)
								
								/*FS - treatment on instrument*/
								reg D Z `controls' i.baselinespyear if group==`group' & `out'_`t'!=. & in_good_cell_r == 1, vce(robust) 
								matrix qualtwosls[`row',`col'+3]   = _b[Z]
								matrix qualtwosls[`row'+1,`col'+3] = _se[Z]
								matrix qualtwosls[`row'+2,`col'+3] = e(N)
								
								/*2SLS - outcome on treatment instrumented by instrument*/
								ivreg2 `out'_`t' (D = Z) `controls' i.baselinespyear if group==`group' & in_good_cell_r == 1, savefirst partial(`controls') robust
								matrix qualtwosls[`row',`col'+4]   = _b[D]
								matrix qualtwosls[`row'+1,`col'+4] = _se[D]
								matrix qualtwosls[`row'+2,`col'+4] = e(N)
									
								*NAIVE 2SLS - no riskset controls
								
									*FS
									reg D Z `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & `out'_`t'!=., vce(robust) 
									matrix qualtwosls[`row',`col'+5]   = _b[Z]
									matrix qualtwosls[`row'+1,`col'+5] = _se[Z]
									matrix qualtwosls[`row'+2,`col'+5] = e(N)
										
									*2SLS
									ivreg2 `out'_`t' (D = Z) `cov' `cov2' `cov_grade' i.baselinespyear if group==`group', savefirst partial(`cov' `cov2' `cov_grade') robust
									matrix qualtwosls[`row',`col'+6]   = _b[D]
									matrix qualtwosls[`row'+1,`col'+6] = _se[D]
									matrix qualtwosls[`row'+2,`col'+6] = e(N)
										
									*2SLS with no covariates		
									*FS
									reg D Z i.r i.baselinespyear if group==`group' & `out'_`t'!=. & in_good_cell_r == 1, vce(robust) 
									matrix qualtwosls[`row',`col'+7]   = _b[Z]
									matrix qualtwosls[`row'+1,`col'+7] = _se[Z]
									matrix qualtwosls[`row'+2,`col'+7] = e(N)
										
									*2SLS
									ivreg2 `out'_`t' (D = Z) i.r i.baselinespyear if group==`group' & in_good_cell_r == 1, savefirst partial(i.r) robust
									matrix qualtwosls[`row',`col'+8]   = _b[D]
									matrix qualtwosls[`row'+1,`col'+8] = _se[D]
									matrix qualtwosls[`row'+2,`col'+8] = e(N)
										
								*2SLS conditional on psample
								*FS
									reg D Z `controls' i.baselinespyear if group==`group' & psample==1 & `out'_`t'!=. & in_good_cell_r == 1, vce(robust) 
									matrix qualtwosls[`row',`col'+9]   = _b[Z]
									matrix qualtwosls[`row'+1,`col'+9] = _se[Z]
									matrix qualtwosls[`row'+2,`col'+9] = e(N)
										
									*2SLS
									ivreg2 `out'_`t' (D = Z) `controls' i.baselinespyear if group==`group' & psample==1 & in_good_cell_r == 1, savefirst partial(`controls') robust
									matrix qualtwosls[`row',`col'+10]   = _b[D]
									matrix qualtwosls[`row'+1,`col'+10] = _se[D]
									matrix qualtwosls[`row'+2,`col'+10] = e(N)

								*Fake FS: offer at any charter regressed on qualification instrument

									reg any_study_offer Z `controls' i.baselinespyear if group==`group' & in_good_cell_r==1 & `out'_`t' != ., vce(robust) 
									matrix qualtwosls[`row',`col'+11]   = _b[Z]
									matrix qualtwosls[`row'+1,`col'+11] = _se[Z]
									matrix qualtwosls[`row'+2,`col'+11] = e(N)
								
							local row = `row' + 3
							}	/*out loop*/
						local ++row	
						}	/* t loop*/
					}	/*group loop*/
			
		
			preserve
				svmat qualtwosls
				keep var qualtwosls*
				keep if _n<=`row'
				ren qualtwosls1 z0
				ren qualtwosls2 OLS_qual
				ren qualtwosls3 RF_qual
				ren qualtwosls4 FS_qual
				ren qualtwosls5 twosls_qual
				
				ren qualtwosls6 FS_qualnaive
				ren qualtwosls7 twosls_qualnaive
				
				ren qualtwosls8 FS_qualnocovs
				ren qualtwosls9 twosls_qualnocovs
				
				ren qualtwosls10 FS_qualpsample
				ren qualtwosls11 twosls_qualpsample
				
				ren qualtwosls12 FS_any_offer_on_qual_Z
				
				ren qualtwosls14 schools_var
				replace schools_var = `unique_schools_var' if var != ""  //adding the schools with var in offer
				outsheet using "`tables'`cut'_qual`type'_twosls_choices_`choice'_stacked.csv", comma replace
				
				
			restore
		clear	
			}	/*Close 2SLS condition*/
		}	/*Close Cut loop*/
	}	/*Close choice loop*/
}
	*******************************************************************
	***************** Qual instruments Subgroup 2SLS ******************
	*******************************************************************

	if `qual_subtwosls'==1{
	
	foreach choice in 1 all{
					
		if "`choice'"=="1"{
			local qnum = 20
		}
		else if "`choice'"=="all"{
			local qnum = 100
		}
				
		foreach type in standard naive{
		
		local qualdata "`cleandata'mdrd_denver_qual`type'_lt`qnum' `qualdate'"
	
		
			foreach cut in inDPSbase /*allapps*/{
				
				u "`qualdata'", clear
				
				if "`cut'"=="inDPSbase"{
					drop if baselinespyear!=2012
				}
				else{
					di in red "Keep all obs"
				}
				
				*Analysis is at student level
				drop appschoolnum
				duplicates drop maskedid, force
				
				destring baselinegrade, replace
				destring appgrade, replace
				
				keep if in_samp == 1
				
				bys maskedid: egen enroll_any_study = max(study)
				duplicates drop maskedid, force
				
				*Flag for whether you are subject to randomization in frequency
				preserve
					u "`cleandata'analysis_frequency", clear
					keep if in_sample & pscore1!=0 & pscore1!=1

					bys r0: egen sd_z= sd(Z)
					keep if sd_z>0 & sd_z!=.
					
					count
					keep maskedid
					tempfile rand
					sa `rand'
				restore
				
				merge 1:1 maskedid using `rand'
				gen psample = (_merge==3)
				drop _merge
				
				ren riskset_qual Z
				ren enroll_any_study D
				
				local col = -4
				gen var =""
				
				tab riskset, gen(r_)
				
				local row =1
				local col = `col' + 5
				
				local risk "r_"
				local controls "`risk'* `cov' `cov2'"
				
				foreach group in `cov'{
					gen group = (`group' == 1)
					
						foreach t in math writing reading {
							foreach out in ss /*prof_up adv*/ {
							replace var="`out'_`t'_`group'" if _n==`row'
							replace var="`out'_`t'_`group'_se" if _n==`row'+1
							replace var="`out'_`t'_`group'_N" if _n==`row'+2
							
							qui summ `out'_`t' if group==1
							
							if r(max)==0 & r(min)==0{
								di in red "No observations for `group'-level estimates"
							}
						
							else{
									
								qui summ `out'_`t' if Z == 0 & group==1
								matrix qualsubtwosls[`row',`col'] = r(mean)
								matrix qualsubtwosls[`row'+2,`col'] = r(N)
							
						
								/*OLS - no control for pscore*/
								reg `out'_`t' D `cov' `cov2' if group==1, r
								matrix qualsubtwosls[`row',`col'+1]   = _b[D]
								matrix qualsubtwosls[`row'+1,`col'+1] = _se[D]
								matrix qualsubtwosls[`row'+2,`col'+1] = e(N)
							
							
								/*RF - outcome on instrument*/
								reg `out'_`t' Z `controls' if group==1, r 
								matrix qualsubtwosls[`row',`col'+2]   = _b[Z]
								matrix qualsubtwosls[`row'+1,`col'+2] = _se[Z]
								matrix qualsubtwosls[`row'+2,`col'+2] = e(N)
							
							
								/*FS - treatment on instrument*/
								reg D Z `controls' if group==1 & `out'_`t'!=., r
									matrix qualsubtwosls[`row',`col'+3]   = _b[Z]
									matrix qualsubtwosls[`row'+1,`col'+3] = _se[Z]
									matrix qualsubtwosls[`row'+2,`col'+3] = e(N)
							
							
								/*2SLS - outcome on treatment instrumented by instrument*/
								ivreg2 `out'_`t' (D = Z) `controls' if group==1, savefirst partial(`controls') r 
									matrix qualsubtwosls[`row',`col'+4]   = _b[D]
									matrix qualsubtwosls[`row'+1,`col'+4] = _se[D]
									matrix qualsubtwosls[`row'+2,`col'+4] = e(N)
									
							
								*NAIVE 2SLS - no riskset controls
									
									*FS
									reg D Z `cov' `cov2' if group==1 & `out'_`t'!=., r
										matrix qualsubtwosls[`row',`col'+5]   = _b[Z]
										matrix qualsubtwosls[`row'+1,`col'+5] = _se[Z]
										matrix qualsubtwosls[`row'+2,`col'+5] = e(N)
										
									*2SLS
									ivreg2 `out'_`t' (D = Z) `cov' `cov2' if group==1, savefirst partial(`cov' `cov2') r 
										matrix qualsubtwosls[`row',`col'+6]   = _b[D]
										matrix qualsubtwosls[`row'+1,`col'+6] = _se[D]
										matrix qualsubtwosls[`row'+2,`col'+6] = e(N)
										
								*2SLS with no covariates		
									*FS
									reg D Z `risk'* if group==1 & `out'_`t'!=., r
										matrix qualsubtwosls[`row',`col'+7]   = _b[Z]
										matrix qualsubtwosls[`row'+1,`col'+7] = _se[Z]
										matrix qualsubtwosls[`row'+2,`col'+7] = e(N)
										
									*2SLS
									ivreg2 `out'_`t' (D = Z) `risk'* if group==1, savefirst partial(`risk'*) r 
										matrix qualsubtwosls[`row',`col'+8]   = _b[D]
										matrix qualsubtwosls[`row'+1,`col'+8] = _se[D]
										matrix qualsubtwosls[`row'+2,`col'+8] = e(N)
										
								*2SLS conditional on psample
								*FS
									reg D Z `controls' if group==1 & psample==1 & `out'_`t'!=., r
										matrix qualsubtwosls[`row',`col'+9]   = _b[Z]
										matrix qualsubtwosls[`row'+1,`col'+9] = _se[Z]
										matrix qualsubtwosls[`row'+2,`col'+9] = e(N)
										
									*2SLS
									ivreg2 `out'_`t' (D = Z) `controls'  if group==1 & psample==1, savefirst partial(`controls') r 
										matrix qualsubtwosls[`row',`col'+10]   = _b[D]
										matrix qualsubtwosls[`row'+1,`col'+10] = _se[D]
										matrix qualsubtwosls[`row'+2,`col'+10] = e(N)
						
							}
							
							local row = `row' + 3
							}	/*out loop*/
						local ++row	
						}	/* t loop*/
					drop group	
					}	/*group loop*/
					
				preserve
					svmat qualsubtwosls
					keep var qualsubtwosls*
					keep if _n<=`row'
					ren qualsubtwosls1 z0
					ren qualsubtwosls2 OLS_qual
					ren qualsubtwosls3 RF_qual
					ren qualsubtwosls4 FS_qual
					ren qualsubtwosls5 twosls_qual
					
					ren qualsubtwosls6 FS_qualnaive
					ren qualsubtwosls7 twosls_qualnaive
					
					ren qualsubtwosls8 FS_qualnocovs
					ren qualsubtwosls9 twosls_qualnocovs
					
					ren qualsubtwosls10 FS_qualpsample
					ren qualsubtwosls11 twosls_qualpsample
					
					outsheet using "`tables'`cut'_qual`type'_subtwosls_choices_`choice'.csv", comma replace
				restore
				
			}	/*Close subgroup 2SLS condition*/
		} /*Close cut loop*/
	} /*Close choice loop*/
}
