/*
This file stacks the files from year 1 and year 2. The output file is then used for
the 2sls analysis. 

*/
clear all
cap log close
set more off
set trace off
set tracedepth 1 
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"
pause on

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"
local rawdata "Y:\DPS\Raw Data\"

matrix demo			  = J(30,30,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,40,.)
matrix subtwosls	  = J(500,40,.)
matrix twosls_nocov    		   = J(500,40,.)
matrix twosls_scorecov 		   = J(500,40,.)
matrix twosls_fcsample   		= J(50	0,40,.)

matrix qualbalance    = J(500,20,.)
matrix qualtwosls	  = J(500,20,.)
matrix qualsubtwosls  = J(500,20,.)



local qual_stacking = 0
local qual_balance=0
local qual_twosls = 0
local qual_subtwosls =0

*Dates of analysis files
local pscoredate_year2	20170310 	/*20160516 20160510 20160428 20160411 20160208 20160112 20151120 20150514 */
local pscoredate_year1  20170310 /*20160516    	20160516 20160510 20160421  20160208 */
local simsdate 			20150603 /* 20150428 */
local qualdate_year1 	20160715 /*20150708  20150420 */
local qualdate_year2	20160715 /*20160303*/
local qualdate			20160303
local qualpscoredate	20160208
local appdatayr1		20170310
local appdatayr2		20170310


local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_*"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1        		/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
local outsheet_rescorr=0	/* If want to outsheet matrix with residual correlations */


		
		foreach file in simulation frequency formula      {
		foreach sch_type in study /*noex nonnoex*/ {
		
			u "`cleandata'analysis_`sch_type'_year_2_`file'.dta", clear
							
			append using "`cleandata'analysis_`sch_type'_`file'.dta"
			
			*dropping duplicates from year 2. 
			duplicates tag masked, gen(dup_masked)
			drop if dup_masked > 0 & baselinespyear == 2013			
			
			drop in_good_cell_* tag_* obs_cell_* r01_* r001_* r0_*
			
			tab r01, gen(r01_)
			tab r001, gen(r001_)
			tab r0, gen(r0_)
			
			* creating indicator variable from continuous pscore variables. Need this to use factor variables
			drop ind_*
			egen ind_r0 = group(r0)
			egen ind_r01 = group(r01)
			egen ind_r001 = group(r001)
			
			foreach x in r01 r001 r0{
				
				*Variation in instruments for each cell
					bys `x': egen sd_z_`x'	= 	sd(Z)
				
				*Good cell you have rounded pscore not 0 1
					gen in_good_cell_`x' = (sd_z_`x'>0 & !missing(sd_z_`x') & `x' >0 & `x' < 1)
					gen in_good_cell_org_`x' = (sd_z_`x'>0 & !missing(sd_z_`x') & in_zero1_pscore == 1)
					gen double tag_`x' = `x'
					replace tag_`x' = -9999999 if in_good_cell_`x'==0
					tab tag_`x', gen(tag_`x'_)
				
				*Drop var for things with no var (the first one). These observations will not be included
				*in bin-based regression and fully saturated regressios. So, do not include their corresponding
				*variables either
				
				drop tag_`x'_1
				
				*Obs per cell
				
				bys `x': egen obs_cell_`x' = count(maskedid)
			}
			
			drop sd_z_*
			

			
			save "`cleandata'analysis_stacked_`sch_type'_`file'.dta", replace	
			
			}
		}
	
