/*

Ignacio 
Changing this so it uses the simulated score. 

*/

clear
set more off
global folder_raw  "Y:\DPS\Raw Data\2013_03_21_enrollment-match\"
global folder_output "Y:\DPS\yusuke\output\"
global folder_data "Y:\DPS\yusuke\data\"

global date 20160509
										
local rawdata   "Y:/DPS/Raw Data/"
local asgndata  "Y:/DPS/Raw Data/2013_03_21_enrollment-match/"
local testdata  "Y:/DPS/Raw Data/2013_10_03_tests2013/"
local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local yusuke	 "Y:/DPS/yusuke/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/"

										
use "`cleandata'pscore_simulation_school_grade", clear

*create app_s_t_randomization=# of students s.t. randomization at each school according to pscore_freq
sort  maskedid appschoolnum appgrade

collapse (sum) pscore_school, by( maskedid appschoolnum appgrade)
gen app_s_t_randomization=0
replace app_s_t_randomization=1 if pscore_school>0 & pscore_school<1
sort appschoolnum appgrade
bys appschoolnum appgrade: egen x = total(app_s_t_randomization)

collapse (sum) app_s_t_randomization, by(appschoolnum appgrade)
tempfile simscore
ren appschoolnum requestedschool
ren appgrade grade 
save `simscore', replace


*create app_s_t_1st_choice_randomization=# of students who rank each school 1st and are s.t. randomization there
*here we use new definition in the notes attached to the figures
use "${folder_data}application_data_year_one_${date}.dta", clear

*define "first choice school" this way. subject to change
* KEEPING FIRST CHOICES 
keep if choiceselection==1 |(choiceselection>=10 & choiceselection<20)
tab choiceselection, gen(choice)
tab choiceselection, gen(priority)
foreach i of varlist choice*{
replace `i'=bucket_id if `i'==1
} 
drop prioritycode
foreach j of varlist priority*{
replace `j'=apppriorityseq if `j'==1
} 
drop choiceselection offer
gen offer=0
replace offer=1 if status=="A"
collapse (sum) choice* priority* offer (mean) requestedschool, by(maskedstudentids grade)
bys choice* priority*: egen max_offer=max(offer)
bys choice* priority*: egen min_offer=min(offer)
keep if max_offer!=min_offer 
gen app_s_t_1st_choice_randomization=1
sort requestedschool grade
collapse (sum) app_s_t_1st_choice_randomization, by(requestedschool grade)

merge 1:m requestedschool grade using `simscore'

drop _merge
replace app_s_t_1st_choice_randomization=0 if app_s_t_1st_choice_randomization==.
tempfile temp1
save "`temp1'", replace

*add capacity
use "${folder_data}application_data_year_one_${date}.dta", clear
collapse (mean) capacity, by(bucket_id requestedschool grade)
collapse (sum) capacity, by(requestedschool grade)
merge 1:m requestedschool grade using "`temp1'"
drop _merge

collapse (sum) app_s_t_1st_choice_randomization app_s_t_randomization capacity, by(requestedschool)

*take log
gen log_s_t_randomization=log(app_s_t_randomization)
gen log_s_t_1st_choice_randomization=log(app_s_t_1st_choice_randomization)
gen log_capacity=log(capacity)
replace log_s_t_randomization=0.1 if log_s_t_randomization==.
replace log_s_t_1st_choice_randomization=0.1 if log_s_t_1st_choice_randomization==.
*drop schools with capacity=0
drop if capacity==0
*drop outlier
drop if log_capacity>10



*make same figures at school level

twoway pcarrow log_s_t_1st_choice_randomization log_capacity log_s_t_randomization log_capacity,/*
*/ title(Sample Size Gains: All Schools, color(black)) /*
*/ graphregion(color(white)) /*
*/ xscale(range(3.5 6)) /*
*/ xlabel(2.302 "10" 3.218 "25" 3.912 "50" 4.605 "100" 5.298 "200" 5.991 "400") /*
*/ ylabel(0 "0" 2.302 "10" 3.218 "25" 3.912 "50" 4.605 "100" 5.298 "200" 5.991 "400") /*
*/ xtitle(School Capacity (Log Scale), size(medsmall))  /*
*/ ytitle(# of Applicants s.t. Randomization (Log Scale), size(small)) /*
*/ mlcolor(red)/*
*/ mcolor(red)/*
*/ color(red)/*
*/ mfcolor(red)/*
*/ || scatter log_s_t_1st_choice_randomization log_capacity, mcolor(black) msize(small) /*
*/ legend(lab(1 "# of Additional Applicants Subject to Randomization (Log Scale)") lab(2 "# of 1st Choice Applicants Subject to Randomization (Log Scale)") order(1 - " " 2 - " "))  /*
*/ xscale(range(3.5 6)) /*
*/ saving("${folder_output}figure_sample_size_gain_all_school_new_def_raw", replace)

graph export "${folder_output}figure_sample_size_gain_all_school_new_def_raw.pdf",  as(pdf) replace

twoway pcarrow app_s_t_1st_choice_randomization capacity app_s_t_randomization capacity if app_s_t_randomization!=0 & app_s_t_1st_choice_randomization!=0,/*
*/ title(Sample Size Gains: All Schools, color(black)) /*
*/ graphregion(color(white)) /*
* xlabel(2(2.5)7.5) 
* ylabel(0(2.5)7.5) 
*/ xtitle(Log School Capacity, size(medsmall))  /*
*/ ytitle(# of Applicants s.t. Randomization, size(small)) /*
*/ mlcolor(red)/*
*/ mcolor(red)/*
*/ color(red)/*
*/ mfcolor(red)/*
*/ || scatter app_s_t_1st_choice_randomization capacity, mcolor(black) msize(small) /*
*/ legend(lab(1 "Log # of Additional Applicants Subject to Randomization") lab(2 "Log # of Applicants Who Rank It 1st & Are Subject to Randomization") order(1 - " " 2 - " "))  /*
*/ saving("${folder_output}figure_sample_size_gain_all_school_new_def", replace)

graph export "${folder_output}figure_sample_size_gain_all_school_new_def.pdf",  as(pdf) replace

egen charter=anymatch(requestedschool), values(167 168 175 176 196 211 212 266 328 330 391 392 393 394 426 428 429 435 436 438 441 443 444 445 446 462 481 485 496 498 512 513)
*keep if charter


twoway pcarrow log_s_t_1st_choice_randomization log_capacity log_s_t_randomization log_capacity if charter==1,/*
*/ title(Sample Size Gains: Charter Schools, color(black)) /*
*/ graphregion(color(white)) /*
*/ xscale(range(3.5 6)) /*
*/ xlabel(3.218 "25" 3.912 "50" 4.605 "100" 5.298 "200" 5.703 "300") /*
*/ ylabel(0 "0" 3.218 "25" 3.912 "50" 4.605 "100" 5.703 "300") /*
*/ xtitle(School Capacity (Log Scale), size(medsmall))  /*
*/ ytitle(# of Applicants s.t. Randomization (Log Scale), size(small)) /*
*/ mlcolor(red)/*
*/ mcolor(red)/*
*/ color(red)/*
*/ mfcolor(red)/*
*/ yscale(titlegap(2)) /*
*/ || scatter log_s_t_1st_choice_randomization log_capacity if charter==1, mcolor(black) msize(small) /*
*/ legend(lab(1 "# of Additional Applicants Subject to Randomization (Log Scale)") lab(2 "# of 1st Choice Applicants Subject to Randomization (Log Scale)") order(1 - " " 2 - " "))  /*
*/ xscale(range(3.5 6)) /*
*/ saving("${folder_output}figure_sample_size_gain_charter_school_new_def", replace)

graph export "${folder_output}figure_sample_size_gain_charter_school_new_def.pdf",  as(pdf) replace

twoway scatter log_s_t_1st_choice_randomization log_capacity if charter==1, mcolor(black) msize(small) /*
*/ title(Sample Size Gains: Charter Schools, color(black)) /*
*/ graphregion(color(white)) /*
*/ xscale(range(3.5 6)) /*
*/ xlabel(3.218 "25" 3.912 "50" 4.605 "100" 5.298 "200" 5.703 "300") /*
*/ ylabel(0 "0" 3.218 "25" 3.912 "50" 4.605 "100" 5.703 "300") /*
*/ xtitle(School Capacity (Log Scale), size(medsmall))  /*
*/ ytitle(# of Applicants s.t. Randomization at 1st Choice (Log Scale), size(small)) /*
*/ xscale(range(3.5 6)) /*
*/ saving("${folder_output}figure_sample_size_gain_charter_school_new_def_dots_only", replace)

graph export "${folder_output}figure_sample_size_gain_charter_school_new_def_dots_only.pdf",  as(pdf) replace

twoway pcarrow app_s_t_1st_choice_randomization capacity app_s_t_randomization capacity if charter==1,/*
*/ title(Sample Size Gains: Charter Schools, color(black)) /*
*/ graphregion(color(white)) /*
* xlabel(0(2.5)7.5) 
* ylabel(0(2.5)7.5) 
*/ xtitle(School Capacity, size(medsmall))  /*
*/ ytitle(# of Applicants s.t. Randomization, size(small)) /*
*/ mlcolor(red)/*
*/ mcolor(red)/*
*/ color(red)/*
*/ mfcolor(red)/*
*/ || scatter app_s_t_1st_choice_randomization capacity if charter==1, mcolor(black) msize(small) /*
*/ legend(lab(1 "# of Additional Applicants Subject to Randomization") lab(2 "# of Applicants Who Rank It 1st & Are Subject to Randomization") order(1 - " " 2 - " "))  /*
*/ saving("${folder_output}figure_sample_size_gain_charter_school_new_def_raw", replace)

graph export "${folder_output}figure_sample_size_gain_charter_school_new_def_raw.pdf",  as(pdf) replace

twoway pcarrow log_s_t_1st_choice_randomization log_capacity log_s_t_randomization log_capacity if charter==0 & capacity >= 25,/*
*/ title(Sample Size Gains: Non Charter Schools, color(black)) /*
*/ graphregion(color(white)) /*
*/ xscale(range(3.5 6)) /*
*/ xlabel(2.302 "10" 3.218 "25" 3.912 "50" 4.605 "100" 5.298 "200" 5.991 "400") /*
*/ ylabel(0 "0" 2.302 "10" 3.218 "25" 3.912 "50" 4.605 "100" 5.298 "200" 5.991 "400") /*
*/ xtitle(School Capacity (Log Scale), size(medsmall))  /*
*/ ytitle(# of Applicants s.t. Randomization (Log Scale), size(small)) /*
*/ mlcolor(red)/*
*/ mcolor(red)/*
*/ color(red)/*
*/ mfcolor(red)/*
*/ yscale(titlegap(2)) /*
*/ || scatter log_s_t_1st_choice_randomization log_capacity if charter==0, mcolor(black) msize(small) /*
*/ legend(lab(1 "# of Additional Applicants Subject to Randomization (Log Scale)") lab(2 "# of 1st Choice Applicants Subject to Randomization (Log Scale)") order(1 - " " 2 - " "))  /*
*/ xscale(range(3.5 6)) /*
*/ saving("${folder_output}figure_sample_size_gain_non_charter_school_new_def", replace)

graph export "${folder_output}figure_sample_size_gain_non_charter_school_new_def.pdf",  as(pdf) replace

*** Same plot but only if capacity is > 25 (stylistic request)
twoway pcarrow log_s_t_1st_choice_randomization log_capacity log_s_t_randomization log_capacity if charter==0 & capacity >= 25,/*
*/ title(Sample Size Gains: Non Charter Schools, color(black)) /*
*/ graphregion(color(white)) /*
*/ xscale(range(3.5 6)) /*
*/ xlabel(/*2.302 "10"*/3.218 "25" 3.912 "50" 4.605 "100" 5.298 "200" 5.991 "400") /*
*/ ylabel(0 "0" 2.302 "10" 3.218 "25" 3.912 "50" 4.605 "100" 5.298 "200" 5.991 "400") /*
*/ xtitle(School Capacity (Log Scale), size(medsmall))  /*
*/ ytitle(# of Applicants s.t. Randomization (Log Scale), size(small)) /*
*/ mlcolor(red)/*
*/ mcolor(red)/*
*/ color(red)/*
*/ mfcolor(red)/*
*/ yscale(titlegap(2)) /*
*/ || scatter log_s_t_1st_choice_randomization log_capacity if charter==0 & capacity >= 25 , mcolor(black) msize(small) /*
*/ legend(lab(1 "# of Additional Applicants Subject to Randomization (Log Scale)") lab(2 "# of 1st Choice Applicants Subject to Randomization (Log Scale)") order(1 - " " 2 - " "))  /*
*/ xscale(range(3.5 6)) /*
*/ saving("${folder_output}figure_sample_size_gain_non_charter_school_new_def_bigger_25", replace)

graph export "${folder_output}figure_sample_size_gain_non_charter_school_new_def_bigger_25.pdf",  as(pdf) replace
