	/*
Ignacio ROdriguez


We want to apply the formula and frequency pscore to each lottery draw and see how
the average compares to the calculated pscore. 

here I am trying to push the market size to have n of each type.

NOTE here we don't scale the lottery numbers and just run 100,000 draws for each market size

You call this subroutine with lottery_simulations_master, and the nice thing is you store results in matrix lotto

*/
	
	set trace off 
	
	
	local setup = 1
	local update_lotto = 0
	local rsd_results = 0
	
	$global kids_per_type = 
	matrix lotto = J(160,50,.)
	
	global direc "Y:\DPS\MDmeetsRD Denver\Data/"
	
	local draws 100000
	local number_of_kids = 4 * $kids_per_type

	if `setup' == 1 {
		
		clear 
	
		*this sets number of draws 
		 
		local number_of_kids = 4 * $kids_per_type
		local obs = `number_of_kids' * `draws'
		
		set obs `obs'
		
		*number of variable created sets number of kids in the economy
		*for 3 kids want 12 
		
		
		gen mod = mod(_n, `number_of_kids')
		bys mod: gen draw = _n
		
		ren mod id 
		replace id = `number_of_kids' if id == 0 
		
		sort draw id 
		
		gen type = ""
		replace type = "josh" if (id / $kids_per_type <= 1)
		replace type = "parag" if (id / $kids_per_type <= 2) & (id / $kids_per_type > 1)
		replace type = "atila" if (id / $kids_per_type <= 3) & (id / $kids_per_type > 2)
		replace type = "yusuke" if (id / $kids_per_type <= 4) & (id / $kids_per_type > 3)
		
		
		set seed 123456
		gen u = runiform() if type == "josh"
		set seed 3156463
		replace u = runiform() if type == "parag"
		set seed 87841
		replace u =runiform() if type == "atila"
		set seed 78564
		replace u = runiform() if type == "yusuke"
				
				
		ren u lottery_number

		* for n = 1, just getting the 24 possible draws but keeping unscaled lottery numbers
		if $kids_per_type == 1 {
		
		
			preserve 
				sort draw lottery_number 
				by draw : gen lottery_raw = _n 	
				
				levelsof type, local(ltype)
				
				foreach type of local ltype {
					bys draw: gen `type'_lotto = lottery_raw if type == "`type'"
				}
				
				collapse (max) *_lotto, by(draw)
				duplicates drop *_lotto, force 
				tempfile uniqueDraws
				sa `uniqueDraws'
			restore
			
			merge m:1 draw using `uniqueDraws', keepusing(draw)
			*only keep unique cases
			keep if _merge ==3
			drop _merge
		}

		
			
		local ltype  "atila josh parag yusuke"
		
		gen apply_c = (type == "josh" | type == "parag")
		bys draw: egen rankC = rank(lottery_number) if apply_c == 1 
		gen offerC = ( rankC <= $kids_per_type )
		
		//parag applies to b if he doesn't get an offer from C
		gen apply_b = (type == "parag" & apply_c == 1 & offerC == 0 | type == "atila")
		bys draw: egen rankB = rank(lottery_number) if apply_b == 1 
		gen offerB = ( rankB <= $kids_per_type  )
		
		
		gen apply_a = ( ( apply_b == 1 & offerB == 0) | type == "yusuke")
		bys draw: egen rankA = rank(lottery_number) if apply_a == 1 
		gen offerA = ( rankA <= $kids_per_type )
		
		bys draw: gen tauatmp = lottery_number if offerA == 1
		bys draw: gen taubtmp = lottery_number if offerB == 1
		bys draw: gen tauctmp = lottery_number if offerC == 1

		bys draw: egen taua = max(tauatmp)
		bys draw: egen taub = max(taubtmp)
		bys draw: egen tauc = max(tauctmp)	
		
		drop *tmp rank* apply_*


		order draw type lottery_number offer* tau*, first 

		//leave mida empty for Josh

		egen mida = rowmax(taub tauc) if type == "parag"
		replace mida = taub if type == "atila"
		replace mida = 0 if type == "yusuke"
		replace mida = . if type == "josh"  
		
		order mida, after(tauc)
	 
		gen midb = . 
		replace midb = tauc if type == "parag"
		replace midb = 0 	if type == "atila"
		
		gen midc = . 
		replace midc = 0	if type == "josh"
		replace midc = 0 	if type == "parag"		
		
		order midb midc, after(mida)
		sort draw id 
			
		gen typeid = . 

		replace typeid = 1 if type == "josh" 
		replace typeid = 2 if type == "parag"
		replace typeid = 3 if type == "atila"
		replace typeid = 4 if type == "yusuke"

		
		***********************************
		*for school a
		***********************************
		*get formula score
		gen formula_score1_a = ( 1 - mida ) * max( 0 , ( taua - mida) / ( 1 - mida ) ) if mida != 1
		replace formula_score1_a = 0 if mida == 1
		bys type: egen avg_pscore_formula1 = mean(formula_score1_a)
		
		*frequency score by mid	

		bys draw mida: egen freq_by_mid_a = mean(offerA)
		bys type: egen avg_freq_by_mid = mean(freq_by_mid_a)		

		*count as offer anyone who has lottery between  mida< lottery <taua

		gen freq_lottery_between_a = ( lottery_number <= taua & lottery_number > mida ) if mida != . //lottery number is in between mid and tau 
		bys draw type: egen freq_by_lottery_a = mean(freq_lottery_between_a)

		bys type: egen avg_freq_lottery_between_a = mean(freq_lottery_between_a)

		*add simulated score 
	
		bys type: egen simulated_score_type_a = mean(offerA)
		
		* do atila and parag share the same mid
		bys draw: egen atila_mid_tmp = max(mida) if type == "atila"
		bys draw: egen parag_mid_tmp  = max(mida) if type == "parag"
		
		bys draw: egen atila_mid = max(atila_mid_tmp) 
		bys draw: egen parag_mid  = max(parag_mid_tmp) 
		
		gen share_mid_a = ( parag_mid == atila_mid ) 
		
		* do atila and parag share the same simscore 
		bys draw: egen atila_counting_tmp_a = max(freq_by_lottery_a) if type == "atila"
		bys draw: egen parag_counting_tmp_a  = max(freq_by_lottery_a) if type == "parag"
		
		bys draw: egen atila_counting_a = max(atila_counting_tmp_a) 
		bys draw: egen parag_counting_a  = max(parag_counting_tmp_a) 
		
		gen share_counting_a = ( atila_counting_a == parag_counting_a ) 
		
		drop *tmp
		
		***********************************
		*for school b
		***********************************
		
		*get formula score
		gen formula_score1_b = ( 1 - midb ) * max( 0 , ( taub - midb) / ( 1 - midb ) ) if midb != 1
		replace formula_score1_b = 0 if midb == 1
		bys type: egen avg_pscore_formula1_b = mean(formula_score1_b)	
		
		*frequency score by mid	

		bys draw midb: egen freq_by_mid_b = mean(offerB)
		bys type: egen avg_freq_by_mid_b = mean(freq_by_mid_b)	
		
		*count as offer anyone who has lottery between  mida< lottery <taua

		gen freq_lottery_between_b = ( lottery_number <= taub & lottery_number > midb ) if midb != . //lottery number is in between mid and tau 
		bys draw type: egen freq_by_lottery_b = mean(freq_lottery_between_b)
		bys type: egen avg_freq_lottery_between_b = mean(freq_lottery_between_b)	
		
		*add simulated score 
	
		bys type: egen simulated_score_type_b = mean(offerB)
		
		***********************************
		*for school c
		***********************************
		
		*get formula score
		gen formula_score1_c = ( 1 - midc ) * max( 0 , ( tauc - midc) / ( 1 - midc ) ) if midc != 1
		replace formula_score1_c = 0 if midc == 1
		bys type: egen avg_pscore_formula1_c = mean(formula_score1_c)	
		
		*frequency score by mid	

		bys draw midc: egen freq_by_mid_c = mean(offerC)
		bys type: egen avg_freq_by_mid_c = mean(freq_by_mid_c)	
		
		*count as offer anyone who has lottery between  mida< lottery <taua

		gen freq_lottery_between_c = ( lottery_number <= tauc & lottery_number > midc ) if midc != . //lottery number is in between mid and tau 
		bys draw type: egen freq_by_lottery_c = mean(freq_lottery_between_c)
		bys type: egen avg_freq_lottery_between_c = mean(freq_lottery_between_c)	
		
		*add simulated score 
	
		bys type: egen simulated_score_type_c = mean(offerC)
		 		
		
		save "${direc}lotto_simulations_`draws'draws_${kids_per_type}kids_unscaled.dta", replace		


			
	}
	
	** the following section is to add stuff you didn't in the previous section - w/o having to rerun the whole thing 
	
	if `update_lotto' == 1 {	
		
		use "${direc}lotto_simulations_`draws'draws_${kids_per_type}kids_unscaled.dta", clear 
		
		gen pseudo_mid_a = mida
		replace pseudo_mid_a = taub if type == "parag"

		gen pseudo_offer_a = ( pseudo_mid_a < lottery_number  &  lottery_number <= taua  ) if pseudo_mid_a != . 
		
		bys draw pseudo_mid_a: egen pseudo_freq_a = mean(pseudo_offer_a)
		bys draw pseudo_mid_a: egen pseudo_freq_a_realOffer = mean(offerA)
		
		sa "${direc}lotto_simulations_`draws'draws_${kids_per_type}kids_unscaled_updated.dta", replace 
		
	}		
	
	* run the various estimates we have 
	if `rsd_results' == 1 {


		use "${direc}lotto_simulations_`draws'draws_${kids_per_type}kids_unscaled_updated.dta", clear 
		
		levelsof typeid, local(ltypeid)
		
		foreach i of local ltypeid {
			

			*first column is type id
			matrix lotto[`i', 1] = `i'
			matrix lotto[`i' + 4 , 1] = `i'
			matrix lotto[`i' + 8, 1] = `i'
			matrix lotto[`i' + 12, 1] = `i'
			matrix lotto[`i' + 16, 1] = `i'
			matrix lotto[`i' + 20, 1] = `i'

			*josh type (1) has no pscore so just skip
			if `i' == 1 {
				continue
			}
			*now we add formula 
			*if one kid, results in column 2, if 2 kids, result in column 3...			
			su formula_score1_a if typeid == `i', meanonly
			matrix lotto[`i',${kids_per_type} + 1] = `r(mean)'
			
			*frequency mid results next
			su freq_by_mid_a if typeid == `i', meanonly
			matrix lotto[`i' + 4 ,${kids_per_type} + 1] = `r(mean)'			

			
			*frequency lottery results next
			su freq_lottery_between_a if typeid == `i', meanonly
			matrix lotto[`i' + 8 ,${kids_per_type} + 1] = `r(mean)'	
			
			*simulated scores next
			su offerA if typeid == `i', meanonly
			matrix lotto[`i' + 12 ,${kids_per_type} + 1] = `r(mean)'
			
			*frequency mid but for cases where parag and atila do share mid
			su freq_by_mid_a if typeid == `i' & share_mid_a == 1, meanonly
			matrix lotto[`i' + 16 ,${kids_per_type} + 1] = `r(mean)'
			
			* simulation but for cases where parag and atila do share mid 
			su offerA if typeid == `i' & share_mid_a == 1, meanonly
			matrix lotto[`i' + 20 ,${kids_per_type} + 1] = `r(mean)'
			
			su share_mid_a 
			*if share mid is 1, then dont do these. 
			if `r(mean)' != 1 {
				*frequency mid but for cases where parag and atila don't share mid
				su freq_by_mid_a if typeid == `i' & share_mid_a == 0, meanonly
				matrix lotto[`i' + 24 ,${kids_per_type} + 1] = `r(mean)'
				
				* simulation but for cases where parag and atila don't share mid 
				su offerA if typeid == `i' & share_mid_a == 0, meanonly
				matrix lotto[`i' + 28 ,${kids_per_type} + 1] = `r(mean)'
			}
			
			
			
		}
		
		* adding in summary results for this simulation

		* same thing but rate
		
		* sample space is 4! = 24

		if ${kids_per_type} == 1 {
			local sample_space = 24
	
		}
		else{
			local sample_space = 100000
		}
		
		
		* number of draws where parag's mid == tauc
		unique draw if typeid == 2 & mida == tauc
		matrix lotto[ 34 , ${kids_per_type} + 1 ] = `r(sum)'		
	
		* same thing but rate 
		matrix lotto[ 35 , ${kids_per_type} + 1 ] = `r(sum)'/`sample_space'	
		
		*number of draws where parag's counting == atila's counting and they share same mid
		unique draw if share_counting_a == 1 & share_mid_a == 1
		local mid_and_counting = `r(sum)'
		
		* proportion of draws where parag's mid == atila's mid given they have the same counting score 
		unique draw if share_counting_a
		matrix lotto[ 36 , ${kids_per_type} + 1 ] = `mid_and_counting'/`r(sum)'			
		
		* proportion of draws where parag's score == atila's score  given they have the same mid 
		unique draw if share_mid_a == 1 & share_mid_a != .
		matrix lotto[ 37 , ${kids_per_type} + 1 ] = `mid_and_counting'/`r(sum)'		
		
		* variance of tau's 
		preserve 
			collapse (first) tau* (max) share_mid, by(draw)
			su taua
			matrix lotto[ 38 , ${kids_per_type} + 1 ] = `r(Var)'	
			
			su taub
			matrix lotto[ 39 , ${kids_per_type} + 1 ] = `r(Var)'	
			
			su tauc
			matrix lotto[ 40 , ${kids_per_type} + 1 ] = `r(Var)'	
		
			* variance of tau's given they share mid
			su taua if share_mid ==1
			matrix lotto[ 41 , ${kids_per_type} + 1 ] = `r(Var)'	
			
			su taub if share_mid ==1
			matrix lotto[ 42 , ${kids_per_type} + 1 ] = `r(Var)'	
			
			su tauc if share_mid ==1
			matrix lotto[ 43 , ${kids_per_type} + 1 ] = `r(Var)'		
		restore 
		
		foreach i of local ltypeid {

			*josh type (1) has no pscore so just skip
			if `i' == 1 {
				continue
			}
			*now we add formula 
			*if one kid, results in column 2, if 2 kids, result in column 3...			
			su formula_score1_a if typeid == `i' 
			matrix lotto[`i' + 44,${kids_per_type} + 1] = `r(Var)'
			
			*frequency mid results next
			su freq_by_mid_a if typeid == `i' 
			matrix lotto[`i' + 48 ,${kids_per_type} + 1] = `r(Var)'			
			local freq_var = `r(Var)'

			
			*frequency lottery results next
			su freq_lottery_between_a if typeid == `i' 
			matrix lotto[`i' + 52 ,${kids_per_type} + 1] = `r(Var)'	
			
			*simulated scores next
			su offerA if typeid == `i' 
			matrix lotto[`i' + 56 ,${kids_per_type} + 1] = `r(Var)'
			local simulated_var = `r(Var)'
			
			*formula but for cases where parag and atila do share mid
			su formula_score1_a if typeid == `i' & share_mid_a == 1 
			matrix lotto[`i' + 60 ,${kids_per_type} + 1] = `r(Var)'
			
			*frequency mid but for cases where parag and atila do share mid
			su freq_by_mid_a if typeid == `i' & share_mid_a == 1 
			matrix lotto[`i' + 64 ,${kids_per_type} + 1] = `r(Var)'
			
			* simulation but for cases where parag and atila do share mid 
			su offerA if typeid == `i' & share_mid_a == 1 
			matrix lotto[`i' + 68 ,${kids_per_type} + 1] = `r(Var)'
			
			su share_mid_a 
			
			*if share mid is 1, then dont do these. 
			if `r(mean)' != 1 {
				*frequency mid but for cases where parag and atila don't share mid
				su formula_score1_a if typeid == `i' & share_mid_a == 0 
				matrix lotto[`i' + 72 ,${kids_per_type} + 1] = `r(Var)'
				
				*frequency mid but for cases where parag and atila don't share mid
				su freq_by_mid_a if typeid == `i' & share_mid_a == 0 
				matrix lotto[`i' + 76 ,${kids_per_type} + 1] = `r(Var)'
				
				* simulation but for cases where parag and atila don't share mid 
				su offerA if typeid == `i' & share_mid_a == 0 
				matrix lotto[`i' + 80 ,${kids_per_type} + 1] = `r(Var)'
			}
			
			*formula when mid is shared 
			su formula_score1_a if typeid == `i' & share_mid_a == 1, meanonly
			matrix lotto[`i' + 84 ,${kids_per_type} + 1] = `r(mean)'
			
			
			su share_mid_a
			*if share mid is 1, then dont do these. 
			if `r(mean)' != 1 {
				*Formula but for cases where parag and atila don't share mid
				su formula_score1_a if typeid == `i' & share_mid_a == 0, meanonly
				matrix lotto[`i' + 88 ,${kids_per_type} + 1] = `r(mean)'

			}
						
			matrix lotto[`i' + 92 ,${kids_per_type} + 1] = `simulated_var' - `freq_var'
			matrix lotto[`i' + 96 ,${kids_per_type} + 1] = sqrt(`simulated_var' - `freq_var')
			
		}		
		
		*****************************************************************		
		***************** means for school b ****************************
		*****************************************************************
		
		foreach i of local ltypeid {
			local rowbump = 96
			
			if `i' == 1 | `i' == 4 { //if yusuke or josh skip 
				continue
			}
			
			su formula_score1_b if typeid == `i' 			
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= `r(mean)'				
			*recording type
			matrix lotto[`i' + `rowbump' ,1]	= `i'	
			
			su freq_by_mid_b if typeid == `i' 			
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= `r(mean)'
			*recording type
			matrix lotto[`i' + `rowbump' ,1]	= `i'
			
			su freq_lottery_between_b if typeid == `i' 			
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= `r(mean)'	
			*recording type
			matrix lotto[`i' + `rowbump' ,1]	= `i'
			
			su offerB if typeid == `i' 			
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= `r(mean)'	
			*recording type
			matrix lotto[`i' + `rowbump' ,1]	= `i'			
	
		}
		
		*save whatever is the last value of rowbum
		local rowreset = 112
		
		*****************************************************************		
		***************** means for school c ****************************
		*****************************************************************		
		foreach i of local ltypeid {
			*reset the row 
			local rowbump = `rowreset'
			
			if `i' == 3 | `i' == 4 { //if yusuke or atila skip 
				continue
			}
			
			su formula_score1_c if typeid == `i' 			
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= `r(mean)'	
			*recording type
			matrix lotto[`i' + `rowbump' ,1]	= `i'
			
			su freq_by_mid_c if typeid == `i' 			
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= `r(mean)'
			*recording type
			matrix lotto[`i' + `rowbump' ,1]	= `i'
			
			su freq_lottery_between_c if typeid == `i' 			
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= `r(mean)'	
			*recording type
			matrix lotto[`i' + `rowbump' ,1]	= `i'
			
			su offerC if typeid == `i' 			
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= `r(mean)'	
			*recording type
			matrix lotto[`i' + `rowbump' ,1]	= `i'
			
			/*
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= 	
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= 
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= 	
			*/			
		}
		
		di "rowbump `rowbump'"
		
		local rowreset = 128
		
		
		*****************************************************************		
		***************** psuedo offers and pseudo freq ****************************
		*****************************************************************		
		foreach i of local ltypeid {
			*reset the row 
			local rowbump = `rowreset'
			
			*josh type (1) has no pscore so just skip
			if `i' == 1 {
				continue
			}
			
			su pseudo_freq_a if typeid == `i' 			
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= `r(mean)'	
			*recording type
			matrix lotto[`i' + `rowbump' ,1]	= `i'
			
			su pseudo_offer_a if typeid == `i' 			
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= `r(mean)'	
			*recording type
			matrix lotto[`i' + `rowbump' ,1]	= `i'
			
			su pseudo_freq_a_realOffer if typeid == `i' 			
			local rowbump = `rowbump' + 4			
			matrix lotto[`i' + `rowbump' ,${kids_per_type} + 1]	= `r(mean)'	
			*recording type
			matrix lotto[`i' + `rowbump' ,1]	= `i'			
				
		}		
		
		
		sort draw id 
	}
	
	

	
