/*

this file looks for the modal values of theta and sbucket making mid across the 2000 simulations
it splits up the data so we can get them wide instead of reshaping which is alot slower. 


It uses files from 3.b create_master_analysis_files_for_2000_bucket_level.do located in 9. randomization pvalue_2000simulation\excercise
*/

clear 
set trace off

pause on

local clean_data "Y:\DPS\MDmeetsRD Denver\Data\"
local app_data "Y:\DPS\MDmeetsRD Denver\Data\application_data_2000_sims\"
local start 1
local end 2000
local millionBucketSim  "Y:/DPS/Raw Data/2014_12_02_DA_probabilities_and_results/probabilities/fixed/1000000_Probabilities.csv"
local graphs "Y:\DPS\MDmeetsRD Denver\Graphs\"



set scheme s1color 



local typedate 		20160425 	/* 20160208 */

local setup = 0
local getModes = 0
local lotteryRegressionsSetup = 0
local freqSetup = 0
local simSetup = 0
local formSetup = 0
local 2000SimSetup = 0

local finalGraphs = 1

if `setup' == 1 {
	
	use "`clean_data'fulltypes`typedate'.dta", clear
	drop type_*c*	
	destring masked, replace
	bys type_all: egen n_per_type = count(masked)
	tempfile types
	sa `types'

	forval sim = `start'/`end' {
		use "`clean_data'2000_sims_modal_file_from_`start'_to_`end'", clear
		keep dstlink masked big_theta`sim' bucket_that_makes_mid_`sim' bucket_id
		drop if masked == . 
		ren big_theta`sim' big_theta
		ren bucket_that_makes_mid_`sim' bucket_that_makes_mid
		gen sim = `sim'
		tempfile file`sim'
		sa `file`sim''
		di in red "at file `sim'"
	}

	clear

	forval sim = `start'/`end' {
		append using `file`sim''
		di in red "appending `sim'"
	}
	



	merge m:1 masked using `types', keep(match) nogen
	 

	// looping through all the dstlink
	bys type_all bucket_id: egen bucket_modemin =  mode( bucket_that_makes_mid), missing minmode
	by type_all bucket_id: egen bucket_modemax =  mode( bucket_that_makes_mid), missing maxmode
	by type_all bucket_id: egen theta_mode_min = mode( big_theta), missing minmode
	by type_all bucket_id: egen theta_mode_max = mode( big_theta), missing maxmode

		
	
	save  "`clean_data'2000_sims_modal_appended_from_`start'_to_`end'", replace

	*use mode() by level of dstlink using egen group. 
	*now you have the mode for each dstlink. 
}

if `getModes' == 1 {

	timer clear 1 
	timer on 1 
	use "`clean_data'2000_sims_modal_appended_from_`start'_to_`end'", clear 

	// how many times is equal to mode

	gen bucket_is_mode = ( bucket_that_makes_mid == bucket_modemin ) 
	gen theta_is_mode = (big_theta == theta_mode_min)
	
	bys bucket_id sim : egen applicants_modal_by_bucket_sim = total(  bucket_is_mode)	
	bys bucket_id  : egen avg_applicants_modal_by_bucket = mean(  applicants_modal_by_bucket_sim)
	egen avg_applicants_modal = mean(avg_applicants_modal_by_bucket)
	
	sort type_all bucket_id
	by type_all bucket_id: egen bucket_type_theta_proportion = mean(bucket_is_mode)
	by type_all bucket_id: egen bucket_type_bucket_proportion = mean(theta_is_mode)

	bys bucket_id: egen bucket_average_bucket = mean(bucket_is_mode)
	by bucket_id: egen bucket_average_theta = mean(theta_is_mode)
	
	bys bucket_id type: gen bucket_type_weight = _N
	*dropping down to type bucket level 
	*modal proportion vars are at type bucket level while bucket_average* are at bucket level. 
	keep type_all bucket_id bucket* theta* avg_applicants_modal_by_bucket applicants_modal_by_bucket_sim avg_applicants_modal n_per_type bucket_type_weight
	duplicates drop type_all bucket_id, force
	save "`clean_data'2000_sims_modal_bucket_type_from_`start'_to_`end'", replace 
	
	timer off 1 
	di in red "whole thing"
	timer list 1 
	
}

if `lotteryRegressionsSetup' {

	use "`clean_data'fulltypes`typedate'.dta", clear

	ren maskedid maskedstudentids
	drop type_*c*	
	tempfile types
	sa `types'

	
	use "`app_data'application_data_year_one_1", clear
	keep masked dstlink bucket_id status lotterynumber never_assigned sometimes_assigned conditionally_always_assigned ///
	MID effective_cutoff choiceselection 
	
	merge m:1 masked using `types', keep(match) nogen
	
	gen sim = 1 
	
	tempfile appendedFiles
	sa `appendedFiles'
	
	local startPlus = `start' + 1
	forval sim = `startPlus'/`end' {
	
		use "`app_data'application_data_year_one_`sim'", clear
		keep masked dstlink bucket_id status lotterynumber never_assigned sometimes_assigned conditionally_always_assigned ///
		MID effective_cutoff choiceselection 
		merge m:1 masked using `types', nogen keep(match)
		
		gen sim = `sim' 
		
		append using `appendedFiles'
		sa `appendedFiles', replace 
		di in red "at file `sim'"
		
	}
	
	save "`clean_data'2000_sims_lottery_mid_reg_`start'_to_`end'", replace
	
}

*** using simulated score from a million sims 
if `simSetup' == 1 {

	insheet using "Y:/DPS/Raw Data/2014_12_02_DA_probabilities_and_results/probabilities/fixed/1000000_Probabilities.csv", double clear
	drop number_of_simulations
	ren assignment_probability pscore_sim_bucket
	sa "`clean_data'2000_sims_bucket_simulated_scores.dta", replace 

}

* this is to get the simscore for first second 3rd choice. 
if `2000SimSetup' {
	
	use "`clean_data'2000_sims_offer_file_from_`start'_to_`end'.dta", clear
	 
	
	preserve 
		*aadding types 
		use "`clean_data'fulltypes`typedate'.dta", clear
		drop type_*c*	
		destring masked, replace
		bys type_all: egen n_per_type = count(masked)
		tempfile types
		sa `types'
	
	restore

	merge m:1 masked using `types'

	keep if _merge == 3
	egen tmpsim_score_1 = rowmean(offer*) if inlist(choice_strict, 1)	
	egen tmpsim_score_12 = rowmean(offer*) if inlist(choice_strict , 1, 2)	

	bys type bucket_id: egen double sim_score_1 = mean(tmpsim_score_1)
	bys type bucket_id: egen double sim_score_12 = mean(tmpsim_score_12)	
	drop offer* tmp*
	
	keep dstlink masked type sim_score*
	 
	sa "`clean_data'2000_sims_bucket_sim_scores_with_123.dta", replace 

}

if `formSetup' == 1 {
	use "`clean_data'2000_sims_formula_file_from_`start'_to_`end'.dta", clear
	egen pscore_bucket_formula = rowmean(pscore_formula_*) 
	drop pscore_formula_*
	ren pscore_bucket_formula pscore_formula_bucket
	sa "`clean_data'2000_sims_bucket_formula_scores.dta", replace 


}

if `freqSetup' == 1 {

	* merge freq scores and analysis the modal groups
	* this is at bucket level i.e. dstlink level. 
	
	use "`clean_data'2000_sims_frequency_file_from_`start'_to_`end'.dta", clear 

	order frequency_offer_full_*, before(frequency_offer_1_1)
	egen freq_full = rowmean(frequency_offer_full_*)

	egen freq_1 = rowmean(frequency_offer_1_*)
	drop frequency_offer_1_*
	egen freq_12 = rowmean(frequency_offer_12_*)
	drop frequency_offer_12_*
	egen freq_123 = rowmean(frequency_offer_123_*)
	drop frequency_offer_123_*
	
	/*
	foreach limit of numlist 10 50 100 500 1000 1500 2000 {

		egen freq_full_`limit' = rowmean(frequency_offer_full_1 - frequency_offer_full_`limit') 		

	}
	*/
	gen freq_full_1  = frequency_offer_full_1  	
	drop frequency_offer_full_*	
	sa "`clean_data'2000_sims_bucket_frequency_scores.dta", replace 

}



if `finalGraphs' == 1 {
						
	 insheet using "`millionBucketSim'", clear
	ren assignment_probability pscore_sim_bucket
	drop number_of_simulations
	tempfile simBucket 
	sa `simBucket'
	
	*aadding types 	
	use "`clean_data'fulltypes`typedate'.dta", clear
	destring masked, replace
	bys type_all: egen n_per_type = count(masked)
	tempfile types
	sa `types'

	use "`clean_data'2000_sims_bucket_frequency_scores.dta", clear  
	
	merge m:1 masked using `types' , nogen
	
	*modal data is unique by bucket id and type 
	
	merge m:1 bucket_id type using "`clean_data'2000_sims_modal_bucket_type_from_`start'_to_`end'"

	drop if _merge != 3
	drop _merge
	
	
	*merging 2000 sim formula score 
	merge 1:1 dstlink using "`clean_data'2000_sims_bucket_formula_scores.dta", nogen
	*merging bucket level 1000000 sim score 
	merge 1:1 dstlink using `simBucket'
	drop if _merge != 3
	drop _merge
	
	* adding in simscore for 1st and second choice with the 2000 sims
	merge 1:1 dstlink using "`clean_data'2000_sims_bucket_sim_scores_with_123.dta"
	drop _merge


	* kind of arbitrary but I'll use the min mode as the mode. 
	*that way you exclude missing if it is the modal value
	bys bucket_id bucket_modemin theta_mode_min: egen modal_sim_b 		= mean(pscore_sim_bucket)
	bys bucket_id bucket_modemin theta_mode_min: egen modal_freq_full_b = mean(freq_full)
	bys bucket_id bucket_modemin theta_mode_min: egen modal_freq_1_b	= mean(freq_1)
	bys bucket_id bucket_modemin theta_mode_min: egen modal_freq_12_b	= mean(freq_12)
	bys bucket_id bucket_modemin theta_mode_min: egen modal_freq_123_b	= mean(freq_123)
	bys bucket_id bucket_modemin theta_mode_min: egen modal_form_b		= mean(pscore_formula_bucket)
	
	tempfile withFamily
	sa `withFamily'
	
	* rounding sim values
	
	gen sim01 = round(pscore_sim_bucket, 0.01)
	gen sim001 = round(pscore_sim_bucket, 0.001)

	**** NO FAMILY LINK KIDS  *******
	
	* dropping apps with family link
	gen family_link = (family!="")
	drop if family_link == 1	
	
	
	*** First and second choice comparison of freq and sim
	
	twoway 		(scatter freq_1 sim_score_1, msize(vsmall) mcolor(black)) ///	
										(function y = x, lwidth(medthick) lcolor(red))  ///
										, ///
										xtitle("Simulated Score") ///
										ytitle("Frequency" "Score", orientation(horizontal)) ///
										title("Frequency vs Simulated for first" "choice buckets", size(medlarge)) ///
										legend(order(2 "45 Degree Line")) ///
										/// note("OLS statistics" "R-sq = `freq_rsq'" "simscore coeff. = `freq_coeff'" "simscore se = `freq_se'" "constant = `freq_cons'" "constant se = `freq_cons_se'") ///
										plotregion(style(none)) ///
										yscale(titlegap(1))  ///
										ysize(5)	
	graph export "`graphs'first choice frequency simulated (bucket-level).png", replace width(4000)	

	twoway 		(scatter freq_1 sim_score_1 if theta_mode_min == "c", msize(vsmall) mcolor(black)) ///	
										(function y = x, lwidth(medthick) lcolor(red))  ///
										, ///
										xtitle("Simulated Score") ///
										ytitle("Frequency" "Score", orientation(horizontal)) ///
										title("Frequency vs Simulated for first" "choice buckets", size(medlarge)) ///
										legend(order(2 "45 Degree Line")) ///
										/// note("OLS statistics" "R-sq = `freq_rsq'" "simscore coeff. = `freq_coeff'" "simscore se = `freq_se'" "constant = `freq_cons'" "constant se = `freq_cons_se'") ///
										plotregion(style(none)) ///
										yscale(titlegap(1))  ///
										ysize(5)	
	graph export "`graphs'first choice frequency simulated (bucket-level) modal value of conditional.png", replace width(4000)	

	twoway 		(scatter  freq_12 sim_score_12 , msize(vsmall) mcolor(black)) ///	
										(function y = x, lwidth(medthick) lcolor(red)) /// 
										, ///
										xtitle("Simulated Score") ///
										ytitle("Frequency" "Score", orientation(horizontal)) ///
										title("Frequency vs Simulated for first" "and second choice buckets", size(medlarge)) ///
										legend(order(2 "45 Degree Line")) ///
										/// note("OLS statistics" "R-sq = `freq_rsq'" "simscore coeff. = `freq_coeff'" "simscore se = `freq_se'" "constant = `freq_cons'" "constant se = `freq_cons_se'") ///
										plotregion(style(none)) ///
										yscale(titlegap(1))  ///
										ysize(5)		
										
	graph export "`graphs'first and second choice frequency simulated (bucket-level).png", replace width(4000)	
	
	* rounding to 0.01 all applications
	preserve 
		
		*get means of frequency and formula by rounded simscore 
		bys sim01: egen freq_full_01_avg = mean(freq_full)
		bys sim01: egen formula_01_avg = mean(pscore_formula)
		
		
		* getting reg stats 
		*frequency
		reg freq_full_01_avg sim01

		local freq_rsq 		= round(`e(r2_a)', 0.0001)
		local freq_cons 	= round(_b[_cons], 0.0001)
		local freq_cons_se 	=  ceil(_se[_cons]*10000)/10000

		local freq_coeff	= round(_b[sim01], 0.0001)
		local freq_se 		= ceil(_se[sim01]*10000)/10000
		
		* formula
		reg formula_01_avg sim01

		local form_rsq 		= round(`e(r2_a)', 0.0001)
		local form_cons 	= round(_b[_cons], 0.0001)
		local form_cons_se 	=  ceil(_se[_cons]*10000)/10000
		
		local form_coeff	=  floor(_b[sim01]*10000)/10000
		local form_se 		= ceil(_se[sim01]*10000)/10000
		
	
		
		collapse (mean) freq_full_01_avg formula_01_avg, by(sim01)
		*** Frequency 
		twoway 		(scatter freq_full_01_avg sim01, msize(vsmall) mcolor(black)) ///	
											(function y = x, lwidth(medthick) lcolor(red)) /// 
											, ///
											xtitle("Simulated Score") ///
											ytitle("Frequency" "Score", orientation(vertical)) ///
											title("Average Frequency Score vs Simulated Score" , size(medlarge)) ///
											legend(off) ///
											/// note("OLS statistics" "R-sq = `freq_rsq'" "simscore coeff. = `freq_coeff'" "simscore se = `freq_se'" "constant = `freq_cons'" "constant se = `freq_cons_se'") ///
											plotregion(style(none)) ///
											yscale(titlegap(1))  ///
											ysize(5)											
									
		graph export "`graphs'Figure 1a frequency vs simulated bucket level scores (binned to sim 01).png", replace width(4000)	
		// export as wmf so you can edit it in word or ppt to make 45 degree line transparent. 
		graph export "`graphs'Figure 1a frequency vs simulated bucket level scores (binned to sim 01).wmf", replace 
		
		*Formula
		twoway 		(scatter formula_01_avg sim01, msize(vsmall) mcolor(black)) ///	
											(function y = x, lwidth(medthick) lcolor(red)) /// 
											, ///
											xtitle("Simulated Score") ///
											ytitle("Formula" "Score", orientation(vertical)) ///
											title("Average Formula Score vs. Simulated Score", size(medlarge)) ///
											legend(off) ///
											/// note("OLS statistics" "R-sq = `form_rsq'" "simscore coeff. = `form_coeff'" "simscore se = `form_se'" "constant = `form_cons'" "constant se = `form_cons_se'") ///
											plotregion(style(none)) ///											
											yscale(titlegap(1))  ///
											ysize(5)

		graph export "`graphs'Figure 1b formula vs simulated bucket level scores (binned to sim 01).png", replace width(4000)	
		graph export "`graphs'Figure 1b formula vs simulated bucket level scores (binned to sim 01).wmf", replace 
			
	restore	
	
	preserve
		
		*get means of frequency and formula by rounded simscore 
		bys sim01: egen freq_full_01_avg = mean(freq_full)
		bys sim01: egen formula_01_avg = mean(pscore_formula)
		
		*get means of frequency and formula by rounded simscore 
		bys sim01: egen freq_full_01_avg_ch = mean(freq_full) if appstudy == 1
		bys sim01: egen formula_01_avg_ch = mean(pscore_formula) if appstudy == 1
		
		bys freq_full_01_avg: gen apps_per_freq_all = _N
		bys freq_full_01_avg_ch: gen apps_per_freq_ch = _N
		
		su apps_per_freq_all if freq_full_01_avg != . 
		su apps_per_freq_ch if freq_full_01_avg_ch != .
		
	restore
	
	
	* rounding to 0.01 charter buckets applications
	preserve 
	
		keep if appstudy == 1
		
		*get means of frequency and formula by rounded simscore 
		bys sim01: egen freq_full_01_avg = mean(freq_full)
		bys sim01: egen formula_01_avg = mean(pscore_formula)
		
		* getting reg stats 
		*frequency
		reg freq_full_01_avg sim01

		local freq_rsq 		= round(`e(r2_a)', 0.0001)
		local freq_cons 	= round(_b[_cons], 0.0001)
		local freq_cons_se 	= round(_se[_cons], 0.0001)

		local freq_coeff	= round(_b[sim01], 0.0001)
		local freq_se 		= round(_se[sim01], 0.0001)	
		
		* formula
		reg formula_01_avg sim01

		local form_rsq 		= round(`e(r2_a)', 0.0001)
		local form_cons 	= round(_b[_cons], 0.0001)
		local form_cons_se 	= ceil(_se[_cons]*10000)/10000

		local form_coeff	= round(_b[sim01], 0.0001)
		local form_se 		= round(_se[sim01], 0.0001)	
		
		
		
		collapse (mean) freq_full_01_avg formula_01_avg, by(sim01)
		*** Frequency 
		twoway 		(scatter freq_full_01_avg sim01, msize(vsmall) mcolor(black)) ///	
											(function y = x, lwidth(medthick) lcolor(red)) /// 
											, ///
											xtitle("Simulated Score") ///
											ytitle("Frequency" "Score", orientation(horizontal)) ///
											title("Average Bucket Frequency Score Plotted" "Against Simulated Score (binned to 0.01)" "Only Charter schools", size(medlarge)) ///
											legend(order(2 "45 Degree Line")) ///
											/// note("OLS statistics" "R-sq = `freq_rsq'" "simscore coeff. = `freq_coeff'" "simscore se = `freq_se'" "constant = `freq_cons'" "constant se = `freq_cons_se'") ///
											plotregion(style(none)) ///
											ysize(5) ///
											yscale(titlegap(1))

											
		graph export "`graphs'Figure 2a frequency vs simulated bucket level scores only charters (binned to sim 01).png", replace width(4000)		
		graph export "`graphs'Figure 2a frequency vs simulated bucket level scores only charters (binned to sim 01).wmf", replace	
		
		*Formula
		twoway 		(scatter formula_01_avg sim01, msize(vsmall) mcolor(black) ) ///	
											(function y = x, lwidth(medthick) lcolor(red)) /// 
											, ///
											xtitle("Simulated Score") ///
											ytitle("Formula" "Score" , orientation(horizontal)) ///
											title("Average Bucket Formula Score Plotted" "Against Simulated Score (binned to 0.01)" "Only Charter schools", size(medlarge)) ///
											legend(order(2 "45 Degree Line")) ///
											/// note("OLS statistics" "R-sq = `form_rsq'" "simscore coeff. = `form_coeff'" "simscore se = `form_se'" "constant = `form_cons'" "constant se = `form_cons_se'") ///
											plotregion(style(none)) ///
											ysize(5) ///
											yscale(titlegap(1)) 

											
		graph export "`graphs'Figure 2b formula vs simulated bucket level scores only charters (binned to sim 01).png", replace width(4000)	
		graph export "`graphs'Figure 2b formula vs simulated bucket level scores only charters (binned to sim 01).wmf", replace 	
		
	restore	
	
		
		
		*get means of frequency and formula by rounded simscore 
		bys modal_sim_b: egen freq_full_modal_avg = mean(modal_freq_full_b)
		bys modal_sim_b: egen formula_modal_avg = mean(modal_form_b)

			
		reg freq_full_modal_avg modal_sim_b

		local freq_rsq 		= round(`e(r2_a)', 0.0001)
		local freq_cons 	= round(_b[_cons], 0.0001)
		local freq_cons_se 	= ceil(_se[_cons]*10000)/10000

		local freq_coeff	= round(_b[modal_sim_b], 0.0001)
		local freq_se 		= ceil(_se[modal_sim_b]*10000)/10000
		
		* formula
		reg formula_modal_avg modal_sim_b
		

		local form_rsq 		= round(`e(r2_a)', 0.0001)
		local form_cons 	= round(_b[_cons], 0.0001)
		local form_cons_se 	= ceil(_se[_cons]*10000)/10000
		
		
		local form_coeff	= round(_b[modal_sim_b], 0.0001)
		local form_se 		= ceil(_se[modal_sim_b]*10000)/10000
		*get floating point error so do this
		local form_coeff	= floor(`form_coeff'*10000)/10000
		

			** with family
		preserve
		
			collapse (mean) freq_full_modal_avg formula_modal_avg, by(modal_sim_b)

			twoway 		(scatter freq_full_modal_avg modal_sim_b, msize(vsmall) mcolor(black)) ///	
												(function y = x, lwidth(medthick) lcolor(red)) /// 
												, ///
												ytitle("Average" "Frequency" "Score", orientation(horizontal) ) ///
												xtitle("Simulated Score") ///											
												title("Frequency Scores" "grouped by modal values" ) ///
												legend(order( 2 "45 degree line"))	///
												/// note("OLS statistics" "R-sq = `freq_rsq'" "simscore coeff. = `freq_coeff'" "simscore se = `freq_se'" "constant = `freq_cons'" "constant se = `freq_cons_se'") ///
												plotregion(style(none)) ///
												ysize(5) ///
												yscale(titlegap(1)) 

												
			graph export "`graphs'Figure 4a frequency vs simulated bucket level grouped by modal values.png", replace width(4000)
			graph export "`graphs'Figure 4a frequency vs simulated bucket level grouped by modal values.wmf", replace
			
			twoway 		(scatter formula_modal_avg modal_sim_b,  msize(vsmall) mcolor(black)) ///	
												(function y = x, lwidth(medthick) lcolor(red)) /// 
												, ///
												ytitle("Average" "Formula" "Score", orientation(horizontal) ) ///
												xtitle("Simulated Score") /// 
												title("Formula Scores" "grouped by modal values" ) ///
												legend(order( 2 "45 degree line"))	///
												/// note("OLS statistics" "R-sq = `form_rsq'" "simscore coeff. = `form_coeff'" "simscore se = `form_se'" "constant = `form_cons'" "constant se = `form_cons_se'") ///
												plotregion(style(none)) /// 
												ysize(5) ///
												yscale(titlegap(1)) 

												
			graph export "`graphs'Figure 4b formula vs simulated bucket level grouped by modal values.png", replace width(4000)		
			graph export "`graphs'Figure 4b formula vs simulated bucket level grouped by modal values.wmf", replace 

		restore
		
}





















