	/* 	Mayara Silva
	1/8/2015
	
	This code imports a modeified version of the assignment file (Based on Atila's DA simulation)
	and produces propensity score files for it.
	
	Ignacio update
	
	This version creates frequency score for aplicants who only rank school s first or second. 
	We need to create a new choice variable that goes from 1-last of students choice. 
	its confusing because the choice variable has values such as 11, 12, 13 which are really also the student's 
	first choice as well, but from aren't the first choice in the list used in the algorithm. 
*/

clear
clear matrix
clear mata
set maxvar 32767
set more off
local cleandata "Y:\DPS\MDmeetsRD Denver\Data\"
local dta "Y:\DPS\MDmeetsRD Denver\Data\simulated_offers\"
local appdta "Y:\DPS\MDmeetsRD Denver\Data\application_data_2000_sims\"

local master_AT_RUN_bucket = 1 

local startFile = 1
local endFile = 2000

local simscoredate		20150603	/*Date of latest simulation mdrd analysis file for realized offers*/
local theorydate		20150708	/*Date of latest theoretical mdrd analysis file for realized offers*/
local atrundate			20150713	/*Date of latest master at run file containing simulated offers*/

local simsdate 		20150603 /*20150428*/
local outputdate	20160601 /*20150713*/



*****************************************************************
* Create file with school-level offers (DA_offers_master_AT_RUN)*
*****************************************************************

if `master_AT_RUN_bucket'==1{		
	
	u "`dta'DA_offers_master_AT_RUN_2000sim `atrundate'.dta", clear

	ren maskedstudentids maskedid
	ren requestedschool appschoolnum
	
	*Not sure what rank=99 means, ask Atila
	gen apprank = choiceselection
	replace apprank = apprank*10 if apprank<10
	replace apprank = floor(apprank/10)
	
	drop if maskedid=="#VALUE!"
	destring maskedid, ignore("#VALUE!") replace
	
	* just to add in bucket id

	merge 1:1 dstlink using "`appdta'application_data_year_one_1", keepusing(bucket_id) nogen

	* need to drop the empty masked which are reintroduced with the merge
	drop if masked == . 
	* need to drop empty buckets. These are invalid applications or applications to schools with no capacity in the first sim of year 1. 
	drop if bucket_id == . 
	drop lotterynumber*
	
	* making new choice variable, which lists choices in the same order as the one used in the DA algorithm. 
	sort masked choice
	bys masked: gen choice_strict = _n
	
	timer clear 1 
	timer on 1 
	
	
	
	*Offers for each status
	forval i = `startFile'/`endFile'{
	
		merge 1:1 dstlink using "`appdta'application_data_year_one_`i'", nogen /*keep(3)*/ keepusing(MID conditionally_always_assigned never_assigned sometimes_assigned bucket_that_makes_mid frequency_offer* pscore_formula)
		
		gen offer`i' = (status`i'=="A")

		
		* need to drop the empty masked which are reintroduced with the merge
		drop if masked == .
		
		* recreating frequency score with first and second choice. 
		drop frequency_offer_1 frequency_offer_12 frequency_offer_123
		
		*first choice
		bys bucket_id MID never_assigned sometimes_assigned conditionally_always_assigned: egen frequency_offer_1 =mean(offer`i') if inlist(choice_strict, 1)
		*for those never assigned, MID is irrelevant. need to take it into account
		bys bucket_id never_assigned: egen frequency_offer_intermediate_1=mean(offer`i') if inlist(choice_strict, 1)
		replace frequency_offer_1=frequency_offer_intermediate_1 if never_assigned==1	
		
		*first or second choice 
		bys bucket_id MID never_assigned sometimes_assigned conditionally_always_assigned: egen frequency_offer_12 =mean(offer`i') if inlist(choice_strict, 1, 2)
		*for those never assigned, MID is irrelevant. need to take it into account
		bys bucket_id never_assigned: egen frequency_offer_intermediate_12=mean(offer`i') if inlist(choice_strict, 1, 2)
		replace frequency_offer_12=frequency_offer_intermediate_12 if never_assigned==1

		*first or second or third choice 
		bys bucket_id MID never_assigned sometimes_assigned conditionally_always_assigned: egen frequency_offer_123 =mean(offer`i') if inlist(choice_strict, 1,2,3)
		*for those never assigned, MID is irrelevant. need to take it into account
		bys bucket_id never_assigned: egen frequency_offer_intermediate_123=mean(offer`i') if inlist(choice_strict, 1 , 2, 3)
		replace frequency_offer_123=frequency_offer_intermediate_123 if never_assigned==1			
		
		drop *offer_intermediate*		
		
		* creating big theta with letters 
		gen big_theta`i' = ""
		replace big_theta`i' = "a" if conditionally_always_assigned == 1
		replace big_theta`i' = "n" if never_assigned == 1
		replace big_theta`i' = "c" if sometimes_assigned == 1
		
		ren frequency_offer frequency_offer_full
				
		drop conditionally_always_assigned never_assigned sometimes_assigned
		renvars bucket_that_makes_mid MID frequency_offer_full frequency_offer_1 frequency_offer_12 frequency_offer_123 pscore_formula, postfix("_`i'")

	}
	drop status*
	timer off 1 
	timer list 1	
	
	*Study charters
	gen appstudy= (	inlist(appschoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(appschoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444,445, 446) | ///
						inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
					)
	
	
	order offer*, last
	
	*splitting up files so we don't end up with one humoungous file. 
	
	* file to figure out modal stuff
	preserve 
		drop offer* frequency_offer* pscore_formula*  MID*
		sa "`cleandata'2000_sims_modal_file_from_`startFile'_to_`endFile'.dta", replace
	restore
	
	*file with MIDS just in case. 
	preserve
		drop offer* frequency_offer* pscore_formula* big_theta* bucket_that_makes_mid*
		sa "`cleandata'2000_sims_mid_file_from_`startFile'_to_`endFile'.dta", replace
	restore
	
	*file with formula scores. 
	preserve
		drop  offer* frequency_offer* big_theta* bucket_that_makes_mid* MID*
		sa "`cleandata'2000_sims_formula_file_from_`startFile'_to_`endFile'.dta", replace
	restore
	
	*file with frequency scores. 
	preserve
		drop  offer* pscore_formula* big_theta* bucket_that_makes_mid* MID*
		sa "`cleandata'2000_sims_frequency_file_from_`startFile'_to_`endFile'.dta", replace
	restore	
	
	*file with offers for simulated score
	preserve
		drop pscore_formula* frequency_offer* big_theta* bucket_that_makes_mid* MID*
		sa "`cleandata'2000_sims_offer_file_from_`startFile'_to_`endFile'.dta", replace
	restore		
}

