/*

All the dofiles export a matrix which you can use to look up specific values using the row and column names. 
For this do file, the relevant columns are: 
Columns: denver	applicants	study_app	simulation01_rnd_001	simulation01_rnd_001_study
Rows : "baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted" 
"baseliness_math baseliness_reading baseliness_writing"

*/


clear all
cap log close
set more off
set trace off
set tracedepth 1 
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"
pause on

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"
local rawdata "Y:\DPS\Raw Data\"

matrix demo			  = J(30,30,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,40,.)
matrix subtwosls	  = J(500,40,.)
matrix twosls_nocov    		   = J(500,40,.)
matrix twosls_scorecov 		   = J(500,40,.)
matrix twosls_fcsample   		= J(50	0,40,.)

matrix qualbalance    = J(500,20,.)
matrix qualtwosls	  = J(500,20,.)
matrix qualsubtwosls  = J(500,20,.)

*Dates of analysis files
local pscoredate_year2	20170310 	/*20160516 20160510 20160428 20160411 20160208 20160112 20151120 20150514 */
local pscoredate_year1  20170310 /*20160516    	20160516 20160510 20160421  20160208 */
local simsdate 			20150603 /* 20150428 */
local qualdate_year1 	20160715 /*20150708  20150420 */
local qualdate_year2	20160715 /*20160303*/
local qualdate			20160303
local qualpscoredate	20160208
local appdatayr1		20170310
local appdatayr2		20170310


local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_*"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1        		/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
local outsheet_rescorr=0	/* If want to outsheet matrix with residual correlations */





	************************************************
	**************** Student Demo ******************
	************************************************
		

		local row = 1
		local col = 1
		
		*Denver mean : 3-9 graders in 2011-2012
			use "`cleandata'sims_csap_long_20102014 `simsdate'", clear
			
			unique maskedid
			
			keep if inlist(grade,"3","4","5","6","7","8","9") & (spyear==2012 | spyear == 2013)
			
			*Ignacio: these are the year 2 study schools, not year 1. 
			
			*Study charters for years 1 and 2
			gen study = (((inlist(schoolnum, 167, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(schoolnum, 330, 391, 392, 393, 394, 426, 428, 429, 435, 436, 437, 438, 441, 443, 444, 445, 446) | ///
						inlist(schoolnum, 462, 481, 485, 496, 498, 512, 513, 517)) & spyear == 2013) | ///
						((inlist(schoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(schoolnum, 330, 391, 392, 393, 394, 426, 428, 429, 435, 436, 438, 441, 443, 444, 445, 446) | ///
						inlist(schoolnum, 462, 481, 485, 496, 498, 512, 513)) & spyear == 2012) ///
					)

			**Study no-excuses charters
			gen noex =  (((inlist(schoolnum, 392, 393, 394, 426, 428, 429, 435, 436, 437) | ///
						inlist(schoolnum, 438, 443, 444, 445, 462, 481, 498, 513, 517)) & spyear == 2013) | ///
						((inlist(schoolnum, 392, 393, 394, 426, 428, 429, 435, 436) | ///
						inlist(schoolnum, 438, 443, 444, 445, 462, 481, 498, 513)) & spyear == 2012) ///
					)
	
			**Study non no-excuses charters
			gen nonnoex = (((inlist(schoolnum, 167, 175, 176, 196, 211, 212) | ///
						  inlist(schoolnum, 266, 328, 330, 391, 441, 446, 485, 496, 512)) & spyear == 2013) | ///
						  ((inlist(schoolnum, 167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
							 inlist(schoolnum, 330, 391, 441, 446, 485, 496, 512)) & spyear == 2012) ///
					)
					
			gen spyear2012_flag =  (spyear==2012)

			
			*Means (exclude alternatives for scores)
			foreach var of varlist study female hisp black foodfred sped lep bilingual gifted white asian ss_math ss_reading ss_writing spyear2012_flag noex nonnoex {
				
				qui summ `var'
				matrix demo[`row',`col']   = r(mean)
				
				qui summ `var' if study == 1
				matrix demo[`row', `col'+1] = r(mean)
				
				qui summ `var' if noex == 1
				matrix demo[`row', `col'+2] = r(mean)
				
				qui summ `var' if nonnoex == 1
				matrix demo[`row', `col'+3] = r(mean)
				local ++row
			}
			
			*Count
				unique maskedid
				local denver_N = r(sum)
				
				
				count if charter == 1
				local charter_N = r(N)
				
				count if noex == 1
				local noex_N = r(N)
				
				count if nonnoex == 1
				local nonnoex_N = r(N)
		
		*Charter applicants who were in Denver at baseline (simulated score not calculated by CMOs. Calculated just for whether charter app or not.)
			
			foreach sch_type in study /*noex nonnoex */	{
			
			
			*starting with year 1 pscore
			use "`cleandata'mdrd_denver_`sch_type'_simulation `pscoredate_year1'", clear
				
			*Go get other scores - assert that files have same obs
			merge 1:1 maskedid appschoolnum using "`cleandata'mdrd_denver_`sch_type'_formula `pscoredate_year1'", keepusing(pscore_`sch_type'_formula)
			assert _merge==3 
			drop _merge
			
			merge 1:1 maskedid appschoolnum using "`cleandata'mdrd_denver_`sch_type'_frequency `pscoredate_year1'", keepusing(pscore_`sch_type'_frequency)
			assert _merge==3 
			drop _merge
				
			*Drop if not in Denver at baseline or not an applicant to any schools in year 1 
			drop if baselinespyear!=2012 | appschoolnum==. | !inlist(appgrade,"4","5","6","7","8","9","10")
			tempfile year1_scores
			sa `year1_scores'
			
			
			
			*adding year 2 applications and pscore
			
			use "`cleandata'mdrd_denver_`sch_type'_year_2_simulation `pscoredate_year2'", clear
			
			merge 1:1 maskedid appschoolnum using "`cleandata'mdrd_denver_`sch_type'_year_2_formula `pscoredate_year2'", keepusing(pscore_`sch_type'_formula)
			assert _merge==3
			drop _merge
			merge 1:1 maskedid appschoolnum spyear using "`cleandata'mdrd_denver_`sch_type'_year_2_frequency `pscoredate_year2'", keepusing(pscore_`sch_type'_frequency)
			assert _merge==3
			drop _merge
			
			*Drop if not in Denver at baseline or not an applicant to any schools in year 2 	
			drop if ( baselinespyear!=2013  | appschoolnum==. | !inlist(appgrade,"4","5","6","7","8","9","10")) 	
						
			**********************************************
			*** We are rounding simulated score to 0.001 here. 
			*** Unrounded doesn't quite work - too much support. 
			*** Simply replacing simulated score with rounded equivalent because
			*** it keeps the matrix logic intact
			**********************************************
			
			replace pscore_study_simulation = round(pscore_study_simulation, 0.001)

			append using `year1_scores', gen(year1) 
			
			/*
			*dropping duplicates from year 2. 
			bys masked: egen min_year = min(baselinespyear)
			bys masked: egen max_year = max(baselinespyear)
			drop if min_year != max_year & baselinespyear == 2013			
			*/
			bys maskedid baselinespyear: egen apply_`sch_type' = max(app`sch_type')
			*Enrolls at charter flag
			bys maskedid baselinespyear: egen enroll_any_`sch_type' 	= max(`sch_type')	
			bys maskedid baselinespyear: egen offer_any_`sch_type'      = max(offer_`sch_type')
			*If study is missing, kid left DPS. Flag as enroll elsewhere
			tab enroll_any_`sch_type', m
			replace enroll_any_`sch_type'=0 if enroll_any_`sch_type'==.
			replace offer_any_`sch_type' =0 if offer_any_`sch_type' == .
			
			
			
						*Ignacio 
			* this tabs the number of charter offers according to the student's choice
			preserve
				tab apprank if offer ==1 &  appstudy == 1 , matcell(freq) matrow(choice)
			
				tab apprank if offer ==1 &  appstudy == 1 & baselinespyear == 2012, matcell(freqy1) 
				tab apprank if offer ==1 &  appstudy == 1 & baselinespyear == 2012 & pscore_study_simulation < 1 & pscore_study_simulation > 0, matcell(freq01y1)
				
				tab apprank if offer ==1 &  appstudy == 1 & baselinespyear == 2013, matcell(freqy2) 
				tab apprank if offer ==1 &  appstudy == 1 & baselinespyear == 2013 & pscore_study_simulation < 1 & pscore_study_simulation > 0, matcell(freq01y2) 
				
				tab apprank if   appstudy == 1 & pscore_study_simulation < 1 & pscore_study_simulation > 0, matcell(applicants01) 
				tab apprank if   appstudy == 1 & baselinespyear == 2012 & pscore_study_simulation < 1 & pscore_study_simulation > 0, matcell(applicants01y1) 
				tab apprank if   appstudy == 1 & baselinespyear == 2013 & pscore_study_simulation < 1 & pscore_study_simulation > 0, matcell(applicants01y2) 
				
				matrix applicants01 = J(5,1,.)
				matrix applicants01y1 = J(5,1,.)
				matrix applicants01y2 = J(5,1,.)
				
				foreach rank of numlist 1/5 {
					unique masked if appstudy == 1 & apprank == `rank' & pscore_study_simulation < 1 & pscore_study_simulation > 0 
					matrix applicants01[`rank',1] = `r(sum)'
					unique masked if appstudy == 1 & apprank == `rank' & pscore_study_simulation < 1 & pscore_study_simulation > 0 & baselinespyear == 2012 
					matrix applicants01y1[`rank',1] = `r(sum)'
					unique masked if appstudy == 1 & apprank == `rank' & pscore_study_simulation < 1 & pscore_study_simulation > 0 & baselinespyear == 2013
					matrix applicants01y2[`rank',1] = `r(sum)'
					
				}
				
				matrix result = choice,freq,freqy1,freq01y1,freqy2,freq01y2,applicants01,applicants01y1,applicants01y2
				svmat result
				keep result* 
				ren result1 choice
				ren result2 frequency
				ren result3 frequencyy1
				ren result4 frequency01y1
				ren result5 frequencyy2
				ren result6 frequency01y2
				ren result7 applicants01
				ren result8 applicants01y1
				ren result9 applicants01y2
				
				 
				
				outsheet using "`tables'denver_offers_and_choice_tab.csv", comma replace
			restore
			
			
			********************************************************************************
			*Analysis file is at application level until here. Drop to get at student-year level*
			********************************************************************************
			
			gen D = enroll_any_`sch_type'
			gen Z = offer_any_`sch_type'
			
			gen kappa = 1 - (((1-D)*Z) / pscore_`sch_type'_frequency) - ((D*(1-Z)) / (1-pscore_`sch_type'_frequency))
			gen gamma = ((D*(1-Z)) / (1-pscore_`sch_type'_frequency))
			
			duplicates drop maskedid baselinespyear, force
			
			*creating year 1 flag
			gen year1_applicant = (baselinespyear == 2012)			
			
			local row = 1
			local col = 4
			gen var=""
			
			*Means
				foreach var of varlist `cov' baselinewhite baselineasian `cov2' year1 /*year1_applicant*/ {

					replace var="`var'" if _n==`row'
					
					*All applicants (excluding applicants to alternative schools)
					qui summ `var'
					matrix demo[`row',`col'+1]   = r(mean)
					
					*Charter applicants
					qui summ `var' if apply_`sch_type'==1
					matrix demo[`row',`col'+2]   = r(mean)

					*Charter enrollees
					qui summ `var' if enroll_any_`sch_type' == 1
					matrix demo[`row',`col'+3]  = r(mean)

					local c = -2
					
					
					foreach file in frequency formula simulation{

						local c = `c' + 3
						
						*All
						qui summ `var' if apply_`sch_type'==1 & !inlist(pscore_`sch_type'_`file',1,0)
						matrix demo[`row',`col'+3 + `c']   = r(mean)
						*Erolls at charter
						qui summ `var' if apply_`sch_type'==1 & !inlist(pscore_`sch_type'_`file',1,0) & enroll_any_`sch_type'==1
						matrix demo[`row',`col'+4 + `c']   = r(mean)
						
						*Enrolls elsewhere
						qui summ `var' if apply_`sch_type'==1 & !inlist(pscore_`sch_type'_`file',1,0) & enroll_any_`sch_type'==0
						matrix demo[`row',`col'+5 + `c']   = r(mean)
						
					}
					
					
						
					gen kappa_`var' = kappa*`var'
					gen gamma_`var' = gamma*`var'
					
					qui summ kappa_`var' if apply_`sch_type' == 1 & !inlist(pscore_`sch_type'_frequency,1,0)
					matrix demo[`row',`col'+6 + `c']   = r(mean)
	
					qui summ kappa if apply_`sch_type' == 1 & !inlist(pscore_`sch_type'_frequency, 1, 0)
					matrix demo[`row', `col'+7 + `c'] = r(mean)
				
					qui summ gamma_`var' if apply_`sch_type' == 1 & !inlist(pscore_`sch_type'_frequency,1,0)
					matrix demo[`row',`col'+8 + `c']   = r(mean)
	
					qui summ gamma if apply_`sch_type' == 1 & !inlist(pscore_`sch_type'_frequency, 1, 0)
					matrix demo[`row', `col'+9 + `c'] = r(mean)
					
					qui summ `var' if enroll_any_`sch_type' == 1
					matrix demo[`row', `col'+10 + `c'] = r(mean)
					local ++row
				
				}
			
			*Count
				
				local col = 1
				
				*Denver 2011-2012
				matrix demo[`row'+3, `col'] = `denver_N'

				*Enrolled in charter
				matrix demo[`row'+3, `col'+1] = `charter_N'

				*Enrolled in noex
				matrix demo[`row'+3, `col'+2] = `noex_N'
				
				*Enrolled in non noex
				matrix demo[`row'+3, `col'+3] = `nonnoex_N'
				
				*All applicants
				count
				matrix demo[`row'+3,`col'+4]   = r(N)
				
				*Charter applicants
				count if apply_`sch_type'==1
				matrix demo[`row'+3,`col'+5]   = r(N)

				*Charter enrollees (among applicants)
				count if enroll_any_`sch_type' == 1
				matrix demo[`row'+3,`col'+6]   = r(N)
								
				
				local c = -2
				foreach file in frequency formula simulation{
					local c = `c' + 3

					*Pscore in (0,1) (all)
					count if apply_`sch_type' ==1 & !inlist(pscore_`sch_type'_`file',1,0)
					matrix demo[`row'+3,`col'+6 + `c']   = r(N)
					
					*Pscore in (0,1)  (enrolls at charter)
					count if apply_`sch_type' ==1 & !inlist(pscore_`sch_type'_`file',1,0) & enroll_any_`sch_type'==1
					matrix demo[`row'+3,`col'+7 + `c']   = r(N)
					
					*Pscore in (0,1)  (enrolls elsewhere)
					count if apply_`sch_type'==1 & !inlist(pscore_`sch_type'_`file',1,0) & enroll_any_`sch_type'==0
					matrix demo[`row'+3,`col'+8 + `c']   = r(N)
				}
					
				* Charter enrollees
				count if enroll_any_`sch_type' == 1
				matrix demo[`row'+3, `col'+13 + `c'] = r(N)
				
				
				*adding in rounded 0.001 for simualted 
				
				
		
		svmat demo
		keep var demo*
		keep if _n <= `row' +3
		
		di "row `row'"
		replace var="baselinenoex" if _n == `row'
		replace var="baselinenonnoex" if _n == `row' + 1
		replace var="N" if _n==`row' +3
		
		ren demo1 denver
		ren demo2 charter
		ren demo3 noex
		ren demo4 nonnoex
		ren demo5 applicants
		ren demo6 study_app
		ren demo7 study_enroll
      		
		ren demo8 frequency01
		ren demo9 frequency01_study
		ren demo10 frequency01_elsewhere
		
		ren demo11 formula01
		ren demo12 formula01_study
		ren demo13 formula01_elsewhere
		
		ren demo14 simulation01_rnd_001
		ren demo15 simulation01_rnd_001_study
		ren demo16 simulation01_rnd_001_elsewhere
		
		ren demo17 kappa_cov
		ren demo18 kappa
		ren demo19 gamma_cov
		ren demo20 gamma
		
		ren demo21 study_enrolled
		outsheet using "`tables'inDPSbase_`sch_type'_years_stacked_student_demo.csv", comma replace
	}


