/* 	Ye Ji Kee
	Denver tables (year 1). These comments last edited: 4/25/2016
		For grades 4-10
		Output: Balance, 2SLS, and subgroup for pscore risksets and qualification instruments
		
		NOTE: This do-file uses datasets created in 2_1_create_pscore_analysis_files_fc_nonfc.do and 2_1_create_pscore_year_2_analysis_files_fc_nonfc.do

	Use the frequency output;
	Columns: jointtwosls_r0_f jointtwosls_r0_o
	Rows: 
	ss_math_410
ss_math_410_se

*/



clear all
cap log close
set more off
set trace off
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"

pause on 


local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"

matrix demo			  = J(30,15,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,150,.)
matrix jointtwosls	  = J(500,150,.)

local pscore_setup_yr1 	= 1
local pscore_setup_yr2 	= 1
local pscore_stacking 	= 1
local pscore_balance 	= 0
local pscore_separate_twosls = 0
local pscore_joint_twosls = 0

*Dates of analysis files
local pscoredate	20170310	/*20160715 20160426  20160317 */
local simsdate 		20150603	/*20150428*/

local appcov_f 	"num_schools num_study rank1_study"
local cov_f 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2		"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_1 g_2 g_3 g_4 g_5 g_6 g_7"

local appcov_o "num_schools num_study rank1_study"
local cov_o    "baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local appcov_either  "num_schools num_study rank1_study"
local cov_either 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=0				/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/






	************************************************
	**************** Pscore Setup ******************
	************************************************		
	
	if `pscore_setup_yr1'==1{
		
		foreach file in frequency simulation formula {
	
			use "`cleandata'mdrd_denver_fc_nonfc_`file' `pscoredate'", clear
			

			
			destring baselinegrade, replace
			destring appgrade, gen(appgrade_tmp)
			drop appgrade
			rename appgrade_tmp appgrade
			
			*High school flag			
				gen hsgrade = ( appgrade>=9)
				gen msgrade = ( appgrade<9)
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
				
			*Analysis at the student level. Generate max for charter offer
			bys maskedid: egen any_study_offer_fc 	= max(offer_study_fc)
			bys maskedid: egen any_study_offer_oc 	= max(offer_study_oc)
			
			bys maskedid: egen num_study = count(cond(appstudy==1,appschoolnum,.))

			bys maskedid: egen num_schools = count(appschoolnum)
			
			*Enrolls at charter flag (student level)
			bys maskedid: egen enroll_any_study_fc 	= max(study_fc)
			bys maskedid: egen enroll_any_study_oc	= max(study_oc)
			
			*Rank 1 at school level
			gen rank1_study= (appstudy==1 & apprank==1)

			******************************************************************************
			
			*First ranked school is charter (student level, for cov balance)
				bys maskedid: egen tempstudy = max(rank1_study)
				drop rank1_study
				ren tempstudy rank1_study
				
				*Flag - apply to any study
				bys maskedid: egen maxstudy = max(appstudy)
				drop appstudy
				ren maxstudy appstudy
				
				*Flag - first choice is any study
				bys maskedid: egen maxstudy_fc = max(appstudy_fc)
				drop appstudy_fc
				ren maxstudy_fc appstudy_fc
				
				bys maskedid: egen maxstudy_oc = max(appstudy_oc)
				drop appstudy_oc
				ren maxstudy_oc appstudy_oc
			
			*Enrolls next year (i.e. has nonmissing schoolnum)
			gen next = (schoolnum!=.)
			bys maskedid: egen enrolls_next = max(next)
			
			*In_sample was previously defined at the student-school level. Redefine it so
			*that it is at the student level
			
			foreach type in studyfc studyoc{
				bys maskedid: egen in_`type'_sample_max = max(in_`type'_sample)
				tab in_`type'_sample_max in_`type'_sample
				
				drop in_`type'_sample
				rename in_`type'_sample_max in_`type'_sample
			}
			
			*** Dataset at student level from here on ****
			duplicates drop maskedid, force
			drop next
			
			*Redefine has scores so that it excludes sciences - very few kids
			replace has_sc = (has_sc_math | has_sc_reading | has_sc_writing)
			
			*School-level obs
			drop appschoolnum appschoolname offer offer_study offer_study_fc offer_study_oc appcharter appstudy appstudy_fc appstudy_oc apprank study study_fc study_oc charter

			*Keep only if in sample
			keep if in_studyfc_sample==1 | in_studyoc_sample==1
			
			*Define r001 as meaning pscore rounded to 0.001 and r01 as rounded to 0.01
			gen f001 = round(pscore_studyfc_`file', 0.001)
			gen f01 = round(pscore_studyfc_`file', 0.01)
			
			gen o001 = round(pscore_studyoc_`file', 0.001)
			gen o01 = round(pscore_studyoc_`file', 0.01)
			
			tab f001, gen(f001_)
			tab f01, gen(f01_)
			
			tab o01, gen(o01_)
			tab o001, gen(o001_)
			
			*Polynomials
			forvalues i=1/4{
				gen double pscore_f`i'=pscore_studyfc_`file'^(`i')
				gen double pscore_o`i'=pscore_studyoc_`file'^(`i')
			}
			
			keep if baselinespyear==2012
			
			*Treatment is any study, instrument is study offer, riskset is pscore
			ren any_study_offer_fc 	Z_f
			ren any_study_offer_oc 	Z_o
			
			ren enroll_any_study_fc  	D_f
			ren enroll_any_study_oc 	D_o
			
			**********************************************************************************
			******************** Flag kids who are ineligble and guaranteed ******************
			**********************************************************************************
			*Flag kids who are ineligble and guaranteed - do not drop them, we still want them 
			*for OLS regressions and naive 2SLS (i.e. not controlling for risksets)
			
				*If simulation, first define in_zero1_pscore based on true score, then
			
					if "`file'"=="simulation"{
					
						ren pscore_studyfc_simulation sim_pscore_f
						ren pscore_studyoc_simulation sim_pscore_o
						
						gen in_zero1_fpscore = (sim_pscore_f > 0 & sim_pscore_f < 1)
						gen in_zero1_opscore = (sim_pscore_o > 0 & sim_pscore_o < 1)

						gen double pscore_studyfc_simulation = round(sim_pscore_f, 0.0001)
						gen double pscore_studyoc_simulation = round(sim_pscore_o, 0.0001)
						
						tab pscore_studyfc_`file', gen(f0_)
						tab pscore_studyoc_`file', gen(o0_)

						
						ren pscore_studyfc_`file' f0
						ren pscore_studyoc_`file' o0						
					}
					else{
						
						tab pscore_studyfc_`file', gen(f0_)
						tab pscore_studyoc_`file', gen(o0_)
						
						ren pscore_studyfc_`file' f0
						ren pscore_studyoc_`file' o0
						
						gen in_zero1_fpscore = (f0 > 0 & f0 < 1)
						gen in_zero1_opscore = (o0 > 0 & o0 < 1)
					}
					
			**********************************************************************************
			********** Flag cells with variation in instrument and characteristics************
			**********************************************************************************
					
			/* 	Flag observations that actually make it into balance (i.e. obs in cells with
				variation in instrument). These flags are used for all types 
				of F-tests, conducted below.
				
				We generate these flags here so that no re-sorting of the data (done in bys x egen:)
				is done between the balanceregs and F tests, which affects how the variable
				"var", which stores the names of the variables corresponding to each balance
				coefficient, appears in the final dataset once we svmat the balance output.
			*/
			
			ren in_studyfc_sample in_f_sample
			ren in_studyoc_sample in_o_sample
			
			tab in_f_sample, m
			tab in_o_sample, m

			
			foreach type in f o{
				foreach x in 01 001 0{
					
					*Variation in instruments for each cell
						bys `type'`x': egen sd_z_`type'`x'	= 	sd(Z_`type')
					
					*Good cell
						gen in_good_cell_`type'`x' = (sd_z_`type'`x'>0 & !missing(sd_z_`type'`x') & in_zero1_`type'pscore==1) if in_`type'_sample==1
					
					gen double tag_`type'`x' = `type'`x' if in_`type'_sample==1
					replace tag_`type'`x' = -9999999 if in_good_cell_`type'`x'==0 & in_`type'_sample==1
					tab tag_`type'`x' if in_`type'_sample==1, gen(tag_`type'`x'_)
					
					*Drop var for things with no var (the first one). These observations will not be included
					*in bin-based regression and fully saturated regressios. So, do not include their corresponding
					*variables either
					
					drop tag_`type'`x'_1
					
					*Obs per cell
					
					bys `type'`x': egen obs_cell_`type'`x' = count(maskedid) if in_`type'_sample==1
				}
			}
			
			drop sd_z_*
			
			save "`cleandata'analysis_fc_oc_2endog_`file'.dta", replace
		}	/*Close file loop*/	
	}	/*Close pscore condition*/
	


	
	if `pscore_setup_yr2'==1{
		
		foreach file in simulation  frequency formula{
	
			use "`cleandata'mdrd_denver_fc_nonfc_year_2_`file' `pscoredate'", clear
			

			
			destring baselinegrade, replace
			destring appgrade, gen(appgrade_tmp)
			drop appgrade
			rename appgrade_tmp appgrade
			
			*High school flag			
				gen hsgrade = ( appgrade>=9)
				gen msgrade = ( appgrade<9)
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
				
			*Analysis at the student level. Generate max for charter offer
			bys maskedid: egen any_study_offer_fc 	= max(offer_study_fc)
			bys maskedid: egen any_study_offer_oc 	= max(offer_study_oc)
			
			bys maskedid: egen num_study = count(cond(appstudy==1,appschoolnum,.))
	
			bys maskedid: egen num_schools = count(appschoolnum)
			
			*Enrolls at charter flag (student level)
			bys maskedid: egen enroll_any_study_fc 	= max(study_fc)
			bys maskedid: egen enroll_any_study_oc	= max(study_oc)
			
			*Rank 1 at school level
			gen rank1_study= (appstudy==1 & apprank==1)


			******************************************************************************
			
			*First ranked school is charter (student level, for cov balance)
				bys maskedid: egen tempstudy = max(rank1_study)
				drop rank1_study
				ren tempstudy rank1_study
				
				*Flag - apply to any study
				bys maskedid: egen maxstudy = max(appstudy)
				drop appstudy
				ren maxstudy appstudy
				
				*Flag - first choice is any study
				bys maskedid: egen maxstudy_fc = max(appstudy_fc)
				drop appstudy_fc
				ren maxstudy_fc appstudy_fc
				
				bys maskedid: egen maxstudy_oc = max(appstudy_oc)
				drop appstudy_oc
				ren maxstudy_oc appstudy_oc
			
			*Enrolls next year (i.e. has nonmissing schoolnum)
			gen next = (schoolnum!=.)
			bys maskedid: egen enrolls_next = max(next)
			
			*In_sample was previously defined at the student-school level. Redefine it so
			*that it is at the student level
			
			foreach type in studyfc studyoc{
				bys maskedid: egen in_`type'_sample_max = max(in_`type'_sample)
				tab in_`type'_sample_max in_`type'_sample
				
				drop in_`type'_sample
				rename in_`type'_sample_max in_`type'_sample
			}
			
			*** Dataset at student level from here on ****
			duplicates drop maskedid, force
			drop next
			
			*Redefine has scores so that it excludes sciences - very few kids
			replace has_sc = (has_sc_math | has_sc_reading | has_sc_writing)
			
			*School-level obs
			drop appschoolnum appschoolname offer offer_study offer_study_fc offer_study_oc appcharter appstudy appstudy_fc appstudy_oc apprank study study_fc study_oc charter

			*Keep only if in sample
			keep if in_studyfc_sample==1 | in_studyoc_sample==1
			
			*Define r001 as meaning pscore rounded to 0.001 and r01 as rounded to 0.01
			gen f001 = round(pscore_studyfc_`file', 0.001)
			gen f01 = round(pscore_studyfc_`file', 0.01)
			
			gen o001 = round(pscore_studyoc_`file', 0.001)
			gen o01 = round(pscore_studyoc_`file', 0.01)
			
			tab f001, gen(f001_)
			tab f01, gen(f01_)
			
			tab o01, gen(o01_)
			tab o001, gen(o001_)
			
			*Polynomials
			forvalues i=1/4{
				gen double pscore_f`i'=pscore_studyfc_`file'^(`i')
				gen double pscore_o`i'=pscore_studyoc_`file'^(`i')
			}
			
			keep if baselinespyear==2013
			
			*Treatment is any study, instrument is study offer, riskset is pscore
			ren any_study_offer_fc 	Z_f
			ren any_study_offer_oc 	Z_o
			
			ren enroll_any_study_fc  	D_f
			ren enroll_any_study_oc 	D_o
			
			**********************************************************************************
			******************** Flag kids who are ineligble and guaranteed ******************
			**********************************************************************************
			*Flag kids who are ineligble and guaranteed - do not drop them, we still want them 
			*for OLS regressions and naive 2SLS (i.e. not controlling for risksets)
			
				*If simulation, first define in_zero1_pscore based on true score, then
			
					if "`file'"=="simulation"{
					
						ren pscore_studyfc_simulation sim_pscore_f
						ren pscore_studyoc_simulation sim_pscore_o
						
						gen in_zero1_fpscore = (sim_pscore_f > 0 & sim_pscore_f < 1)
						gen in_zero1_opscore = (sim_pscore_o > 0 & sim_pscore_o < 1)

						gen double pscore_studyfc_simulation = round(sim_pscore_f, 0.0001)
						gen double pscore_studyoc_simulation = round(sim_pscore_o, 0.0001)
						
						tab pscore_studyfc_`file', gen(f0_)
						tab pscore_studyoc_`file', gen(o0_)

						
						ren pscore_studyfc_`file' f0
						ren pscore_studyoc_`file' o0	
						}
						
					else{
							
						tab pscore_studyfc_`file', gen(f0_)
						tab pscore_studyoc_`file', gen(o0_)
						
						ren pscore_studyfc_`file' f0
						ren pscore_studyoc_`file' o0
						
						gen in_zero1_fpscore = (f0 > 0 & f0 < 1)
						gen in_zero1_opscore = (o0 > 0 & o0 < 1)
						}
					
			**********************************************************************************
			********** Flag cells with variation in instrument and characteristics************
			**********************************************************************************
					
			/* 	Flag observations that actually make it into balance (i.e. obs in cells with
				variation in instrument). These flags are used for all types 
				of F-tests, conducted below.
				
				We generate these flags here so that no re-sorting of the data (done in bys x egen:)
				is done between the balanceregs and F tests, which affects how the variable
				"var", which stores the names of the variables corresponding to each balance
				coefficient, appears in the final dataset once we svmat the balance output.
			*/
			
			ren in_studyfc_sample in_f_sample
			ren in_studyoc_sample in_o_sample
			
			tab in_f_sample, m
			tab in_o_sample, m
			
			foreach type in f o{
				foreach x in 01 001 0{
					
					*Variation in instruments for each cell
						bys `type'`x': egen sd_z_`type'`x'	= 	sd(Z_`type')
					
					*Good cell
						gen in_good_cell_`type'`x' = (sd_z_`type'`x'>0 & !missing(sd_z_`type'`x') & in_zero1_`type'pscore==1) if in_`type'_sample==1
					
					gen double tag_`type'`x' = `type'`x' if in_`type'_sample==1
					replace tag_`type'`x' = -9999999 if in_good_cell_`type'`x'==0 & in_`type'_sample==1
					tab tag_`type'`x' if in_`type'_sample==1, gen(tag_`type'`x'_)
					
					*Drop var for things with no var (the first one). These observations will not be included
					*in bin-based regression and fully saturated regressios. So, do not include their corresponding
					*variables either
					
					drop tag_`type'`x'_1
					
					*Obs per cell
					
					bys `type'`x': egen obs_cell_`type'`x' = count(maskedid) if in_`type'_sample==1
				}
			}
			
			drop sd_z_*
			
			save "`cleandata'analysis_fc_oc_2endog_year_2_`file'.dta", replace
		}	/*Close file loop*/	
	}	/*Close pscore condition*/	
	
	
	if `pscore_stacking' == 1{
		
		foreach file in  frequency simulation formula {
		
			u "`cleandata'analysis_fc_oc_2endog_`file'.dta", clear
			
			append using "`cleandata'analysis_fc_oc_2endog_year_2_`file'.dta"
						
			drop f001_* f01_* f0_* o001_* o01_* o0_* in_good_cell_* tag_* obs_cell_*

	
			tab f001, gen(f001_)
			tab f01, gen(f01_)
			
			tab o001, gen(o001_)
			tab o01, gen(o01_)
			
			tab f0, gen(f0_)
			tab o0, gen(o0_)
			

			
			* generating indicator to use factor variables in 2sls regs (much faster)
			egen ind_f0 = group(f0)
			egen ind_f001 = group(f001)
			egen ind_f01 = group(f01)
			
			egen ind_o0 = group(o0)
			egen ind_o001 = group(o001)
			egen ind_o01 = group(o01)
			
			*dropping duplicates from year 2.
			
			duplicates tag masked, gen(dup_masked)
			drop if dup_masked > 0 & baselinespyear == 2013				
						
				foreach type in f o{
				foreach x in 01 001 0{
					
					*Variation in instruments for each cell
						bys `type'`x': egen sd_z_`type'`x'	= 	sd(Z_`type')
					
					*Good cell
						gen in_good_cell_`type'`x' = (sd_z_`type'`x'>0 & !missing(sd_z_`type'`x') & in_zero1_`type'pscore==1) if in_`type'_sample==1
					
					gen double tag_`type'`x' = `type'`x' if in_`type'_sample==1
					replace tag_`type'`x' = -9999999 if in_good_cell_`type'`x'==0 & in_`type'_sample==1
					tab tag_`type'`x' if in_`type'_sample==1, gen(tag_`type'`x'_)
					
					*Drop var for things with no var (the first one). These observations will not be included
					*in bin-based regression and fully saturated regressios. So, do not include their corresponding
					*variables either
					
					drop tag_`type'`x'_1
					
					*Obs per cell
					
					bys `type'`x': egen obs_cell_`type'`x' = count(maskedid) if in_`type'_sample==1
					}
				}
		
			
			drop sd_z_*
		
			save "`cleandata'analysis_fc_oc_2endog_stacked_`file'.dta", replace
		}
	}

	*****************************************
	******** Balance and attrition **********
	*****************************************
	
	if `pscore_balance'==1{
	
		foreach file in /*simulation*/ frequency formula{
			
				u "`cleandata'analysis_fc_oc_2endog_stacked_`file'.dta", clear
				
				
				**********************************************************************************
				************************ Balance and attrition coefficients **********************
				**********************************************************************************
		
				gen var =""
				
				*Count points in the support
				replace var = "Riskset points of support" if _n ==1
			
				
				foreach type in f o{
						
						if "`type'"=="f"{
							local colbump = 0
						}
						else if "`type'"=="o"{
							local colbump = 7
						}

						local row = 2
						local col = 1 + `colbump'
						
						
						*Place it on top of balance table
						unique pscore_`type'1 		if in_zero1_`type'pscore==1
						matrix balance[1,3 + `colbump'] = r(sum)
						
						unique `type'01				if in_good_cell_`type'01==1
						matrix balance[1,5 + `colbump'] = r(sum)
						
						unique `type'001				if in_good_cell_`type'001==1
						matrix balance[1,6 + `colbump'] = r(sum)

						unique `type'0				if in_good_cell_`type'0==1
						matrix balance[1,7 + `colbump'] = r(sum)
						
					
					foreach var in `appcov_`type'' `cov_`type'' `cov2' `attrition_vars'{
						replace var="`var'" if _n==`row'
						replace var="`var'_se" if _n==`row'+1
						replace var="`var'_N" if _n==`row'+2
						replace var="`var'_N(var in Z)" if _n==`row'+3
							
						*All non-offered applicants
						qui summ `var' if Z_`type'==0 & in_`type'_sample==1
							matrix balance[`row',`col']   = r(mean)
							matrix balance[`row'+1,`col'] = r(sd)
							matrix balance[`row'+2,`col' ] = r(N)
					
						*Balance and attrition, no control(OLS)
						qui reg `var' Z_`type' if in_`type'_sample==1, vce(cluster maskedid)
							matrix balance[`row',`col' + 1]   = _b[Z]
							matrix balance[`row'+1,`col'  +1] = _se[Z]
							matrix balance[`row'+2,`col'  +1] = e(N)
						
						*Balance and attrition, pscore linear control(OLS)
						qui reg `var' Z_`type' pscore_`type'1 if in_zero1_`type'pscore==1, vce(cluster maskedid)
							matrix balance[`row',`col'  +2]   = _b[Z]
							matrix balance[`row'+1,`col'  +2] = _se[Z]
							matrix balance[`row'+2,`col'  +2] = e(N)
							
						*Balance and attrition, pscore polynomial control(OLS)
						qui reg `var' Z_`type' pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 if in_zero1_`type'pscore==1, vce(cluster maskedid)
							matrix balance[`row',`col'  +3]   = _b[Z]
							matrix balance[`row'+1,`col'  +3] = _se[Z]
							matrix balance[`row'+2,`col' + 3] = e(N)
						
						*Balance and attrition, risks
						local c = 0
						foreach i in 01 001 0{
							local c = `c' +1
							
							preserve
								qui areg `var' Z_`type' if in_zero1_`type'pscore==1, absorb(`type'`i') vce(cluster maskedid)
							
								matrix balance[`row',`col' + 3 + `c']   = _b[Z]
								matrix balance[`row'+1,`col' + 3 + `c'] = _se[Z]
								matrix balance[`row'+2,`col' + 3 + `c'] = e(N)
								
								bys `type'`i': egen sd = sd(`var')
								bys `type'`i': egen sd_z = sd(Z_`type')
								
								*Count kids with variation in instrument within cell (no ref to characteristic)
								count if sd_z > 0 & !missing(sd_z)  & e(sample)
								matrix balance[`row'+3,`col' + 3 + `c'] = r(N)
								
								drop sd sd_z		
							restore
						}
						
					local row = `row' + 6
					} /*Close covariates loop*/
				
			} /*Close n, e loop*/
			
			************************************* F tests **************************************	
				*MS 2/23/2015: 
				*Minimum number of observations neeed for the SUEST F test to run according to each pscore type
				*Arrived at these numbers by running F tests for several different minimum number of obs. These
				*are the minimum needed for the set of covariates being tested as of 2/23/2015.
				
				local min_obs_cell_simulation 	= 9			
				local min_obs_cell_formula 		= 9
				local min_obs_cell_frequency 	= 9
				
			
			if `ftest'==1{
				
				**********************************************************************************
				************************************* Logit Fs ***********************************
				**********************************************************************************
				
				foreach f in robust traditional{
					
						foreach covs in apponly balanceonly all{
						
							foreach type in f o{
						
								if "`type'"=="f"{
									local colbump = 0
								}
								else if "`type'"=="o"{
									local colbump = 7
								}
								
								local col = 1 + `colbump'
						
								if "`covs'"=="apponly"{
									local balF "`appcov_`type''"
									local Fname "Application Balance"
								}
								
								else if "`covs'"=="balanceonly"{
									local balF "`cov_`type'' `cov2'"
									local Fname "Baseline Balance"
								}
								else if "`covs'"=="all"{
									local balF "`appcov_`type'' `cov_`type'' `cov2'"
									local Fname "All Balance"
								}
								
								
								if "`f'"=="robust"{
									local option "nocnsreport vce(cluster maskedid)"
								}
								else{
									local option "nocnsreport"
								}
							
							
								* Balance logit
								replace var="Logit `f' `Fname' F" if _n==`row'
								replace var="Logit `f' `Fname' F (pval)" if _n==`row'+1
								replace var="Logit `f' `Fname' F (df)" if _n==`row'+2
								replace var="Logit `f' `Fname' F (N)" if _n==`row'+3
							
								*No riskset
								qui logit Z_`type' `balF', `option'
								test `balF'
								matrix balance[`row', `col' +1] = r(chi2)
								matrix balance[`row'+1, `col' +1] = r(p)
								matrix balance[`row'+2, `col' +1] = r(df)
								matrix balance[`row'+3, `col' +1] = e(N)	
			
								*Linear riskset
								qui logit Z_`type' pscore_`type'1 `balF' if in_zero1_`type'pscore==1, `option'
								test `balF'
								matrix balance[`row', `col' +2] = r(chi2)
								matrix balance[`row'+1, `col' +2] = r(p)
								matrix balance[`row'+2, `col' +2] = r(df)
								matrix balance[`row'+3, `col' +2] = e(N)
								
								*Poly riskset
								qui logit Z_`type' pscore_`type'* `balF' if in_zero1_`type'pscore==1, `option'
								test `balF'
								matrix balance[`row', `col' +3] = r(chi2)
								matrix balance[`row'+1, `col' +3] = r(p)
								matrix balance[`row'+2, `col' +3] = r(df)
								matrix balance[`row'+3, `col' +3] = e(N)
								
								*Bins
								local c=0
								foreach i in 01 001 0{
									local c = `c' + 1
									qui logit Z_`type' tag_`type'`i'_* `balF' if in_good_cell_`type'`i'==1, `option'
									test `balF'
									matrix balance[`row', `col'+3 + `c'] = r(chi2)
									matrix balance[`row' +1, `col'  +3 + `c'] = r(p)
									matrix balance[`row'+2, `col'  +3 + `c'] = r(df)
									matrix balance[`row'+3, `col'  +3 + `c'] = e(N)
								}

						} /*CLose n e loop*/			
					local row = `row' + 4
					}	/*Close balance covariates loop*/
			}	/*CLose robust/traditional F*/
				
				********************************************************
				*************** Balance Traditional F ******************
				********************************************************
						
				foreach covs in apponly balanceonly all{
				
					foreach type in f o{
						
						if "`type'"=="f"{
							local colbump = 0
						}
						else if "`type'"=="o"{
							local colbump = 7
						}
						
						local col = 1 + `colbump'
											
							if "`covs'"=="apponly"{
								local balF "`appcov_`type''"
								local Fname "Application Balance"
							}
							
							else if "`covs'"=="balanceonly"{
								local balF "`cov_`type'' `cov2'"
								local Fname "Baseline Balance"
							}
							else if "`covs'"=="all"{
								local balF "`appcov_`type'' `cov_`type'' `cov2'"
								local Fname "All Balance"
							}
									
							replace var="Traditional `Fname' F-test" if _n==`row'
							replace var="Traditional `Fname' F-test (pval)" if _n==`row' +1
							replace var="Traditional `Fname' F-test (residuals df)" if _n==`row' +2
							replace var="Traditional `Fname' F-test (model df)" if _n==`row' +3
							replace var="Traditional `Fname' F-test (N)" if _n==`row' +4
							replace var="Traditional `Fname' F-test (Breusch-Pagan chi2)" if _n==`row'+5
					
						
						*no propensity score
							mvreg `balF' = Z_`type', corr
							test Z_`type'
							matrix balance[`row',`col'  + 1] = r(F)
							matrix balance[`row'+1,`col' + 1]= r(p)
							matrix balance[`row'+2, `col' +1] = r(df_r)
							matrix balance[`row'+3, `col' +1] = r(df)
							matrix balance[`row'+4, `col' +1] = e(N)
							matrix balance[`row'+5, `col' +1] = 1- chi2(e(df_chi2),e(chi2))
						
						*linear pscore control
							mvreg `balF' = Z_`type' pscore_`type'1 if in_zero1_`type'pscore==1, corr
							test Z_`type'
							matrix balance[`row',`col'  +2]=r(F)
							matrix balance[`row'+1,`col'  +2]=r(p)
							matrix balance[`row'+2, `col' +2] = r(df_r)
							matrix balance[`row'+3, `col' +2] = r(df)
							matrix balance[`row'+4, `col' +2] = e(N)
							matrix balance[`row'+5, `col' +2] = 1- chi2(e(df_chi2),e(chi2))
							
						*polynomial pscore control
							mvreg `balF' = Z_`type' pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 if in_zero1_`type'pscore==1, corr
							test Z_`type'
							matrix balance[`row',`col'  + 3]=r(F)
							matrix balance[`row'+1,`col' + 3]=r(p)
							matrix balance[`row'+2, `col' +3] = r(df_r)
							matrix balance[`row'+3, `col' +3] = r(df)
							matrix balance[`row'+4, `col' +3] = e(N)
							matrix balance[`row'+5, `col' +3] = 1- chi2(e(df_chi2),e(chi2))
							
						*Bins
							local c=0
							foreach i in 01 001 0{
								local c = `c' + 1
								mvreg `balF' = Z_`type' tag_`type'`i'_*  if in_good_cell_`type'`i'==1, corr
								test Z_`type'
								matrix balance[`row',`col'  + 3 + `c'] = r(F)
								matrix balance[`row'+1,`col'  + 3 + `c'] = r(p)
								matrix balance[`row'+2,`col'  + 3 + `c'] = r(df_r)
								matrix balance[`row'+3,`col'  + 3 + `c'] = r(df)
								matrix balance[`row'+4,`col'  + 3 + `c'] = e(N)
								matrix balance[`row'+5,`col'  + 3 + `c'] = 1- chi2(e(df_chi2),e(chi2))
								
							}

										
						}	/*Close c i loop*/
				local row = `row' + 6
				}	/*Close covariates loop*/
			
				
				**********************************************************************************
				*************************** Balance Stacked Robust F *****************************
				**********************************************************************************
				
				*Run a cut keeping cells with small size and one dropping cells with small size
				
				local col = 1	
					
				foreach f_cut in keep_small_cells drop_small_cells{
					
						foreach covs in apponly balanceonly all{
						
							foreach type in f o{
				
								if "`type'"=="f"{
									local colbump = 0
								}
								else if "`type'"=="o"{
									local colbump = 7
								}
								
								local col = 1 + `colbump'
							
								if "`covs'"=="apponly"{
									local balF "`appcov_`type''"
									local Fname "Application Balance"
									local expand = 3
									local test "Z_1 Z_2 Z_3"
								}
								
								else if "`covs'"=="balanceonly"{
									local balF "`cov_`type'' `cov2'"
									local Fname "Baseline Balance"
									local expand = 12
									local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12"
								}
								else if "`covs'"=="all"{
									local balF "`appcov_`type'' `cov_`type'' `cov2'"
									local Fname "All Balance"
									local expand = 15
									local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12 Z_13 Z_14 Z_15"
								}
								
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test" if _n==`row'
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (pval)" if _n==`row'+1
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (residuals df)" if _n==`row'+2
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (model df)" if _n==`row'+3
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (N)" if _n==`row'+4
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (cells)" if _n==`row'+5
								
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test" if _n==`row'+6
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (pval)" if _n==`row'+7
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (residuals df)" if _n==`row'+8
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (model df)" if _n==`row'+9
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (N)" if _n==`row'+10
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (cells)" if _n==`row'+11
								
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test" if _n==`row' +12
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (pval)" if _n==`row'+13
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (residuals df)" if _n==`row'+14
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (model df)" if _n==`row'+15
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (N)" if _n==`row'+16
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (cells)" if _n==`row'+17
						
						
								preserve
									
									if "`f_cut'" === "keep_small_cells"{
										local min_obs = 1
									}
									else if "`f_cut'"=="drop_small_cells"{
										local min_obs = `min_obs_cell_`file''
									}
									
									keep 	maskedid `balF' Z_`type' tag_`type'01 tag_`type'001 tag_`type'0 `type'01 `type'001 `type'0 ///
											pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 in_*_sample in_good_cell_* obs_cell_* in_zero1_`type'pscore
											
									expand `expand'
									bys maskedid: gen copy = _n
									sort copy
									
									*Single vector of outcomes
									gen cov =.
									gen covname=""
									tokenize `balF'
									forval i=1/`expand'{
										replace cov = ``i'' if copy==`i' 
										replace covname = "``i''" if copy==`i' 
									}
									drop `balF'
									
									*Instruments, pscore variables, and dummies for each copy
									forval i=1/`expand'{
										gen Z_`i' = Z_`type'*(copy == `i')
										gen alpha_`i' = (copy == `i')
										
										*Pscore variables for each copy
										foreach x in `type'01 `type'001 `type'0 pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4{
											gen c_`x'_`i'= `x'*(copy == `i')
										}
									}
									
									*Absorb takes only one var. Create grouped vars
									foreach x in tag_`type'01 tag_`type'001 tag_`type'0 `type'01 `type'001 `type'0 {
											egen group_`x' = group(copy `x')
										
									}
									
									*Replace with missing if first category
									foreach x in tag_`type'01 tag_`type'001 tag_`type'0{
										replace group_`x' =. if `x'==-9999999
									
									}
															
									*** One tall regression for each specification ***
									
									*No controls				
										*Clustered
										qui reg cov alpha_* Z_* if in_`type'_sample==1, vce(cluster maskedid)
										test `test'
										
										matrix balance[`row',`col' + 1]=r(F)
										matrix balance[`row'+1,`col' +1]=r(p)
										matrix balance[`row'+2,`col' +1]=r(df_r)
										matrix balance[`row'+3,`col' +1]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+4,`col' + 1]=r(sum)
										
										
										*Traditional
										qui reg cov alpha_* Z_*  if in_`type'_sample==1
										test `test'
										
										matrix balance[`row'+6,`col' + 1]=r(F)
										matrix balance[`row'+7,`col' +1]=r(p)
										matrix balance[`row'+8,`col' +1]=r(df_r)
										matrix balance[`row'+9,`col' +1]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+10,`col' + 1]=r(sum)
										
										
										*Robust
										qui reg cov alpha_* Z_* if in_`type'_sample==1, r
										test `test'
										
										matrix balance[`row'+12,`col' + 1]=r(F)
										matrix balance[`row'+13,`col' +1]=r(p)
										matrix balance[`row'+14,`col' +1]=r(df_r)
										matrix balance[`row'+15,`col' +1]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+16,`col' + 1]=r(sum)
										

									*Linear
										*Cluster
										qui reg cov alpha_* Z_* c_pscore_`type'1_* if in_zero1_`type'pscore==1, vce(cluster maskedid)
										test `test'
										
										matrix balance[`row',`col' + 2]=r(F)
										matrix balance[`row'+1,`col' +2]=r(p)
										matrix balance[`row'+2,`col' +2]=r(df_r)
										matrix balance[`row'+3,`col' +2]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+4,`col' + 2]=r(sum)
										
										
										*Traditional
										qui reg cov alpha_* Z_* c_pscore_`type'1_* if in_zero1_`type'pscore==1
										test `test'
										
										matrix balance[`row'+6,`col' + 2]=r(F)
										matrix balance[`row'+7,`col' +2]=r(p)
										matrix balance[`row'+8,`col' +2]=r(df_r)
										matrix balance[`row'+9,`col' +2]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+10,`col' + 2]=r(sum)
										
									
										*Robust
										qui reg cov alpha_* Z_* c_pscore_`type'1_* if in_zero1_`type'pscore==1, r
										test `test'
										
										matrix balance[`row'+12,`col' + 2]=r(F)
										matrix balance[`row'+13,`col' +2]=r(p)
										matrix balance[`row'+14,`col' +2]=r(df_r)
										matrix balance[`row'+15,`col' +2]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+16,`col' + 2]=r(sum)
										
										
									*Polynomial
										*Cluster
										qui reg cov alpha_* Z_* c_pscore_`type'1_* c_pscore_`type'2_* c_pscore_`type'3_* c_pscore_`type'4_* if in_zero1_`type'pscore==1, vce(cluster maskedid)
										test `test'
										
										matrix balance[`row',`col' + 3]=r(F)
										matrix balance[`row'+1,`col' +3]=r(p)
										matrix balance[`row'+2,`col' +3]=r(df_r)
										matrix balance[`row'+3,`col' +3]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+4,`col' + 3]=r(sum)
										
										
										*Traditional
										qui reg cov alpha_* Z_* c_pscore_`type'1_* c_pscore_`type'2_* c_pscore_`type'3_* c_pscore_`type'4_* if in_zero1_`type'pscore==1
										test `test'
										
										matrix balance[`row'+6,`col' + 3]=r(F)
										matrix balance[`row'+7,`col' +3]=r(p)
										matrix balance[`row'+8,`col' +3]=r(df_r)
										matrix balance[`row'+9,`col' +3]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+10,`col' + 3]=r(sum)
										
										
										
										*Robust
										qui reg cov alpha_* Z_* c_pscore_`type'1_* c_pscore_`type'2_* c_pscore_`type'3_* c_pscore_`type'4_* if in_zero1_`type'pscore==1, r
										test `test'
										
										matrix balance[`row'+12,`col' + 3]=r(F)
										matrix balance[`row'+13,`col' +3]=r(p)
										matrix balance[`row'+14,`col' +3]=r(df_r)
										matrix balance[`row'+15,`col' +3]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+16,`col' + 3]=r(sum)
										
										
									*Bins
										local c = 0
										foreach i in 01 001 0{
											local c = `c' +1
											
											*Cluster
											qui areg cov alpha_* Z_* if in_good_cell_`type'`i'==1 & obs_cell_`type'`i' >= `min_obs' & !missing(obs_cell_`type'`i'), absorb(group_tag_`type'`i') vce(cluster maskedid)
											test `test'
											
											matrix balance[`row',`col' +3 + `c']   = r(F)
											matrix balance[`row'+1,`col' +3 + `c'] = r(p)
											matrix balance[`row'+2,`col'+ 3 + `c'] = r(df_r)
											matrix balance[`row'+3,`col'+3 + `c'] = r(df)
										
											
											unique maskedid if e(sample)
											matrix balance[`row'+4,`col'+3 + `c'] = r(sum)
											
											unique group_tag_`type'`i' if e(sample)
											matrix balance[`row'+5,`col'+3 + `c'] = r(sum)
											
											
											*Traditional
											qui areg cov alpha_* Z_* if in_good_cell_`type'`i'==1 & obs_cell_`type'`i' >= `min_obs' & !missing(obs_cell_`type'`i'), absorb(group_tag_`type'`i')
											test `test'
											
											matrix balance[`row'+6,`col'+3 + `c'] = r(F)
											matrix balance[`row'+7,`col'+3 + `c'] = r(p)
											matrix balance[`row'+8,`col'+3 + `c'] = r(df_r)
											matrix balance[`row'+9,`col'+3 + `c'] = r(df)
											
											
											unique maskedid if e(sample)
											matrix balance[`row'+10,`col'+3 + `c'] = r(sum)
											
											unique group_tag_`type'`i' if e(sample)
											matrix balance[`row'+11,`col'+3 + `c'] = r(sum)
											
											
											*Robust
											qui areg cov alpha_* Z_* if in_good_cell_`type'`i'==1 & obs_cell_`type'`i' >= `min_obs' & !missing(obs_cell_`type'`i'), absorb(group_tag_`type'`i') r
											test `test'
											
											matrix balance[`row'+12,`col'+3 + `c'] = r(F)
											matrix balance[`row'+13,`col'+3 + `c'] = r(p)
											matrix balance[`row'+14,`col'+3 + `c'] = r(df_r)
											matrix balance[`row'+15,`col'+3 + `c'] = r(df)
											
											
											unique maskedid if e(sample)
											matrix balance[`row'+16,`col'+3 + `c'] = r(sum)
											
											unique group_tag_`type'`i' if e(sample)
											matrix balance[`row'+17,`col'+3 + `c'] = r(sum)
											
										}
								
								restore
							} /*Close n, e loop*/
						
						local row = `row' + 18	
						} /*Close vocs loop*/		
				} /*Close f cut*/	
				
				
				**********************************************************************************
				*********************** Suest Robust standard errors *****************************
				**********************************************************************************
				
				** F-tests using suest, which allows for robus or clustered SEs **
				* 1) Run models without clustering option
				* 2) Store results
				* 3) suest [stored values], vce(cluster clustvar)
				* 4) test coefficient
				
				local col = 1
				
				foreach f_cut in keep_small_cells drop_small_cells{

						if "`f_cut'" === "keep_small_cells"{
							local min_obs = 1
						}
						else if "`f_cut'"=="drop_small_cells"{
							local min_obs = `min_obs_cell_`file''
						}
						
							replace var="[`f_cut'] Suest Application Balance F" if _n==`row'
							replace var="[`f_cut'] Suest Application Balance F (pval)" if _n==`row' +1
							replace var="[`f_cut'] Suest Application Balance F (df)" if _n==`row' +2
							replace var="[`f_cut'] Suest Application Balance F (N)" if _n==`row' +3
							replace var="[`f_cut'] Suest Application Balance F (cells)" if _n==`row' +4
							
							replace var="[`f_cut'] Suest Baseline Balance F" if _n==`row' + 5
							replace var="[`f_cut'] Suest Baseline Balance F (pval)" if _n==`row' +6
							replace var="[`f_cut'] Suest Baseline Balance F (df)" if _n==`row' +7
							replace var="[`f_cut'] Suest Baseline Balance F (N)" if _n==`row' +8
							replace var="[`f_cut'] Suest Baseline Balance F (cells)" if _n==`row' +9
							
							replace var="[`f_cut'] Suest All Balance F" if _n==`row' + 10
							replace var="[`f_cut'] Suest All Balance F (pval)" if _n==`row' +11
							replace var="[`f_cut'] Suest All Balance F (df)" if _n==`row' +12
							replace var="[`f_cut'] Suest All Balance F (N)" if _n==`row' +13
							replace var="[`f_cut'] Suest All Balance F (cells)" if _n==`row' +14
						
					foreach type in f o{
						
						if "`type'"=="f"{
							local colbump = 0
						}
						else if "`type'"=="o"{
							local colbump = 7
						}
						
						local col = 1 + `colbump'
								

						*Balance F
							foreach var in `appcov_`type'' `cov_`type'' `cov2'{
								
								* no pscores
								qui reg `var' Z_`type'
								est store `var'
								
								* linear
								qui reg `var' Z_`type' pscore_`type'1 if in_zero1_`type'pscore==1
								est store L`var'
								
								* poly
								qui reg `var' Z_`type' pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 if in_zero1_`type'pscore==1
								est store P`var'
								
								* Pscore bins
								foreach i in 01 001 0{
									qui reg `var' Z_`type' tag_`type'`i'_* 	if in_good_cell_`type'`i'==1 & obs_cell_`type'`i'>=`min_obs' & !missing(obs_cell_`type'`i')
									est store B`i'`var'
								}
	
							}
							
							*** Apcov suest ***
								cap suest 	num_schools num_`type' rank1_`type', vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row', `col' + 1] = r(chi2)
									matrix balance[`row'+1, `col' + 1] = r(p)
									matrix balance[`row'+2, `col' + 1] = r(df)
									matrix balance[`row'+3, `col' + 1] = e(N)
								}	
									
								cap suest 	Lnum_schools Lnum_`type'  Lrank1_`type', vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row', `col' +2] = r(chi2)
									matrix balance[`row'+1, `col' +2] = r(p)
									matrix balance[`row'+2, `col' +2] = r(df)
									matrix balance[`row'+3, `col' +2] = e(N)
								}
								
								cap suest 	Pnum_schools Pnum_`type' Prank1_`type', vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row', `col' +3] = r(chi2)
									matrix balance[`row'+1, `col' +3] = r(p)
									matrix balance[`row'+2, `col' +3] = r(df)
									matrix balance[`row'+3, `col' +3] = e(N)
								}
								
								local c=0
								foreach i in 01 001 0{
									local c = `c' + 1
									
									cap suest 	B`i'num_schools B`i'num_`type' B`i'rank1_`type', vce(cluster maskedid)
									if !_rc{
										test Z_`type'
										matrix balance[`row', `col' + 3 + `c'] = r(chi2)
										matrix balance[`row'+1, `col' + 3 + `c'] = r(p)
										matrix balance[`row'+2, `col' + 3 + `c'] = r(df)
										matrix balance[`row'+3, `col' + 3 + `c'] = e(N)
										
										
										unique tag_`type'`i' if e(sample)
										matrix balance[`row'+4, `col' + 3 + `c'] = r(sum)
										
									}
								}
								
						** Baseline covs suest ***
							
							cap suest	baseline`type' baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
										baseliness_math baseliness_reading baseliness_writing, vce(cluster maskedid)
							if !_rc{			
								test Z_`type'
								matrix balance[`row'+5, `col'+1] = r(chi2)
								matrix balance[`row'+6, `col'+1] = r(p)
								matrix balance[`row'+7, `col'+1] = r(df)
								matrix balance[`row'+8, `col'+1] = e(N)
							}
								
							cap suest	Lbaseline`type' Lbaselinefemale Lbaselinehisp Lbaselineblack Lbaselinefoodfred Lbaselinesped Lbaselinebilingual Lbaselinelep Lbaselinegifted ///
									Lbaseliness_math Lbaseliness_reading Lbaseliness_writing, vce(cluster maskedid)
							if !_rc{
								test Z_`type'
								matrix balance[`row'+5, `col'+2] = r(chi2)
								matrix balance[`row'+6, `col' +2] = r(p)
								matrix balance[`row'+7, `col'+2] = r(df)
								matrix balance[`row'+8, `col'+2] = e(N)
							}
							
							cap suest	Pbaseline`type' Pbaselinefemale Pbaselinehisp Pbaselineblack Pbaselinefoodfred Pbaselinesped Pbaselinelep Pbaselinebilingual Pbaselinegifted ///
									Pbaseliness_math Pbaseliness_reading Pbaseliness_writing, vce(cluster maskedid)
							if !_rc{
								test Z_`type'
								matrix balance[`row'+5, `col'+3] = r(chi2)
								matrix balance[`row'+6, `col' +3] = r(p)
								matrix balance[`row'+7, `col'+3] = r(df)
								matrix balance[`row'+8, `col'+3] = e(N)
							}
							
							local c=0
							foreach i in 01 001 0{
								local c = `c' + 1
								cap suest	B`i'baseline`type' B`i'baselinefemale B`i'baselinehisp B`i'baselineblack B`i'baselinefoodfred B`i'baselinesped B`i'baselinelep B`i'baselinebilingual B`i'baselinegifted ///
										B`i'baseliness_math B`i'baseliness_reading B`i'baseliness_writing, vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row'+5, `col' +3 + `c'] = r(chi2)
									matrix balance[`row'+6, `col' +3 + `c'] = r(p)
									matrix balance[`row'+7, `col' +3 + `c'] = r(df)
									matrix balance[`row'+8, `col' +3 + `c'] = e(N)
									
									unique tag_`type'`i' if e(sample)
									matrix balance[`row'+9, `col' +3 + `c'] = r(sum)
									
								}
							}
							
							
						*** All covariates suest ***

							cap suest	num_schools num_`type' rank1_`type' ///
									baseline`type' baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
									baseliness_math baseliness_reading baseliness_writing, vce(cluster maskedid)
							
							if !_rc{
								test Z_`type'
								matrix balance[`row'+10, `col'+1] = r(chi2)
								matrix balance[`row'+11, `col'+1] = r(p)
								matrix balance[`row'+12, `col'+1] = r(df)
								matrix balance[`row'+13, `col'+1] = e(N)
							}
							
								
							cap suest	Lnum_schools Lnum_`type' Lrank1_`type' ///
									Lbaseline`type' Lbaselinefemale Lbaselinehisp Lbaselineblack Lbaselinefoodfred Lbaselinesped Lbaselinebilingual Lbaselinelep Lbaselinegifted ///
									Lbaseliness_math Lbaseliness_reading Lbaseliness_writing, vce(cluster maskedid)
							
							if !_rc{
								test Z_`type'
								matrix balance[`row'+10, `col'+2] = r(chi2)
								matrix balance[`row'+11, `col' +2] = r(p)
								matrix balance[`row'+12, `col'+2] = r(df)
								matrix balance[`row'+13, `col'+2] = e(N)
							}
							
							cap suest	Pnum_schools Pnum_`type'  Prank1_`type' ///
									Pbaseline`type' Pbaselinefemale Pbaselinehisp Pbaselineblack Pbaselinefoodfred Pbaselinesped Pbaselinelep Pbaselinebilingual Pbaselinegifted ///
									Pbaseliness_math Pbaseliness_reading Pbaseliness_writing, vce(cluster maskedid)
							
							if !_rc{
								test Z_`type'
								matrix balance[`row'+10, `col'+3] = r(chi2)
								matrix balance[`row'+11, `col' +3] = r(p)
								matrix balance[`row'+12, `col'+3] = r(df)
								matrix balance[`row'+13, `col'+3] = e(N)
							}
							
							local c=0
							foreach i in 01 001 0{
								local c = `c' + 1
								cap suest	B`i'num_schools B`i'num_`type'  B`i'rank1_`type' ///
										B`i'baseline`type' B`i'baselinefemale B`i'baselinehisp B`i'baselineblack B`i'baselinefoodfred B`i'baselinesped B`i'baselinelep B`i'baselinebilingual B`i'baselinegifted ///
										B`i'baseliness_math B`i'baseliness_reading B`i'baseliness_writing, vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row'+10, `col' +3 + `c'] = r(chi2)
									matrix balance[`row'+11, `col' +3 + `c'] = r(p)
									matrix balance[`row'+12, `col' +3 + `c'] = r(df)
									matrix balance[`row'+13, `col' +3 + `c'] = e(N)
									
									
									unique tag_`type'`i' if e(sample)
									matrix balance[`row'+14, `col' +3 + `c'] = r(sum)
									
								}
							}
						
					} /*Close n e loop*/
					
				local row = `row'+ 15
				} /*Close f_cut loop*/
				
					**********************************************************************************
					
				*Outsheet
				preserve
					svmat balance
					keep var balance*
					keep if _n<=`row'+1
					ren balance1 z0_f
					ren balance2 ols_z1_f
					ren balance3 ols_z1_pscore_f
					ren balance4 ols_z1_poly_f
					ren balance5 ols_z1_r01_f
					ren balance6 ols_z1_r001_f
					ren balance7 ols_z1_psat_f
					
					ren balance8 	z0_o
					ren balance9 	ols_z1_o
					ren balance10 	ols_z1_pscore_o
					ren balance11 	ols_z1_poly_o
					ren balance12 	ols_z1_r01_o
					ren balance13 	ols_z1_r001_o
					ren balance14 	ols_z1_psat_o
					
					
					outsheet using "`tables'inDPSbase_`file'_fc_oc_stacked_balance.csv", comma replace
				restore
				
				} /*Close F test condition*/
		}	/*Close file loop*/
	} /*Close balance condition*/
		

					

					
					


	*****************************************************
	******************* Separate 2SLS *******************
	*****************************************************
	
	if `pscore_separate_twosls'==1{
		
		foreach file in frequency simulation formula {
				foreach type in f o{
								
					if "`type'"=="f"{
						local colbump = 0
					}
					else if "`type'"=="o"{
						local colbump = 19
					}
					
					local row = 1
					local col = 1 + `colbump'
				
					u "`cleandata'analysis_fc_oc_2endog_stacked_`file'.dta", clear
					
					keep if in_f_sample == 1 | in_o_sample == 1
				
					gen var =""
					
					*By grade vars
					gen group = .
				
					foreach group in 410 /*45 68 48 910*/{
								if `group'==410{
									replace group=410
								}
								else if `group'==48{
									replace group=48 if inlist(appgrade,4,5,6,7,8)
								}
								else{
								replace group=45 if inlist(appgrade,4,5)
								replace group=68 if inlist(appgrade,6,7,8)
								replace group=910 if inlist(appgrade,9,10)
								}
		
								foreach t in math writing reading{
									foreach out in ss /*prof_up adv*/{
							
									
									replace var="`out'_`t'_`group'" if _n==`row'
									replace var="`out'_`t'_`group'_se" if _n==`row'+1
									replace var="`out'_`t'_`group'_N" if _n==`row'+2
									
				
									qui summ `out'_`t' if group==`group' & in_zero1_`type'pscore==1
									
									if ( r(max)==0 & r(min)==0 ) | r(N)<50{ 
										di in red "No observations for `group'-level 2SLS estimates"
									}
								
									else{
								
										qui summ `out'_`t' if Z_`type' == 0 & group==`group'
										matrix twosls[`row',`col'] = r(mean)
										matrix twosls[`row'+2,`col'] = r(N)
										
											/*OLS - no control for pscore*/
											qui reg `out'_`t' D_`type' `cov' `cov2' `cov_grade' baselinespyear if group==`group' & in_`type'_sample==1, vce(cluster maskedid)
											matrix twosls[`row',`col'+1]   = _b[D]
											matrix twosls[`row'+1,`col'+1] = _se[D]
											matrix twosls[`row'+2,`col'+1] = e(N)
											
											/*OLS conditional on fully saturated pscore sample*/
											qui reg `out'_`t' D_`type' `cov' `cov2' `cov_grade' baselinespyear if group==`group' & in_good_cell_`type'0==1, vce(cluster maskedid)
											matrix twosls[`row',`col'+2]   = _b[D]
											matrix twosls[`row'+1,`col'+2] = _se[D]
											matrix twosls[`row'+2,`col'+2] = e(N)
											
											/*OLS conditional controlling for fully saturated pscores*/
											*qui reg `out'_`t' D_`type' i.ind_`type'0 `cov' `cov2'  `cov_grade' baselinespyear if group==`group' & in_good_cell_`type'0==1, vce(cluster maskedid)

											matrix twosls[`row',`col'+3]   = 0 //_b[D]
											matrix twosls[`row'+1,`col'+3] =  0 //_se[D]
											matrix twosls[`row'+2,`col'+3] = 0 //e(N)
									
									foreach instrument_type in ordinary /*over_id*/  {
										
										if "`instrument_type'" == "ordinary" {
											local instrument "Z_`type'"
										}
										else {
											local instrument "Z_`type' Z_`type'_first"
										}
										
										foreach risk in `type'01 `type'001 `type'0{
							
											local col = `col' + 2
											
											local controls "i.ind_`risk' `cov_`type'' `cov2'  `cov_grade'"		
											 
												
											/*2SLS - outcome on treatment instrumented by instrument*/
											ivreg2 `out'_`t' (D_`type' = `instrument') `controls' baselinespyear if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls' baselinespyear) cluster(maskedid)

												*2SLS
												matrix twosls[`row',`col'+4]   = _b[D]
												matrix twosls[`row'+1,`col'+4] = _se[D]
												matrix twosls[`row'+2,`col'+4] = e(N)
												
												estimates restore firstD_`type'
												
												*First stage
												matrix twosls[`row',`col'+3]   = _b[Z_`type']
												matrix twosls[`row'+1,`col'+3] = _se[Z_`type']
												
											*If last loop, do these
											if "`risk'"=="`type'0"{
												
												*NAIVE 2SLS - no riskset controls
													*2SLS naive
													ivreg2 `out'_`t' (D_`type' = `instrument') `cov_`type'' `cov2' `cov_grade' baselinespyear if group==`group', savefirst savefprefix(first) partial(`cov_`type'' `cov2' `cov_grade' baselinespyear) cluster(maskedid)

														*2SLS
														matrix twosls[`row',`col'+6]   = _b[D]
														matrix twosls[`row'+1,`col'+6] = _se[D]
														matrix twosls[`row'+2,`col'+6] = e(N)
														
														estimates restore firstD_`type'
												
														*First stage
														matrix twosls[`row',`col'+5]   = _b[Z_`type']
														matrix twosls[`row'+1,`col'+5] = _se[Z_`type']
														
													*2SLS naive conditional on r0 sample
													ivreg2 `out'_`t' (D_`type' = `instrument') `cov_`type'' `cov2' `cov_grade' baselinespyear if group==`group' & in_good_cell_`type'0==1, savefirst savefprefix(first) partial(`cov_`type'' `cov2' `cov_grade' baselinespyear) cluster(maskedid)

														*2SLS
														matrix twosls[`row',`col'+8]   = _b[D]
														matrix twosls[`row'+1,`col'+8] = _se[D]
														matrix twosls[`row'+2,`col'+8] = e(N)
														
														estimates restore firstD_`type'
												
														*First stage
														matrix twosls[`row',`col'+7]   = _b[Z_`type']
														matrix twosls[`row'+1,`col'+7] = _se[Z_`type']													
												
												*2SLS with no cov controls								
													*2SLS
													ivreg2 `out'_`t' (D_`type' = `instrument') i.ind_`risk' baselinespyear if group==`group' & in_good_cell_`type'0==1, savefirst savefprefix(first) partial(i.ind_`risk' baselinespyear) cluster(maskedid) 
														
														*2SLS
														matrix twosls[`row',`col'+10]   = _b[D]
														matrix twosls[`row'+1,`col'+10] = _se[D]
														matrix twosls[`row'+2,`col'+10] = e(N)
														
														estimates restore firstD_`type'
												
														*First stage
														matrix twosls[`row',`col'+9]   = _b[Z_`type']
														matrix twosls[`row'+1,`col'+9] = _se[Z_`type']
											} /*close if condition*/
										}  /*risk loop*/
										local col = `col' + 9
										} /* instrument type loop */
									
								} /*close condition on number of observations*/
								
								local row = `row' + 3
								local col = 1 + `colbump'
							} 	 /*out loop*/
							
							local ++row
						}	/* t loop*/
						
					}	/*group loop*/
				} /*f o loop*/
			
				preserve
					svmat twosls
					keep var twosls*
					
					keep if _n<=`row'
					
					ren twosls1 z0_f
					ren twosls2 OLS_f
					ren twosls3 OLS_psample_f
					ren twosls4 OLS_psat_f
					
					ren twosls6 FS_r01_f
					ren twosls7 twoSLS_r01_f
					ren twosls8 FS_r001_f
					ren twosls9 twoSLS_r001_f
					ren twosls10 FS_psat_f
					ren twosls11 twoSLS_psat_f
					ren twosls12 FS_naive_f
					ren twosls13 twoSLS_naive_f
					ren twosls14 FS_naive_psample_f
					ren twosls15 twoSLS_naive_psample_f
					ren twosls16 FS_nocovs_f
					ren twosls17 twoSLS_nocovs_f
					
					/*
					ren twosls21 FS_r01_n_overid
					ren twosls22 twoSLS_r01_n_overid
					ren twosls23 FS_r001_n_overid
					ren twosls24 twoSLS_r001_n_overid
					ren twosls25 FS_psat_n_overid
					ren twosls26 twoSLS_psat_n_overid
					ren twosls27 FS_naive_n_overid
					ren twosls28 twoSLS_naive_n_overid
					ren twosls29 FS_naive_psample_n_overid
					ren twosls30 twoSLS_naive_psample_n_overid
					ren twosls31 FS_nocovs_n_overid
					ren twosls32 twoSLS_nocovs_n_overid
					*/
					
					ren twosls20 z0_o
					ren twosls21 OLS_o
					ren twosls22 OLS_psample_o
					ren twosls23 OLS_psat_o
					
					ren twosls25 FS_r01_o
					ren twosls26 twoSLS_r01_o
					ren twosls27 FS_r001_o
					ren twosls28 twoSLS_r001_o					
					ren twosls29 FS_psat_o
					ren twosls30 twoSLS_psat_o
					ren twosls31 FS_naive_o
					ren twosls32 twoSLS_naive_o
					ren twosls33 FS_naive_psample_o
					ren twosls34 twoSLS_naive_psample_o
					ren twosls35 FS_nocovs_o
					ren twosls36 twoSLS_nocovs_o
					
					outsheet using "`tables'inDPSbase_fc_oc_2endog_`file'_twosls.csv", comma replace
				restore
			clear	

			} /*Close 2SLS condition*/
	} /*Close file loop*/
	
	*****************************************************
	******************** Joint 2SLS *********************
	*****************************************************
	
	if `pscore_joint_twosls'==1{
		
		foreach file in  frequency simulation formula{
		
				u "`cleandata'analysis_fc_oc_2endog_stacked_`file'.dta", clear
				
				
				gen in_joint_sample = (in_f_sample==1 | in_o_sample==1)
				keep if in_joint_sample
				
				
				********************************************************************************************
				**************** Make propensity score replacements and regenerate dummies *****************
				********************************************************************************************
				*Drop old bucket variables
				
					drop f0_* f01_* f001_* o0_* o01_* o001_* in_good_* tag_* obs_cell* pscore_* ind_*

				*other-charter applicants that do not apply to first-choice charter. Replace missing score with zero
					*Score
					replace f0 = 0 if (in_o_sample==1 & in_f_sample==0) & f0==.
					
					*First rounded to hundredth bucket
					replace f01 = 1 if (in_o_sample==1 & in_f_sample==0) & f0==0
					
					*First rounded to thousandth bucket
					replace f001 = 1 if (in_o_sample==1 & in_f_sample==0) & f0==0
				
				
				*first-choice charter applicants who do not apply to other charters. Replace missing score with zero
					*Score
					replace o0 = 0 if (in_o_sample==0 & in_f_sample==1) & o0==.
					
					*First rounded to hundredth bucket
					replace o01 = 1 if (in_o_sample==0 & in_f_sample==1) & o0==0
					
					*First rounded to thousandth bucket
					replace o001 = 1 if (in_o_sample==0 & in_f_sample==1) & o0==0
					
				*In (0,1) for either charter or innovation
					
					gen in_zero1_joint =  ( (f0 > 0 & f0 < 1 ) | (o0 > 0 & o0 < 1 ) )
				
				*Joint propensity score
					
					egen fo0 = group(f0 o0) if in_zero1_joint
					egen fo01 = group(f01 o01) if in_zero1_joint
					egen fo001 = group(f001 o001) if in_zero1_joint
					
				
					
				******************************************************************
				************ Replace missing treatment and instrument ************
				******************************************************************
				
				foreach x in Z D{
					replace `x'_f = 0 if `x'_f ==.
					replace `x'_o = 0 if `x'_o ==.
				}
				
				*In cell with variation?
					
					foreach x in 01 001 0{
						
						*Variation in instruments for each cell
							bys f`x': egen sd_z_f`x'	= 	sd(Z_f)
							bys o`x': egen sd_z_o`x'	= 	sd(Z_o)
						
						*Good cell
							gen in_good_cell_`x' = ( in_zero1_joint==1	& ///
								 ( 	(sd_z_f`x'>0 & !missing(sd_z_f`x')) | (sd_z_o`x'>0 & !missing(sd_z_o`x'))	) )
						
					}
					drop sd_z_*
					
					foreach x in 01 001 0{
						bys fo`x': egen sd_z_f	= 	sd(Z_f)
						bys fo`x': egen sd_z_o	= 	sd(Z_o)
						
						gen in_good_cell_fo`x' = ( in_zero1_joint==1	& ///
										(	( sd_z_f>0 & !missing(sd_z_f) ) | ( sd_z_o>0 & !missing(sd_z_o) ) ) )
						
						tab fo`x' if in_good_cell_fo`x' ==1
						
						drop sd_z_*
					}
					
				*Regenerate buckets
				
				foreach x in f0 f001 f01  o0 o001 o01 fo0 fo001 fo01{
					tab `x', gen(`x'_)
				}
				
				* Regenerating indicator to use factor variables in 2sls regs (much faster)
				egen ind_f0 = group(f0)
				egen ind_f001 = group(f001)
				egen ind_f01 = group(f01)
				
				egen ind_o0 = group(o0)
				egen ind_o001 = group(o001)
				egen ind_o01 = group(o01)	
				
				* not necessary to do this, but we can keep consistent variable names this way. 
				egen ind_fo0 = group(ind_f0 ind_o0) if in_zero1_joint
				egen ind_fo01 = group(ind_f01 ind_o01) if in_zero1_joint
				egen ind_fo001 = group(ind_f001 ind_o001) if in_zero1_joint					
					
				local col = 1
				local row = 1
				gen var =""
				
				
				*By grade vars
				gen group = .
			
				count if in_zero1_joint==1 & inlist(appgrade,4,5,6,7,8)
				count if in_good_cell_0==1 & inlist(appgrade,4,5,6,7,8)
				
				foreach group in 410 /*45 68 48 910*/{
							if `group'==410{
								replace group=410
							}
							else if `group'==48{
								replace group=48 if inlist(appgrade,4,5,6,7,8)
							}
							else{
							replace group=45 if inlist(appgrade,4,5)
							replace group=68 if inlist(appgrade,6,7,8)
							replace group=910 if inlist(appgrade,9,10)
							}
	
							foreach t in math writing reading{
								foreach out in ss /*prof_up adv*/{
								
								local col = 1
								
								replace var="`out'_`t'_`group'" if _n==`row'
								replace var="`out'_`t'_`group'_se" if _n==`row'+1
								replace var="`out'_`t'_`group'_N" if _n==`row'+2
								
			
								qui summ `out'_`t' if group==`group'
								
								if ( r(max)==0 & r(min)==0 ) | r(N)<50{ 
									di in red "Not enough observations for `group'-level 2SLS estimates"
								}
							
								else{
							
									summ `out'_`t' if Z_f == 0 & in_f_sample & group==`group'
									matrix jointtwosls[`row',`col'] = r(mean)
									matrix jointtwosls[`row'+2,`col'] = r(N)
									
									 summ `out'_`t' if Z_o == 0 & in_o_sample & group==`group'
									matrix jointtwosls[`row',`col'+1] = r(mean)
									matrix jointtwosls[`row'+2,`col'+1] = r(N)
									
									local col = `col' + 1
									
										/*OLS - no control for pscore*/
										qui reg `out'_`t' D_f D_o `cov_either' `cov_grade' baselinespyear if group==`group', vce(cluster maskedid)
										matrix jointtwosls[`row',`col'+1]   = _b[D_f]
										matrix jointtwosls[`row'+1,`col'+1] = _se[D_f]
										matrix jointtwosls[`row'+2,`col'+1] = e(N)
										
										matrix jointtwosls[`row',`col'+2]   = _b[D_o]
										matrix jointtwosls[`row'+1,`col'+2] = _se[D_o]
										matrix jointtwosls[`row'+2,`col'+2] = e(N)

										
										/*OLS conditional on fully saturated pscore sample*/
										qui reg `out'_`t' D_f D_o `cov_either' `cov_grade' baselinespyear if group==`group' & in_zero1_joint==1, vce(cluster maskedid)
										matrix jointtwosls[`row',`col'+3]   = _b[D_f]
										matrix jointtwosls[`row'+1,`col'+3] = _se[D_f]
										matrix jointtwosls[`row'+2,`col'+3] = e(N)
										
										matrix jointtwosls[`row',`col'+4]   = _b[D_o]
										matrix jointtwosls[`row'+1,`col'+4] = _se[D_o]
										matrix jointtwosls[`row'+2,`col'+4] = e(N)

										/*OLS conditional controlling for fully saturated pscores*/
										*qui reg `out'_`t' D_f D_o i.ind_f0 i.ind_o0 baselinespyear if group==`group' & in_zero1_joint==1, vce(cluster maskedid)
										matrix jointtwosls[`row',`col'+5]   = 0 //_b[D_f]
										matrix jointtwosls[`row'+1,`col'+5] = 0 //_se[D_f]
										matrix jointtwosls[`row'+2,`col'+5] = 0 //e(N)
										
										matrix jointtwosls[`row',`col'+6]   = 0 //_b[D_o]
										matrix jointtwosls[`row'+1,`col'+6] = 0 //_se[D_o]
										matrix jointtwosls[`row'+2,`col'+6] = 0 //e(N)

										local col = `col' + 6
										
										*qui reg `out'_`t' D_f D_o i.ind_f0 i.ind_o0 baselinespyear if group==`group' & in_zero1_joint==1, vce(cluster maskedid)
										local f_factor_b = 0 //_b[D_f]
										local f_factor_se = 0 //_se[D_f]
										
										
										*qui reg `out'_`t' D_f D_o f0_* o0_* baselinespyear if group==`group' & in_zero1_joint==1, vce(cluster maskedid)		
										local f_old_b =  0 //_b[D_f]
										local f_old_se = 0 //_se[D_f]	
										

										
									foreach instrument_type in ordinary /*over_id*/  {
										
										if "`instrument_type'" == "ordinary" {
											local instrument "Z_f Z_o"
										}
										else {
											local instrument "Z_n Z_n_first Z_e Z_e_first Z_i Z_i_first"
										}
										
										*Joint 2SLS conditioning separately for risksets
										foreach x in 01 001 0{

											ivreg2 `out'_`t' (D_f D_o = `instrument') i.ind_f`x' i.ind_o`x' `cov_either' `cov2' `cov_grade' baselinespyear if group==`group' ///
													& in_good_cell_`x'==1, savefirst savefprefix(first) partial(i.ind_f`x' i.ind_o`x' `cov_either' `cov2' `cov_grade' baselinespyear) cluster(maskedid) 
												
			
											*2SLS - first choice charter
											matrix jointtwosls[`row',`col'+1]   = _b[D_f]
											matrix jointtwosls[`row'+1,`col'+1] = _se[D_f]
											matrix jointtwosls[`row'+2,`col'+1] = e(N)
											
											*2SLS - other choice charter
											matrix jointtwosls[`row',`col'+2] = _b[D_o]
											matrix jointtwosls[`row'+1,`col'+2] = _se[D_o]
											matrix jointtwosls[`row'+2,`col'+2] = e(N)
											
											
											
											
											*First stage first-choice charter
											estimates restore firstD_f
											matrix jointtwosls[`row',`col'+3]   = _b[Z_f]
											matrix jointtwosls[`row'+1,`col'+3] = _se[Z_f]

											matrix jointtwosls[`row',`col'+4]   = _b[Z_o]
											matrix jointtwosls[`row'+1,`col'+4] = _se[Z_o]
											
											*First stage other choice charter
											estimates restore firstD_o
											matrix jointtwosls[`row',`col'+5] = _b[Z_f]
											matrix jointtwosls[`row'+1,`col'+5] = _se[Z_f]
											
											matrix jointtwosls[`row',`col'+6] = _b[Z_o]
											matrix jointtwosls[`row'+1,`col'+6] = _se[Z_o]
											
											local col = `col' + 6
										}
											
										*Joint 2SLS conditioning for joint risksets
										foreach x in 01 001 0{
											ivreg2 `out'_`t' (D_f D_o = `instrument') i.ind_fo`x' `cov_either' `cov2' `cov_grade' baselinespyear if group==`group' ///
													& in_good_cell_fo`x'==1, savefirst savefprefix(first) partial(i.ind_fo`x' `cov_either' `cov2' `cov_grade' baselinespyear) cluster(maskedid)  

											*2SLS - first choice charter
											matrix jointtwosls[`row',`col'+1]   = _b[D_f]
											matrix jointtwosls[`row'+1,`col'+1] = _se[D_f]
											matrix jointtwosls[`row'+2,`col'+1] = e(N)
											
											*2SLS - other choice charter
											matrix jointtwosls[`row',`col'+2] = _b[D_o]
											matrix jointtwosls[`row'+1,`col'+2] = _se[D_o]
											matrix jointtwosls[`row'+2,`col'+2] = e(N)
											
											
											*First stage first-choice charter
											estimates restore firstD_f
											matrix jointtwosls[`row',`col'+3]   = _b[Z_f]
											matrix jointtwosls[`row'+1,`col'+3] = _se[Z_f]

											matrix jointtwosls[`row',`col'+4]   = _b[Z_o]
											matrix jointtwosls[`row'+1,`col'+4] = _se[Z_o]
											
											*First stage other choice charter
											estimates restore firstD_o
											matrix jointtwosls[`row',`col'+5] = _b[Z_f]
											matrix jointtwosls[`row'+1,`col'+5] = _se[Z_f]
											
											matrix jointtwosls[`row',`col'+6] = _b[Z_o]
											matrix jointtwosls[`row'+1,`col'+6] = _se[Z_o]
											
											local col = `col' + 6
										
										} /*Close riskset loop*/
										
										*NAIVE 2SLS - no riskset controls
											*2SLS naive
											ivreg2 `out'_`t' 	(D_f D_o = `instrument') `cov_either' `cov2' `cov_grade' baselinespyear if group==`group', ///
																savefirst savefprefix(first) partial(`cov_either' `cov2' `cov_grade' baselinespyear) cluster(maskedid)  

											*2SLS - first choice charter
												matrix jointtwosls[`row',`col'+1]   = _b[D_f]
												matrix jointtwosls[`row'+1,`col'+1] = _se[D_f]
												matrix jointtwosls[`row'+2,`col'+1] = e(N)
												
												*2SLS - other choice charter
												matrix jointtwosls[`row',`col'+2] = _b[D_o]
												matrix jointtwosls[`row'+1,`col'+2] = _se[D_o]
												matrix jointtwosls[`row'+2,`col'+2] = e(N)
												
												*First stage first-choice charter
												estimates restore firstD_f
												matrix jointtwosls[`row',`col'+3]   = _b[Z_f]
												matrix jointtwosls[`row'+1,`col'+3] = _se[Z_f]

												matrix jointtwosls[`row',`col'+4]   = _b[Z_o]
												matrix jointtwosls[`row'+1,`col'+4] = _se[Z_o]
												
												*First stage other choice charter
												estimates restore firstD_o
												matrix jointtwosls[`row',`col'+5] = _b[Z_f]
												matrix jointtwosls[`row'+1,`col'+5] = _se[Z_f]
												
												matrix jointtwosls[`row',`col'+6] = _b[Z_o]
												matrix jointtwosls[`row'+1,`col'+6] = _se[Z_o]
													
												
											*2SLS naive conditional on r0 sample
											ivreg2 `out'_`t' 	(D_f D_o = `instrument') `cov_either' `cov2' `cov_grade' baselinespyear if group==`group' & in_zero1_joint==1, ///
																savefirst savefprefix(first) partial(`cov_either' `cov2' `cov_grade' baselinespyear) cluster(maskedid)  
												*2SLS - first choice charter
												matrix jointtwosls[`row',`col'+7]   = _b[D_f]
												matrix jointtwosls[`row'+1,`col'+7] = _se[D_f]
												matrix jointtwosls[`row'+2,`col'+7] = e(N)
											
												*2SLS - other choice charter
												matrix jointtwosls[`row',`col'+8] = _b[D_o]
												matrix jointtwosls[`row'+1,`col'+8] = _se[D_o]
												matrix jointtwosls[`row'+2,`col'+8] = e(N)
												
												*First stage first-choice charter
												estimates restore firstD_f
												matrix jointtwosls[`row',`col'+9]   = _b[Z_f]
												matrix jointtwosls[`row'+1,`col'+9] = _se[Z_f]

												matrix jointtwosls[`row',`col'+10]   = _b[Z_o]
												matrix jointtwosls[`row'+1,`col'+10] = _se[Z_o]
												
												*First stage other choice charter
												estimates restore firstD_o
												matrix jointtwosls[`row',`col'+11] = _b[Z_f]
												matrix jointtwosls[`row'+1,`col'+11] = _se[Z_f]
												
												matrix jointtwosls[`row',`col'+12] = _b[Z_o]
												matrix jointtwosls[`row'+1,`col'+12] = _se[Z_o]									

									
										*2SLS with no cov controls								
											*2SLS
											ivreg2 `out'_`t' 	(D_f D_o = `instrument') baselinespyear if group==`group' & in_zero1_joint==1, ///
																savefirst savefprefix(first) partial(baselinespyear) cluster(maskedid)  
											
												*2SLS - first choice charter
												matrix jointtwosls[`row',`col'+13]   = _b[D_f]
												matrix jointtwosls[`row'+1,`col'+13] = _se[D_f]
												matrix jointtwosls[`row'+2,`col'+13] = e(N)
												
												*2SLS - other choice charter 
												matrix jointtwosls[`row',`col'+14] = _b[D_o]
												matrix jointtwosls[`row'+1,`col'+14] = _se[D_o]
												matrix jointtwosls[`row'+2,`col'+14] = e(N)
												
												*First stage first-choice charter
												estimates restore firstD_f
												matrix jointtwosls[`row',`col'+15]   = _b[Z_f]
												matrix jointtwosls[`row'+1,`col'+15] = _se[Z_f]

												matrix jointtwosls[`row',`col'+16]   = _b[Z_o]
												matrix jointtwosls[`row'+1,`col'+16] = _se[Z_o]
												
												*First stage other choice charter
												estimates restore firstD_o
												matrix jointtwosls[`row',`col'+17] = _b[Z_f]
												matrix jointtwosls[`row'+1,`col'+17] = _se[Z_f]
												
												matrix jointtwosls[`row',`col'+18] = _b[Z_o]
												matrix jointtwosls[`row'+1,`col'+18] = _se[Z_o]		
										
												
									local col = `col' + 18
									} /* instrument type loop */													
							} /*close condition on number of observations*/
							
							local row = `row' + 3
						} 	 /*out loop*/
						
						local ++row
					}	/* t loop*/
					
				}	/*group loop*/
		
			preserve
				svmat jointtwosls
				keep var jointtwosls*

				keep if _n<=`row'
				ren jointtwosls1 z0_f
				ren jointtwosls2 z0_o
				ren jointtwosls3 OLS_f
				ren jointtwosls4 OLS_o
			
				ren jointtwosls5 OLS_psample_f
				ren jointtwosls6 OLS_psample_o
				ren jointtwosls7 OLS_psat_f
				ren jointtwosls8 OLS_psat_o
		
				ren jointtwosls9 jointtwosls_r01_f
				ren jointtwosls10 jointtwosls_r01_o
				ren jointtwosls11 FS_r01_D_f_f
				ren jointtwosls12 FS_r01_D_f_o
				ren jointtwosls13 FS_r01_D_o_f
				ren jointtwosls14 FS_r01_D_o_o
				
				ren jointtwosls15 jointtwosls_r001_f
				ren jointtwosls16 jointtwosls_r001_o
				ren jointtwosls17 FS_r001_D_f_f
				ren jointtwosls18 FS_r001_D_f_o
				ren jointtwosls19 FS_r001_D_o_f
				ren jointtwosls20 FS_r001_D_o_o
				
				ren jointtwosls21 jointtwosls_r0_f
				ren jointtwosls22 jointtwosls_r0_o
				ren jointtwosls23 FS_r0_D_f_f
				ren jointtwosls24 FS_r0_D_f_o
				ren jointtwosls25 FS_r0_D_o_f
				ren jointtwosls26 FS_r0_D_o_o
	
				ren jointtwosls27 jointtwosls_joint_r01_f
				ren jointtwosls28 jointtwosls_joint_r01_o
				ren jointtwosls29 FS_joint_r01_D_f_f
				ren jointtwosls30 FS_joint_r01_D_f_o
				ren jointtwosls31 FS_joint_r01_D_o_f
				ren jointtwosls32 FS_joint_r01_D_o_o
				
				ren jointtwosls33 jointtwosls_joint_r001_f
				ren jointtwosls34 jointtwosls_joint_r001_o
				ren jointtwosls35 FS_joint_r001_D_f_f
				ren jointtwosls36 FS_joint_r001_D_f_o
				ren jointtwosls37 FS_joint_r001_D_o_f
				ren jointtwosls38 FS_joint_r001_D_o_o
			
				ren jointtwosls39 jointtwosls_joint_r0_f
				ren jointtwosls40 jointtwosls_joint_r0_o
				ren jointtwosls41 FS_joint_r0_D_f_f
				ren jointtwosls42 FS_joint_r0_D_f_o
				ren jointtwosls43 FS_joint_r0_D_o_f
				ren jointtwosls44 FS_joint_r0_D_o_o
		
				ren jointtwosls45 jointtwosls_naive_f
				ren jointtwosls46 jointtwosls_naive_o
				ren jointtwosls47 FS_naive_D_f_f
				ren jointtwosls48 FS_naive_D_f_o
				ren jointtwosls49 FS_naive_D_o_f
				ren jointtwosls50 FS_naive_D_o_o
				
				ren jointtwosls51 jointtwosls_naive_psample_f
				ren jointtwosls52 jointtwosls_naive_psample_o
				ren jointtwosls53 FS_naive_psample_D_f_f
				ren jointtwosls54 FS_naive_psample_D_f_o
				ren jointtwosls55 FS_naive_psample_D_o_f
				ren jointtwosls56 FS_naive_psample_D_o_o
				
				ren jointtwosls57 jointtwosls_naive_nocovs_f
				ren jointtwosls58 jointtwosls_naive_nocovs_o
				ren jointtwosls59 FS_naive_nocovs_D_f_f
				ren jointtwosls60 FS_naive_nocovs_D_f_o
				ren jointtwosls61 FS_naive_nocovs_D_o_f
				ren jointtwosls62 FS_naive_nocovs_D_o_o
				
				outsheet using "`tables'inDPSbase_fc_oc_2endog_`file'_joint_twosls.csv", comma replace
			restore
		clear	

		} /*Close 2SLS condition*/
} /*Close file loop*/
					
					
					
					
