/* 	Ye Ji Kee
	Denver tables (year 1). These comments last edited: 4/25/2016
		For grades 4-10
		Output: Balance, 2SLS, and subgroup for pscore risksets and qualification instruments
		
		NOTE: This do-file uses datasets created in 2_1_create_pscore_analysis_files_fc_nonfc.do and 2_1_create_pscore_year_2_analysis_files_fc_nonfc.do
	
	This si for the 4 sector model. 
	Use the frequency output;
	Columns: jointtwosls_r0_fn jointtwosls_r0_fe jointtwosls_r0_on jointtwosls_r0_oe 
	Rows: 
	ss_math_410
ss_math_410_se
*/



clear all
cap log close
set more off
set trace off
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"


local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"

matrix demo			  = J(30,15,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,150,.)
matrix jointtwosls	  = J(500,150,.)

local pscore_setup_yr1 = 1
local pscore_setup_yr2 = 1
local pscore_stacking = 1
local pscore_separate_twosls = 0
local pscore_joint_twosls = 0

*Dates of analysis files
local pscoredate	20170310 	/*20160715 20160513  20160426 */
local simsdate 		20150603	/*20150428*/

local appcov_fn "num_schools num_study rank1_study"
local cov_fn 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2		"baseliness_math baseliness_read ing baseliness_writing"

local appcov_fe "num_schools num_study rank1_study"
local cov_fe 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_1 g_2 g_3 g_4 g_5 g_6 g_7"

local appcov_on "num_schools num_study rank1_study"
local cov_on    "baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local appcov_oe "num_schools num_study rank1_study"
local cov_oe    "baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local appcov_either  "num_schools num_study rank1_study"
local cov_either 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=0				/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/






	************************************************
	**************** Pscore Setup ******************
	************************************************		
	
	if `pscore_setup_yr1'==1{
		
		foreach file in frequency simulation formula {
	
			use "`cleandata'mdrd_denver_fc_nonfc_`file' `pscoredate'", clear
			
			destring baselinegrade, replace
			destring appgrade, gen(appgrade_tmp)
			drop appgrade
			rename appgrade_tmp appgrade
			
			*High school flag			
				gen hsgrade = ( appgrade>=9)
				gen msgrade = ( appgrade<9)
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
				
			*Analysis at the student level. Generate max for charter offer
			bys maskedid: egen any_study_offer_fc_3cmo = max(offer_study_fc_3cmo)
			bys maskedid: egen any_study_offer_fc_n3cmo = max(offer_study_fc_n3cmo)
			bys maskedid: egen any_study_offer_oc_3cmo = max(offer_study_oc_3cmo)
			bys maskedid: egen any_study_offer_oc_n3cmo = max(offer_study_oc_n3cmo)			
			
			bys maskedid: egen num_study = count(cond(appstudy==1,appschoolnum,.))

			bys maskedid: egen num_schools = count(appschoolnum)
			
			*Enrolls at charter flag (student level)
			bys maskedid: egen enroll_any_study_fc_3cmo 	= max(study_fc_3cmo)
			bys maskedid: egen enroll_any_study_fc_n3cmo 	= max(study_fc_n3cmo)
			bys maskedid: egen enroll_any_study_oc_3cmo		= max(study_oc_3cmo)			
			bys maskedid: egen enroll_any_study_oc_n3cmo	= max(study_oc_n3cmo)
			
			*Rank 1 at school level
			gen rank1_study= (appstudy==1 & apprank==1)

			******************************************************************************
			
			*First ranked school is charter (student level, for cov balance)
				bys maskedid: egen tempstudy = max(rank1_study)
				drop rank1_study
				ren tempstudy rank1_study

				*Flag - apply to any study
				bys maskedid: egen maxstudy = max(appstudy)
				drop appstudy
				ren maxstudy appstudy
				
				
				foreach type in fc_3cmo fc_n3cmo oc_3cmo oc_n3cmo {
				
				*Flag - first choice is any study
				bys maskedid: egen maxstudy_`type' = max(appstudy_`type')
				drop appstudy_`type'
				ren maxstudy_`type' appstudy_`type'
				}
				
			*Enrolls next year (i.e. has nonmissing schoolnum)
			gen next = (schoolnum!=.)
			bys maskedid: egen enrolls_next = max(next)
			
			*In_sample was previously defined at the student-school level. Redefine it so
			*that it is at the student level
			
			foreach type in fc_3cmo fc_n3cmo oc_3cmo oc_n3cmo {
				bys maskedid: egen in_study`type'_sample_max = max(in_study`type'_sample)
				tab in_study`type'_sample_max in_study`type'_sample
				
				drop in_study`type'_sample
				rename in_study`type'_sample_max in_study`type'_sample
			}
			
			*** Dataset at student level from here on ****
			duplicates drop maskedid, force
			drop next
			
			*Redefine has scores so that it excludes sciences - very few kids
			replace has_sc = (has_sc_math | has_sc_reading | has_sc_writing)
			
			*School-level obs
			drop appschoolnum appschoolname offer offer_study offer_study_fc offer_study_oc offer_study_fc_3cmo offer_study_fc_n3cmo offer_study_oc_3cmo offer_study_oc_n3cmo ///
				appcharter appstudy appstudy_fc appstudy_oc appstudy_fc_3cmo appstudy_fc_n3cmo appstudy_oc_3cmo appstudy_oc_n3cmo apprank ///
				study study_fc study_oc study_fc_3cmo study_fc_n3cmo study_oc_3cmo study_oc_n3cmo charter

			*Keep only if in sample
			keep if in_studyfc_3cmo_sample==1 | in_studyfc_n3cmo_sample == 1 | in_studyoc_3cmo_sample==1 | in_studyoc_n3cmo_sample == 1
			
			*Define r001 as meaning pscore rounded to 0.001 and r01 as rounded to 0.01
			gen fn001 = round(pscore_studyfc_3cmo, 0.001)
			gen fn01 = round(pscore_studyfc_3cmo, 0.01)
			
			gen fe001 = round(pscore_studyfc_n3cmo, 0.001)
			gen fe01 = round(pscore_studyfc_n3cmo, 0.01)
			
			gen on001 = round(pscore_studyoc_3cmo, 0.001)
			gen on01 = round(pscore_studyoc_3cmo, 0.01)

			gen oe001 = round(pscore_studyoc_n3cmo, 0.001)
			gen oe01 = round(pscore_studyoc_n3cmo, 0.01)
			
			tab fn001, gen(fn001_)
			tab fn01, gen(fn01_)			
			tab fe001, gen(fe001_)
			tab fe01, gen(fe01_)
			
			tab on01, gen(on01_)
			tab on001, gen(on001_)
			tab oe01, gen(oe01_)
			tab oe001, gen(oe001_)
			
			*Polynomials
			forvalues i=1/4{
				gen double pscore_fn`i'=pscore_studyfc_3cmo^(`i')	
				gen double pscore_fe`i'=pscore_studyfc_n3cmo^(`i')

				gen double pscore_on`i'=pscore_studyoc_3cmo^(`i')
				gen double pscore_oe`i'=pscore_studyoc_n3cmo^(`i')
			}
			
			keep if baselinespyear==2012
			
			*Treatment is any study, instrument is study offer, riskset is pscore
			ren any_study_offer_fc_3cmo 	Z_fn
			ren any_study_offer_fc_n3cmo 	Z_fe
			ren any_study_offer_oc_3cmo 	Z_on
			ren any_study_offer_oc_n3cmo 	Z_oe
			
			ren enroll_any_study_fc_3cmo  	D_fn
			ren enroll_any_study_fc_n3cmo  	D_fe
			ren enroll_any_study_oc_3cmo 	D_on
			ren enroll_any_study_oc_n3cmo 	D_oe
			
			**********************************************************************************
			******************** Flag kids who are ineligble and guaranteed ******************
			**********************************************************************************
			*Flag kids who are ineligble and guaranteed - do not drop them, we still want them 
			*for OLS regressions and naive 2SLS (i.e. not controlling for risksets)
			
				*If simulation, first define in_zero1_pscore based on true score, then
			
					if "`file'"=="simulation"{
					
						ren pscore_studyfc_3cmo_simulation sim_pscore_fn
						ren pscore_studyfc_n3cmo_simulation sim_pscore_fe
						ren pscore_studyoc_3cmo_simulation sim_pscore_on
						ren pscore_studyoc_n3cmo_simulation sim_pscore_oe
						
						gen in_zero1_fnpscore = (sim_pscore_fn > 0 & sim_pscore_fn < 1)
						gen in_zero1_fepscore = (sim_pscore_fe > 0 & sim_pscore_fe < 1)
						gen in_zero1_onpscore = (sim_pscore_on > 0 & sim_pscore_on < 1)
						gen in_zero1_oepscore = (sim_pscore_oe > 0 & sim_pscore_oe < 1)

						gen double pscore_studyfc_3cmo_simulation = round(sim_pscore_fn, 0.0001)
						gen double pscore_studyfc_n3cmo_simulation = round(sim_pscore_fe, 0.0001)
						gen double pscore_studyoc_3cmo_simulation = round(sim_pscore_on, 0.0001)
						gen double pscore_studyoc_n3cmo_simulation = round(sim_pscore_oe, 0.0001)
						
						tab pscore_studyfc_3cmo_`file', gen(fn0_)
						tab pscore_studyoc_3cmo_`file', gen(on0_)
						tab pscore_studyfc_n3cmo_`file', gen(fe0_)
						tab pscore_studyoc_n3cmo_`file', gen(oe0_)
						
						ren pscore_studyfc_3cmo_`file' fn0
						ren pscore_studyfc_n3cmo_`file' fe0
						ren pscore_studyoc_3cmo_`file' on0					
						ren pscore_studyoc_n3cmo_`file' oe0						
				
					}
					else{
						
						tab pscore_studyfc_3cmo_`file', gen(fn0_)
						tab pscore_studyoc_3cmo_`file', gen(on0_)
						
						tab pscore_studyfc_n3cmo_`file', gen(fe0_)
						tab pscore_studyoc_n3cmo_`file', gen(oe0_)
						
						ren pscore_studyfc_3cmo_`file' fn0
						ren pscore_studyoc_3cmo_`file' on0
						
						ren pscore_studyfc_n3cmo_`file' fe0
						ren pscore_studyoc_n3cmo_`file' oe0
						
						gen in_zero1_fnpscore = (fn0 > 0 & fn0 < 1)
						gen in_zero1_onpscore = (on0 > 0 & on0 < 1)
						gen in_zero1_fepscore = (fe0 > 0 & fe0 < 1)
						gen in_zero1_oepscore = (oe0 > 0 & oe0 < 1)
					}
					
			**********************************************************************************
			********** Flag cells with variation in instrument and characteristics************
			**********************************************************************************
					
			/* 	Flag observations that actually make it into balance (i.e. obs in cells with
				variation in instrument). These flags are used for all types 
				of F-tests, conducted below.
				
				We generate these flags here so that no re-sorting of the data (done in bys x egen:)
				is done between the balanceregs and F tests, which affects how the variable
				"var", which stores the names of the variables corresponding to each balance
				coefficient, appears in the final dataset once we svmat the balance output.
			*/
			
			ren in_studyfc_3cmo_sample in_fn_sample
			ren in_studyfc_n3cmo_sample in_fe_sample
			ren in_studyoc_3cmo_sample in_on_sample
			ren in_studyoc_n3cmo_sample in_oe_sample
			
			tab in_fn_sample, m
			tab in_on_sample, m
			tab in_fe_sample, m
			tab in_oe_sample, m
			
			foreach type in fn on fe oe{
				foreach x in 01 001 0{
					
					*Variation in instruments for each cell
						bys `type'`x': egen sd_z_`type'`x'	= 	sd(Z_`type')
					
					*Good cell
						gen in_good_cell_`type'`x' = (sd_z_`type'`x'>0 & !missing(sd_z_`type'`x') & in_zero1_`type'pscore==1) if in_`type'_sample==1
					
					gen double tag_`type'`x' = `type'`x' if in_`type'_sample==1
					replace tag_`type'`x' = -9999999 if in_good_cell_`type'`x'==0 & in_`type'_sample==1
					tab tag_`type'`x' if in_`type'_sample==1, gen(tag_`type'`x'_)
					
					*Drop var for things with no var (the first one). These observations will not be included
					*in bin-based regression and fully saturated regressios. So, do not include their corresponding
					*variables either
					
					drop tag_`type'`x'_1
					
					*Obs per cell
					
					bys `type'`x': egen obs_cell_`type'`x' = count(maskedid) if in_`type'_sample==1
				}
			}
			
			drop sd_z_*
			
			save "`cleandata'analysis_fc_oc_3cmo_4endog_`file'.dta", replace
		}	/*Close file loop*/	
	}	/*Close pscore condition*/
	


	
	if `pscore_setup_yr2'==1{
		
		foreach file in simulation frequency formula{
	
			use "`cleandata'mdrd_denver_fc_nonfc_year_2_`file' `pscoredate'", clear
			
			
			destring baselinegrade, replace
			destring appgrade, gen(appgrade_tmp)
			drop appgrade
			rename appgrade_tmp appgrade
			
			*High school flag			
				gen hsgrade = ( appgrade>=9)
				gen msgrade = ( appgrade<9)
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
				
			*Analysis at the student level. Generate max for charter offer
			bys maskedid: egen any_study_offer_fc_3cmo = max(offer_study_fc_3cmo)
			bys maskedid: egen any_study_offer_fc_n3cmo = max(offer_study_fc_n3cmo)
			bys maskedid: egen any_study_offer_oc_3cmo = max(offer_study_oc_3cmo)
			bys maskedid: egen any_study_offer_oc_n3cmo = max(offer_study_oc_n3cmo)	
			
			bys maskedid: egen num_study = count(cond(appstudy==1,appschoolnum,.))
	
			bys maskedid: egen num_schools = count(appschoolnum)
			
			*Enrolls at charter flag (student level)
			bys maskedid: egen enroll_any_study_fc_3cmo 	= max(study_fc_3cmo)
			bys maskedid: egen enroll_any_study_fc_n3cmo 	= max(study_fc_n3cmo)
			bys maskedid: egen enroll_any_study_oc_3cmo		= max(study_oc_3cmo)			
			bys maskedid: egen enroll_any_study_oc_n3cmo	= max(study_oc_n3cmo)
			
			*Rank 1 at school level
			gen rank1_study= (appstudy==1 & apprank==1)


			******************************************************************************
			
			
			*First ranked school is charter (student level, for cov balance)
				bys maskedid: egen tempstudy = max(rank1_study)
				drop rank1_study
				ren tempstudy rank1_study

				*Flag - apply to any study
				bys maskedid: egen maxstudy = max(appstudy)
				drop appstudy
				ren maxstudy appstudy
				
				
				foreach type in fc_3cmo fc_n3cmo oc_3cmo oc_n3cmo {
				
				*Flag - first choice is any study
				bys maskedid: egen maxstudy_`type' = max(appstudy_`type')
				drop appstudy_`type'
				ren maxstudy_`type' appstudy_`type'
				}
			
			*Enrolls next year (i.e. has nonmissing schoolnum)
			gen next = (schoolnum!=.)
			bys maskedid: egen enrolls_next = max(next)
			
			*In_sample was previously defined at the student-school level. Redefine it so
			*that it is at the student level
			
			foreach type in fc_3cmo fc_n3cmo oc_3cmo oc_n3cmo {
				bys maskedid: egen in_study`type'_sample_max = max(in_study`type'_sample)
				tab in_study`type'_sample_max in_study`type'_sample
				
				drop in_study`type'_sample
				rename in_study`type'_sample_max in_study`type'_sample
			}
			
			*** Dataset at student level from here on ****
			duplicates drop maskedid, force
			drop next
			
			*Redefine has scores so that it excludes sciences - very few kids
			replace has_sc = (has_sc_math | has_sc_reading | has_sc_writing)
			
			*School-level obs
			drop appschoolnum appschoolname offer offer_study offer_study_fc offer_study_oc offer_study_fc_3cmo offer_study_fc_n3cmo offer_study_oc_3cmo offer_study_oc_n3cmo ///
				appcharter appstudy appstudy_fc appstudy_oc appstudy_fc_3cmo appstudy_fc_n3cmo appstudy_oc_3cmo appstudy_oc_n3cmo apprank ///
				study study_fc study_oc study_fc_3cmo study_fc_n3cmo study_oc_3cmo study_oc_n3cmo charter

			*Keep only if in sample
			keep if in_studyfc_3cmo_sample==1 | in_studyfc_n3cmo_sample == 1 | in_studyoc_3cmo_sample==1 | in_studyoc_n3cmo_sample == 1
			
			*Define r001 as meaning pscore rounded to 0.001 and r01 as rounded to 0.01
			gen fn001 = round(pscore_studyfc_3cmo, 0.001)
			gen fn01 = round(pscore_studyfc_3cmo, 0.01)
			
			gen fe001 = round(pscore_studyfc_n3cmo, 0.001)
			gen fe01 = round(pscore_studyfc_n3cmo, 0.01)
			
			gen on001 = round(pscore_studyoc_3cmo, 0.001)
			gen on01 = round(pscore_studyoc_3cmo, 0.01)

			gen oe001 = round(pscore_studyoc_n3cmo, 0.001)
			gen oe01 = round(pscore_studyoc_n3cmo, 0.01)
			
			tab fn001, gen(fn001_)
			tab fn01, gen(fn01_)			
			tab fe001, gen(fe001_)
			tab fe01, gen(fe01_)
			
			tab on01, gen(on01_)
			tab on001, gen(on001_)
			tab oe01, gen(oe01_)
			tab oe001, gen(oe001_)
			
			*Polynomials
			forvalues i=1/4{
				gen double pscore_fn`i'=pscore_studyfc_3cmo^(`i')	
				gen double pscore_fe`i'=pscore_studyfc_n3cmo^(`i')

				gen double pscore_on`i'=pscore_studyoc_3cmo^(`i')
				gen double pscore_oe`i'=pscore_studyoc_n3cmo^(`i')
			}
			keep if baselinespyear==2013
			
			*Treatment is any study, instrument is study offer, riskset is pscore
			ren any_study_offer_fc_3cmo 	Z_fn
			ren any_study_offer_fc_n3cmo 	Z_fe
			ren any_study_offer_oc_3cmo 	Z_on
			ren any_study_offer_oc_n3cmo 	Z_oe
			
			ren enroll_any_study_fc_3cmo  	D_fn
			ren enroll_any_study_fc_n3cmo  	D_fe
			ren enroll_any_study_oc_3cmo 	D_on
			ren enroll_any_study_oc_n3cmo 	D_oe
			
			**********************************************************************************
			******************** Flag kids who are ineligble and guaranteed ******************
			**********************************************************************************
			*Flag kids who are ineligble and guaranteed - do not drop them, we still want them 
			*for OLS regressions and naive 2SLS (i.e. not controlling for risksets)
			
				*If simulation, first define in_zero1_pscore based on true score, then
			
					if "`file'"=="simulation"{
					
						ren pscore_studyfc_3cmo_simulation sim_pscore_fn
						ren pscore_studyfc_n3cmo_simulation sim_pscore_fe
						ren pscore_studyoc_3cmo_simulation sim_pscore_on
						ren pscore_studyoc_n3cmo_simulation sim_pscore_oe
						
						gen in_zero1_fnpscore = (sim_pscore_fn > 0 & sim_pscore_fn < 1)
						gen in_zero1_fepscore = (sim_pscore_fe > 0 & sim_pscore_fe < 1)
						gen in_zero1_onpscore = (sim_pscore_on > 0 & sim_pscore_on < 1)
						gen in_zero1_oepscore = (sim_pscore_oe > 0 & sim_pscore_oe < 1)

						gen double pscore_studyfc_3cmo_simulation = round(sim_pscore_fn, 0.0001)
						gen double pscore_studyfc_n3cmo_simulation = round(sim_pscore_fe, 0.0001)
						gen double pscore_studyoc_3cmo_simulation = round(sim_pscore_on, 0.0001)
						gen double pscore_studyoc_n3cmo_simulation = round(sim_pscore_oe, 0.0001)
						
						tab pscore_studyfc_3cmo_`file', gen(fn0_)
						tab pscore_studyoc_3cmo_`file', gen(on0_)
						tab pscore_studyfc_n3cmo_`file', gen(fe0_)
						tab pscore_studyoc_n3cmo_`file', gen(oe0_)
						
						ren pscore_studyfc_3cmo_`file' fn0
						ren pscore_studyfc_n3cmo_`file' fe0
						ren pscore_studyoc_3cmo_`file' on0					
						ren pscore_studyoc_n3cmo_`file' oe0	
						}
						
					else{
							
						tab pscore_studyfc_3cmo_`file', gen(fn0_)
						tab pscore_studyoc_3cmo_`file', gen(on0_)
						
						tab pscore_studyfc_n3cmo_`file', gen(fe0_)
						tab pscore_studyoc_n3cmo_`file', gen(oe0_)
						
						ren pscore_studyfc_3cmo_`file' fn0
						ren pscore_studyoc_3cmo_`file' on0
						
						ren pscore_studyfc_n3cmo_`file' fe0
						ren pscore_studyoc_n3cmo_`file' oe0
						
						gen in_zero1_fnpscore = (fn0 > 0 & fn0 < 1)
						gen in_zero1_onpscore = (on0 > 0 & on0 < 1)
						gen in_zero1_fepscore = (fe0 > 0 & fe0 < 1)
						gen in_zero1_oepscore = (oe0 > 0 & oe0 < 1)
						}
					
			**********************************************************************************
			********** Flag cells with variation in instrument and characteristics************
			**********************************************************************************
					
			/* 	Flag observations that actually make it into balance (i.e. obs in cells with
				variation in instrument). These flags are used for all types 
				of F-tests, conducted below.
				
				We generate these flags here so that no re-sorting of the data (done in bys x egen:)
				is done between the balanceregs and F tests, which affects how the variable
				"var", which stores the names of the variables corresponding to each balance
				coefficient, appears in the final dataset once we svmat the balance output.
			*/
			
			ren in_studyfc_3cmo_sample in_fn_sample
			ren in_studyfc_n3cmo_sample in_fe_sample
			ren in_studyoc_3cmo_sample in_on_sample
			ren in_studyoc_n3cmo_sample in_oe_sample
			
			tab in_fn_sample, m
			tab in_on_sample, m
			tab in_fe_sample, m
			tab in_oe_sample, m
			
			foreach type in fn on fe oe{
				foreach x in 01 001 0{
					
					*Variation in instruments for each cell
						bys `type'`x': egen sd_z_`type'`x'	= 	sd(Z_`type')
					
					*Good cell
						gen in_good_cell_`type'`x' = (sd_z_`type'`x'>0 & !missing(sd_z_`type'`x') & in_zero1_`type'pscore==1) if in_`type'_sample==1
					
					gen double tag_`type'`x' = `type'`x' if in_`type'_sample==1
					replace tag_`type'`x' = -9999999 if in_good_cell_`type'`x'==0 & in_`type'_sample==1
					tab tag_`type'`x' if in_`type'_sample==1, gen(tag_`type'`x'_)
					
					*Drop var for things with no var (the first one). These observations will not be included
					*in bin-based regression and fully saturated regressios. So, do not include their corresponding
					*variables either
					
					drop tag_`type'`x'_1
					
					*Obs per cell
					
					bys `type'`x': egen obs_cell_`type'`x' = count(maskedid) if in_`type'_sample==1
				}
			}
			
			drop sd_z_*
			
			save "`cleandata'analysis_fc_oc_3cmo_4endog_year_2_`file'.dta", replace
		}	/*Close file loop*/	
	}	/*Close pscore condition*/	
	
	
	if `pscore_stacking' == 1{
		
		foreach file in simulation frequency formula {
		
			u "`cleandata'analysis_fc_oc_3cmo_4endog_`file'.dta", clear
			
			append using "`cleandata'analysis_fc_oc_3cmo_4endog_year_2_`file'.dta"
			
			drop fn001_* fn01_* fn0_* fe001_* fe01_* fe0_* on001_* on01_* on0_* oe001_* oe01_* oe0_* in_good_cell_* tag_* obs_cell_*
			
			tab fn001, gen(fn001_)
			tab fn01, gen(fn01_)			
			tab fe001, gen(fe001_)
			tab fe01, gen(fe01_)
			
			tab on01, gen(on01_)
			tab on001, gen(on001_)
			tab oe01, gen(oe01_)
			tab oe001, gen(oe001_)
			
			tab fn0, gen(fn0_)
			tab fe0, gen(fe0_)
			tab on0, gen(on0_)
			tab oe0, gen(oe0_)
			
			* generating indicator to use factor variables in 2sls regs (much faster)
			
			foreach risk in 0 01 001 {
			
				egen ind_fe`risk' = group(fe`risk')
				egen ind_fn`risk' = group(fn`risk')
				egen ind_oe`risk' = group(oe`risk')
				egen ind_on`risk' = group(on`risk')
				
			
			}
		
			*dropping duplicates from year 2.
			
			duplicates tag masked, gen(dup_masked)
			drop if dup_masked > 0 & baselinespyear == 2013		
	
				foreach type in fn on fe oe{
				foreach x in 01 001 0{
					
					*Variation in instruments for each cell
						bys `type'`x': egen sd_z_`type'`x'	= 	sd(Z_`type')
					
					*Good cell
						gen in_good_cell_`type'`x' = (sd_z_`type'`x'>0 & !missing(sd_z_`type'`x') & in_zero1_`type'pscore==1) if in_`type'_sample==1
					
					gen double tag_`type'`x' = `type'`x' if in_`type'_sample==1
					replace tag_`type'`x' = -9999999 if in_good_cell_`type'`x'==0 & in_`type'_sample==1
					tab tag_`type'`x' if in_`type'_sample==1, gen(tag_`type'`x'_)
					
					*Drop var for things with no var (the first one). These observations will not be included
					*in bin-based regression and fully saturated regressios. So, do not include their corresponding
					*variables either
					
					drop tag_`type'`x'_1
					
					*Obs per cell
					
					bys `type'`x': egen obs_cell_`type'`x' = count(maskedid) if in_`type'_sample==1
					}
				}
			
			drop sd_z_*
		
					
		
			save "`cleandata'analysis_fc_oc_3cmo_4endog_stacked_`file'.dta", replace
		}
	}
		

					
					


	*****************************************************
	******************* Separate 2SLS *******************
	*****************************************************
	
	if `pscore_separate_twosls'==1{
		
		foreach file in frequency simulation formula {
				foreach type in fn fe on oe{
								
					if "`type'"=="fn"{
						local colbump = 0
					}
					
					if "`type'"=="fe"{
						local colbump = 19
					}
					else if "`type'"=="on"{
						local colbump = 38
					}
					else if "`type'"=="oe"{
						local colbump = 57
					}
					
					local row = 1
					local col = 1 + `colbump'
				
					u "`cleandata'analysis_fc_oc_3cmo_4endog_stacked_`file'.dta", clear
					
					keep if in_fn_sample == 1 | in_on_sample == 1 | in_fe_sample == 1 | in_oe_sample == 1
				
					gen var =""
					
					*By grade vars
					gen group = .
				
					foreach group in 410 /*45 68 48 910*/{
								if `group'==410{
									replace group=410
								}
								else if `group'==48{
									replace group=48 if inlist(appgrade,4,5,6,7,8)
								}
								else{
								replace group=45 if inlist(appgrade,4,5)
								replace group=68 if inlist(appgrade,6,7,8)
								replace group=910 if inlist(appgrade,9,10)
								}
		
								foreach t in math writing reading{
									foreach out in ss /*prof_up adv*/{
							
									
									replace var="`out'_`t'_`group'" if _n==`row'
									replace var="`out'_`t'_`group'_se" if _n==`row'+1
									replace var="`out'_`t'_`group'_N" if _n==`row'+2
									
				
									qui summ `out'_`t' if group==`group' & in_zero1_`type'pscore==1
									
									if ( r(max)==0 & r(min)==0 ) | r(N)<50{ 
										di in red "No observations for `group'-level 2SLS estimates"
									}
								
									else{
								
										qui summ `out'_`t' if Z_`type' == 0 & group==`group'
										matrix twosls[`row',`col'] = r(mean)
										matrix twosls[`row'+2,`col'] = r(N)
										
											/*OLS - no control for pscore*/
											qui reg `out'_`t' D_`type' `cov' `cov2' `cov_grade' baselinespyear if group==`group' & in_`type'_sample==1, vce(cluster maskedid)
											matrix twosls[`row',`col'+1]   = _b[D]
											matrix twosls[`row'+1,`col'+1] = _se[D]
											matrix twosls[`row'+2,`col'+1] = e(N)
											
											/*OLS conditional on fully saturated pscore sample*/
											qui reg `out'_`t' D_`type' `cov' `cov2' `cov_grade' baselinespyear if group==`group' & in_good_cell_`type'0==1, vce(cluster maskedid)
											matrix twosls[`row',`col'+2]   = _b[D]
											matrix twosls[`row'+1,`col'+2] = _se[D]
											matrix twosls[`row'+2,`col'+2] = e(N)
											
											/*OLS conditional controlling for fully saturated pscores*/
											qui reg `out'_`t' D_`type' `type'0_* `cov' `cov2'  `cov_grade' baselinespyear if group==`group' & in_good_cell_`type'0==1, vce(cluster maskedid)

											matrix twosls[`row',`col'+3]   = _b[D]
											matrix twosls[`row'+1,`col'+3] = _se[D]
											matrix twosls[`row'+2,`col'+3] = e(N)
									
									foreach instrument_type in ordinary /*over_id*/  {
										
										if "`instrument_type'" == "ordinary" {
											local instrument "Z_`type'"
										}
										else {
											local instrument "Z_`type' Z_`type'_first"
										}
										
										foreach risk in `type'01 `type'001 `type'0{
							
											local col = `col' + 2
											
											local controls "`risk'_* `cov_`type'' `cov2'  `cov_grade'"		
											 
												
											/*2SLS - outcome on treatment instrumented by instrument*/
											ivreg2 `out'_`t' (D_`type' = `instrument') `controls' baselinespyear if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls' baselinespyear) cluster(maskedid)

												*2SLS
												matrix twosls[`row',`col'+4]   = _b[D]
												matrix twosls[`row'+1,`col'+4] = _se[D]
												matrix twosls[`row'+2,`col'+4] = e(N)
												
												estimates restore firstD_`type'
												
												*First stage
												matrix twosls[`row',`col'+3]   = _b[Z_`type']
												matrix twosls[`row'+1,`col'+3] = _se[Z_`type']
												
											*If last loop, do these
											if "`risk'"=="`type'0"{
												
												*NAIVE 2SLS - no riskset controls
													*2SLS naive
													ivreg2 `out'_`t' (D_`type' = `instrument') `cov_`type'' `cov2' `cov_grade' baselinespyear if group==`group', savefirst savefprefix(first) partial(`cov_`type'' `cov2' `cov_grade' baselinespyear) cluster(maskedid)

														*2SLS
														matrix twosls[`row',`col'+6]   = _b[D]
														matrix twosls[`row'+1,`col'+6] = _se[D]
														matrix twosls[`row'+2,`col'+6] = e(N)
														
														estimates restore firstD_`type'
												
														*First stage
														matrix twosls[`row',`col'+5]   = _b[Z_`type']
														matrix twosls[`row'+1,`col'+5] = _se[Z_`type']
														
													*2SLS naive conditional on r0 sample
													ivreg2 `out'_`t' (D_`type' = `instrument') `cov_`type'' `cov2' `cov_grade' baselinespyear if group==`group' & in_good_cell_`type'0==1, savefirst savefprefix(first) partial(`cov_`type'' `cov2' `cov_grade' baselinespyear) cluster(maskedid)

														*2SLS
														matrix twosls[`row',`col'+8]   = _b[D]
														matrix twosls[`row'+1,`col'+8] = _se[D]
														matrix twosls[`row'+2,`col'+8] = e(N)
														
														estimates restore firstD_`type'
												
														*First stage
														matrix twosls[`row',`col'+7]   = _b[Z_`type']
														matrix twosls[`row'+1,`col'+7] = _se[Z_`type']													
												
												*2SLS with no cov controls								
													*2SLS
													ivreg2 `out'_`t' (D_`type' = `instrument') `risk'_* baselinespyear if group==`group' & in_good_cell_`type'0==1, savefirst savefprefix(first) partial(`risk'_* baselinespyear) cluster(maskedid) 
														
														*2SLS
														matrix twosls[`row',`col'+10]   = _b[D]
														matrix twosls[`row'+1,`col'+10] = _se[D]
														matrix twosls[`row'+2,`col'+10] = e(N)
														
														estimates restore firstD_`type'
												
														*First stage
														matrix twosls[`row',`col'+9]   = _b[Z_`type']
														matrix twosls[`row'+1,`col'+9] = _se[Z_`type']
											} /*close if condition*/
										}  /*risk loop*/
										local col = `col' + 9
										} /* instrument type loop */
									
								} /*close condition on number of observations*/
								
								local row = `row' + 3
								local col = 1 + `colbump'
							} 	 /*out loop*/
							
							local ++row
						}	/* t loop*/
						
					}	/*group loop*/
				} /*f o loop*/
			
				preserve
					svmat twosls
					keep var twosls*
					
					keep if _n<=`row'
					
					ren twosls1 z0_fn
					ren twosls2 OLS_fn
					ren twosls3 OLS_psample_fn
					ren twosls4 OLS_psat_fn
					
					ren twosls6 FS_r01_fn
					ren twosls7 twoSLS_r01_fn
					ren twosls8 FS_r001_fn
					ren twosls9 twoSLS_r001_fn
					ren twosls10 FS_psat_fn
					ren twosls11 twoSLS_psat_fn
					ren twosls12 FS_naive_fn
					ren twosls13 twoSLS_naive_fn
					ren twosls14 FS_naive_psample_fn
					ren twosls15 twoSLS_naive_psample_fn
					ren twosls16 FS_nocovs_fn
					ren twosls17 twoSLS_nocovs_fn
					
					/*
					ren twosls21 FS_r01_n_overid
					ren twosls22 twoSLS_r01_n_overid
					ren twosls23 FS_r001_n_overid
					ren twosls24 twoSLS_r001_n_overid
					ren twosls25 FS_psat_n_overid
					ren twosls26 twoSLS_psat_n_overid
					ren twosls27 FS_naive_n_overid
					ren twosls28 twoSLS_naive_n_overid
					ren twosls29 FS_naive_psample_n_overid
					ren twosls30 twoSLS_naive_psample_n_overid
					ren twosls31 FS_nocovs_n_overid
					ren twosls32 twoSLS_nocovs_n_overid
					*/
					
					ren twosls20 z0_fe
					ren twosls21 OLS_fe
					ren twosls22 OLS_psample_fe
					ren twosls23 OLS_psat_fe
					
					ren twosls25 FS_r01_fe
					ren twosls26 twoSLS_r01_fe
					ren twosls27 FS_r001_fe
					ren twosls28 twoSLS_r001_fe					
					ren twosls29 FS_psat_fe
					ren twosls30 twoSLS_psat_fe
					ren twosls31 FS_naive_fe
					ren twosls32 twoSLS_naive_fe
					ren twosls33 FS_naive_psample_fe
					ren twosls34 twoSLS_naive_psample_fe
					ren twosls35 FS_nocovs_fe
					ren twosls36 twoSLS_nocovs_fe

					ren twosls39 z0_on
					ren twosls40 OLS_on
					ren twosls41 OLS_psample_on
					ren twosls42 OLS_psat_on
					
					ren twosls44 FS_r01_on
					ren twosls45 twoSLS_r01_on
					ren twosls46 FS_r001_on
					ren twosls47 twoSLS_r001_on					
					ren twosls48 FS_psat_on
					ren twosls49 twoSLS_psat_on
					ren twosls50 FS_naive_on
					ren twosls51 twoSLS_naive_on
					ren twosls52 FS_naive_psample_on
					ren twosls53 twoSLS_naive_psample_on
					ren twosls54 FS_nocovs_on
					ren twosls55 twoSLS_nocovs_on
					
					ren twosls58 z0_oe
					ren twosls59 OLS_oe
					ren twosls60 OLS_psample_oe
					ren twosls61 OLS_psat_oe
					
					ren twosls63 FS_r01_oe
					ren twosls64 twoSLS_r01_oe
					ren twosls65 FS_r001_oe
					ren twosls66 twoSLS_r001_oe					
					ren twosls67 FS_psat_oe
					ren twosls68 twoSLS_psat_oe
					ren twosls69 FS_naive_oe
					ren twosls70 twoSLS_naive_oe
					ren twosls71 FS_naive_psample_oe
					ren twosls72 twoSLS_naive_psample_oe
					ren twosls73 FS_nocovs_oe
					ren twosls74 twoSLS_nocovs_oe
					
					outsheet using "`tables'inDPSbase_fc_oc_3cmo_4endog_`file'_twosls.csv", comma replace
				restore
			clear	

			} /*Close 2SLS condition*/
	} /*Close file loop*/
	
	*****************************************************
	******************** Joint 2SLS *********************
	*****************************************************
	
	if `pscore_joint_twosls'==1{
		
		foreach file in frequency /*simulation  formula*/ {
		
				u "`cleandata'analysis_fc_oc_3cmo_4endog_stacked_`file'.dta", clear
				
				
				gen in_joint_sample = (in_fn_sample==1 | in_fe_sample == 1 | in_on_sample==1 | in_oe_sample == 1)
				keep if in_joint_sample
				
				
				********************************************************************************************
				**************** Make propensity score replacements and regenerate dummies *****************
				********************************************************************************************
				*Drop old bucket variables
				
					drop fn0_* fn01_* fn001_* fe0_* fe01_* fe001_* on0_* on01_* on001_* oe0_* oe01_* oe001_* in_good_* tag_* obs_cell* pscore_* ind_*

				*other-charter or first-choice*non 3CMO applicants that do not apply to first-choice*3CMO charters. Replace missing score with zero
					*Score
					replace fn0 = 0 if ((in_fe_sample==1 | in_on_sample == 1 | in_oe_sample == 1) & in_fn_sample==0) & fn0==.
					
					*First rounded to hundredth bucket
					replace fn01 = 0 if ((in_fe_sample==1 | in_on_sample == 1 | in_oe_sample == 1) & in_fn_sample==0) & fn0==0
					
					*First rounded to thousandth bucket
					replace fn001 = 0 if ((in_fe_sample==1 | in_on_sample == 1 | in_oe_sample == 1) & in_fn_sample==0) & fn0==0


				*other-charter or first-choice* 3CMO applicants that do not apply to first-choice* non 3CMO charters. Replace missing score with zero
					*Score
					replace fe0 = 0 if ((in_fn_sample==1 | in_on_sample == 1 | in_oe_sample == 1) & in_fe_sample==0) & fe0==.
					
					*First rounded to hundredth bucket
					replace fe01 = 0 if ((in_fn_sample==1 | in_on_sample == 1 | in_oe_sample == 1) & in_fe_sample==0) & fe0==0
					
					*First rounded to thousandth bucket
					replace fe001 = 0 if ((in_fn_sample==1 | in_on_sample == 1 | in_oe_sample == 1) & in_fe_sample==0) & fe0==0
									
				
				*First choice-charter or other-choice*non 3CMO applicants that do not apply to other-choice*3CMO charters. Replace missing score with zero
					*Score
					replace on0 = 0 if ((in_fe_sample==1 | in_fn_sample == 1 | in_oe_sample == 1) & in_on_sample==0) & on0==.
					
					*First rounded to hundredth bucket
					replace on01 = 0 if ((in_fe_sample==1 | in_fn_sample == 1 | in_oe_sample == 1) & in_on_sample==0) & on0==0
					
					*First rounded to thousandth bucket
					replace on001 = 0 if ((in_fe_sample==1 | in_fn_sample == 1 | in_oe_sample == 1) & in_on_sample==0) & on0==0


				*First choice-charter or other-choice*3CMO applicants that do not apply to other-choice* non 3CMO charters. Replace missing score with zero
					*Score
					replace oe0 = 0 if ((in_fn_sample==1 | in_on_sample == 1 | in_fe_sample == 1) & in_oe_sample==0) & oe0==.
					
					*First rounded to hundredth bucket
					replace oe01 = 0 if ((in_fn_sample==1 | in_on_sample == 1 | in_fe_sample == 1) & in_oe_sample==0) & oe0==0
					
					*First rounded to thousandth bucket
					replace oe001 = 0 if ((in_fn_sample==1 | in_on_sample == 1 | in_fe_sample == 1) & in_oe_sample==0) & oe0==0
						
					
				*In (0,1) for either charter or innovation
					
					gen in_zero1_joint =  ( (fn0 > 0 & fn0 < 1 ) | (on0 > 0 & on0 < 1 ) | (fe0 > 0 & fe0 < 1 ) | (oe0 > 0 & oe0 < 1 )  )
				
				*Joint propensity score
					
					egen fone0 = group(fn0 fe0 on0 oe0) if in_zero1_joint
					egen fone01 = group(fn01 fe01 on01 oe01) if in_zero1_joint
					egen fone001 = group(fn001 fe001 on001 oe001) if in_zero1_joint
					
				******************************************************************
				************ Replace missing treatment and instrument ************
				******************************************************************
				
				foreach x in Z D{
					replace `x'_fn = 0 if `x'_fn ==.
					replace `x'_fe = 0 if `x'_fe ==.
					replace `x'_on = 0 if `x'_on ==.
					replace `x'_oe = 0 if `x'_oe ==.
				}
				
				*In cell with variation?
					
					foreach x in 01 001 0{
						
						*Variation in instruments for each cell
							bys fn`x': egen sd_z_fn`x'	= 	sd(Z_fn)
							bys fe`x': egen sd_z_fe`x'	= 	sd(Z_fe)
							bys on`x': egen sd_z_on`x'	= 	sd(Z_on)
							bys oe`x': egen sd_z_oe`x'	= 	sd(Z_oe)
						
						*Good cell
							gen in_good_cell_`x' = ( in_zero1_joint==1	& ///
								 ( 	(sd_z_fn`x'>0 & !missing(sd_z_fn`x')) | (sd_z_on`x'>0 & !missing(sd_z_on`x')) | ///
								 (sd_z_fe`x'>0 & !missing(sd_z_fe`x')) | (sd_z_oe`x'>0 & !missing(sd_z_oe`x')) ) )
						
					}
					drop sd_z_*
					
					foreach x in 01 001 0{
						bys fone`x': egen sd_z_fn	= 	sd(Z_fn)
						bys fone`x': egen sd_z_on	= 	sd(Z_on)
						bys fone`x': egen sd_z_fe	= 	sd(Z_fe)
						bys fone`x': egen sd_z_oe	= 	sd(Z_oe)
						
						gen in_good_cell_fone`x' = ( in_zero1_joint==1	& ///
										(	( sd_z_fn>0 & !missing(sd_z_fn) ) | ( sd_z_on>0 & !missing(sd_z_on) ) | ///
										( sd_z_fe>0 & !missing(sd_z_fe) ) | ( sd_z_oe>0 & !missing(sd_z_oe) ) ) )
						
						tab fone`x' if in_good_cell_fone`x' ==1
						
						drop sd_z_*
					}

				* regenerating indicator to use factor variables in 2sls regs (much faster)
				
				foreach risk in 0 01 001 {
				
					egen ind_fe`risk' = group(fe`risk')
					egen ind_fn`risk' = group(fn`risk')
					egen ind_oe`risk' = group(oe`risk')
					egen ind_on`risk' = group(on`risk')
					
					* not necessary to do this, but we can keep consistent variable names this way. 
					egen ind_fone`risk' = group(ind_fe`risk' ind_fn`risk' ind_oe`risk' ind_on`risk' ) if in_zero1_joint
	
				
				}
						
					
				*Regenerate buckets
				
				foreach x in fn0 fn001 fn01  fe0 fe001 fe01 on0 on001 on01 oe0 oe001 oe01 fone0 fone001 fone01{
					tab `x', gen(`x'_)
				}
				
				local col = 1
				local row = 1
				gen var =""
				
				
				*By grade vars
				gen group = .
			
				count if in_zero1_joint==1 & inlist(appgrade,4,5,6,7,8)
				count if in_good_cell_0==1 & inlist(appgrade,4,5,6,7,8)
				
				foreach group in 410 /*45 68 48 910*/{
							if `group'==410{
								replace group=410
							}
							else if `group'==48{
								replace group=48 if inlist(appgrade,4,5,6,7,8)
							}
							else{
							replace group=45 if inlist(appgrade,4,5)
							replace group=68 if inlist(appgrade,6,7,8)
							replace group=910 if inlist(appgrade,9,10)
							}
	
							foreach t in math writing reading{
								foreach out in ss /*prof_up adv*/{
								
								local col = 1
								
								replace var="`out'_`t'_`group'" if _n==`row'
								replace var="`out'_`t'_`group'_se" if _n==`row'+1
								replace var="`out'_`t'_`group'_N" if _n==`row'+2
								
			
								qui summ `out'_`t' if group==`group'
								
								if ( r(max)==0 & r(min)==0 ) | r(N)<50{ 
									di in red "Not enough observations for `group'-level 2SLS estimates"
								}
							
								else{
							
									qui summ `out'_`t' if Z_fn == 0 & in_fn_sample & group==`group'
									matrix jointtwosls[`row',`col'] = r(mean)
									matrix jointtwosls[`row'+2,`col'] = r(N)
									
									qui summ `out'_`t' if Z_fe == 0 & in_fe_sample & group==`group'
									matrix jointtwosls[`row',`col'+1] = r(mean)
									matrix jointtwosls[`row'+2,`col'+1] = r(N)
									
									qui summ `out'_`t' if Z_on == 0 & in_on_sample & group==`group'
									matrix jointtwosls[`row',`col'+2] = r(mean)
									matrix jointtwosls[`row'+2,`col'+2] = r(N)
									
									qui summ `out'_`t' if Z_oe == 0 & in_oe_sample & group==`group'
									matrix jointtwosls[`row',`col'+3] = r(mean)
									matrix jointtwosls[`row'+2,`col'+3] = r(N)
									
									local col = `col' + 3
									
										/*OLS - no control for pscore*/
										qui reg `out'_`t' D_fn D_fe D_on D_oe `cov_either' `cov_grade' baselinespyear if group==`group', vce(cluster maskedid)
										matrix jointtwosls[`row',`col'+1]   = _b[D_fn]
										matrix jointtwosls[`row'+1,`col'+1] = _se[D_fn]
										matrix jointtwosls[`row'+2,`col'+1] = e(N)
										
										matrix jointtwosls[`row',`col'+2]   = _b[D_fe]
										matrix jointtwosls[`row'+1,`col'+2] = _se[D_fe]
										matrix jointtwosls[`row'+2,`col'+2] = e(N)

										matrix jointtwosls[`row',`col'+3]   = _b[D_on]
										matrix jointtwosls[`row'+1,`col'+3] = _se[D_on]
										matrix jointtwosls[`row'+2,`col'+3] = e(N)
										
										matrix jointtwosls[`row',`col'+4]   = _b[D_oe]
										matrix jointtwosls[`row'+1,`col'+4] = _se[D_oe]
										matrix jointtwosls[`row'+2,`col'+4] = e(N)
										
										/*OLS conditional on fully saturated pscore sample*/
										qui reg `out'_`t' D_fn D_fe D_on D_oe `cov_either' `cov_grade' baselinespyear if group==`group' & in_zero1_joint==1, vce(cluster maskedid)
										matrix jointtwosls[`row',`col'+5]   = _b[D_fn]
										matrix jointtwosls[`row'+1,`col'+5] = _se[D_fn]
										matrix jointtwosls[`row'+2,`col'+5] = e(N)
										
										matrix jointtwosls[`row',`col'+6]   = _b[D_fe]
										matrix jointtwosls[`row'+1,`col'+6] = _se[D_fe]
										matrix jointtwosls[`row'+2,`col'+6] = e(N)

										matrix jointtwosls[`row',`col'+7]   = _b[D_on]
										matrix jointtwosls[`row'+1,`col'+7] = _se[D_on]
										matrix jointtwosls[`row'+2,`col'+7] = e(N)
										
										matrix jointtwosls[`row',`col'+8]   = _b[D_oe]
										matrix jointtwosls[`row'+1,`col'+8] = _se[D_oe]
										matrix jointtwosls[`row'+2,`col'+8] = e(N)

										/*OLS conditional controlling for fully saturated pscores*/
										qui reg `out'_`t' D_fn D_fe D_on D_oe i.ind_fn0 i.ind_fe0 i.ind_on0 i.ind_oe0 baselinespyear if group==`group' & in_zero1_joint==1, vce(cluster maskedid)
										matrix jointtwosls[`row',`col'+9]   = _b[D_fn]
										matrix jointtwosls[`row'+1,`col'+9] = _se[D_fn]
										matrix jointtwosls[`row'+2,`col'+9] = e(N)
										
										matrix jointtwosls[`row',`col'+10]   = _b[D_fe]
										matrix jointtwosls[`row'+1,`col'+10] = _se[D_fe]
										matrix jointtwosls[`row'+2,`col'+10] = e(N)

										matrix jointtwosls[`row',`col'+11]   = _b[D_on]
										matrix jointtwosls[`row'+1,`col'+11] = _se[D_on]
										matrix jointtwosls[`row'+2,`col'+11] = e(N)
										
										matrix jointtwosls[`row',`col'+12]   = _b[D_oe]
										matrix jointtwosls[`row'+1,`col'+12] = _se[D_oe]
										matrix jointtwosls[`row'+2,`col'+12] = e(N)

										local col = `col' + 12
										
										
									foreach instrument_type in ordinary /*over_id*/  {
										
										if "`instrument_type'" == "ordinary" {
											local instrument "Z_fn Z_fe Z_on Z_oe"
										}
										else {
											local instrument "Z_n Z_n_first Z_e Z_e_first Z_i Z_i_first"
										}
										
										*Joint 2SLS conditioning separately for risksets
										foreach x in 01 001 0{

											ivreg2 `out'_`t' (D_fn D_fe D_on D_oe = `instrument') i.ind_fn`x' i.ind_fe`x' i.ind_on`x' i.ind_oe`x' `cov_either' `cov2' `cov_grade' baselinespyear if group==`group' ///
													& in_good_cell_`x'==1, savefirst savefprefix(first) partial(i.ind_fn`x' i.ind_fe`x' i.ind_on`x' i.ind_oe`x' `cov_either' `cov2' `cov_grade' baselinespyear) cluster(maskedid) 
												
			
											*2SLS 											
											matrix jointtwosls[`row',`col'+1]   = _b[D_fn]
											matrix jointtwosls[`row'+1,`col'+1] = _se[D_fn]
											matrix jointtwosls[`row'+2,`col'+1] = e(N)
											
											matrix jointtwosls[`row',`col'+2]   = _b[D_fe]
											matrix jointtwosls[`row'+1,`col'+2] = _se[D_fe]
											matrix jointtwosls[`row'+2,`col'+2] = e(N)

											matrix jointtwosls[`row',`col'+3]   = _b[D_on]
											matrix jointtwosls[`row'+1,`col'+3] = _se[D_on]
											matrix jointtwosls[`row'+2,`col'+3] = e(N)
											
											matrix jointtwosls[`row',`col'+4]   = _b[D_oe]
											matrix jointtwosls[`row'+1,`col'+4] = _se[D_oe]
											matrix jointtwosls[`row'+2,`col'+4] = e(N)
											
											
											*First stage first-choice*3CMO charter
											estimates restore firstD_fn
											matrix jointtwosls[`row',`col'+5]   = _b[Z_fn]
											matrix jointtwosls[`row'+1,`col'+5] = _se[Z_fn]
											matrix jointtwosls[`row',`col'+6]   = _b[Z_fe]
											matrix jointtwosls[`row'+1,`col'+6] = _se[Z_fe]
											matrix jointtwosls[`row',`col'+7]   = _b[Z_on]
											matrix jointtwosls[`row'+1,`col'+7] = _se[Z_on]
											matrix jointtwosls[`row',`col'+8]   = _b[Z_oe]
											matrix jointtwosls[`row'+1,`col'+8] = _se[Z_oe]
											
											*First stage first-choice*non 3CMO charter
											estimates restore firstD_fe
											matrix jointtwosls[`row',`col'+9]   = _b[Z_fn]
											matrix jointtwosls[`row'+1,`col'+9] = _se[Z_fn]
											matrix jointtwosls[`row',`col'+10]   = _b[Z_fe]
											matrix jointtwosls[`row'+1,`col'+10] = _se[Z_fe]
											matrix jointtwosls[`row',`col'+11]   = _b[Z_on]
											matrix jointtwosls[`row'+1,`col'+11] = _se[Z_on]
											matrix jointtwosls[`row',`col'+12]   = _b[Z_oe]
											matrix jointtwosls[`row'+1,`col'+12] = _se[Z_oe]

											*First stage first-choice*3CMO charter
											estimates restore firstD_on
											matrix jointtwosls[`row',`col'+13]   = _b[Z_fn]
											matrix jointtwosls[`row'+1,`col'+13] = _se[Z_fn]
											matrix jointtwosls[`row',`col'+14]   = _b[Z_fe]
											matrix jointtwosls[`row'+1,`col'+14] = _se[Z_fe]
											matrix jointtwosls[`row',`col'+15]   = _b[Z_on]
											matrix jointtwosls[`row'+1,`col'+15] = _se[Z_on]
											matrix jointtwosls[`row',`col'+16]   = _b[Z_oe]
											matrix jointtwosls[`row'+1,`col'+16] = _se[Z_oe]
											
											*First stage first-choice*non 3CMO charter
											estimates restore firstD_oe
											matrix jointtwosls[`row',`col'+17]   = _b[Z_fn]
											matrix jointtwosls[`row'+1,`col'+17] = _se[Z_fn]
											matrix jointtwosls[`row',`col'+18]   = _b[Z_fe]
											matrix jointtwosls[`row'+1,`col'+18] = _se[Z_fe]
											matrix jointtwosls[`row',`col'+19]   = _b[Z_on]
											matrix jointtwosls[`row'+1,`col'+19] = _se[Z_on]
											matrix jointtwosls[`row',`col'+20]   = _b[Z_oe]
											matrix jointtwosls[`row'+1,`col'+20] = _se[Z_oe]											
											local col = `col' + 20
										}
											
										*Joint 2SLS conditioning for joint risksets
										foreach x in 01 001 0{
											ivreg2 `out'_`t' (D_fn D_fe D_on D_oe = `instrument') i.ind_fone`x' `cov_either' `cov2' `cov_grade' baselinespyear if group==`group' ///
													& in_good_cell_fone`x'==1, savefirst savefprefix(first) partial(i.ind_fone`x' `cov_either' `cov2' `cov_grade' baselinespyear) cluster(maskedid)  

	
											*2SLS 											
											matrix jointtwosls[`row',`col'+1]   = _b[D_fn]
											matrix jointtwosls[`row'+1,`col'+1] = _se[D_fn]
											matrix jointtwosls[`row'+2,`col'+1] = e(N)
											
											matrix jointtwosls[`row',`col'+2]   = _b[D_fe]
											matrix jointtwosls[`row'+1,`col'+2] = _se[D_fe]
											matrix jointtwosls[`row'+2,`col'+2] = e(N)

											matrix jointtwosls[`row',`col'+3]   = _b[D_on]
											matrix jointtwosls[`row'+1,`col'+3] = _se[D_on]
											matrix jointtwosls[`row'+2,`col'+3] = e(N)
											
											matrix jointtwosls[`row',`col'+4]   = _b[D_oe]
											matrix jointtwosls[`row'+1,`col'+4] = _se[D_oe]
											matrix jointtwosls[`row'+2,`col'+4] = e(N)
											
											
											*First stage first-choice*3CMO charter
											estimates restore firstD_fn
											matrix jointtwosls[`row',`col'+5]   = _b[Z_fn]
											matrix jointtwosls[`row'+1,`col'+5] = _se[Z_fn]
											matrix jointtwosls[`row',`col'+6]   = _b[Z_fe]
											matrix jointtwosls[`row'+1,`col'+6] = _se[Z_fe]
											matrix jointtwosls[`row',`col'+7]   = _b[Z_on]
											matrix jointtwosls[`row'+1,`col'+7] = _se[Z_on]
											matrix jointtwosls[`row',`col'+8]   = _b[Z_oe]
											matrix jointtwosls[`row'+1,`col'+8] = _se[Z_oe]
											
											*First stage first-choice*non 3CMO charter
											estimates restore firstD_fe
											matrix jointtwosls[`row',`col'+9]   = _b[Z_fn]
											matrix jointtwosls[`row'+1,`col'+9] = _se[Z_fn]
											matrix jointtwosls[`row',`col'+10]   = _b[Z_fe]
											matrix jointtwosls[`row'+1,`col'+10] = _se[Z_fe]
											matrix jointtwosls[`row',`col'+11]   = _b[Z_on]
											matrix jointtwosls[`row'+1,`col'+11] = _se[Z_on]
											matrix jointtwosls[`row',`col'+12]   = _b[Z_oe]
											matrix jointtwosls[`row'+1,`col'+12] = _se[Z_oe]

											*First stage first-choice*3CMO charter
											estimates restore firstD_on
											matrix jointtwosls[`row',`col'+13]   = _b[Z_fn]
											matrix jointtwosls[`row'+1,`col'+13] = _se[Z_fn]
											matrix jointtwosls[`row',`col'+14]   = _b[Z_fe]
											matrix jointtwosls[`row'+1,`col'+14] = _se[Z_fe]
											matrix jointtwosls[`row',`col'+15]   = _b[Z_on]
											matrix jointtwosls[`row'+1,`col'+15] = _se[Z_on]
											matrix jointtwosls[`row',`col'+16]   = _b[Z_oe]
											matrix jointtwosls[`row'+1,`col'+16] = _se[Z_oe]
											
											*First stage first-choice*non 3CMO charter
											estimates restore firstD_oe
											matrix jointtwosls[`row',`col'+17]   = _b[Z_fn]
											matrix jointtwosls[`row'+1,`col'+17] = _se[Z_fn]
											matrix jointtwosls[`row',`col'+18]   = _b[Z_fe]
											matrix jointtwosls[`row'+1,`col'+18] = _se[Z_fe]
											matrix jointtwosls[`row',`col'+19]   = _b[Z_on]
											matrix jointtwosls[`row'+1,`col'+19] = _se[Z_on]
											matrix jointtwosls[`row',`col'+20]   = _b[Z_oe]
											matrix jointtwosls[`row'+1,`col'+20] = _se[Z_oe]		
											
											local col = `col' + 20
										
										} /*Close riskset loop*/
										/*
										*NAIVE 2SLS - no riskset controls
											*2SLS naive
											ivreg2 `out'_`t' 	(D_f D_o = `instrument') `cov_either' `cov2' `cov_grade' baselinespyear if group==`group', ///
																savefirst savefprefix(first) partial(`cov_either' `cov2' `cov_grade' baselinespyear) cluster(maskedid)  

											*2SLS - first choice charter
												matrix jointtwosls[`row',`col'+1]   = _b[D_f]
												matrix jointtwosls[`row'+1,`col'+1] = _se[D_f]
												matrix jointtwosls[`row'+2,`col'+1] = e(N)
												
												*2SLS - other choice charter
												matrix jointtwosls[`row',`col'+2] = _b[D_o]
												matrix jointtwosls[`row'+1,`col'+2] = _se[D_o]
												matrix jointtwosls[`row'+2,`col'+2] = e(N)
												
												*First stage first-choice charter
												estimates restore firstD_f
												matrix jointtwosls[`row',`col'+3]   = _b[Z_f]
												matrix jointtwosls[`row'+1,`col'+3] = _se[Z_f]

												matrix jointtwosls[`row',`col'+4]   = _b[Z_o]
												matrix jointtwosls[`row'+1,`col'+4] = _se[Z_o]
												
												*First stage other choice charter
												estimates restore firstD_o
												matrix jointtwosls[`row',`col'+5] = _b[Z_f]
												matrix jointtwosls[`row'+1,`col'+5] = _se[Z_f]
												
												matrix jointtwosls[`row',`col'+6] = _b[Z_o]
												matrix jointtwosls[`row'+1,`col'+6] = _se[Z_o]
													
												
											*2SLS naive conditional on r0 sample
											ivreg2 `out'_`t' 	(D_f D_o = `instrument') `cov_either' `cov2' `cov_grade' baselinespyear if group==`group' & in_zero1_joint==1, ///
																savefirst savefprefix(first) partial(`cov_either' `cov2' `cov_grade' baselinespyear) cluster(maskedid)  
												*2SLS - first choice charter
												matrix jointtwosls[`row',`col'+7]   = _b[D_f]
												matrix jointtwosls[`row'+1,`col'+7] = _se[D_f]
												matrix jointtwosls[`row'+2,`col'+7] = e(N)
											
												*2SLS - other choice charter
												matrix jointtwosls[`row',`col'+8] = _b[D_o]
												matrix jointtwosls[`row'+1,`col'+8] = _se[D_o]
												matrix jointtwosls[`row'+2,`col'+8] = e(N)
												
												*First stage first-choice charter
												estimates restore firstD_f
												matrix jointtwosls[`row',`col'+9]   = _b[Z_f]
												matrix jointtwosls[`row'+1,`col'+9] = _se[Z_f]

												matrix jointtwosls[`row',`col'+10]   = _b[Z_o]
												matrix jointtwosls[`row'+1,`col'+10] = _se[Z_o]
												
												*First stage other choice charter
												estimates restore firstD_o
												matrix jointtwosls[`row',`col'+11] = _b[Z_f]
												matrix jointtwosls[`row'+1,`col'+11] = _se[Z_f]
												
												matrix jointtwosls[`row',`col'+12] = _b[Z_o]
												matrix jointtwosls[`row'+1,`col'+12] = _se[Z_o]									

									
										*2SLS with no cov controls								
											*2SLS
											ivreg2 `out'_`t' 	(D_f D_o = `instrument') baselinespyear if group==`group' & in_zero1_joint==1, ///
																savefirst savefprefix(first) partial(baselinespyear) cluster(maskedid)  
											
												*2SLS - first choice charter
												matrix jointtwosls[`row',`col'+13]   = _b[D_f]
												matrix jointtwosls[`row'+1,`col'+13] = _se[D_f]
												matrix jointtwosls[`row'+2,`col'+13] = e(N)
												
												*2SLS - other choice charter 
												matrix jointtwosls[`row',`col'+14] = _b[D_o]
												matrix jointtwosls[`row'+1,`col'+14] = _se[D_o]
												matrix jointtwosls[`row'+2,`col'+14] = e(N)
												
												*First stage first-choice charter
												estimates restore firstD_f
												matrix jointtwosls[`row',`col'+15]   = _b[Z_f]
												matrix jointtwosls[`row'+1,`col'+15] = _se[Z_f]

												matrix jointtwosls[`row',`col'+16]   = _b[Z_o]
												matrix jointtwosls[`row'+1,`col'+16] = _se[Z_o]
												
												*First stage other choice charter
												estimates restore firstD_o
												matrix jointtwosls[`row',`col'+17] = _b[Z_f]
												matrix jointtwosls[`row'+1,`col'+17] = _se[Z_f]
												
												matrix jointtwosls[`row',`col'+18] = _b[Z_o]
												matrix jointtwosls[`row'+1,`col'+18] = _se[Z_o]		
										
												
									local col = `col' + 18
									*/
									} /* instrument type loop */													
							} /*close condition on number of observations*/
							
							local row = `row' + 3
						} 	 /*out loop*/
						
						local ++row
					}	/* t loop*/
					
				}	/*group loop*/
		
			preserve
				svmat jointtwosls
				keep var jointtwosls*

				keep if _n<=`row'
				ren jointtwosls1 z0_fn
				ren jointtwosls2 z0_fe
				ren jointtwosls3 z0_on
				ren jointtwosls4 z0_oe
				ren jointtwosls5 OLS_fn
				ren jointtwosls6 OLS_fe
				ren jointtwosls7 OLS_on
				ren jointtwosls8 OLS_oe
				
				ren jointtwosls9 OLS_psample_fn
				ren jointtwosls10 OLS_psample_fe
				ren jointtwosls11 OLS_psample_on
				ren jointtwosls12 OLS_psample_oe
				ren jointtwosls13 OLS_psat_fn
				ren jointtwosls14 OLS_psat_fe
				ren jointtwosls15 OLS_psat_on
				ren jointtwosls16 OLS_psat_oe
				
				ren jointtwosls17 jointtwosls_r01_fn
				ren jointtwosls18 jointtwosls_r01_fe
				ren jointtwosls19 jointtwosls_r01_on
				ren jointtwosls20 jointtwosls_r01_oe
				ren jointtwosls21 FS_r01_D_fn_fn
				ren jointtwosls22 FS_r01_D_fn_fe
				ren jointtwosls23 FS_r01_D_fn_on
				ren jointtwosls24 FS_r01_D_fn_oe
				ren jointtwosls25 FS_r01_D_fe_fn
				ren jointtwosls26 FS_r01_D_fe_fe
				ren jointtwosls27 FS_r01_D_fe_on
				ren jointtwosls28 FS_r01_D_fe_oe
				ren jointtwosls29 FS_r01_D_on_fn
				ren jointtwosls30 FS_r01_D_on_fe
				ren jointtwosls31 FS_r01_D_on_on
				ren jointtwosls32 FS_r01_D_on_oe
				ren jointtwosls33 FS_r01_D_oe_fn
				ren jointtwosls34 FS_r01_D_oe_fe
				ren jointtwosls35 FS_r01_D_oe_on
				ren jointtwosls36 FS_r01_D_oe_oe
				
				ren jointtwosls37 jointtwosls_r001_fn
				ren jointtwosls38 jointtwosls_r001_fe
				ren jointtwosls39 jointtwosls_r001_on
				ren jointtwosls40 jointtwosls_r001_oe
				ren jointtwosls41 FS_r001_D_fn_fn
				ren jointtwosls42 FS_r001_D_fn_fe
				ren jointtwosls43 FS_r001_D_fn_on
				ren jointtwosls44 FS_r001_D_fn_oe
				ren jointtwosls45 FS_r001_D_fe_fn
				ren jointtwosls46 FS_r001_D_fe_fe
				ren jointtwosls47 FS_r001_D_fe_on
				ren jointtwosls48 FS_r001_D_fe_oe
				ren jointtwosls49 FS_r001_D_on_fn
				ren jointtwosls50 FS_r001_D_on_fe
				ren jointtwosls51 FS_r001_D_on_on
				ren jointtwosls52 FS_r001_D_on_oe
				ren jointtwosls53 FS_r001_D_oe_fn
				ren jointtwosls54 FS_r001_D_oe_fe
				ren jointtwosls55 FS_r001_D_oe_on
				ren jointtwosls56 FS_r001_D_oe_oe
				
				ren jointtwosls57 jointtwosls_r0_fn
				ren jointtwosls58 jointtwosls_r0_fe
				ren jointtwosls59 jointtwosls_r0_on
				ren jointtwosls60 jointtwosls_r0_oe
				ren jointtwosls61 FS_r0_D_fn_fn
				ren jointtwosls62 FS_r0_D_fn_fe
				ren jointtwosls63 FS_r0_D_fn_on
				ren jointtwosls64 FS_r0_D_fn_oe
				ren jointtwosls65 FS_r0_D_fe_fn
				ren jointtwosls66 FS_r0_D_fe_fe
				ren jointtwosls67 FS_r0_D_fe_on
				ren jointtwosls68 FS_r0_D_fe_oe
				ren jointtwosls69 FS_r0_D_on_fn
				ren jointtwosls70 FS_r0_D_on_fe
				ren jointtwosls71 FS_r0_D_on_on
				ren jointtwosls72 FS_r0_D_on_oe
				ren jointtwosls73 FS_r0_D_oe_fn
				ren jointtwosls74 FS_r0_D_oe_fe
				ren jointtwosls75 FS_r0_D_oe_on
				ren jointtwosls76 FS_r0_D_oe_oe
	
				ren jointtwosls77 jointtwosls_joint_r01_fn
				ren jointtwosls78 jointtwosls_joint_r01_fe
				ren jointtwosls79 jointtwosls_joint_r01_on
				ren jointtwosls80 jointtwosls_joint_r01_oe
				ren jointtwosls81 FS_joint_r01_D_fn_fn
				ren jointtwosls82 FS_joint_r01_D_fn_fe
				ren jointtwosls83 FS_joint_r01_D_fn_on
				ren jointtwosls84 FS_joint_r01_D_fn_oe
				ren jointtwosls85 FS_joint_r01_D_fe_fn
				ren jointtwosls86 FS_joint_r01_D_fe_fe
				ren jointtwosls87 FS_joint_r01_D_fe_on
				ren jointtwosls88 FS_joint_r01_D_fe_oe
				ren jointtwosls89 FS_joint_r01_D_on_fn
				ren jointtwosls90 FS_joint_r01_D_on_fe
				ren jointtwosls91 FS_joint_r01_D_on_on
				ren jointtwosls92 FS_joint_r01_D_on_oe
				ren jointtwosls93 FS_joint_r01_D_oe_fn
				ren jointtwosls94 FS_joint_r01_D_oe_fe
				ren jointtwosls95 FS_joint_r01_D_oe_on
				ren jointtwosls96 FS_joint_r01_D_oe_oe
				
				ren jointtwosls97 jointtwosls_joint_r001_fn
				ren jointtwosls98 jointtwosls_joint_r001_fe
				ren jointtwosls99 jointtwosls_joint_r001_on
				ren jointtwosls100 jointtwosls_joint_r001_oe
				ren jointtwosls101 FS_joint_r001_D_fn_fn
				ren jointtwosls102 FS_joint_r001_D_fn_fe
				ren jointtwosls103 FS_joint_r001_D_fn_on
				ren jointtwosls104 FS_joint_r001_D_fn_oe
				ren jointtwosls105 FS_joint_r001_D_fe_fn
				ren jointtwosls106 FS_joint_r001_D_fe_fe
				ren jointtwosls107 FS_joint_r001_D_fe_on
				ren jointtwosls108 FS_joint_r001_D_fe_oe
				ren jointtwosls109 FS_joint_r001_D_on_fn
				ren jointtwosls110 FS_joint_r001_D_on_fe
				ren jointtwosls111 FS_joint_r001_D_on_on
				ren jointtwosls112 FS_joint_r001_D_on_oe
				ren jointtwosls113 FS_joint_r001_D_oe_fn
				ren jointtwosls114 FS_joint_r001_D_oe_fe
				ren jointtwosls115 FS_joint_r001_D_oe_on
				ren jointtwosls116 FS_joint_r001_D_oe_oe
				
				ren jointtwosls117 jointtwosls_joint_r0_fn
				ren jointtwosls118 jointtwosls_joint_r0_fe
				ren jointtwosls119 jointtwosls_joint_r0_on
				ren jointtwosls120 jointtwosls_joint_r0_oe
				ren jointtwosls121 FS_joint_r0_D_fn_fn
				ren jointtwosls122 FS_joint_r0_D_fn_fe
				ren jointtwosls123 FS_joint_r0_D_fn_on
				ren jointtwosls124 FS_joint_r0_D_fn_oe
				ren jointtwosls125 FS_joint_r0_D_fe_fn
				ren jointtwosls126 FS_joint_r0_D_fe_fe
				ren jointtwosls127 FS_joint_r0_D_fe_on
				ren jointtwosls128 FS_joint_r0_D_fe_oe
				ren jointtwosls129 FS_joint_r0_D_on_fn
				ren jointtwosls130 FS_joint_r0_D_on_fe
				ren jointtwosls131 FS_joint_r0_D_on_on
				ren jointtwosls132 FS_joint_r0_D_on_oe
				ren jointtwosls133 FS_joint_r0_D_oe_fn
				ren jointtwosls134 FS_joint_r0_D_oe_fe
				ren jointtwosls135 FS_joint_r0_D_oe_on
				ren jointtwosls136 FS_joint_r0_D_oe_oe

				
				outsheet using "`tables'inDPSbase_fc_oc_3cmo_4endog_`file'_joint_twosls.csv", comma replace
			restore
		clear	

		} /*Close 2SLS condition*/
} /*Close file loop*/
					
					
					
					
