/*
This do file outputs 3 different files for the 3 different score strategies 
(formula, frequency and simulation)
For table 2 you need simulation. 
Columns: tot_rank_year_1	capacity_year_1		tot_offer_year_1		tot_school_not01_year_1		tot_school_not01_1st_year_1
Rows: The school ids which are outputted. 

*/


clear all
cap log close
set more off
set trace off
set tracedepth 1 
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"
pause on

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"
local rawdata "Y:\DPS\Raw Data\"

matrix demo			  = J(30,30,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,40,.)
matrix subtwosls	  = J(500,40,.)
matrix twosls_nocov    		   = J(500,40,.)
matrix twosls_scorecov 		   = J(500,40,.)
matrix twosls_fcsample   		= J(50	0,40,.)

matrix qualbalance    = J(500,20,.)
matrix qualtwosls	  = J(500,20,.)
matrix qualsubtwosls  = J(500,20,.)

local pscore_stacking 	= 0
local schools 			= 1
local student_demo 		= 0
local pscore_balance 	= 0
local pscore_twosls 	= 0
local pscore_twosls_fcsample = 0
local nocov_twosls 		= 0
local pscore_subtwosls 	= 0

local qual_stacking = 0
local qual_balance=0
local qual_twosls = 0
local qual_subtwosls =0

*Dates of analysis files
local pscoredate_year2	20170310 	/*20160516 20160510 20160428 20160411 20160208 20160112 20151120 20150514 */
local pscoredate_year1  20170310 /*20160516    	20160516 20160510 20160421  20160208 */
local simsdate 			20150603 /* 20150428 */
local qualdate_year1 	20160715 /*20150708  20150420 */
local qualdate_year2	20160715 /*20160303*/
local qualdate			20160303
local qualpscoredate	20160208
local appdatayr1		20170310
local appdatayr2		20170310


local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_*"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1        		/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
local outsheet_rescorr=0	/* If want to outsheet matrix with residual correlations */


	************************************************
	**************** Schools Table *****************
	************************************************
	
	
		* preparing school capacities 
		use "`cleandata'application_data_year_one_`appdatayr1'.dta", clear
		
		keep capacity requestedschool bucket_id 
		duplicates drop  
		collapse (sum) capacity, by(requested)
		ren requested appschoolnum		
		tempfile capacityyr1
		sa `capacityyr1'
		
		use "`cleandata'application_data_year_two_`appdatayr2'.dta", clear
		
		keep capacity requestedschool bucket_id 
		duplicates drop  
		collapse (sum) capacity, by(requested)
		ren requested appschoolnum
		tempfile capacityyr2
		sa `capacityyr2'

		foreach file in simulation frequency formula{
		
			foreach sch_type in study /*noex nonnoex */ {
		
				use "`cleandata'mdrd_denver_`sch_type'_`file' `pscoredate_year1'", clear

				merge m:1 appschoolnum using `capacityyr1'
				ren capacity capacity_year_1
				drop _merge
				
				append using  "`cleandata'mdrd_denver_`sch_type'_year_2_`file' `pscoredate_year2'", gen(appended)
				merge m:1 appschoolnum using `capacityyr2'
				ren capacity capacity_year_2	

				*Simulation has huge support. Use rounded values for full saturation
				if "`file'"=="simulation"{
					ren pscore_`sch_type'_simulation sim_pscore
					* USing rounded to 1000 
					gen double pscore_`sch_type'_simulation = round(sim_pscore,0.001)
				}
				

				destring baselinegrade, replace
				destring appgrade, gen(appgrade_tmp)
				drop appgrade
				gen appgrade = appgrade_tmp
				drop appgrade_tmp
				
				*High school flag			
				gen hsgrade = ( appgrade >= 9 )
				gen msgrade = ( appgrade < 9 )
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
					
				*Analysis at the student level. Generate max for charter offer
				bys maskedid appended: egen any_`sch_type'_offer 	= max(offer_`sch_type')
				bys maskedid appended: egen num_schools = count(appschoolnum)
				bys maskedid appended: egen num_study = count(cond(appstudy==1,appschoolnum,.))
				gen pct_study = num_study / num_schools
				
				*Enrolls at charter flag (student level)
				bys maskedid appended: egen enroll_any_`sch_type' 	= max(`sch_type')
				
				*Rank 1 at school level
				gen rank1_study = (appstudy==1 & apprank==1)
			
				************************** School oversubscription table **************************
				*Schools ranked, assigned, 0 < prob assignment < 1 
				preserve
					keep if app`sch_type'==1
					
					bys appschoolnum: egen tot_rank1 = sum(rank1_study)
					bys appschoolnum: egen tot_rank_year_1 = count(maskedid) if appended == 0
					bys appschoolnum: egen tot_rank_year_2 = count(maskedid) if appended == 1
					bys appschoolnum: egen tot_offer_year_1 = sum(offer) if appended == 0
					bys appschoolnum: egen tot_offer_year_2 = sum(offer) if appended == 1
					
					*IR: Breaking down by year 1 and year 2. 
					
					bys appschoolnum: egen tot_`sch_type'_not01_year_1 = count(cond(pscore_`sch_type'_`file' > 0 & pscore_`sch_type'_`file'<1,maskedid,.)) if appended == 0
					bys appschoolnum: egen tot_`sch_type'_not01_year_2 = count(cond(pscore_`sch_type'_`file' > 0 & pscore_`sch_type'_`file'<1,maskedid,.)) if appended == 1
					
					bys appschoolnum: egen tot_school_not01_year_1 = count(cond(pscore_school>0 & pscore_school<1,maskedid,.)) if appended == 0
					bys appschoolnum: egen tot_school_not01_year_2 = count(cond(pscore_school>0 & pscore_school<1,maskedid,.)) if appended == 1

					bys appschoolnum: egen tot_school_is1_year_1 = count(cond(pscore_school==1,maskedid,.)) if appended == 0
					bys appschoolnum: egen tot_school_is1_year_2 = count(cond(pscore_school==1,maskedid,.)) if appended == 1

					bys appschoolnum: egen tot_school_not01_1st_year_1 = count(cond(pscore_school > 0 & pscore_school < 1 & apprank == 1,maskedid,.)) if appended == 0 
					bys appschoolnum: egen tot_school_not01_1st_year_2 = count(cond(pscore_school > 0 & pscore_school < 1 & apprank == 1,maskedid,.)) if appended == 1

					*IR variables stacked. 
					bys appschoolnum: egen tot_`sch_type'_not01 = count(cond(pscore_`sch_type'_`file'>0 & pscore_`sch_type'_`file'<1,maskedid,.))
					bys appschoolnum: egen tot_school_not01_stacked = count(cond(pscore_school>0 & pscore_school<1,maskedid,.))
					bys appschoolnum: egen tot_school_is1_stacked = count(cond(pscore_school==1,maskedid,.))
					bys appschoolnum: egen tot_school_not01_1st_stacked = count(cond(pscore_school > 0 & pscore_school < 1 & apprank == 1,maskedid,.))
					la var tot_school_not01_1st_stacked "tot school pscore not 01 and first choice in study with years stacked"
					collapse (firstnm) appschoolname tot_* MS HS capacity*, by(appschoolnum)
										
					tempfile temp
					sa `temp'
				restore
				
				*Subject to randomization at the FULLY SATURATED PSCORE level (variation in instrument for every value of PSCORE) preserve
				
				preserve

					drop if ( baselinespyear != 2013 & appended == 1 ) | ( baselinespyear != 2012 & appended == 0)
			
					keep if app`sch_type'==1
					
					*Any charter offer pscore - by school, the number of kids in pscore sets with variation in any offer
						bys pscore_`sch_type'_`file': egen min_offer = min(any_`sch_type'_offer)
						bys pscore_`sch_type'_`file': egen max_offer = max(any_`sch_type'_offer)
						
						gen sum = min_offer + max_offer
						bys appschoolnum: egen tot_`sch_type'_rand = count(cond(sum==1 ,maskedid,.))
						bys appschoolnum: egen tot_`sch_type'_rand_not01 = count(cond(sum==1 & pscore_`sch_type'_`file'>0 & pscore_`sch_type'_`file'<1,maskedid,.))
						
					*School offer - if within the same school, there are kids with the same pscore and var in offer
						drop min_offer max_offer sum
						bys appschoolnum pscore_school: egen min_offer = min(offer)
						bys appschoolnum pscore_school: egen max_offer = max(offer)
						
						gen sum = min_offer + max_offer
						bys appschoolnum: egen tot_school_rand = count(cond(sum==1,maskedid,.))
						bys appschoolnum: egen tot_school_rand_not01 = count(cond(sum==1 & pscore_school>0 & pscore_school<1,maskedid,.))
						drop min_offer max_offer
						
						*Number of kids with family link
						bys appschoolnum: egen tot_family = sum(family)
						
						duplicates drop appschoolnum, force
						keep appschoolnum  tot_*
						
						merge 1:1 appschoolnum using `temp'
						drop _merge
						
						order appschoolnum appschoolname MS HS ///
							tot_rank1 tot_rank_year_?	tot_offer_year_? ///
							tot_`sch_type'_not01	tot_school_not01_stacked tot_school_is1_stacked ///
							tot_`sch_type'_rand	tot_`sch_type'_rand_not01	///
							tot_school_rand	tot_school_rand_not01
						gsort - MS + appschoolname
						
						outsheet using "`tables'inDPSbase_`file'_`sch_type'_years_stacked_schools.csv", comma replace

				restore
				
			}
		}
	
		
