/* Ye Ji Kee
		For grades 4-10. 
		
		Purpose: investigating sample size decrease in Table 1 of the draft.
			- How many students are assigned to each of 3 big theta groups?
			- how many students have MID = 0?
			- how many students have MID = 1?
			- how many students have tau = 0 in the marginal priority group?
			- how many students have tau = 1 in the marginal priority group?
		
This file uses the at run file for year 1 , which has preferences and priorities for the applicants in 2013. 

Relevant columns of outputted files; 
bucket_count	max_capacity	offer_count		num_thetaminus_worsethanmarg		num_thetaminus_marginal	num_thetastar_mid_b_tau		num_thetaminus_betterthanmarg	num_thetaplus_mid_o0	num_thetaplus_mid_0 num_thetaminus_worsethanmarg_add		num_thetastar_mid_tau
Rows: 
393
444
445
392
428
429

*/

clear all
cap log close
set more off
set trace off
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"

pause on 

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local yusuke_data "Y:/DPS/Yusuke/data/"
local asgndata  "Y:/DPS/Raw Data/2013_03_21_enrollment-match/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/"

local appdate  20170310 /*20160512 20160208 20150708 */

matrix app_anatomy    = J(500,50,.)



*Retrieve application, offer, and grade information from AT RUN file
insheet using "`asgndata'1_AT_RUN_022262012.csv", double clear
	ren maskedstudentids maskedid
	ren requestedschool appschoolnum
	
	drop if maskedid=="#VALUE!"
	destring maskedid, ignore("#VALUE!") replace

	keep if inlist(appschoolnum, 443, 426, 428, 393, 445, 444, 392, 429, 441, 462, 496, 513)
	
	
	** Tabulating the number of applicants in each bucket
	preserve
	
	collapse (count) maskedid, by(appschoolnum grade capacityprogram)
	rename maskedid bucket_count
	
	tostring appschoolnum, replace
	destring grade, replace
	tostring grade, replace
	
	gen bucket = appschoolnum + grade + capacityprogram
	
	order bucket appschoolnum grade capacityprogram bucket_count
	
	outsheet using "`tables'buckets_count.csv", comma replace
	
	restore
	
	
	
	** Tabulating capacity, number of students in big theta groups, number of students with MID = 0 or 1, number of students with lottery cutoff = 0 or 1 for each bucket
	preserve
	
		use "`cleandata'application_data_year_one_`appdate'.dta", clear
		
		keep if inlist(requestedschool, 443, 426, 428, 393, 445, 444, 392, 429, 441, 462, 496, 513)
		
		rename requestedschool appschoolnum
		rename maskedstudentids maskedid
		
		keep maskedid appschoolnum capacityprogram capacity frequency_offer pscore_formula lottery_cutoff marginal_priority marginal effective_cutoff MID better_than_marginal worse_than_marginal always_assigned_more_preferred never_assigned sometimes_assigned conditionally_always_assigned
		
		destring maskedid, replace
		
		keep if maskedid > 100000
		
		tempfile appdata
		save `appdata'
	
	restore
	
	merge 1:1 maskedid appschoolnum capacityprogram using `appdata'
	tab _merge
	drop _merge
	

	
	/*
	merge 1:1 maskedid appschoolnum capacityprogram using "Y:\DPS\MDmeetsRD Denver\Data\application_data_year_one_mdrd_anatomy.dta"
	tab _merge
	drop if _merge == 2
	drop _merge
	
	replace MID_alt = MID_alt / 1000000
	replace MID_alt = MID if MID_alt == . & MID == 0
	*/
	
	
	preserve
	
	
	gen bucket_freq_pscore_in01 = .
	gen bucket_formula_pscore_in01 = .
	replace bucket_freq_pscore_in01 = 1 if frequency_offer > 0 & frequency_offer < 1
	replace bucket_formula_pscore_in01 = 1 if pscore_formula > 0 & pscore_formula < 1
	
	/*
	* Special cases * 
	replace bucket_formula_pscore_in01 = 1 if appschoolnum == 443 & apppriorityseq != 0 & MID == . & grade == "06" & capacityprogram == "99FRL80239" & (maskedid == 692379 | maskedid == 702207)
	replace bucket_formula_pscore_in01 = 1 if appschoolnum == 428 & apppriorityseq != 0 & MID == . & grade == "06" & capacityprogram == "99" & (maskedid == 660118 | maskedid == 661459)
	replace bucket_formula_pscore_in01 = 1 if appschoolnum == 462 & apppriorityseq != 0 & MID == . & grade == "09" & capacityprogram == "9980249" & (maskedid == 605426 | maskedid == 611309)
	
	* special cases * (higher-ranked school's marginal priority = -999)
	replace MID = 0 if appschoolnum == 443 & apppriorityseq != 0 & MID == . & grade == "06" & capacityprogram == "99FRL80239" & (maskedid == 692379 | maskedid == 702207)
	replace MID = 0 if appschoolnum == 428 & apppriorityseq != 0 & MID == . & grade == "06" & capacityprogram == "99" & (maskedid == 660118 | maskedid == 661459)
	replace MID = 0 if appschoolnum == 444 & apppriorityseq != 0 & MID == . & grade == "06" & capacityprogram == "99" & maskedid == 652503
	replace MID = 0 if appschoolnum == 462 & apppriorityseq != 0 & MID == . & grade == "09" & capacityprogram == "9980249" & (maskedid == 605426 | maskedid == 611309)
	replace MID = 0 if appschoolnum == 496 & apppriorityseq != 0 & MID == . & grade == "09" & capacityprogram == "99General" & maskedid == 609906
	replace MID = 0 if appschoolnum == 513 & apppriorityseq != 0 & MID == . & grade == "09" & capacityprogram == "99" & (maskedid == 721232 | maskedid == 621948)
	*/
	
	gen mid_0 = .
	gen mid_1 = .
	gen mid_in01 = .
	gen mid_missing = .

	
	replace mid_0 = 1 if MID == 0
	replace mid_1 = 1 if MID == 1
	replace mid_in01 = 1 if MID > 0 & MID < 1
	replace mid_missing = 1 if MID  == .

	
	** When apppriorityseq = 0, ineligible. So, lottery_cutoff = 0.
	replace effective_cutoff = 0 if apppriorityseq == 0
	
	** When capacity = 0 & offer = 0, lottery cutoff = 0.
	replace effective_cutoff = 0 if capacity == . & effective_cutoff == .
	
	gen double effective_cutoff_tmp = effective_cutoff / 1000000
	
	*fixing floating point error 
	
	drop effective_cutoff
	ren effective_cutoff_tmp effective_cutoff
	
		
	/*
	* Special cases (formula pscore) *
	#delimit ;
	replace pscore_formula = effective_cutoff if (appschoolnum == 443 & apppriorityseq != 0 & MID == . & grade == "06" & capacityprogram == "99FRL80239" & (maskedid == 692379 | maskedid == 702207))
	| (appschoolnum == 428 & apppriorityseq != 0 & MID == . & grade == "06" & capacityprogram == "99" & (maskedid == 660118 | maskedid == 661459))
	| (appschoolnum == 462 & apppriorityseq != 0 & MID == . & grade == "09" & capacityprogram == "9980249" & (maskedid == 605426 | maskedid == 611309)) ;
	#delimit cr
	*/
	
	** Compute the number of seats randomized **
	gen randomized_prob_freq = frequency_offer if frequency_offer > 0 & frequency_offer < 1
	gen randomized_prob_form = 1 if pscore_formula > 0 & pscore_formula < 1 & status == "A"
	
	** 5 cases where formula pscore = 0
	gen thetaminus_always_more_pref = .
	replace thetaminus_always_more_pref = 1 if never_assigned == 1 & always_assigned_more_preferred == 1
	
	gen thetaminus_worsethanmarg = .
	replace thetaminus_worsethanmarg = 1 if never_assigned == 1 & always_assigned_more_preferred == 0 & worse_than_marginal == 1
	
	gen thetaminus_worsethanmarg_add = .
	replace thetaminus_worsethanmarg_add = 1 if never_assigned == 1 & always_assigned_more_preferred == 1 & worse_than_marginal == 1
	
	gen thetaminus_marginal = .
	replace thetaminus_marginal = 1 if never_assigned == 1 & always_assigned_more_preferred == 1 & marginal == 1
	
	gen thetaminus_betterthanmarg = .
	replace thetaminus_betterthanmarg = 1 if never_assigned == 1 & always_assigned_more_preferred == 1 & better_than_marginal == 1
	
	gen thetaplus_mid_1 = .
	replace thetaplus_mid_1 = 1 if conditionally_always_assigned == 1 & MID == 1
	
	gen thetastar_mid_1 = .
	replace thetastar_mid_1 = 1 if sometimes_assigned == 1 & MID == 1
	
	gen thetastar_mid_tau = .
	replace thetastar_mid_tau = 1 if sometimes_assigned == 1 & MID >= effective_cutoff & MID != .
	
	
	** 2 cases where formula pscore = 1
	gen thetaplus_mid_0 = .
	replace thetaplus_mid_0 = 1 if conditionally_always_assigned == 1 & MID == 0
	
	gen thetastar_mid_0_tau_1 = .
	replace thetastar_mid_0_tau_1 = 1 if sometimes_assigned == 1 & MID == 0 & effective_cutoff == 1
	
	** 2 cases where formula pscore in (0,1)
	gen thetaplus_mid_o0 = .
	replace thetaplus_mid_o0 = 1 if conditionally_always_assigned == 1 & MID > 0 & MID < 1
	
	gen thetastar_mid_b_tau = .
	replace thetastar_mid_b_tau = 1 if sometimes_assigned == 1 & MID < effective_cutoff
	
	
	** Lottery cutoff
	gen tau_0 = .
	gen tau_1 = .
	gen tau_in01 = .
	
	replace tau_0 = 1 if effective_cutoff == 0
	replace tau_1 = 1 if effective_cutoff == 1
	replace tau_in01 = 1 if effective_cutoff > 0 & effective_cutoff < 1
	
	bys appschoolnum grade capacityprogram: egen max_capacity = max(capacity)
	bys appschoolnum grade capacityprogram: egen num_thetaminus = total(never_assigned)
	bys appschoolnum grade capacityprogram: egen num_thetaplus = total(conditionally_always_assigned)
	bys appschoolnum grade capacityprogram: egen num_thetastar = total(sometimes_assigned)
	bys appschoolnum grade capacityprogram: egen num_mid_0 = total(mid_0)
	bys appschoolnum grade capacityprogram: egen num_mid_1 = total(mid_1)
	bys appschoolnum grade capacityprogram: egen num_mid_in01 = total(mid_in01)
	bys appschoolnum grade capacityprogram: egen num_mid_missing = total(mid_missing)
	bys appschoolnum grade capacityprogram: egen num_tau_0 = total(tau_0)
	bys appschoolnum grade capacityprogram: egen num_tau_1 = total(tau_1)
	bys appschoolnum grade capacityprogram: egen num_tau_in01 = total(tau_in01)
	bys appschoolnum grade capacityprogram: egen num_thetaminus_always_more_pref = total(thetaminus_always_more_pref)
	bys appschoolnum grade capacityprogram: egen num_thetaminus_worsethanmarg = total(thetaminus_worsethanmarg)
	bys appschoolnum grade capacityprogram: egen num_thetaminus_worsethanmarg_add = total(thetaminus_worsethanmarg_add)
	bys appschoolnum grade capacityprogram: egen num_thetaminus_marginal = total(thetaminus_marginal)
	bys appschoolnum grade capacityprogram: egen num_thetaminus_betterthanmarg = total(thetaminus_betterthanmarg)
	bys appschoolnum grade capacityprogram: egen num_thetaplus_mid_1 = total(thetaplus_mid_1)
	bys appschoolnum grade capacityprogram: egen num_thetastar_mid_1 = total(thetastar_mid_1)
	bys appschoolnum grade capacityprogram: egen num_thetastar_mid_tau = total(thetastar_mid_tau)
	bys appschoolnum grade capacityprogram: egen num_thetaplus_mid_0 = total(thetaplus_mid_0)
	bys appschoolnum grade capacityprogram: egen num_thetastar_mid_0_tau_1 = total(thetastar_mid_0_tau_1)
	bys appschoolnum grade capacityprogram: egen num_thetaplus_mid_o0 = total(thetaplus_mid_o0)
	bys appschoolnum grade capacityprogram: egen num_thetastar_mid_b_tau = total(thetastar_mid_b_tau)
	
	bys appschoolnum grade capacityprogram: egen seat_randomized_freq = total(randomized_prob_freq)
	bys appschoolnum grade capacityprogram: egen seat_randomized_form = total(randomized_prob_form)

	bys appschoolnum grade capacityprogram: egen num_bucket_freq_pscore_in01 = total(bucket_freq_pscore_in01)
	bys appschoolnum grade capacityprogram: egen num_bucket_formula_pscore_in01 = total(bucket_formula_pscore_in01)

	
	duplicates drop appschoolnum grade capacityprogram, force
	
	#delimit ;

	keep appschoolnum grade capacityprogram max_capacity num_thetaminus num_thetaplus num_thetastar num_mid_0 num_mid_1 num_mid_in01 num_mid_missing num_tau_0 num_tau_1 num_tau_in01 num_bucket_freq_pscore_in01
		num_thetastar_mid_tau num_thetastar_mid_1 num_thetaplus_mid_1 num_thetaminus_worsethanmarg num_thetaminus_always_more_pref
		num_thetaplus_mid_0 num_thetastar_mid_0_tau_1 num_thetaplus_mid_o0 num_thetastar_mid_b_tau num_bucket_freq_pscore_in01 num_bucket_formula_pscore_in01 num_bucket_formula_pscore_in01 seat_randomized_freq
		seat_randomized_form num_thetaminus_worsethanmarg_add num_thetaminus_marginal num_thetaminus_betterthanmarg ;
	#delimit cr

	
	tostring appschoolnum, replace
	destring grade, replace
	tostring grade, replace
	
	gen bucket = appschoolnum + grade + capacityprogram
	
	order bucket appschoolnum grade capacityprogram max_capacity
	
	outsheet using "`tables'buckets_capacity_thetagroup_mid.csv", comma replace

	restore
	
	
	** Tabulating offers for each bucket
	gen offer = .
	replace offer = 1 if status == "A"
	
	preserve
	
	collapse (count) offer, by(appschoolnum grade capacityprogram)
	rename offer offer_count
	
	tostring appschoolnum, replace
	destring grade, replace
	tostring grade, replace
	
	gen bucket = appschoolnum + grade + capacityprogram
	
	order bucket appschoolnum grade capacityprogram offer_count
	
	outsheet using "`tables'buckets_offercount.csv", comma replace
	
	restore
	
	
