/* 	Mayara Silva
	1/8/2015
	
	This code imports a modeified version of the assignment file (Based on Atila's DA simulation)
	and produces propensity score files for it.
	
*/

set more off
local cleandata "Y:\DPS\MDmeetsRD Denver\Data\"
local dta "Y:\DPS\MDmeetsRD Denver\Data\simulated_offers\"

local master_AT_RUN_school = 1
local simulation_analysis_file =0
local theory_analysis_file =0

local simscoredate		20170310 /*20160815 20160715 20160428 20160421    20150603*/	/*Date of latest simulation mdrd analysis file for realized offers*/
local theorydate		20170310 /*20160428 20160208 	20150708*/	/*Date of latest theoretical mdrd analysis file for realized offers*/
local atrundate			20150708	/*Date of latest master at run file containing simulated offers*/

local simsdate 		20150603 /*20150428*/
local outputdate	20170310 /*20160815 20160715 20160428 20160421*/

*****************************************************************
* Create file with school-level offers (DA_offers_master_AT_RUN)*
*****************************************************************

if `master_AT_RUN_school'==1{		
	
	u "`dta'DA_offers_master_AT_RUN `atrundate'.dta", clear
	
	
	ren maskedstudentids maskedid
	ren requestedschool appschoolnum
	
	*Not sure what rank=99 means, ask Atila
	gen apprank = choiceselection
	replace apprank = apprank*10 if apprank<10
	replace apprank = floor(apprank/10)
	
	drop if maskedid=="#VALUE!"
	destring maskedid, ignore("#VALUE!") replace

	*Offers for each status
	forval i =1/400{
		gen offer`i' = (status`i'=="A")
		bys maskedid appschoolnum: egen max_offer`i' = max(offer`i')
		drop offer`i'
		ren max_offer`i' offer`i'
	}
	
		gen family = (family!="")
		
		keep grade maskedid offer* appschoolnum apprank family

		ren grade appgrade
		duplicates drop appgrade maskedid appschoolnum apprank, force
		
		duplicates tag maskedid appgrade appschoolnum, gen(dup)
		tab dup
		
		*Cases of multiple rankings for the same school. Choose the smallest rank
		bys maskedid appgrade appschoolnum: egen min_rank = min(apprank)
		replace apprank = min_rank if dup
		
		*Keep if has nonmissing family link within school
		bys maskedid appgrade appschoolnum: egen max_fam=max(family)
		drop family
		ren max_fam family
		
		duplicates drop maskedid appgrade appschoolnum, force

		*Identify charters in application
		preserve
			insheet using "Y:\DPS\Raw Data\dps_charters_20112012_20150420 update.csv", double clear
			keep schoolname schoolnum charter alternative
			ren schoolnum appschoolnum
			gen appcharter = (charter==1)
			keep appschoolnum appcharter
			tempfile charter
			sa `charter'
		restore
	
		merge m:1 appschoolnum using `charter'
		keep if _merge!=2
		drop _merge
		replace appcharter=0 if appcharter==.
		
		*Study charters
		gen appstudy= (	inlist(appschoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
							inlist(appschoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444,445, 446) | ///
							inlist(appschoolnum, 462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
						)
		
		
		order offer*, last
		save "`dta'DA_offers_master_AT_RUN_school_level `outputdate'.dta", replace
}
****************************************************************************************
*********** Create true propensity score analysis files by simply updating offer *******
****************************************************************************************

if `simulation_analysis_file'==1{
		
		*"Simulation" analysis file contains Actual propensity score
		u "`cleandata'mdrd_denver_study_simulation `simscoredate'.dta", clear
		drop offer offer_study
		
		merge 1:1 maskedid appschoolnum using "`dta'DA_offers_master_AT_RUN_school_level `atrundate'.dta", keepusing(offer*)
		keep if _merge!=2
		drop _merge
		
		order offer*, last
		
		save "`dta'mdrd_denver_simulation_master `outputdate'", replace

}
****************************************************************************************
******************* Create theory propensity score analysis files **********************
****************************************************************************************

if `theory_analysis_file'==1{

	foreach score in frequency formula{
		
		*Main analysis file
		u "`cleandata'mdrd_denver_study_`score' `theorydate'.dta", clear

		drop offer offer_study pscore*
		
		*Get offers for each simulation from master AT RUN school-level file
		merge 1:1 maskedid appschoolnum using "`dta'DA_offers_master_AT_RUN_school_level `atrundate'.dta", keepusing(offer*)
		keep if _merge!=2
		drop _merge
		
		order offer*, last
		unique maskedid
		tostring maskedid, gen(maskedstudentids)
		
		*Get study charter-level propensity scores. Do this in blocks so that analysis file is not so big
		
		foreach first in 1 /*401 501 1201*/{
			local last = `first' + 399
			forval sims=`first'/`last'{
			
				merge m:1 maskedstudentids using "`dta'theoretical_pscore_files\pscore_charter_`score'_year_one_`sims'.dta"
				keep if _merge!=2
				drop _merge
				
				*Replace with zero if missing
				replace pscore_charter_`score'=0 if pscore_charter_`score'==.			
				ren pscore_charter_`score' pscore_charter_`score'`sims'		
			}
			
			drop maskedstudentid
			
			save "`dta'mdrd_denver_`score'_master `outputdate'", replace
			unique maskedid
			
		} /*Close first code*/
	}
}
