/* Ignacio Rodriguez

This file produces the full type balance controls estiamtes for the covariate balance table. 
Its equivalent for year 1 and 2 is 6_fulltype_balance_tables.do

It uses output from 6_fulltype_balance_tables_year_2 and 6_fulltype_balance_tables
*/


clear all
cap log close
set more off
set trace off
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"
pause on

global cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
global tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"

local setup = 1
local balance = 1
local fulltype_twosls = 0

matrix balance        = J(500,20,.)
matrix twosls         = J(500,40,.)

local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

local cov_grade "g_*"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1


if `setup' == 1 {
	u "${cleandata}analysis_fulltype.dta", clear
	*keep if baselinespyear==2012	

	append using "${cleandata}analysis_fulltype_year_2.dta", gen(year2)
	*drop if baselinespyear !=2013 & year2 == 1

		drop if baselinespyear !=2013 & year2 == 1
		drop if baselinespyear !=2012 & year2 == 0

	
	*making type unique across both years. 
	su type_all if year2 == 0
	local max_type_year1 = `r(max)'
	replace type_all = type_all + `max_type_year1' if year2 == 1
	
	drop type_?c_all
	
	* regenerating risk sets to account for new type range

	drop sd_z* in_good_cell* 
	
	bys type_all: egen sd_z = sd(Z)
	
	*do not count a missing type as a cell
	gen in_good_cell = (	sd_z>0 & !missing(sd_z) & type_all != .	)
	
	
	* get an error if you try to do this for all values of type. 
	tab type_all if year2 == 0, gen(d_y1_)
	tab type_all if year2 == 1, gen(d_y2_)

	* generating dummies for types whose cells are 'good' (have variation in the insrument)
	* this is for computational purposes, In keeping with how the logit F tests are run in file 5
	* which use the tag_r0_ flags (which only flag the risksets with variation in offer. 

	tab type_all  if in_good_cell ==1 , gen(tag_type_)
	
	*dropping duplicates from year 2. 
	duplicates tag masked, gen(dup_masked)
	drop if dup_masked > 0 & baselinespyear == 2013	
	
	
	save "${cleandata}analysis_fulltype_stacked.dta", replace
}




foreach cut in inDPSbase /*allapps*/{

	*****************************************
	******** Balance and attrition **********
	*****************************************
	
	if `balance'==1{
	
		use "${cleandata}analysis_fulltype_stacked.dta", clear 
		


		local row = 1
		local col = 1
		gen var =""
		
		replace var = "Riskset points of support" if _n ==1
		
		unique type_all 		if in_good_cell==1
		matrix balance[1,3] = r(sum)
		local row = `row' + 1

		foreach var in `appcov' `cov' `cov2' `attrition_vars'{
			replace var="`var'" if _n==`row'
			replace var="`var'_se" if _n==`row'+1
			replace var="`var'_N" if _n==`row'+2
		
			*All non-offered applicants
			qui summ `var' if any_in_sample == 1 & Z==0
				matrix balance[`row',`col']   = r(mean)
				matrix balance[`row'+1,`col'] = r(sd)
				matrix balance[`row'+2,`col'] = r(N)
			
			
			*Balance and attrition, no control(OLS)
			qui reg `var' Z if any_in_sample==1, r
				matrix balance[`row',`col'+1]   = _b[Z]
				matrix balance[`row'+1,`col'+1] = _se[Z]
				matrix balance[`row'+2,`col'+1] = e(N)
			
			*Fully staturated in type
			qui areg `var' Z i.baselinespyear if any_in_sample==1 & in_good_cell==1, absorb(type_all) r  //do areg to make it run faster
			
				matrix balance[`row',`col'+2]   = _b[Z]
				matrix balance[`row'+1,`col'+2] = _se[Z]
				matrix balance[`row'+2,`col'+2] = e(N)
			
		local row = `row' + 4
		}
		
		**********************************************************************************
		************************************* Logit Fs ***********************************
		**********************************************************************************
		
		foreach type in robust traditional{
				
				local row = `row' + 3
				
				if "`type'"=="robust"{
					local option "nocnsreport vce(r)"
				}
				else{
					local option "nocnsreport"
				}
				
				* Balance logit
				replace var="Logit Balance F" if _n==`row'
				replace var="Logit Balance F (pval)" if _n==`row'+1
				replace var="Logit Balance F (df)" if _n==`row'+2
				replace var="Logit Balance F (N)" if _n==`row'+3
			
				*No riskset
				qui logit Z `appcov' `cov' `cov2', `option'
				test `appcov' `cov' `cov2'
				matrix balance[`row', `col'+1] = r(chi2)
				matrix balance[`row'+1, `col'+1] = r(p)
				matrix balance[`row'+2, `col'+1] = r(df)
				matrix balance[`row'+3, `col'+1] = e(N)


				*Fully saturated
				qui logit Z i.type_all i.baselinespyear `appcov' `cov' `cov2' if in_good_cell==1, `option'
				test `appcov' `cov' `cov2'
				matrix balance[`row', `col' + 2] = r(chi2)
				matrix balance[`row' +1,`col' + 2] = r(p)
				matrix balance[`row'+2, `col' + 2] = r(df)
				matrix balance[`row'+3, `col' + 2] = e(N)
			
			local row = `row' + 4
			
		}

		********************************************************
		*************** Balance Traditional F ******************
		********************************************************
		
		foreach covs in apponly balanceonly all{
		
			if "`covs'"=="apponly"{
				local balF "`appcov'"
				local Fname "Application Balance"
			}
			
			else if "`covs'"=="balanceonly"{
				local balF "`cov' `cov2'"
				local Fname "Baseline Balance"
			}
			else if "`covs'"=="all"{
				local balF "`appcov' `cov' `cov2'"
				local Fname "All Balance"
			}
		
				replace var="Traditional `Fname' F-test" if _n==`row'
				replace var="Traditional `Fname' F-test (pval)" if _n==`row' +1
				replace var="Traditional `Fname' F-test (residuals df)" if _n==`row' +2
				replace var="Traditional `Fname' F-test (model df)" if _n==`row' +3
				replace var="Traditional `Fname' F-test (N)" if _n==`row' +4
			
			*no propensity score
				mvreg `balF' = Z
				test Z
				matrix balance[`row',`col' + 1]=r(F)
				matrix balance[`row'+1,`col' + 1]=r(p)
				matrix balance[`row'+2, `col'+1] = r(df_r)
				matrix balance[`row'+3, `col'+1] = r(df)
				matrix balance[`row'+4, `col'+1] = e(N)
				
				*Pscore saturated
				mvreg `balF' = Z i.type_all i.baselinespyear  if in_good_cell==1
				test Z
				matrix balance[`row',`col' + 2]=r(F)
				matrix balance[`row'+1,`col' +2]=r(p)
				matrix balance[`row'+2, `col'+2] = r(df_r)
				matrix balance[`row'+3, `col'+2] = r(df)
				matrix balance[`row'+4, `col'+2] = e(N)
			
		
		local row = `row' + 5
		}
		
		**********************************************************************************
		******************************** Stacked Clustered F ********************************
		**********************************************************************************
		
		foreach covs in apponly balanceonly all{
					
					if "`covs'"=="apponly"{
						local balF "`appcov'"
						local Fname "Application Balance"
						local expand = 3
						local test "Z_1 Z_2 Z_3"
					}
					
					else if "`covs'"=="balanceonly"{
						local balF "`cov' `cov2'"
						local Fname "Baseline Balance"
						local expand = 12
						local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12"
					}
					else if "`covs'"=="all"{
						local balF "`appcov' `cov' `cov2'"
						local Fname "All Balance"
						local expand = 15
						local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12 Z_13 Z_14 Z_15"
					}
		
		************************************* Balance ************************************
			preserve
				keep maskedid `balF' Z D type_all family any_in_sample in_good_cell baselinespyear
				expand `expand'
				bys maskedid: gen copy = _n
				sort copy
				
				*Single vector of outcomes
				gen cov =.
				gen covname=""
				tokenize `balF'
				forval i=1/`expand'{
					replace cov = ``i'' if copy==`i' 
					replace covname = "``i''" if copy==`i' 
				}
				drop `balF'
				
				*Instruments, pscore variables, and dummies for each copy
				forval i=1/`expand'{
					gen Z_`i' = Z*(copy == `i')
					gen alpha_`i' = (copy == `i')
					gen c_baselinespyear_`i' = (copy == `i') * baselinespyear
				}
				
				*Absorb takes only one var. Create grouped vars
				foreach x in type {
					egen group_`x' = group(copy `x')
				}	
				
				*** One tall regression for each specification ***
				
				*No controls				
					*Clustered
					qui reg cov alpha_* Z_* , vce(cluster maskedid)
					test `test'
					
					matrix balance[`row',`col' + 1]=r(F)
					matrix balance[`row'+1,`col' + 1]=r(p)
					matrix balance[`row'+2,`col' + 1]=r(df_r)
					matrix balance[`row'+3,`col' + 1]=r(df)
					matrix balance[`row'+4,`col' + 1]=e(N)
					
					*Traditional
					qui reg cov alpha_* Z_* 
					test `test'
					
					matrix balance[`row'+5,`col' + 1]=r(F)
					matrix balance[`row'+6,`col' + 1]=r(p)
					matrix balance[`row'+7,`col' + 1]=r(df_r)
					matrix balance[`row'+8,`col' + 1]=r(df)
					matrix balance[`row'+9,`col' + 1]=e(N)
				
				*Saturated
					*Clustered
					qui areg cov alpha_* Z_* i.c_baselinespyear_* if in_good_cell==1, absorb(group_type) vce(cluster maskedid)
					test `test'
					
					matrix balance[`row',`col' + 2]	=	r(F)
					matrix balance[`row'+1,`col' + 2]	=	r(p)
					matrix balance[`row'+2,`col' + 2]	=	r(df_r)
					matrix balance[`row'+3,`col' + 2] =	r(df)
					matrix balance[`row'+4,`col' + 2] = e(N)
					
					*Traditional
					qui areg cov alpha_* Z_* i.c_baselinespyear_* if in_good_cell==1, absorb(group_type)
					test `test'
					
					matrix balance[`row'+5,`col' + 2]	=	r(F)
					matrix balance[`row'+6,`col' + 2]	=	r(p)
					matrix balance[`row'+7,`col' + 2]	=	r(df_r)
					matrix balance[`row'+8,`col' + 2] =	r(df)
					matrix balance[`row'+9,`col' + 2] = e(N)
					
			
			restore
			
			replace var = "[keep_small_cells] Stacked Clustered `Fname' F-test" if _n==`row'
			replace var = "[keep_small_cells] Stacked Clustered `Fname' F-test (pval)" if _n==`row'+1
			replace var = "[keep_small_cells] Stacked Clustered `Fname' F-test (residuals df)" if _n==`row'+2
			replace var = "[keep_small_cells] Stacked Clustered `Fname' F-test (model df)" if _n==`row'+3
			replace var = "[keep_small_cells] Stacked Clustered `Fname' F-test (N)" if _n==`row'+4
			
			replace var = "[keep_small_cells] Stacked Traditional `Fname' F-test" if _n==`row'+5
			replace var = "[keep_small_cells] Stacked Traditional `Fname' F-test (pval)" if _n==`row'+6
			replace var = "[keep_small_cells] Stacked Traditional `Fname' F-test (residuals df)" if _n==`row'+7
			replace var = "[keep_small_cells] Stacked Traditional `Fname' F-test (model df)" if _n==`row'+8
			replace var = "[keep_small_cells] Stacked Traditional `Fname' F-test (N)" if _n==`row'+9
		
		local row = `row' + 10
		}
			
		**********************************************************************************
		*********************** Suest Robust standard errors *****************************
		**********************************************************************************
		
		** F-tests using suest, which allows for robus or clustered SEs **
		* 1) Run models without clustering option
		* 2) Store results
		* 3) suest [stored values], vce(cluster clustvar)
		* 4) test coefficient
						
		local row = `row' + 10
	
		*Balance F
			replace var="Suest Application Balance F" if _n==`row'
			replace var="Suest Application Balance F (pval)" if _n==`row' +1
			replace var="Suest Application Balance F (df)" if _n==`row' +2
			replace var="Suest Application Balance F (N)" if _n==`row' +3
			
			replace var="Suest Baseline Balance F" if _n==`row' +4
			replace var="Suest Baseline Balance F (pval)" if _n==`row' +5
			replace var="Suest Baseline Balance F (df)" if _n==`row' +6
			replace var="Suest Baseline Balance F (N)" if _n==`row' +7
			
			replace var="Suest All Balance F" if _n==`row'+8
			replace var="Suest All Balance F (pval)" if _n==`row' +9
			replace var="Suest All Balance F (df)" if _n==`row' +10
			replace var="Suest All Balance F (N)" if _n==`row' +11
			
			foreach var in `appcov' `cov' `cov2'{
				
				* no pscores
				qui reg `var' Z
				est store `var'
				
				*Pscore saturated
				qui reg `var' Z tag_type_* if in_good_cell==1
				est store S`var'
				
			}
			
				*** Apcov suest ***
					cap  suest 	num_schools num_study rank1_study, vce(robust)
					if !_rc{
						test Z
						matrix balance[`row', `col'+1] = r(chi2)
						matrix balance[`row'+1, `col'+1] = r(p)
						matrix balance[`row'+2, `col'+1] = r(df)
						matrix balance[`row'+3, `col'+1] = e(N)
					}
						
					cap suest 	Snum_schools Snum_study  Srank1_study, vce(robust)
					if !_rc{
						test Z
						matrix balance[`row', `col'+2] = r(chi2)
						matrix balance[`row'+1, `col' +2] = r(p)
						matrix balance[`row'+2, `col'+2] = r(df)
						matrix balance[`row'+3, `col'+2] = e(N)
					}
				** Baseline covs suest ***
					
					local row = `row' + 4
					cap suest	baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
							baseliness_math baseliness_reading baseliness_writing, vce(robust)
					if !_rc{
						test Z
						matrix balance[`row', `col'+1] = r(chi2)
						matrix balance[`row'+1, `col'+1] = r(p)
						matrix balance[`row'+2, `col'+1] = r(df)
						matrix balance[`row'+3, `col'+1] = e(N)
					}
					
					cap suest	Sbaselinestudy Sbaselinefemale Sbaselinehisp Sbaselineblack Sbaselinefoodfred Sbaselinesped Sbaselinebilingual Sbaselinelep Sbaselinegifted ///
							Sbaseliness_math Sbaseliness_reading Sbaseliness_writing, vce(robust)
					if !_rc{
						test Z
						matrix balance[`row', `col'+2] = r(chi2)
						matrix balance[`row'+1, `col' +2] = r(p)
						matrix balance[`row'+2, `col'+2] = r(df)
						matrix balance[`row'+3, `col'+2] = e(N)
					}
					

				*** All covariates suest ***
				
					local row = `row' + 4
					cap suest	num_schools num_study rank1_study ///
							baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
							baseliness_math baseliness_reading baseliness_writing, vce(robust)
					if !_rc{
						test Z
						matrix balance[`row', `col'+1] = r(chi2)
						matrix balance[`row'+1, `col'+1] = r(p)
						matrix balance[`row'+2, `col'+1] = r(df)
						matrix balance[`row'+3, `col'+1] = e(N)
					}
						
					cap suest	Snum_schools Snum_study Srank1_study ///
							Sbaselinestudy Sbaselinefemale Sbaselinehisp Sbaselineblack Sbaselinefoodfred Sbaselinesped Sbaselinebilingual Sbaselinelep Sbaselinegifted ///
							Sbaseliness_math Sbaseliness_reading Sbaseliness_writing, vce(robust)
					if !_rc{
						test Z
						matrix balance[`row', `col'+2] = r(chi2)
						matrix balance[`row'+1, `col' +2] = r(p)
						matrix balance[`row'+2, `col'+2] = r(df)
						matrix balance[`row'+3, `col'+2] = e(N)
					}
											
		local row = `row' + 4
		
		**********************************************************************************

	
		*Outsheet
		preserve
			svmat balance
			keep var balance*
			keep if _n<=`row'+1
			ren balance1 z0
			ren balance2 ols_z1_OLS
			ren balance3 ols_z1_psat
			ren balance4 ols_z1_psat_nofam
			outsheet using "${tables}`cut'_fulltype_balance_years_stacked.csv", comma replace
			
		restore
	}
	
}


	*****************************************
	***************** 2SLS ******************
	*****************************************
	
	if `fulltype_twosls'==1{

	u "${cleandata}analysis_fulltype_stacked.dta", clear
	
	keep if baselinespyear == 2012 | baselinespyear == 2013
	* keep if in_sample_tc == 1
	
		gen var =""
				
		*By grade vars
		gen group = .
				
		local col = 1
		local row = 1
			
			foreach group in 410 /*45 68 48 910*/{
				if `group'==410{
					replace group=410
				}
				else if `group'==48{
					replace group=48 if inlist(appgrade,4,5,6,7,8)
				}
				else{
					replace group=45 if inlist(appgrade,4,5)
					replace group=68 if inlist(appgrade,6,7,8)
					replace group=910 if inlist(appgrade,9,10)
				}
	
				foreach t in math writing reading {
					foreach out in ss /*prof_up adv*/ {
						
								
						replace var="`out'_`t'_`group'" if _n==`row'
						replace var="`out'_`t'_`group'_se" if _n==`row'+1
						replace var="`out'_`t'_`group'_N" if _n==`row'+2
								
			
						qui summ `out'_`t' if group==`group'
								
						if r(max)==0 & r(min)==0{
							di in red "No observations for `group'-level 2SLS estimates"
						}
							
						else{
							
							qui summ `out'_`t' if Z == 0 & group==`group'
							matrix twosls[`row',`col'] = r(mean)
							matrix twosls[`row'+2,`col'] = r(N)
									
							/*OLS - no control for type*/
							qui reg `out'_`t' D `cov' `cov2' `cov_grade' baselinespyear if group==`group', vce(cluster maskedid)
							matrix twosls[`row',`col'+1]   = _b[D]
							matrix twosls[`row'+1,`col'+1] = _se[D]
							matrix twosls[`row'+2,`col'+1] = e(N)
										
							/*OLS conditional on the type sample */
							qui reg `out'_`t' D `cov' `cov2' `cov_grade' baselinespyear if group==`group' & in_good_cell_tc==1, vce(cluster maskedid)
							matrix twosls[`row',`col'+2]   = _b[D]
							matrix twosls[`row'+1,`col'+2] = _se[D]
							matrix twosls[`row'+2,`col'+2] = e(N)
										
							/*OLS conditional controlling for type risksets*/
							qui reg `out'_`t' D tag_type_tc_* `cov' `cov2' `cov_grade' baselinespyear if group==`group' & in_good_cell_tc==1, vce(cluster maskedid)
							matrix twosls[`row',`col'+3]   = _b[D]
							matrix twosls[`row'+1,`col'+3] = _se[D]
							matrix twosls[`row'+2,`col'+3] = e(N)

						
							local col = `col' + 2
										
							local controls "tag_type_tc_* `cov' `cov2' `cov_grade'"		
										
											
							/*2SLS - outcome on treatment instrumented by instrument*/
							ivreg2 `out'_`t' (D = Z) `controls' baselinespyear if group==`group' & in_good_cell_tc==1, savefirst savefprefix(first) partial(`controls' baselinespyear) cluster(maskedid) 
											
							*2SLS
							matrix twosls[`row',`col'+5]   = _b[D]
							matrix twosls[`row'+1,`col'+5] = _se[D]
							matrix twosls[`row'+2,`col'+5] = e(N)
											
							estimates restore firstD
											
							*First stage
							matrix twosls[`row',`col'+4]   = _b[Z]
							matrix twosls[`row'+1,`col'+4] = _se[Z]
											
								*NAIVE 2SLS - no riskset controls
									*2SLS naive
									ivreg2 `out'_`t' (D = Z) `cov' `cov2' `cov_grade' baselinespyear if group==`group', savefirst savefprefix(first) partial(`cov' `cov2' `cov_grade' baselinespyear) cluster(maskedid) 

									*2SLS
									matrix twosls[`row',`col'+7]   = _b[D]
									matrix twosls[`row'+1,`col'+7] = _se[D]
									matrix twosls[`row'+2,`col'+7] = e(N)
													
									estimates restore firstD
											
									*First stage
									matrix twosls[`row',`col'+6]   = _b[Z]
									matrix twosls[`row'+1,`col'+6] = _se[Z]
													
									*2SLS naive conditional on r0 sample
									ivreg2 `out'_`t' (D = Z) `cov' `cov2' `cov_grade' baselinespyear if group==`group' & in_good_cell_tc==1, savefirst savefprefix(first) partial(`cov' `cov2' `cov_grade' baselinespyear) cluster(maskedid) 

									*2SLS
									matrix twosls[`row',`col'+9]   = _b[D]
									matrix twosls[`row'+1,`col'+9] = _se[D]
									matrix twosls[`row'+2,`col'+9] = e(N)
													
									estimates restore firstD
											
									*First stage
									matrix twosls[`row',`col'+8]   = _b[Z]
									matrix twosls[`row'+1,`col'+8] = _se[Z]													
											
									*2SLS with no cov controls								
									ivreg2 `out'_`t' (D = Z) baselinespyear `cov_grade' tag_type_tc_* if group==`group' & in_good_cell_tc==1, savefirst savefprefix(first) partial(tag_type_tc_* baselinespyear `cov_grade') cluster(maskedid) 
													
									*2SLS
									matrix twosls[`row',`col'+11]   = _b[D]
									matrix twosls[`row'+1,`col'+11] = _se[D]
									matrix twosls[`row'+2,`col'+11] = e(N)
													
									estimates restore firstD
											
									*First stage
									matrix twosls[`row',`col'+10]   = _b[Z]
									matrix twosls[`row'+1,`col'+10] = _se[Z]									
							
							} /*close condition on number of observations*/
							
							local row = `row' + 3
							local col = 1
						} 	 /*out loop*/
						
						local ++row
					}	/* t loop*/
					
				}	/*group loop*/
	
				preserve
					svmat twosls
					keep var twosls*
					keep if _n<=`row'
					ren twosls1 z0
					ren twosls2 OLS
					ren twosls3 OLS_typesample
					ren twosls4 OLS_typesat
					
					ren twosls7  FS_type
					ren twosls8  twoSLS_type
					
					ren twosls9 FS_naive
					ren twosls10 twoSLS_naive
					
					ren twosls11 FS_naive_typesample
					ren twosls12 twoSLS_naive_typesample
					
					ren twosls13 FS_nocovs
					ren twosls14 twoSLS_nocovs

					outsheet using "${tables}fulltype_years_stacked_twosls.csv", comma replace
				restore
			clear	

			} /*Close 2SLS condition*/	


