/*

this file needs the stacked analysis file made from years 2013 and 2014. 
it outputs files for frequency, simulated and formula. 
Columns and rows to lookup
Columns: 
z0	ols_z1		ols_z1_bin01	ols_z1_bin001		ols_z1_psat	ols_z1_psat	ols_z1_psat
Rows : 
num_schools
num_schools_se
num_study
num_study_se
rank1_study
rank1_study_se


baselinestudy
baselinestudy_se
baselinefemale
baselinefemale_se
baselinehisp
baselinehisp_se
baselineblack
baselineblack_se
baselinefoodfred
baselinefoodfred_se
baselinelep
baselinelep_se
baselinesped
baselinesped_se
baselinefemale_N(var in Z)

baseliness_math
baseliness_math_se
baseliness_reading
baseliness_reading_se
baseliness_writing
baseliness_writing_se
baseliness_math_N(var in Z)

riskset points of support

Traditional All Balance F-test
Traditional All Balance F-test (pval)
*/



clear all
cap log close
set more off
set trace off
set tracedepth 1 
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"
pause on

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"
local rawdata "Y:\DPS\Raw Data\"

matrix demo			  = J(30,30,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,40,.)
matrix subtwosls	  = J(500,40,.)
matrix twosls_nocov    		   = J(500,40,.)
matrix twosls_scorecov 		   = J(500,40,.)
matrix twosls_fcsample   		= J(50	0,40,.)

matrix qualbalance    = J(500,20,.)
matrix qualtwosls	  = J(500,20,.)
matrix qualsubtwosls  = J(500,20,.)

*Dates of analysis files
local pscoredate_year2	20170310 	/*20160516 20160510 20160428 20160411 20160208 20160112 20151120 20150514 */
local pscoredate_year1  20170310 /*20160516    	20160516 20160510 20160421  20160208 */
local simsdate 			20150603 /* 20150428 */
local qualdate_year1 	20160715 /*20150708  20150420 */
local qualdate_year2	20160715 /*20160303*/
local qualdate			20160303
local qualpscoredate	20160208
local appdatayr1		20170310
local appdatayr2		20170310


local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_*"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1        		/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
local outsheet_rescorr=0	/* If want to outsheet matrix with residual correlations */


	
	foreach file in formula  /* simulation frequency */    {
		foreach sch_type in study {

			foreach cut in inDPSbase {
			
				u "`cleandata'analysis_stacked_`sch_type'_`file'.dta", clear

				if "`cut'"=="inDPSbase"{
					keep if baselinespyear==2012 | baselinespyear == 2013
				}
				else{
					di in red "Keep all applicants"
				}
				
				**********************************************************************************
				************************ Balance and attrition coefficients **********************
				**********************************************************************************
				
				local row = 1
				local col = 1
				gen var =""
				
				*Count points in the support
				replace var = "Riskset points of support" if _n ==1
							
				*Place it on top of balance table
				
				unique pscore1 		if in_zero1_pscore==1
				matrix balance[1,3] = r(sum)
			
				unique r01			if in_good_cell_r01==1
				matrix balance[1,5] = r(sum)
				
				unique r001			if in_good_cell_r001==1
				matrix balance[1,6] = r(sum)
				
				unique r0			if in_good_cell_r0==1
				matrix balance[1,7] = r(sum)

				
				local row = `row' + 1
				
				foreach var in `attrition_vars' `appcov' `cov' `cov2' {
				
					replace var="`var'" if _n==`row'
					replace var="`var'_se" if _n==`row'+1
					replace var="`var'_N" if _n==`row'+2
					replace var="`var'_N(var in Z)" if _n==`row'+3
					replace var="`var'_N(var in Z & X)" if _n==`row'+4
						
					*All non-offered applicants
					qui summ `var' if Z==0
						matrix balance[`row',`col']   = r(mean)
						matrix balance[`row'+1,`col'] = r(sd)
						matrix balance[`row'+2,`col'] = r(N)

					*Balance and attrition, no control(OLS)
					reg `var' Z i.baselinespyear, vce(robust)
						matrix balance[`row',`col'+1]   = _b[Z]
						matrix balance[`row'+1,`col'+1] = _se[Z]
						matrix balance[`row'+2,`col'+1] = e(N)
					
					*Balance and attrition, pscore linear control(OLS)
					qui reg `var' Z pscore1 i.baselinespyear if !inlist(r0,1,0) , vce(robust)
						matrix balance[`row',`col'+2]   = _b[Z]
						matrix balance[`row'+1,`col'+2] = _se[Z]
						matrix balance[`row'+2,`col'+2] = e(N)
						
					*Balance and attrition, pscore polynomial control(OLS)
					qui reg `var' Z i.baselinespyear pscore1 pscore2 pscore3 pscore4 if !inlist(r0,1,0), vce(robust)
						matrix balance[`row',`col'+3]   = _b[Z]
						matrix balance[`row'+1,`col'+3] = _se[Z]
						matrix balance[`row'+2,`col'+3] = e(N)
							
					*Balance and attrition, risks
					* IR 04152016 now includes 1000's
							
							
					local c = 0
					foreach i in 01 001{
						local c = `c' +1
							
						preserve
							
							areg `var' Z i.baselinespyear  if !inlist(r`i',1,0), absorb(r`i') vce(robust)
							areg `var' Z i.baselinespyear  if in_zero1_pscore == 1, absorb(r`i') vce(robust)
							
							matrix balance[`row',`col'+3 + `c']   = _b[Z]
							matrix balance[`row'+1,`col'+3 + `c'] = _se[Z]
							matrix balance[`row'+2,`col'+3 + `c'] = e(N)
							
							bys r`i': egen sd = sd(`var')
							bys r`i': egen sd_z = sd(Z)
							
							*Count kids with variation in instrument within cell (no ref to characteristic)
							count if sd_z > 0 & !missing(sd_z)  & e(sample)
							matrix balance[`row'+3,`col'+3 + `c'] = r(N)
							
							*Count kids with variation in characteristic & instrument at cell level
							count if sd>0 & sd_z >0 & !missing(sd)  & !missing(sd_z)  & e(sample)
							matrix balance[`row'+4,`col'+3 + `c'] = r(N)
							
							drop sd sd_z		
							
						restore
					}
					
					
					*Fully saturated in pscore
						preserve
						
							qui areg `var' Z i.baselinespyear if !inlist(r0,1,0) , absorb(r0) vce(robust)
							matrix balance[`row',`col'+4 + `c']   = _b[Z]
							matrix balance[`row'+1,`col'+4 + `c'] = _se[Z]
							matrix balance[`row'+2,`col'+4 + `c'] = e(N)
							
							bys r0: egen sd = sd(`var')
							bys r0: egen sd_z = sd(Z)

							*Count kids with variation in instrument within cell (no ref to characteristic)
							count if sd_z > 0 & !missing(sd_z) & e(sample)
							matrix balance[`row'+3,`col'+4 + `c'] = r(N)
							
							*Count kids with variation in characteristic & instrument at cell level
							count if sd>0 & sd_z >0 & !missing(sd)  & !missing(sd_z)  & e(sample)
							matrix balance[`row'+4,`col'+4 + `c'] = r(N)
							
							drop sd sd_z
							
						restore
					
				local row = `row' + 6
				}
				
			************************************* F tests **************************************	
				*MS 2/23/2015: 
				*Minimum number of observations neeed for the SUEST F test to run according to each pscore type
				*Arrived at these numbers by running F tests for several different minimum number of obs. These
				*are the minimum needed for the set of covariates being tested as of 2/23/2015.
				
				local min_obs_cell_simulation 	= 9			
				local min_obs_cell_formula 		= 9
				local min_obs_cell_frequency 	= 9

				
			if `ftest'==1{
				
				**********************************************************************************
				************************************* Logit Fs ***********************************
				**********************************************************************************
				
				foreach type in robust traditional{
				
						foreach covs in apponly balanceonly all{
						
							if "`covs'"=="apponly"{
								local balF "`appcov'"
								local Fname "Application Balance"
							}
							
							else if "`covs'"=="balanceonly"{
								local balF "`cov' `cov2'"
								local Fname "Baseline Balance"
							}
							else if "`covs'"=="all"{
								local balF "`appcov' `cov' `cov2'"
								local Fname "All Balance"
							}
							
							local row = `row' + 3
							
							if "`type'"=="robust"{
								local option "nocnsreport vce(robust)"
							}
							else{
								local option "nocnsreport"
							}
							
							* Balance logit
							replace var="Logit `type' `Fname' F (chi2)" if _n==`row'
							replace var="Logit `type' `Fname' F (pval)" if _n==`row'+1
							replace var="Logit `type' `Fname' F (df)" if _n==`row'+2
							replace var="Logit `type' `Fname' F (N)" if _n==`row'+3
						
							*No riskset
							qui logit Z i.baselinespyear `balF', `option'
							test `balF'
							matrix balance[`row', `col'+1] = r(chi2)
							matrix balance[`row'+1, `col'+1] = r(p)
							matrix balance[`row'+2, `col'+1] = r(df)
							matrix balance[`row'+3, `col'+1] = e(N)	
		
							*Linear riskset
							qui logit Z i.baselinespyear pscore1 `balF' if in_zero1_pscore==1, `option'
							test `balF'
							matrix balance[`row', `col'+2] = r(chi2)
							matrix balance[`row'+1, `col'+2] = r(p)
							matrix balance[`row'+2, `col'+2] = r(df)
							matrix balance[`row'+3, `col'+2] = e(N)
							
							*Poly riskset
							qui logit Z i.baselinespyear pscore* `balF' if in_zero1_pscore==1, `option'
							test `balF'
							matrix balance[`row', `col'+3] = r(chi2)
							matrix balance[`row'+1, `col'+3] = r(p)
							matrix balance[`row'+2, `col'+3] = r(df)
							matrix balance[`row'+3, `col'+3] = e(N)
							
							*Bins
							local c=0
							foreach i in 01 001{
								local c = `c' + 1
								qui logit Z i.baselinespyear i.ind_r`i' `balF' if in_good_cell_r`i'==1, `option'
								test `balF'
								matrix balance[`row', `col' +3 + `c'] = r(chi2)
								matrix balance[`row' +1, `col' +3 + `c'] = r(p)
								matrix balance[`row'+2, `col' +3 + `c'] = r(df)
								matrix balance[`row'+3, `col' +3 + `c'] = e(N)
							}
							
							*Fully saturated
							qui logit Z i.baselinespyear i.ind_r0 `balF' if in_good_cell_r0==1, `option'
							test `balF'
							matrix balance[`row', `col' +4 + `c'] = r(chi2)
							matrix balance[`row' +1,`col' +4 + `c'] = r(p)
							matrix balance[`row'+2, `col' +4 + `c'] = r(df)
							matrix balance[`row'+3, `col' +4 + `c'] = e(N)
					
						local row = `row' + 4
					}	/*Close balance covariates loop*/

			} /*CLose robust/traditional F*/
				
				local row = `row' + 4
				********************************************************
				*************** Balance Traditional F ******************
				********************************************************
				
				foreach covs in apponly balanceonly all{
						
					if "`covs'"=="apponly"{
						local balF "`appcov'"
						local Fname "Application Balance"
					}
					
					else if "`covs'"=="balanceonly"{
						local balF "`cov' `cov2'"
						local Fname "Baseline Balance"
					}
					else if "`covs'"=="all"{
						local balF "`appcov' `cov' `cov2'"
						local Fname "All Balance"
					}
							
					replace var="Traditional `Fname' F-test" if _n==`row'
					replace var="Traditional `Fname' F-test (pval)" if _n==`row' +1
					replace var="Traditional `Fname' F-test (residuals df)" if _n==`row' +2
					replace var="Traditional `Fname' F-test (model df)" if _n==`row' +3
					replace var="Traditional `Fname' F-test (N)" if _n==`row' +4
					replace var="Traditional `Fname' F-test (Breusch-Pagan chi2)" if _n==`row'+5
					
					*no propensity score
						mvreg `balF' = Z i.baselinespyear , corr
						test Z
						matrix balance[`row',`col' + 1] = r(F)
						matrix balance[`row'+1,`col' + 1]= r(p)
						matrix balance[`row'+2, `col'+1] = r(df_r)
						matrix balance[`row'+3, `col'+1] = r(df)
						matrix balance[`row'+4, `col'+1] = e(N)
						matrix balance[`row'+5, `col'+1] = 1- chi2(e(df_chi2),e(chi2))
					
					*linear pscore control
						mvreg `balF' = Z i.baselinespyear pscore1 if !inlist(r0,1,0), corr
						test Z
						matrix balance[`row',`col' +2]=r(F)
						matrix balance[`row'+1,`col' +2]=r(p)
						matrix balance[`row'+2, `col'+2] = r(df_r)
						matrix balance[`row'+3, `col'+2] = r(df)
						matrix balance[`row'+4, `col'+2] = e(N)
						matrix balance[`row'+5, `col'+2] = 1- chi2(e(df_chi2),e(chi2))
						
					*polynomial pscore control
						mvreg `balF' = Z i.baselinespyear pscore1 pscore2 pscore3 pscore4 if !inlist(r0,1,0), corr
						test Z
						matrix balance[`row',`col' + 3]=r(F)
						matrix balance[`row'+1,`col'+ 3]=r(p)
						matrix balance[`row'+2, `col'+3] = r(df_r)
						matrix balance[`row'+3, `col'+3] = r(df)
						matrix balance[`row'+4, `col'+3] = e(N)
						matrix balance[`row'+5, `col'+3] = 1- chi2(e(df_chi2),e(chi2))
						
					*Bins
						local c=0
						foreach i in 01 001{
							local c = `c' + 1
							mvreg `balF' = Z i.baselinespyear i.ind_r`i' if  !inlist(r`i',1,0), corr
							test Z
							matrix balance[`row',`col' + 3 + `c'] = r(F)
							matrix balance[`row'+1,`col' + 3 + `c'] = r(p)
							matrix balance[`row'+2,`col' + 3 + `c'] = r(df_r)
							matrix balance[`row'+3,`col' + 3 + `c'] = r(df)
							matrix balance[`row'+4,`col' + 3 + `c'] = e(N)
							matrix balance[`row'+5,`col' + 3 + `c'] = 1- chi2(e(df_chi2),e(chi2))
							
							matrix B_ols_z1_bin`i'_rescorr = corr(e(Sigma))
						}
					
						*Pscore saturated
						mvreg `balF' = Z i.baselinespyear i.ind_r0  if in_good_cell_r0==1, corr
						test Z
						matrix balance[`row',`col' + 4 + `c'] = r(F)
						matrix balance[`row'+1,`col' + 4 + `c'] = r(p)
						matrix balance[`row'+2,`col' + 4 + `c'] = r(df_r)
						matrix balance[`row'+3,`col' + 4 + `c'] = r(df)
						matrix balance[`row'+4,`col' + 4 + `c'] = e(N)
						matrix balance[`row'+5,`col' + 4 + `c'] = 1- chi2(e(df_chi2),e(chi2))
						
					local row = `row' + 6
				}	/*Close covariates loop*/
				
			
				
				**********************************************************************************
				*************************** Balance Stacked Robust F *****************************
				**********************************************************************************
				
				*Run a cut keeping cells with small size and one dropping cells with small size
				foreach f_cut in keep_small_cells drop_small_cells{
					
					foreach covs in apponly balanceonly all{
						
						if "`covs'"=="apponly"{
							local balF "`appcov'"
							local Fname "Application Balance"
							local expand = 3
							local test "Z_1 Z_2 Z_3"
						}
						
						else if "`covs'"=="balanceonly"{
							local balF "`cov' `cov2'"
							local Fname "Baseline Balance"
							local expand = 12
							local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12"
						}
						else if "`covs'"=="all"{
							local balF "`appcov' `cov' `cov2'"
							local Fname "All Balance"
							local expand = 15
							local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12 Z_13 Z_14 Z_15"
						}
					
						preserve
							
							if "`f_cut'" == "keep_small_cells"{
								local min_obs = 1
							}
							else if "`f_cut'"=="drop_small_cells"{
								local min_obs = `min_obs_cell_`file''
							}
							
							keep maskedid `balF' Z D tag_r01 tag_r0 tag_r001 r01 r001 r0 pscore1 pscore2 pscore3 pscore4 family in_sample in_good_cell_* obs_cell_* in_zero1_pscore baselinespyear

							expand `expand'						

							bys maskedid baselinespyear: gen copy = _n
							sort copy
						
							* if kids repeat years you get a larger max value of copy, which will break 
							* the logic used for the tokenize chunk below. 
							su copy 
							if `r(max)' !=  `expand' {
								di as error " Somethign is off when you make a copy of the data. You need to end up with only `expand' copies per kid. Rn you have `r(max)'"
								exit  
								
							}
																		
							*Single vector of outcomes
							gen cov = .
							gen covname = "" 
	
							tokenize `balF'


							forval i=1/`expand'{

								replace cov = ``i'' if copy==`i' 
								replace covname = "``i''" if copy==`i' 
								
							}					
							
							drop `balF'
							
							

							
							*Instruments, pscore variables, and dummies for each copy
							forval i=1/`expand'{
								gen Z_`i' = Z*(copy == `i')
								gen alpha_`i' = (copy == `i')
								
								*Pscore variables for each copy
								foreach x in r01 r0 r001 pscore1 pscore2 pscore3 pscore4 baselinespyear{
									gen c_`x'_`i'= `x'*(copy == `i')
								}
							}
							

							
							*Absorb takes only one var. Create grouped vars
							foreach x in tag_r01 tag_r0 tag_r001 r001 r01 r0 {
									egen group_`x' = group(copy `x')
								
							}
							
							*Replace with missing if first categoryobs_
							foreach x in tag_r001 tag_r01 tag_r0{
								replace group_`x' =. if `x'==-9999999
							
							}
							*** One tall regression for each specification ***BREAK

							
							*No controls				
								*Clustered

								
								reg cov alpha_* Z_* i.c_baselinespyear*, vce(cluster maskedid)
								test `test'
								
								matrix balance[`row',`col' + 1]=r(F)
								matrix balance[`row'+1,`col' +1]=r(p)
								matrix balance[`row'+2,`col' +1]=r(df_r)
								matrix balance[`row'+3,`col' +1]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+4,`col' + 1]=r(sum)
							
								
								*Traditional
								qui reg cov alpha_* Z_*  i.c_baselinespyear*
								test `test'
								
								matrix balance[`row'+6,`col' + 1]=r(F)
								matrix balance[`row'+7,`col' +1]=r(p)
								matrix balance[`row'+8,`col' +1]=r(df_r)
								matrix balance[`row'+9,`col' +1]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+10,`col' + 1]=r(sum)
								
								
								*Robust
								qui reg cov alpha_* Z_* i.c_baselinespyear*, vce(cluster maskedid)
								test `test'
								
								matrix balance[`row'+12,`col' + 1]=r(F)
								matrix balance[`row'+13,`col' +1]=r(p)
								matrix balance[`row'+14,`col' +1]=r(df_r)
								matrix balance[`row'+15,`col' +1]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+16,`col' + 1]=r(sum)
								
								

							*Linear
								*Cluster
								qui reg cov alpha_* Z_* i.c_baselinespyear* c_pscore1_* if in_zero1_pscore==1, vce(cluster maskedid)
								test `test'
								
								matrix balance[`row',`col' + 2]=r(F)
								matrix balance[`row'+1,`col' +2]=r(p)
								matrix balance[`row'+2,`col' +2]=r(df_r)
								matrix balance[`row'+3,`col' +2]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+4,`col' + 2]=r(sum)
								
								*Traditional
								qui reg cov alpha_* Z_* i.c_baselinespyear* c_pscore1_* if in_zero1_pscore==1
								test `test'
								
								matrix balance[`row'+6,`col' + 2]=r(F)
								matrix balance[`row'+7,`col' +2]=r(p)
								matrix balance[`row'+8,`col' +2]=r(df_r)
								matrix balance[`row'+9,`col' +2]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+10,`col' + 2]=r(sum)
								
								*Robust
								qui reg cov alpha_* Z_* c_pscore1_* i.c_baselinespyear* if in_zero1_pscore==1, vce(cluster maskedid)
								test `test'
								
								matrix balance[`row'+12,`col' + 2]=r(F)
								matrix balance[`row'+13,`col' +2]=r(p)
								matrix balance[`row'+14,`col' +2]=r(df_r)
								matrix balance[`row'+15,`col' +2]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+16,`col' + 2]=r(sum)
							
							*Polynomial
								*Cluster
								qui reg cov alpha_* Z_* i.c_baselinespyear* c_pscore1_* c_pscore2_* c_pscore3_* c_pscore4_* if in_zero1_pscore==1, vce(cluster maskedid)
								test `test'
								
								matrix balance[`row',`col' + 3]=r(F)
								matrix balance[`row'+1,`col' +3]=r(p)
								matrix balance[`row'+2,`col' +3]=r(df_r)
								matrix balance[`row'+3,`col' +3]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+4,`col' + 3]=r(sum)
								
								*Traditional
								qui reg cov alpha_* Z_* i.c_baselinespyear* c_pscore1_* c_pscore2_* c_pscore3_* c_pscore4_* if in_zero1_pscore==1
								test `test'
								
								matrix balance[`row'+6,`col' + 3]=r(F)
								matrix balance[`row'+7,`col' +3]=r(p)
								matrix balance[`row'+8,`col' +3]=r(df_r)
								matrix balance[`row'+9,`col' +3]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+10,`col' + 3]=r(sum)
								
								*Robust
								qui reg cov alpha_* Z_* i.c_baselinespyear* c_r0_* c_pscore2_* c_pscore3_* c_pscore4_* if in_zero1_pscore==1, vce(cluster maskedid)
								test `test'
								
								matrix balance[`row'+12,`col' + 3]=r(F)
								matrix balance[`row'+13,`col' +3]=r(p)
								matrix balance[`row'+14,`col' +3]=r(df_r)
								matrix balance[`row'+15,`col' +3]=r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+16,`col' + 3]=r(sum)
								
							*Bins
								local c = 0
								foreach i in 01 001{
									local c = `c' +1
									
									*Cluster
									qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_r`i'==1 & obs_cell_r`i' >= `min_obs' & !missing(obs_cell_r`i'), absorb(group_tag_r`i') vce(cluster maskedid)
									test `test'
									
									matrix balance[`row',`col'+3 + `c']   = r(F)
									matrix balance[`row'+1,`col'+3 + `c'] = r(p)
									matrix balance[`row'+2,`col'+3 + `c'] = r(df_r)
									matrix balance[`row'+3,`col'+3 + `c'] = r(df)
									
									unique maskedid if e(sample)
									matrix balance[`row'+4,`col'+3 + `c'] = r(sum)
									
									unique group_tag_r`i' if e(sample)
									matrix balance[`row'+5,`col'+3 + `c'] = r(sum) / `expand'
									
									*Traditional
									qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_r`i'==1 & obs_cell_r`i' >= `min_obs' & !missing(obs_cell_r`i'), absorb(group_tag_r`i')
									test `test'
									
									matrix balance[`row'+6,`col'+3 + `c'] = r(F)
									matrix balance[`row'+7,`col'+3 + `c'] = r(p)
									matrix balance[`row'+8,`col'+3 + `c'] = r(df_r)
									matrix balance[`row'+9,`col'+3 + `c'] = r(df)
									
									unique maskedid if e(sample)
									matrix balance[`row'+10,`col'+3 + `c'] = r(sum)
									
									unique group_tag_r`i' if e(sample)
									matrix balance[`row'+11,`col'+3 + `c'] = r(sum) / `expand'
									
									*Robust
									qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_r`i'==1 & obs_cell_r`i' >= `min_obs' & !missing(obs_cell_r`i'), absorb(group_tag_r`i') vce(robust)
									test `test'
									
									matrix balance[`row'+12,`col'+3 + `c'] = r(F)
									matrix balance[`row'+13,`col'+3 + `c'] = r(p)
									matrix balance[`row'+14,`col'+3 + `c'] = r(df_r)
									matrix balance[`row'+15,`col'+3 + `c'] = r(df)
									
									unique maskedid if e(sample)
									matrix balance[`row'+16,`col'+3 + `c'] = r(sum)
									
									unique group_tag_r`i' if e(sample)
									matrix balance[`row'+17,`col'+3 + `c'] = r(sum) / `expand'
								}

							*Saturated
								*Cluster
								qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_r0==1 & obs_cell_r0 >= `min_obs' & !missing(obs_cell_r0), absorb(group_tag_r0) vce(cluster maskedid)
								test `test'
								
								matrix balance[`row',`col' + 4 + `c']	=	r(F)
								matrix balance[`row'+1,`col' + 4 + `c']	=	r(p)
								matrix balance[`row'+2,`col' + 4 + `c']	=	r(df_r)
								matrix balance[`row'+3,`col' + 4 + `c'] =	r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+4,`col' + 4 + `c'] = r(sum)
								
								unique group_tag_r0 if e(sample)
								matrix balance[`row'+5,`col' + 4 + `c'] = r(sum) / `expand'
								
								*Traditional
								qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_r0==1 & obs_cell_r0 >= `min_obs' & !missing(obs_cell_r0), absorb(group_tag_r0)
								test `test'
								
								matrix balance[`row'+6,`col' + 4 + `c']	=	r(F)
								matrix balance[`row'+7,`col' + 4 + `c']	=	r(p)
								matrix balance[`row'+8,`col' + 4 + `c']	=	r(df_r)
								matrix balance[`row'+9,`col' + 4 + `c'] =	r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+10,`col' + 4 + `c'] = r(sum)
								
								unique group_tag_r0 if e(sample)
								matrix balance[`row'+11,`col' + 4 + `c'] = r(sum) / `expand'
								
								*Robust
								qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_r0==1 & obs_cell_r0 >= `min_obs' & !missing(obs_cell_r0), absorb(group_tag_r0) vce(robust)
								test `test'
								
								matrix balance[`row'+12,`col' + 4 + `c'] =	r(F)
								matrix balance[`row'+13,`col' + 4 + `c'] =	r(p)
								matrix balance[`row'+14,`col' + 4 + `c'] =	r(df_r)
								matrix balance[`row'+15,`col' + 4 + `c'] =	r(df)
								
								unique maskedid if e(sample)
								matrix balance[`row'+16,`col' + 4 + `c'] = r(sum)
								
								unique group_tag_r0 if e(sample)
								matrix balance[`row'+17,`col' + 4 + `c'] = r(sum) / `expand'
						restore
						
						replace var = "[`f_cut'] Stacked Clustered `Fname' F-test" if _n==`row'
						replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (pval)" if _n==`row'+1
						replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (residuals df)" if _n==`row'+2
						replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (model df)" if _n==`row'+3
						replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (N)" if _n==`row'+4
						replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (cells)" if _n==`row'+5
						
						replace var = "[`f_cut'] Stacked Traditional `Fname' F-test" if _n==`row'+6
						replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (pval)" if _n==`row'+7
						replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (residuals df)" if _n==`row'+8
						replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (model df)" if _n==`row'+9
						replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (N)" if _n==`row'+10
						replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (cells)" if _n==`row'+11
						
						replace var = "[`f_cut'] Stacked Robust `Fname' F-test" if _n==`row' +12
						replace var = "[`f_cut'] Stacked Robust `Fname' F-test (pval)" if _n==`row'+13
						replace var = "[`f_cut'] Stacked Robust `Fname' F-test (residuals df)" if _n==`row'+14
						replace var = "[`f_cut'] Stacked Robust `Fname' F-test (model df)" if _n==`row'+15
						replace var = "[`f_cut'] Stacked Robust `Fname' F-test (N)" if _n==`row'+16
						replace var = "[`f_cut'] Stacked Robust `Fname' F-test (cells)" if _n==`row'+17
						
						
						local row = `row' + 18
				}	
				
				**********************************************************************************
				*********************** Suest Robust standard errors *****************************
				**********************************************************************************
				
				** F-tests using suest, which allows for robust or clustered SEs **
				* 1) Run models without clustering option
				* 2) Store results
				* 3) suest [stored values], vce(cluster clustvar)
				* 4) test coefficient
				
				
				foreach f_cut in keep_small_cells drop_small_cells{
				
						if "`f_cut'" === "keep_small_cells"{
							local min_obs = 1
						}
						else if "`f_cut'"=="drop_small_cells"{
							local min_obs = `min_obs_cell_`file''
						}
						

					*Balance F
						replace var="[`f_cut'] Suest Application Balance F" if _n==`row'
						replace var="[`f_cut'] Suest Application Balance F (pval)" if _n==`row' +1
						replace var="[`f_cut'] Suest Application Balance F (df)" if _n==`row' +2
						replace var="[`f_cut'] Suest Application Balance F (N)" if _n==`row' +3
						replace var="[`f_cut'] Suest Application Balance F (cells)" if _n==`row' +4
						
						replace var="[`f_cut'] Suest Baseline Balance F" if _n==`row' + 5
						replace var="[`f_cut'] Suest Baseline Balance F (pval)" if _n==`row' +6
						replace var="[`f_cut'] Suest Baseline Balance F (df)" if _n==`row' +7
						replace var="[`f_cut'] Suest Baseline Balance F (N)" if _n==`row' +8
						replace var="[`f_cut'] Suest Baseline Balance F (cells)" if _n==`row' +9
						
						replace var="[`f_cut'] Suest All Balance F" if _n==`row' + 10
						replace var="[`f_cut'] Suest All Balance F (pval)" if _n==`row' +11
						replace var="[`f_cut'] Suest All Balance F (df)" if _n==`row' +12
						replace var="[`f_cut'] Suest All Balance F (N)" if _n==`row' +13
						replace var="[`f_cut'] Suest All Balance F (cells)" if _n==`row' +14
						
						foreach var in `appcov' `cov' `cov2'{
							
							* no pscores
							qui reg `var' Z  i.baselinespyear
							est store `var'
							
							* linear
							qui reg `var' Z i.baselinespyear pscore1 if in_zero1_pscore==1
							est store L`var'
							
							* poly
							qui reg `var' Z i.baselinespyear pscore1 pscore2 pscore3 pscore4 if in_zero1_pscore==1
							est store P`var'
							
							* Pscore bins
							foreach i in 01 001{
								qui reg `var' Z i.baselinespyear i.ind_r`i' 	if in_good_cell_r`i'==1 & obs_cell_r`i'>=`min_obs' & !missing(obs_cell_r`i')
								est store B`i'`var'
							}
							
							*Pscore saturated
							qui reg `var' Z i.baselinespyear i.ind_r0 if in_good_cell_r0==1 & obs_cell_r0>= `min_obs' & !missing(obs_cell_r0) 
							est store S`var'
						}
						
						*** Apcov suest ***
							suest 	num_schools num_study rank1_study, vce(robust)
							test Z
							matrix balance[`row', `col'+1] = r(chi2)
							matrix balance[`row'+1, `col'+1] = r(p)
							matrix balance[`row'+2, `col'+1] = r(df)
							matrix balance[`row'+3, `col'+1] = e(N)
							
								
							suest 	Lnum_schools Lnum_study  Lrank1_study, vce(robust)
							test Z
							matrix balance[`row', `col'+2] = r(chi2)
							matrix balance[`row'+1, `col' +2] = r(p)
							matrix balance[`row'+2, `col'+2] = r(df)
							matrix balance[`row'+3, `col'+2] = e(N)
							
							suest 	Pnum_schools Pnum_study Prank1_study, vce(robust)
							test Z
							matrix balance[`row', `col'+3] = r(chi2)
							matrix balance[`row'+1, `col' +3] = r(p)
							matrix balance[`row'+2, `col'+3] = r(df)
							matrix balance[`row'+3, `col'+3] = e(N)
							
							local c=0
							foreach i in 01 001{
								local c = `c' + 1
								suest 	B`i'num_schools B`i'num_study  B`i'rank1_study, vce(robust)
								test Z
								matrix balance[`row', `col' +3 + `c'] = r(chi2)
								matrix balance[`row'+1, `col' +3 + `c'] = r(p)
								matrix balance[`row'+2, `col' +3 + `c'] = r(df)
								matrix balance[`row'+3, `col' +3 + `c'] = e(N)
								
								unique tag_r`i' if e(sample)
								matrix balance[`row'+4, `col' +3 + `c'] = r(sum)
							}
								suest 	Snum_schools Snum_study Srank1_study, vce(robust)
								test Z
								matrix balance[`row',`col'+ 4 + `c'] = r(chi2)
								matrix balance[`row'+1, `col'+ 4 + `c'] = r(p)
								matrix balance[`row'+2, `col'+ 4 + `c'] = r(df)
								matrix balance[`row'+3, `col' +4 + `c'] = e(N)
								
								unique tag_r0 if e(sample)
								matrix balance[`row'+4, `col' +4 + `c'] = r(sum)

						** Baseline covs suest ***
						
						local row = `row' + 5
						suest	baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
								baseliness_math baseliness_reading baseliness_writing, vce(robust)
						test Z
						matrix balance[`row', `col'+1] = r(chi2)
						matrix balance[`row'+1, `col'+1] = r(p)
						matrix balance[`row'+2, `col'+1] = r(df)
						matrix balance[`row'+3, `col'+1] = e(N)
							
						suest	Lbaselinestudy Lbaselinefemale Lbaselinehisp Lbaselineblack Lbaselinefoodfred Lbaselinesped Lbaselinebilingual Lbaselinelep Lbaselinegifted ///
								Lbaseliness_math Lbaseliness_reading Lbaseliness_writing, vce(robust)
						test Z
						matrix balance[`row', `col'+2] = r(chi2)
						matrix balance[`row'+1, `col' +2] = r(p)
						matrix balance[`row'+2, `col'+2] = r(df)
						matrix balance[`row'+3, `col'+2] = e(N)
						
						suest	Pbaselinestudy Pbaselinefemale Pbaselinehisp Pbaselineblack Pbaselinefoodfred Pbaselinesped Pbaselinelep Pbaselinebilingual Pbaselinegifted ///
								Pbaseliness_math Pbaseliness_reading Pbaseliness_writing, vce(robust)
						test Z
						matrix balance[`row', `col'+3] = r(chi2)
						matrix balance[`row'+1, `col' +3] = r(p)
						matrix balance[`row'+2, `col'+3] = r(df)
						matrix balance[`row'+3, `col'+3] = e(N)
						
						local c=0
						foreach i in 01 001{
							local c = `c' + 1
							suest	B`i'baselinestudy B`i'baselinefemale B`i'baselinehisp B`i'baselineblack B`i'baselinefoodfred B`i'baselinesped B`i'baselinelep B`i'baselinebilingual B`i'baselinegifted ///
									B`i'baseliness_math B`i'baseliness_reading B`i'baseliness_writing, vce(robust)
							test Z
							matrix balance[`row', `col' +3 + `c'] = r(chi2)
							matrix balance[`row'+1, `col' +3 + `c'] = r(p)
							matrix balance[`row'+2, `col' +3 + `c'] = r(df)
							matrix balance[`row'+3, `col' +3 + `c'] = e(N)
							
							unique tag_r`i' if e(sample)
							matrix balance[`row'+4, `col' +3 + `c'] = r(sum)
						}
						
							suest	Sbaselinestudy Sbaselinefemale Sbaselinehisp Sbaselineblack Sbaselinefoodfred Sbaselinesped Sbaselinelep Sbaselinebilingual Sbaselinegifted ///
									Sbaseliness_math Sbaseliness_reading Sbaseliness_writing, vce(robust)
							test Z
							matrix balance[`row',`col'+ 4 + `c'] = r(chi2)
							matrix balance[`row'+1, `col'+ 4 + `c'] = r(p)
							matrix balance[`row'+2, `col'+ 4 + `c'] = r(df)
							matrix balance[`row'+3, `col' +4 + `c'] = e(N)
							
							unique tag_r0 if e(sample)
							matrix balance[`row'+4, `col' +4 + `c'] = r(sum)

					*** All covariates suest ***
					
						local row = `row' + 5
						suest	num_schools num_study rank1_study ///
								baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
								baseliness_math baseliness_reading baseliness_writing, vce(robust)
						test Z
						matrix balance[`row', `col'+1] = r(chi2)
						matrix balance[`row'+1, `col'+1] = r(p)
						matrix balance[`row'+2, `col'+1] = r(df)
						matrix balance[`row'+3, `col'+1] = e(N)
						
							
						suest	Lnum_schools Lnum_study Lrank1_study ///
								Lbaselinestudy Lbaselinefemale Lbaselinehisp Lbaselineblack Lbaselinefoodfred Lbaselinesped Lbaselinebilingual Lbaselinelep Lbaselinegifted ///
								Lbaseliness_math Lbaseliness_reading Lbaseliness_writing, vce(robust)
						test Z
						matrix balance[`row', `col'+2] = r(chi2)
						matrix balance[`row'+1, `col' +2] = r(p)
						matrix balance[`row'+2, `col'+2] = r(df)
						matrix balance[`row'+3, `col'+2] = e(N)
						
						suest	Pnum_schools Pnum_study  Prank1_study ///
								Pbaselinefemale Pbaselinehisp Pbaselineblack Pbaselinefoodfred Pbaselinesped Pbaselinelep Pbaselinebilingual Pbaselinegifted ///
								Pbaseliness_math Pbaseliness_reading Pbaseliness_writing, vce(robust)
						test Z
						matrix balance[`row', `col'+3] = r(chi2)
						matrix balance[`row'+1, `col' +3] = r(p)
						matrix balance[`row'+2, `col'+3] = r(df)
						matrix balance[`row'+3, `col'+3] = e(N)
						
						local c=0
						foreach i in 01 001{
							local c = `c' + 1
							suest	B`i'num_schools B`i'num_study  B`i'rank1_study ///
									B`i'baselinestudy B`i'baselinefemale B`i'baselinehisp B`i'baselineblack B`i'baselinefoodfred B`i'baselinesped B`i'baselinelep B`i'baselinebilingual B`i'baselinegifted ///
									B`i'baseliness_math B`i'baseliness_reading B`i'baseliness_writing, vce(robust)
							test Z
							matrix balance[`row', `col' +3 + `c'] = r(chi2)
							matrix balance[`row'+1, `col' +3 + `c'] = r(p)
							matrix balance[`row'+2, `col' +3 + `c'] = r(df)
							matrix balance[`row'+3, `col' +3 + `c'] = e(N)
							
							unique tag_r`i' if e(sample)
							matrix balance[`row'+4, `col' +3 + `c'] = r(sum)
						}
							suest	Snum_schools Snum_study Srank1_study ///
									Sbaselinestudy Sbaselinefemale Sbaselinehisp Sbaselineblack Sbaselinefoodfred Sbaselinesped Sbaselinelep Sbaselinebilingual Sbaselinegifted ///
									Sbaseliness_math Sbaseliness_reading Sbaseliness_writing, vce(robust)
							test Z
							matrix balance[`row',`col'+ 4 + `c'] = r(chi2)
							matrix balance[`row'+1, `col'+ 4 + `c'] = r(p)
							matrix balance[`row'+2, `col'+ 4 + `c'] = r(df)
							matrix balance[`row'+3, `col' +4 + `c'] = e(N)
							
							unique tag_r0 if e(sample)
							matrix balance[`row'+4, `col' +4 + `c'] = r(sum)

						local row = `row' + 5
					} /*Close cov group loop*/
					
				} /*Close f_cut loop*/
				
					**********************************************************************************
					
				*Outsheet
				preserve
					svmat balance
					keep var balance*
					keep if _n<=`row'+5
					ren balance1 z0
					ren balance2 ols_z1
					ren balance3 ols_z1_pscore
					ren balance4 ols_z1_poly
					ren balance5 ols_z1_bin01
					ren balance6 ols_z1_bin001
					ren balance7 ols_z1_psat
					
					outsheet using "`tables'`cut'_`sch_type'_`file'_years_stacked_balance.csv", comma replace
				restore
				
				} /*Close F test condition*/
			}	/*Close balance condition*/
		}	/*Close cut (in DPS at baseline or not) loop*/
	} /* Close sch_type loop */
	
