/*
This file outputs 2sls for formula frequency and simulation, with no covariates
Relevant columns: 
FS_r01_nocov
twoSLS_r01_nocov

Rows: 
ss_math_410
ss_math_410_se

ss_math_410
ss_math_410_se

ss_reading_410
ss_reading_410_se

ss_writing_410
ss_writing_410_se

ss_math_410_N
*/


clear all
cap log close
set more off
set trace off
set tracedepth 1 
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"
pause on

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"
local rawdata "Y:\DPS\Raw Data\"

matrix demo			  = J(30,30,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,40,.)
matrix subtwosls	  = J(500,40,.)
matrix twosls_nocov    		   = J(500,40,.)
matrix twosls_scorecov 		   = J(500,40,.)
matrix twosls_fcsample   		= J(50	0,40,.)

matrix qualbalance    = J(500,20,.)
matrix qualtwosls	  = J(500,20,.)
matrix qualsubtwosls  = J(500,20,.)

*Dates of analysis files
local pscoredate_year2	20170310 	/*20160516 20160510 20160428 20160411 20160208 20160112 20151120 20150514 */
local pscoredate_year1  20170310 /*20160516    	20160516 20160510 20160421  20160208 */
local simsdate 			20150603 /* 20150428 */
local qualdate_year1 	20160715 /*20150708  20150420 */
local qualdate_year2	20160715 /*20160303*/
local qualdate			20160303
local qualpscoredate	20160208
local appdatayr1		20170310
local appdatayr2		20170310


local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_*"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1        		/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
local outsheet_rescorr=0	/* If want to outsheet matrix with residual correlations */




*****************************************
***************** 2SLS (no covariates) ******************
*****************************************


foreach file in  simulation /*frequency formula */ {

	u "`cleandata'analysis_stacked_study_`file'.dta", clear
					
	keep if in_sample == 1
	keep if baselinespyear == 2012 | baselinespyear == 2013

	gen var =""
	gen Z_HS = Z*HS
	
	*By grade vars
	gen group = .
	
	local col = 1
	local row = 1
			

	foreach group in 410 /*48 910*/{
				if `group'==410{
					replace group=410
				}
				else if `group'==48{
					replace group=48 if inlist(appgrade,4,5,6,7,8)
				}
				else{
					replace group=910 if inlist(appgrade,9,10)
				}

				foreach t in math writing reading {
					foreach out in ss /*prof_up adv*/ {
			
					
					replace var="`out'_`t'_`group'" if _n==`row'
					replace var="`out'_`t'_`group'_se" if _n==`row'+1
					replace var="`out'_`t'_`group'_N" if _n==`row'+2
					

					qui summ `out'_`t' if group==`group' & in_zero1_pscore==1
					
					if r(max)==0 & r(min)==0{
						di in red "No observations for `group'-level 2SLS estimates"
					}
				
					else{
						
						qui: summ `out'_`t' if Z == 0 & group==`group'
						matrix twosls_nocov[`row',`col'] = r(mean)
						matrix twosls_nocov[`row'+2,`col'] = r(N)
						
						qui summ `out'_`t' if Z == 0 & group==`group'
						matrix twosls_scorecov[`row',`col'] = r(mean)
						matrix twosls_scorecov[`row'+2,`col'] = r(N)
						
							/*OLS - no control for pscore nor covariates (but including baseline score covariates) */
							qui reg `out'_`t' D g_* i.baselinespyear `cov2' if group==`group', vce(robust)
							matrix twosls_scorecov[`row',`col'+1]   = _b[D]
							matrix twosls_scorecov[`row'+1,`col'+1] = _se[D]
							matrix twosls_scorecov[`row'+2,`col'+1] = e(N)
							
							/*OLS - no control for pscore nor covariates */
							qui reg `out'_`t' D g_* i.baselinespyear if group==`group' & baseliness_math != . & baseliness_reading != . & baseliness_writing != ., vce(robust)
							matrix twosls_nocov[`row',`col'+1]   = _b[D]
							matrix twosls_nocov[`row'+1,`col'+1] = _se[D]
							matrix twosls_nocov[`row'+2,`col'+1] = e(N)

							/*OLS conditional controlling for fully saturated pscores (only including baseline score covariates) */
							qui reg `out'_`t' D i.ind_r0 g_* i.baselinespyear `cov2' if group==`group' & in_good_cell_r0==1, vce(robust)
							matrix twosls_scorecov[`row',`col'+2]   = _b[D]
							matrix twosls_scorecov[`row'+1,`col'+2] = _se[D]
							matrix twosls_scorecov[`row'+2,`col'+2] = e(N)
							
							/*OLS conditional controlling for fully saturated pscores*/
							qui reg `out'_`t' D i.ind_r0 g_* i.baselinespyear if group==`group' & in_good_cell_r0==1 & baseliness_math != . & baseliness_reading != . & baseliness_writing != ., vce(robust)
							matrix twosls_nocov[`row',`col'+2]   = _b[D]
							matrix twosls_nocov[`row'+1,`col'+2] = _se[D]
							matrix twosls_nocov[`row'+2,`col'+2] = e(N)
						
					
						foreach instrument_type in ordinary /*over_id */ {
						
							if "`instrument_type'" == "ordinary" {
							local instrument  "Z"
							}
							else {
							local instrument "Z Z_HS"
							}
							
							
								*2SLS with neither score controls nor covariates (but including baseline score covs)							
									*2SLS
									ivreg2 `out'_`t' (D = `instrument') g_* i.baselinespyear `cov2' if group==`group', savefirst savefprefix(first) robust
										 
									*2SLS
									matrix twosls_scorecov[`row',`col'+4]   = _b[D]
									matrix twosls_scorecov[`row'+1,`col'+4] = _se[D]
									matrix twosls_scorecov[`row'+2,`col'+4] = e(N)
									
									estimates restore firstD
							
									*First stage
									matrix twosls_scorecov[`row',`col'+3]   = _b[Z]
									matrix twosls_scorecov[`row'+1,`col'+3] = _se[Z]
									
									*estimates restore reduced`out'_`t'
									
									*reduced form
									matrix twosls_scorecov[`row',`col'+5]   = 0
									matrix twosls_scorecov[`row'+1,`col'+5] = 0
								
								*2SLS with neither score controls nor covariates							
									*2SLS
									ivreg2 `out'_`t' (D = `instrument') g_* i.baselinespyear if group==`group' & baseliness_math != . & baseliness_reading != . & baseliness_writing != ., savefirst savefprefix(first) robust
										
									*2SLS
									matrix twosls_nocov[`row',`col'+4]   = _b[D]
									matrix twosls_nocov[`row'+ 1,`col'+4] = _se[D]
									matrix twosls_nocov[`row'+ 2, `col'+4] = e(N)
									
									estimates restore firstD
							
									*First stage
									matrix twosls_nocov[`row',`col'+3]   = _b[Z]
									matrix twosls_nocov[`row'+ 1,`col'+3] = _se[Z]
									
									*estimates restore reduced`out'_`t'
									
									*reduced form
									matrix twosls_nocov[`row' ,`col'+5]   = 0
									matrix twosls_nocov[`row'+ 1,`col'+5] = 0
									
									di "row `row'"
									
					
																			
								***Fully Saturated 2sls
									
								*2SLS with no cov controls (but with grade and baseline score covs) rounded to the hundredths
									*2SLS
									ivreg2 `out'_`t' (D = `instrument') i.ind_r0 g_* i.baselinespyear `cov2' if group==`group' & in_good_cell_r0==1, savefirst savefprefix(first) partial(i.ind_r0 g_* `cov2') robust 
										
									*2SLS
									matrix twosls_scorecov[`row',`col'+7]   = _b[D]
									matrix twosls_scorecov[`row'+1,`col'+7] = _se[D]
									matrix twosls_scorecov[`row'+2,`col'+7] = e(N)
									
									estimates restore firstD
							
									*First stage
									matrix twosls_scorecov[`row',`col'+6]   = _b[Z]
									matrix twosls_scorecov[`row'+1,`col'+6] = _se[Z]
									
									*estimates restore reduced`out'_`t'
									
									*reduced form
									matrix twosls_scorecov[`row',`col'+8]   = 0
									matrix twosls_scorecov[`row'+1,`col'+8] = 0
							
								
								
								*2SLS with no cov controls (but with grade) rounded to the hundredths
									*2SLS
									ivreg2 `out'_`t' (D = `instrument') i.ind_r0 g_* i.baselinespyear if group==`group' & in_good_cell_r0==1 & baseliness_math != . & baseliness_reading != . & baseliness_writing != ., savefirst savefprefix(first)  partial(i.ind_r0 g_*)  robust
									
									*2SLS
									matrix twosls_nocov[`row',`col'+7]   = _b[D]
									matrix twosls_nocov[`row'+1,`col'+7] = _se[D]
									matrix twosls_nocov[`row'+2,`col'+7] = e(N)
									
									estimates restore firstD
							
									*First stage
									matrix twosls_nocov[`row',`col'+6]   = _b[Z]
									matrix twosls_nocov[`row'+1,`col'+6] = _se[Z]
									
									*estimates restore reduced`out'_`t'
									
									*reduced form
									matrix twosls_nocov[`row',`col'+8]   = 0
									matrix twosls_nocov[`row'+1,`col'+8] = 0
									
							
							local col = 11
							
							foreach risk in r01 r001{
								*2SLS with no cov controls (but with grade and baseline score covs)
									*2SLS
									ivreg2 `out'_`t' (D = `instrument') i.ind_`risk' g_* i.baselinespyear `cov2' if group==`group' & in_good_cell_`risk'==1,  savefirst savefprefix(first) partial(i.ind_`risk' g_* `cov2') robust
										
										*2SLS
										matrix twosls_scorecov[`row',`col']   = _b[D]
										matrix twosls_scorecov[`row'+1,`col'] = _se[D]
										matrix twosls_scorecov[`row'+2,`col'] = e(N)
										
										estimates restore firstD
								
										*First stage
										matrix twosls_scorecov[`row',`col'-1]   = _b[Z]
										matrix twosls_scorecov[`row'+1,`col'-1] = _se[Z]
										
										*estimates restore reduced`out'_`t'
										
										*reduced form
										matrix twosls_scorecov[`row',`col'+1]   = 0
										matrix twosls_scorecov[`row'+1,`col'+1] = 0
									
								
									
								*2SLS with no cov controls (but with grade)
									*2SLS
									ivreg2 `out'_`t' (D = `instrument') i.ind_`risk' g_* i.baselinespyear if group==`group' & in_good_cell_`risk'==1 & baseliness_math != . & baseliness_reading != . & baseliness_writing != ., savefirst savefprefix(first) partial(i.ind_`risk' g_*) robust 
										
										*2SLS
										matrix twosls_nocov[`row',`col']   = _b[D]
										matrix twosls_nocov[`row'+1,`col'] = _se[D]
										matrix twosls_nocov[`row'+2,`col'] = e(N)
										
										estimates restore firstD
								
										*First stage
										matrix twosls_nocov[`row',`col'-1]   = _b[Z]
										matrix twosls_nocov[`row'+1,`col'-1] = _se[Z]
										
										*estimates restore reduced`out'_`t'
										
										*reduced form
										matrix twosls_nocov[`row',`col'+1]   = 0
										matrix twosls_nocov[`row'+1,`col'+1] = 0
									
									local col = `col' + 3
			
						} /*risk loop*/
						
					} /* instrument loop */
				} /*close condition on number of observations*/
				
				local row = `row' + 3
				local col = 1
			} 	 /*out loop*/
			
			local ++row
		}	/* t loop*/
		
	}	/*group loop*/


	preserve
		svmat twosls_nocov
		keep var twosls_nocov*
		keep if _n<=`row'
		ren twosls_nocov1 z0
		ren twosls_nocov2 OLS
		ren twosls_nocov3 OLS_psat
		
		ren twosls_nocov4 FS_nocov
		ren twosls_nocov5 twoSLS_nocov
		ren twosls_nocov6 RF_nocov
		
		ren twosls_nocov7 FS_psat_nocov
		ren twosls_nocov8 twoSLS_psat_nocov
		ren twosls_nocov9 RF_psat_nocov
		
		ren twosls_nocov10 FS_r01_nocov
		ren twosls_nocov11 twoSLS_r01_nocov
		ren twosls_nocov12 RF_r01_nocov
		
		ren twosls_nocov13 FS_r001_nocov
		ren twosls_nocov14 twoSLS_r001_nocov
		ren twosls_nocov15 RF_r001_nocov
		
		/*
		ren twosls_nocov16 FS_nocov_overid
		ren twosls_nocov17 twoSLS_nocov_overid
		ren twosls_nocov18 RF_nocov_overid
		
		ren twosls_nocov19 FS_psat_nocov_overid
		ren twosls_nocov20 twoSLS_psat_nocov_overid
		ren twosls_nocov21 RF_psat_nocov_overid
		
		ren twosls_nocov22 FS_r01_nocov_overid
		ren twosls_nocov23 twoSLS_r01_nocov_overid
		ren twosls_nocov24 RF_r01_nocov_overid
			*/			
		
		outsheet using "`tables'inDPSbase_`file'_twosls_years_stacked_nocovs.csv", comma replace
	restore

		
	preserve
		svmat twosls_scorecov
		keep var twosls_scorecov*
		keep if _n<=`row'
		
		ren twosls_scorecov1 z0
		ren twosls_scorecov2 OLS
		ren twosls_scorecov3 OLS_psat
		
		ren twosls_scorecov4 FS_nocov
		ren twosls_scorecov5 twosls_scorecov
		ren twosls_scorecov6 RF_nocov
		
		ren twosls_scorecov7 FS_psat_nocov
		ren twosls_scorecov8 twoSLS_psat_nocov
		ren twosls_scorecov9 RF_psat_nocov
		
		ren twosls_scorecov10 FS_r01_nocov
		ren twosls_scorecov11 twoSLS_r01_nocov
		ren twosls_scorecov12 RF_r01_nocov
		
		ren twosls_scorecov13 FS_r001_nocov
		ren twosls_scorecov14 twoSLS_r001_nocov
		ren twosls_scorecov15 RF_r001_nocov
		
		
		outsheet using "`tables'inDPSbase_`file'_twosls_years_stacked_scorecov.csv", comma replace
	restore
	

clear	

} /*Close file loop*/	


