


clear all
cap log close
set more off
set trace off
set tracedepth 1 
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"
pause on

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"
local rawdata "Y:\DPS\Raw Data\"

matrix demo			  = J(30,30,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,40,.)
matrix subtwosls	  = J(500,40,.)
matrix twosls_nocov    		   = J(500,40,.)
matrix twosls_scorecov 		   = J(500,40,.)
matrix twosls_fcsample   		= J(50	0,40,.)

matrix qualbalance    = J(500,20,.)
matrix qualtwosls	  = J(500,20,.)
matrix qualsubtwosls  = J(500,20,.)

*Dates of analysis files
local pscoredate_year2	20170310 	/*20160516 20160510 20160428 20160411 20160208 20160112 20151120 20150514 */
local pscoredate_year1  20170310 /*20160516    	20160516 20160510 20160421  20160208 */
local simsdate 			20150603 /* 20150428 */
local qualdate_year1 	20160715 /*20150708  20150420 */
local qualdate_year2	20160715 /*20160303*/
local qualdate			20160303
local qualpscoredate	20160208
local appdatayr1		20170310
local appdatayr2		20170310


local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_*"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1        		/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
local outsheet_rescorr=0	/* If want to outsheet matrix with residual correlations */



	*****************************************
	***************** 2SLS ******************
	*****************************************
	
		
	foreach file in frequency /*simulation  formula  */ {
		foreach sch_type in study /*noex nonnoex*/ {
			foreach cut in inDPSbase /*allapps*/{
			
				u "`cleandata'analysis_stacked_`sch_type'_`file'.dta", clear
								
				if "`cut'"=="inDPSbase"{
					keep if baselinespyear==2012 | baselinespyear == 2013
				}
				else{
					di in red "Keep all obs"
				}
				
				keep if in_sample == 1
				
				gen var =""
				
				*By grade vars
				gen group = .
				
				local col = 1
				local row = 1
				
				
				
				foreach group in 410 /*45 68 48 910*/{
							if `group'==410{
								replace group=410
							}
							else if `group'==48{
								replace group=48 if inlist(appgrade,4,5,6,7,8)
							}
							else{
							replace group=45 if inlist(appgrade,4,5)
							replace group=68 if inlist(appgrade,6,7,8)
							replace group=910 if inlist(appgrade,9,10)
							}
	
							foreach t in math writing reading {
								foreach out in ss /*prof_up adv*/ {
						
								
								replace var="`out'_`t'_`group'" if _n==`row'
								replace var="`out'_`t'_`group'_se" if _n==`row'+1
								replace var="`out'_`t'_`group'_N" if _n==`row'+2
								
			
								qui summ `out'_`t' if group==`group' & in_zero1_pscore==1
								
								if r(max)==0 & r(min)==0{
									di in red "No observations for `group'-level 2SLS estimates"
								}
							
								else{
							
									qui summ `out'_`t' if Z == 0 & group==`group'
									matrix twosls[`row',`col'] = r(mean)
									matrix twosls[`row'+2,`col'] = r(N)
								
									/*OLS - no control for pscore*/
									qui reg `out'_`t' D `cov' `cov2' `cov_grade' i.baselinespyear if group==`group', vce(robust)
									matrix twosls[`row',`col'+1]   = _b[D]
									matrix twosls[`row'+1,`col'+1] = _se[D]
									matrix twosls[`row'+2,`col'+1] = e(N)
										

										/*OLS conditional on fully saturated pscore sample */
										qui reg `out'_`t' D `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & in_good_cell_r0==1, vce(robust)
										matrix twosls[`row',`col'+2]   = _b[D]
										matrix twosls[`row'+1,`col'+2] = _se[D]
										matrix twosls[`row'+2,`col'+2] = e(N)
									
										
										/*OLS conditional controlling for fully saturated pscores*/
										qui reg `out'_`t' D i.ind_r0 `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & in_good_cell_r0==1, vce(robust)
										matrix twosls[`row',`col'+3]   = _b[D]
										matrix twosls[`row'+1,`col'+3] = _se[D]
										matrix twosls[`row'+2,`col'+3] = e(N)
									
									
									/*OLS conditional controlling for pscores rounded to the hundredths*/
									qui reg `out'_`t' D i.ind_r01 `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & in_good_cell_r01==1, vce(robust)
									matrix twosls[`row',`col'+4]   = _b[D]
									matrix twosls[`row'+1,`col'+4] = _se[D]
									matrix twosls[`row'+2,`col'+4] = e(N)
									
									/*OLS conditional controlling for pscores rounded to the thousandths*/
									qui reg `out'_`t' D i.ind_r001 `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & in_good_cell_r001==1, vce(robust)
									matrix twosls[`row',`col'+5]   = _b[D]
									matrix twosls[`row'+1,`col'+5] = _se[D]
									matrix twosls[`row'+2,`col'+5] = e(N)
										
									foreach risk in  r01 r001 r0 {
						
										local col = `col' + 2
										
										local controls "i.ind_`risk' `cov' `cov2' `cov_grade'"		
											
										/*2SLS - outcome on treatment instrumented by instrument*/
										ivreg2 `out'_`t' (D = Z) `controls' i.baselinespyear if group==`group' & in_good_cell_`risk'==1 , savefirst savefprefix(first) partial(`controls') robust
										ivreg2 `out'_`t' (D = Z) `controls' i.baselinespyear if group==`group' & in_good_cell_org_`risk'==1 , savefirst savefprefix(first) partial(`controls') robust
										
								
										*2SLS
										matrix twosls[`row',`col'+5]   = _b[D]
										matrix twosls[`row'+1,`col'+5] = _se[D]
										matrix twosls[`row'+2,`col'+5] = e(N)
										
										estimates restore firstD
										
										*First stage
										matrix twosls[`row',`col'+4]   = _b[Z]
										matrix twosls[`row'+1,`col'+4] = _se[Z]
											
											
										*If last loop, do these
										if "`risk'"=="r0"{
												
												 
											*NAIVE 2SLS - no riskset controlsq
												
												*2SLS naive
												ivreg2 `out'_`t' (D = Z) `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & !inlist(`risk', 0, 1) , savefirst savefprefix(first) partial(`cov' `cov2') robust

												*2SLS
												matrix twosls[`row',`col'+7]   = _b[D]
												matrix twosls[`row'+1,`col'+7] = _se[D]
												matrix twosls[`row'+2,`col'+7] = e(N)
												
												estimates restore firstD
										
												*First stage
												matrix twosls[`row',`col'+6]   = _b[Z]
												matrix twosls[`row'+1,`col'+6] = _se[Z]
													
												*2SLS naive conditional on r0 sample
												ivreg2 `out'_`t' (D = Z) `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & in_good_cell_r0==1 & !inlist(`risk', 0, 1) , savefirst savefprefix(first) partial(`cov' `cov2') robust

												*2SLS
												matrix twosls[`row',`col'+9]   = _b[D]
												matrix twosls[`row'+1,`col'+9] = _se[D]
												matrix twosls[`row'+2,`col'+9] = e(N)
												
												estimates restore firstD
										
												*First stage
												matrix twosls[`row',`col'+8]   = _b[Z]
												matrix twosls[`row'+1,`col'+8] = _se[Z]													
											
											*2SLS with no cov controls								
												*2SLS
												ivreg2 `out'_`t' (D = Z) i.baselinespyear i.ind_`risk' if group==`group' & in_good_cell_r0==1 & !inlist(`risk', 0, 1) , savefirst savefprefix(first) partial(i.ind_`risk') robust
													
												*2SLS
												matrix twosls[`row',`col'+11]   = _b[D]
												matrix twosls[`row'+1,`col'+11] = _se[D]
												matrix twosls[`row'+2,`col'+11] = e(N)
												
												estimates restore firstD
										
												*First stage
												matrix twosls[`row',`col'+10]   = _b[Z]
												matrix twosls[`row'+1,`col'+10] = _se[Z]

										} /*close if condition*/
									
									
									} /*risk loop*/
							
							} /*close condition on number of observations*/
							
							local row = `row' + 3
							local col = 1
						} 	 /*out loop*/
						
						local ++row
					}	/* t loop*/
					
				}	/*group loop*/
				
				*ivreg2 ss_math (D = Z) baselinespyear i.ind_r0 if in_good_cell_r0==1, savefirst savefprefix(first) partial(i.ind_r0) cluster(maskedid) 				

				
				preserve
					svmat twosls
					keep var twosls*
					keep if _n<=`row'
					ren twosls1 z0
					ren twosls2 OLS
					ren twosls3 OLS_psample
					ren twosls4 OLS_psat
					ren twosls5 OLS_r01
					ren twosls6 OLS_r001
					
					ren twosls7  FS_r01
					ren twosls8  twoSLS_r01
					ren twosls9  FS_r001
					ren twosls10 twosls_r001
					
					ren twosls11 FS_psat
					ren twosls12 twoSLS_psat
					
					ren twosls13 FS_naive
					ren twosls14 twoSLS_naive
					
					ren twosls15 FS_naive_psample
					ren twosls16 twoSLS_naive_psample
					
					ren twosls17 FS_nocovs
					ren twosls18 twoSLS_nocovs

					outsheet using "`tables'`cut'_`sch_type'_`file'_years_stacked_twosls.csv", comma replace
					
				restore
			clear	

			} /*Close 2SLS condition*/
		}	/*Close cut (in DPS at baseline or not) loop*/
	} /* Close sch_type loop */

	
	
