/* 	Ye Ji Kee (Created on 3/16/2016)

		For grades 4-10 (2years stacked)
		Output: weighting estimators and their bootstrapped std.errors. 2SLS bootstrapped std. errors. (Bayesian/nonparametric)
		
	Ignacio Rodriguez 07/18/2016
		Am adding bootstrapped std errors for 2sls estimates with no covariates. Only adding this for the Bayesian bootstraps. 

	analysis stacked now only has first time application

	Relevant columns are: 
	r01
	
	Rows: 
	ss_math_410_hir_fs
	ss_math_410_hir_fs_se_tenth

	ss_math_410_hir
	ss_math_410_hir_se_tenth

	ss_reading_410_hir
	ss_reading_410_hir_se_tenth

	ss_writing_410_hir
	ss_writing_410_hir_se_tenth

	ss_math_410_hir_N


*/


clear all
cap log close
set more off
set trace off
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"

pause on 

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"

local pscore_weighting_est = 0
local pscore_weighting_est_fulltype = 0
local pscore_weighting_est_type_nocov = 0
local pscore_weighting_est_nocov = 1
local pscore_twosls_bstrap = 0
local pscore_twosls_bstrap_half = 0

matrix demo			  = J(30,15,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,40,.)
matrix rf             = J(500,40,.)


matrix simulation_est = J(500,5,.)
matrix frequency_est = J(500,5,.)
matrix formula_est = J(500,5,.)

matrix simulation_nocov_est = J(500,5,.)
matrix frequency_nocov_est = J(500,5,.)
matrix formula_nocov_est = J(500,5,.)

matrix simulation_bayesian_est = J(500,5,.)
matrix frequency_bayesian_est = J(500,5,.)
matrix formula_bayesian_est = J(500,5,.)

matrix simulation_bayesian_nocov_est = J(500,10,.)
matrix frequency_bayesian_nocov_est = J(500,10,.)
matrix formula_bayesian_nocov_est = J(500,10,.)

matrix simulation_2sls_bs = J(500,5,.)
matrix frequency_2sls_bs = J(500,5,.)
matrix formula_2sls_bs = J(500,5,.)

matrix simulation_2sls_bs_half = J(500,5,.)
matrix frequency_2sls_bs_half = J(500,5,.)
matrix formula_2sls_bs_half = J(500,5,.)

matrix simulation_2sls_bayesian_bs = J(500,5,.)
matrix frequency_2sls_bayesian_bs = J(500,5,.)
matrix formula_2sls_bayesian_bs = J(500,5,.)

matrix simulation_nocov_2sls_bayes_bs = J(500,5,.)
matrix frequency_nocov_2sls_bayes_bs = J(500,5,.)
matrix formula_nocov_2sls_bayes_bs = J(500,5,.)

matrix fulltype_bayesian_est = J(500,5,.)
matrix fulltype_nonparm_est = J(500,5,.)

*local pscore_weighting_est_nonparm = 0

local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"


*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_*"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1				/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
		
local trimm_diff = 0.025   // amount to trim no cov semi param sample. 
		
		
	**************************************************************
	******* pscore weighting estimators (bayesian & nonparm) *****
	**************************************************************
	
	if `pscore_weighting_est'==1 {
		
	capture: program drop ciest_fuzzy
	program ciest_fuzzy, rclass
		version 11.0
		syntax varlist(fv), outcome(string) method(string) wgt(string)
		
		noisily {
			if ("`method'" == "abadie") {
				capture: xi: logit Z `varlist' [pw=`wgt'] if `outcome' != ., iterate(100)
				
				if (_rc == 0 & `e(converged)' == 1) {
					
					predict pred_p if `outcome' != .
				
					gen w = (Z - pred_p) / ((1 - pred_p) * pred_p)
					gen w_y = w*`outcome'
					
					gen kappa = 1 - ((D*(1 - Z)) / (1 - pred_p)) - (((1 - D) * Z) / pred_p)
					
					su w_y [aw=`wgt']
					local rf = r(mean)
					local rf_N = r(N)
					
					su kappa [aw=`wgt']
					local fs = r(mean)
					local fs_N = r(N)
					
					local late = `rf' / `fs'
					
					drop pred_p w w_y kappa
					}
					else {
					local rf = .
					local fs = .
					local late = .
					local rf_N = .
					local fs_N = .
				}
			}
			else if ("`method'" == "hir") {
				capture noisily: xi: logit Z `varlist' [pw=`wgt'] if `outcome' != ., iterate(100)
				if (_rc == 0 & `e(converged)' == 1) {
					predict pred_p if `outcome' != .
					
					gen w = (Z - pred_p) / ((1 - pred_p) * pred_p)
					gen w_D = w*D
					gen w_y = w*`outcome'
					
					su w_D [aw=`wgt']
					local fs = r(mean)
					local fs_N = r(N)
					
					su w_y [aw=`wgt']
					local rf = r(mean)
					local rf_N = r(N)
					
					local late = `rf' / `fs'
					
					drop pred_p w w_D w_y
					}
					else {
					local rf = .
					local fs = .
					local late = .
					local rf_N = .
					local fs_N = .
				}
			}
		}
		return scalar late = `late'
		return scalar rf = `rf'
		return scalar fs = `fs'
		return scalar rf_N = `rf_N'
		return scalar fs_N = `fs_N'

	end
				

		** Calculate the weighting estimates		
		foreach file in simulation frequency formula {
		
			u "`cleandata'analysis_stacked_study_`file'.dta", clear
			
			
			keep if in_sample == 1
			keep if baselinespyear == 2012 | baselinespyear == 2013
			
			drop if (maskedid == 674081 | maskedid == 710385) & baselinespyear == 2013
			
			gen var =""
			
			*By grade vars
			gen group = .
			
			local col = 1
			local row = 1
			
		foreach method in /*abadie*/ hir {
		
			foreach group in 410 /*48 910*/ {
						if `group'==410{
							replace group=410
						}
						else if `group'==48{
							replace group=48 if inlist(appgrade,4,5,6,7,8)
						}
						else{
							replace group=910 if inlist(appgrade,9,10)
						}

						foreach t in math writing reading {
							foreach out in ss /*prof_up adv*/ {
								

								replace var="`out'_`t'_`group'_`method'" if _n==`row'
								replace var="`out'_`t'_`group'_`method'_se" if _n==`row'+1
								replace var="`out'_`t'_`group'_`method'_se_tenth" if _n==`row'+2							
								replace var="`out'_`t'_`group'_`method'_fs" if _n==`row'+3
								replace var="`out'_`t'_`group'_`method'_fs_se" if _n==`row'+4
								replace var="`out'_`t'_`group'_`method'_fs_se_tenth" if _n==`row'+5							
								replace var="`out'_`t'_`group'_`method'_N" if _n==`row'+6
								replace var="`out'_`t'_`group'_`method'_fs_N" if _n==`row'+7
								replace var="`out'_`t'_`group'_`method'_bstrap_N" if _n==`row'+8
								replace var="`out'_`t'_`group'_`method'_bstrap_N_tenth" if _n==`row'+9
								replace var="`out'_`t'_`group'_`method'_se_fifth" if _n==`row'+10
								replace var="`out'_`t'_`group'_`method'_fs_se_fifth" if _n==`row'+11
								replace var="`out'_`t'_`group'_`method'_bstrap_N_fifth" if _n==`row'+12

								qui summ `out'_`t' if group==`group' & in_zero1_pscore==1
								
								if r(max)==0 & r(min)==0{
									di in red "No observations for `group'-level weighting estimates"
								}
							
								else{
								
								if "`file'"=="simulation" {
										
									local col = 1
										
									foreach risk in r01 {
										
										foreach bs_method in bayesian /*nonparm*/ {
																		
											preserve
										
												keep if group == `group' & in_good_cell_`risk' == 1
												egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
												keep if temp == 0
												drop temp
												
					
												*using factor variables
												local controls "i.ind_`risk' `cov' `cov2' `cov_grade' baselinespyear"		
												
												gen one = 1
												
												ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("one")
												local late = `r(late)'
												local rf = `r(rf)'
												local fs = `r(fs)'
												local rf_N = `r(rf_N)'
												local fs_N = `r(fs_N)'
												
												save "`cleandata'weighted_est_temp_stacked.dta", replace
												
												
												set seed 123
												mat bootstrap_est = J(1, 2, .)
											
												if "`bs_method'" == "bayesian" {
												
												forval b = 1/1000 {
													gen bs_wgt = rgamma(1,1)
													ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("bs_wgt")
													mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))						
													use "`cleandata'weighted_est_temp_stacked.dta", clear
												}
												
												svmat bootstrap_est
												
												gen bootstrap_est2_tenth = bootstrap_est2
												gen bootstrap_est1_tenth = bootstrap_est1
												
												gen bootstrap_est2_fifth = bootstrap_est2
												gen bootstrap_est1_fifth = bootstrap_est1
												
												gen bootstrap_est2_tmp = bootstrap_est2
												** When bootstrapped, there could be FS that are too small. Drop them.
												replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
												replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
												
												replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
												replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
												
												replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
												replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

												drop bootstrap_est2_tmp
											
												su bootstrap_est1
												local bstrap_N = `r(N)'
												local late_se = `r(sd)' 
												
												su bootstrap_est2
												local fs_bstrap_N = `r(N)'
												local fs_se = `r(sd)' 
												
												su bootstrap_est1_tenth
												local bstrap_N_tenth = `r(N)'
												local late_se_tenth = `r(sd)' 
												
												
												su bootstrap_est2_tenth
												local fs_bstrap_N_tenth = `r(N)'
												local fs_se_tenth = `r(sd)' 
												
												su bootstrap_est1_fifth 
												local bstrap_N_fifth = `r(N)'								
												local late_se_fifth = `r(sd)' 
												
												su bootstrap_est2_fifth
												local fs_bstrap_N_fifth = `r(N)'
												local fs_se_fifth = `r(sd)'
												
												
												matrix simulation_bayesian_est[`row', `col'] = `late'
												matrix simulation_bayesian_est[`row'+1, `col'] = `late_se'
												matrix simulation_bayesian_est[`row'+2, `col'] = `late_se_tenth'
												matrix simulation_bayesian_est[`row'+3, `col'] = `fs'
												matrix simulation_bayesian_est[`row'+4, `col'] = `fs_se'
												matrix simulation_bayesian_est[`row'+5, `col'] = `fs_se_tenth'
												matrix simulation_bayesian_est[`row'+6, `col'] = `rf_N'
												matrix simulation_bayesian_est[`row'+7, `col'] = `fs_N'
												matrix simulation_bayesian_est[`row'+8, `col'] = `bstrap_N'
												matrix simulation_bayesian_est[`row'+9, `col'] = `bstrap_N_tenth'
												matrix simulation_bayesian_est[`row'+10, `col'] = `late_se_fifth'
												matrix simulation_bayesian_est[`row'+11, `col'] = `fs_se_fifth'
												matrix simulation_bayesian_est[`row'+12, `col'] = `bstrap_N_fifth'
								
											restore
											}
											
										
											else {
											forval b = 1/1000 {
												bsample
												*sample 50
												ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("one")
												mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))						
												use "`cleandata'weighted_est_temp_stacked.dta", clear
											}
											
											svmat bootstrap_est
											
											gen bootstrap_est2_tenth = bootstrap_est2
											gen bootstrap_est1_tenth = bootstrap_est1
											
											gen bootstrap_est2_fifth = bootstrap_est2
											gen bootstrap_est1_fifth = bootstrap_est1
											
											gen bootstrap_est2_tmp = bootstrap_est2
											** When bootstrapped, there could be FS that are too small. Drop them.
											replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
											replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
											
											replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
											replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
											
											replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
											replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

											drop bootstrap_est2_tmp
										
											su bootstrap_est1
											local bstrap_N = `r(N)'
											local late_se = `r(sd)' 
											
											su bootstrap_est2
											local fs_bstrap_N = `r(N)'
											local fs_se = `r(sd)' 
											
											su bootstrap_est1_tenth
											local bstrap_N_tenth = `r(N)'
											local late_se_tenth = `r(sd)' 
											
											
											su bootstrap_est2_tenth
											local fs_bstrap_N_tenth = `r(N)'
											local fs_se_tenth = `r(sd)'
											
											su bootstrap_est1_fifth 
											local bstrap_N_fifth = `r(N)'								
											local late_se_fifth = `r(sd)' 
											
											su bootstrap_est2_fifth
											local fs_bstrap_N_fifth = `r(N)'
											local fs_se_fifth = `r(sd)' 
											
											
											matrix simulation_est[`row', `col'] = `late'
											matrix simulation_est[`row'+1, `col'] = `late_se'
											matrix simulation_est[`row'+2, `col'] = `late_se_tenth'
											matrix simulation_est[`row'+3, `col'] = `fs'
											matrix simulation_est[`row'+4, `col'] = `fs_se'
											matrix simulation_est[`row'+5, `col'] = `fs_se_tenth'
											matrix simulation_est[`row'+6, `col'] = `rf_N'
											matrix simulation_est[`row'+7, `col'] = `fs_N'
											matrix simulation_est[`row'+8, `col'] = `bstrap_N'
											matrix simulation_est[`row'+9, `col'] = `bstrap_N_tenth'
											matrix simulation_est[`row'+10, `col'] = `late_se_fifth'
											matrix simulation_est[`row'+11, `col'] = `fs_se_fifth'
											matrix simulation_est[`row'+12, `col'] = `bstrap_N_fifth'
							
											restore
											}
											
										} /* closes bs method */								
									} /* closes risk */
								} /* closes simulation */	
									
									
								else if "`file'"=="frequency" {
									
									local col = 1
									
									foreach risk in r0{
									
									foreach bs_method in bayesian /*nonparametric*/ {
									
									preserve
								
									keep if group == `group' & in_good_cell_`risk' == 1
									egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
									keep if temp == 0
									drop temp

									gen one = 1
									
									local controls "`risk'_* `cov' `cov2' `cov_grade' baselinespyear"
									
									ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("one")
									local late = `r(late)'
									local rf = `r(rf)'
									local fs = `r(fs)'
									local rf_N = `r(rf_N)'
									local fs_N = `r(fs_N)'
									
									save "`cleandata'weighted_est_temp_stacked.dta", replace
									
									set seed 123
									mat bootstrap_est = J(1, 2, .)
									
																
									if "`bs_method'" == "bayesian" {
									
									forval b = 1/1000 {
										gen bs_wgt = rgamma(1,1)
										ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("bs_wgt")
										mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))						
										use "`cleandata'weighted_est_temp_stacked.dta", clear
									}
									
									svmat bootstrap_est
									
									gen bootstrap_est2_tenth = bootstrap_est2
									gen bootstrap_est1_tenth = bootstrap_est1
									
									gen bootstrap_est2_fifth = bootstrap_est2
									gen bootstrap_est1_fifth = bootstrap_est1
									
									gen bootstrap_est2_tmp = bootstrap_est2
									** When bootstrapped, there could be FS that are too small. Drop them.
									replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
									replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
									
									replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
									replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
									
									replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
									replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

									drop bootstrap_est2_tmp
								
									su bootstrap_est1
									local bstrap_N = `r(N)'
									local late_se = `r(sd)'
									
									su bootstrap_est2
									local fs_bstrap_N = `r(N)'
									local fs_se = `r(sd)' 
									
									su bootstrap_est1_tenth
									local bstrap_N_tenth = `r(N)'
									local late_se_tenth = `r(sd)' 
									
									
									su bootstrap_est2_tenth
									local fs_bstrap_N_tenth = `r(N)'
									local fs_se_tenth = `r(sd)'
									
									su bootstrap_est1_fifth 
									local bstrap_N_fifth = `r(N)'								
									local late_se_fifth = `r(sd)' 
									
									su bootstrap_est2_fifth
									local fs_bstrap_N_fifth = `r(N)'
									local fs_se_fifth = `r(sd)' 
									
									
									matrix frequency_bayesian_est[`row', `col'] = `late'
									matrix frequency_bayesian_est[`row'+1, `col'] = `late_se'
									matrix frequency_bayesian_est[`row'+2, `col'] = `late_se_tenth'
									matrix frequency_bayesian_est[`row'+3, `col'] = `fs'
									matrix frequency_bayesian_est[`row'+4, `col'] = `fs_se'
									matrix frequency_bayesian_est[`row'+5, `col'] = `fs_se_tenth'
									matrix frequency_bayesian_est[`row'+6, `col'] = `rf_N'
									matrix frequency_bayesian_est[`row'+7, `col'] = `fs_N'
									matrix frequency_bayesian_est[`row'+8, `col'] = `bstrap_N'
									matrix frequency_bayesian_est[`row'+9, `col'] = `bstrap_N_tenth'
									matrix frequency_bayesian_est[`row'+10, `col'] = `late_se_fifth'
									matrix frequency_bayesian_est[`row'+11, `col'] = `fs_se_fifth'
									matrix frequency_bayesian_est[`row'+12, `col'] = `bstrap_N_fifth'
					
									restore
									}
									
									else {
									forval b = 1/1000 {
										bsample
										*sample 50
										ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("one")
										mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))	
										use "`cleandata'weighted_est_temp_stacked.dta", clear
									}
									svmat bootstrap_est
				
									gen bootstrap_est2_tenth = bootstrap_est2
									gen bootstrap_est1_tenth = bootstrap_est1
									
									gen bootstrap_est2_fifth = bootstrap_est2
									gen bootstrap_est1_fifth = bootstrap_est1
									
									gen bootstrap_est2_tmp = bootstrap_est2
									** When bootstrapped, there could be FS that are too small. Drop them.
									replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
									replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
									
									replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
									replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
									
									replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
									replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

									drop bootstrap_est2_tmp
								
									su bootstrap_est1
									local bstrap_N = `r(N)'
									local late_se = `r(sd)' 
									
									su bootstrap_est2
									local fs_bstrap_N = `r(N)'
									local fs_se = `r(sd)'
									
									su bootstrap_est1_tenth
									local bstrap_N_tenth = `r(N)'
									local late_se_tenth = `r(sd)'
									
									
									su bootstrap_est2_tenth
									local fs_bstrap_N_tenth = `r(N)'
									local fs_se_tenth = `r(sd)' 
									
									su bootstrap_est1_fifth 
									local bstrap_N_fifth = `r(N)'								
									local late_se_fifth = `r(sd)' 
									
									su bootstrap_est2_fifth
									local fs_bstrap_N_fifth = `r(N)'
									local fs_se_fifth = `r(sd)' 
									
									
									matrix frequency_est[`row', `col'] = `late'
									matrix frequency_est[`row'+1, `col'] = `late_se'
									matrix frequency_est[`row'+2, `col'] = `late_se_tenth'
									matrix frequency_est[`row'+3, `col'] = `fs'
									matrix frequency_est[`row'+4, `col'] = `fs_se'
									matrix frequency_est[`row'+5, `col'] = `fs_se_tenth'
									matrix frequency_est[`row'+6, `col'] = `rf_N'
									matrix frequency_est[`row'+7, `col'] = `fs_N'
									matrix frequency_est[`row'+8, `col'] = `bstrap_N'
									matrix frequency_est[`row'+9, `col'] = `bstrap_N_tenth'
									matrix frequency_est[`row'+10, `col'] = `late_se_fifth'
									matrix frequency_est[`row'+11, `col'] = `fs_se_fifth'
									matrix frequency_est[`row'+12, `col'] = `bstrap_N_fifth'
									
				
									restore
									}
									
									} /* closes bs method */
									} /* closes risk */
								} /* closes frequency */	
														
								else {
									
									local col = 1
									
									foreach risk in r0{
									
									foreach bs_method in bayesian /*nonparametric*/ {
									
									preserve
								
									keep if group == `group' & in_good_cell_`risk' == 1
									egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
									keep if temp == 0
									drop temp
									
									gen one = 1
									
									local controls "`risk'_* `cov' `cov2' `cov_grade' baselinespyear"
								
									ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("one")
									local late = `r(late)'
									local rf = `r(rf)'
									local fs = `r(fs)'
									local rf_N = `r(rf_N)'
									local fs_N = `r(fs_N)'
									
									save "`cleandata'weighted_est_temp_stacked.dta", replace
									
									set seed 123
									mat bootstrap_est = J(1, 2, .)

									
									if "`bs_method'" == "bayesian" {
									
									forval b = 1/1000 {
										gen bs_wgt = rgamma(1,1)
										ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("bs_wgt")
										mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))	
										use "`cleandata'weighted_est_temp_stacked.dta", clear
									}
									svmat bootstrap_est
									
									gen bootstrap_est2_tenth = bootstrap_est2
									gen bootstrap_est1_tenth = bootstrap_est1
									
									gen bootstrap_est2_fifth = bootstrap_est2
									gen bootstrap_est1_fifth = bootstrap_est1
									
									gen bootstrap_est2_tmp = bootstrap_est2
									** When bootstrapped, there could be FS that are too small. Drop them.
									replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
									replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
									
									replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
									replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
									
									replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
									replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

									drop bootstrap_est2_tmp
								
									su bootstrap_est1
									local bstrap_N = `r(N)'
									local late_se = `r(sd)' 
									
									su bootstrap_est2
									local fs_bstrap_N = `r(N)'
									local fs_se = `r(sd)'
									
									su bootstrap_est1_tenth
									local bstrap_N_tenth = `r(N)'
									local late_se_tenth = `r(sd)' 
									
									
									su bootstrap_est2_tenth
									local fs_bstrap_N_tenth = `r(N)'
									local fs_se_tenth = `r(sd)'
									
									su bootstrap_est1_fifth 
									local bstrap_N_fifth = `r(N)'								
									local late_se_fifth = `r(sd)'
									
									su bootstrap_est2_fifth
									local fs_bstrap_N_fifth = `r(N)'
									local fs_se_fifth = `r(sd)'
									
									
									matrix formula_bayesian_est[`row', `col'] = `late'
									matrix formula_bayesian_est[`row'+1, `col'] = `late_se'
									matrix formula_bayesian_est[`row'+2, `col'] = `late_se_tenth'
									matrix formula_bayesian_est[`row'+3, `col'] = `fs'
									matrix formula_bayesian_est[`row'+4, `col'] = `fs_se'
									matrix formula_bayesian_est[`row'+5, `col'] = `fs_se_tenth'
									matrix formula_bayesian_est[`row'+6, `col'] = `rf_N'
									matrix formula_bayesian_est[`row'+7, `col'] = `fs_N'
									matrix formula_bayesian_est[`row'+8, `col'] = `bstrap_N'
									matrix formula_bayesian_est[`row'+9, `col'] = `bstrap_N_tenth'
									matrix formula_bayesian_est[`row'+10, `col'] = `late_se_fifth'
									matrix formula_bayesian_est[`row'+11, `col'] = `fs_se_fifth'
									matrix formula_bayesian_est[`row'+12, `col'] = `bstrap_N_fifth'
									
								
									restore
									}
									
									else {
									forval b = 1/1000 {
										bsample
										*sample 50
										ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("one")
										mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))	
										use "`cleandata'weighted_est_temp_stacked.dta", clear
									}
									svmat bootstrap_est
				
									gen bootstrap_est2_tenth = bootstrap_est2
									gen bootstrap_est1_tenth = bootstrap_est1
									
									gen bootstrap_est2_fifth = bootstrap_est2
									gen bootstrap_est1_fifth = bootstrap_est1
									
									gen bootstrap_est2_tmp = bootstrap_est2
									** When bootstrapped, there could be FS that are too small. Drop them.
									replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
									replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
									
									replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
									replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
									
									replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
									replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

									drop bootstrap_est2_tmp
								
									su bootstrap_est1
									local bstrap_N = `r(N)'
									local late_se = `r(sd)' 
									
									su bootstrap_est2
									local fs_bstrap_N = `r(N)'
									local fs_se = `r(sd)'
									
									su bootstrap_est1_tenth
									local bstrap_N_tenth = `r(N)'
									local late_se_tenth = `r(sd)'
									
									
									su bootstrap_est2_tenth
									local fs_bstrap_N_tenth = `r(N)'
									local fs_se_tenth = `r(sd)' 
									
									su bootstrap_est1_fifth 
									local bstrap_N_fifth = `r(N)'								
									local late_se_fifth = `r(sd)' 
									
									su bootstrap_est2_fifth
									local fs_bstrap_N_fifth = `r(N)'
									local fs_se_fifth = `r(sd)' 
									
									
									matrix formula_est[`row', `col'] = `late'
									matrix formula_est[`row'+1, `col'] = `late_se'
									matrix formula_est[`row'+2, `col'] = `late_se_tenth'
									matrix formula_est[`row'+3, `col'] = `fs'
									matrix formula_est[`row'+4, `col'] = `fs_se'
									matrix formula_est[`row'+5, `col'] = `fs_se_tenth'
									matrix formula_est[`row'+6, `col'] = `rf_N'
									matrix formula_est[`row'+7, `col'] = `fs_N'
									matrix formula_est[`row'+8, `col'] = `bstrap_N'
									matrix formula_est[`row'+9, `col'] = `bstrap_N_tenth'
									matrix formula_est[`row'+10, `col'] = `late_se_fifth'
									matrix formula_est[`row'+11, `col'] = `fs_se_fifth'
									matrix formula_est[`row'+12, `col'] = `bstrap_N_fifth'
									
				
									restore
									}
									} /* closes bs method */
									} /* closes risk */
								} /* closes formula */	
							}
							
							local row = `row' + 14
						} /* closes out */
						local ++row
					
					} /* closes t */
				} /* closes group */
			} /* closes method */ 
		} /* closes file */
			preserve
			
			svmat simulation_est
			keep var simulation_est*
			ren simulation_est1 r01
			
			outsheet using "`tables'simulation_weighting_estimates_nonparm_stacked.csv", comma replace

			restore
			
			preserve
		
			svmat frequency_est
			keep var frequency_est*
			ren frequency_est1 r0
			
			outsheet using "`tables'frequency_weighting_estimates_nonparm_stacked.csv", comma replace
	
			restore
	
			preserve
		
			svmat formula_est
			keep var formula_est*
			ren formula_est1 r0
			
			outsheet using "`tables'formula_weighting_estimates_nonparm_stacked.csv", comma replace
		
			restore
			
			preserve
			
			svmat simulation_bayesian_est
			keep var simulation_bayesian_est*
			ren simulation_bayesian_est1 r01
			
			outsheet using "`tables'simulation_bayesian_bs_stacked.csv", comma replace

			restore
			
			preserve
		
			svmat frequency_bayesian_est
			keep var frequency_bayesian_est*
			ren frequency_bayesian_est1 r0
			
			outsheet using "`tables'frequency_bayesian_bs_stacked.csv", comma replace
	
			restore
	
			preserve
		
			svmat formula_bayesian_est
			keep var formula_bayesian_est*
			ren formula_bayesian_est1 r0
			
			outsheet using "`tables'formula_bayesian_bs_stacked.csv", comma replace
		
			restore
} 







	
		
	*****************************************************************************************************************
	******* pscore weighting estimators (bayesian & nonparm) using full type controls when computing the pscore *****
	*****************************************************************************************************************
	
	if `pscore_weighting_est_fulltype'==1 {
		
	capture: program drop ciest_fuzzy
	program ciest_fuzzy, rclass
		version 11.0
		syntax varlist(fv), outcome(string) method(string) wgt(string)
		
		noisily {
			if ("`method'" == "abadie") {
				capture: xi: logit Z `varlist' [pw=`wgt'] if `outcome' != ., iterate(100)
				
				if (_rc == 0 & `e(converged)' == 1) {
					
					predict pred_p if `outcome' != .
				
				gen w = (Z - pred_p) / ((1 - pred_p) * pred_p)
				gen w_y = w*`outcome'
				
				gen kappa = 1 - ((D*(1 - Z)) / (1 - pred_p)) - (((1 - D) * Z) / pred_p)
				
				su w_y [aw=`wgt']
				local rf = r(mean)
				local rf_N = r(N)
				
				su kappa [aw=`wgt']
				local fs = r(mean)
				local fs_N = r(N)
				
				local late = `rf' / `fs'
				
				drop pred_p w w_y kappa
				}
				else {
				local rf = .
				local fs = .
				local late = .
				local rf_N = .
				local fs_N = .
				}
			}
			else if ("`method'" == "hir") {
				capture noisily: xi: logit Z `varlist' [pw=`wgt'] if `outcome' != ., iterate(100)
				if (_rc == 0 & `e(converged)' == 1) {
					predict pred_p if `outcome' != .
					
				gen w = (Z - pred_p) / ((1 - pred_p) * pred_p)
				gen w_D = w*D
				gen w_y = w*`outcome'
				
				su w_D [aw=`wgt']
				local fs = r(mean)
				local fs_N = r(N)
				
				su w_y [aw=`wgt']
				local rf = r(mean)
				local rf_N = r(N)
				
				local late = `rf' / `fs'
				
				drop pred_p w w_D w_y
				}
				else {
				local rf = .
				local fs = .
				local late = .
				local rf_N = .
				local fs_N = .
				}
			}
		}
		return scalar late = `late'
		return scalar rf = `rf'
		return scalar fs = `fs'
		return scalar rf_N = `rf_N'
		return scalar fs_N = `fs_N'

	end
				

		** Calculate the weighting estimates		
		
			u "`cleandata'analysis_fulltype_stacked.dta", clear
			
			
			keep if in_sample_tc == 1
			keep if baselinespyear == 2012 | baselinespyear == 2013

			drop if (maskedid == 674081 | maskedid == 710385) & baselinespyear == 2013
						
			gen var =""
			
			*By grade vars
			gen group = .
			
			local col = 1
			local row = 1
			
		foreach method in /*abadie*/ hir {
		
			foreach group in 410 /*48 910*/ {
						if `group'==410{
							replace group=410
						}
						else if `group'==48{
							replace group=48 if inlist(appgrade,4,5,6,7,8)
						}
						else{
							replace group=910 if inlist(appgrade,9,10)
						}

						foreach t in math writing reading {
							foreach out in ss /*prof_up adv*/ {
							

							replace var="`out'_`t'_`group'_`method'" if _n==`row'
							replace var="`out'_`t'_`group'_`method'_se" if _n==`row'+1
							replace var="`out'_`t'_`group'_`method'_se_tenth" if _n==`row'+2							
							replace var="`out'_`t'_`group'_`method'_fs" if _n==`row'+3
							replace var="`out'_`t'_`group'_`method'_fs_se" if _n==`row'+4
							replace var="`out'_`t'_`group'_`method'_fs_se_tenth" if _n==`row'+5							
							replace var="`out'_`t'_`group'_`method'_N" if _n==`row'+6
							replace var="`out'_`t'_`group'_`method'_fs_N" if _n==`row'+7
							replace var="`out'_`t'_`group'_`method'_bstrap_N" if _n==`row'+8
							replace var="`out'_`t'_`group'_`method'_bstrap_N_tenth" if _n==`row'+9
							replace var="`out'_`t'_`group'_`method'_se_fifth" if _n==`row'+10
							replace var="`out'_`t'_`group'_`method'_fs_se_fifth" if _n==`row'+11
							replace var="`out'_`t'_`group'_`method'_bstrap_N_fifth" if _n==`row'+12

							qui summ `out'_`t' if group==`group' & any_in_sample_tc == 1
							
							if r(max)==0 & r(min)==0{
								di in red "No observations for `group'-level weighting estimates"
							}
						
							else{
								
							local col = 1
								
							foreach bs_method in bayesian /*nonparm*/ {
															
								preserve
							
								keep if group == `group' & in_good_cell_tc == 1
								egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
								keep if temp == 0
								drop temp
								
	
								
								local controls "tag_type_tc_* `cov' `cov2' `cov_grade' baselinespyear"		
								
								gen one = 1
								
								ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("one")
								local late = `r(late)'
								local rf = `r(rf)'
								local fs = `r(fs)'
								local rf_N = `r(rf_N)'
								local fs_N = `r(fs_N)'
								
								save "`cleandata'weighted_est_temp_stacked.dta", replace
								
								
								set seed 123
								mat bootstrap_est = J(1, 2, .)
							
								if "`bs_method'" == "bayesian" {
								
								forval b = 1/500 {
									gen bs_wgt = rgamma(1,1)
									ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("bs_wgt")
									mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))						
									use "`cleandata'weighted_est_temp_stacked.dta", clear
								}
								
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' 
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' 
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)'
								
								
								matrix fulltype_bayesian_est[`row', `col'] = `late'
								matrix fulltype_bayesian_est[`row'+1, `col'] = `late_se'
								matrix fulltype_bayesian_est[`row'+2, `col'] = `late_se_tenth'
								matrix fulltype_bayesian_est[`row'+3, `col'] = `fs'
								matrix fulltype_bayesian_est[`row'+4, `col'] = `fs_se'
								matrix fulltype_bayesian_est[`row'+5, `col'] = `fs_se_tenth'
								matrix fulltype_bayesian_est[`row'+6, `col'] = `rf_N'
								matrix fulltype_bayesian_est[`row'+7, `col'] = `fs_N'
								matrix fulltype_bayesian_est[`row'+8, `col'] = `bstrap_N'
								matrix fulltype_bayesian_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix fulltype_bayesian_est[`row'+10, `col'] = `late_se_fifth'
								matrix fulltype_bayesian_est[`row'+11, `col'] = `fs_se_fifth'
								matrix fulltype_bayesian_est[`row'+12, `col'] = `bstrap_N_fifth'
				
								restore
								}
								
							
								else {
								forval b = 1/500 {
									bsample
									*sample 50
									ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("one")
									mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))						
									use "`cleandata'weighted_est_temp_stacked.dta", clear
								}
								
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' 
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)'
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' 
								
								
								matrix fulltype_nonparm_est[`row', `col'] = `late'
								matrix fulltype_nonparm_est[`row'+1, `col'] = `late_se'
								matrix fulltype_nonparm_est[`row'+2, `col'] = `late_se_tenth'
								matrix fulltype_nonparm_est[`row'+3, `col'] = `fs'
								matrix fulltype_nonparm_est[`row'+4, `col'] = `fs_se'
								matrix fulltype_nonparm_est[`row'+5, `col'] = `fs_se_tenth'
								matrix fulltype_nonparm_est[`row'+6, `col'] = `rf_N'
								matrix fulltype_nonparm_est[`row'+7, `col'] = `fs_N'
								matrix fulltype_nonparm_est[`row'+8, `col'] = `bstrap_N'
								matrix fulltype_nonparm_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix fulltype_nonparm_est[`row'+10, `col'] = `late_se_fifth'
								matrix fulltype_nonparm_est[`row'+11, `col'] = `fs_se_fifth'
								matrix fulltype_nonparm_est[`row'+12, `col'] = `bstrap_N_fifth'
				
								restore
								}
								
								} /* closes bs method */														
							}
							local row = `row' + 14
						} /* closes out */
						local ++row
					} /* closes t */
				} /* closes group */
			} /* closes method */ 
			
			preserve
			
			svmat fulltype_nonparm_est
			keep var fulltype_nonparm_est*
			ren fulltype_nonparm_est1 type_upto_tc
			
			outsheet using "`tables'fulltype_weighting_estimates_nonparm_stacked.csv", comma replace

			restore
			
			preserve
			
			svmat fulltype_bayesian_est
			keep var fulltype_bayesian_est*
			ren fulltype_bayesian_est1 type_upto_tc
			
			outsheet using "`tables'fulltype_bayesian_bs_stacked.csv", comma replace

			restore
} 






	*****************************************************************************************************************
	******* pscore weighting estimators (bayesian & nonparm) using full type controls when computing the pscore *****
	*****************************************************************************************************************
	
	if `pscore_weighting_est_type_nocov'==1 {
		
	capture: program drop ciest_fuzzy
	program ciest_fuzzy, rclass
		version 11.0
		syntax varlist(fv), outcome(string) method(string) wgt(string)
		
		noisily {
			if ("`method'" == "abadie") {
				capture: xi: logit Z `varlist' [pw=`wgt'] if `outcome' != ., iterate(100)
				
				if (_rc == 0 & `e(converged)' == 1) {
					
					predict pred_p if `outcome' != .
				
				gen w = (Z - pred_p) / ((1 - pred_p) * pred_p)
				gen w_y = w*`outcome'
				
				gen kappa = 1 - ((D*(1 - Z)) / (1 - pred_p)) - (((1 - D) * Z) / pred_p)
				
				su w_y [aw=`wgt']
				local rf = r(mean)
				local rf_N = r(N)
				
				su kappa [aw=`wgt']
				local fs = r(mean)
				local fs_N = r(N)
				
				local late = `rf' / `fs'
				
				drop pred_p w w_y kappa
				}
				else {
				local rf = .
				local fs = .
				local late = .
				local rf_N = .
				local fs_N = .
				}
			}
			else if ("`method'" == "hir") {
				capture noisily: xi: logit Z `varlist' [pw=`wgt'] if `outcome' != ., iterate(100)
				if (_rc == 0 & `e(converged)' == 1) {
					predict pred_p if `outcome' != .
					
				gen w = (Z - pred_p) / ((1 - pred_p) * pred_p)
				gen w_D = w*D
				gen w_y = w*`outcome'
				
				su w_D [aw=`wgt']
				local fs = r(mean)
				local fs_N = r(N)
				
				su w_y [aw=`wgt']
				local rf = r(mean)
				local rf_N = r(N)
				
				local late = `rf' / `fs'
				
				drop pred_p w w_D w_y
				}
				else {
				local rf = .
				local fs = .
				local late = .
				local rf_N = .
				local fs_N = .
				}
			}
		}
		return scalar late = `late'
		return scalar rf = `rf'
		return scalar fs = `fs'
		return scalar rf_N = `rf_N'
		return scalar fs_N = `fs_N'

	end
				

		** Calculate the weighting estimates		
		
			u "`cleandata'analysis_fulltype_stacked.dta", clear
			
			
			keep if in_sample_tc == 1
			keep if baselinespyear == 2012 | baselinespyear == 2013

			drop if (maskedid == 674081 | maskedid == 710385) & baselinespyear == 2013
						
			gen var =""
			
			*By grade vars
			gen group = .
			
			local col = 1
			local row = 1
			
		foreach method in /*abadie*/ hir {
		
			foreach group in 410 /*48 910*/ {
						if `group'==410{
							replace group=410
						}
						else if `group'==48{
							replace group=48 if inlist(appgrade,4,5,6,7,8)
						}
						else{
							replace group=910 if inlist(appgrade,9,10)
						}

						foreach t in math writing reading {
							foreach out in ss /*prof_up adv*/ {
							

							replace var="`out'_`t'_`group'_`method'" if _n==`row'
							replace var="`out'_`t'_`group'_`method'_se" if _n==`row'+1
							replace var="`out'_`t'_`group'_`method'_se_tenth" if _n==`row'+2							
							replace var="`out'_`t'_`group'_`method'_fs" if _n==`row'+3
							replace var="`out'_`t'_`group'_`method'_fs_se" if _n==`row'+4
							replace var="`out'_`t'_`group'_`method'_fs_se_tenth" if _n==`row'+5							
							replace var="`out'_`t'_`group'_`method'_N" if _n==`row'+6
							replace var="`out'_`t'_`group'_`method'_fs_N" if _n==`row'+7
							replace var="`out'_`t'_`group'_`method'_bstrap_N" if _n==`row'+8
							replace var="`out'_`t'_`group'_`method'_bstrap_N_tenth" if _n==`row'+9
							replace var="`out'_`t'_`group'_`method'_se_fifth" if _n==`row'+10
							replace var="`out'_`t'_`group'_`method'_fs_se_fifth" if _n==`row'+11
							replace var="`out'_`t'_`group'_`method'_bstrap_N_fifth" if _n==`row'+12

							qui summ `out'_`t' if group==`group'
							
							if r(max)==0 & r(min)==0{
								di in red "No observations for `group'-level weighting estimates"
							}
						
							else{
								
							local col = 1
								
							foreach bs_method in bayesian /*nonparm*/ {
															
								preserve
							
								keep if group == `group' & in_good_cell_tc == 1
								egen temp = rowmiss(`out'_`t' `cov_grade')
								keep if temp == 0
								drop temp
								
	
								
								local controls "tag_type_tc_* `cov_grade' baselinespyear"		
								
								gen one = 1
								
								ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("one")
								local late = `r(late)'
								local rf = `r(rf)'
								local fs = `r(fs)'
								local rf_N = `r(rf_N)'
								local fs_N = `r(fs_N)'
								
								save "`cleandata'weighted_est_temp_stacked.dta", replace
								
								
								set seed 123
								mat bootstrap_est = J(1, 2, .)
							
								if "`bs_method'" == "bayesian" {
								
								forval b = 1/1000 {
									gen bs_wgt = rgamma(1,1)
									ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("bs_wgt")
									mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))						
									use "`cleandata'weighted_est_temp_stacked.dta", clear
								}
								
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' 
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' 
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)'
								
								
								matrix fulltype_bayesian_est[`row', `col'] = `late'
								matrix fulltype_bayesian_est[`row'+1, `col'] = `late_se'
								matrix fulltype_bayesian_est[`row'+2, `col'] = `late_se_tenth'
								matrix fulltype_bayesian_est[`row'+3, `col'] = `fs'
								matrix fulltype_bayesian_est[`row'+4, `col'] = `fs_se'
								matrix fulltype_bayesian_est[`row'+5, `col'] = `fs_se_tenth'
								matrix fulltype_bayesian_est[`row'+6, `col'] = `rf_N'
								matrix fulltype_bayesian_est[`row'+7, `col'] = `fs_N'
								matrix fulltype_bayesian_est[`row'+8, `col'] = `bstrap_N'
								matrix fulltype_bayesian_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix fulltype_bayesian_est[`row'+10, `col'] = `late_se_fifth'
								matrix fulltype_bayesian_est[`row'+11, `col'] = `fs_se_fifth'
								matrix fulltype_bayesian_est[`row'+12, `col'] = `bstrap_N_fifth'
				
								restore
								}
								
							
								else {
								forval b = 1/1000 {
									bsample
									*sample 50
									ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("one")
									mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))						
									use "`cleandata'weighted_est_temp_stacked.dta", clear
								}
								
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' 
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)'
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' 
								
								
								matrix fulltype_nonparm_est[`row', `col'] = `late'
								matrix fulltype_nonparm_est[`row'+1, `col'] = `late_se'
								matrix fulltype_nonparm_est[`row'+2, `col'] = `late_se_tenth'
								matrix fulltype_nonparm_est[`row'+3, `col'] = `fs'
								matrix fulltype_nonparm_est[`row'+4, `col'] = `fs_se'
								matrix fulltype_nonparm_est[`row'+5, `col'] = `fs_se_tenth'
								matrix fulltype_nonparm_est[`row'+6, `col'] = `rf_N'
								matrix fulltype_nonparm_est[`row'+7, `col'] = `fs_N'
								matrix fulltype_nonparm_est[`row'+8, `col'] = `bstrap_N'
								matrix fulltype_nonparm_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix fulltype_nonparm_est[`row'+10, `col'] = `late_se_fifth'
								matrix fulltype_nonparm_est[`row'+11, `col'] = `fs_se_fifth'
								matrix fulltype_nonparm_est[`row'+12, `col'] = `bstrap_N_fifth'
				
								restore
								}
								
								} /* closes bs method */														
							}
							local row = `row' + 14
						} /* closes out */
						local ++row
					} /* closes t */
				} /* closes group */
			} /* closes method */ 
			
			preserve
			
			svmat fulltype_nonparm_est
			keep var fulltype_nonparm_est*
			ren fulltype_nonparm_est1 type_upto_tc
			
			outsheet using "`tables'fulltype_weighting_estimates_nonparm_nocov_stacked.csv", comma replace

			restore
			
			preserve
			
			svmat fulltype_bayesian_est
			keep var fulltype_bayesian_est*
			ren fulltype_bayesian_est1 type_upto_tc
			
			outsheet using "`tables'fulltype_bayesian_bs_nocov_stacked.csv", comma replace

			restore
} 






	*****************************************************************************************
	******* pscore weighting estimators (bayesian & nonparm) without covariate controls *****
	*****************************************************************************************
	
	if `pscore_weighting_est_nocov'==1 {
	

		** Calculate the weighting estimates		
		foreach file in simulation frequency formula  {
		
			u "`cleandata'analysis_stacked_study_`file'.dta", clear
				
			keep if in_sample == 1
			
			keep if baselinespyear == 2012 | baselinespyear == 2013
			* IR 08082016 these kids are outliers 
			*drop if (maskedid == 674081 | maskedid == 710385) & baselinespyear == 2013
			 
			gen var =""
			
			*By grade vars
			gen group = .
			
			local col = 1
			local row = 1
			
			foreach method in abadie hir {
			
				foreach group in 410 /*48 910*/ {
						if `group'==410{
							replace group=410
						}
						else if `group'==48{
							replace group=48 if inlist(appgrade,4,5,6,7,8)
						}
						else{
							replace group=910 if inlist(appgrade,9,10)
						}

						foreach t in math writing reading {
							foreach out in ss /*prof_up adv*/ {
							

							replace var="`out'_`t'_`group'_`method'" if _n==`row'
							replace var="`out'_`t'_`group'_`method'_se" if _n==`row'+1
							replace var="`out'_`t'_`group'_`method'_se_tenth" if _n==`row'+2							
							replace var="`out'_`t'_`group'_`method'_fs" if _n==`row'+3
							replace var="`out'_`t'_`group'_`method'_fs_se" if _n==`row'+4
							replace var="`out'_`t'_`group'_`method'_fs_se_tenth" if _n==`row'+5							
							replace var="`out'_`t'_`group'_`method'_N" if _n==`row'+6
							replace var="`out'_`t'_`group'_`method'_fs_N" if _n==`row'+7
							replace var="`out'_`t'_`group'_`method'_bstrap_N" if _n==`row'+8
							replace var="`out'_`t'_`group'_`method'_bstrap_N_tenth" if _n==`row'+9
							replace var="`out'_`t'_`group'_`method'_se_fifth" if _n==`row'+10
							replace var="`out'_`t'_`group'_`method'_fs_se_fifth" if _n==`row'+11
							replace var="`out'_`t'_`group'_`method'_bstrap_N_fifth" if _n==`row'+12

							qui summ `out'_`t' if group==`group' & in_zero1_pscore==1
							
							if r(max)==0 & r(min)==0{
								di in red "No observations for `group'-level weighting estimates"
							}
						
							else{
							
							if "`file'"=="simulation" {
								
							local col = 1
								
							foreach risk in r01 r001 {
								
							foreach bs_method in bayesian /*nonparm*/ {
															
								preserve

								
									keep if group == `group' & in_good_cell_`risk' == 1
									egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
									keep if temp == 0
									drop temp
										
									gen one = 1
									
									noisily {
										if ("`method'" == "abadie") {
											
											gen w = (Z - `risk') / ((1 - `risk') * `risk')
											gen w_y = w*`out'_`t'
											
											gen kappa = 1 - ((D*(1 - Z)) / (1 - `risk')) - (((1 - D) * Z) / `risk')
											
											su w_y [aw=one] if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .  
											local rf_orig = r(mean)
											local rf_N_orig = r(N)
											
											su kappa [aw=one]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local fs_orig = r(mean)
											local fs_N_orig = r(N)
											
											local late_orig = `rf_orig' / `fs_orig'

											drop w w_y kappa						
											
										}
										else if ("`method'" == "hir") {
												
											gen w = (Z - `risk') / ((1 - `risk') * `risk')
											gen w_D = w*D
											gen w_y = w*`out'_`t'
											
											su w_D [aw=one]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local fs_orig = r(mean)
											local fs_N_orig = r(N)
											
											su w_y [aw=one]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local rf_orig = r(mean)
											local rf_N_orig = r(N)
											
											local late_orig = `rf_orig' / `fs_orig'
											drop w w_D w_y
											
										}
									}		
									
									
									
									save "`cleandata'weighted_est_temp_stacked.dta", replace
									
									
									set seed 351864
									mat bootstrap_est = J(1, 2, .)
								
									if "`bs_method'" == "bayesian" {
									
									forval b = 1/500 {
										gen bs_wgt = rgamma(1,1)
										noisily {
										
											if ("`method'" == "abadie") {
												
												gen w = (Z - `risk') / ((1 - `risk') * `risk')
												gen w_01 = (Z - r01) / ((1 - r01) * r01)
												gen w_y = w*`out'_`t'
												
												gen kappa = 1 - ((D*(1 - Z)) / (1 - `risk')) - (((1 - D) * Z) / `risk')
												
												su w_y [aw=bs_wgt]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
												local rf = r(mean)
												local rf_N = r(N)
												
												su kappa [aw=bs_wgt]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
												local fs = r(mean)
												local fs_N = r(N)
												
												local late = `rf' / `fs'
										
												drop w w_y kappa
									
											}
											
											else if ("`method'" == "hir") {
													
												gen w = (Z - `risk') / ((1 - `risk') * `risk')
												gen w_D = w*D
												gen w_y = w*`out'_`t'
												
												su w_D [aw=bs_wgt]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
												local fs = r(mean)
												local fs_N = r(N)
												
												su w_y [aw=bs_wgt]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
												local rf = r(mean)
												local rf_N = r(N)
												
												local late = `rf' / `fs'
												
												drop w w_D w_y
												
											}
										}
										
										mat bootstrap_est = (bootstrap_est \ (`late', `fs'))						
										use "`cleandata'weighted_est_temp_stacked.dta", clear
									}
									
									svmat bootstrap_est
																											
									gen bootstrap_est2_tenth = bootstrap_est2
									gen bootstrap_est1_tenth = bootstrap_est1
									
									gen bootstrap_est2_fifth = bootstrap_est2
									gen bootstrap_est1_fifth = bootstrap_est1
									

									gen bootstrap_est2_tmp = bootstrap_est2
									** When bootstrapped, there could be FS that are too small. Drop them.
									replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 10)
									replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 10)
									
									replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 5)
									replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 5)
									
									replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 100)
									replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 100)
									

									
									drop bootstrap_est2_tmp
																	
									su bootstrap_est1
									local bstrap_N = `r(N)'
									local late_se = `r(sd)' 
									
									su bootstrap_est2
									local fs_bstrap_N = `r(N)'
									local fs_se = `r(sd)' 
									
									su bootstrap_est1_tenth
									local bstrap_N_tenth = `r(N)'
									local late_se_tenth = `r(sd)' 
									
									
									su bootstrap_est2_tenth
									local fs_bstrap_N_tenth = `r(N)'
									local fs_se_tenth = `r(sd)' 
									
									su bootstrap_est1_fifth 
									local bstrap_N_fifth = `r(N)'								
									local late_se_fifth = `r(sd)' 
									
									su bootstrap_est2_fifth
									local fs_bstrap_N_fifth = `r(N)'
									local fs_se_fifth = `r(sd)'
									
									
									matrix simulation_bayesian_nocov_est[`row', `col'] = `late_orig'
									matrix simulation_bayesian_nocov_est[`row'+1, `col'] = `late_se'
									matrix simulation_bayesian_nocov_est[`row'+2, `col'] = `late_se_tenth'
									matrix simulation_bayesian_nocov_est[`row'+3, `col'] = `fs_orig'
									matrix simulation_bayesian_nocov_est[`row'+4, `col'] = `fs_se'
									matrix simulation_bayesian_nocov_est[`row'+5, `col'] = `fs_se_tenth'
									matrix simulation_bayesian_nocov_est[`row'+6, `col'] = `rf_N_orig'
									matrix simulation_bayesian_nocov_est[`row'+7, `col'] = `fs_N_orig'
									matrix simulation_bayesian_nocov_est[`row'+8, `col'] = `bstrap_N'
									matrix simulation_bayesian_nocov_est[`row'+9, `col'] = `bstrap_N_tenth'
									matrix simulation_bayesian_nocov_est[`row'+10, `col'] = `late_se_fifth'
									matrix simulation_bayesian_nocov_est[`row'+11, `col'] = `fs_se_fifth'
									matrix simulation_bayesian_nocov_est[`row'+12, `col'] = `bstrap_N_fifth'
					
								restore
								}
								
							
								else {
								forval b = 1/500 {
									bsample
									*sample 50
									noisily {
									if ("`method'" == "abadie") {
										
										gen w = (Z - `risk') / ((1 - `risk') * `risk')
										gen w_y = w*`out'_`t'
										
										gen kappa = 1 - ((D*(1 - Z)) / (1 - `risk')) - (((1 - D) * Z) / `risk')
										
										su w_y [aw=one]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
										local rf = r(mean)
										local rf_N = r(N)
										
										su kappa [aw=one]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
										local fs = r(mean)
										local fs_N = r(N)
										
										local late = `rf' / `fs'
										
										drop w w_y kappa
							
									}
									else if ("`method'" == "hir") {
											
										gen w = (Z - `risk') / ((1 - `risk') * `risk')
										gen w_D = w*D
										gen w_y = w*`out'_`t'
										
										su w_D [aw=one]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
										local fs = r(mean)
										local fs_N = r(N)
										
										su w_y [aw=one]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
										local rf = r(mean)
										local rf_N = r(N)
										
										local late = `rf' / `fs'
										
										drop w w_D w_y
										
									}
								}				
									mat bootstrap_est = (bootstrap_est \ (`late', `fs'))						
									use "`cleandata'weighted_est_temp_stacked.dta", clear
								}
								
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' 
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)'
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' 
								
								
								matrix simulation_nocov_est[`row', `col'] = `late_orig'
								matrix simulation_nocov_est[`row'+1, `col'] = `late_se'
								matrix simulation_nocov_est[`row'+2, `col'] = `late_se_tenth'
								matrix simulation_nocov_est[`row'+3, `col'] = `fs_orig'
								matrix simulation_nocov_est[`row'+4, `col'] = `fs_se'
								matrix simulation_nocov_est[`row'+5, `col'] = `fs_se_tenth'
								matrix simulation_nocov_est[`row'+6, `col'] = `rf_N_orig'
								matrix simulation_nocov_est[`row'+7, `col'] = `fs_N_orig'
								matrix simulation_nocov_est[`row'+8, `col'] = `bstrap_N'
								matrix simulation_nocov_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix simulation_nocov_est[`row'+10, `col'] = `late_se_fifth'
								matrix simulation_nocov_est[`row'+11, `col'] = `fs_se_fifth'
								matrix simulation_nocov_est[`row'+12, `col'] = `bstrap_N_fifth'
				
								restore
								}
								
								} /* closes bs method */
								
								local col = `col' + 1
								} /* closes risk */
							} /* closes simulation */	
								
								
							else if "`file'"=="frequency" {
								
								local col = 1
								
								foreach risk in r0{
								
								foreach bs_method in bayesian /*nonparametric*/ {
								
								preserve
							
								keep if group == `group' & in_good_cell_`risk' == 1
								egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
								keep if temp == 0
								drop temp

								gen one = 1
																
								noisily {
									if ("`method'" == "abadie") {
										
										gen w = (Z - `risk') / ((1 - `risk') * `risk')
										gen w_y = w*`out'_`t'
										
										gen kappa = 1 - ((D*(1 - Z)) / (1 - `risk')) - (((1 - D) * Z) / `risk')
										
										su w_y [aw=one]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
										local rf_orig = r(mean)
										local rf_N_orig = r(N)
										
										su kappa [aw=one]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
										local fs_orig = r(mean)
										local fs_N_orig = r(N)
										
										local late_orig = `rf_orig' / `fs_orig'
										
										drop w w_y kappa
							
									}
									else if ("`method'" == "hir") {
											
										gen w = (Z - `risk') / ((1 - `risk') * `risk')
										gen w_D_orig = w*D
										gen w_y_orig = w*`out'_`t'
										
										su w_D [aw=one]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
										local fs_orig = r(mean)
										local fs_N_orig = r(N)
										
										su w_y [aw=one]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
										local rf_orig = r(mean)
										local rf_N_orig = r(N)
										
										local late_orig = `rf_orig' / `fs_orig'
										
										drop w w_D w_y
										
									}
								}									
								save "`cleandata'weighted_est_temp_stacked.dta", replace
								
								set seed 123
								mat bootstrap_est = J(1, 2, .)
								
															
								if "`bs_method'" == "bayesian" {
								
								forval b = 1/500 {
									gen bs_wgt = rgamma(1,1)
									noisily {
										if ("`method'" == "abadie") {
											
											gen w = (Z - `risk') / ((1 - `risk') * `risk')
											gen w_y = w*`out'_`t'
											
											gen kappa = 1 - ((D*(1 - Z)) / (1 - `risk')) - (((1 - D) * Z) / `risk')
											
											su w_y [aw=bs_wgt]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local rf = r(mean)
											local rf_N = r(N)
											
											su kappa [aw=bs_wgt]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local fs = r(mean)
											local fs_N = r(N)
											
											local late = `rf' / `fs'
											
											drop w w_y kappa
								
										}
										else if ("`method'" == "hir") {
												
											gen w = (Z - `risk') / ((1 - `risk') * `risk')
											gen w_D = w*D
											gen w_y = w*`out'_`t'
											
											su w_D [aw=bs_wgt]
											local fs = r(mean)
											local fs_N = r(N)
											
											su w_y [aw=bs_wgt]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local rf = r(mean)
											local rf_N = r(N)
											
											local late = `rf' / `fs'
											
											drop w w_D w_y
											
										}
									}		
									mat bootstrap_est = (bootstrap_est \ (`late', `fs'))						
									use "`cleandata'weighted_est_temp_stacked.dta", clear
								}
								
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)'
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' 
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)'
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' 
								
								
								matrix frequency_bayesian_nocov_est[`row', `col'] = `late_orig'
								matrix frequency_bayesian_nocov_est[`row'+1, `col'] = `late_se'
								matrix frequency_bayesian_nocov_est[`row'+2, `col'] = `late_se_tenth'
								matrix frequency_bayesian_nocov_est[`row'+3, `col'] = `fs_orig'
								matrix frequency_bayesian_nocov_est[`row'+4, `col'] = `fs_se'
								matrix frequency_bayesian_nocov_est[`row'+5, `col'] = `fs_se_tenth'
								matrix frequency_bayesian_nocov_est[`row'+6, `col'] = `rf_N_orig'
								matrix frequency_bayesian_nocov_est[`row'+7, `col'] = `fs_N_orig'
								matrix frequency_bayesian_nocov_est[`row'+8, `col'] = `bstrap_N'
								matrix frequency_bayesian_nocov_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix frequency_bayesian_nocov_est[`row'+10, `col'] = `late_se_fifth'
								matrix frequency_bayesian_nocov_est[`row'+11, `col'] = `fs_se_fifth'
								matrix frequency_bayesian_nocov_est[`row'+12, `col'] = `bstrap_N_fifth'
				
								restore
								}
								
								else {
								forval b = 1/500 {
									bsample
									*sample 50
									noisily {
										if ("`method'" == "abadie") {
											
											gen w = (Z - `risk') / ((1 - `risk') * `risk')
											gen w_y = w*`out'_`t'
											
											gen kappa = 1 - ((D*(1 - Z)) / (1 - `risk')) - (((1 - D) * Z) / `risk')
											
											su w_y [aw=one] if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != . 
											local rf = r(mean)
											local rf_N = r(N)
											
											su kappa [aw=one] if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != . 
											local fs = r(mean)
											local fs_N = r(N)
											
											local late = `rf' / `fs'
											
											drop w w_y kappa
								
										}
										else if ("`method'" == "hir") {
												
											gen w = (Z - `risk') / ((1 - `risk') * `risk')
											gen w_D = w*D
											gen w_y = w*`out'_`t'
											
											su w_D [aw=one] if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != . 
											local fs = r(mean)
											local fs_N = r(N)
											
											su w_y [aw=one] if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != . 
											local rf = r(mean)
											local rf_N = r(N)
											
											local late = `rf' / `fs'
											
											drop w w_D w_y
											
										}
									}		
									mat bootstrap_est = (bootstrap_est \ (`late', `fs'))	
									use "`cleandata'weighted_est_temp_stacked.dta", clear
								}
								svmat bootstrap_est
			
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)'
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)'
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' 
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' 
								
								
								matrix frequency_nocov_est[`row', `col'] = `late_orig'
								matrix frequency_nocov_est[`row'+1, `col'] = `late_se'
								matrix frequency_nocov_est[`row'+2, `col'] = `late_se_tenth'
								matrix frequency_nocov_est[`row'+3, `col'] = `fs_orig'
								matrix frequency_nocov_est[`row'+4, `col'] = `fs_se'
								matrix frequency_nocov_est[`row'+5, `col'] = `fs_se_tenth'
								matrix frequency_nocov_est[`row'+6, `col'] = `rf_N_orig'
								matrix frequency_nocov_est[`row'+7, `col'] = `fs_N_orig'
								matrix frequency_nocov_est[`row'+8, `col'] = `bstrap_N'
								matrix frequency_nocov_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix frequency_nocov_est[`row'+10, `col'] = `late_se_fifth'
								matrix frequency_nocov_est[`row'+11, `col'] = `fs_se_fifth'
								matrix frequency_nocov_est[`row'+12, `col'] = `bstrap_N_fifth'
								
			
								restore
								}
								
								} /* closes bs method */
								} /* closes risk */
							} /* closes frequency */	
													
							else {
								
								local col = 1
								
								foreach risk in r0{
								
								foreach bs_method in bayesian nonparametric {
								
								preserve
							
								keep if group == `group' & in_good_cell_`risk' == 1
								egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
								keep if temp == 0
								drop temp
								
								gen one = 1
															
								noisily {
										if ("`method'" == "abadie") {
											
											gen w = (Z - `risk') / ((1 - `risk') * `risk')
											gen w_y = w*`out'_`t'
											
											gen kappa = 1 - ((D*(1 - Z)) / (1 - `risk')) - (((1 - D) * Z) / `risk')
											
											su w_y [aw=one] if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != . 
											local rf_orig = r(mean)
											local rf_N_orig = r(N)
											
											su kappa [aw=one] if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != . 
											local fs_orig = r(mean)
											local fs_N_orig = r(N)
											
											local late_orig = `rf_orig' / `fs_orig'
											
											drop w w_y kappa
								
										}
										else if ("`method'" == "hir") {
												
											gen w = (Z - `risk') / ((1 - `risk') * `risk')
											gen w_D = w*D
											gen w_y = w*`out'_`t'
											
											su w_D [aw=one] if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != . 
											local fs_orig = r(mean)
											local fs_N_orig = r(N)
											
											su w_y [aw=one] if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != . 
											local rf_orig = r(mean)
											local rf_N_orig = r(N)
											
											local late_orig = `rf_orig' / `fs_orig'
											
											drop w w_D w_y
											
										}
									}		
								
								save "`cleandata'weighted_est_temp_stacked.dta", replace
								
								set seed 123
								mat bootstrap_est = J(1, 2, .)

								
								if "`bs_method'" == "bayesian" {
								
								forval b = 1/500 {
									gen bs_wgt = rgamma(1,1)
									noisily {
										if ("`method'" == "abadie") {
											
											gen w = (Z - `risk') / ((1 - `risk') * `risk')
											gen w_y = w*`out'_`t'
											
											gen kappa = 1 - ((D*(1 - Z)) / (1 - `risk')) - (((1 - D) * Z) / `risk')
											
											su w_y [aw=bs_wgt]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local rf = r(mean)
											local rf_N = r(N)
											
											su kappa [aw=bs_wgt]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local fs = r(mean)
											local fs_N = r(N)
											
											local late = `rf' / `fs'
											
											drop w w_y kappa
								
										}
										else if ("`method'" == "hir") {
												
											gen w = (Z - `risk') / ((1 - `risk') * `risk')
											gen w_D = w*D
											gen w_y = w*`out'_`t'
											
											su w_D [aw=bs_wgt]   if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local fs = r(mean)
											local fs_N = r(N)
											
											su w_y [aw=bs_wgt]  if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local rf = r(mean)
											local rf_N = r(N)
											
											local late = `rf' / `fs'
											
											drop w w_D w_y
											
										}
									}		
									mat bootstrap_est = (bootstrap_est \ (`late', `fs'))	
									use "`cleandata'weighted_est_temp_stacked.dta", clear
								}
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)'
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)'
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)'
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)'
								
								
								matrix formula_bayesian_nocov_est[`row', `col'] = `late_orig'
								matrix formula_bayesian_nocov_est[`row'+1, `col'] = `late_se'
								matrix formula_bayesian_nocov_est[`row'+2, `col'] = `late_se_tenth'
								matrix formula_bayesian_nocov_est[`row'+3, `col'] = `fs_orig'
								matrix formula_bayesian_nocov_est[`row'+4, `col'] = `fs_se'
								matrix formula_bayesian_nocov_est[`row'+5, `col'] = `fs_se_tenth'
								matrix formula_bayesian_nocov_est[`row'+6, `col'] = `rf_N_orig'
								matrix formula_bayesian_nocov_est[`row'+7, `col'] = `fs_N_orig'
								matrix formula_bayesian_nocov_est[`row'+8, `col'] = `bstrap_N'
								matrix formula_bayesian_nocov_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix formula_bayesian_nocov_est[`row'+10, `col'] = `late_se_fifth'
								matrix formula_bayesian_nocov_est[`row'+11, `col'] = `fs_se_fifth'
								matrix formula_bayesian_nocov_est[`row'+12, `col'] = `bstrap_N_fifth'
								
							
								restore
								}
								
								else {
								forval b = 1/500 {
									bsample
									*sample 50
									noisily {
										if ("`method'" == "abadie") {
											
											gen w = (Z - `risk') / ((1 - `risk') * `risk')
											gen w_y = w*`out'_`t'
											
											gen kappa = 1 - ((D*(1 - Z)) / (1 - `risk')) - (((1 - D) * Z) / `risk')
											
											su w_y [aw=one]   if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local rf = r(mean)
											local rf_N = r(N)
											
											su kappa [aw=one]   if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local fs = r(mean)
											local fs_N = r(N)
											
											local late = `rf' / `fs'
											
											drop w w_y kappa
								
										}
										else if ("`method'" == "hir") {
												
											gen w = (Z - `risk') / ((1 - `risk') * `risk')
											gen w_D = w*D
											gen w_y = w*`out'_`t'
											
											su w_D [aw=one]   if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local fs = r(mean)
											local fs_N = r(N)
											
											su w_y [aw=one]   if `risk' < ( 1 - `trimm_diff') & `risk'> `trimm_diff' & !inlist(`risk', 0,1) & `risk' != .
											local rf = r(mean)
											local rf_N = r(N)
											
											local late = `rf' / `fs'
											
											drop w w_D w_y
											
										}
									}		
									mat bootstrap_est = (bootstrap_est \ (`late', `fs'))	
									use "`cleandata'weighted_est_temp_stacked.dta", clear
								}
								svmat bootstrap_est
			
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_orig' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)'
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)'
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' 
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' 
								
								
								matrix formula_nocov_est[`row', `col'] = `late_orig'
								matrix formula_nocov_est[`row'+1, `col'] = `late_se'
								matrix formula_nocov_est[`row'+2, `col'] = `late_se_tenth'
								matrix formula_nocov_est[`row'+3, `col'] = `fs_orig'
								matrix formula_nocov_est[`row'+4, `col'] = `fs_se'
								matrix formula_nocov_est[`row'+5, `col'] = `fs_se_tenth'
								matrix formula_nocov_est[`row'+6, `col'] = `rf_N_orig'
								matrix formula_nocov_est[`row'+7, `col'] = `fs_N_orig'
								matrix formula_nocov_est[`row'+8, `col'] = `bstrap_N'
								matrix formula_nocov_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix formula_nocov_est[`row'+10, `col'] = `late_se_fifth'
								matrix formula_nocov_est[`row'+11, `col'] = `fs_se_fifth'
								matrix formula_nocov_est[`row'+12, `col'] = `bstrap_N_fifth'
								
			
								restore
								}
								} /* closes bs method */
								} /* closes risk */
							} /* closes formula */	
							}
							
							local row = `row' + 14
						} /* closes out */
						local ++row
					
					} /* closes t */
				} /* closes group */
			} /* closes method */ 
		} /* closes file */
			preserve
			
			svmat simulation_nocov_est
			keep var simulation_nocov_est*
			ren simulation_nocov_est1 r01
			ren simulation_nocov_est2 r001
			
			outsheet using "`tables'simulation_weighting_estimates_nonparm_nocov.csv", comma replace

			restore
			
			preserve
		
			svmat frequency_nocov_est
			keep var frequency_nocov_est*
			ren frequency_nocov_est1 r0
			
			outsheet using "`tables'frequency_weighting_estimates_nonparm_nocov.csv", comma replace
	
			restore
	
			preserve
		
			svmat formula_nocov_est
			keep var formula_nocov_est*
			ren formula_nocov_est1 r0
			
			outsheet using "`tables'formula_weighting_estimates_nonparm_nocov.csv", comma replace
		
			restore
			
			preserve
			
			svmat simulation_bayesian_nocov_est
			keep var simulation_bayesian_nocov_est*
			ren simulation_bayesian_nocov_est1 r01
			ren simulation_bayesian_nocov_est2 r001
			
			outsheet using "`tables'simulation_bayesian_bs_nocov.csv", comma replace

			restore
			
			preserve
		
			svmat frequency_bayesian_nocov_est
			keep var frequency_bayesian_nocov_est*
			ren frequency_bayesian_nocov_est1 r0
			
			outsheet using "`tables'frequency_bayesian_bs_nocov.csv", comma replace
	
			restore
	
			preserve
		
			svmat formula_bayesian_nocov_est
			keep var formula_bayesian_nocov_est*
			ren formula_bayesian_nocov_est1 r0
			
			outsheet using "`tables'formula_bayesian_bs_nocov.csv", comma replace
		
			restore
} 








		

		

/*
	*****************************************
	******* pscore weighting estimators (nonparametric bootstrapping) *****
	*****************************************
	
	if `pscore_weighting_est_nonparm'==1 {
		
	capture: program drop ciest_fuzzy
	program ciest_fuzzy, rclass
		version 11.0
		syntax varlist(fv), outcome(string) method(string)
		
		noisily {
			if ("`method'" == "abadie") {
				capture: xi: logit Z `varlist' if `outcome' != ., iterate(100)
				
				if (_rc == 0 & `e(converged)' == 1) {
					
					predict pred_p if `outcome' != .
				
				gen w = (Z - pred_p) / ((1 - pred_p) * pred_p)
				gen w_y = w*`outcome'
				
				gen kappa = 1 - ((D*(1 - Z)) / (1 - pred_p)) - (((1 - D) * Z) / pred_p)
				
				su w_y 
				local rf = r(mean)
				local rf_N = r(N)
				
				su kappa
				local fs = r(mean)
				local fs_N = r(N)
				
				local late = `rf' / `fs'
				
				drop pred_p w w_y kappa
				}
				else {
				local rf = .
				local fs = .
				local late = .
				local rf_N = .
				local fs_N = .
				}
			}
			else if ("`method'" == "hir") {
				capture noisily: xi: logit Z `varlist' if `outcome' != ., iterate(100)
				if (_rc == 0 & `e(converged)' == 1) {
					predict pred_p if `outcome' != .
					
				gen w = (Z - pred_p) / ((1 - pred_p) * pred_p)
				gen w_D = w*D
				gen w_y = w*`outcome'
				
				su w_D
				local fs = r(mean)
				local fs_N = r(N)
				
				su w_y
				local rf = r(mean)
				local rf_N = r(N)
				
				local late = `rf' / `fs'
				
				drop pred_p w w_D w_y
				}
				else {
				local rf = .
				local fs = .
				local late = .
				local rf_N = .
				local fs_N = .
				}
			}
		}
		return scalar late = `late'
		return scalar rf = `rf'
		return scalar fs = `fs'
		return scalar rf_N = `rf_N'
		return scalar fs_N = `fs_N'

	end
				

		** Calculate the weighting estimates		
		foreach file in /*simulation*/ frequency formula {
		
			u "`cleandata'analysis_`file'.dta", clear
				
			keep if in_sample == 1
		
			gen var =""
			
			*By grade vars
			gen group = .
			
			local col = 1
			local row = 1
			
		foreach method in abadie hir {
		
			foreach group in 410 /*48 910*/ {
						if `group'==410{
							replace group=410
						}
						else if `group'==48{
							replace group=48 if inlist(appgrade,4,5,6,7,8)
						}
						else{
							replace group=910 if inlist(appgrade,9,10)
						}

						foreach t in math writing reading {
							foreach out in ss /*prof_up adv*/ {
							

							replace var="`out'_`t'_`group'_`method'" if _n==`row'
							replace var="`out'_`t'_`group'_`method'_se" if _n==`row'+1
							replace var="`out'_`t'_`group'_`method'_se_tenth" if _n==`row'+2							
							replace var="`out'_`t'_`group'_`method'_fs" if _n==`row'+3
							replace var="`out'_`t'_`group'_`method'_fs_se" if _n==`row'+4
							replace var="`out'_`t'_`group'_`method'_fs_se_tenth" if _n==`row'+5							
							replace var="`out'_`t'_`group'_`method'_N" if _n==`row'+6
							replace var="`out'_`t'_`group'_`method'_fs_N" if _n==`row'+7
							replace var="`out'_`t'_`group'_`method'_bstrap_N" if _n==`row'+8
							replace var="`out'_`t'_`group'_`method'_bstrap_N_tenth" if _n==`row'+9
							replace var="`out'_`t'_`group'_`method'_se_fifth" if _n==`row'+10
							replace var="`out'_`t'_`group'_`method'_fs_se_fifth" if _n==`row'+11
							replace var="`out'_`t'_`group'_`method'_bstrap_N_fifth" if _n==`row'+12

							qui summ `out'_`t' if group==`group' & in_zero1_pscore==1
							
							if r(max)==0 & r(min)==0{
								di in red "No observations for `group'-level weighting estimates"
							}
						
							else{
							
							if "`file'"=="simulation" {
								
							local col = 1
								
							foreach risk in r01 {
								
							foreach bs_method in /*bayesian*/ nonparm {
															
								preserve
							
								keep if group == `group' & in_good_cell_`risk' == 1
								egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
								keep if temp == 0
								drop temp
								
	
								
								local controls "`risk'_* `cov' `cov2' `cov_grade'"		
								
								
								ciest_fuzzy `controls', outcome(`out'_`t') method(`method')
								local late = `r(late)'
								local rf = `r(rf)'
								local fs = `r(fs)'
								local rf_N = `r(rf_N)'
								local fs_N = `r(fs_N)'
								
								save "`cleandata'weighted_est_temp.dta", replace
								
								
								set seed 123
								mat bootstrap_est = J(1, 2, .)
							
								if "`bs_method'" == "bayesian" {
								
								forval b = 1/500 {
									gen bs_wgt = rgamma(1,1)
									ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("bs_wgt")
									mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))						
									use "`cleandata'weighted_est_temp.dta", clear
								}
								
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' 
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' 
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)'
								
								
								matrix simulation_bayesian_est[`row', `col'] = `late'
								matrix simulation_bayesian_est[`row'+1, `col'] = `late_se'
								matrix simulation_bayesian_est[`row'+2, `col'] = `late_se_tenth'
								matrix simulation_bayesian_est[`row'+3, `col'] = `fs'
								matrix simulation_bayesian_est[`row'+4, `col'] = `fs_se'
								matrix simulation_bayesian_est[`row'+5, `col'] = `fs_se_tenth'
								matrix simulation_bayesian_est[`row'+6, `col'] = `rf_N'
								matrix simulation_bayesian_est[`row'+7, `col'] = `fs_N'
								matrix simulation_bayesian_est[`row'+8, `col'] = `bstrap_N'
								matrix simulation_bayesian_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix simulation_bayesian_est[`row'+10, `col'] = `late_se_fifth'
								matrix simulation_bayesian_est[`row'+11, `col'] = `fs_se_fifth'
								matrix simulation_bayesian_est[`row'+12, `col'] = `bstrap_N_fifth'
				
								restore
								}
								
							
								else {
								forval b = 1/500 {
									bsample
									*sample 50
									ciest_fuzzy `controls', outcome(`out'_`t') method(`method')
									mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))						
									use "`cleandata'weighted_est_temp.dta", clear
								}
								
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' 
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)'
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' 
								
								
								matrix simulation_est[`row', `col'] = `late'
								matrix simulation_est[`row'+1, `col'] = `late_se'
								matrix simulation_est[`row'+2, `col'] = `late_se_tenth'
								matrix simulation_est[`row'+3, `col'] = `fs'
								matrix simulation_est[`row'+4, `col'] = `fs_se'
								matrix simulation_est[`row'+5, `col'] = `fs_se_tenth'
								matrix simulation_est[`row'+6, `col'] = `rf_N'
								matrix simulation_est[`row'+7, `col'] = `fs_N'
								matrix simulation_est[`row'+8, `col'] = `bstrap_N'
								matrix simulation_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix simulation_est[`row'+10, `col'] = `late_se_fifth'
								matrix simulation_est[`row'+11, `col'] = `fs_se_fifth'
								matrix simulation_est[`row'+12, `col'] = `bstrap_N_fifth'
				
								restore
								}
								
								} /* closes bs method */								
								} /* closes risk */
							} /* closes simulation */	
								
								
							else if "`file'"=="frequency" {
								
								local col = 1
								
								foreach risk in r0{
								
								foreach bs_method in /*bayesian*/ nonparametric {
								
								preserve
							
								keep if group == `group' & in_good_cell_`risk' == 1
								egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
								keep if temp == 0
								drop temp
								
								local controls "`risk'_* `cov' `cov2' `cov_grade'"
								
								ciest_fuzzy `controls', outcome(`out'_`t') method(`method')
								local late = `r(late)'
								local rf = `r(rf)'
								local fs = `r(fs)'
								local rf_N = `r(rf_N)'
								local fs_N = `r(fs_N)'
								
								save "`cleandata'weighted_est_temp.dta", replace
								
								set seed 123
								mat bootstrap_est = J(1, 2, .)
								
															
								if "`bs_method'" == "bayesian" {
								
								forval b = 1/500 {
									gen bs_wgt = rgamma(1,1)
									ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("bs_wgt")
									mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))						
									use "`cleandata'weighted_est_temp.dta", clear
								}
								
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)'
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' 
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)'
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' 
								
								
								matrix frequency_bayesian_est[`row', `col'] = `late'
								matrix frequency_bayesian_est[`row'+1, `col'] = `late_se'
								matrix frequency_bayesian_est[`row'+2, `col'] = `late_se_tenth'
								matrix frequency_bayesian_est[`row'+3, `col'] = `fs'
								matrix frequency_bayesian_est[`row'+4, `col'] = `fs_se'
								matrix frequency_bayesian_est[`row'+5, `col'] = `fs_se_tenth'
								matrix frequency_bayesian_est[`row'+6, `col'] = `rf_N'
								matrix frequency_bayesian_est[`row'+7, `col'] = `fs_N'
								matrix frequency_bayesian_est[`row'+8, `col'] = `bstrap_N'
								matrix frequency_bayesian_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix frequency_bayesian_est[`row'+10, `col'] = `late_se_fifth'
								matrix frequency_bayesian_est[`row'+11, `col'] = `fs_se_fifth'
								matrix frequency_bayesian_est[`row'+12, `col'] = `bstrap_N_fifth'
				
								restore
								}
								
								else {
								forval b = 1/500 {
									bsample
									*sample 50
									ciest_fuzzy `controls', outcome(`out'_`t') method(`method')
									mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))	
									use "`cleandata'weighted_est_temp.dta", clear
								}
								svmat bootstrap_est
			
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)'
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)'
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' 
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' 
								
								
								matrix frequency_est[`row', `col'] = `late'
								matrix frequency_est[`row'+1, `col'] = `late_se'
								matrix frequency_est[`row'+2, `col'] = `late_se_tenth'
								matrix frequency_est[`row'+3, `col'] = `fs'
								matrix frequency_est[`row'+4, `col'] = `fs_se'
								matrix frequency_est[`row'+5, `col'] = `fs_se_tenth'
								matrix frequency_est[`row'+6, `col'] = `rf_N'
								matrix frequency_est[`row'+7, `col'] = `fs_N'
								matrix frequency_est[`row'+8, `col'] = `bstrap_N'
								matrix frequency_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix frequency_est[`row'+10, `col'] = `late_se_fifth'
								matrix frequency_est[`row'+11, `col'] = `fs_se_fifth'
								matrix frequency_est[`row'+12, `col'] = `bstrap_N_fifth'
								
			
								restore
								}
								
								} /* closes bs method */
								} /* closes risk */
							} /* closes frequency */	
													
							else {
								
								local col = 1
								
								foreach risk in r0{
								
								foreach bs_method in /*bayesian*/ nonparametric {
								
								preserve
							
								keep if group == `group' & in_good_cell_`risk' == 1
								egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
								keep if temp == 0
								drop temp
																
								local controls "`risk'_* `cov' `cov2' `cov_grade'"
							
								ciest_fuzzy `controls', outcome(`out'_`t') method(`method')
								local late = `r(late)'
								local rf = `r(rf)'
								local fs = `r(fs)'
								local rf_N = `r(rf_N)'
								local fs_N = `r(fs_N)'
								
								save "`cleandata'weighted_est_temp.dta", replace
								
								set seed 123
								mat bootstrap_est = J(1, 2, .)

								
								if "`bs_method'" == "bayesian" {
								
								forval b = 1/500 {
									gen bs_wgt = rgamma(1,1)
									ciest_fuzzy `controls', outcome(`out'_`t') method(`method') wgt("bs_wgt")
									mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))	
									use "`cleandata'weighted_est_temp.dta", clear
								}
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)'
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)'
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)'
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)'
								
								
								matrix formula_bayesian_est[`row', `col'] = `late'
								matrix formula_bayesian_est[`row'+1, `col'] = `late_se'
								matrix formula_bayesian_est[`row'+2, `col'] = `late_se_tenth'
								matrix formula_bayesian_est[`row'+3, `col'] = `fs'
								matrix formula_bayesian_est[`row'+4, `col'] = `fs_se'
								matrix formula_bayesian_est[`row'+5, `col'] = `fs_se_tenth'
								matrix formula_bayesian_est[`row'+6, `col'] = `rf_N'
								matrix formula_bayesian_est[`row'+7, `col'] = `fs_N'
								matrix formula_bayesian_est[`row'+8, `col'] = `bstrap_N'
								matrix formula_bayesian_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix formula_bayesian_est[`row'+10, `col'] = `late_se_fifth'
								matrix formula_bayesian_est[`row'+11, `col'] = `fs_se_fifth'
								matrix formula_bayesian_est[`row'+12, `col'] = `bstrap_N_fifth'
								
							
								restore
								}
								
								else {
								forval b = 1/500 {
									bsample
									*sample 50
									ciest_fuzzy `controls', outcome(`out'_`t') method(`method')
									mat bootstrap_est = (bootstrap_est \ (`r(late)', `r(fs)'))	
									use "`cleandata'weighted_est_temp.dta", clear
								}
								svmat bootstrap_est
			
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs' / 100)

								drop bootstrap_est2_tmp
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)'
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)'
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' 
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' 
								
								
								matrix formula_est[`row', `col'] = `late'
								matrix formula_est[`row'+1, `col'] = `late_se'
								matrix formula_est[`row'+2, `col'] = `late_se_tenth'
								matrix formula_est[`row'+3, `col'] = `fs'
								matrix formula_est[`row'+4, `col'] = `fs_se'
								matrix formula_est[`row'+5, `col'] = `fs_se_tenth'
								matrix formula_est[`row'+6, `col'] = `rf_N'
								matrix formula_est[`row'+7, `col'] = `fs_N'
								matrix formula_est[`row'+8, `col'] = `bstrap_N'
								matrix formula_est[`row'+9, `col'] = `bstrap_N_tenth'
								matrix formula_est[`row'+10, `col'] = `late_se_fifth'
								matrix formula_est[`row'+11, `col'] = `fs_se_fifth'
								matrix formula_est[`row'+12, `col'] = `bstrap_N_fifth'
								
			
								restore
								}
								} /* closes bs method */
								} /* closes risk */
							} /* closes formula */	
							}
							
							local row = `row' + 14
						} /* closes out */
						local ++row
					
					} /* closes t */
				} /* closes group */
			} /* closes method */ 
		} /* closes file */
			preserve
			
			svmat simulation_est
			keep var simulation_est*
			ren simulation_est1 r01
			
			outsheet using "`tables'simulation_weighting_estimates.csv", comma replace

			restore
			
			preserve
		
			svmat frequency_est
			keep var frequency_est*
			ren frequency_est1 r0
			
			outsheet using "`tables'frequency_weighting_estimates.csv", comma replace
	
			restore
	
			preserve
		
			svmat formula_est
			keep var formula_est*
			ren formula_est1 r0
			
			outsheet using "`tables'formula_weighting_estimates.csv", comma replace
		
			restore

} 
*/




	*****************************************
	******* bootstrapping twosls        *****
	*****************************************
	
	if `pscore_twosls_bstrap' ==1 {
			
		** Calculate the weighting estimates		
		foreach file in simulation /*frequency   formula*/ {
		
			u "`cleandata'analysis_stacked_study_`file'.dta", clear
			
			keep if baselinespyear==2012 | baselinespyear == 2013

			keep if in_sample == 1
		
			gen var =""
			
			*By grade vars
			gen group = .
			
			local col = 1
			local row = 1
		
			foreach group in 410 /*48 910*/ {
						if `group'==410{
							replace group=410
						}
						else if `group'==48{
							replace group=48 if inlist(appgrade,4,5,6,7,8)
						}
						else{
							replace group=910 if inlist(appgrade,9,10)
						}

						foreach t in math writing reading {
							foreach out in ss /*prof_up adv*/ {

							replace var="`out'_`t'_`group'" if _n==`row'
							replace var="`out'_`t'_`group'_se" if _n==`row'+1
							replace var="`out'_`t'_`group'_se_tenth" if _n==`row'+2							
							replace var="`out'_`t'_`group'_fs" if _n==`row'+3
							replace var="`out'_`t'_`group'_fs_se" if _n==`row'+4
							replace var="`out'_`t'_`group'_fs_se_tenth" if _n==`row'+5							
							replace var="`out'_`t'_`group'_N" if _n==`row'+6
							replace var="`out'_`t'_`group'_bstrap_N" if _n==`row'+7
							replace var="`out'_`t'_`group'_bstrap_N_tenth" if _n==`row'+8
							replace var="`out'_`t'_`group'_se_fifth" if _n==`row'+9
							replace var="`out'_`t'_`group'_fs_se_fifth" if _n==`row'+10
							replace var="`out'_`t'_`group'_bstrap_N_fifth" if _n==`row'+11

							qui summ `out'_`t' if group==`group' & in_zero1_pscore==1
							
							if r(max)==0 & r(min)==0{
								di in red "No observations for `group'-level weighting estimates"
							}
						
							else{
							
							if "`file'"=="simulation" {
								
								local col = 1
								
								foreach risk in r01 r001 {

								foreach bs_method in bayesian /*nonparametric*/ {
								
								preserve
							
								keep if group == `group' & in_good_cell_`risk' == 1
								egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
								keep if temp == 0
								drop temp
								
								
								local controls "`risk'_* `cov' `cov2' `cov_grade' baselinespyear"		
								* for no_covs bootstrap. Keep score controls, grade and baseline year like in 2sls no covs (file 5.). 
								local nocov_controls "`risk'_*  `cov_grade' baselinespyear"		
								
								ivreg2 `out'_`t' (D = Z) `controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls') cluster(maskedid)
								local late_2sls = _b[D]
								local rf_2sls_N = e(N)
								
								estimates restore firstD
								
								local fs_2sls = _b[Z]
								
								
								ivreg2 `out'_`t' (D = Z) `nocovs_controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`nocovs_controls') cluster(maskedid)
								local nocov_late_2sls = _b[D]
								local nocov_rf_2sls_N = e(N)	
								
								estimates restore firstD
								
								local nocov_fs_2sls = _b[Z]								
																
								
								save "`cleandata'weighted_est_temp.dta", replace
								
								
								set seed 123
								mat bootstrap_est = J(1, 2, .)
								mat nocov_bootstrap_est = J(1, 2, .)
								
								if "`bs_method'" == "bayesian" {
								
									forval b = 1/500 {   
										gen bs_wgt = rgamma(1,1)
										ivreg2 `out'_`t' (D = Z) `controls' [aw=bs_wgt] if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls') cluster(maskedid)
										local late_bs = _b[D]
										estimates restore firstD
										local fs_bs = _b[Z]
										mat bootstrap_est = (bootstrap_est \ ( `late_bs', `fs_bs'))
										
										ivreg2 `out'_`t' (D = Z) `nocov_controls' [aw=bs_wgt] if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`nocov_controls') cluster(maskedid)
										local nocov_late_bs = _b[D]
										estimates restore firstD
										local nocov_fs_bs = _b[Z]
										mat nocov_bootstrap_est = (nocov_bootstrap_est \ ( `late_bs', `fs_bs'))											
										
										use "`cleandata'weighted_est_temp.dta", clear
									}
									
									svmat bootstrap_est
									
									gen bootstrap_est2_tenth = bootstrap_est2
									gen bootstrap_est1_tenth = bootstrap_est1
									
									gen bootstrap_est2_fifth = bootstrap_est2
									gen bootstrap_est1_fifth = bootstrap_est1
									
									gen bootstrap_est2_tmp = bootstrap_est2
									** When bootstrapped, there could be FS that are too small. Drop them.
									replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
									replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
									
									replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
									replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
									
									replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)
									replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)

									drop bootstrap_est2_tmp
								
								
									su bootstrap_est1
									local bstrap_N = `r(N)'
									local late_se = `r(sd)'
									
									su bootstrap_est2
									local fs_bstrap_N = `r(N)'
									local fs_se = `r(sd)' 
									
									su bootstrap_est1_tenth
									local bstrap_N_tenth = `r(N)'
									local late_se_tenth = `r(sd)'
									
									
									su bootstrap_est2_tenth
									local fs_bstrap_N_tenth = `r(N)'
									local fs_se_tenth = `r(sd)'
									
									su bootstrap_est1_fifth 
									local bstrap_N_fifth = `r(N)'								
									local late_se_fifth = `r(sd)'
									
									su bootstrap_est2_fifth
									local fs_bstrap_N_fifth = `r(N)'
									local fs_se_fifth = `r(sd)' 
									
									
									matrix simulation_2sls_bayesian_bs[`row', `col'] = `late_2sls'
									matrix simulation_2sls_bayesian_bs[`row'+1, `col'] = `late_se'
									matrix simulation_2sls_bayesian_bs[`row'+2, `col'] = `late_se_tenth'
									matrix simulation_2sls_bayesian_bs[`row'+3, `col'] = `fs_2sls'
									matrix simulation_2sls_bayesian_bs[`row'+4, `col'] = `fs_se'
									matrix simulation_2sls_bayesian_bs[`row'+5, `col'] = `fs_se_tenth'
									matrix simulation_2sls_bayesian_bs[`row'+6, `col'] = `rf_2sls_N'
									matrix simulation_2sls_bayesian_bs[`row'+7, `col'] = `bstrap_N'
									matrix simulation_2sls_bayesian_bs[`row'+8, `col'] = `bstrap_N_tenth'
									matrix simulation_2sls_bayesian_bs[`row'+9, `col'] = `late_se_fifth'
									matrix simulation_2sls_bayesian_bs[`row'+10, `col'] = `fs_se_fifth'
									matrix simulation_2sls_bayesian_bs[`row'+11, `col'] = `bstrap_N_fifth'
									
									*** No cov results ***
									
		
									svmat nocov_bootstrap_est
				
									gen nocov_bootstrap_est2_tenth = nocov_bootstrap_est2
									gen nocov_bootstrap_est1_tenth = nocov_bootstrap_est1
									
									gen nocov_bootstrap_est2_fifth = nocov_bootstrap_est2
									gen nocov_bootstrap_est1_fifth = nocov_bootstrap_est1
									
									gen nocov_bootstrap_est2_tmp = bootstrap_est2
									** When bootstrapped, there could be FS that are too small. Drop them.
									replace nocov_bootstrap_est2_tenth = . if abs(nocov_bootstrap_est2_tmp) < (`nocov_fs_2sls' / 10)
									replace nocov_bootstrap_est1_tenth = . if abs(nocov_bootstrap_est2_tmp) < (`nocov_fs_2sls' / 10)
									
									replace nocov_bootstrap_est2_fifth = . if abs(nocov_bootstrap_est2_tmp) < (`nocov_fs_2sls' / 5)
									replace nocov_bootstrap_est1_fifth = . if abs(nocov_bootstrap_est2_tmp) < (`nocov_fs_2sls' / 5)
									
									replace nocov_bootstrap_est1 = . if abs(nocov_bootstrap_est2_tmp) < (`nocov_fs_2sls' / 100)
									replace nocov_bootstrap_est2 = . if abs(nocov_bootstrap_est2_tmp) < (`nocov_fs_2sls' / 100)

								
									drop nocov_bootstrap_est2_tmp
								
															
									su nocov_bootstrap_est1
									local nocov_bstrap_N = `r(N)'
									local nocov_late_se = `r(sd)'
									
									su nocov_bootstrap_est2
									local nocov_fs_bstrap_N = `r(N)'
									local nocov_fs_se = `r(sd)' 
									
									su nocov_bootstrap_est1_tenth
									local nocov_bstrap_N_tenth = `r(N)'
									local nocov_late_se_tenth = `r(sd)' 				
									
									su nocov_bootstrap_est2_tenth
									local nocov_fs_bstrap_N_tenth = `r(N)'
									local nocov_fs_se_tenth = `r(sd)' 
									
									su nocov_bootstrap_est1_fifth 
									local nocov_bstrap_N_fifth = `r(N)'								
									local nocov_late_se_fifth = `r(sd)' 
									
									su nocov_bootstrap_est2_fifth
									local nocov_fs_bstrap_N_fifth = `r(N)'
									local nocov_fs_se_fifth = `r(sd)' 
									
									
									matrix simulation_nocov_2sls_bayes_bs[`row', `col'] = `nocov_late_2sls'
									matrix simulation_nocov_2sls_bayes_bs[`row'+1, `col'] = `nocov_late_se'
									matrix simulation_nocov_2sls_bayes_bs[`row'+2, `col'] = `nocov_late_se_tenth'
									matrix simulation_nocov_2sls_bayes_bs[`row'+3, `col'] = `nocov_fs_2sls'
									matrix simulation_nocov_2sls_bayes_bs[`row'+4, `col'] = `nocov_fs_se'
									matrix simulation_nocov_2sls_bayes_bs[`row'+5, `col'] = `nocov_fs_se_tenth'
									matrix simulation_nocov_2sls_bayes_bs[`row'+6, `col'] = `nocov_rf_2sls_N'
									matrix simulation_nocov_2sls_bayes_bs[`row'+7, `col'] = `nocov_bstrap_N'
									matrix simulation_nocov_2sls_bayes_bs[`row'+8, `col'] = `nocov_bstrap_N_tenth'
									matrix simulation_nocov_2sls_bayes_bs[`row'+9, `col'] = `nocov_late_se_fifth'
									matrix simulation_nocov_2sls_bayes_bs[`row'+10, `col'] = `nocov_fs_se_fifth'
									matrix simulation_nocov_2sls_bayes_bs[`row'+11, `col'] = `nocov_bstrap_N_fifth'		
																		
									
									restore
								}
								
								else {

										forval b = 1/500 {
											bsample
											ivreg2 `out'_`t' (D = Z) `controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls') cluster(maskedid)
											local late_bs = _b[D]
											estimates restore firstD
											local fs_bs = _b[Z]
											mat bootstrap_est = (bootstrap_est \ ( `late_bs', `fs_bs'))
											
											
											use "`cleandata'weighted_est_temp.dta", clear
										}
										
									svmat bootstrap_est
									
									gen bootstrap_est2_tenth = bootstrap_est2
									gen bootstrap_est1_tenth = bootstrap_est1
									
									gen bootstrap_est2_fifth = bootstrap_est2
									gen bootstrap_est1_fifth = bootstrap_est1
									
									gen bootstrap_est2_tmp = bootstrap_est2
									** When bootstrapped, there could be FS that are too small. Drop them.
									replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
									replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
									
									replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
									replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
									
									replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)
									replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)

									drop bootstrap_est2_tmp
								
								
									su bootstrap_est1
									local bstrap_N = `r(N)'
									local late_se = `r(sd)' 
									
									su bootstrap_est2
									local fs_bstrap_N = `r(N)'
									local fs_se = `r(sd)' 
									
									su bootstrap_est1_tenth
									local bstrap_N_tenth = `r(N)'
									local late_se_tenth = `r(sd)' 
									
									
									su bootstrap_est2_tenth
									local fs_bstrap_N_tenth = `r(N)'
									local fs_se_tenth = `r(sd)'
									
									su bootstrap_est1_fifth 
									local bstrap_N_fifth = `r(N)'								
									local late_se_fifth = `r(sd)'
									
									su bootstrap_est2_fifth
									local fs_bstrap_N_fifth = `r(N)'
									local fs_se_fifth = `r(sd)'
									
									
									matrix simulation_2sls_bs[`row', `col'] = `late_2sls'
									matrix simulation_2sls_bs[`row'+1, `col'] = `late_se'
									matrix simulation_2sls_bs[`row'+2, `col'] = `late_se_tenth'
									matrix simulation_2sls_bs[`row'+3, `col'] = `fs_2sls'
									matrix simulation_2sls_bs[`row'+4, `col'] = `fs_se'
									matrix simulation_2sls_bs[`row'+5, `col'] = `fs_se_tenth'
									matrix simulation_2sls_bs[`row'+6, `col'] = `rf_2sls_N'
									matrix simulation_2sls_bs[`row'+7, `col'] = `bstrap_N'
									matrix simulation_2sls_bs[`row'+8, `col'] = `bstrap_N_tenth'
									matrix simulation_2sls_bs[`row'+9, `col'] = `late_se_fifth'
									matrix simulation_2sls_bs[`row'+10, `col'] = `fs_se_fifth'
									matrix simulation_2sls_bs[`row'+11, `col'] = `bstrap_N_fifth'
									
									
														
									
								restore
								
								}
								} /* closes bs method */
								local col = `col' + 1
								} /* closes risk */
							} /* closes simulation */	
								
								
							else if "`file'"=="frequency" {
								
								local col = 1
								
								foreach risk in r0{
								
								foreach bs_method in bayesian /*nonparametric*/ {
								
								preserve
							
								keep if group == `group' & in_good_cell_`risk' == 1
								egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
								keep if temp == 0
								drop temp
																
								local controls "`risk'_* `cov' `cov2' `cov_grade' baselinespyear"		
								* for no_covs bootstrap. Keep score controls, grade and baseline year like in 2sls no covs (file 5.). 
								local nocov_controls "`risk'_*  `cov_grade' baselinespyear"		
								
								
								ivreg2 `out'_`t' (D = Z) `controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls') cluster(maskedid)
								local late_2sls = _b[D]
								local rf_2sls_N = e(N)
								
								estimates restore firstD
								
								local fs_2sls = _b[Z]

								save "`cleandata'weighted_est_temp.dta", replace
								
								set seed 123
								mat bootstrap_est = J(1, 2, .)
								
								if "`bs_method'" == "bayesian" {

								forval b = 1/500 {    
									gen bs_wgt = rgamma(1,1)
									ivreg2 `out'_`t' (D = Z) `controls' [aw=bs_wgt] if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls') cluster(maskedid)
									local late_bs = _b[D]
									estimates restore firstD
									local fs_bs = _b[Z]
									mat bootstrap_est = (bootstrap_est \ ( `late_bs', `fs_bs'))
									
							
									
									use "`cleandata'weighted_est_temp.dta", clear
								}
								
								svmat bootstrap_est
			
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)

							
								drop bootstrap_est2_tmp
							
														
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)'
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' 
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 				
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' 
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' 
								
								
								matrix frequency_2sls_bayesian_bs[`row', `col'] = `late_2sls'
								matrix frequency_2sls_bayesian_bs[`row'+1, `col'] = `late_se'
								matrix frequency_2sls_bayesian_bs[`row'+2, `col'] = `late_se_tenth'
								matrix frequency_2sls_bayesian_bs[`row'+3, `col'] = `fs_2sls'
								matrix frequency_2sls_bayesian_bs[`row'+4, `col'] = `fs_se'
								matrix frequency_2sls_bayesian_bs[`row'+5, `col'] = `fs_se_tenth'
								matrix frequency_2sls_bayesian_bs[`row'+6, `col'] = `rf_2sls_N'
								matrix frequency_2sls_bayesian_bs[`row'+7, `col'] = `bstrap_N'
								matrix frequency_2sls_bayesian_bs[`row'+8, `col'] = `bstrap_N_tenth'
								matrix frequency_2sls_bayesian_bs[`row'+9, `col'] = `late_se_fifth'
								matrix frequency_2sls_bayesian_bs[`row'+10, `col'] = `fs_se_fifth'
								matrix frequency_2sls_bayesian_bs[`row'+11, `col'] = `bstrap_N_fifth'
								
								
							
								
								restore
								}
								
								else {
								forval b = 1/500 {
									bsample
									*sample 50
									ivreg2 `out'_`t' (D = Z) `controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls') cluster(maskedid)
									local late_bs = _b[D]
									estimates restore firstD
									local fs_bs = _b[Z]
									mat bootstrap_est = (bootstrap_est \ ( `late_bs', `fs_bs'))
									
									use "`cleandata'weighted_est_temp.dta", clear
								}
								svmat bootstrap_est
			
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)

							
								drop bootstrap_est2_tmp
							
														
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' 
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' 
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' 
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' 
								
								
								matrix frequency_2sls_bs[`row', `col'] = `late_2sls'
								matrix frequency_2sls_bs[`row'+1, `col'] = `late_se'
								matrix frequency_2sls_bs[`row'+2, `col'] = `late_se_tenth'
								matrix frequency_2sls_bs[`row'+3, `col'] = `fs_2sls'
								matrix frequency_2sls_bs[`row'+4, `col'] = `fs_se'
								matrix frequency_2sls_bs[`row'+5, `col'] = `fs_se_tenth'
								matrix frequency_2sls_bs[`row'+6, `col'] = `rf_2sls_N'
								matrix frequency_2sls_bs[`row'+7, `col'] = `bstrap_N'
								matrix frequency_2sls_bs[`row'+8, `col'] = `bstrap_N_tenth'
								matrix frequency_2sls_bs[`row'+9, `col'] = `late_se_fifth'
								matrix frequency_2sls_bs[`row'+10, `col'] = `fs_se_fifth'
								matrix frequency_2sls_bs[`row'+11, `col'] = `bstrap_N_fifth'
			
								restore
								}
								
								} /* closes bs method */
								} /* closes risk */
							} /* closes frequency */	
													
							else {
								
								local col = 1
								
								foreach risk in r0{

								foreach bs_method in bayesian /*nonparametric*/ {

								preserve
							
								keep if group == `group' & in_good_cell_`risk' == 1
								egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
								keep if temp == 0
								drop temp
							
								
								local controls "`risk'_* `cov' `cov2' `cov_grade' baselinespyear"		
								* for no_covs bootstrap. Keep score controls, grade and baseline year like in 2sls no covs (file 5.). 
								local nocov_controls "`risk'_*  `cov_grade' baselinespyear"										
								
								
								ivreg2 `out'_`t' (D = Z) `controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls') cluster(maskedid)
								local late_2sls = _b[D]
								local rf_2sls_N = e(N)
								
								estimates restore firstD
								
								local fs_2sls = _b[Z]
								
								
								save "`cleandata'weighted_est_temp.dta", replace
								
								set seed 123
								mat bootstrap_est = J(1, 2, .)

								if "`bs_method'" == "bayesian" {
								
								forval b = 1/500 {
									gen bs_wgt = rgamma(1,1)
									ivreg2 `out'_`t' (D = Z) `controls' [aw=bs_wgt] if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls') cluster(maskedid)
									local late_bs = _b[D]
									estimates restore firstD
									local fs_bs = _b[Z]
									mat bootstrap_est = (bootstrap_est \ ( `late_bs', `fs_bs'))
									
									use "`cleandata'weighted_est_temp.dta", clear
								}
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)

						
								drop bootstrap_est2_tmp
							
							
								
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' 
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' 
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' 
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)'
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)'
								
								
								matrix formula_2sls_bayesian_bs[`row', `col'] = `late_2sls'
								matrix formula_2sls_bayesian_bs[`row'+1, `col'] = `late_se'
								matrix formula_2sls_bayesian_bs[`row'+2, `col'] = `late_se_tenth'
								matrix formula_2sls_bayesian_bs[`row'+3, `col'] = `fs_2sls'
								matrix formula_2sls_bayesian_bs[`row'+4, `col'] = `fs_se'
								matrix formula_2sls_bayesian_bs[`row'+5, `col'] = `fs_se_tenth'
								matrix formula_2sls_bayesian_bs[`row'+6, `col'] = `rf_2sls_N'
								matrix formula_2sls_bayesian_bs[`row'+7, `col'] = `bstrap_N'
								matrix formula_2sls_bayesian_bs[`row'+8, `col'] = `bstrap_N_tenth'
								matrix formula_2sls_bayesian_bs[`row'+9, `col'] = `late_se_fifth'
								matrix formula_2sls_bayesian_bs[`row'+10, `col'] = `fs_se_fifth'
								matrix formula_2sls_bayesian_bs[`row'+11, `col'] = `bstrap_N_fifth'
								
								
							
								restore
								}
								
								else {
								forval b = 1/500 {
									bsample
									*sample 50
									ivreg2 `out'_`t' (D = Z) `controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls') cluster(maskedid)
									local late_bs = _b[D]
									estimates restore firstD
									local fs_bs = _b[Z]
									mat bootstrap_est = (bootstrap_est \ ( `late_bs', `fs_bs'))
									use "`cleandata'weighted_est_temp.dta", clear
								}
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)

						
								drop bootstrap_est2_tmp
							
							
								
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' 
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' 
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)'
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' 
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)'
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' 
								
								
								matrix formula_2sls_bs[`row', `col'] = `late_2sls'
								matrix formula_2sls_bs[`row'+1, `col'] = `late_se'
								matrix formula_2sls_bs[`row'+2, `col'] = `late_se_tenth'
								matrix formula_2sls_bs[`row'+3, `col'] = `fs_2sls'
								matrix formula_2sls_bs[`row'+4, `col'] = `fs_se'
								matrix formula_2sls_bs[`row'+5, `col'] = `fs_se_tenth'
								matrix formula_2sls_bs[`row'+6, `col'] = `rf_2sls_N'
								matrix formula_2sls_bs[`row'+7, `col'] = `bstrap_N'
								matrix formula_2sls_bs[`row'+8, `col'] = `bstrap_N_tenth'
								matrix formula_2sls_bs[`row'+9, `col'] = `late_se_fifth'
								matrix formula_2sls_bs[`row'+10, `col'] = `fs_se_fifth'
								matrix formula_2sls_bs[`row'+11, `col'] = `bstrap_N_fifth'
							
								restore
								}
								
								} /* closes bs method */
								} /* closes risk */
							} /* closes formula */	
							}
							
							local row = `row' + 12
						} /* closes out */
						local ++row
					
					} /* closes t */
				} /* closes group */ 
		} /* closes file */
			
			preserve
			
			svmat simulation_2sls_bs
			keep var simulation_2sls_bs*
			ren simulation_2sls_bs1 r01
			ren simulation_2sls_bs2 r001
			
			outsheet using "`tables'simulation_twosls_weighting_estimates_N_stacked.csv", comma replace

			restore
			
			preserve
		
			svmat frequency_2sls_bs
			keep var frequency_2sls_bs*
			ren frequency_2sls_bs1 r0
			
			outsheet using "`tables'frequency_twosls_weighting_estimates_N_stacked.csv", comma replace
	
			restore
	
			preserve
		
			svmat formula_2sls_bs
			keep var formula_2sls_bs*
			ren formula_2sls_bs1 r0
			
			outsheet using "`tables'formula_twosls_weighting_estimates_N_stacked.csv", comma replace
		
			restore

			preserve
			
			svmat simulation_2sls_bayesian_bs
			keep var simulation_2sls_bayesian_bs*
			ren simulation_2sls_bayesian_bs1 r01
			ren simulation_2sls_bayesian_bs2 r001
			
			outsheet using "`tables'simulation_twosls_bayesian_bs_stacked.csv", comma replace

			restore
			
			* No covs bootstrap
			preserve
			
			svmat simulation_nocov_2sls_bayes_bs
			keep var simulation_nocov_2sls_bayes_bs*
			ren simulation_nocov_2sls_bayes_bs1 r01
			ren simulation_nocov_2sls_bayes_bs2 r001
			
			outsheet using "`tables'simulation_twosls_nocovs_bayesian_bs_stacked.csv", comma replace

			restore			
			
			
			preserve
		
			svmat frequency_2sls_bayesian_bs
			keep var frequency_2sls_bayesian_bs*
			ren frequency_2sls_bayesian_bs1 r0
			
			outsheet using "`tables'frequency_twosls_bayesian_bs_stacked.csv", comma replace
	
			restore
	
			preserve
		
			svmat formula_2sls_bayesian_bs
			keep var formula_2sls_bayesian_bs*
			ren formula_2sls_bayesian_bs1 r0
			
			outsheet using "`tables'formula_twosls_bayesian_bs_stacked.csv", comma replace
		
			restore
			
			} 




	*****************************************
	******* bootstrapping twosls (N/2)  *****
	*****************************************
	
	if `pscore_twosls_bstrap_half' ==1 {
			
		** Calculate the weighting estimates		
		foreach file in /*simulation*/ frequency formula {
		
			u "`cleandata'analysis_`file'.dta", clear
				
			keep if in_sample == 1
		
			gen var =""
			
			*By grade vars
			gen group = .
			
			local col = 1
			local row = 1
		
			foreach group in /*410*/ 48 910 {
						if `group'==410{
							replace group=410
						}
						else if `group'==48{
							replace group=48 if inlist(appgrade,4,5,6,7,8)
						}
						else{
							replace group=910 if inlist(appgrade,9,10)
						}

						foreach t in math writing reading {
							foreach out in ss /*prof_up adv*/ {

							replace var="`out'_`t'_`group'" if _n==`row'
							replace var="`out'_`t'_`group'_se" if _n==`row'+1
							replace var="`out'_`t'_`group'_se_tenth" if _n==`row'+2							
							replace var="`out'_`t'_`group'_fs" if _n==`row'+3
							replace var="`out'_`t'_`group'_fs_se" if _n==`row'+4
							replace var="`out'_`t'_`group'_fs_se_tenth" if _n==`row'+5							
							replace var="`out'_`t'_`group'_N" if _n==`row'+6
							replace var="`out'_`t'_`group'_bstrap_N" if _n==`row'+7
							replace var="`out'_`t'_`group'_bstrap_N_tenth" if _n==`row'+8
							replace var="`out'_`t'_`group'_se_fifth" if _n==`row'+9
							replace var="`out'_`t'_`group'_fs_se_fifth" if _n==`row'+10
							replace var="`out'_`t'_`group'_bstrap_N_fifth" if _n==`row'+11

							qui summ `out'_`t' if group==`group' & in_zero1_pscore==1
							
							if r(max)==0 & r(min)==0{
								di in red "No observations for `group'-level weighting estimates"
							}
						
							else{
							
							if "`file'"=="simulation" {
								
								local col = 1
								
								foreach risk in r01 {

															
								preserve
							
								keep if group == `group' & in_good_cell_`risk' == 1
								egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
								keep if temp == 0
								drop temp
								
								
								local controls "`risk'_* `cov' `cov2' `cov_grade'"		
								
								ivreg2 `out'_`t' (D = Z) `controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`cov' `cov2' `cov_grade') r
								local late_2sls = _b[D]
								local rf_2sls_N = e(N)
								
								estimates restore firstD
								
								local fs_2sls = _b[Z]
								
								save "`cleandata'weighted_est_temp.dta", replace
								
								
								set seed 123
								mat bootstrap_est = J(1, 2, .)
								
								forval b = 1/500 {
									*bsample
									sample 50
									ivreg2 `out'_`t' (D = Z) `controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`cov' `cov2' `cov_grade') r
									local late_bs = _b[D]
									estimates restore firstD
									local fs_bs = _b[Z]
									mat bootstrap_est = (bootstrap_est \ ( `late_bs', `fs_bs'))
									
									use "`cleandata'weighted_est_temp.dta", clear
								}
								
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)

								drop bootstrap_est2_tmp
							
							
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' / sqrt(2)
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' / sqrt(2)
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' / sqrt(2)
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' / sqrt(2)
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' / sqrt(2)
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' / sqrt(2)
								
								
								matrix simulation_2sls_bs_half[`row', `col'] = `late_2sls'
								matrix simulation_2sls_bs_half[`row'+1, `col'] = `late_se'
								matrix simulation_2sls_bs_half[`row'+2, `col'] = `late_se_tenth'
								matrix simulation_2sls_bs_half[`row'+3, `col'] = `fs_2sls'
								matrix simulation_2sls_bs_half[`row'+4, `col'] = `fs_se'
								matrix simulation_2sls_bs_half[`row'+5, `col'] = `fs_se_tenth'
								matrix simulation_2sls_bs_half[`row'+6, `col'] = `rf_2sls_N'
								matrix simulation_2sls_bs_half[`row'+7, `col'] = `bstrap_N'
								matrix simulation_2sls_bs_half[`row'+8, `col'] = `bstrap_N_tenth'
								matrix simulation_2sls_bs_half[`row'+9, `col'] = `late_se_fifth'
								matrix simulation_2sls_bs_half[`row'+10, `col'] = `fs_se_fifth'
								matrix simulation_2sls_bs_half[`row'+11, `col'] = `bstrap_N_fifth'
								
				
								restore
								
								} /* closes risk */
							} /* closes simulation */	
								
								
							else if "`file'"=="frequency" {
								
								local col = 1
								
								foreach risk in r0{
								
								preserve
							
								keep if group == `group' & in_good_cell_`risk' == 1
								egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
								keep if temp == 0
								drop temp
								
								
								local controls "`risk'_* `cov' `cov2' `cov_grade'"		
								
								ivreg2 `out'_`t' (D = Z) `controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`cov' `cov2' `cov_grade') r
								local late_2sls = _b[D]
								local rf_2sls_N = e(N)
								
								estimates restore firstD
								
								local fs_2sls = _b[Z]
								
								save "`cleandata'weighted_est_temp.dta", replace
								
								set seed 123
								mat bootstrap_est = J(1, 2, .)
								
								forval b = 1/500 {
									*bsample
									sample 50
									ivreg2 `out'_`t' (D = Z) `controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`cov' `cov2' `cov_grade') r
									local late_bs = _b[D]
									estimates restore firstD
									local fs_bs = _b[Z]
									mat bootstrap_est = (bootstrap_est \ ( `late_bs', `fs_bs'))
									
									use "`cleandata'weighted_est_temp.dta", clear
								}
								svmat bootstrap_est
			
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)

							
								drop bootstrap_est2_tmp
							
														
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' / sqrt(2)
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' / sqrt(2)
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' / sqrt(2)
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' / sqrt(2)
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' / sqrt(2)
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' / sqrt(2)
								
								
								matrix frequency_2sls_bs_half[`row', `col'] = `late_2sls'
								matrix frequency_2sls_bs_half[`row'+1, `col'] = `late_se'
								matrix frequency_2sls_bs_half[`row'+2, `col'] = `late_se_tenth'
								matrix frequency_2sls_bs_half[`row'+3, `col'] = `fs_2sls'
								matrix frequency_2sls_bs_half[`row'+4, `col'] = `fs_se'
								matrix frequency_2sls_bs_half[`row'+5, `col'] = `fs_se_tenth'
								matrix frequency_2sls_bs_half[`row'+6, `col'] = `rf_2sls_N'
								matrix frequency_2sls_bs_half[`row'+7, `col'] = `bstrap_N'
								matrix frequency_2sls_bs_half[`row'+8, `col'] = `bstrap_N_tenth'
								matrix frequency_2sls_bs_half[`row'+9, `col'] = `late_se_fifth'
								matrix frequency_2sls_bs_half[`row'+10, `col'] = `fs_se_fifth'
								matrix frequency_2sls_bs_half[`row'+11, `col'] = `bstrap_N_fifth'
								
								
			
								restore
								} /* closes risk */
							} /* closes frequency */	
													
							else {
								
								local col = 1
								
								foreach risk in r0{
								
								preserve
							
								keep if group == `group' & in_good_cell_`risk' == 1
								egen temp = rowmiss(`out'_`t' `cov' `cov2' `cov_grade')
								keep if temp == 0
								drop temp
							
								
								local controls "`risk'_* `cov' `cov2' `cov_grade'"		
								
								ivreg2 `out'_`t' (D = Z) `controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`cov' `cov2' `cov_grade') r
								local late_2sls = _b[D]
								local rf_2sls_N = e(N)
								
								estimates restore firstD
								
								local fs_2sls = _b[Z]
								
								
								save "`cleandata'weighted_est_temp.dta", replace
								
								set seed 123
								mat bootstrap_est = J(1, 2, .)
								
								forval b = 1/500 {
									*bsample
									sample 50
									ivreg2 `out'_`t' (D = Z) `controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`cov' `cov2' `cov_grade') r
									local late_bs = _b[D]
									estimates restore firstD
									local fs_bs = _b[Z]
									mat bootstrap_est = (bootstrap_est \ ( `late_bs', `fs_bs'))
									
									use "`cleandata'weighted_est_temp.dta", clear
								}
								svmat bootstrap_est
								
								gen bootstrap_est2_tenth = bootstrap_est2
								gen bootstrap_est1_tenth = bootstrap_est1
								
								gen bootstrap_est2_fifth = bootstrap_est2
								gen bootstrap_est1_fifth = bootstrap_est1
								
								gen bootstrap_est2_tmp = bootstrap_est2
								** When bootstrapped, there could be FS that are too small. Drop them.
								replace bootstrap_est2_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								replace bootstrap_est1_tenth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 10)
								
								replace bootstrap_est2_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								replace bootstrap_est1_fifth = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 5)
								
								replace bootstrap_est1 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)
								replace bootstrap_est2 = . if abs(bootstrap_est2_tmp) < (`fs_2sls' / 100)

						
								drop bootstrap_est2_tmp
							
							
								
								su bootstrap_est1
								local bstrap_N = `r(N)'
								local late_se = `r(sd)' / sqrt(2)
								
								su bootstrap_est2
								local fs_bstrap_N = `r(N)'
								local fs_se = `r(sd)' / sqrt(2)
								
								su bootstrap_est1_tenth
								local bstrap_N_tenth = `r(N)'
								local late_se_tenth = `r(sd)' / sqrt(2)
								
								
								su bootstrap_est2_tenth
								local fs_bstrap_N_tenth = `r(N)'
								local fs_se_tenth = `r(sd)' / sqrt(2)
								
								su bootstrap_est1_fifth 
								local bstrap_N_fifth = `r(N)'								
								local late_se_fifth = `r(sd)' / sqrt(2)
								
								su bootstrap_est2_fifth
								local fs_bstrap_N_fifth = `r(N)'
								local fs_se_fifth = `r(sd)' / sqrt(2)
								
								
								matrix formula_2sls_bs_half[`row', `col'] = `late_2sls'
								matrix formula_2sls_bs_half[`row'+1, `col'] = `late_se'
								matrix formula_2sls_bs_half[`row'+2, `col'] = `late_se_tenth'
								matrix formula_2sls_bs_half[`row'+3, `col'] = `fs_2sls'
								matrix formula_2sls_bs_half[`row'+4, `col'] = `fs_se'
								matrix formula_2sls_bs_half[`row'+5, `col'] = `fs_se_tenth'
								matrix formula_2sls_bs_half[`row'+6, `col'] = `rf_2sls_N'
								matrix formula_2sls_bs_half[`row'+7, `col'] = `bstrap_N'
								matrix formula_2sls_bs_half[`row'+8, `col'] = `bstrap_N_tenth'
								matrix formula_2sls_bs_half[`row'+9, `col'] = `late_se_fifth'
								matrix formula_2sls_bs_half[`row'+10, `col'] = `fs_se_fifth'
								matrix formula_2sls_bs_half[`row'+11, `col'] = `bstrap_N_fifth'
								
								
							
								restore
								} /* closes risk */
							} /* closes formula */	
							}
							
							local row = `row' + 12
						} /* closes out */
						local ++row
					
					} /* closes t */
				} /* closes group */ 
		} /* closes file */
			
			preserve
			
			svmat simulation_2sls_bs_half
			keep var simulation_2sls_bs_half*
			ren simulation_2sls_bs_half1 r01
			
			outsheet using "`tables'simulation_twosls_weighting_estimates_half.csv", comma replace

			restore
			
			preserve
		
			svmat frequency_2sls_bs_half
			keep var frequency_2sls_bs_half*
			ren frequency_2sls_bs_half1 r0
			
			outsheet using "`tables'frequency_twosls_weighting_estimates_half.csv", comma replace
	
			restore
	
			preserve
		
			svmat formula_2sls_bs_half
			keep var formula_2sls_bs_half*
			ren formula_2sls_bs_half1 r0
			
			outsheet using "`tables'formula_twosls_weighting_estimates_half.csv", comma replace
		
			restore
} 
