/*	Ye Ji Kee

	Goal: generate table describing where compliers attend based on the type of school. 
	(stacking years 1 and 2. using datasets created in "7_DPS_destinies.do" and "7_DPS_destinies_year2.do")

Output
Use the frequency score table
Columns: 
z0	z1	z0_zero1			r0_2sls_compliers_z0	r0_2sls_compliers_z1
Rows: 
[All] study
[all] noex
[all] nonnoex
[All] study

[All] traditional
[All] innovation
[All] magnet
[All] alternative_trad
[All] contract
[All] other_charter

[All] _N
	
*/

clear all
cap log close
set more off
set trace off
set matsize 10000
set seed 42
adopath + "Z:\ado\plus\"

pause on 

matrix destinies           = J(100,20,.)
matrix twosls_destinies    = J(100,20,.)
matrix ols_destinies       = J(100,20,.)

matrix twosls_destinies_test    = J(100,20,.)


local rawdata		"Y:/DPS/Raw Data/"
local cleandata		"Y:/DPS/MDmeetsRD Denver/Data/"
local tables		"Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"

local cov 	"baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"


*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_1 g_2 g_3 g_4 g_5 g_6 g_7"


local simsdate 				= 20150603
local pscoredate			= 20160715 /*20160208*/

local pscore_stacking = 1
local destinies = 1

***********************************************
********** Complier school choice *************
***********************************************

if `pscore_stacking' == 1 {
	foreach sch_type in study /*noex nonnoex */{
		foreach file in simulation frequency formula {
		
			u "`cleandata'destinies_analysis_`sch_type'_`file'.dta", clear
			
			append using "`cleandata'destinies_analysis_`sch_type'_year_2_`file'.dta"
			
			drop in_good_cell_* tag_* obs_cell_* r01_* r001_* r0_*
			
			tab r01, gen(r01_)
			tab r001, gen(r001_)
			tab r0, gen(r0_)
						
			foreach x in r01 r001 r0{
				
				*Variation in instruments for each cell
					bys `x': egen sd_z_`x'	= 	sd(Z)
				
				*Good cell
					gen in_good_cell_`x' = (sd_z_`x'>0 & !missing(sd_z_`x') & in_zero1_pscore==1)
				
					gen double tag_`x' = `x'
					replace tag_`x' = -9999999 if in_good_cell_`x'==0
					tab tag_`x', gen(tag_`x'_)
				
				*Drop var for things with no var (the first one). These observations will not be included
				*in bin-based regression and fully saturated regressios. So, do not include their corresponding
				*variables either
				
				drop tag_`x'_1
				
				*Obs per cell
				
				bys `x': egen obs_cell_`x' = count(maskedid)
			}
	
			drop sd_z_*
			
			*dropping duplicates from year 2. 
			duplicates tag masked, gen(dup_masked)
			drop if dup_masked > 0 & baselinespyear == 2013			
			
			save "`cleandata'destinies_analysis_stacked_`sch_type'_`file'", replace
		}
	}
}


	***************************************************************
	******************** Complier characteristics *****************
	***************************************************************

if `destinies'==1{

	foreach sch_type in study /*noex nonnoex*/ {

	local cov3  "baseline`sch_type'"

	local all_covs "`cov' `cov2' `cov3' `cov_grade' i.baselinespyear"
	
		foreach file in frequency simulation formula {

				local col = 0
				foreach risk in none r0 r01 r001{
							
				u "`cleandata'/destinies_analysis_stacked_`sch_type'_`file'.dta", clear
					
					local row = 1
					local col = `col' + 1
					
					gen var =""	
					destring appgrade, gen(temp)
					drop appgrade
					rename temp appgrade
								
					foreach group in MS HS all{		
						
						if "`group'"=="MS"{
							local cond "!inlist(appgrade,9,10) & ss_math!=."
						}
						else if "`group'"=="HS"{
							local cond  "inlist(appgrade,9,10) & ss_math!=."
						}
						else if "`group'"=="all"{
							local cond "!inlist(appgrade,0) & ss_math!=."
						}
						
							
						if "`risk'"=="none"{
								foreach s in 	study noex nonnoex other_charter innovation magnet contract ///
											alternative_charter alternative_trad traditional notDPS{									
		
										*All students in (0,1) , Z=0 and Z=1
											summ `s' if Z==0  & `cond'
												matrix destinies[`row',`col'] = r(mean)
			
											summ `s' if Z==1 & `cond'
												matrix destinies[`row',`col'+1] = r(mean)

											
										*All students in (0,1) , Z=0 and Z=1 (conditioned on being in a fully saturated cell with variation in offer)
											summ `s' if Z==0  & `cond' & in_zero1_pscore==1 & in_good_cell_r0==1
												matrix destinies[`row',`col'+2] = r(mean)
								 
											summ `s' if Z==1 & `cond' & in_zero1_pscore==1 & in_good_cell_r0==1
												matrix destinies[`row',`col'+3] = r(mean)
		
								local row = `row'+2
								} /*Close s loop*/
							
								*At end of loop, count the number of kids offered and non-offered		
									*All students in (0,1) , Z=0 and Z=1

									count if Z==0  & `cond'
										matrix destinies[`row',`col'] = r(N)
										
									count if Z==1  & `cond'
										matrix destinies[`row',`col'+1] = r(N)
									
								*All students in (0,1) , Z=0 and Z=1

									count if Z==0  & `cond' & in_zero1_pscore==1 & in_good_cell_r0==1
										matrix destinies[`row',`col'+2] = r(N)

									count if Z==1 & `cond' & in_zero1_pscore==1 & in_good_cell_r0==1
										matrix destinies[`row',`col'+3] = r(N)
										
						}	/*Close risk==none condition*/
						
						else if "`risk'"!="none"{
					
						
								foreach s in 	 study noex nonnoex other_charter innovation magnet contract ///
										alternative_charter alternative_trad traditional notDPS{
										
										replace var  = "[`group'] `s'_mean" if _n==`row'
										replace var  = "[`group'] `s'_se" if _n==`row'+1
										
								**************************************************************
								*************************** Kappa ****************************
								**************************************************************
												
								*Predicted Z conditional on beingin (0,1) as well
									qui probit Z tag_`risk'_* `all_covs' if in_good_cell_`risk'==1 & `cond' & in_zero1_pscore==1
									predict pZ if e(sample)
									
									gen kappa_0 = (1-D)*((1-Z)-(1-pZ))/(pZ*(1-pZ))  if in_good_cell_`risk'==1 & `cond' & in_zero1_pscore==1
									
								
								*Untreated compliers
									gen Y = `s'*(1-D) if in_good_cell_`risk'==1 & `cond' & in_zero1_pscore==1
									cap noisily {
										summ `s' [iweight=kappa_0] if in_good_cell_`risk'==1 & `cond' & in_zero1_pscore==1
										matrix destinies[`row',`col'+3] = r(mean)
									}	/*Close cap noisily*/
												
												
								**************************************************************
								*************************** OLS (D on Z) ****************************
								**************************************************************

								*D on Z (First-stage) conditional on beingin (0,1) as well
									reg `s' Z tag_`risk'_* `all_covs' if in_good_cell_`risk'==1 & `cond' & in_zero1_pscore==1
									matrix ols_destinies[`row', `col'] = _b[Z]
									matrix ols_destinies[`row'+1, `col'] = _se[Z]

								**************************************************************
								***************************** 2SLS ***************************
								**************************************************************
									
									*Untreated compliers
									gen double oneminusD = 1-D
									
									ivreg2 `s' (oneminusD = Z) tag_`risk'_* `all_covs' if in_good_cell_`risk'==1 & `cond' & in_zero1_pscore==1, partial(tag_`risk'_* `all_covs')
									matrix twosls_destinies[`row',`col'] = _b[oneminusD]
									matrix twosls_destinies[`row'+1,`col'] = _se[oneminusD]
									
									* treated compliers
									ivreg2 `s' (D = Z) tag_`risk'_* `all_covs' if in_good_cell_`risk'==1 & `cond' & in_zero1_pscore==1, partial(tag_`risk'_* `all_covs')
									matrix twosls_destinies[`row',`col' + 3] = _b[D]
									matrix twosls_destinies[`row'+1,`col' + 3] = _se[D]									
									
									local row = `row'+2
									drop Y oneminusD pZ kappa_0 
								
								}	/*Close s loop*/
								

								*At end of loop, count the number of treated and untreated compliers
									/*	Untreated compliers =
										[Sample size]*[Probability of being a complier - First stage]*[1 - Probability of getting offer]
									*/
									
									replace var  = "[`group'] `s'_N" if _n==`row'
									
									*Sample size
									count if in_good_cell_`risk'==1 & `cond' & in_zero1_pscore==1
									local samplesize = r(N)
									
									*Probability of being a complier
									qui reg D Z tag_`risk'_* `all_covs' if in_good_cell_`risk'==1 & `cond' & in_zero1_pscore==1
									local prob_complier= _b[Z]
									
									* OLS samplesize
									qui reg `sch_type' Z tag_`risk'_* `all_covs' if in_good_cell_`risk'==1 & `cond' & in_zero1_pscore==1
									count if e(sample) == 1
									matrix ols_destinies[`row',`col'] = r(N)
								
									
									*Probability of getting an offer
									summ Z if e(sample)
									local prob_Z = r(mean)
									
										*Untreated compliers
										matrix twosls_destinies[`row',`col'] = `samplesize'*`prob_complier'*(1-`prob_Z')
										
										*Treated compliers
										matrix twosls_destinies[`row',`col'+3] = `samplesize'*`prob_complier'*(`prob_Z')

						} /*Close risk condition*/				
					
					local row = `row' + 2
					} /*Close MS HS group*/
			}	/*Close riskset loop*/
		
			keep var
			keep if `row'<= `row' + 2
			svmat destinies
			svmat twosls_destinies
			svmat ols_destinies
			
			ren destinies1 z0
			ren destinies2 z1
			
			ren destinies3 z0_zero1
			ren destinies4 z1_zero1
			
			ren destinies5 r0_kappa_compliers_z0
			ren destinies6 r01_kappa_compliers_z0
			ren destinies7 r001_kappa_compliers_z0
			
			ren twosls_destinies2 r0_2sls_compliers_z0
			ren twosls_destinies3 r01_2sls_compliers_z0
			ren twosls_destinies4 r001_2sls_compliers_z0
			
			ren twosls_destinies5 r0_2sls_compliers_z1
			ren twosls_destinies6 r01_2sls_compliers_z1
			ren twosls_destinies7 r001_2sls_compliers_z1
			
			ren ols_destinies2 r0_ols_zero1
			ren ols_destinies3 r01_ols_zero1
			ren ols_destinies4 r001_ols_zero1
			
			
			drop *destinies*
			outsheet using "`tables'inDPSbase_stacked_`sch_type'_`file'_destinies.csv", comma replace
			
			
	}	/*Close file loop*/
	} /* Close sch_type loop */
}	/*Close choice condition*/
