/* 	Ye Ji Kee
	Denver tables (year 1). These comments last edited: 2/18/2016
		For grades 4-10
		Output: Balance, 2SLS, and subgroup for pscore risksets and qualification instruments
		
		NOTE: This do-file uses datasets created in 5_pscore_balance_2SLS_tables.do and 5_pscore_balance_2SLS_year_2_tables.do

		Ignacio. We are going to drop duplicates from year 2 (duplicates are kids that show up in both year 1 and 2). Only keeping first time application. 


Use the frequency score output: 
Look up columns: 
fs_psat	fs_psat	fs_psat		fs_r0	fs_joint_r0
twosls_psat	twosls_psat	twosls_psat		jointtwosls_r0	jointtwosls_joint_r0

Rows: 
ss_math_410_n
ss_math_410_se_n

ss_math_410_e
ss_math_410_se_e

ss_math_410_i
ss_math_410_se_i
ss_writing_410_N_e
		
		*/



clear all
cap log close
set more off
set trace off
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"

pause on

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"

matrix demo			  = J(30,15,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,150,.)
matrix jointtwosls	  = J(500,150,.)

local pscore_setup_yr1 = 1
local pscore_setup_yr2 = 1
local pscore_stacking = 1
local pscore_balance = 0
local pscore_separate_twosls = 0
local pscore_joint_twosls = 0

* saturated sim takes a lng time to run. Set to 1 if you need these results. 
local sim_saturated = 0

*Dates of analysis files
local innovdate     20170310 /*20160715 20160222*/
local innovdate_yr2 20170310 /* 20160715 20160411*/
local pscoredate	20170310 /*20160715 20160208*/
local pscoredate_yr2 20170310 /*20160715 20160411*/
local simsdate 		20150603	/*20150428*/
	
local appcov_n 	"num_schools num_noex rank1_noex"
local cov_n 	"baselinenoex baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2		"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_1 g_2 g_3 g_4 g_5 g_6 g_7"

local appcov_i "num_schools num_innov rank1_innov"
local cov_i    "baselineinnov baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local appcov_e  "num_schools num_nonnoex rank1_nonnoex"
local cov_e		"baselinenonnoex baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local appcov_either  "num_schools num_noex rank1_noex num_nonnoex rank1_nonnoex num_innov rank1_innov"
local cov_either 	"baselinenoex baselinenonnoex baselineinnov baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1				/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/






	************************************************
	**************** Pscore Setup ******************
	************************************************		
	
	if `pscore_setup_yr1'==1{
		
		foreach file in frequency /* simulation formula*/{
	
			use "`cleandata'mdrd_denver_noex_`file' `pscoredate'", clear
			
			rename in_sample in_noex_sample
			
			merge 1:1 maskedid appschoolnum using "`cleandata'mdrd_denver_nonnoex_`file' `pscoredate'", keepusing(offer_nonnoex pscore_nonnoex in_sample)
			tab _merge
			drop _merge
			
			rename in_sample in_nonnoex_sample
			
			merge 1:1 maskedid appschoolnum using "`cleandata'mdrd_denver_innov_`file' `innovdate'", keepusing(baselineinnov appinnov innov offer_innov pscore_innov in_innov_sample)
			tab _merge
			drop _merge
			
					
			destring baselinegrade, replace
			destring appgrade, gen(appgrade_tmp)
			drop appgrade
			rename appgrade_tmp appgrade
			
			*High school flag			
				gen hsgrade = ( appgrade>=9)
				gen msgrade = ( appgrade<9)
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
				
			*Analysis at the student level. Generate max for charter offer
			bys maskedid: egen any_noex_offer 	= max(offer_noex)
			bys maskedid: egen any_nonnoex_offer 	= max(offer_nonnoex)
			bys maskedid: egen any_innov_offer	= max(offer_innov)
			
			bys maskedid: egen num_noex = count(cond(appnoex==1,appschoolnum,.))
			bys maskedid: egen num_nonnoex = count(cond(appnonnoex==1,appschoolnum,.))
			bys maskedid: egen num_innov = count(cond(appinnov==1,appschoolnum,.))
			
			bys maskedid: egen num_schools = count(appschoolnum)
			
			*Enrolls at charter flag (student level)
			bys maskedid: egen enroll_any_noex 	= max(noex)
			bys maskedid: egen enroll_any_nonnoex		= max(nonnoex)
			bys maskedid: egen enroll_any_innov = max(innov)
			
			*Rank 1 at school level
			gen rank1_noex= (appnoex==1 & apprank==1)
			gen rank1_nonnoex= (appnonnoex==1 & apprank==1)
			gen rank1_innov = (appinnov==1 & apprank == 1)
			******************************************************************************
			
			*First ranked school is charter (student level, for cov balance)
			foreach type in noex nonnoex innov{
				bys maskedid: egen temp`type' = max(rank1_`type')
				drop rank1_`type'
				ren temp`type' rank1_`type'
				
				*Flag - apply to any study
				bys maskedid: egen max`type' = max(app`type')
				drop app`type'
				ren max`type' app`type'
			}
			
			*Enrolls next year (i.e. has nonmissing schoolnum)
			gen next = (schoolnum!=.)
			bys maskedid: egen enrolls_next = max(next)
			
			*In_sample was previously defined at the student-school level. Redefine it so
			*that it is at the student level
			
			foreach type in noex nonnoex innov{
				bys maskedid: egen in_`type'_sample_max = max(in_`type'_sample)
				tab in_`type'_sample_max in_`type'_sample
				
				drop in_`type'_sample
				rename in_`type'_sample_max in_`type'_sample
			}
			
			*** Dataset at student level from here on ****
			duplicates drop maskedid, force
			drop next
			
			*Redefine has scores so that it excludes sciences - very few kids
			replace has_sc = (has_sc_math | has_sc_reading | has_sc_writing)
			
			*School-level obs
			drop appschoolnum appschoolname offer offer_noex offer_nonnoex offer_innov appcharter apprank noex nonnoex innov charter

			*Keep only if in sample
			keep if in_noex_sample==1 | in_nonnoex_sample==1 | in_innov_sample == 1
			
			*Define r001 as meaning pscore rounded to 0.001 and r01 as rounded to 0.01
			gen n001 = round(pscore_noex, 0.001)
			gen n01 = round(pscore_noex, 0.01)
			
			gen e001 = round(pscore_nonnoex, 0.001)
			gen e01 = round(pscore_nonnoex, 0.01)
			
			gen i001 = round(pscore_innov, 0.001)
			gen i01 = round(pscore_innov, 0.01)
			
			tab n001, gen(n001_)
			tab n01, gen(n01_)
			
			tab e01, gen(e01_)
			tab e001, gen(e001_)
			
			tab i01, gen(i01_)
			tab i001, gen(i001_)
		
			*Polynomials
			forvalues i=1/4{
				gen double pscore_n`i'=pscore_noex^(`i')
				gen double pscore_e`i'=pscore_nonnoex^(`i')
				gen double pscore_i`i'=pscore_innov^(`i')
			}
			
			keep if baselinespyear==2012
			
			*Treatment is any study, instrument is study offer, riskset is pscore
			ren any_noex_offer 	Z_n
			ren any_nonnoex_offer 	Z_e
			ren any_innov_offer	Z_i
			
			ren enroll_any_noex  	D_n
			ren enroll_any_nonnoex  	D_e
			ren enroll_any_innov	D_i
			
			**********************************************************************************
			******************** Flag kids who are ineligble and guaranteed ******************
			**********************************************************************************
			*Flag kids who are ineligble and guaranteed - do not drop them, we still want them 
			*for OLS regressions and naive 2SLS (i.e. not controlling for risksets)
			
				*If simulation, first define in_zero1_pscore based on true score, then
			
					if "`file'"=="simulation"{
					
						ren pscore_noex_simulation sim_pscore_n
						ren pscore_nonnoex_simulation sim_pscore_e
						ren pscore_innov_simulation sim_pscore_i
						
						gen in_zero1_npscore = (sim_pscore_n > 0 & sim_pscore_n < 1)
						gen in_zero1_epscore = (sim_pscore_e > 0 & sim_pscore_e < 1)
						gen in_zero1_ipscore = (sim_pscore_i > 0 & sim_pscore_i < 1)

						gen double pscore_noex_simulation = round(sim_pscore_n, 0.0001)
						gen double pscore_nonnoex_simulation = round(sim_pscore_e, 0.0001)
						gen double pscore_innov_simulation = round(sim_pscore_i, 0.0001)
						
						tab pscore_noex_`file', gen(n0_)
						tab pscore_nonnoex_`file', gen(e0_)
						tab pscore_innov_`file', gen(i0_)

						
						ren pscore_noex_`file' n0
						ren pscore_nonnoex_`file' e0
						ren pscore_innov_`file' i0
						
					}
					else{
						
						tab pscore_noex_`file', gen(n0_)
						tab pscore_nonnoex_`file', gen(e0_)
						tab pscore_innov_`file', gen(i0_)
						
						ren pscore_noex_`file' n0
						ren pscore_nonnoex_`file' e0
						ren pscore_innov_`file' i0
						
						gen in_zero1_npscore = (n0 > 0 & n0 < 1)
						gen in_zero1_epscore = (e0 > 0 & e0 < 1)
						gen in_zero1_ipscore = (i0 > 0 & i0 < 1)						
					}
					
			**********************************************************************************
			********** Flag cells with variation in instrument and characteristics************
			**********************************************************************************
					
			/* 	Flag observations that actually make it into balance (i.e. obs in cells with
				variation in instrument). These flags are used for all types 
				of F-tests, conducted below.
				
				We generate these flags here so that no re-sorting of the data (done in bys x egen:)
				is done between the balanceregs and F tests, which affects how the variable
				"var", which stores the names of the variables corresponding to each balance
				coefficient, appears in the final dataset once we svmat the balance output.
			*/
			
			ren in_noex_sample in_n_sample
			ren in_nonnoex_sample in_e_sample
			ren in_innov_sample in_i_sample
			
			tab in_n_sample, m
			tab in_e_sample, m
			tab in_i_sample, m
			
			

			
			foreach type in n e i{
				foreach x in 01 001 0{
					
					*Variation in instruments for each cell
						bys `type'`x': egen sd_z_`type'`x'	= 	sd(Z_`type')
					
					*Good cell
						gen in_good_cell_`type'`x' = (sd_z_`type'`x'>0 & !missing(sd_z_`type'`x') & in_zero1_`type'pscore==1) if in_`type'_sample==1
					
					gen double tag_`type'`x' = `type'`x' if in_`type'_sample==1
					replace tag_`type'`x' = -9999999 if in_good_cell_`type'`x'==0 & in_`type'_sample==1
					tab tag_`type'`x' if in_`type'_sample==1, gen(tag_`type'`x'_)
					
					*Drop var for things with no var (the first one). These observations will not be included
					*in bin-based regression and fully saturated regressios. So, do not include their corresponding
					*variables either
					
					drop tag_`type'`x'_1
					
					*Obs per cell
					
					bys `type'`x': egen obs_cell_`type'`x' = count(maskedid) if in_`type'_sample==1
				}
			}
			
			drop sd_z_*
			
			save "`cleandata'analysis_noex_3endog_`file'.dta", replace
		}	/*Close file loop*/	
	}	/*Close pscore condition*/
	


	
	if `pscore_setup_yr2'==1{
		
		foreach file in frequency simulation formula{
	
			use "`cleandata'mdrd_denver_noex_year_2_`file' `pscoredate_yr2'", clear
			
	
			rename in_sample in_noex_sample
			
			merge 1:1 maskedid appschoolnum using "`cleandata'mdrd_denver_nonnoex_year_2_`file' `pscoredate_yr2'", keepusing(offer_nonnoex pscore_nonnoex in_sample)
			tab _merge
			drop _merge
		
			rename in_sample in_nonnoex_sample

			merge 1:1 maskedid appschoolnum using "`cleandata'mdrd_denver_innov_year_2_`file' `innovdate_yr2'", keepusing(baselineinnov appinnov innov offer_innov pscore_innov in_innov_sample)
			tab _merge
			drop _merge
						
			
			destring baselinegrade, replace
			destring appgrade, gen(appgrade_tmp)
			drop appgrade
			rename appgrade_tmp appgrade
			
			*High school flag			
				gen hsgrade = ( appgrade>=9)
				gen msgrade = ( appgrade<9)
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
				
			*Analysis at the student level. Generate max for charter offer
			bys maskedid: egen any_noex_offer 	= max(offer_noex)
			bys maskedid: egen any_nonnoex_offer 	= max(offer_nonnoex)
			bys maskedid: egen any_innov_offer  = max(offer_innov)
			
			bys maskedid: egen num_noex = count(cond(appnoex==1,appschoolnum,.))
			bys maskedid: egen num_nonnoex = count(cond(appnonnoex==1,appschoolnum,.))
			bys maskedid: egen num_innov = count(cond(appinnov==1,appschoolnum,.))
			
			bys maskedid: egen num_schools = count(appschoolnum)
			
			*Enrolls at charter flag (student level)
			bys maskedid: egen enroll_any_noex 	= max(noex)
			bys maskedid: egen enroll_any_nonnoex		= max(nonnoex)
			bys maskedid: egen enroll_any_innov		= max(innov)
			
			*Rank 1 at school level
			gen rank1_noex= (appnoex==1 & apprank==1)
			gen rank1_nonnoex= (appnonnoex==1 & apprank==1)
			gen rank1_innov= (appinnov==1 & apprank==1)

			******************************************************************************
			
			*First ranked school is charter (student level, for cov balance)
			foreach type in noex nonnoex innov{
				bys maskedid: egen temp`type' = max(rank1_`type')
				drop rank1_`type'
				ren temp`type' rank1_`type'
				
				*Flag - apply to any study
				bys maskedid: egen max`type' = max(app`type')
				drop app`type'
				ren max`type' app`type'
			}
			
			*Enrolls next year (i.e. has nonmissing schoolnum)
			gen next = (schoolnum!=.)
			bys maskedid: egen enrolls_next = max(next)
			
			*In_sample was previously defined at the student-school level. Redefine it so
			*that it is at the student level
			
			foreach type in noex nonnoex innov{
				bys maskedid: egen in_`type'_sample_max = max(in_`type'_sample)
				tab in_`type'_sample_max in_`type'_sample
				
				drop in_`type'_sample
				rename in_`type'_sample_max in_`type'_sample
			}
			
			*** Dataset at student level from here on ****
			duplicates drop maskedid, force
			drop next
			
			*Redefine has scores so that it excludes sciences - very few kids
			replace has_sc = (has_sc_math | has_sc_reading | has_sc_writing)
			
			*School-level obs
			drop appschoolnum appschoolname offer offer_noex offer_nonnoex offer_innov appcharter apprank noex nonnoex innov charter

			*Keep only if in sample
			keep if in_noex_sample==1 | in_nonnoex_sample==1 | in_innov_sample == 1
			
			*Define r001 as meaning pscore rounded to 0.001 and r01 as rounded to 0.01
			gen n001 = round(pscore_noex, 0.001)
			gen n01 = round(pscore_noex, 0.01)
			
			gen e001 = round(pscore_nonnoex, 0.001)
			gen e01 = round(pscore_nonnoex, 0.01)
			
			gen i001 = round(pscore_innov, 0.001)
			gen i01 = round(pscore_innov, 0.01)
			
			tab n001, gen(n001_)
			tab n01, gen(n01_)
			
			tab e01, gen(e01_)
			tab e001, gen(e001_)
		
			tab i01, gen(i01_)
			tab i001, gen(i001_)
			
			*Polynomials
			forvalues i=1/4{
				gen double pscore_n`i'=pscore_noex^(`i')
				gen double pscore_e`i'=pscore_nonnoex^(`i')
				gen double pscore_i`i'=pscore_innov^(`i')
			}
			
			keep if baselinespyear==2013
			
			*Treatment is any study, instrument is study offer, riskset is pscore
			ren any_noex_offer 	Z_n
			ren any_nonnoex_offer 	Z_e
			ren any_innov_offer	Z_i
			
			ren enroll_any_noex  	D_n
			ren enroll_any_nonnoex  	D_e
			ren enroll_any_innov	D_i
			
			**********************************************************************************
			******************** Flag kids who are ineligble and guaranteed ******************
			**********************************************************************************
			*Flag kids who are ineligble and guaranteed - do not drop them, we still want them 
			*for OLS regressions and naive 2SLS (i.e. not controlling for risksets)
			
				*If simulation, first define in_zero1_pscore based on true score, then
			
					if "`file'"=="simulation"{
					
						ren pscore_noex_simulation sim_pscore_n
						ren pscore_nonnoex_simulation sim_pscore_e
						ren pscore_innov_simulation	sim_pscore_i
						
						gen in_zero1_npscore = (sim_pscore_n > 0 & sim_pscore_n < 1)
						gen in_zero1_epscore = (sim_pscore_e > 0 & sim_pscore_e < 1)
						gen in_zero1_ipscore = (sim_pscore_i > 0 & sim_pscore_i < 1)
						
						gen double pscore_noex_simulation = round(sim_pscore_n, 0.0001)
						gen double pscore_nonnoex_simulation = round(sim_pscore_e, 0.0001)
						gen double pscore_innov_simulation = round(sim_pscore_i, 0.0001)
						
						tab pscore_noex_`file', gen(n0_)
						tab pscore_nonnoex_`file', gen(e0_)
						tab pscore_innov_`file', gen(i0_)

						ren pscore_noex_`file' n0
						ren pscore_nonnoex_`file' e0
						ren pscore_innov_`file' i0
						}
						
					else{
						
						tab pscore_noex_`file', gen(n0_)
						tab pscore_nonnoex_`file', gen(e0_)
						tab pscore_innov_`file', gen(i0_)
						
						ren pscore_noex_`file' n0
						ren pscore_nonnoex_`file' e0
						ren pscore_innov_`file' i0
						
						gen in_zero1_npscore = (n0 > 0 & n0 < 1)
						gen in_zero1_epscore = (e0 > 0 & e0 < 1)
						gen in_zero1_ipscore = (i0 > 0 & i0 < 1)
						}
					
			**********************************************************************************
			********** Flag cells with variation in instrument and characteristics************
			**********************************************************************************
					
			/* 	Flag observations that actually make it into balance (i.e. obs in cells with
				variation in instrument). These flags are used for all types 
				of F-tests, conducted below.
				
				We generate these flags here so that no re-sorting of the data (done in bys x egen:)
				is done between the balanceregs and F tests, which affects how the variable
				"var", which stores the names of the variables corresponding to each balance
				coefficient, appears in the final dataset once we svmat the balance output.
			*/
			
			ren in_noex_sample in_n_sample
			ren in_nonnoex_sample in_e_sample
			ren in_innov_sample in_i_sample
			
			tab in_n_sample, m
			tab in_e_sample, m
			tab in_i_sample, m
			
			foreach type in n e i{
				foreach x in 01 001 0{
					
					*Variation in instruments for each cell
						bys `type'`x': egen sd_z_`type'`x'	= 	sd(Z_`type')
					
					*Good cell
						gen in_good_cell_`type'`x' = (sd_z_`type'`x'>0 & !missing(sd_z_`type'`x') & in_zero1_`type'pscore==1) if in_`type'_sample==1
					
					gen double tag_`type'`x' = `type'`x' if in_`type'_sample==1
					replace tag_`type'`x' = -9999999 if in_good_cell_`type'`x'==0 & in_`type'_sample==1
					tab tag_`type'`x' if in_`type'_sample==1, gen(tag_`type'`x'_)
					
					*Drop var for things with no var (the first one). These observations will not be included
					*in bin-based regression and fully saturated regressios. So, do not include their corresponding
					*variables either
					
					drop tag_`type'`x'_1
					
					*Obs per cell
					
					bys `type'`x': egen obs_cell_`type'`x' = count(maskedid) if in_`type'_sample==1
				}
			}
			
			drop sd_z_*
			
			save "`cleandata'analysis_noex_3endog_year_2_`file'.dta", replace
		}	/*Close file loop*/	
	}	/*Close pscore condition*/	
	
	
	if `pscore_stacking' == 1{
		
		foreach file in simulation frequency formula {
		
			u "`cleandata'analysis_noex_3endog_`file'.dta", clear
			
			append using "`cleandata'analysis_noex_3endog_year_2_`file'.dta", gen(year2)
			
			drop n001_* n01_* n0_* e001_* e01_* e0_* i001_* i01_* i0_* in_good_cell_* tag_* obs_cell_*

	
			tab n001, gen(n001_)
			tab n01, gen(n01_)
			
			tab e001, gen(e001_)
			tab e01, gen(e01_)
			
			tab i001, gen(i001_)
			tab i01, gen(i01_)
			
			tab n0, gen(n0_)
			tab e0, gen(e0_)
			tab i0, gen(i0_)
			
			foreach x in 0 01 001 {
				foreach score in i e n {
					
					egen ind_`score'`x' = group( `score'`x' ) 
				
				}
			}
			
			*dropping duplicates from year 2.
			
			duplicates tag masked, gen(dup_masked)
			drop if dup_masked > 0 & baselinespyear == 2013		
	
			foreach type in n e i{
				foreach x in 01 001 0{
					
					*Variation in instruments for each cell
						bys `type'`x': egen sd_z_`type'`x'	= 	sd(Z_`type')
					
					*Good cell
						gen in_good_cell_`type'`x' = (sd_z_`type'`x'>0 & !missing(sd_z_`type'`x') & in_zero1_`type'pscore==1) if in_`type'_sample==1
					
					gen double tag_`type'`x' = `type'`x' if in_`type'_sample==1
					replace tag_`type'`x' = -9999999 if in_good_cell_`type'`x'==0 & in_`type'_sample==1
					tab tag_`type'`x' if in_`type'_sample==1, gen(tag_`type'`x'_)
					
					*Drop var for things with no var (the first one). These observations will not be included
					*in bin-based regression and fully saturated regressios. So, do not include their corresponding
					*variables either
					
					drop tag_`type'`x'_1
					
					*Obs per cell
					
					bys `type'`x': egen obs_cell_`type'`x' = count(maskedid) if in_`type'_sample==1
					}
				}
			
			drop sd_z_*		
			
			save "`cleandata'analysis_noex_3endog_stacked_`file'.dta", replace
		}
	}

	*****************************************
	******** Balance and attrition **********
	*****************************************
	
	if `pscore_balance'==1{
	
		foreach file in simulation frequency formula{
			
				u "`cleandata'analysis_noex_3endog_stacked_`file'.dta", clear
				
				
				**********************************************************************************
				************************ Balance and attrition coefficients **********************
				**********************************************************************************
		
				gen var =""
				
				*Count points in the support
				replace var = "Riskset points of support" if _n ==1
			
				
				foreach type in n e i{
						
						if "`type'"=="n"{
							local colbump = 0
						}
						else if "`type'"=="e"{
							local colbump = 7
						}
						else if "`type'"=="i"{
							local colbump = 14
						}
						
						local row = 2
						local col = 1 + `colbump'
						
						
						*Place it on top of balance table
						unique pscore_`type'1 		if in_zero1_`type'pscore==1
						matrix balance[1,3 + `colbump'] = r(sum)
						
						unique `type'01				if in_good_cell_`type'01==1
						matrix balance[1,5 + `colbump'] = r(sum)
						
						unique `type'001				if in_good_cell_`type'001==1
						matrix balance[1,6 + `colbump'] = r(sum)

						unique `type'0				if in_good_cell_`type'0==1
						matrix balance[1,7 + `colbump'] = r(sum)
						
					
					foreach var in `appcov_`type'' `cov_`type'' `cov2' `attrition_vars'{
						replace var="`var'" if _n==`row'
						replace var="`var'_se" if _n==`row'+1
						replace var="`var'_N" if _n==`row'+2
						replace var="`var'_N(var in Z)" if _n==`row'+3
							
						*All non-offered applicants
						qui summ `var' if Z_`type'==0 & in_`type'_sample==1
							matrix balance[`row',`col']   = r(mean)
							matrix balance[`row'+1,`col'] = r(sd)
							matrix balance[`row'+2,`col' ] = r(N)
					
						*Balance and attrition, no control(OLS)
						qui reg `var' Z_`type' if in_`type'_sample==1, vce(cluster maskedid)
							matrix balance[`row',`col' + 1]   = _b[Z]
							matrix balance[`row'+1,`col'  +1] = _se[Z]
							matrix balance[`row'+2,`col'  +1] = e(N)
						
						*Balance and attrition, pscore linear control(OLS)
						qui reg `var' Z_`type' pscore_`type'1 if in_zero1_`type'pscore==1, vce(cluster maskedid)
							matrix balance[`row',`col'  +2]   = _b[Z]
							matrix balance[`row'+1,`col'  +2] = _se[Z]
							matrix balance[`row'+2,`col'  +2] = e(N)
							
						*Balance and attrition, pscore polynomial control(OLS)
						qui reg `var' Z_`type' pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 if in_zero1_`type'pscore==1, vce(cluster maskedid)
							matrix balance[`row',`col'  +3]   = _b[Z]
							matrix balance[`row'+1,`col'  +3] = _se[Z]
							matrix balance[`row'+2,`col' + 3] = e(N)
						
						*Balance and attrition, risks
						local c = 0
						foreach i in 01 001 0{
							local c = `c' +1
							
							preserve
								qui areg `var' Z_`type' if in_zero1_`type'pscore==1, absorb(`type'`i') vce(cluster maskedid)
							
								matrix balance[`row',`col' + 3 + `c']   = _b[Z]
								matrix balance[`row'+1,`col' + 3 + `c'] = _se[Z]
								matrix balance[`row'+2,`col' + 3 + `c'] = e(N)
								
								bys `type'`i': egen sd = sd(`var')
								bys `type'`i': egen sd_z = sd(Z_`type')
								
								*Count kids with variation in instrument within cell (no ref to characteristic)
								count if sd_z > 0 & !missing(sd_z)  & e(sample)
								matrix balance[`row'+3,`col' + 3 + `c'] = r(N)
								
								drop sd sd_z		
							restore
						}
						
					local row = `row' + 6
					} /*Close covariates loop*/
				
			} /*Close n, e loop*/
			
			************************************* F tests **************************************	
				*MS 2/23/2015: 
				*Minimum number of observations neeed for the SUEST F test to run according to each pscore type
				*Arrived at these numbers by running F tests for several different minimum number of obs. These
				*are the minimum needed for the set of covariates being tested as of 2/23/2015.
				
				local min_obs_cell_simulation 	= 9			
				local min_obs_cell_formula 		= 9
				local min_obs_cell_frequency 	= 9
				
			
			if `ftest'==1{
				
				**********************************************************************************
				************************************* Logit Fs ***********************************
				**********************************************************************************
				
				foreach f in robust traditional{
					
						foreach covs in apponly balanceonly all{
						
							foreach type in n e i{
						
								if "`type'"=="n"{
									local colbump = 0
								}
								else if "`type'"=="e"{
									local colbump = 7
								}
								else if "`type'"=="i"{
									local colbump = 14
								}
								
								local col = 1 + `colbump'
						
								if "`covs'"=="apponly"{
									local balF "`appcov_`type''"
									local Fname "Application Balance"
								}
								
								else if "`covs'"=="balanceonly"{
									local balF "`cov_`type'' `cov2'"
									local Fname "Baseline Balance"
								}
								else if "`covs'"=="all"{
									local balF "`appcov_`type'' `cov_`type'' `cov2'"
									local Fname "All Balance"
								}
								
								
								if "`f'"=="robust"{
									local option "nocnsreport vce(cluster maskedid)"
								}
								else{
									local option "nocnsreport"
								}
							
							
								* Balance logit
								replace var="Logit `f' `Fname' F" if _n==`row'
								replace var="Logit `f' `Fname' F (pval)" if _n==`row'+1
								replace var="Logit `f' `Fname' F (df)" if _n==`row'+2
								replace var="Logit `f' `Fname' F (N)" if _n==`row'+3
							
								*No riskset
								qui logit Z_`type' `balF', `option'
								test `balF'
								matrix balance[`row', `col' +1] = r(chi2)
								matrix balance[`row'+1, `col' +1] = r(p)
								matrix balance[`row'+2, `col' +1] = r(df)
								matrix balance[`row'+3, `col' +1] = e(N)	
			
								*Linear riskset
								qui logit Z_`type' pscore_`type'1 `balF' if in_zero1_`type'pscore==1, `option'
								test `balF'
								matrix balance[`row', `col' +2] = r(chi2)
								matrix balance[`row'+1, `col' +2] = r(p)
								matrix balance[`row'+2, `col' +2] = r(df)
								matrix balance[`row'+3, `col' +2] = e(N)
								
								*Poly riskset
								qui logit Z_`type' pscore_`type'* `balF' if in_zero1_`type'pscore==1, `option'
								test `balF'
								matrix balance[`row', `col' +3] = r(chi2)
								matrix balance[`row'+1, `col' +3] = r(p)
								matrix balance[`row'+2, `col' +3] = r(df)
								matrix balance[`row'+3, `col' +3] = e(N)
								
								*Bins
								local c=0
								foreach i in 01 001 0{
									local c = `c' + 1
									qui logit Z_`type' tag_`type'`i'_* `balF' if in_good_cell_`type'`i'==1, `option'
									test `balF'
									matrix balance[`row', `col'+3 + `c'] = r(chi2)
									matrix balance[`row' +1, `col'  +3 + `c'] = r(p)
									matrix balance[`row'+2, `col'  +3 + `c'] = r(df)
									matrix balance[`row'+3, `col'  +3 + `c'] = e(N)
								}

						} /*CLose n e loop*/			
					local row = `row' + 4
					}	/*Close balance covariates loop*/
			}	/*CLose robust/traditional F*/
				
				********************************************************
				*************** Balance Traditional F ******************
				********************************************************
						
				foreach covs in apponly balanceonly all{
				
					foreach type in n e i{
						
						if "`type'"=="n"{
							local colbump = 0
						}
						else if "`type'"=="e"{
							local colbump = 7
						}
						else if "`type'"=="i"{
							local colbump = 14
						}
						
						local col = 1 + `colbump'
											
							if "`covs'"=="apponly"{
								local balF "`appcov_`type''"
								local Fname "Application Balance"
							}
							
							else if "`covs'"=="balanceonly"{
								local balF "`cov_`type'' `cov2'"
								local Fname "Baseline Balance"
							}
							else if "`covs'"=="all"{
								local balF "`appcov_`type'' `cov_`type'' `cov2'"
								local Fname "All Balance"
							}
									
							replace var="Traditional `Fname' F-test" if _n==`row'
							replace var="Traditional `Fname' F-test (pval)" if _n==`row' +1
							replace var="Traditional `Fname' F-test (residuals df)" if _n==`row' +2
							replace var="Traditional `Fname' F-test (model df)" if _n==`row' +3
							replace var="Traditional `Fname' F-test (N)" if _n==`row' +4
							replace var="Traditional `Fname' F-test (Breusch-Pagan chi2)" if _n==`row'+5
					
						
						*no propensity score
							mvreg `balF' = Z_`type', corr
							test Z_`type'
							matrix balance[`row',`col'  + 1] = r(F)
							matrix balance[`row'+1,`col' + 1]= r(p)
							matrix balance[`row'+2, `col' +1] = r(df_r)
							matrix balance[`row'+3, `col' +1] = r(df)
							matrix balance[`row'+4, `col' +1] = e(N)
							matrix balance[`row'+5, `col' +1] = 1- chi2(e(df_chi2),e(chi2))
						
						*linear pscore control
							mvreg `balF' = Z_`type' pscore_`type'1 if in_zero1_`type'pscore==1, corr
							test Z_`type'
							matrix balance[`row',`col'  +2]=r(F)
							matrix balance[`row'+1,`col'  +2]=r(p)
							matrix balance[`row'+2, `col' +2] = r(df_r)
							matrix balance[`row'+3, `col' +2] = r(df)
							matrix balance[`row'+4, `col' +2] = e(N)
							matrix balance[`row'+5, `col' +2] = 1- chi2(e(df_chi2),e(chi2))
							
						*polynomial pscore control
							mvreg `balF' = Z_`type' pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 if in_zero1_`type'pscore==1, corr
							test Z_`type'
							matrix balance[`row',`col'  + 3]=r(F)
							matrix balance[`row'+1,`col' + 3]=r(p)
							matrix balance[`row'+2, `col' +3] = r(df_r)
							matrix balance[`row'+3, `col' +3] = r(df)
							matrix balance[`row'+4, `col' +3] = e(N)
							matrix balance[`row'+5, `col' +3] = 1- chi2(e(df_chi2),e(chi2))
							
						*Bins
							local c=0
							foreach i in 01 001 0{
								local c = `c' + 1
								mvreg `balF' = Z_`type' tag_`type'`i'_*  if in_good_cell_`type'`i'==1, corr
								test Z_`type'
								matrix balance[`row',`col'  + 3 + `c'] = r(F)
								matrix balance[`row'+1,`col'  + 3 + `c'] = r(p)
								matrix balance[`row'+2,`col'  + 3 + `c'] = r(df_r)
								matrix balance[`row'+3,`col'  + 3 + `c'] = r(df)
								matrix balance[`row'+4,`col'  + 3 + `c'] = e(N)
								matrix balance[`row'+5,`col'  + 3 + `c'] = 1- chi2(e(df_chi2),e(chi2))
								
							}

										
						}	/*Close c i loop*/
				local row = `row' + 6
				}	/*Close covariates loop*/
			
				
				**********************************************************************************
				*************************** Balance Stacked Robust F *****************************
				**********************************************************************************
				
				*Run a cut keeping cells with small size and one dropping cells with small size
				
				local col = 1	
					
				foreach f_cut in keep_small_cells drop_small_cells{
					
						foreach covs in apponly balanceonly all{
						
							foreach type in n e i{
				
								if "`type'"=="n"{
									local colbump = 0
								}
								else if "`type'"=="e"{
									local colbump = 7
								}
								else if "`type'"=="i"{
									local colbump = 14
								}
								
								local col = 1 + `colbump'
							
								if "`covs'"=="apponly"{
									local balF "`appcov_`type''"
									local Fname "Application Balance"
									local expand = 3
									local test "Z_1 Z_2 Z_3"
								}
								
								else if "`covs'"=="balanceonly"{
									local balF "`cov_`type'' `cov2'"
									local Fname "Baseline Balance"
									local expand = 12
									local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12"
								}
								else if "`covs'"=="all"{
									local balF "`appcov_`type'' `cov_`type'' `cov2'"
									local Fname "All Balance"
									local expand = 15
									local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12 Z_13 Z_14 Z_15"
								}
								
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test" if _n==`row'
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (pval)" if _n==`row'+1
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (residuals df)" if _n==`row'+2
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (model df)" if _n==`row'+3
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (N)" if _n==`row'+4
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (cells)" if _n==`row'+5
								
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test" if _n==`row'+6
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (pval)" if _n==`row'+7
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (residuals df)" if _n==`row'+8
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (model df)" if _n==`row'+9
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (N)" if _n==`row'+10
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (cells)" if _n==`row'+11
								
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test" if _n==`row' +12
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (pval)" if _n==`row'+13
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (residuals df)" if _n==`row'+14
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (model df)" if _n==`row'+15
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (N)" if _n==`row'+16
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (cells)" if _n==`row'+17
						
						
								preserve
									
									if "`f_cut'" === "keep_small_cells"{
										local min_obs = 1
									}
									else if "`f_cut'"=="drop_small_cells"{
										local min_obs = `min_obs_cell_`file''
									}
									
									keep 	maskedid `balF' Z_`type' tag_`type'01 tag_`type'001 tag_`type'0 `type'01 `type'001 `type'0 ///
											pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 in_*_sample in_good_cell_* obs_cell_* in_zero1_`type'pscore
											
									expand `expand'
									bys maskedid: gen copy = _n
									sort copy
									
									*Single vector of outcomes
									gen cov =.
									gen covname=""
									tokenize `balF'
									forval i=1/`expand'{
										replace cov = ``i'' if copy==`i' 
										replace covname = "``i''" if copy==`i' 
									}
									drop `balF'
									
									*Instruments, pscore variables, and dummies for each copy
									forval i=1/`expand'{
										gen Z_`i' = Z_`type'*(copy == `i')
										gen alpha_`i' = (copy == `i')
										
										*Pscore variables for each copy
										foreach x in `type'01 `type'001 `type'0 pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4{
											gen c_`x'_`i'= `x'*(copy == `i')
										}
									}
									
									*Absorb takes only one var. Create grouped vars
									foreach x in tag_`type'01 tag_`type'001 tag_`type'0 `type'01 `type'001 `type'0 {
											egen group_`x' = group(copy `x')
										
									}
									
									*Replace with missing if first category
									foreach x in tag_`type'01 tag_`type'001 tag_`type'0{
										replace group_`x' =. if `x'==-9999999
									
									}
															
									*** One tall regression for each specification ***
									
									*No controls				
										*Clustered
										qui reg cov alpha_* Z_* if in_`type'_sample==1, vce(cluster maskedid)
										test `test'
										
										matrix balance[`row',`col' + 1]=r(F)
										matrix balance[`row'+1,`col' +1]=r(p)
										matrix balance[`row'+2,`col' +1]=r(df_r)
										matrix balance[`row'+3,`col' +1]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+4,`col' + 1]=r(sum)
										
										
										*Traditional
										qui reg cov alpha_* Z_*  if in_`type'_sample==1
										test `test'
										
										matrix balance[`row'+6,`col' + 1]=r(F)
										matrix balance[`row'+7,`col' +1]=r(p)
										matrix balance[`row'+8,`col' +1]=r(df_r)
										matrix balance[`row'+9,`col' +1]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+10,`col' + 1]=r(sum)
										
										
										*Robust
										qui reg cov alpha_* Z_* if in_`type'_sample==1, r
										test `test'
										
										matrix balance[`row'+12,`col' + 1]=r(F)
										matrix balance[`row'+13,`col' +1]=r(p)
										matrix balance[`row'+14,`col' +1]=r(df_r)
										matrix balance[`row'+15,`col' +1]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+16,`col' + 1]=r(sum)
										

									*Linear
										*Cluster
										qui reg cov alpha_* Z_* c_pscore_`type'1_* if in_zero1_`type'pscore==1, vce(cluster maskedid)
										test `test'
										
										matrix balance[`row',`col' + 2]=r(F)
										matrix balance[`row'+1,`col' +2]=r(p)
										matrix balance[`row'+2,`col' +2]=r(df_r)
										matrix balance[`row'+3,`col' +2]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+4,`col' + 2]=r(sum)
										
										
										*Traditional
										qui reg cov alpha_* Z_* c_pscore_`type'1_* if in_zero1_`type'pscore==1
										test `test'
										
										matrix balance[`row'+6,`col' + 2]=r(F)
										matrix balance[`row'+7,`col' +2]=r(p)
										matrix balance[`row'+8,`col' +2]=r(df_r)
										matrix balance[`row'+9,`col' +2]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+10,`col' + 2]=r(sum)
										
									
										*Robust
										qui reg cov alpha_* Z_* c_pscore_`type'1_* if in_zero1_`type'pscore==1, r
										test `test'
										
										matrix balance[`row'+12,`col' + 2]=r(F)
										matrix balance[`row'+13,`col' +2]=r(p)
										matrix balance[`row'+14,`col' +2]=r(df_r)
										matrix balance[`row'+15,`col' +2]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+16,`col' + 2]=r(sum)
										
										
									*Polynomial
										*Cluster
										qui reg cov alpha_* Z_* c_pscore_`type'1_* c_pscore_`type'2_* c_pscore_`type'3_* c_pscore_`type'4_* if in_zero1_`type'pscore==1, vce(cluster maskedid)
										test `test'
										
										matrix balance[`row',`col' + 3]=r(F)
										matrix balance[`row'+1,`col' +3]=r(p)
										matrix balance[`row'+2,`col' +3]=r(df_r)
										matrix balance[`row'+3,`col' +3]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+4,`col' + 3]=r(sum)
										
										
7										*Traditional
										qui reg cov alpha_* Z_* c_pscore_`type'1_* c_pscore_`type'2_* c_pscore_`type'3_* c_pscore_`type'4_* if in_zero1_`type'pscore==1
										test `test'
										
										matrix balance[`row'+6,`col' + 3]=r(F)
										matrix balance[`row'+7,`col' +3]=r(p)
										matrix balance[`row'+8,`col' +3]=r(df_r)
										matrix balance[`row'+9,`col' +3]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+10,`col' + 3]=r(sum)
										
										
										
										*Robust
										qui reg cov alpha_* Z_* c_pscore_`type'1_* c_pscore_`type'2_* c_pscore_`type'3_* c_pscore_`type'4_* if in_zero1_`type'pscore==1, r
										test `test'
										
										matrix balance[`row'+12,`col' + 3]=r(F)
										matrix balance[`row'+13,`col' +3]=r(p)
										matrix balance[`row'+14,`col' +3]=r(df_r)
										matrix balance[`row'+15,`col' +3]=r(df)
										
										
										unique maskedid if e(sample)
										matrix balance[`row'+16,`col' + 3]=r(sum)
										
										
									*Bins
										local c = 0
										foreach i in 01 001 0{
											local c = `c' +1
											
											*Cluster
											qui areg cov alpha_* Z_* if in_good_cell_`type'`i'==1 & obs_cell_`type'`i' >= `min_obs' & !missing(obs_cell_`type'`i'), absorb(group_tag_`type'`i') vce(cluster maskedid)
											test `test'
											
											matrix balance[`row',`col' +3 + `c']   = r(F)
											matrix balance[`row'+1,`col' +3 + `c'] = r(p)
											matrix balance[`row'+2,`col'+ 3 + `c'] = r(df_r)
											matrix balance[`row'+3,`col'+3 + `c'] = r(df)
										
											
											unique maskedid if e(sample)
											matrix balance[`row'+4,`col'+3 + `c'] = r(sum)
											
											unique group_tag_`type'`i' if e(sample)
											matrix balance[`row'+5,`col'+3 + `c'] = r(sum)
											
											
											*Traditional
											qui areg cov alpha_* Z_* if in_good_cell_`type'`i'==1 & obs_cell_`type'`i' >= `min_obs' & !missing(obs_cell_`type'`i'), absorb(group_tag_`type'`i')
											test `test'
											
											matrix balance[`row'+6,`col'+3 + `c'] = r(F)
											matrix balance[`row'+7,`col'+3 + `c'] = r(p)
											matrix balance[`row'+8,`col'+3 + `c'] = r(df_r)
											matrix balance[`row'+9,`col'+3 + `c'] = r(df)
											
											
											unique maskedid if e(sample)
											matrix balance[`row'+10,`col'+3 + `c'] = r(sum)
											
											unique group_tag_`type'`i' if e(sample)
											matrix balance[`row'+11,`col'+3 + `c'] = r(sum)
											
											
											*Robust
											qui areg cov alpha_* Z_* if in_good_cell_`type'`i'==1 & obs_cell_`type'`i' >= `min_obs' & !missing(obs_cell_`type'`i'), absorb(group_tag_`type'`i') r
											test `test'
											
											matrix balance[`row'+12,`col'+3 + `c'] = r(F)
											matrix balance[`row'+13,`col'+3 + `c'] = r(p)
											matrix balance[`row'+14,`col'+3 + `c'] = r(df_r)
											matrix balance[`row'+15,`col'+3 + `c'] = r(df)
											
											
											unique maskedid if e(sample)
											matrix balance[`row'+16,`col'+3 + `c'] = r(sum)
											
											unique group_tag_`type'`i' if e(sample)
											matrix balance[`row'+17,`col'+3 + `c'] = r(sum)
											
										}
								
								restore
							} /*Close n, e loop*/
						
						local row = `row' + 18	
						} /*Close vocs loop*/		
				} /*Close f cut*/	
				
				
				**********************************************************************************
				*********************** Suest Robust standard errors *****************************
				**********************************************************************************
				
				** F-tests using suest, which allows for robus or clustered SEs **
				* 1) Run models without clustering option
				* 2) Store results
				* 3) suest [stored values], vce(cluster clustvar)
				* 4) test coefficient
				
				local col = 1
				
				foreach f_cut in keep_small_cells drop_small_cells{

						if "`f_cut'" === "keep_small_cells"{
							local min_obs = 1
						}
						else if "`f_cut'"=="drop_small_cells"{
							local min_obs = `min_obs_cell_`file''
						}
						
							replace var="[`f_cut'] Suest Application Balance F" if _n==`row'
							replace var="[`f_cut'] Suest Application Balance F (pval)" if _n==`row' +1
							replace var="[`f_cut'] Suest Application Balance F (df)" if _n==`row' +2
							replace var="[`f_cut'] Suest Application Balance F (N)" if _n==`row' +3
							replace var="[`f_cut'] Suest Application Balance F (cells)" if _n==`row' +4
							
							replace var="[`f_cut'] Suest Baseline Balance F" if _n==`row' + 5
							replace var="[`f_cut'] Suest Baseline Balance F (pval)" if _n==`row' +6
							replace var="[`f_cut'] Suest Baseline Balance F (df)" if _n==`row' +7
							replace var="[`f_cut'] Suest Baseline Balance F (N)" if _n==`row' +8
							replace var="[`f_cut'] Suest Baseline Balance F (cells)" if _n==`row' +9
							
							replace var="[`f_cut'] Suest All Balance F" if _n==`row' + 10
							replace var="[`f_cut'] Suest All Balance F (pval)" if _n==`row' +11
							replace var="[`f_cut'] Suest All Balance F (df)" if _n==`row' +12
							replace var="[`f_cut'] Suest All Balance F (N)" if _n==`row' +13
							replace var="[`f_cut'] Suest All Balance F (cells)" if _n==`row' +14
						
					foreach type in n e i{
						
						if "`type'"=="n"{
							local colbump = 0
						}
						else if "`type'"=="e"{
							local colbump = 7
						}
						else if "`type'"=="i"{
							local colbump = 14
						}					
						
						local col = 1 + `colbump'
								

						*Balance F
							foreach var in `appcov_`type'' `cov_`type'' `cov2'{
								
								* no pscores
								qui reg `var' Z_`type'
								est store `var'
								
								* linear
								qui reg `var' Z_`type' pscore_`type'1 if in_zero1_`type'pscore==1
								est store L`var'
								
								* poly
								qui reg `var' Z_`type' pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 if in_zero1_`type'pscore==1
								est store P`var'
								
								* Pscore bins
								foreach i in 01 001 0{
									qui reg `var' Z_`type' tag_`type'`i'_* 	if in_good_cell_`type'`i'==1 & obs_cell_`type'`i'>=`min_obs' & !missing(obs_cell_`type'`i')
									est store B`i'`var'
								}
	
							}
							
							*** Apcov suest ***
								cap suest 	num_schools num_`type' rank1_`type', vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row', `col' + 1] = r(chi2)
									matrix balance[`row'+1, `col' + 1] = r(p)
									matrix balance[`row'+2, `col' + 1] = r(df)
									matrix balance[`row'+3, `col' + 1] = e(N)
								}	
									
								cap suest 	Lnum_schools Lnum_`type'  Lrank1_`type', vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row', `col' +2] = r(chi2)
									matrix balance[`row'+1, `col' +2] = r(p)
									matrix balance[`row'+2, `col' +2] = r(df)
									matrix balance[`row'+3, `col' +2] = e(N)
								}
								
								cap suest 	Pnum_schools Pnum_`type' Prank1_`type', vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row', `col' +3] = r(chi2)
									matrix balance[`row'+1, `col' +3] = r(p)
									matrix balance[`row'+2, `col' +3] = r(df)
									matrix balance[`row'+3, `col' +3] = e(N)
								}
								
								local c=0
								foreach i in 01 001 0{
									local c = `c' + 1
									
									cap suest 	B`i'num_schools B`i'num_`type' B`i'rank1_`type', vce(cluster maskedid)
									if !_rc{
										test Z_`type'
										matrix balance[`row', `col' + 3 + `c'] = r(chi2)
										matrix balance[`row'+1, `col' + 3 + `c'] = r(p)
										matrix balance[`row'+2, `col' + 3 + `c'] = r(df)
										matrix balance[`row'+3, `col' + 3 + `c'] = e(N)
										
										
										unique tag_`type'`i' if e(sample)
										matrix balance[`row'+4, `col' + 3 + `c'] = r(sum)
										
									}
								}
								
						** Baseline covs suest ***
							
							cap suest	baseline`type' baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
										baseliness_math baseliness_reading baseliness_writing, vce(cluster maskedid)
							if !_rc{			
								test Z_`type'
								matrix balance[`row'+5, `col'+1] = r(chi2)
								matrix balance[`row'+6, `col'+1] = r(p)
								matrix balance[`row'+7, `col'+1] = r(df)
								matrix balance[`row'+8, `col'+1] = e(N)
							}
								
							cap suest	Lbaseline`type' Lbaselinefemale Lbaselinehisp Lbaselineblack Lbaselinefoodfred Lbaselinesped Lbaselinebilingual Lbaselinelep Lbaselinegifted ///
									Lbaseliness_math Lbaseliness_reading Lbaseliness_writing, vce(cluster maskedid)
							if !_rc{
								test Z_`type'
								matrix balance[`row'+5, `col'+2] = r(chi2)
								matrix balance[`row'+6, `col' +2] = r(p)
								matrix balance[`row'+7, `col'+2] = r(df)
								matrix balance[`row'+8, `col'+2] = e(N)
							}
							
							cap suest	Pbaseline`type' Pbaselinefemale Pbaselinehisp Pbaselineblack Pbaselinefoodfred Pbaselinesped Pbaselinelep Pbaselinebilingual Pbaselinegifted ///
									Pbaseliness_math Pbaseliness_reading Pbaseliness_writing, vce(cluster maskedid)
							if !_rc{
								test Z_`type'
								matrix balance[`row'+5, `col'+3] = r(chi2)
								matrix balance[`row'+6, `col' +3] = r(p)
								matrix balance[`row'+7, `col'+3] = r(df)
								matrix balance[`row'+8, `col'+3] = e(N)
							}
							
							local c=0
							foreach i in 01 001 0{
								local c = `c' + 1
								cap suest	B`i'baseline`type' B`i'baselinefemale B`i'baselinehisp B`i'baselineblack B`i'baselinefoodfred B`i'baselinesped B`i'baselinelep B`i'baselinebilingual B`i'baselinegifted ///
										B`i'baseliness_math B`i'baseliness_reading B`i'baseliness_writing, vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row'+5, `col' +3 + `c'] = r(chi2)
									matrix balance[`row'+6, `col' +3 + `c'] = r(p)
									matrix balance[`row'+7, `col' +3 + `c'] = r(df)
									matrix balance[`row'+8, `col' +3 + `c'] = e(N)
									
									unique tag_`type'`i' if e(sample)
									matrix balance[`row'+9, `col' +3 + `c'] = r(sum)
									
								}
							}
							
							
						*** All covariates suest ***

							cap suest	num_schools num_`type' rank1_`type' ///
									baseline`type' baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
									baseliness_math baseliness_reading baseliness_writing, vce(cluster maskedid)
							
							if !_rc{
								test Z_`type'
								matrix balance[`row'+10, `col'+1] = r(chi2)
								matrix balance[`row'+11, `col'+1] = r(p)
								matrix balance[`row'+12, `col'+1] = r(df)
								matrix balance[`row'+13, `col'+1] = e(N)
							}
							
								
							cap suest	Lnum_schools Lnum_`type' Lrank1_`type' ///
									Lbaseline`type' Lbaselinefemale Lbaselinehisp Lbaselineblack Lbaselinefoodfred Lbaselinesped Lbaselinebilingual Lbaselinelep Lbaselinegifted ///
									Lbaseliness_math Lbaseliness_reading Lbaseliness_writing, vce(cluster maskedid)
							
							if !_rc{
								test Z_`type'
								matrix balance[`row'+10, `col'+2] = r(chi2)
								matrix balance[`row'+11, `col' +2] = r(p)
								matrix balance[`row'+12, `col'+2] = r(df)
								matrix balance[`row'+13, `col'+2] = e(N)
							}
							
							cap suest	Pnum_schools Pnum_`type'  Prank1_`type' ///
									Pbaseline`type' Pbaselinefemale Pbaselinehisp Pbaselineblack Pbaselinefoodfred Pbaselinesped Pbaselinelep Pbaselinebilingual Pbaselinegifted ///
									Pbaseliness_math Pbaseliness_reading Pbaseliness_writing, vce(cluster maskedid)
							
							if !_rc{
								test Z_`type'
								matrix balance[`row'+10, `col'+3] = r(chi2)
								matrix balance[`row'+11, `col' +3] = r(p)
								matrix balance[`row'+12, `col'+3] = r(df)
								matrix balance[`row'+13, `col'+3] = e(N)
							}
							
							local c=0
							foreach i in 01 001 0{
								local c = `c' + 1
								cap suest	B`i'num_schools B`i'num_`type'  B`i'rank1_`type' ///
										B`i'baseline`type' B`i'baselinefemale B`i'baselinehisp B`i'baselineblack B`i'baselinefoodfred B`i'baselinesped B`i'baselinelep B`i'baselinebilingual B`i'baselinegifted ///
										B`i'baseliness_math B`i'baseliness_reading B`i'baseliness_writing, vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row'+10, `col' +3 + `c'] = r(chi2)
									matrix balance[`row'+11, `col' +3 + `c'] = r(p)
									matrix balance[`row'+12, `col' +3 + `c'] = r(df)
									matrix balance[`row'+13, `col' +3 + `c'] = e(N)
									
									
									unique tag_`type'`i' if e(sample)
									matrix balance[`row'+14, `col' +3 + `c'] = r(sum)
									
								}
							}
						
					} /*Close n e loop*/
					
				local row = `row'+ 15
				} /*Close f_cut loop*/
				
					**********************************************************************************
					
				*Outsheet
				preserve
					svmat balance
					keep var balance*
					keep if _n<=`row'+1
					ren balance1 z0_n
					ren balance2 ols_z1_n
					ren balance3 ols_z1_pscore_n
					ren balance4 ols_z1_poly_n
					ren balance5 ols_z1_r01_n
					ren balance6 ols_z1_r001_n
					ren balance7 ols_z1_psat_n
					
					ren balance8 	z0_e
					ren balance9 	ols_z1_e
					ren balance10 	ols_z1_pscore_e
					ren balance11 	ols_z1_poly_e
					ren balance12 	ols_z1_r01_e
					ren balance13 	ols_z1_r001_e
					ren balance14 	ols_z1_psat_e
					
					ren balance15 	z0_i
					ren balance16	ols_z1_i
					ren balance17 	ols_z1_pscore_i
					ren balance18 	ols_z1_poly_i
					ren balance19 	ols_z1_r01_i
					ren balance20 	ols_z1_r001_i
					ren balance21 	ols_z1_psat_i
					
					
					outsheet using "`tables'inDPSbase_`file'_stacked_balance.csv", comma replace
				restore
				
				} /*Close F test condition*/
		}	/*Close file loop*/
	} /*Close balance condition*/
		

					

					
					


	*****************************************************
	******************* Separate 2SLS *******************
	*****************************************************
	
	if `pscore_separate_twosls'==1{
		
		foreach file in frequency /*  formula simulation */{
				foreach type in n e i{
								
					if "`type'"=="n"{
						local colbump = 0
					}
					else if "`type'"=="e"{
						local colbump = 34
					}
					else if "`type'"=="i"{
						local colbump = 68
					}
					
					local row = 1
					local col = 1 + `colbump'
				
					u "`cleandata'analysis_noex_3endog_stacked_`file'.dta", clear
					
					keep if in_n_sample == 1 | in_e_sample==1 | in_i_sample == 1
				
					gen var =""
					
					* Create interaction instruments
					gen Z_n_first = Z_n * rank1_noex
					gen Z_e_first = Z_e * rank1_nonnoex
					gen Z_i_first = Z_i * rank1_innov
					
					*By grade vars
					gen group = .
				
					foreach group in 410 /*45 68 48 910*/{
								if `group'==410{
									replace group=410
								}
								else if `group'==48{
									replace group=48 if inlist(appgrade,4,5,6,7,8)
								}
								else{
								replace group=45 if inlist(appgrade,4,5)
								replace group=68 if inlist(appgrade,6,7,8)
								replace group=910 if inlist(appgrade,9,10)
								}
		
								foreach t in math writing reading{
									foreach out in ss /*prof_up adv*/{
							
									
									replace var="`out'_`t'_`group'" if _n==`row'
									replace var="`out'_`t'_`group'_se" if _n==`row'+1
									replace var="`out'_`t'_`group'_N" if _n==`row'+2
									
				
									qui summ `out'_`t' if group==`group' & in_zero1_`type'pscore==1
									
									if ( r(max)==0 & r(min)==0 ) | r(N)<50{ 
										di in red "No observations for `group'-level 2SLS estimates"
									}
								
									else{
								
										qui summ `out'_`t' if Z_`type' == 0 & group==`group'
										matrix twosls[`row',`col'] = r(mean)
										matrix twosls[`row'+2,`col'] = r(N)
										
											/*OLS - no control for pscore*/
											qui reg `out'_`t' D_`type' `cov' `cov2' `cov_grade' if group==`group' & in_`type'_sample==1, vce(cluster maskedid)
											matrix twosls[`row',`col'+1]   = _b[D]
											matrix twosls[`row'+1,`col'+1] = _se[D]
											matrix twosls[`row'+2,`col'+1] = e(N)
											
											/*OLS conditional on fully saturated pscore sample*/
											qui reg `out'_`t' D_`type' `cov' `cov2' `cov_grade' if group==`group' & in_good_cell_`type'0==1, vce(cluster maskedid)
											matrix twosls[`row',`col'+2]   = _b[D]
											matrix twosls[`row'+1,`col'+2] = _se[D]
											matrix twosls[`row'+2,`col'+2] = e(N)
											
										
											/*OLS conditional controlling for fully saturated pscores*/
											qui reg `out'_`t' D_`type' i.ind_`type'0 `cov' `cov2'  `cov_grade' if group==`group' & in_good_cell_`type'0==1, vce(cluster maskedid)

											matrix twosls[`row',`col'+3]   = _b[D]
											matrix twosls[`row'+1,`col'+3] = _se[D]
											matrix twosls[`row'+2,`col'+3] = e(N)
										
									foreach instrument_type in ordinary over_id  {
										
										if "`instrument_type'" == "ordinary" {
											local instrument "Z_`type'"
										}
										else {
											local instrument "Z_`type' Z_`type'_first"
										}
										
										foreach risk in `type'01 `type'001  `type'0 {
							
											local col = `col' + 2
											
											local controls "i.ind_`risk' `cov_`type'' `cov2'  `cov_grade'"		
										 

											/*2SLS - outcome on treatment instrumented by instrument*/
											ivreg2 `out'_`t' (D_`type' = `instrument') `controls' if group==`group' & in_good_cell_`risk'==1, savefirst savefprefix(first) partial(`controls') cluster(maskedid)

											*2SLS
											matrix twosls[`row',`col'+4]   = _b[D]
											matrix twosls[`row'+1,`col'+4] = _se[D]
											matrix twosls[`row'+2,`col'+4] = e(N)
											
											estimates restore firstD_`type'
											
											*First stage
											matrix twosls[`row',`col'+3]   = _b[Z_`type']
											matrix twosls[`row'+1,`col'+3] = _se[Z_`type']
										
											*If last loop, do these
											if "`risk'"=="`type'0"{
												
												*NAIVE 2SLS - no riskset controls
													*2SLS naive
													ivreg2 `out'_`t' (D_`type' = `instrument') `cov_`type'' `cov2' `cov_grade' if group==`group', savefirst savefprefix(first) partial(`cov_`type'' `cov2' `cov_grade') cluster(maskedid)

														*2SLS
														matrix twosls[`row',`col'+6]   = _b[D]
														matrix twosls[`row'+1,`col'+6] = _se[D]
														matrix twosls[`row'+2,`col'+6] = e(N)
														
														estimates restore firstD_`type'
												
														*First stage
														matrix twosls[`row',`col'+5]   = _b[Z_`type']
														matrix twosls[`row'+1,`col'+5] = _se[Z_`type']
														
													*2SLS naive conditional on r0 sample
													ivreg2 `out'_`t' (D_`type' = `instrument') `cov_`type'' `cov2' `cov_grade' if group==`group' & in_good_cell_`type'0==1, savefirst savefprefix(first) partial(`cov_`type'' `cov2' `cov_grade') cluster(maskedid)

														*2SLS
														matrix twosls[`row',`col'+8]   = _b[D]
														matrix twosls[`row'+1,`col'+8] = _se[D]
														matrix twosls[`row'+2,`col'+8] = e(N)
														
														estimates restore firstD_`type'
												
														*First stage
														matrix twosls[`row',`col'+7]   = _b[Z_`type']
														matrix twosls[`row'+1,`col'+7] = _se[Z_`type']													
												
												*2SLS with no cov controls								
													*2SLS
													ivreg2 `out'_`t' (D_`type' = `instrument') `risk'_* if group==`group' & in_good_cell_`type'0==1, savefirst savefprefix(first) partial(`risk'_*) cluster(maskedid) 
														
														*2SLS
														matrix twosls[`row',`col'+10]   = _b[D]
														matrix twosls[`row'+1,`col'+10] = _se[D]
														matrix twosls[`row'+2,`col'+10] = e(N)
														
														estimates restore firstD_`type'
												
														*First stage
														matrix twosls[`row',`col'+9]   = _b[Z_`type']
														matrix twosls[`row'+1,`col'+9] = _se[Z_`type']
											} /*close if condition*/
										}  /*risk loop*/
										local col = `col' + 9
										} /* instrument type loop */
									
								} /*close condition on number of observations*/
								
								local row = `row' + 3
								local col = 1 + `colbump'
							} 	 /*out loop*/
							
							local ++row
						}	/* t loop*/
						
					}	/*group loop*/
				} /*n e loop*/
			
				preserve
					svmat twosls
					keep var twosls*
					
					keep if _n<=`row'
					
					ren twosls1 z0_n
					ren twosls2 OLS_n
					ren twosls3 OLS_psample_n
					ren twosls4 OLS_psat_n
					
					ren twosls6 FS_r01_n
					ren twosls7 twoSLS_r01_n
					ren twosls8 FS_r001_n
					ren twosls9 twoSLS_r001_n
					ren twosls10 FS_psat_n
					ren twosls11 twoSLS_psat_n
					ren twosls12 FS_naive_n
					ren twosls13 twoSLS_naive_n
					ren twosls14 FS_naive_psample_n
					ren twosls15 twoSLS_naive_psample_n
					ren twosls16 FS_nocovs_n
					ren twosls17 twoSLS_nocovs_n
					
					ren twosls21 FS_r01_n_overid
					ren twosls22 twoSLS_r01_n_overid
					ren twosls23 FS_r001_n_overid
					ren twosls24 twoSLS_r001_n_overid
					ren twosls25 FS_psat_n_overid
					ren twosls26 twoSLS_psat_n_overid
					ren twosls27 FS_naive_n_overid
					ren twosls28 twoSLS_naive_n_overid
					ren twosls29 FS_naive_psample_n_overid
					ren twosls30 twoSLS_naive_psample_n_overid
					ren twosls31 FS_nocovs_n_overid
					ren twosls32 twoSLS_nocovs_n_overid
					
					
					ren twosls35 z0_e
					ren twosls36 OLS_e
					ren twosls37 OLS_psample_e
					ren twosls38 OLS_psat_e
					
					ren twosls40 FS_r01_e
					ren twosls41 twoSLS_r01_e
					ren twosls42 FS_r001_e
					ren twosls43 twoSLS_r001_e					
					ren twosls44 FS_psat_e
					ren twosls45 twoSLS_psat_e
					ren twosls46 FS_naive_e
					ren twosls47 twoSLS_naive_e
					ren twosls48 FS_naive_psample_e
					ren twosls49 twoSLS_naive_psample_e
					ren twosls50 FS_nocovs_e
					ren twosls51 twoSLS_nocovs_e
					
					ren twosls55 FS_r01_e_overid
					ren twosls56 twoSLS_r01_e_overid
					ren twosls57 FS_r001_e_overid
					ren twosls58 twoSLS_r001_e_overid					
					ren twosls59 FS_psat_e_overid
					ren twosls60 twoSLS_psat_e_overid
					ren twosls61 FS_naive_e_overid
					ren twosls62 twoSLS_naive_e_overid
					ren twosls63 FS_naive_psample_e_overid
					ren twosls64 twoSLS_naive_psample_e_overid
					ren twosls65 FS_nocovs_e_overid
					ren twosls66 twoSLS_nocovs_e_overid
					
					ren twosls69 z0_i
					ren twosls70 OLS_i
					ren twosls71 OLS_psample_i
					ren twosls72 OLS_psat_i
					
					ren twosls74 FS_r01_i
					ren twosls75 twoSLS_r01_i
					ren twosls76 FS_r001_i
					ren twosls77 twoSLS_r001_i					
					ren twosls78 FS_psat_i
					ren twosls79 twoSLS_psat_i
					ren twosls80 FS_naive_i
					ren twosls81 twoSLS_naive_i
					ren twosls82 FS_naive_psample_i
					ren twosls83 twoSLS_naive_psample_i
					ren twosls84 FS_nocovs_i
					ren twosls85 twoSLS_nocovs_i
					
					ren twosls89 FS_r01_i_overid
					ren twosls90 twoSLS_r01_i_overid
					ren twosls91 FS_r001_i_overid
					ren twosls92 twoSLS_r001_i_overid					
					ren twosls93 FS_psat_i_overid
					ren twosls94 twoSLS_psat_i_overid
					ren twosls95 FS_naive_i_overid
					ren twosls96 twoSLS_naive_i_overid
					ren twosls97 FS_naive_psample_i_overid
					ren twosls98 twoSLS_naive_psample_i_overid
					ren twosls99 FS_nocovs_i_overid
					ren twosls100 twoSLS_nocovs_i_overid

					outsheet using "`tables'inDPSbase_3endog_`file'_twosls.csv", comma replace
				restore
			clear	

			} /*Close 2SLS condition*/
	} /*Close file loop*/
	
	*****************************************************
	******************** Joint 2SLS *********************
	*****************************************************
	
	if `pscore_joint_twosls'==1{
		
		foreach file in  frequency /*simulation formula  */ {
		
				u "`cleandata'analysis_noex_3endog_stacked_`file'.dta", clear
				
				
				gen in_joint_sample = (in_n_sample==1 | in_e_sample==1 | in_i_sample == 1)
				keep if in_joint_sample
				
				
				********************************************************************************************
				**************** Make propensity score replacements and regenerate dummies *****************
				********************************************************************************************
				*Drop old bucket variables
					drop e0_* e01_* e001_* i0_* i01_* i001_* /*in_good_**/ n*_* tag_* obs_cell* pscore_* ind_*

				*No-excuses or innovation applicants that do not apply to non no-excuses. Replace missing score with zero
					*Score
					replace e0 = 0 if ((in_n_sample == 1 | in_i_sample == 1) & in_e_sample==0) & e0==.
					
					*First rounded to hundredth bucket
					replace e01 = 0 if ((in_n_sample == 1 | in_i_sample == 1) & in_e_sample==0) & e0==0
					
					*First rounded to thousandth bucket
					replace e001 = 0 if ((in_n_sample == 1 | in_i_sample == 1) & in_e_sample==0) & e0==0
				
				
				*Non no-excuses or innovation applicants who do not apply to no-excuses. Replace missing score with zero
					*Score
					replace n0 = 0 if (in_n_sample==0 & (in_e_sample==1 | in_i_sample==1)) & n0==.
					
					*First rounded to hundredth bucket
					replace n01 = 0 if (in_n_sample==0 & (in_e_sample==1 | in_i_sample==1)) & n0==0
					
					*First rounded to thousandth bucket
					replace n001 = 0 if (in_n_sample==0 & (in_e_sample==1 | in_i_sample==1)) & n0==0
					
				*no-excuses or non no-excuses applicants that do not apply to innovation. Replace missing score with zero
					*Score
					replace i0 = 0 if ((in_n_sample == 1 | in_e_sample == 1) & in_i_sample == 0) & i0 == .
					
					*First rounded to hundredth bucket
					replace i01 = 0 if ((in_n_sample == 1 | in_e_sample == 1) & in_i_sample == 0) & i0 == 0

					*First rounded to thousandth bucket
					replace i001 = 0 if ((in_n_sample == 1 | in_e_sample == 1) & in_i_sample == 0) & i0 == 0
					
				*In (0,1) for either no-excuses, non no-excuses, or innovation
					
					gen in_zero1_joint =  ( (e0 > 0 & e0 < 1 ) | (n0 > 0 & n0 < 1 ) | (i0 > 0 & i0 < 1))
				
				*Joint propensity score
					
					egen nei0 = group(e0 n0 i0) if in_zero1_joint
					egen nei01 = group(e01 n01 i01) if in_zero1_joint
					egen nei001 = group(e001 n001 i001) if in_zero1_joint
					
				* re making indicator for score
				foreach x in 0 01 001 {
					foreach score in i e n {
						
						egen ind_`score'`x' = group( `score'`x' ) 
					
					}
				}						
					
				******************************************************************
				************ Replace missing treatment and instrument ************
				******************************************************************
				
				foreach x in Z D{
					replace `x'_n = 0 if `x'_n ==.
					replace `x'_e = 0 if `x'_e ==.
					replace `x'_i = 0 if `x'_i ==.
				}
				
				*recoding offer for students with sim = 0 
				foreach score in e i n {
					
					replace Z_`score' = 0 if `score'0 == 0
				}
								
				
				*In cell with variation?
					
					foreach x in 01 001 0{
						
						*Variation in instruments for each cell
							bys n`x': egen sd_z_n`x'	= 	sd(Z_n)
							bys e`x': egen sd_z_e`x'	= 	sd(Z_e)
							bys i`x': egen sd_z_i`x'    =   sd(Z_i)
						
						*Good cell
							gen in_good_cell_`x' = ( in_zero1_joint==1	& ///
								 ( 	(sd_z_n`x'>0 & !missing(sd_z_n`x')) | (sd_z_e`x'>0 & !missing(sd_z_e`x')) | (sd_z_i`x'>0 & !missing(sd_z_i`x')) 	) )
						
					}
				
					drop sd_z_*
					
					foreach x in 01 001 0{
						bys nei`x': egen sd_z_n	= 	sd(Z_n)
						bys nei`x': egen sd_z_e	= 	sd(Z_e)
						bys nei`x': egen sd_z_i =   sd(Z_i)  
						
						gen in_good_cell_nei`x' = ( in_zero1_joint==1	& ///
										(	( sd_z_n>0 & !missing(sd_z_n) ) | ( sd_z_e>0 & !missing(sd_z_e) ) | ( sd_z_i>0 & !missing(sd_z_i)) ) )
						
						tab nei`x' if in_good_cell_nei`x' ==1
						
						drop sd_z_*
					}
					
				*Regenerate buckets
				
				foreach x in n0 n001 n01  e0 e001 e01 i0 i001 i01 nei0 nei001 nei01{
					tab `x', gen(`x'_)
				}
				
				local col = 1
				local row = 1
				gen var =""
				
				* Create interaction instruments
				gen Z_n_first = Z_n * rank1_noex
				gen Z_e_first = Z_e * rank1_nonnoex
				gen Z_i_first = Z_i * rank1_innov
				
				*By grade vars
				gen group = .
			
				count if in_zero1_joint==1 & inlist(appgrade,4,5,6,7,8)
				count if in_good_cell_0==1 & inlist(appgrade,4,5,6,7,8)
				
				foreach group in 410 /*45 68 48 910*/{
				
							if `group'==410{
								replace group=410
							}
							
							else if `group'==48{
								replace group=48 if inlist(appgrade,4,5,6,7,8)
							}
							
							else{
							replace group=45 if inlist(appgrade,4,5)
							replace group=68 if inlist(appgrade,6,7,8)
							replace group=910 if inlist(appgrade,9,10)
							}
	
							foreach t in math writing reading{
								foreach out in ss /*prof_up adv*/{
								
								local col = 1
								
								replace var="`out'_`t'_`group'" if _n==`row'
								replace var="`out'_`t'_`group'_se" if _n==`row'+1
								replace var="`out'_`t'_`group'_N" if _n==`row'+2
								
			
								qui summ `out'_`t' if group==`group'
								
								if ( r(max)==0 & r(min)==0 ) | r(N)<50{ 
									di in red "Not enough observations for `group'-level 2SLS estimates"
								}
							
								else{
							
									qui summ `out'_`t' if Z_n == 0 & in_n_sample & group==`group'
									matrix jointtwosls[`row',`col'] = r(mean)
									matrix jointtwosls[`row'+2,`col'] = r(N)
									
									qui summ `out'_`t' if Z_e == 0 & in_e_sample & group==`group'
									matrix jointtwosls[`row',`col'+1] = r(mean)
									matrix jointtwosls[`row'+2,`col'+1] = r(N)
									
									qui summ `out'_`t' if Z_i == 0 & in_i_sample & group==`group'
									matrix jointtwosls[`row',`col'+2] = r(mean)
									matrix jointtwosls[`row'+2,`col'+2] = r(N)
									
									local col = `col' + 2
									
										/*OLS - no control for pscore*/
										* commenting this out so it can skip it and run faster. 
										qui reg `out'_`t' D_n D_e D_i `cov_either' `cov_grade' if group==`group', vce(cluster maskedid)
										matrix jointtwosls[`row',`col'+1]   = _b[D_n]
										matrix jointtwosls[`row'+1,`col'+1] = _se[D_n]
										matrix jointtwosls[`row'+2,`col'+1] =  e(N)
										
										matrix jointtwosls[`row',`col'+2]   = _b[D_e]
										matrix jointtwosls[`row'+1,`col'+2] = _se[D_e]
										matrix jointtwosls[`row'+2,`col'+2] =  e(N)

										matrix jointtwosls[`row',`col'+3]   = _b[D_i]
										matrix jointtwosls[`row'+1,`col'+3] = _se[D_i]
										matrix jointtwosls[`row'+2,`col'+3] =  e(N)
										
										
										/*OLS conditional on fully saturated pscore sample*/
										qui reg `out'_`t' D_n D_e D_i `cov_either' `cov_grade' if group==`group' & in_zero1_joint==1, vce(cluster maskedid)
										matrix jointtwosls[`row',`col'+4]   = _b[D_n]
										matrix jointtwosls[`row'+1,`col'+4] = _se[D_n]
										matrix jointtwosls[`row'+2,`col'+4] =  e(N)
										
										matrix jointtwosls[`row',`col'+5]   = _b[D_e]
										matrix jointtwosls[`row'+1,`col'+5] = _se[D_e]
										matrix jointtwosls[`row'+2,`col'+5] =  e(N)

										matrix jointtwosls[`row',`col'+6]   = _b[D_i]
										matrix jointtwosls[`row'+1,`col'+6] = _se[D_i]
										matrix jointtwosls[`row'+2,`col'+6] =  e(N)
										
										/*OLS conditional controlling for fully saturated pscores*/
										
										*qui reg `out'_`t' D_n D_e D_i i.ind_n0 i.ind_e0 i.ind_i0 if group==`group' & in_zero1_joint==1, vce(cluster maskedid)

										matrix jointtwosls[`row',`col'+7]   = _b[D_n]
										matrix jointtwosls[`row'+1,`col'+7] = _se[D_n]
										matrix jointtwosls[`row'+2,`col'+7] =  e(N)
										
										matrix jointtwosls[`row',`col'+8]   = _b[D_e]
										matrix jointtwosls[`row'+1,`col'+8] = _se[D_e]
										matrix jointtwosls[`row'+2,`col'+8] =  e(N)

										matrix jointtwosls[`row',`col'+9]   = _b[D_i]
										matrix jointtwosls[`row'+1,`col'+9] = _se[D_i]
										matrix jointtwosls[`row'+2,`col'+9] =  e(N)
										
									
									
										local col = `col' + 9
										
										
									foreach instrument_type in ordinary over_id  {
										
										if "`instrument_type'" == "ordinary" {
											local instrument "Z_n Z_e Z_i"
										}
										else {
											local instrument "Z_n Z_n_first Z_e Z_e_first Z_i Z_i_first"
										}
										
										
										*Joint 2SLS conditioning separately for risksets
										foreach x in 01 001 0 {

												ivreg2 `out'_`t' (D_n D_e D_i = `instrument') i.ind_n`x' i.ind_e`x' i.ind_i`x' `cov_either' `cov2' `cov_grade' if group==`group' ///
														& in_good_cell_`x'==1, savefirst savefprefix(first) partial(i.ind_n`x' i.ind_e`x' i.ind_i`x' `cov_either' `cov2' `cov_grade') cluster(maskedid) 
													
				
												*2SLS - noex
												matrix jointtwosls[`row',`col'+1]   = _b[D_n]
												matrix jointtwosls[`row'+1,`col'+1] = _se[D_n]
												matrix jointtwosls[`row'+2,`col'+1] = e(N)
												
												*2SLS - non noex
												matrix jointtwosls[`row',`col'+2] = _b[D_e]
												matrix jointtwosls[`row'+1,`col'+2] = _se[D_e]
												matrix jointtwosls[`row'+2,`col'+2] = e(N)
												
												*2SLS - innovation
												matrix jointtwosls[`row',`col'+3] = _b[D_i]
												matrix jointtwosls[`row'+1,`col'+3] = _se[D_i]
												matrix jointtwosls[`row'+2,`col'+3] = e(N)
												
												
												
												*First stage noex
												estimates restore firstD_n
												matrix jointtwosls[`row',`col'+4]   = _b[Z_n]
												matrix jointtwosls[`row'+1,`col'+4] = _se[Z_n]
												
												*First stage non noex
												estimates restore firstD_e
												matrix jointtwosls[`row',`col'+5] = _b[Z_e]
												matrix jointtwosls[`row'+1,`col'+5] = _se[Z_e]
												
												*First stage innovation
												estimates restore firstD_i
												matrix jointtwosls[`row',`col'+6] = _b[Z_i]
												matrix jointtwosls[`row'+1,`col'+6] = _se[Z_i]
											
											local col = `col' + 6
										}
										
										*Joint 2SLS conditioning for joint risksets
										foreach x in 0 01 001 {
											
											
											
												ivreg2 `out'_`t' (D_n D_e D_i = `instrument') i.nei`x' `cov_either' `cov2' `cov_grade' if group==`group' ///
														& in_good_cell_nei`x'==1, savefirst savefprefix(first) partial(i.nei`x' `cov_either' `cov2' `cov_grade') cluster(maskedid)  

												*2SLS - noex
												matrix jointtwosls[`row',`col'+1]   = _b[D_n]
												matrix jointtwosls[`row'+1,`col'+1] = _se[D_n]
												matrix jointtwosls[`row'+2,`col'+1] = e(N)
												
												*2SLS - non noex
												matrix jointtwosls[`row',`col'+2] = _b[D_e]
												matrix jointtwosls[`row'+1,`col'+2] = _se[D_e]
												matrix jointtwosls[`row'+2,`col'+2] = e(N)
												
												*2SLS - innovation
												matrix jointtwosls[`row',`col'+3] = _b[D_i]
												matrix jointtwosls[`row'+1,`col'+3] = _se[D_i]
												matrix jointtwosls[`row'+2,`col'+3] = e(N)
												
												*First stage noex
												estimates restore firstD_n
												matrix jointtwosls[`row',`col'+4]   = _b[Z_n]
												matrix jointtwosls[`row'+1,`col'+4] = _se[Z_n]
												
												*First stage non noex
												estimates restore firstD_e
												matrix jointtwosls[`row',`col'+5] = _b[Z_e]
												matrix jointtwosls[`row'+1,`col'+5] = _se[Z_e]
												
												*First stage non noex
												estimates restore firstD_i
												matrix jointtwosls[`row',`col'+6] = _b[Z_i]
												matrix jointtwosls[`row'+1,`col'+6] = _se[Z_i]
											
											
											local col = `col' + 6
										
										} /*Close riskset loop*/
										
										*NAIVE 2SLS - no riskset controls
											*2SLS naive
											ivreg2 `out'_`t' 	(D_n D_e D_i = `instrument') `cov_either' `cov2' `cov_grade' if group==`group', ///
																savefirst savefprefix(first) partial(`cov_either' `cov2' `cov_grade') cluster(maskedid)  

											*2SLS - noex
												matrix jointtwosls[`row',`col'+1]   = _b[D_n]
												matrix jointtwosls[`row'+1,`col'+1] = _se[D_n]
												matrix jointtwosls[`row'+2,`col'+1] = e(N)
												
												*2SLS - non noex
												matrix jointtwosls[`row',`col'+2] = _b[D_e]
												matrix jointtwosls[`row'+1,`col'+2] = _se[D_e]
												matrix jointtwosls[`row'+2,`col'+2] = e(N)
												
												*2SLS - innovation
												matrix jointtwosls[`row',`col'+3] = _b[D_i]
												matrix jointtwosls[`row'+1,`col'+3] = _se[D_i]
												matrix jointtwosls[`row'+2,`col'+3] = e(N)
												
												*First stage noex
												estimates restore firstD_n
												matrix jointtwosls[`row',`col'+4]   = _b[Z_n]
												matrix jointtwosls[`row'+1,`col'+4] = _se[Z_n]
												
												*First stage non noex
												estimates restore firstD_e
												matrix jointtwosls[`row',`col'+5] = _b[Z_e]
												matrix jointtwosls[`row'+1,`col'+5] = _se[Z_e]
												
												*First stage innovation
												estimates restore firstD_i
												matrix jointtwosls[`row',`col'+6] = _b[Z_i]
												matrix jointtwosls[`row'+1,`col'+6] = _se[Z_i]
												
											*2SLS naive conditional on r0 sample
											ivreg2 `out'_`t' 	(D_n D_e D_i = `instrument') `cov_either' `cov2' `cov_grade' if group==`group' & in_zero1_joint==1, ///
																savefirst savefprefix(first) partial(`cov_either' `cov2' `cov_grade') cluster(maskedid)  
												*2SLS - noex
												matrix jointtwosls[`row',`col'+7]   = _b[D_n]
												matrix jointtwosls[`row'+1,`col'+7] = _se[D_n]
												matrix jointtwosls[`row'+2,`col'+7] = e(N)
											
												*2SLS - non noex
												matrix jointtwosls[`row',`col'+8] = _b[D_e]
												matrix jointtwosls[`row'+1,`col'+8] = _se[D_e]
												matrix jointtwosls[`row'+2,`col'+8] = e(N)
											
												*2SLS - innovation
												matrix jointtwosls[`row',`col'+9] = _b[D_i]
												matrix jointtwosls[`row'+1,`col'+9] = _se[D_i]
												matrix jointtwosls[`row'+2,`col'+9] = e(N)
												
												*First stage noex
												estimates restore firstD_n
												matrix jointtwosls[`row',`col'+10]   = _b[Z_n]
												matrix jointtwosls[`row'+1,`col'+10] = _se[Z_n]
												
												*First stage non noex
												estimates restore firstD_e
												matrix jointtwosls[`row',`col'+11] = _b[Z_e]
												matrix jointtwosls[`row'+1,`col'+11] = _se[Z_e]										

												*First stage innovation
												estimates restore firstD_i
												matrix jointtwosls[`row',`col'+12] = _b[Z_i]
												matrix jointtwosls[`row'+1,`col'+12] = _se[Z_i]											
									
									
										*2SLS with no cov controls								
											*2SLS
											ivreg2 `out'_`t' 	(D_n D_e D_i = `instrument') if group==`group' & in_zero1_joint==1, ///
																savefirst savefprefix(first) cluster(maskedid)  
											
												*2SLS - noex
												matrix jointtwosls[`row',`col'+13]   = _b[D_n]
												matrix jointtwosls[`row'+1,`col'+13] = _se[D_n]
												matrix jointtwosls[`row'+2,`col'+13] = e(N)
												
												*2SLS - non noex
												matrix jointtwosls[`row',`col'+14] = _b[D_e]
												matrix jointtwosls[`row'+1,`col'+14] = _se[D_e]
												matrix jointtwosls[`row'+2,`col'+14] = e(N)
												
												*2SLS - innovation
												matrix jointtwosls[`row',`col'+15] = _b[D_i]
												matrix jointtwosls[`row'+1,`col'+15] = _se[D_i]
												matrix jointtwosls[`row'+2,`col'+15] = e(N)
												
												*First stage noex
												estimates restore firstD_n
												matrix jointtwosls[`row',`col'+16]   = _b[Z_n]
												matrix jointtwosls[`row'+1,`col'+16] = _se[Z_n]
												
												*First stage non noex
												estimates restore firstD_e
												matrix jointtwosls[`row',`col'+17] = _b[Z_e]
												matrix jointtwosls[`row'+1,`col'+17] = _se[Z_e]	
										
												*First stage innovation
												estimates restore firstD_i
												matrix jointtwosls[`row',`col'+18] = _b[Z_i]
												matrix jointtwosls[`row'+1,`col'+18] = _se[Z_i]	
												
									local col = `col' + 18
									} /* instrument type loop */													
							} /*close condition on number of observations*/
							
							local row = `row' + 3
						} 	 /*out loop*/
						
						local ++row
					}	/* t loop*/
					
				}	/*group loop*/
		
			preserve
				svmat jointtwosls
				keep var jointtwosls*

				keep if _n<=`row'
				ren jointtwosls1 z0_n
				ren jointtwosls2 z0_e
				ren jointtwosls3 z0_i
				ren jointtwosls4 OLS_n
				ren jointtwosls5 OLS_e
				ren jointtwosls6 OLS_i
			
				ren jointtwosls7 OLS_psample_n
				ren jointtwosls8 OLS_psample_e
				ren jointtwosls9 OLS_psample_i
				ren jointtwosls10 OLS_psat_n
				ren jointtwosls11 OLS_psat_e
				ren jointtwosls12 OLS_psat_i
		
				ren jointtwosls13 jointtwosls_r01_n
				ren jointtwosls14 jointtwosls_r01_e
				ren jointtwosls15 jointtwosls_r01_i
				ren jointtwosls16 FS_r01_n
				ren jointtwosls17 FS_r01_e
				ren jointtwosls18 FS_r01_i
				
				ren jointtwosls19 jointtwosls_r001_n
				ren jointtwosls20 jointtwosls_r001_e
				ren jointtwosls21 jointtwosls_r001_i
				ren jointtwosls22 FS_r001_n
				ren jointtwosls23 FS_r001_e
				ren jointtwosls24 FS_r001_i
				
				ren jointtwosls25 jointtwosls_r0_n
				ren jointtwosls26 jointtwosls_r0_e
				ren jointtwosls27 jointtwosls_r0_i
				ren jointtwosls28 FS_r0_n
				ren jointtwosls29 FS_r0_e
				ren jointtwosls30 FS_r0_i
	
				ren jointtwosls31 jointtwosls_joint_r01_n
				ren jointtwosls32 jointtwosls_joint_r01_e
				ren jointtwosls33 jointtwosls_joint_r01_i
				ren jointtwosls34 FS_joint_r01_n
				ren jointtwosls35 FS_joint_r01_e
				ren jointtwosls36 FS_joint_r01_i

				ren jointtwosls37 jointtwosls_joint_r001_n
				ren jointtwosls38 jointtwosls_joint_r001_e
				ren jointtwosls39 jointtwosls_joint_r001_i
				ren jointtwosls40 FS_joint_r001_n
				ren jointtwosls41 FS_joint_r001_e
				ren jointtwosls42 FS_joint_r001_i
			
				ren jointtwosls43 jointtwosls_joint_r0_n
				ren jointtwosls44 jointtwosls_joint_r0_e
				ren jointtwosls45 jointtwosls_joint_r0_i
				ren jointtwosls46 FS_joint_r0_n
				ren jointtwosls47 FS_joint_r0_e
				ren jointtwosls48 FS_joint_r0_i
		
				ren jointtwosls49 jointtwosls_naive_n
				ren jointtwosls50 jointtwosls_naive_e
				ren jointtwosls51 jointtwosls_naive_i
				ren jointtwosls52 FS_naive_n
				ren jointtwosls53 FS_naive_e
				ren jointtwosls54 FS_naive_i
		
				ren jointtwosls55 jointtwosls_naive_psample_n
				ren jointtwosls56 jointtwosls_naive_psample_e
				ren jointtwosls57 jointtwosls_naive_psample_i
				ren jointtwosls58 FS_naive_psample_n
				ren jointtwosls59 FS_naive_psample_e
				ren jointtwosls60 FS_naive_psample_i
			
				ren jointtwosls61 jointtwosls_naive_nocovs_n
				ren jointtwosls62 jointtwosls_naive_nocovs_e
				ren jointtwosls63 jointtwosls_naive_nocovs_i
				ren jointtwosls64 FS_naive_nocovs_n
				ren jointtwosls65 FS_naive_nocovs_e
				ren jointtwosls66 FS_naive_nocovs_i

				ren jointtwosls67 jointtwosls_r01_n_oi
				ren jointtwosls68 jointtwosls_r01_e_oi
				ren jointtwosls69 jointtwosls_r01_i_oi
				ren jointtwosls70 FS_r01_n_oi
				ren jointtwosls71 FS_r01_e_oi
				ren jointtwosls72 FS_r01_i_oi
				
				ren jointtwosls73 jointtwosls_r001_n_oi
				ren jointtwosls74 jointtwosls_r001_e_oi
				ren jointtwosls75 jointtwosls_r001_i_oi
				ren jointtwosls76 FS_r001_n_oi
				ren jointtwosls77 FS_r001_e_oi
				ren jointtwosls78 FS_r001_i_oi
		
				ren jointtwosls79 jointtwosls_r0_n_oi
				ren jointtwosls80 jointtwosls_r0_e_oi
				ren jointtwosls81 jointtwosls_r0_i_oi
				ren jointtwosls82 FS_r0_n_oi
				ren jointtwosls83 FS_r0_e_oi
				ren jointtwosls84 FS_r0_i_oi
			
				ren jointtwosls85 jointtwosls_joint_r01_n_oi
				ren jointtwosls86 jointtwosls_joint_r01_e_oi
				ren jointtwosls87 jointtwosls_joint_r01_i_oi	
				ren jointtwosls88 FS_joint_r01_n_oi
				ren jointtwosls89 FS_joint_r01_e_oi
				ren jointtwosls90 FS_joint_r01_i_oi
				
				ren jointtwosls91 jointtwosls_joint_r001_n_oi
				ren jointtwosls92 jointtwosls_joint_r001_e_oi
				ren jointtwosls93 jointtwosls_joint_r001_i_oi
				ren jointtwosls94 FS_joint_r001_n_oi
				ren jointtwosls95 FS_joint_r001_e_oi
				ren jointtwosls96 FS_joint_r001_i_oi
					
				ren jointtwosls97 jointtwosls_joint_r0_n_oi
				ren jointtwosls98 jointtwosls_joint_r0_e_oi
				ren jointtwosls99 jointtwosls_joint_r0_i_oi
				ren jointtwosls100 FS_joint_r0_n_oi
				ren jointtwosls101 FS_joint_r0_e_oi
				ren jointtwosls102 FS_joint_r0_i_oi
				
				ren jointtwosls103 jointtwosls_naive_n_oi
				ren jointtwosls104 jointtwosls_naive_e_oi
				ren jointtwosls105 jointtwosls_naive_i_oi
				ren jointtwosls106 FS_naive_n_oi
				ren jointtwosls107 FS_naive_e_oi
				ren jointtwosls108 FS_naive_i_oi
			
				ren jointtwosls109 jointtwosls_naive_psample_n_oi
				ren jointtwosls110 jointtwosls_naive_psample_e_oi
				ren jointtwosls111 jointtwosls_naive_psample_i_oi
				ren jointtwosls112 FS_naive_psample_n_oi
				ren jointtwosls113 FS_naive_psample_e_oi
				ren jointtwosls114 FS_naive_psample_i_oi
				
				ren jointtwosls115 jointtwosls_naive_nocovs_n_oi
				ren jointtwosls116 jointtwosls_naive_nocovs_e_oi
				ren jointtwosls117 jointtwosls_naive_nocovs_i_oi
				ren jointtwosls118 FS_naive_nocovs_n_oi
				ren jointtwosls119 FS_naive_nocovs_e_oi
				ren jointtwosls120 FS_naive_nocovs_i_oi
				
				outsheet using "`tables'inDPSbase_3endog_`file'_joint_twosls.csv", comma replace
			restore
		clear	

		} /*Close 2SLS condition*/
} /*Close file loop*/
					
					
					
					
