/*
This file uses the qualification files setup in years 1 and 2. 

Relevant columns: 
fs_qual
twosls_qual

Rows: 
ss_math_410
ss_math_410_se

ss_math_410
ss_math_410_se


ss_math_410
ss_math_410_se

ss_reading_410
ss_reading_410_se

ss_writing_410
ss_writing_410_se

ss_math_410_N

*/



clear all
cap log close
set more off
set trace off
set tracedepth 1 
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"
pause on

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"
local rawdata "Y:\DPS\Raw Data\"

matrix demo			  = J(30,30,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,40,.)
matrix subtwosls	  = J(500,40,.)
matrix twosls_nocov    		   = J(500,40,.)
matrix twosls_scorecov 		   = J(500,40,.)
matrix twosls_fcsample   		= J(50	0,40,.)

matrix qualbalance    = J(500,20,.)
matrix qualtwosls	  = J(500,20,.)
matrix qualsubtwosls  = J(500,20,.)


local qual_stacking = 0
local qual_balance=0
local qual_twosls = 0
local qual_subtwosls =0

*Dates of analysis files
local pscoredate_year2	20170310 	/*20160516 20160510 20160428 20160411 20160208 20160112 20151120 20150514 */
local pscoredate_year1  20170310 /*20160516    	20160516 20160510 20160421  20160208 */
local simsdate 			20150603 /* 20150428 */
local qualdate_year1 	20160715 /*20150708  20150420 */
local qualdate_year2	20160715 /*20160303*/
local qualdate			20160303
local qualpscoredate	20160208
local appdatayr1		20170310
local appdatayr2		20170310


local appcov "num_schools num_study rank1_study"
local cov 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_*"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1        		/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
local outsheet_rescorr=0	/* If want to outsheet matrix with residual correlations */
	
	
	***************************************************************************
	***************** Qual instruments stacking ******************
	***************************************************************************

	if `qual_stacking' == 1{
	
		foreach qnum in 20 100{
			
			foreach type in standard naive{
		
				u "`cleandata'mdrd_denver_qual`type'_lt`qnum' `qualdate_year1'.dta", clear
			
				rename riskset_lt`qnum' riskset_lt`qnum'_year1
				keep if baselinespyear == 2012
			
				append using "`cleandata'mdrd_denver_qual`type'_lt`qnum'_year_2 `qualdate_year2'.dta", gen(year2)
				drop if year2 == 1 & baselinespyear != 2013
			
				rename riskset_lt`qnum' riskset_lt`qnum'_year2
				
				egen riskset_lt`qnum'_year1_max = max(riskset_lt`qnum'_year1)
				
				gen riskset_lt`qnum' = riskset_lt`qnum'_year1 if riskset_lt`qnum'_year2 == .
				replace riskset_lt`qnum' = riskset_lt`qnum'_year2 + riskset_lt`qnum'_year1_max if riskset_lt`qnum'_year1 == .
				
				save "`cleandata'mdrd_denver_qual`type'_lt`qnum'_stacked `qualdate'.dta", replace		
			}
		}
	}	
	
**********************************************************
	***************** Qual instruments 2SLS ******************
	**********************************************************

	if `qual_twosls'==1{
	
	foreach choice in  1  all /*2*/ {
	
			if "`choice'"=="1"{
				local qnum = 20
			}
			else if "`choice'"=="all"{
				local qnum = 100
			}
		
		foreach type in standard /*naive*/{
		
			local qualdata "`cleandata'mdrd_denver_qual`type'_lt`qnum'_stacked `qualdate'"
			
			foreach cut in inDPSbase /*allapps*/{
				
				u "`qualdata'", clear
				
				*Analysis is at student level
				duplicates drop maskedid baselinespyear, force
				
				*dropping duplicates from year 2. 
				drop if baselinespyear == . 
				duplicates tag masked, gen(dup_masked)
				drop if dup_masked > 0 & baselinespyear == 2013
				
				
				destring baselinegrade, replace
				gen appgrade_tmp = appgrade
				destring appgrade_tmp, replace
				drop appgrade
				rename appgrade_tmp appgrade
				
				bys maskedid spyear: egen enroll_any_study = max(study)
				duplicates drop maskedid spyear, force
				
				
				*Bring in variable with any charter offer for "offer on qualification" first stage
				* (Doesn't matter which score we pull this from)
				preserve
					u "`cleandata'mdrd_denver_study_formula `qualpscoredate'", clear
					
					append using "`cleandata'mdrd_denver_study_year_2_formula `qualpscoredate'"
					
					cap drop offer_study
					gen offer_study = (offer==1 & appstudy ==1)
					bys maskedid spyear: egen any_study_offer = max(offer_study)
					
					keep maskedid spyear any_study_offer
					duplicates drop
					
					tempfile offers
					sa `offers'
				restore
				
				merge 1:1 maskedid spyear using `offers'
				keep if _merge!=2
				drop _merge
				
				*Keep if in Denver at baseline
				keep if (baselinespyear==2012 | baselinespyear == 2013) & in_sample==1
				
				
				
				*Flag for whether you are subject to randomization in frequency01
				preserve
					u "`cleandata'analysis_stacked_study_frequency", clear
					keep if in_sample & pscore1!=0 & pscore1!=1

					bys r0: egen sd_z= sd(Z)
					keep if sd_z>0 & sd_z!=.
					
					count
					keep maskedid spyear
					tempfile rand
					sa `rand'
				restore
				
				merge 1:1 maskedid spyear using `rand'
				gen psample = (_merge==3)
				drop _merge
				
				ren qual_lt`qnum' Z
				ren enroll_any_study D
				
				
				tab riskset_lt`qnum', gen(r_)
				ren riskset_lt`qnum' r
				
				**********************************************************************************
				******************** Find cells with variation in instrument**********************
				**********************************************************************************
				
				*NOTE that qualification instruments file was already constructed such that obs are
				*in cells with variation in instrument, but do this so that qual balance code can follow
				*the same structure as pscore code

					foreach x in r{
							
						*Variation in instruments for each cell
							bys `x': egen sd_z_`x'	= 	sd(Z)
						
						*Good cell
							gen in_good_cell_`x' = (sd_z_`x'>0 & !missing(sd_z))
						
						drop sd_z_* 
					}
				
	
				******* We want to know the schools applied to by the risksets
				***** with variation in offer. We then want to see if these schools 
				***** have variation in sample for the applicants in the sample. 
				***** Not pretty but it will make sense <3
				
				preserve
					keep if in_good_cell == 1 
					keep masked r in_good_cell
					tempfile good_cells
					sa 	`good_cells'
					unique r if in_good_cell == 1 
					
					u "`qualdata'", clear
					
					merge m:1 masked using `good_cells', keep(match)
					*Risksets with var in offer
					bys r: egen sd_z_lt`choice' = sd(qual_lt`qnum')
					 
					collapse (max) sd_z_lt`choice'  , by(appschoolnum)
					keep if sd_z_lt`choice'  >0 & sd_z_lt`choice' != .
					unique appschoolnum if sd >0 & sd != .
					* now have unique set of schools with var in offer and such. 
					*recording the number and adding it to the output of this section. 
					local unique_schools_var = `r(sum)'
				restore
				
							
				local col = -4
				gen var =""
				
				*By grade vars
				gen group = .
				

					local row =1
					local col = `col' + 5
					
					local risk "r_"
					local controls "i.r `cov' `cov2' `cov_grade'"
				
				foreach group in 410  /* 48 910 */{
							if `group'==410{
								replace group=410
							}
							else if `group'==48{
								replace group=48 if inlist(appgrade,4,5,6,7,8)
							}
							else{
							replace group=45 if inlist(appgrade,4,5)
							replace group=68 if inlist(appgrade,6,7,8)
							replace group=910 if inlist(appgrade,9,10)
							}
				
				
						foreach t in math writing reading {
							foreach out in ss prof_up adv {
							replace var="`out'_`t'_`group'" if _n==`row'
							replace var="`out'_`t'_`group'_se" if _n==`row'+1
							replace var="`out'_`t'_`group'_N" if _n==`row'+2
							
						
							qui summ `out'_`t' if Z == 0 & group==`group'
							matrix qualtwosls[`row',`col'] = r(mean)
							matrix qualtwosls[`row' +2,`col'] = r(N)

								/*OLS*/
								reg `out'_`t' D  `cov1' `cov2' `cov_grade' i.baselinespyear if group==`group', vce(robust) 
								matrix qualtwosls[`row',`col'+1]   = _b[D]
								matrix qualtwosls[`row'+1,`col'+1] = _se[D]
								matrix qualtwosls[`row'+2,`col'+1] = e(N)
								
								/*RF - outcome on instrument*/
								reg `out'_`t' Z `controls' i.baselinespyear if group==`group' & in_good_cell_r == 1, vce(robust)  
								matrix qualtwosls[`row',`col'+2]   = _b[Z]
								matrix qualtwosls[`row'+1,`col'+2] = _se[Z]
								matrix qualtwosls[`row'+2,`col'+2] = e(N)
								
								/*FS - treatment on instrument*/
								reg D Z `controls' i.baselinespyear if group==`group' & `out'_`t'!=. & in_good_cell_r == 1, vce(robust) 
								matrix qualtwosls[`row',`col'+3]   = _b[Z]
								matrix qualtwosls[`row'+1,`col'+3] = _se[Z]
								matrix qualtwosls[`row'+2,`col'+3] = e(N)
								
								/*2SLS - outcome on treatment instrumented by instrument*/
								ivreg2 `out'_`t' (D = Z) `controls' i.baselinespyear if group==`group' & in_good_cell_r == 1, savefirst partial(`controls') robust
								matrix qualtwosls[`row',`col'+4]   = _b[D]
								matrix qualtwosls[`row'+1,`col'+4] = _se[D]
								matrix qualtwosls[`row'+2,`col'+4] = e(N)
									
								*NAIVE 2SLS - no riskset controls
								
									*FS
									reg D Z `cov' `cov2' `cov_grade' i.baselinespyear if group==`group' & `out'_`t'!=., vce(robust) 
									matrix qualtwosls[`row',`col'+5]   = _b[Z]
									matrix qualtwosls[`row'+1,`col'+5] = _se[Z]
									matrix qualtwosls[`row'+2,`col'+5] = e(N)
										
									*2SLS
									ivreg2 `out'_`t' (D = Z) `cov' `cov2' `cov_grade' i.baselinespyear if group==`group', savefirst partial(`cov' `cov2' `cov_grade') robust
									matrix qualtwosls[`row',`col'+6]   = _b[D]
									matrix qualtwosls[`row'+1,`col'+6] = _se[D]
									matrix qualtwosls[`row'+2,`col'+6] = e(N)
										
									*2SLS with no covariates		
									*FS
									reg D Z i.r i.baselinespyear if group==`group' & `out'_`t'!=. & in_good_cell_r == 1, vce(robust) 
									matrix qualtwosls[`row',`col'+7]   = _b[Z]
									matrix qualtwosls[`row'+1,`col'+7] = _se[Z]
									matrix qualtwosls[`row'+2,`col'+7] = e(N)
										
									*2SLS
									ivreg2 `out'_`t' (D = Z) i.r i.baselinespyear if group==`group' & in_good_cell_r == 1, savefirst partial(i.r) robust
									matrix qualtwosls[`row',`col'+8]   = _b[D]
									matrix qualtwosls[`row'+1,`col'+8] = _se[D]
									matrix qualtwosls[`row'+2,`col'+8] = e(N)
										
								*2SLS conditional on psample
								*FS
									reg D Z `controls' i.baselinespyear if group==`group' & psample==1 & `out'_`t'!=. & in_good_cell_r == 1, vce(robust) 
									matrix qualtwosls[`row',`col'+9]   = _b[Z]
									matrix qualtwosls[`row'+1,`col'+9] = _se[Z]
									matrix qualtwosls[`row'+2,`col'+9] = e(N)
										
									*2SLS
									ivreg2 `out'_`t' (D = Z) `controls' i.baselinespyear if group==`group' & psample==1 & in_good_cell_r == 1, savefirst partial(`controls') robust
									matrix qualtwosls[`row',`col'+10]   = _b[D]
									matrix qualtwosls[`row'+1,`col'+10] = _se[D]
									matrix qualtwosls[`row'+2,`col'+10] = e(N)

								*Fake FS: offer at any charter regressed on qualification instrument

									reg any_study_offer Z `controls' i.baselinespyear if group==`group' & in_good_cell_r==1 & `out'_`t' != ., vce(robust) 
									matrix qualtwosls[`row',`col'+11]   = _b[Z]
									matrix qualtwosls[`row'+1,`col'+11] = _se[Z]
									matrix qualtwosls[`row'+2,`col'+11] = e(N)
								
							local row = `row' + 3
							}	/*out loop*/
						local ++row	
						}	/* t loop*/
					}	/*group loop*/
			
		
			preserve
				svmat qualtwosls
				keep var qualtwosls*
				keep if _n<=`row'
				ren qualtwosls1 z0
				ren qualtwosls2 OLS_qual
				ren qualtwosls3 RF_qual
				ren qualtwosls4 FS_qual
				ren qualtwosls5 twosls_qual
				
				ren qualtwosls6 FS_qualnaive
				ren qualtwosls7 twosls_qualnaive
				
				ren qualtwosls8 FS_qualnocovs
				ren qualtwosls9 twosls_qualnocovs
				
				ren qualtwosls10 FS_qualpsample
				ren qualtwosls11 twosls_qualpsample
				
				ren qualtwosls12 FS_any_offer_on_qual_Z
				
				ren qualtwosls14 schools_var
				replace schools_var = `unique_schools_var' if var != ""  //adding the schools with var in offer
				outsheet using "`tables'`cut'_qual`type'_twosls_choices_`choice'_stacked.csv", comma replace
				
				
			restore
		clear	
			}	/*Close 2SLS condition*/
		}	/*Close Cut loop*/
	}	/*Close choice loop*/
	
	
	