/*
THis file outputs the obscount csv, which is used for appendix B1. 
It also tabs types to get their frequency.

It does this individually by year.  

Use the formula output. 

Columns: var_applicants	var_applicant_type
Rows: 
allapps
app410
app_study


*/




clear all
cap log close
set more off
set trace off
set maxvar 32767
adopath + "Z:\ado\plus\"

pause on

local rawdata   "Y:/DPS/Raw Data/"
local asgndata  "Y:/DPS/Raw Data/2013_03_21_enrollment-match/"
local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local yusuke	 "Y:/DPS/yusuke/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"


local pscore_setup =1		/*Propensity score matching setup*/

local simsdate 		20150603 	/*20150428 20150420 20150107*/
local date 			20160815 /*20160512 20160510 20160411 20160208 20160325 20160112 20151120 20150820 20150514 20150427 20150420 20150109 20141201 20141110 20141106 20141104*/
local outputdate	20170310 /*20160715 20160516	20160510 20160428  20160411 */
local typedate 		20170310
matrix obs = J(50,50,.)

cap program drop set_bl_spyear
* this program just sets baseline year. 
*it basically extracts the year a student his or her baseline grade. 
program define set_bl_spyear
	syntax, cldata(str) simdate(integer)

	** No baseline for "EC"
	drop if appgrade == "EC"
	destring appgrade, replace
	
	gen gradebase = 	appgrade-1
	gen pregradebase =	appgrade-2
	
	tostring appgrade, replace
	tostring gradebase, replace
	
	replace gradebase = "EC" if gradebase == "-1"
	replace gradebase = "K" if gradebase == "0"
	
	merge m:1 maskedid using "`cldata'sims_csap_wide_20102014 `simdate'"
	drop if _merge == 2
	drop _merge
	
	gen baselinespyear = .
	
	foreach grade in EC K 1 2 3 4 5 6 7 8 9 10 11 {
		replace baselinespyear = spyear`grade' if gradebase == "`grade'"
	}

end

cap program drop add_unique_rows
*this takes a var whose unique vals you want to tab. 
*it gets the uniques for allapps, apps 4-10 and apps in charter
* and then shoves the results into a variable of the dataset. 
program define add_unique_rows
	syntax, unvar(varname) newv(str) [condit(varname )]
	if "`condit'" == "" {
		gen `newv' = . 
		unique `unvar' 
		replace `newv' = `r(sum)' if reason == "allapps"
		
		unique `unvar' if app410 == 1 
		replace `newv' = `r(sum)' if reason == "app410"
		
		unique `unvar' if any_study == 1 & app410 == 1 
		replace `newv' = `r(sum)' if reason == "app_study"	
		
	}
	
	else {
		gen `newv' = . 
		unique `unvar' if `condit' == 1
		replace `newv' = `r(sum)' if reason == "allapps" 
		
		unique `unvar' if app410 == 1 & `condit' == 1
		replace `newv' = `r(sum)' if reason == "app410" 
		
		unique `unvar' if any_study == 1 & app410 == 1  & `condit' == 1
		replace `newv' = `r(sum)' if reason == "app_study"			
		
		
	}
	
end

cap program drop get_sum_stats 
*this sums the relevant variables and stores the stats in the dataset. 
*of interest we want to know how many kids there are per type
program define get_sum_stats
	syntax, [condit(varname )]	
	
	foreach cond in /*allapps app410*/ app_study {
	
		foreach apptype in applicant_type applicant_type_fc applicant_type_sc applicant_type_tc { 
			if "`condit'" == "" {
				
				*We collapse dataset to have unique kids ID and another column, 
				* the other column is the number of kids in the kid's type 
				preserve 
					keep if `cond' == 1
					drop if `apptype' == . 
					egen tag_type =  tag(masked `apptype')
					egen kids_type = total(tag_type), by(`apptype') missing 	
					collapse (first) kids , by(masked)
					su kids_type, detail
				restore 
				
				capture confirm variable mean_kids_`apptype' min_kids_`apptype'  q1_kids_`apptype'	q2_kids_`apptype' q3_kids_`apptype' q4_kids_`apptype' max_kids_`apptype'
				if _rc  != 0 {
					gen mean_kids_`apptype'		= . 
					gen min_kids_`apptype' 		= .	
					
					gen q1_kids_`apptype' 	= . 
					gen q2_kids_`apptype' 	= . 	
					gen q3_kids_`apptype' 	= . 	
					gen q4_kids_`apptype' 	= . 	
					gen max_kids_`apptype' 		= . 	
				}
				
				replace mean_kids_`apptype' 	= `r(mean)' if reason == "`cond'"
				replace min_kids_`apptype' 		= `r(min)' 	if reason == "`cond'"	
				replace q1_kids_`apptype' 	= `r(p25)'  if reason == "`cond'"
				replace q2_kids_`apptype' 	= `r(p25)' 	if reason == "`cond'"
				replace q3_kids_`apptype' 	= `r(p50)' 	if reason == "`cond'"
				replace q4_kids_`apptype' 	= `r(p75)' 	if reason == "`cond'"
				replace max_kids_`apptype' 		= `r(max)' 	if reason == "`cond'"
			}
			else {
				*We collapse dataset to have unique kids ID and another column, 
				* the other column is the number of kids in the kid's type 			
				preserve 
					keep if `cond' == 1 & `condit' == 1
					egen tag_type =  tag(masked `apptype')
					egen kids_type = total(tag_type), by(`apptype') missing
					collapse (first) kids, by(masked)
					su kids_type, detail
				restore 
				
				capture confirm variable mean_kids_`apptype'_`condit' q1_kids_`apptype'_`condit'	q2_kids_`apptype'_`condit' q3_kids_`apptype'_`condit' q4_kids_`apptype'_`condit' max_kids_`apptype'_`condit'
				if _rc  != 0 {
					gen mean_kids_`apptype'_`condit'	= . 
					gen q1_kids_`apptype'_`condit' 	= . 
					gen q2_kids_`apptype'_`condit' 	= . 	
					gen q3_kids_`apptype'_`condit' 	= . 	
					gen q4_kids_`apptype'_`condit' 	= . 	
					gen max_kids_`apptype'_`condit' 		= . 	
				}
				
				replace mean_kids_`apptype'_`condit' 	= `r(mean)' if reason == "`cond'"
				replace max_kids_`apptype'_`condit' 		= `r(min)' 	if reason == "`cond'"	
				replace q1_kids_`apptype'_`condit' 	= `r(p25)'  if reason == "`cond'"
				replace q2_kids_`apptype'_`condit' 	= `r(p25)' 	if reason == "`cond'"
				replace q3_kids_`apptype'_`condit' 	= `r(p50)' 	if reason == "`cond'"
				replace q4_kids_`apptype'_`condit' 	= `r(p75)' 	if reason == "`cond'"
				replace max_kids_`apptype'_`condit' 		= `r(max)' 	if reason == "`cond'"			
			
			}
		}
	}
end


**********************************************
** YEAR 1
*************************************************

insheet using "`asgndata'1_AT_RUN_022262012.csv", double clear


foreach file in frequency {

	if "`file'"=="simulation"{
		*20150420 is for old definition of simulated pscore, aggregated at student level
		local pscoredateyr1 20170310  /*20160816 20160425 20160421 */
	}
	else{
		local pscoredateyr1 year_one_20170310
	}
	
	u "`cleandata'pscore_study_`file'_`pscoredateyr1'.dta", clear	

	
	gen spyear = 2013
	cap ren maskedstu maskedid
	cap destring maskedid, replace


	*Grades that have scores
	merge 1:m maskedid using "`cleandata'AT_RUN_school_level.dta", keepusing(offer appgrade appcharter  appschoolnum appschoolname apprank family applicant_type applicant_type_fc applicant_type_sc applicant_type_tc)
	tostring masked, gen(strmasked)
	gen phantom = ( length(strmasked) < 5 )
	drop if phantom == 1
	
	*Study charters
	gen appstudy = (	inlist(appschoolnum,167, 168, 175, 176, 196, 211, 212, 266, 328) | ///
						inlist(appschoolnum,330, 426, 428, 429, 435, 436, 438, 443, 444, 445, 446) | ///
						inlist(appschoolnum,462, 481, 485, 496, 498, 392, 393, 394, 513, 441, 512, 391) ///
					)	

	*Applicants to any charters/no-excuses/non no-excuses
	bys maskedid: egen any_study = max(appstudy)

	
	gen app410  = inlist(appgrade,"04","05","06","07","08","09","10")
	gen allapps = 1
	gen app_study = (any_study & inlist(appgrade,"04","05","06","07","08","09","10"))
	
	gen reason = ""
	replace reason = "allapps" if _n == 1
	replace reason = "app410" if _n == 2
	replace reason = "app_study" if _n == 3
	add_unique_rows, unvar(masked) newv("var_applicants")
	
	foreach var of varlist applicant_* {
		add_unique_rows, unvar(`var') newv("var_`var'")
	}
	
	unique applicant_type 

	*All kids with missing pscores should be treated as pscore==0
	replace pscore = 0 if pscore==. & any_study==1
	
	gen has_pscore = (pscore!=.)
	drop _merge
	
	set_bl_spyear, cldata("`cleandata'") simdate(`simsdate')
				
	gen app_baseline = ( baselinespyear == 2012)
	
	add_unique_rows, unvar(masked) condit(app_baseline) newv("var_applicants_baseline")
	
	foreach var of varlist applicant_* {
		add_unique_rows, unvar(`var') condit(app_baseline) newv("var_`var'_baseline")
	}
	gen app_baseline_01 = ( baselinespyear == 2012 & pscore < 1 & pscore > 0 )
	
	add_unique_rows, unvar(masked) condit(app_baseline_01) newv("var_applicants_01")
	tab app_baseline_01 app_baseline
	
	foreach var of varlist applicant_* {
		add_unique_rows, unvar(`var') condit(app_baseline_01) newv("var_`var'_01")
	}
	
	gen bs01 = app_baseline_01 
	get_sum_stats
	get_sum_stats, condit(bs01)
	
	keep reason var* mean* min* max* q*
	outsheet using "`tables'obscount_with_types_yr1_`file'.csv", c replace
}


**********************************************
** YEAR 2 
*************************************************



insheet using "`asgndata'1_AT_RUN_022262012.csv", double clear
* getting year 1 masked to tkae away second year apps 
keep masked 
duplicates drop
destring masked, replace
ren masked maskedid 

gen in2013 = 1 

tempfile yr1_masked
sa `yr1_masked'

foreach file in simulation {

	if "`file'"=="simulation"{
		local pscoredate  20170310 /*20160815  20160425 20160324 */
	}
	else{
		local pscoredate 20170310 /*20160411  20160512 20160510  20160208*/
	}	
	
	u "`cleandata'pscore_study_`file'_year_two_`pscoredate'.dta", clear
	
	* removing second times app 
	merge m:1 masked using `yr1_masked'
	drop _merge
	drop if in2013 == 1
	 
	
	gen spyear =2014 
	cap ren maskedstu maskedid
	cap destring maskedid, replace


	if "`file'"=="simulation"{
		**Duplicates exist for maskedid = 786731 (Applied to the same set of schools, but different grade)
		sort maskedid type
		drop if maskedid == 786731 & type[_n] > type[_n-1] & maskedid[_n-1] == 786731
	}	
	 
	*Grades that have scores
	merge 1:m maskedid using "`cleandata'AT_RUN_year_2_school_level.dta", keepusing(offer appgrade appcharter appstudy appstudy_fc appstudy_sc appstudy_tc apptrad appnoex appnonnoex appschoolnum appschoolname apprank family applicant_type applicant_type_fc applicant_type_sc applicant_type_tc)
	tostring masked, gen(strmasked)
	gen phantom = ( length(strmasked) < 5 )
	drop if phantom == 1

	*Applicants to any charters/no-excuses/non no-excuses
	bys maskedid: egen any_study = max(appstudy)
	bys maskedid: egen any_study_fc = max(appstudy_fc)
	bys maskedid: egen any_study_sc = max(appstudy_sc)
	bys maskedid: egen any_study_tc = max(appstudy_tc)
	
	gen app410  = inlist(appgrade,"04","05","06","07","08","09","10")
	gen allapps = 1
	gen app_study = (any_study  & inlist(appgrade,"04","05","06","07","08","09","10") )
	
	gen reason = ""
	replace reason = "allapps" if _n == 1
	replace reason = "app410" if _n == 2
	replace reason = "app_study" if _n == 3
	add_unique_rows, unvar(masked) newv("var_applicants")
	
	foreach var of varlist applicant_* {
		add_unique_rows, unvar(`var') newv("var_`var'")
	}
	
	unique applicant_type 

	*All kids with missing pscores should be treated as pscore==0
	replace pscore = 0 if pscore==. & any_study==1
	
	gen has_pscore = (pscore!=.)
	drop _merge
	
	set_bl_spyear, cldata("`cleandata'") simdate(`simsdate')
				
	gen app_baseline = ( baselinespyear == 2013)
	
	add_unique_rows, unvar(masked) condit(app_baseline) newv("var_applicants_baseline")
	
	foreach var of varlist applicant_* {
		add_unique_rows, unvar(`var') condit(app_baseline) newv("var_`var'_baseline")
	}
	gen app_baseline_01 = ( baselinespyear == 2013 & pscore < 1 & pscore > 0 )
	
	add_unique_rows, unvar(masked) condit(app_baseline_01) newv("var_applicants_01")
	
	foreach var of varlist applicant_* {
		add_unique_rows, unvar(`var') condit(app_baseline_01) newv("var_`var'_01")
	}
	
	gen bs01 = app_baseline_01 
	get_sum_stats
	get_sum_stats, condit(bs01)
	
	keep reason var* mean* min* max* q*
	outsheet using "`tables'obscount_with_types_yr2_`file'.csv", c replace
}




























