/* 	Ye Ji Kee
	Denver main deck tables. These comments last edited: 4/25/2015
		For grades 4-10
		Output: Balance, 2SLS, and subgroup for pscore risksets and qualification instruments

Columns: 
tot_rank	tot_rank		tot_offer	tot_offer		tot_innov_not01	tot_innov_not01	tot_school_not01_firstchoice	tot_school_not01_firstchoice

Rows: 
297
188
159
166
447
177
390
243
405
282
419
395
299
257
434
174
189
205
397
396
289

468
466
471
464
469
467
510
511		
*/

clear all
cap log close
set more off
set trace off
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"

matrix demo			  = J(30,30,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,100,.)
matrix jointtwosls	  = J(500,100,.)

local pscore_setup_yr1 = 1
local pscore_setup_yr2 = 1
local pscore_stacking = 0
local pscore_balance = 0


*Dates of analysis files
local pscoredate	20170310 /*20160715 20160425*/
local simsdate 		20150603	/*20150428*/

local appcov_c 	"num_schools num_study rank1_study"
local cov_c 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov_innov 	"baselineinnov baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_1 g_2 g_3 g_4 g_5 g_6 g_7"

local pscoredate_year2	20170310
local pscoredate_year1  20170310

local appcov_i  "num_schools num_innov rank1_innov"
local cov_i		"baselineinnov baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local appcov_either  "num_schools num_study rank1_study num_innov rank1_innov"
local cov_either 	"baselinestudy baselineinnov baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1				/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
		
	


	************************************************
	**************** Pscore Setup (for yr 1) ******************
	************************************************	
	
	if `pscore_setup_yr1'==1{
		
		foreach file in simulation frequency formula{
	
			use "`cleandata'mdrd_denver_innov_`file' `pscoredate'", clear
			
			destring baselinegrade, replace
			destring appgrade, gen(appgrade_tmp)
			drop appgrade
			rename appgrade_tmp appgrade
			
			*High school flag			
				gen hsgrade = ( appgrade>=9)
				gen msgrade = ( appgrade<9)
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
				
			*Analysis at the student level. Generate max for charter offer
			bys maskedid: egen any_study_offer 	= max(offer_study)
			bys maskedid: egen any_innov_offer 	= max(offer_innov)
			
			bys maskedid: egen num_study = count(cond(appstudy==1,appschoolnum,.))
			bys maskedid: egen num_innov = count(cond(appinnov==1,appschoolnum,.))
			
			bys maskedid: egen num_schools = count(appschoolnum)
			
			*Enrolls at charter flag (student level)
			bys maskedid: egen enroll_any_study 	= max(study)
			bys maskedid: egen enroll_any_innov		= max(innov)
			
			*Rank 1 at school level
			gen rank1_study= (appstudy==1 & apprank==1)
			gen rank1_innov= (appinnov==1 & apprank==1)
		
			************************** School oversubscription table **************************
			*Schools ranked, assigned, 0 < prob assignment < 1 
			preserve
				keep if appinnov==1
				bys appschoolnum: egen tot_rank1 = sum(rank1_innov)
				bys appschoolnum: egen tot_rank = count(maskedid)
				bys appschoolnum: egen tot_offer = sum(offer)
				bys appschoolnum: egen tot_innov_not01 = count(cond(pscore_innov_`file'>0 & pscore_innov_`file'<1,maskedid,.))
				bys appschoolnum: egen tot_school_not01 = count(cond(pscore_school>0 & pscore_school<1,maskedid,.))
				bys appschoolnum: egen tot_school_is1 = count(cond(pscore_school==1,maskedid,.))
				bys appschoolnum: egen tot_school_not01_firstchoice = count(cond(pscore_school > 0 & pscore_school < 1 & apprank == 1,maskedid,.))
						
				duplicates drop appschoolnum, force
				keep appschoolnum appschoolname tot_* MS HS
				
				tempfile temp
				sa `temp'
			restore
			
			
			*Subject to randomization at the FULLY SATURATED PSCORE level (variation in instrument for every value of PSCORE) preserve
				
			preserve
				keep if baselinespyear==2012
				keep if appinnov==1
				
				*Simulation has huge support. Use rounded values for full saturation
				if "`file'"=="simulation"{
					ren pscore_innov_simulation sim_pscore_i
					gen double pscore_innov_simulation = round(sim_pscore_i,0.001)
				}
				
				*Any charter offer pscore - by school, the number of kids in pscore sets with variation in any offer
					bys pscore_innov_`file': egen min_offer = min(any_innov_offer)
					bys pscore_innov_`file': egen max_offer = max(any_innov_offer)
					
					gen sum = min_offer + max_offer
					bys appschoolnum: egen tot_innov_rand = count(cond(sum==1 ,maskedid,.))
					bys appschoolnum: egen tot_innov_rand_not01 = count(cond(sum==1 & pscore_innov_`file'>0 & pscore_innov_`file'<1,maskedid,.))
						
				*School offer - if within the same school, there are kids with the same pscore and var in offer
					drop min_offer max_offer sum
					bys appschoolnum pscore_school: egen min_offer = min(offer)
					bys appschoolnum pscore_school: egen max_offer = max(offer)
					
					gen sum = min_offer + max_offer
					bys appschoolnum: egen tot_school_rand = count(cond(sum==1,maskedid,.))
					bys appschoolnum: egen tot_school_rand_not01 = count(cond(sum==1 & pscore_school>0 & pscore_school<1,maskedid,.))
					drop min_offer max_offer
					
					*Number of kids with family link
					bys appschoolnum: egen tot_family = sum(family)
					
					duplicates drop appschoolnum, force
					keep appschoolnum  tot_*
					
					merge 1:1 appschoolnum using `temp'
					drop _merge
					
					order appschoolnum appschoolname MS HS ///
						tot_rank1 tot_rank	tot_offer ///
						tot_innov_not01	tot_school_not01 ///
						tot_innov_rand	tot_innov_rand_not01	///
						tot_school_rand	tot_school_rand_not01
				
					outsheet using "`tables'inDPSbase_`file'_innov_schools.csv", comma replace

			restore

		}	/*Close file loop*/	
	}	/*Close pscore condition*/

	
	

	************************************************
	**************** Pscore Setup (for yr 2) ******************
	************************************************		
	
	if `pscore_setup_yr2'==1{
		
		foreach file in simulation frequency formula{
	
			use "`cleandata'mdrd_denver_innov_year_2_`file' `pscoredate'", clear
			
			destring baselinegrade, replace
			destring appgrade, gen(appgrade_tmp)
			drop appgrade
			rename appgrade_tmp appgrade
			
			*High school flag			
				gen hsgrade = ( appgrade>=9)
				gen msgrade = ( appgrade<9)
				bys appschoolnum: egen HS = max(hsgrade)
				bys appschoolnum: egen MS = max(msgrade)
				drop hsgrade msgrade
				
			*Analysis at the student level. Generate max for charter offer
			bys maskedid: egen any_study_offer 	= max(offer_study)
			bys maskedid: egen any_innov_offer 	= max(offer_innov)
			
			bys maskedid: egen num_study = count(cond(appstudy==1,appschoolnum,.))
			bys maskedid: egen num_innov = count(cond(appinnov==1,appschoolnum,.))
			
			bys maskedid: egen num_schools = count(appschoolnum)
			
			*Enrolls at charter flag (student level)
			bys maskedid: egen enroll_any_study 	= max(study)
			bys maskedid: egen enroll_any_innov		= max(innov)
			
			*Rank 1 at school level
			gen rank1_study= (appstudy==1 & apprank==1)
			gen rank1_innov= (appinnov==1 & apprank==1)
		
			************************** School oversubscription table **************************
			*Schools ranked, assigned, 0 < prob assignment < 1 
			preserve
				keep if appinnov==1
				bys appschoolnum: egen tot_rank1 = sum(rank1_innov)
				bys appschoolnum: egen tot_rank = count(maskedid)
				bys appschoolnum: egen tot_offer = sum(offer)
				bys appschoolnum: egen tot_innov_not01 = count(cond(pscore_innov_`file'>0 & pscore_innov_`file'<1,maskedid,.))
				bys appschoolnum: egen tot_school_not01 = count(cond(pscore_school>0 & pscore_school<1,maskedid,.))
				bys appschoolnum: egen tot_school_is1 = count(cond(pscore_school==1,maskedid,.))
				bys appschoolnum: egen tot_school_not01_firstchoice = count(cond(pscore_school > 0 & pscore_school < 1 & apprank == 1,maskedid,.))
						
				duplicates drop appschoolnum, force
				keep appschoolnum appschoolname tot_* MS HS
				
				tempfile temp
				sa `temp'
			restore
			
			
			*Subject to randomization at the FULLY SATURATED PSCORE level (variation in instrument for every value of PSCORE) preserve
				
			preserve
				keep if baselinespyear==2013
				keep if appinnov==1
				
				*Simulation has huge support. Use rounded values to 0.001
				if "`file'"=="simulation"{
					ren pscore_innov_simulation sim_pscore_i
					gen double pscore_innov_simulation = round(sim_pscore_i,0.001)
				}
				
				*Any charter offer pscore - by school, the number of kids in pscore sets with variation in any offer
					bys pscore_innov_`file': egen min_offer = min(any_innov_offer)
					bys pscore_innov_`file': egen max_offer = max(any_innov_offer)
					
					gen sum = min_offer + max_offer
					bys appschoolnum: egen tot_innov_rand = count(cond(sum==1 ,maskedid,.))
					bys appschoolnum: egen tot_innov_rand_not01 = count(cond(sum==1 & pscore_innov_`file'>0 & pscore_innov_`file'<1,maskedid,.))
						
				*School offer - if within the same school, there are kids with the same pscore and var in offer
					drop min_offer max_offer sum
					bys appschoolnum pscore_school: egen min_offer = min(offer)
					bys appschoolnum pscore_school: egen max_offer = max(offer)
					
					gen sum = min_offer + max_offer
					bys appschoolnum: egen tot_school_rand = count(cond(sum==1,maskedid,.))
					bys appschoolnum: egen tot_school_rand_not01 = count(cond(sum==1 & pscore_school>0 & pscore_school<1,maskedid,.))
					drop min_offer max_offer
					
					*Number of kids with family link
					bys appschoolnum: egen tot_family = sum(family)
					
					duplicates drop appschoolnum, force
					keep appschoolnum  tot_*
					
					merge 1:1 appschoolnum using `temp'
					drop _merge
					
					order appschoolnum appschoolname MS HS ///
						tot_rank1 tot_rank	tot_offer ///
						tot_innov_not01	tot_school_not01 ///
						tot_innov_rand	tot_innov_rand_not01	///
						tot_school_rand	tot_school_rand_not01
				
					outsheet using "`tables'inDPSbase_`file'_innov_year_2_schools.csv", comma replace

			restore

			******************************************************************************
		
		}	/*Close file loop*/	
	}	/*Close pscore condition*/

	
	
