/* 	Ye Ji Kee
	Denver main deck tables. These comments last edited: 4/25/2015
		For grades 4-10
		Output: Balance, 2SLS, and subgroup for pscore risksets and qualification instruments

		Outputs files for simulationf frequency and formula
		Columns: 
		z0_i	ols_z1_i	ols_z1_r001_i	ols_z1_r01_i	ols_z1_r001_i	ols_z1_pscore_i	ols_z1_psat_i	ols_z1_psat_i
		Rows: 

num_schools
num_schools_se
num_innov
num_innov_se
rank1_innov
rank1_innov_se


baselineinnov
baselineinnov_se
baselinefemale
baselinefemale_se
baselinehisp
baselinehisp_se
baselineblack
baselineblack_se
baselinegifted
baselinegifted_se
baselinebilingual
baselinebilingual_se
baselinefoodfred
baselinefoodfred_se
baselinelep
baselinelep_se
baselinesped
baselinesped_se
baselinefemale_N(var in Z)

baseliness_math
baseliness_math_se
baseliness_reading
baseliness_reading_se
baseliness_writing
baseliness_writing_se
baseliness_math_N(var in Z)


enrolls_next
enrolls_next_se
has_sc
has_sc_se
has_sc_N(var in Z)

riskset points of support

Traditional All Balance F-test
Traditional All Balance F-test (pval)
		
*/

clear all
cap log close
set more off
set trace off
set maxvar 32767
set matsize 11000
set seed 42
adopath + "Z:/ado/plus/"

local cleandata "Y:/DPS/MDmeetsRD Denver/Data/"
local tables	 "Y:/DPS/MDmeetsRD Denver/Tables/stata output/years_stacked/"

matrix demo			  = J(30,30,.)
matrix balance        = J(500,40,.)
matrix twosls         = J(500,100,.)
matrix jointtwosls	  = J(500,100,.)


local pscore_balance = 0


local appcov_c 	"num_schools num_study rank1_study"
local cov_c 	"baselinestudy baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov_innov 	"baselineinnov baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"
local cov2	"baseliness_math baseliness_reading baseliness_writing"

*g_1 -g_7 are grade dummies for outcome grades 4-10
local cov_grade "g_1 g_2 g_3 g_4 g_5 g_6 g_7"

local appcov_i  "num_schools num_innov rank1_innov"
local cov_i		"baselineinnov baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local appcov_either  "num_schools num_study rank1_study num_innov rank1_innov"
local cov_either 	"baselinestudy baselineinnov baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted"

local attrition_vars 		"has_sc enrolls_next"
local attrition_vars_test 	"has_sc enrolls_next"
local ftest=1				/* If want to run balance F-tests - always set to one unless you just want to de-bug balance estimates*/
		
	

	*****************************************
	******** Balance and attrition **********
	*****************************************
	
	
		foreach file in  frequency /*simulation formula*/{
			
				u "`cleandata'analysis_innov_stacked_`file'.dta", clear
				
				**********************************************************************************
				************************ Balance and attrition coefficients **********************
				**********************************************************************************
		
				gen var =""
				
				*Count points in the support
				replace var = "Riskset points of support" if _n ==1
			
				
				foreach type in c i{
						
						if "`type'"=="c"{
							local colbump = 0
						}
						else if "`type'"=="i"{
							local colbump = 7
						}
						
						local row = 2
						local col = 1 + `colbump'
						
						*Place it on top of balance table
						unique pscore_`type'1 		if in_zero1_`type'pscore==1
						matrix balance[1,3 + `colbump'] = r(sum)
						
						unique `type'01				if in_good_cell_`type'01==1
						matrix balance[1,5 + `colbump'] = r(sum)
						
						unique `type'001				if in_good_cell_`type'001==1
						matrix balance[1,6 + `colbump'] = r(sum)

						unique `type'0				if in_good_cell_`type'0==1
						matrix balance[1,7 + `colbump'] = r(sum)

					
					foreach var in `appcov_`type'' `cov_`type'' `cov2' `attrition_vars'{
						replace var="`var'" if _n==`row'
						replace var="`var'_se" if _n==`row'+1
						replace var="`var'_N" if _n==`row'+2
						replace var="`var'_N(var in Z)" if _n==`row'+3
							
						*All non-offered applicants
						qui summ `var' if Z_`type'==0 & in_`type'_sample==1
							matrix balance[`row',`col']   = r(mean)
							matrix balance[`row'+1,`col'] = r(sd)
							matrix balance[`row'+2,`col' ] = r(N)
					
						*Balance and attrition, no control(OLS)
						qui reg `var' Z_`type' if in_`type'_sample==1, vce(cluster maskedid)
							matrix balance[`row',`col' + 1]   = _b[Z]
							matrix balance[`row'+1,`col'  +1] = _se[Z]
							matrix balance[`row'+2,`col'  +1] = e(N)
						
						*Balance and attrition, pscore linear control(OLS)
						qui reg `var' Z_`type' pscore_`type'1 if in_zero1_`type'pscore==1, vce(cluster maskedid)
							matrix balance[`row',`col'  +2]   = _b[Z]
							matrix balance[`row'+1,`col'  +2] = _se[Z]
							matrix balance[`row'+2,`col'  +2] = e(N)
							
						*Balance and attrition, pscore polynomial control(OLS)
						qui reg `var' Z_`type' pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 if in_zero1_`type'pscore==1, vce(cluster maskedid)
							matrix balance[`row',`col'  +3]   = _b[Z]
							matrix balance[`row'+1,`col'  +3] = _se[Z]
							matrix balance[`row'+2,`col' + 3] = e(N)
						
						*Balance and attrition, risks
						local c = 0
						foreach i in 01 001 0{
							local c = `c' +1
							
							preserve
								qui areg `var' Z_`type' i.baselinespyear if in_zero1_`type'pscore==1, absorb(`type'`i') vce(cluster maskedid)
							
								matrix balance[`row',`col' + 3 + `c']   = _b[Z]
								matrix balance[`row'+1,`col' + 3 + `c'] = _se[Z]
								matrix balance[`row'+2,`col' + 3 + `c'] = e(N)
								
								bys `type'`i': egen sd = sd(`var')
								bys `type'`i': egen sd_z = sd(Z_`type')
								
								*Count kids with variation in instrument within cell (no ref to characteristic)
								count if sd_z > 0 & !missing(sd_z)  & e(sample)
								matrix balance[`row'+3,`col' + 3 + `c'] = r(N)
								
								drop sd sd_z		
							restore
						}
						
					local row = `row' + 6
					} /*Close covariates loop*/
				
			} /*Close c, i loop*/
			
			************************************* F tests **************************************	
				*MS 2/23/2015: 
				*Minimum number of observations neeed for the SUEST F test to run according to each pscore type
				*Arrived at these numbers by running F tests for several different minimum number of obs. These
				*are the minimum needed for the set of covariates being tested as of 2/23/2015.
				
				local min_obs_cell_simulation 	= 9			
				local min_obs_cell_formula 		= 9
				local min_obs_cell_frequency 	= 9
				
			
			if `ftest'==1{
				
				**********************************************************************************
				************************************* Logit Fs ***********************************
				**********************************************************************************
				
				foreach f in robust traditional{
					
						foreach covs in apponly balanceonly all{
						
							foreach type in c i{
						
								if "`type'"=="c"{
									local colbump = 0
								}
								else if "`type'"=="i"{
									local colbump = 7
								}
								
								local col = 1 + `colbump'
						
								if "`covs'"=="apponly"{
									local balF "`appcov_`type''"
									local Fname "Application Balance"
								}
								
								else if "`covs'"=="balanceonly"{
									local balF "`cov_`type'' `cov2'"
									local Fname "Baseline Balance"
								}
								else if "`covs'"=="all"{
									local balF "`appcov_`type'' `cov_`type'' `cov2'"
									local Fname "All Balance"
								}
								
								
								if "`f'"=="robust"{
									local option "nocnsreport vce(cluster maskedid)"
								}
								else{
									local option "nocnsreport"
								}
							
							
								* Balance logit
								replace var="Logit `f' `Fname' F" if _n==`row'
								replace var="Logit `f' `Fname' F (pval)" if _n==`row'+1
								replace var="Logit `f' `Fname' F (df)" if _n==`row'+2
								replace var="Logit `f' `Fname' F (N)" if _n==`row'+3
							
								*No riskset
								qui logit Z_`type' `balF', `option'
								test `balF'
								matrix balance[`row', `col' +1] = r(chi2)
								matrix balance[`row'+1, `col' +1] = r(p)
								matrix balance[`row'+2, `col' +1] = r(df)
								matrix balance[`row'+3, `col' +1] = e(N)	
			
								*Linear riskset
								qui logit Z_`type' pscore_`type'1 `balF' if in_zero1_`type'pscore==1, `option'
								test `balF'
								matrix balance[`row', `col' +2] = r(chi2)
								matrix balance[`row'+1, `col' +2] = r(p)
								matrix balance[`row'+2, `col' +2] = r(df)
								matrix balance[`row'+3, `col' +2] = e(N)
								
								*Poly riskset
								qui logit Z_`type' pscore_`type'* `balF' if in_zero1_`type'pscore==1, `option'
								test `balF'
								matrix balance[`row', `col' +3] = r(chi2)
								matrix balance[`row'+1, `col' +3] = r(p)
								matrix balance[`row'+2, `col' +3] = r(df)
								matrix balance[`row'+3, `col' +3] = e(N)
								
								*Bins
								local c=0
								foreach i in 01 001 0{
									local c = `c' + 1
									qui logit Z_`type' i.ind_i`i' `balF' if in_good_cell_`type'`i'==1, `option'
									test `balF'
									matrix balance[`row', `col'+3 + `c'] = r(chi2)
									matrix balance[`row' +1, `col'  +3 + `c'] = r(p)
									matrix balance[`row'+2, `col'  +3 + `c'] = r(df)
									matrix balance[`row'+3, `col'  +3 + `c'] = e(N)
								}

						} /*CLose c i loop*/			
					local row = `row' + 4
					}	/*Close balance covariates loop*/
			}	/*CLose robust/traditional F*/
				
				********************************************************
				*************** Balance Traditional F ******************
				********************************************************
						
				foreach covs in apponly balanceonly all{
				
					foreach type in c i{
						
						if "`type'"=="c"{
							local colbump = 0
						}
						else if "`type'"=="i"{
							local colbump = 7
						}
						
						local col = 1 + `colbump'
											
							if "`covs'"=="apponly"{
								local balF "`appcov_`type''"
								local Fname "Application Balance"
							}
							
							else if "`covs'"=="balanceonly"{
								local balF "`cov_`type'' `cov2'"
								local Fname "Baseline Balance"
							}
							else if "`covs'"=="all"{
								local balF "`appcov_`type'' `cov_`type'' `cov2'"
								local Fname "All Balance"
							}
									
							replace var="Traditional `Fname' F-test" if _n==`row'
							replace var="Traditional `Fname' F-test (pval)" if _n==`row' +1
							replace var="Traditional `Fname' F-test (residuals df)" if _n==`row' +2
							replace var="Traditional `Fname' F-test (model df)" if _n==`row' +3
							replace var="Traditional `Fname' F-test (N)" if _n==`row' +4
							replace var="Traditional `Fname' F-test (Breusch-Pagan chi2)" if _n==`row'+5
					
						
						*no propensity score
							mvreg `balF' = Z_`type', corr
							test Z_`type'
							matrix balance[`row',`col'  + 1] = r(F)
							matrix balance[`row'+1,`col' + 1]= r(p)
							matrix balance[`row'+2, `col' +1] = r(df_r)
							matrix balance[`row'+3, `col' +1] = r(df)
							matrix balance[`row'+4, `col' +1] = e(N)
							matrix balance[`row'+5, `col' +1] = 1- chi2(e(df_chi2),e(chi2))
						
						*linear pscore control
							mvreg `balF' = Z_`type' pscore_`type'1 if in_zero1_`type'pscore==1, corr
							test Z_`type'
							matrix balance[`row',`col'  +2]=r(F)
							matrix balance[`row'+1,`col'  +2]=r(p)
							matrix balance[`row'+2, `col' +2] = r(df_r)
							matrix balance[`row'+3, `col' +2] = r(df)
							matrix balance[`row'+4, `col' +2] = e(N)
							matrix balance[`row'+5, `col' +2] = 1- chi2(e(df_chi2),e(chi2))
							
						*polynomial pscore control
							mvreg `balF' = Z_`type' pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 if in_zero1_`type'pscore==1, corr
							test Z_`type'
							matrix balance[`row',`col'  + 3]=r(F)
							matrix balance[`row'+1,`col' + 3]=r(p)
							matrix balance[`row'+2, `col' +3] = r(df_r)
							matrix balance[`row'+3, `col' +3] = r(df)
							matrix balance[`row'+4, `col' +3] = e(N)
							matrix balance[`row'+5, `col' +3] = 1- chi2(e(df_chi2),e(chi2))
							
						*Bins
							local c=0
							foreach i in 01 001 0{
								local c = `c' + 1
								mvreg `balF' = Z_`type' i.ind_i`i' i.baselinespyear if in_good_cell_`type'`i'==1, corr
								test Z_`type'
								matrix balance[`row',`col'  + 3 + `c'] = r(F)
								matrix balance[`row'+1,`col'  + 3 + `c'] = r(p)
								matrix balance[`row'+2,`col'  + 3 + `c'] = r(df_r)
								matrix balance[`row'+3,`col'  + 3 + `c'] = r(df)
								matrix balance[`row'+4,`col'  + 3 + `c'] = e(N)
								matrix balance[`row'+5,`col'  + 3 + `c'] = 1- chi2(e(df_chi2),e(chi2))
								
							}

										
						}	/*Close c i loop*/
				local row = `row' + 6
				}	/*Close covariates loop*/
			
				
				**********************************************************************************
				*************************** Balance Stacked Robust F *****************************
				**********************************************************************************
				
				*Run a cut keeping cells with small size and one dropping cells with small size
				
				local col = 1	
					
				foreach f_cut in keep_small_cells drop_small_cells{
					
						foreach covs in apponly balanceonly all{
						
							foreach type in c i{
				
								if "`type'"=="c"{
									local colbump = 0
								}
								else if "`type'"=="i"{
									local colbump = 7
								}
								
								local col = 1 + `colbump'
							
								if "`covs'"=="apponly"{
									local balF "`appcov_`type''"
									local Fname "Application Balance"
									local expand = 3
									local test "Z_1 Z_2 Z_3"
								}
								
								else if "`covs'"=="balanceonly"{
									local balF "`cov_`type'' `cov2'"
									local Fname "Baseline Balance"
									local expand = 12
									local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12"
								}
								else if "`covs'"=="all"{
									local balF "`appcov_`type'' `cov_`type'' `cov2'"
									local Fname "All Balance"
									local expand = 15
									local test "Z_1 Z_2 Z_3 Z_4 Z_5 Z_6 Z_7 Z_8 Z_9 Z_10 Z_11 Z_12 Z_13 Z_14 Z_15"
								}
								
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test" if _n==`row'
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (pval)" if _n==`row'+1
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (residuals df)" if _n==`row'+2
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (model df)" if _n==`row'+3
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (N)" if _n==`row'+4
								replace var = "[`f_cut'] Stacked Clustered `Fname' F-test (cells)" if _n==`row'+5
								
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test" if _n==`row'+6
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (pval)" if _n==`row'+7
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (residuals df)" if _n==`row'+8
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (model df)" if _n==`row'+9
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (N)" if _n==`row'+10
								replace var = "[`f_cut'] Stacked Traditional `Fname' F-test (cells)" if _n==`row'+11
								
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test" if _n==`row' +12
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (pval)" if _n==`row'+13
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (residuals df)" if _n==`row'+14
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (model df)" if _n==`row'+15
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (N)" if _n==`row'+16
								replace var = "[`f_cut'] Stacked Robust `Fname' F-test (cells)" if _n==`row'+17
						
						
								preserve
									
									if "`f_cut'" === "keep_small_cells"{
										local min_obs = 1
									}
									else if "`f_cut'"=="drop_small_cells"{
										local min_obs = `min_obs_cell_`file''
									}
									
									keep 	maskedid `balF' Z_`type' tag_`type'01 tag_`type'001 tag_`type'0 `type'01 `type'001 `type'0 ///
											pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 in_*_sample in_good_cell_* obs_cell_* in_zero1_`type'pscore baselinespyear
											
									expand `expand'
									bys maskedid baselinespyear: gen copy = _n
									sort copy
									
									* if kids repeat years you get a larger max value of copy, which will break 
									* the logic used for the tokenize chunk below. 
									su copy 
									if `r(max)' !=  `expand' {
										di as error " Somethign is off when you make a copy of the data. You need to end up with only `expand' copies per kid. Rn you have `r(max)'"
										exit  
										
									}
																			
									
									*Single vector of outcomes
									gen cov =.
									gen covname=""
									tokenize `balF'
									forval i=1/`expand'{
										replace cov = ``i'' if copy==`i' 
										replace covname = "``i''" if copy==`i' 
									}
									drop `balF'
									
									*Instruments, pscore variables, and dummies for each copy
									forval i=1/`expand'{
										gen Z_`i' = Z_`type'*(copy == `i')
										gen alpha_`i' = (copy == `i')
										
										*Pscore variables for each copy
										foreach x in `type'01 `type'001 `type'0 pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 baselinespyear{
											gen c_`x'_`i'= `x'*(copy == `i')
										}
									}
									
									*Absorb takes only one var. Create grouped vars
									foreach x in tag_`type'01 tag_`type'001 tag_`type'0 `type'01 `type'001 `type'0 {
											egen group_`x' = group(copy `x')
										
									}
									
									*Replace with missing if first category
									foreach x in tag_`type'01 tag_`type'001 tag_`type'0{
										replace group_`x' =. if `x'==-9999999
									
									}
															
									*** One tall regression for each specification ***
									
									*No controls				
										*Clustered
										qui reg cov alpha_* Z_* if in_`type'_sample==1, vce(cluster maskedid)
										test `test'
										
										matrix balance[`row',`col' + 1]=r(F)
										matrix balance[`row'+1,`col' +1]=r(p)
										matrix balance[`row'+2,`col' +1]=r(df_r)
										matrix balance[`row'+3,`col' +1]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+4,`col' + 1]=r(sum)
										
										*Traditional
										qui reg cov alpha_* Z_*  if in_`type'_sample==1
										test `test'
										
										matrix balance[`row'+6,`col' + 1]=r(F)
										matrix balance[`row'+7,`col' +1]=r(p)
										matrix balance[`row'+8,`col' +1]=r(df_r)
										matrix balance[`row'+9,`col' +1]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+10,`col' + 1]=r(sum)
										
										*Robust
										qui reg cov alpha_* Z_* if in_`type'_sample==1, r
										test `test'
										
										matrix balance[`row'+12,`col' + 1]=r(F)
										matrix balance[`row'+13,`col' +1]=r(p)
										matrix balance[`row'+14,`col' +1]=r(df_r)
										matrix balance[`row'+15,`col' +1]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+16,`col' + 1]=r(sum)
										

									*Linear
										*Cluster
										qui reg cov alpha_* Z_* c_pscore_`type'1_* if in_zero1_`type'pscore==1, vce(cluster maskedid)
										test `test'
										
										matrix balance[`row',`col' + 2]=r(F)
										matrix balance[`row'+1,`col' +2]=r(p)
										matrix balance[`row'+2,`col' +2]=r(df_r)
										matrix balance[`row'+3,`col' +2]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+4,`col' + 2]=r(sum)
										
										*Traditional
										qui reg cov alpha_* Z_* c_pscore_`type'1_* if in_zero1_`type'pscore==1
										test `test'
										
										matrix balance[`row'+6,`col' + 2]=r(F)
										matrix balance[`row'+7,`col' +2]=r(p)
										matrix balance[`row'+8,`col' +2]=r(df_r)
										matrix balance[`row'+9,`col' +2]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+10,`col' + 2]=r(sum)
										
										*Robust
										qui reg cov alpha_* Z_* c_pscore_`type'1_* if in_zero1_`type'pscore==1, r
										test `test'
										
										matrix balance[`row'+12,`col' + 2]=r(F)
										matrix balance[`row'+13,`col' +2]=r(p)
										matrix balance[`row'+14,`col' +2]=r(df_r)
										matrix balance[`row'+15,`col' +2]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+16,`col' + 2]=r(sum)
									
									*Polynomial
										*Cluster
										qui reg cov alpha_* Z_* c_pscore_`type'1_* c_pscore_`type'2_* c_pscore_`type'3_* c_pscore_`type'4_* if in_zero1_`type'pscore==1, vce(cluster maskedid)
										test `test'
										
										matrix balance[`row',`col' + 3]=r(F)
										matrix balance[`row'+1,`col' +3]=r(p)
										matrix balance[`row'+2,`col' +3]=r(df_r)
										matrix balance[`row'+3,`col' +3]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+4,`col' + 3]=r(sum)
										
										*Traditional
										qui reg cov alpha_* Z_* c_pscore_`type'1_* c_pscore_`type'2_* c_pscore_`type'3_* c_pscore_`type'4_* if in_zero1_`type'pscore==1
										test `test'
										
										matrix balance[`row'+6,`col' + 3]=r(F)
										matrix balance[`row'+7,`col' +3]=r(p)
										matrix balance[`row'+8,`col' +3]=r(df_r)
										matrix balance[`row'+9,`col' +3]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+10,`col' + 3]=r(sum)
										
										*Robust
										qui reg cov alpha_* Z_* c_pscore_`type'1_* c_pscore_`type'2_* c_pscore_`type'3_* c_pscore_`type'4_* if in_zero1_`type'pscore==1, r
										test `test'
										
										matrix balance[`row'+12,`col' + 3]=r(F)
										matrix balance[`row'+13,`col' +3]=r(p)
										matrix balance[`row'+14,`col' +3]=r(df_r)
										matrix balance[`row'+15,`col' +3]=r(df)
										
										unique maskedid if e(sample)
										matrix balance[`row'+16,`col' + 3]=r(sum)
										
									*Bins
										local c = 0
										foreach i in 01 001 0{
											local c = `c' +1
											
											*Cluster
											qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_`type'`i'==1 & obs_cell_`type'`i' >= `min_obs' & !missing(obs_cell_`type'`i'), absorb(group_tag_`type'`i') vce(cluster maskedid)
											test `test'
											
											matrix balance[`row',`col' +3 + `c']   = r(F)
											matrix balance[`row'+1,`col' +3 + `c'] = r(p)
											matrix balance[`row'+2,`col'+ 3 + `c'] = r(df_r)
											matrix balance[`row'+3,`col'+3 + `c'] = r(df)
											
											unique maskedid if e(sample)
											matrix balance[`row'+4,`col'+3 + `c'] = r(sum)
											
											unique group_tag_`type'`i' if e(sample)
											matrix balance[`row'+5,`col'+3 + `c'] = r(sum)
											
											*Traditional
											qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_`type'`i'==1 & obs_cell_`type'`i' >= `min_obs' & !missing(obs_cell_`type'`i'), absorb(group_tag_`type'`i')
											test `test'
											
											matrix balance[`row'+6,`col'+3 + `c'] = r(F)
											matrix balance[`row'+7,`col'+3 + `c'] = r(p)
											matrix balance[`row'+8,`col'+3 + `c'] = r(df_r)
											matrix balance[`row'+9,`col'+3 + `c'] = r(df)
											
											unique maskedid if e(sample)
											matrix balance[`row'+10,`col'+3 + `c'] = r(sum)
											
											unique group_tag_`type'`i' if e(sample)
											matrix balance[`row'+11,`col'+3 + `c'] = r(sum)
											
											*Robust
											qui areg cov alpha_* Z_* i.c_baselinespyear* if in_good_cell_`type'`i'==1 & obs_cell_`type'`i' >= `min_obs' & !missing(obs_cell_`type'`i'), absorb(group_tag_`type'`i') r
											test `test'
											
											matrix balance[`row'+12,`col'+3 + `c'] = r(F)
											matrix balance[`row'+13,`col'+3 + `c'] = r(p)
											matrix balance[`row'+14,`col'+3 + `c'] = r(df_r)
											matrix balance[`row'+15,`col'+3 + `c'] = r(df)
											
											unique maskedid if e(sample)
											matrix balance[`row'+16,`col'+3 + `c'] = r(sum)
											
											unique group_tag_`type'`i' if e(sample)
											matrix balance[`row'+17,`col'+3 + `c'] = r(sum)
										}
								
								restore
							} /*Close c, i loop*/
						
						local row = `row' + 18	
						} /*Close vocs loop*/		
				} /*Close f cut*/	
				
				
				**********************************************************************************
				*********************** Suest Robust standard errors *****************************
				**********************************************************************************
				
				** F-tests using suest, which allows for robus or clustered SEs **
				* 1) Run models without clustering option
				* 2) Store results
				* 3) suest [stored values], vce(cluster clustvar)
				* 4) test coefficient
				
				local col = 1
				
				foreach f_cut in keep_small_cells drop_small_cells{

						if "`f_cut'" === "keep_small_cells"{
							local min_obs = 1
						}
						else if "`f_cut'"=="drop_small_cells"{
							local min_obs = `min_obs_cell_`file''
						}
						
							replace var="[`f_cut'] Suest Application Balance F" if _n==`row'
							replace var="[`f_cut'] Suest Application Balance F (pval)" if _n==`row' +1
							replace var="[`f_cut'] Suest Application Balance F (df)" if _n==`row' +2
							replace var="[`f_cut'] Suest Application Balance F (N)" if _n==`row' +3
							replace var="[`f_cut'] Suest Application Balance F (cells)" if _n==`row' +4
							
							replace var="[`f_cut'] Suest Baseline Balance F" if _n==`row' + 5
							replace var="[`f_cut'] Suest Baseline Balance F (pval)" if _n==`row' +6
							replace var="[`f_cut'] Suest Baseline Balance F (df)" if _n==`row' +7
							replace var="[`f_cut'] Suest Baseline Balance F (N)" if _n==`row' +8
							replace var="[`f_cut'] Suest Baseline Balance F (cells)" if _n==`row' +9
							
							replace var="[`f_cut'] Suest All Balance F" if _n==`row' + 10
							replace var="[`f_cut'] Suest All Balance F (pval)" if _n==`row' +11
							replace var="[`f_cut'] Suest All Balance F (df)" if _n==`row' +12
							replace var="[`f_cut'] Suest All Balance F (N)" if _n==`row' +13
							replace var="[`f_cut'] Suest All Balance F (cells)" if _n==`row' +14
						
					foreach type in c i{
						
						if "`type'"=="c"{
							local colbump = 0
						}
						else if "`type'"=="i"{
							local colbump = 7
						}
						
						local col = 1 + `colbump'
								

						*Balance F
							foreach var in `appcov_`type'' `cov_`type'' `cov2'{
								
								* no pscores
								qui reg `var' Z_`type'
								est store `var'
								
								* linear
								qui reg `var' Z_`type' pscore_`type'1 if in_zero1_`type'pscore==1
								est store L`var'
								
								* poly
								qui reg `var' Z_`type' pscore_`type'1 pscore_`type'2 pscore_`type'3 pscore_`type'4 if in_zero1_`type'pscore==1
								est store P`var'
								
								* Pscore bins
								foreach i in 01 001 0{
									qui reg `var' Z_`type' i.ind_i`i' 	if in_good_cell_`type'`i'==1 & obs_cell_`type'`i'>=`min_obs' & !missing(obs_cell_`type'`i')
									est store B`i'`var'
								}
	
							}
							
							*** Apcov suest ***
								cap suest 	num_schools num_`type' rank1_`type', vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row', `col' + 1] = r(chi2)
									matrix balance[`row'+1, `col' + 1] = r(p)
									matrix balance[`row'+2, `col' + 1] = r(df)
									matrix balance[`row'+3, `col' + 1] = e(N)
								}	
									
								cap suest 	Lnum_schools Lnum_`type'  Lrank1_`type', vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row', `col' +2] = r(chi2)
									matrix balance[`row'+1, `col' +2] = r(p)
									matrix balance[`row'+2, `col' +2] = r(df)
									matrix balance[`row'+3, `col' +2] = e(N)
								}
								
								cap suest 	Pnum_schools Pnum_`type' Prank1_`type', vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row', `col' +3] = r(chi2)
									matrix balance[`row'+1, `col' +3] = r(p)
									matrix balance[`row'+2, `col' +3] = r(df)
									matrix balance[`row'+3, `col' +3] = e(N)
								}
								
								local c=0
								foreach i in 01 001 0{
									local c = `c' + 1
									
									cap suest 	B`i'num_schools B`i'num_`type' B`i'rank1_`type', vce(cluster maskedid)
									if !_rc{
										test Z_`type'
										matrix balance[`row', `col' + 3 + `c'] = r(chi2)
										matrix balance[`row'+1, `col' + 3 + `c'] = r(p)
										matrix balance[`row'+2, `col' + 3 + `c'] = r(df)
										matrix balance[`row'+3, `col' + 3 + `c'] = e(N)
										
										unique tag_`type'`i' if e(sample)
										matrix balance[`row'+4, `col' + 3 + `c'] = r(sum)
									}
								}
								
						** Baseline covs suest ***
							
							cap suest	baseline`type' baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
										baseliness_math baseliness_reading baseliness_writing, vce(cluster maskedid)
							if !_rc{			
								test Z_`type'
								matrix balance[`row'+5, `col'+1] = r(chi2)
								matrix balance[`row'+6, `col'+1] = r(p)
								matrix balance[`row'+7, `col'+1] = r(df)
								matrix balance[`row'+8, `col'+1] = e(N)
							}
								
							cap suest	Lbaseline`type' Lbaselinefemale Lbaselinehisp Lbaselineblack Lbaselinefoodfred Lbaselinesped Lbaselinebilingual Lbaselinelep Lbaselinegifted ///
									Lbaseliness_math Lbaseliness_reading Lbaseliness_writing, vce(cluster maskedid)
							if !_rc{
								test Z_`type'
								matrix balance[`row'+5, `col'+2] = r(chi2)
								matrix balance[`row'+6, `col' +2] = r(p)
								matrix balance[`row'+7, `col'+2] = r(df)
								matrix balance[`row'+8, `col'+2] = e(N)
							}
							
							cap suest	Pbaseline`type' Pbaselinefemale Pbaselinehisp Pbaselineblack Pbaselinefoodfred Pbaselinesped Pbaselinelep Pbaselinebilingual Pbaselinegifted ///
									Pbaseliness_math Pbaseliness_reading Pbaseliness_writing, vce(cluster maskedid)
							if !_rc{
								test Z_`type'
								matrix balance[`row'+5, `col'+3] = r(chi2)
								matrix balance[`row'+6, `col' +3] = r(p)
								matrix balance[`row'+7, `col'+3] = r(df)
								matrix balance[`row'+8, `col'+3] = e(N)
							}
							
							local c=0
							foreach i in 01 001 0{
								local c = `c' + 1
								cap suest	B`i'baseline`type' B`i'baselinefemale B`i'baselinehisp B`i'baselineblack B`i'baselinefoodfred B`i'baselinesped B`i'baselinelep B`i'baselinebilingual B`i'baselinegifted ///
										B`i'baseliness_math B`i'baseliness_reading B`i'baseliness_writing, vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row'+5, `col' +3 + `c'] = r(chi2)
									matrix balance[`row'+6, `col' +3 + `c'] = r(p)
									matrix balance[`row'+7, `col' +3 + `c'] = r(df)
									matrix balance[`row'+8, `col' +3 + `c'] = e(N)
									
									unique tag_`type'`i' if e(sample)
									matrix balance[`row'+9, `col' +3 + `c'] = r(sum)
								}
							}
							
							
						*** All covariates suest ***

							cap suest	num_schools num_`type' rank1_`type' ///
									baseline`type' baselinefemale baselinehisp baselineblack baselinefoodfred baselinesped baselinelep baselinebilingual baselinegifted ///
									baseliness_math baseliness_reading baseliness_writing, vce(cluster maskedid)
							
							if !_rc{
								test Z_`type'
								matrix balance[`row'+10, `col'+1] = r(chi2)
								matrix balance[`row'+11, `col'+1] = r(p)
								matrix balance[`row'+12, `col'+1] = r(df)
								matrix balance[`row'+13, `col'+1] = e(N)
							}
							
								
							cap suest	Lnum_schools Lnum_`type' Lrank1_`type' ///
									Lbaseline`type' Lbaselinefemale Lbaselinehisp Lbaselineblack Lbaselinefoodfred Lbaselinesped Lbaselinebilingual Lbaselinelep Lbaselinegifted ///
									Lbaseliness_math Lbaseliness_reading Lbaseliness_writing, vce(cluster maskedid)
							
							if !_rc{
								test Z_`type'
								matrix balance[`row'+10, `col'+2] = r(chi2)
								matrix balance[`row'+11, `col' +2] = r(p)
								matrix balance[`row'+12, `col'+2] = r(df)
								matrix balance[`row'+13, `col'+2] = e(N)
							}
							
							cap suest	Pnum_schools Pnum_`type'  Prank1_`type' ///
									Pbaseline`type' Pbaselinefemale Pbaselinehisp Pbaselineblack Pbaselinefoodfred Pbaselinesped Pbaselinelep Pbaselinebilingual Pbaselinegifted ///
									Pbaseliness_math Pbaseliness_reading Pbaseliness_writing, vce(cluster maskedid)
							
							if !_rc{
								test Z_`type'
								matrix balance[`row'+10, `col'+3] = r(chi2)
								matrix balance[`row'+11, `col' +3] = r(p)
								matrix balance[`row'+12, `col'+3] = r(df)
								matrix balance[`row'+13, `col'+3] = e(N)
							}
							
							local c=0
							foreach i in 01 001 0{
								local c = `c' + 1
								cap suest	B`i'num_schools B`i'num_`type'  B`i'rank1_`type' ///
										B`i'baseline`type' B`i'baselinefemale B`i'baselinehisp B`i'baselineblack B`i'baselinefoodfred B`i'baselinesped B`i'baselinelep B`i'baselinebilingual B`i'baselinegifted ///
										B`i'baseliness_math B`i'baseliness_reading B`i'baseliness_writing, vce(cluster maskedid)
								if !_rc{
									test Z_`type'
									matrix balance[`row'+10, `col' +3 + `c'] = r(chi2)
									matrix balance[`row'+11, `col' +3 + `c'] = r(p)
									matrix balance[`row'+12, `col' +3 + `c'] = r(df)
									matrix balance[`row'+13, `col' +3 + `c'] = e(N)
									
									unique tag_`type'`i' if e(sample)
									matrix balance[`row'+14, `col' +3 + `c'] = r(sum)
								}
							}
						
					} /*Close c i loop*/
					
				local row = `row'+ 15
				} /*Close f_cut loop*/
				
					**********************************************************************************
					
				*Outsheet
				preserve
					svmat balance
					keep var balance*
					keep if _n<=`row'+1
					ren balance1 z0_c
					ren balance2 ols_z1_c
					ren balance3 ols_z1_pscore_c
					ren balance4 ols_z1_poly_c
					ren balance5 ols_z1_r01_c
					ren balance6 ols_z1_r001_c
					ren balance7 ols_z1_psat_c
					
					ren balance8 	z0_i
					ren balance9 	ols_z1_i
					ren balance10 	ols_z1_pscore_i
					ren balance11 	ols_z1_poly_i
					ren balance12 	ols_z1_r01_i
					ren balance13 	ols_z1_r001_i
					ren balance14 	ols_z1_psat_i
					
					outsheet using "`tables'inDPSbase_`file'_innov_stacked_balance.csv", comma replace
				restore
				
				} /*Close F test condition*/
		}	/*Close file loop*/
	
